<!--delete modal-->
<div class="modal fade" id="delete-modal" tabindex="-1" role="dialog" aria-hidden="true">
    <div class="modal-dialog modal-dialog-centered" role="document">
        <div class="modal-content">
            <div class="modal-header">
                <h5 class="modal-title">{{i18n("config_scripts.config_delete", {}) }}: <span id='delete-name'></span></h5>
            </div>
            <div class="modal-body">{{i18n("config_scripts.config_delete_message", {}) }}<br>
                <b>Attention</b>: this process is irreversible! 
            </div>
            <div class="modal-footer">
                <button type="button" class="btn btn-secondary" data-bs-dismiss="modal">{{i18n("cancel", {}) }}</button>
                <button type="button" id='btn-confirm-delete' class="btn btn-danger">Confirm Deleting</button>
            </div>
        </div>
    </div>
</div>

