
			     {
			     title: "Bytes Sent",
				 field: "column_bytes_sent",
				 sortable: true,
                             css: {
			        textAlign: 'center'
			     }

				 },
			     {
			     title: "Bytes Received",
				 field: "column_bytes_rcvd",
				 sortable: true,
                             css: {
			        textAlign: 'center'
			     }

				 },
		
			     {
			     title: "Duration",
				 field: "column_duration",
				 sortable: true,
                             css: {
			        textAlign: 'center'
			     }

				 },
