<!--import modal-->
<div class="modal fade" id="import-modal" tabindex="-1" role="dialog" aria-hidden="true">
    <div class="modal-dialog modal-dialog-centered" role="document">
        <div class="modal-content">
        <div class="modal-header">
            <h5 class="modal-title">{{ dialog.title }}</h5>
        </div>
        <div class="modal-body">
            <form class='form'>
                <div class="form-group mb-3 ">
                    <div class='custom-file'>
                        <input id="import-input" class="ays-ignore custom-file-input" name="JSON" type="file" />
                        <label class='custom-file-label' for='#import-input'>{{ i18n("browse_config", {}) }}</label>
                    </div>
                    <small class="text-muted form-text">
                        {* dialog.message *}
                    </small>
                </div>
                <div class="invalid-feedback" id='import-error'>
                    {message}
                </div>
            </form>
        </div>
        <div class="modal-footer">
            <button type="button" class="btn btn-secondary" data-bs-dismiss="modal">{{ dialog.cancel }}</button>
            <button type="button" id='btn-confirm-import' class="btn btn-primary">{{ dialog.apply }}</button>
        </div>
        </div>
    </div>
</div>
