  <div class="table-responsive">
  <table class="table table-bordered table-sm table-striped">
  <thead>
    <tr>
      {% for j in range(2, #report.series) do %}
        {% if j == 2 then %}
        <th class='text-center' colspan='3'>{{report.series[j]}}</th>
        {% else %}
        <th class='text-center' colspan='2'>{{report.series[j]}}</th>
        {% end %}
      {% end %}
    </tr><tr>
      <th class='text-center'>{{report.series[1]}}</th>
    {% for j in range(2, #report.series) do %}
      <th class='text-right'>{{i18n('traffic')}}</th>
      <th class='text-center'>{{i18n('duration')}}</th>
    {% end %}
    </tr>
  </thead>
  <tbody>
  {% for i in range(report.rows) do %}
    <tr>
      {% if i == report.rows then %}
        <th class=' text-right'>{{report.values[1][i]}}</th> <!-- Total -->
      {% else %}
        {% if report.values[1][i].highlighted then %}
          <td class=' text-right' ><div style="width:65%;"><span style="color: #FF5454;"><a style="color:inherit;" href="{{report.values[1][i].link}}">{{report.values[1][i].time}}</a></span></div></td>
        {% else %}
          <td class=' text-right' ><div style="width:65%;"><a style="color:inherit;" href="{{report.values[1][i].link}}">{{report.values[1][i].time}}</a></div></td>
        {% end %}
      {% end %}

      {% for j in range(2, #report.values) do %}
        {% if i == report.rows then %}
      <th class='text-right '>{{report.values[j][i].bytes}}</th><th class='text-right '>{{report.values[j][i].activity}}</th>  <!-- Total-->
        {% else %}
      <td class='text-right '>{{report.values[j][i].bytes}}</td><td class='text-right '>{{report.values[j][i].activity}}</td>
        {% end %}
      {% end %}
    </tr>
  {% end %}
  </tbody>
  </table>
</div>
