/*
 * DO NOT EDIT THIS FILE
 *
 * Put your custom theme in /etc/ntopng/custom_theme.css
 * This is currently supported only in the nedge package.
 */

/*
.logo-brand {
  width: 130px;
  height: 40px;
  padding-top: -5px;
}

.masthead {
  background-color: #313131;
  padding: 8px 4px 5px 4px;
  margin-left: -20px;
  margin-right: -20px;
}

.masthead h3 {
  margin-top: 4px;
}

#footer > .container-fluid {
  background-color: #313131;
  color: white;
  margin-left: -20px;
  margin-right: -20px;
  padding: 10px 5px;
  position: fixed;
  bottom: 0;
  width: 100%;
}
*/
