--
-- (C) 2020 - ntop.org
--

return {
    telegram_token = "Token",
    telegram_channel = "Channel Id",
    validation = {
       invalid_token = "Token di Telegram non valido.",
       invalid_channel_name = "Nome del canale non valido.",
    },
    telegram_send_error = "Errore nell'invio del messaggio a Telegram.",
 
    webhook_description = {
       token_description = "Istruzioni:<ul><li>Inizia una nuova chat con @BotFather<li>Scrivi nella chat '/newbot'<li>Dai un nome al tuo bot<li>Dai uno username al tuo bot<li>Copia qui il token che @BotFather ti ha fornito</ul>",
       channel_id_description = "Istruzioni se vuoi utilizzare il bot in un gruppo:<ul><li>Aggiungi al tuo gruppo il bot che hai creato<li>Aggiungi al tuo gruppo @getidsbot<li>Copia qui l'id che @getidsbot ti ha fornito</ul>Istruzioni se vuoi utilizzare il bot in una chat:<ul><li>Inizia una nuova chat con @getidsbot<li>Copia qui l'id che @getidsbot ti ha fornito</ul>",
    }
 
 }

