--
-- (C) 2019-22 - ntop.org
--

return {
  title = "Unexpected Device Connected",
  description = "Trigger an alert when an unexpected device connects to the network.",
  author = "ntop",
  dependencies = {},
}
