--
-- (C) 2019-22 - ntop.org
--

return {
  title = "Intrusion Detection and Prevention Log",
  description = "Logs intrusion detection and prevention events, such as when an host has been added to (or removed from) the jailed hosts pool",
  author = "ntop",
  dependencies = {},
}
