--
-- (C) 2020 - ntop.org
--

return {
    password = "Password",
    shared_secret = "Shared Secret",
    url = "URL",
    username = "Username",
    webhook_description = "Instructions:<ul><li>Alerts information are delivered to the configured URL in JSON format using POST requests.<li>The Shared Secret, when configured, is included in all JSON messages.<li>Username and Password (optional) use HTTP Basic authentication.",
}

