html, body, #particles-js {
  background-color: #121212 !important;
  color: #e2e2e2;
}

code {
  color: rgb(248, 248, 242) !important;
}

.btn-warning{
 color: #000!important; 
}

.graph-val-total-title, .graph-val-95percentile-title, .graph-val-average-title, .graph-val-max-title, .graph-val-total{
  border-color:  #2c2c2c!important;
}

.ntop_notes{
  color: white;
}

.leaflet-top.leaflet-left{
  z-index: auto;
}

.btn-primary {
  color: #000;
  background-color: var(--ntop-orange);
  border-color: var(--ntop-orange);
}
.btn-primary:hover {
  color: #000;
  background-color: #ffa026;
  border-color: #ff9a1a;
}

.btn-check:focus + .btn-primary, .btn-primary:focus {
  color: #000;
  background-color: #ffa026;
  border-color: #ff9a1a;
  box-shadow: 0 0 0 0.25rem rgba(217, 122, 0, 0.5);
}
.btn-check:checked + .btn-primary, .btn-check:active + .btn-primary, .btn-primary:active, .btn-primary.active, .show > .btn-primary.dropdown-toggle {
  color: #000;
  background-color: #ffa533;
  border-color: #ff9a1a;
}
.btn-check:checked + .btn-primary:focus, .btn-check:active + .btn-primary:focus, .btn-primary:active:focus, .btn-primary.active:focus, .show > .btn-primary.dropdown-toggle:focus {
  box-shadow: 0 0 0 0.25rem rgba(217, 122, 0, 0.5);
}
.btn-primary:disabled, .btn-primary.disabled {
  color: #000;
  background-color: var(--ntop-orange);
  border-color: var(--ntop-orange);
}

.btn-secondary {
  color: #fff;
  background-color: var(--ntop-blue);
  border-color: var(--ntop-blue);
}
.btn-secondary:hover {
  color: #fff;
  background-color: #2f3c43;
  border-color: #2c393f;
}
.btn-check:focus + .btn-secondary, .btn-secondary:focus {
  color: #fff;
  background-color: #2f3c43;
  border-color: #2c393f;
  box-shadow: 0 0 0 0.25rem rgba(85, 99, 105, 0.5);
}
.btn-check:checked + .btn-secondary, .btn-check:active + .btn-secondary, .btn-secondary:active, .btn-secondary.active, .show > .btn-secondary.dropdown-toggle {
  color: #fff;
  background-color: #2c393f;
  border-color: #29353b;
}
.btn-check:checked + .btn-secondary:focus, .btn-check:active + .btn-secondary:focus, .btn-secondary:active:focus, .btn-secondary.active:focus, .show > .btn-secondary.dropdown-toggle:focus {
  box-shadow: 0 0 0 0.25rem rgba(85, 99, 105, 0.5);
}
.btn-secondary:disabled, .btn-secondary.disabled {
  color: #fff;
  background-color: var(--ntop-blue);
  border-color: var(--ntop-blue);
}

.alert-primary {
  color: #995600;
  background-color: #ffe9cc;
  border-color: #ffddb3;
}
.alert-primary .alert-link {
  color: #7a4500;
}

.alert-secondary {
  color: #212b2f;
  background-color: #d7dadc;
  border-color: #c3c8ca;
}
.alert-secondary .alert-link {
  color: #1a2226;
}

.list-group-item-primary {
  color: #995600;
  background-color: #ffe9cc;
}
.list-group-item-primary.list-group-item-action:hover, .list-group-item-primary.list-group-item-action:focus {
  color: #995600;
  background-color: #e6d2b8;
}
.list-group-item-primary.list-group-item-action.active {
  color: #fff;
  background-color: #995600;
  border-color: #995600;
}

.list-group-item-secondary {
  color: #212b2f;
  background-color: #d7dadc;
}
.list-group-item-secondary.list-group-item-action:hover, .list-group-item-secondary.list-group-item-action:focus {
  color: #212b2f;
  background-color: #c2c4c6;
}
.list-group-item-secondary.list-group-item-action.active {
  color: #fff;
  background-color: #212b2f;
  border-color: #212b2f;
}

.link-primary {
  color: var(--ntop-orange);
}
.link-primary:hover, .link-primary:focus {
  color: #ffa533;
}

.link-secondary {
  color: var(--ntop-blue);
}
.link-secondary:hover, .link-secondary:focus {
  color: #2c393f;
}

.border-primary {
  border-color: var(--ntop-orange) !important;
}

.border-secondary {
  border-color: var(--ntop-blue) !important;
}

.text-primary {
  color: var(--ntop-orange) !important;
}

.text-secondary {
  color: var(--ntop-blue) !important;
}

.pie-chart {
  background-color: transparent !important;
}

svg:not(.leaflet-zoom-animated) {
  background-color: #121212;
  color: #eee;
}

#ntop-logo > svg, svg.peity {
  background: none;
}

a,.btn-link {
  color: #ff8a26;
}
a:hover, .btn-link:hover {
  color: #f39f55;
}

.btn {
  color: #eee;
}

.btn:hover {
  color: #d6d6d6;
}

.btn-light {
  color: #eee;
  background-color: #232323;
  border-color: #272727;
}
.btn-light:hover {
  color: #eee;
  background-color: #333333;
  border-color: #272727;
}
.btn-light:not(:disabled):not(.disabled).active, .btn-light:not(:disabled):not(.disabled):active, .show>.btn-light.dropdown-toggle {
  color: #eee;
  background-color: #333333;
  border-color: #272727;
}

.dropdown-header {
  color: #9a9a9a;
}
.dropdown-divider {
  border-top: 1px solid #0a0a0a
}

.bg-primary {
    background-color: #ff8a26 !important;
}

.bg-light, .card {
  background-color: #1f1f1f !important;
  color: #e2e2e2 !important;
}

.card-body {
  background-color: #262626 !important;
}

.bg-white {
  background-color: #1f1f1f !important;
  color: #e2e2e2;
}

.bg-black {
  background-color: #eee !important;
}

.info-stats a {
  color: #e2e2e2 !important;
}

.navbar-light, .navbar-light .navbar-brand, ul.navbar-nav li.nav-item > a {
  color: #e2e2e2 !important;
}
.nav-pills .nav-link.active, .nav-pills .show>.nav-link {
  background-color: #ff7500;
}

.nav-link {
  color: rgb(255 255 255 / 50%) !important;
}

.extrainfo {
  position: absolute;
  z-index: 1;
  border-radius: 4px;
  left: 8px;
  top: 8px;
  padding: 8px;
  background: rgba(0, 0, 0, 0.5);
}

ul.navbar-nav li.nav-item.active,
ul.navbar-nav li.nav-item.active > a.active {
  color: #e9e9e9 !important;
  font-weight: 600;
}

.breadcrumb {
  background-color: #1d1d1d !important;
}
.breadcrumb-item.active {
    color: #a2a2a2;
}

.dropdown-menu {
  background-color: #1d1d1d !important;
  color: #eee;
}

.blog-section a, .blog-section h6, .blog-section i, .blog-section p, .blog-section small,
.dropdown-item {
  color: #e2e2e2;
}
.blog-section a:hover p, .blog-section a:hover small, .blog-section a:hover h6, .blog-section a:hover i,
.list-group-item-action:focus, .list-group-item-action:hover {
  color: #F39F53;
}

.dropdown-menu .dropdown-item:hover {
  background-color: #2c2c2c !important;
  color: #e2e2e2;
}

.popover {
    background-color: #343434;
}
.popover-body {
    color: #eee;
}
.popover-header {
    background-color: #323232;
  color: #eee;
  border-bottom: 1px solid #272727;
}

.table {
  background-color: #1f1f1f !important;
  color: #e2e2e2 !important;
}

.table-primary {
  --bs-table-bg: #161616 !important;
}

.table-bordered,
.table-bordered td,
.table-bordered th {
  border: 1px solid #2c2c2c;
}

.table thead th{
  border-bottom: 2px solid #2c2c2c;
}
.table td,
.table th {
  border-top: 1px solid #2c2c2c !important;
  border-right-color: #2c2c2c !important;
  border-top-color: #2c2c2c !important;
  border-bottom-color: #2c2c2c !important;
  color: #e2e2e2;
}
.table-hover tbody tr:hover {
  color: #e2e2e2 !important;
  background-color: rgba(48, 48, 48) !important;
}
.table .thead-light th {
  color: #e2e2e2 !important;
  background-color: #333;
}
.tablesorter,
table.tablesorter thead tr th,
table.tablesorter tfoot tr th {
  background-color: #2c2c2c !important;
}
table.tablesorter tbody td {
  background-color: #121212 !important;
  color: #e2e2e2 !important;
}

.btn-outline-dark {
  border-color: #f8f9fa !important;
}
.btn-outline-dark:hover,  ul.navbar-nav li.nav-item > a.btn-outline-dark:hover {
  color: #212529 !important;
  background-color: #f8f9fa !important;
  border-color: #f8f9fa !important;
}

.pagination .page-item .page-link,
.list-group-item {
  background-color: #2d2d2d !important;
  border: 1px solid #262626;
  color: #fff;
}
.pagination .page-item.disabled .page-link {
  color: #636363 !important;
}
.pagination .page-item.active .page-link {
  background-color: #ff7500 !important;
  color: #fff !important;
}
ul.navbar-nav li.nav-item.active>a.active {
  color: #ff7500 !important;
}

.list-group-item.active, .form-check-input:checked {
  background-color: #ff8a26 !important;
  border-color: #e26a04;
}

.nav-tabs {
    border-bottom: 0px;
}
.nav-tabs .nav-item.show .nav-link, .nav-tabs .nav-link.active {
    color: white !important;
    background-color: #262626;
    border-color: #2c2c2c #2c2c2c #212121;
}
.nav-tabs .nav-link:focus, .nav-tabs .nav-link:hover {
    border-color: #2c2c2c #2c2c2c #212121;
}

.modal-header, .modal-footer {
    border-color: #2c2c2c !important;
}
.modal-content {
    background-color: #242424;
}

.alert-light {
    background-color: #1f1f1f !important;
    color: #eee;
    border-color: #2c2c2c;
}

.filter-scripts-button {
    color: #ff7500 !important;
}
.filter-scripts-button:hover {
    color: #f39f55 !important;
}
.filter-scripts-button.active {
    background-color: #ff7500 !important;
    color: #eee !important;
}

.form-control, .form-select, .btn[role=combobox] {
  color: #d1d1d1 !important;
  background-color: #181818 !important;
  border: 1px solid #3c3c3c !important;
}
.form-control:disabled, .form-control[readonly] {
  background-color: #696969;
}
.form-control:focus {
    border-color: #f39f55;
    background-color: #333333;
    color: #eee;
    box-shadow: 0 0 0 0.2rem rgba(255, 118, 0, 0.25);
}
.input-group-text {
    color: rgb(216, 216, 216);
    background-color: #232323;
    border: 1px solid #3c3c3c;
}

.form-signin {
    background-color: #242424 !important;
    border: 1px solid #3c3c3c !important;
}

.progress {
  background-color: #383838;
}

table.table svg, table.table .pie-chart circle {
  background-color: #1d1d1d;
}

.pie-chart circle {
  fill: #121212 !important;
}
text, g text.tick {
  fill: #eee;
  text-shadow: none;
}

.table tbody+tbody {
  border-top: 2px solid #464646
}

#legend {
  background: transparent !important;
  color: #ccc !important;
}

.switches-list-container {
  background: none  !important;
}

.border-top {
  border-top: 1px solid #2d2d2d!important;
}
.border-bottom {
  border-bottom: 1px solid #2d2d2d!important;
}
.border-left {
  border-left: 1px solid #2d2d2d!important;
}
.border-right {
  border-right: 1px solid #2d2d2d!important;
}

.btn-primary, .primary {
  color: #000;
}

.border {
  border: 1px solid #323232!important;
}

.nvd3 .nv-axis line, .nvd3 .nv-axis path {
  stroke-width: 0.8px !important;
  stroke: #2C2C2C !important;
}

.widget-name {
  align-items: center;
}

.modal-header .close {
  color: white;
}

#n-navbar {
  border-radius: 0px !important;
}
#n-navbar.scrolled.bg-light {
  background-color: #151515 !important;
}

.toast {
  box-shadow: 0 20px 25px -5px rgba(255, 255, 255, 0.1), 0 10px 10px -5px rgba(255, 255, 255, 0.04);
}
.toast, .toast-body {
  background-color: rgba(16, 16, 16, 0.85);
}

.btn-gray {
  background-color: #414141!important;
}
.overlay {
  background-color: rgba(15, 15, 15, 0.70) !important;
}

.tagify__input {
  color: #eee !important;
}

.apexcharts-tooltip .apexcharts-tooltip-title{
  background-color: #414141!important;
}
.apexcharts-tooltip.apexcharts-theme-light {
  background-color: rgb(32, 32, 32) !important;
}

.btn-close {
  color: white;
  background: transparent url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='%23fff'%3e%3cpath d='M.293.293a1 1 0 011.414 0L8 6.586 14.293.293a1 1 0 111.414 1.414L9.414 8l6.293 6.293a1 1 0 01-1.414 1.414L8 9.414l-6.293 6.293a1 1 0 01-1.414-1.414L6.586 8 .293 1.707a1 1 0 010-1.414z'/%3e%3c/svg%3e") center/1em auto no-repeat;
}

.btn-check:active+.btn-outline-primary, .btn-check:checked+.btn-outline-primary, .btn-outline-primary.active, .btn-outline-primary.dropdown-toggle.show, .btn-outline-primary:active {
  border-color: var(--ntop-orange-dark);
}

.btn-check:active+.btn-outline-primary, .btn-check:checked+.btn-outline-primary, .btn-outline-primary.active, .btn-outline-primary.dropdown-toggle.show, .btn-outline-primary:active,
.dropdown-item.active, .dropdown-item:active {
  background-color: var(--ntop-orange);
}

summary, .apexcharts-legend-text {
  color: #e2e2e2 !important;
}

.fa-external-link-alt{
  color: #ff8a26;
  cursor: pointer;
}

.btn{
   -webkit-appearance: none !important;
   -moz-appearance: none !important;
   appearance: none !important;
}


li > a > .fa-external-link-alt{
  color: #ff8a26;
}

.widget-box {
  background-color: #1f1f1f;
}

