--
-- (C) 2019-22 - ntop.org
--

return {
  title = "LLDP Topology Monitor",
  description = "Detects changes in the SNMP network topology and triggers alerts",
  author = "ntop",
  dependencies = {},
}
