--
-- (C) 2019-22 - ntop.org
--

return {
  title = "Remote Connection",
  description = "Detects if an host is taking part to one or more flows with a remote access protocol (e.g. TeamViewer)",
  author = "ntop",
  dependencies = {},
}
