--
-- (C) 2019-22 - ntop.org
--

return {
  title = "Dropped Alerts",
  description = "Triggers an alert on the system when any interface has dropped alerts",
  author = "ntop",
  dependencies = {},
}
