<p>&nbsp;<p><small><b>NOTE</b></small>
<ol>
<li> <small><i class="icon-map-marker"></i> <span id=mylocation></span></small>
<li> <small>In order to visualize maps you must:</small>
<ol>
<li> <small>Have a working Internet connection.</small>
<li> <small>Have compiled the software with geolocation and started with it.</small>
<li> <small>Have active flows between peers with public IP addresses.</small>
</ol>
<li> <small>HTML <A HREF=http://diveintohtml5.info/geolocation.html>browser geolocation</A> is used to place on map hosts based on unknown locations.</small>
<li> <small>A Google Maps Browser API Key may be required. Detailed information on key generation <a href="https://googlegeodevelopers.blogspot.it/2016/06/building-for-scale-updates-to-google.html">can be found here</a>.<br>
   Once generated, the key can be submitted via the <a href="@HTTP_PREFIX@/lua/admin/prefs.lua?tab=users"><i class="fa fa-flask"></i> Preferences</a>.
   </small>
</ol>
</small>
