/* Cal-HeatMap CSS */


.graph
{
	clear: both;
	display: block;
	font-family: "Lucida Grande", Lucida, Verdana, sans-serif;
	overflow: hidden;
}

.graph-label
{
	fill: #999;
	font-size: 10px
}

.graph-rect
{
	fill: #ededed;
	shape-rendering: crispedges
}

.graph-rect:hover
{
	stroke: #000;
	stroke-width: 1px
}

.subdomain-text {
	font-size: 8px;
	fill: #999;
	pointer-events: none;
}

.hover_cursor:hover {
	cursor: pointer;
}

.qi {
	background-color: #999;
	fill: #999;
}

.q0
{
	background-color: #fff;
	fill: #fff;
	stroke: #ededed
}

.q1
{
	background-color: #dae289;
	fill: #dae289
}

.q2
{
	background-color: #cedb9c;
	fill: #9cc069
}

.q3
{
	background-color: #b5cf6b;
	fill: #669d45
}

.q4
{
	background-color: #637939;
	fill: #637939
}

.q5
{
	background-color: #3b6427;
	fill: #3b6427
}

rect.highlight
{
	stroke:#444;
	stroke-width:1;
}

text.highlight
{
	fill: #444;
}

rect.now
{
	stroke: red;
}

text.now
{
	fill: red;
	font-weight: 800;
}

.domain-background {
	fill: none;
	shape-rendering: crispedges;
}

