div.tooltip-heatmap {	
    position: absolute;			
    text-align: center;			
    width: 60px;					
    height: 40px;					
    padding: 2px;				
    font: 12px sans-serif bold;		
    background: rgb(171, 203, 245);	
    border: 0px;		
    border-radius: 8px;			
    pointer-events: none;			
}

.x_label, .y_label{
    cursor:pointer;
}

