--
-- (C) 2019-22 - ntop.org
--

return {
  title = "Broadcas Domain Too Large",
  description = "Detects ARP traffic between two MACS addresses beloging to different broadcast domains",
  author = "ntop",
  dependencies = {},
}
