<div class="modal fade" role="dialog" id='modal-script'>
    <div class="modal-dialog modal-lg ">
        <div class="modal-content">
            <form id='edit-form' method='post'>
                <div class="modal-header">
                    <h5 class="modal-title"><span id='script-name'></span></h5>
                    <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
                </div>
                <div class="modal-body">
                    <table class='table table-borderless' id='script-config-editor'>
                    </table>
                    <div id='script-description' class='alert alert-light' role='alert'>
                    </div>
                    <span class='invalid-feedback' id='apply-error'></span>
		</div>
                <div class="modal-footer">
                    <button id='btn-reset' title='Reset Default ntopng values' type='button'
                        class='btn btn-danger'>
                        {{ i18n("scripts_list.reset_default") }}
                    </button>
                    <div class="action-button-container" >
                        <span class="border mr-2 vertical-align"></span>
                        <button id="btn-action" class="btn btn-secondary">{ action }</button>
                    </div>
                    <button id="btn-apply" title='{{ i18n("apply", {}) }}' type="submit" class="btn btn-primary ml-auto">
                        {{ i18n("apply", {}) }}
                    </button>
                    <div class="alert alert-danger w-100" id="action-error" style="display: none;">
                    </div>
                </div>
            </form>
        </div>
    </div>
</div>

<template id="exclusion-list-template">
    <textarea name="exclusion-list" id="exclusionlist-textarea" class="w-100 form-control" rows="6"></textarea>
    <small>{{ i18n("scripts_list.exclusion_list") }}</small>
</template>

