-- This file contains the localized english strings for this plugin.
-- They can be referenced by prefixing them with the plugin key,
-- e.g. "example.title".

-- Other locales can be defined in other files with a specific language code as the file name,
-- for example "it.lua" for the Italian locale. The full list of supported locales is
-- defined in the "supported_locales" table in scripts/lua/modules/locales_utils.lua .
return {
  alert_title = "Example Alert",
  alert_description = "An example alert for the 'example' plugin",
  simple_status = "This is a constraint flow status description",
  status_invalid_port = "Example status. The server port is %{port_number}.",
  status_title = "Example Status",
  snmp_script_title = "SNMP Example Script",
  snmp_script_description = "An example user script executed on SNMP devices/interfaces",
  flow_script_title = "Flow Example Script",
  flow_script_description = "An example user script executed on flows",
  host_script_title = "Host Example Script",
  host_script_description = "An example user script executed on hosts",
  interface_script_title = "Interface Example Script",
  interface_script_description = "An example user script executed on network interfaces",
  network_script_title = "Network Example Script",
  network_script_description = "An example user script executed on local networks",
  system_script_title = "System Example Script",
  system_script_description = "An example user script executed once per ntopng instance",
  custom_menu_entry = "Example Menu Entry",
  custom_page_title = "Example page",
  toggle_example_notification_title = "Toggle Example Notification",
  toggle_example_notification_description = "An example alert notification endpoint",
  example_notification = "Example Notification",
}
