--
-- (C) 2020 - ntop.org
--

return {
   notification_webhook = "Notification Webhook",
   sender_username = "Sender Username",
   webhook_description = "Instructions:<ul><li>Create a new channel to be used for notifications<li>Create a Slack app from the <a href='https://api.slack.com/apps?new_app=1' target='_blank'>Slack API page</a><li>Activate Incoming Webhooks<li>Add a new Webhook and select the notifications channel<li>Copy the Webhook URL to <b>Notification Webhook</b></ul>"
}
