--
-- (C) 2020 - ntop.org
--

return {
   no_if_activity_description = "Trigger an alert when no activity from an interface is detected",
   no_if_activity_title = "No activity on interface",

-- ####################### Status strings

   status_no_activity_description = "No activity reported on network interface.",

-- ####################### Alert strings

   alert_no_activity_title = "No activity on interface"
}
