--
-- (C) 2019-22 - ntop.org
--

return {
  title = "SYN Scan detector",
  description = "Detects SYN scan attacks on hosts and networks and triggers alerts",
  author = "ntop",
  dependencies = {},
}
