<div id="{{dialog.id}}" class="modal fade" tabindex="-1" role="dialog" aria-labelledby="{{dialog.id}}_label" aria-hidden="true">
  <div class="modal-dialog" role="document">
    <div class="modal-content">
      <div class="modal-header">
        <h5 class="modal-title">{*dialog.title*}</h5>
        <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
      </div>
      <div class="modal-body {{dialog.custom_dialog_class}}">
        <div  class="message {{dialog.custom_alert_class or "alert alert-warning"}}">
          {*dialog.message*}
          <span id="url_{{dialog.id}}" class="text-break"><span>
        </div>
      </div>
      {% if dialog.no_footer then %}
        {# empty #}
      {% else %}
      <div class="modal-footer">
        <span class="invalid-feedback" id="{{dialog.id}}_more_content" style="display: none;">
          {*dialog.err_msg*}
        </span>
        {% if dialog.action ~= nil then %}
	  <button type="submit" class="btn {{dialog.confirm_button or "btn-primary"}}" onclick="{{dialog.action}}">{*dialog.confirm*}</button>
        {% else %}
	<a {% if not dialog.no_confirm_id then %} target="_blank" id="btn-confirm-action_{{dialog.id}}" {% end %} {% if dialog.dismiss_on_confirm then %} data-bs-dismiss="modal" {% else %} type="submit" {%  end %} class="btn {{dialog.confirm_button or "btn-primary"}}">{*dialog.confirm*}</a>
        {% end %}
      </div>
      {% end %}
    </div>
  </div>
</div>
