--
-- (C) 2020 - ntop.org
--

return {
    no_if_activity_description = "Attiva un'allerta quando è riscontrata nessuna attività da parte di un'interfaccia di rete",
    no_if_activity_title = "Nessuna attività da un'interfaccia",
 
 -- ####################### Status strings
 
    status_no_activity_description = "Nessuna attività riportata dall'interfaccia di rete.",
 
 -- ####################### Alert strings
 
    alert_no_activity_title = "Nessuna attività dall'interfaccia"
 }