print[[{
  title: "]] print(i18n("chart")) print[[",
  field: "column_chart",
  sortable: true,
  css: {
    textAlign: 'center'
  }
}, {
  title: "]] print(i18n("flows_page.flows_as_client")) print[[",
  field: "column_num_flows_as_client",
  sortable: false,
  css: {
    textAlign: 'center'
  }
}, {
  title: "]] print(i18n("flows_page.flows_as_server")) print[[",
  field: "column_num_flows_as_server",
  sortable: false,
  css: {
    textAlign: 'center'
  }
}, {
  title: "]] print(i18n("containers_stats.avg_rtt_as_client")) print[[",
  field: "column_avg_rtt_as_client",
  sortable: false,
  css: {
    textAlign: 'right'
  }
}, {
  title: "]] print(i18n("containers_stats.avg_rtt_as_server")) print[[",
  field: "column_avg_rtt_as_server",
  sortable: false,
  css: {
    textAlign: 'right'
  }
}, {
  title: "]] print(i18n("containers_stats.avg_rtt_variance_as_client")) print[[",
  field: "column_avg_rtt_variance_as_client",
  sortable: false,
  css: {
    textAlign: 'right'
  }
}, {
  title: "]] print(i18n("containers_stats.avg_rtt_variance_as_server")) print[[",
  field: "column_avg_rtt_variance_as_server",
  sortable: false,
  css: {
    textAlign: 'right'
  }
}
]]
