--
-- (C) 2020 - ntop.org
--

return {
  title = "Interface activity check",
  description = "Detects if there is activity on a network interface, if not calls and triggers an alert",
  author = "ntop",
  dependencies = {},
}
