-- This file contains the localized english strings for this plugin.

return {
   title = "Suricata",
   description = "Collect alerts and metadata from Suricata",
   statistics = "Suricata Statistics",
}
