<div id='am-delete-modal' class="modal fade" tabindex="-1" role="dialog">
	<form id='am-delete-form'>
		<div class="modal-dialog modal-dialog-centered modal-lg" role="document">
			<div class="modal-content">
				<div class="modal-header">
					<h5 class="modal-title">{{ dialog.delete }}: <span id="delete-host"></span></h5>
					<button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
				</div>
				<div class="modal-body">
					<div class="alert alert-danger">
						{{ dialog.confirm_delete }}
					</div>
					<span class="invalid-feedback"></span>
				</div>
				<div class="modal-footer">
					<button id="btn-delete-am" type="submit" class="btn btn-danger">{{ dialog.delete }}</button>
				</div>
			</div>
		</div>
	</form>
</div>