local lang = {
  ["abort"] = "中止",
  ["acknowledge"] = "确认",
  ["acknowledge_alerts"] = "确认警告",
  ["acknowledged_all"] = "所有",
  ["acknowledged_only"] = "仅确认",
  ["action"] = "操作",
  ["actions"] = "操作",
  ["active_flows"] = "活动流",
  ["activities"] = "活动",
  ["activity"] = "活动",
  ["add"] = "添加",
  ["admin_status"] = "管理员",
  ["advanced"] = "高级",
  ["aggregation"] = "汇聚",
  ["alias"] = "别名",
  ["all"] = "所有",
  ["all_hosts"] = "所有主机",
  ["allowed"] = "允许",
  ["and"] = "和",
  ["and_x_more"] = "和%{num} 个",
  ["any"] = "任何",
  ["app_name"] = "应用名称",
  ["application"] = "应用程序",
  ["applications"] = "应用程序",
  ["apply"] = "应用",
  ["are_you_sure"] = "确定吗?",
  ["arp"] = "ARP",
  ["arp_type"] = "ARP类型",
  ["as"] = "AS",
  ["as_number"] = "AS号",
  ["asn"] = "ASN",
  ["asn_id"] = "ASN %{id}",
  ["attacker"] = "攻击者",
  ["autolayout_description"] = "执行自动布局可能会大大改变网络的当前布局",
  ["availability"] = "可用性",
  ["available"] = "可用的",
  ["backup"] = "备份",
  ["begin"] = "开始",
  ["bind"] = "绑定",
  ["breakdown"] = "细分",
  ["bridge"] = "桥",
  ["broadcast"] = "广播",
  ["broadcast_domain"] = "广播域",
  ["broadcast_traffic"] = "广播流量",
  ["browse_config"] = "浏览配置文件以导入",
  ["busy"] = "繁忙",
  ["bytes"] = "字节",
  ["cancel"] = "取消",
  ["categories"] = "类别",
  ["category"] = "类别",
  ["change"] = "更改",
  ["change_number_of_rows"] = "更改每页的行数",
  ["change_policy"] = "更改主机池策略",
  ["chart"] = "图表",
  ["cli_port"] = "CLI端口",
  ["click_to_expand"] = "点击展开",
  ["client"] = "客户端",
  ["client_and_server"] = "客户端和服务器",
  ["client_to_server"] = "客户端到服务器",
  ["clone"] = "克隆",
  ["close"] = "关闭",
  ["configure"] = "配置",
  ["confirm"] = "确认",
  ["copied"] = "复制的",
  ["countries"] = "国家",
  ["create"] = "创建",
  ["current_filter"] = "禁用检查",
  ["current_filter_for"] = "禁用",
  ["dark"] = "暗",
  ["data_flush_confirm"] = "确认要刷新所有数据吗？",
  ["date_from"] = "开始日期/时间：",
  ["date_to"] = "结束日期/时间：",
  ["day"] = "天",
  ["default"] = "默认值",
  ["define"] = "定义",
  ["delete"] = "删除",
  ["delete_alerts"] = "删除警告",
  ["delete_disabled_alerts"] = "删除警告",
  ["description"] = "描述",
  ["destination"] = "目的",
  ["developer"] = "开发者",
  ["device_protocols_description"] = "在此页面中,可以按设备类型和方向（相对于应用程序而言,设备是客户端还是服务器）配置可接受的应用程序列表。 通过在\"用户脚本\"配置页面中启用流用户脚本\"不允许使用设备应用程序\",当检测到异常应用程序时会生成警告。",
  ["devices"] = "设备",
  ["difference"] = "区别",
  ["disable"] = "禁用",
  ["disabled"] = "已禁用",
  ["discarded"] = "丢弃",
  ["dismiss"] = "解除",
  ["dns"] = "DNS",
  ["domain"] = "域名",
  ["double_num_flows_hosts"] = "双数",
  ["download"] = "下载",
  ["download_x"] = "%{what}下载",
  ["downloaders"] = "下载容量",
  ["drilldown"] = "向下钻取",
  ["dscp"] = "DSCP",
  ["dscp_dst2src"] = "服务器DSCP",
  ["dscp_src2dst"] = "客户端DSCP",
  ["duplexstatus"] = "双工状态",
  ["duration"] = "持续时间",
  ["edit"] = "编辑",
  ["edit_configuration"] = "编辑配置",
  ["egress"] = "出口",
  ["empty"] = "空",
  ["enable"] = "启用",
  ["enable_them"] = "启用它们",
  ["enabled"] = "已启用",
  ["end"] = "结束",
  ["enterpriseOnly"] = "此功能仅在ntopng企业版中可用",
  ["error"] = "错误",
  ["error_no_search_results"] = "未找到结果。 请修改您的搜索条件。",
  ["error_not_granted"] = "请求不允许",
  ["error_rrd_cannot_complete_dump"] = "无法完成本地主机时间序列转储。 磁盘速度慢或者本地主机过多？",
  ["error_rrd_low_resolution"] = "您请求获取的数据分辨率低于RRD上可用的分辨率,这将导致数据无效。<br>如果您仍然希望获得如此细粒度的数据,请调整<a href=\"%{prefs}\"> 应用程序/网络时间序列</a>首选项",
  ["errors"] = "错误",
  ["expired_csrf"] = "由于页面已过期,因此无法执行请求的操作。 请重新加载页面,然后再次执行操作。",
  ["export"] = "导出",
  ["external_link"] = "外部链路",
  ["factory_reset"] = "恢复出厂设置",
  ["filter"] = "禁用",
  ["filter_by"] = "过滤",
  ["fingerprint_note"] = "仅当将ntopng配置为通过ZMQ接收eBPF时,才设置应用程序名称",
  ["flow"] = "流",
  ["flow_alerts_search_criteria"] = "流警告搜索条件",
  ["flow_dump_not_working"] = "请检查服务日志以获取更多信息,这可能是文件系统权限或可用空间存在问题。",
  ["flow_dump_not_working_title"] = "流转储未起用",
  ["flow_search_criteria"] = "流搜索条件",
  ["flow_search_from_aggregated"] = "（汇总）",
  ["flow_search_results"] = "流搜索结果",
  ["flows"] = "流",
  ["flush_periodicity_map_data"] = "刷新数据",
  ["flush_service_map_data"] = "刷新数据",
  ["forbidden"] = "禁止的",
  ["free"] = "free",
  ["frequency"] = "频率",
  ["from"] = "从",
  ["full"] = "Full",
  ["gateway"] = "网关",
  ["geolocation_unavailable"] = "请遵循以下<a href='%{url}' target='%{target}'>说明<i class ='%{icon}'> </i> </a>。",
  ["geolocation_unavailable_title"] = "在没有地理位置支持的情况下运行",
  ["granularity"] = "粒度",
  ["has_attacker"] = "有攻击者",
  ["has_victim"] = "有受害者",
  ["hassh_fingerprint"] = "HASH指纹",
  ["health"] = "健康状态",
  ["help"] = "帮助",
  ["host"] = "主机%{host}",
  ["host_address"] = "主机地址",
  ["hosts"] = "主机",
  ["hosts_map"] = "主机地图",
  ["hour"] = "小时",
  ["howto_multiselect"] = "要选择多个项目,请按住<kbd> Ctrl </kbd>（在Mac上为<kbd> Cmd </kbd>）。",
  ["http"] = "HTTP",
  ["http_s"] = "HTTP(S)",
  ["https"] = "HTTPS",
  ["icmp"] = "ICMP",
  ["icmpv4"] = "ICMPv4",
  ["icmpv6"] = "ICMPv6",
  ["iface_download"] = "%{iface}下载",
  ["iface_upload"] = "%{iface}上传",
  ["import"] = "导入",
  ["import_export_config"] = "导入/导出配置",
  ["incomingflows"] = "活动输入流",
  ["info"] = "信息",
  ["ingress"] = "入口",
  ["interface"] = "接口",
  ["interface_ifname"] = "接口%{ifname}",
  ["interfaces"] = "接口",
  ["invalid_value"] = "无效值",
  ["ip_address"] = "IP地址",
  ["ipv4"] = "IPv4",
  ["ipv6"] = "IPv6",
  ["is_attacker_to_victim"] = "攻击者到被攻击者",
  ["is_victim_to_attacker"] = "被攻击者到攻击者",
  ["ja3_client_fingerprint"] = "JA3客户端指纹",
  ["ja3_fingerprint"] = "JA3指纹",
  ["key"] = "Key",
  ["l7_id"] = "应用程序 %{id}",
  ["label"] = "标签",
  ["label_alerts"] = "标签警告",
  ["language"] = "语言",
  ["last_seen"] = "最后发现",
  ["layer_2"] = "MAC地址",
  ["legenda"] = "Legenda",
  ["level"] = "等级",
  ["light"] = "亮",
  ["local_hosts"] = "本地主机",
  ["local_traffic"] = "本地流量",
  ["lua_c_api"] = "Lua/C API",
  ["mac_address"] = "MAC地址",
  ["maps"] = "地图",
  ["max_rate"] = "最大速率",
  ["memory"] = "内存",
  ["missing_x_parameter"] = "缺少\"%{param} \"参数",
  ["model"] = "模型",
  ["month"] = "月",
  ["mtu"] = "MTU",
  ["multicast"] = "组播",
  ["name"] = "名称",
  ["netmask"] = "子网掩码",
  ["network_id"] = "网络 %{id}",
  ["networks"] = "网络",
  ["new"] = "新的",
  ["next"] = "下一个",
  ["no"] = "no",
  ["noTraffic"] = "在指定的日期/时间选择中,未报告任何流量",
  ["no_attacker_no_victim"] = "没有攻击者/受害者",
  ["no_data_available"] = "无可用数据",
  ["no_flows"] = "没有流",
  ["no_ports"] = "没有端口",
  ["no_results_found"] = "未找到结果",
  ["no_talkers_for_the_host"] = "当前主机没有talker",
  ["not_admin"] = "需要管理员权限",
  ["note_flow_search_allowed_networks"] = "当前用户仅被允许浏览以下网络：%{nets}。",
  ["note_flow_search_allowed_networks_applications"] = "不属于任何允许的网络的主机将从应用程序对话者中隐藏。",
  ["note_flow_search_allowed_networks_counter"] = "流搜索结果中的计数器是总计,但仅显示允许网络中具有客户端和服务器的流。",
  ["note_flow_search_allowed_networks_ipv6"] = "允许的非零IPv6网络将导致无法显示所有IPv6流。",
  ["note_flow_search_allowed_networks_talkers"] = "不属于任何允许的网络的主机将从Top talkers中隐藏。",
  ["notes"] = "注意",
  ["num_different_ports"] = "%{num}个端口。 没有流量%{threshold}%% +。",
  ["num_uses"] = "使用数量",
  ["observationpoints"] = "观测点",
  ["observations"] = "观测",
  ["off"] = "关",
  ["open_documentation"] = "在文档中打开",
  ["operating_systems"] = "操作系统",
  ["optional_field"] = "（此字段是可选的）",
  ["os"] = "OS",
  ["other"] = "其他",
  ["others"] = "其他",
  ["outgoingflows"] = "活动输出流",
  ["overview"] = "概述",
  ["packets"] = "数据包",
  ["passive"] = "被动的",
  ["peers"] = "peers",
  ["percentage"] = "百分比",
  ["perform_autolayout"] = "执行自动布局",
  ["periodicity"] = "周期性",
  ["periodicity_map"] = "周期图",
  ["pkts"] = "Pkts",
  ["plugin_browser"] = "插件查看器：%{plugin_name}",
  ["plugin_contents"] = "插件文件",
  ["plugins"] = "插件",
  ["pollers"] = "轮询器",
  ["port"] = "端口",
  ["ports"] = "端口",
  ["posted"] = "发布于",
  ["previous"] = "以前的",
  ["protocol"] = "协议",
  ["protocol_categories"] = "协议类别",
  ["protocols"] = "协议",
  ["purged_error_message"] = "它很可能已过期,并且%{product}已将其从内存中清除。 您可以从<a href=\"%{url}\"> <i class = \" fas fa-flask \"> </i>首选项</a>设置清除空闲超时设置。",
  ["push_notification"] = "推送通知",
  ["quotas"] = "配额",
  ["ram_available"] = "可用的",
  ["ram_total"] = "总计",
  ["ram_used"] = "使用的",
  ["reason"] = "原因",
  ["received"] = "收到的",
  ["records_to_show"] = "显示记录",
  ["redirect"] = "重定向",
  ["refresh"] = "刷新",
  ["reload_graph"] = "重新加载图",
  ["reload_map"] = "重新加载地图",
  ["remote_hosts"] = "远程主机",
  ["remote_networks"] = "远程网络",
  ["remote_probe_clock_drift"] = "远程探针时钟漂移",
  ["remove"] = "移除",
  ["request_failed_message"] = "请求失败！",
  ["reset"] = "重置",
  ["reset_drops_if_message"] = "您确定要重置接口丢包计数器吗？",
  ["reset_drops_if_title"] = "重置接口丢包统计信息",
  ["reset_drops_ifs_message"] = "您确定要重置所有接口丢包计数器吗？",
  ["reset_drops_ifs_title"] = "重置接口丢包状态",
  ["reset_if_message"] = "您确定要重置接口统计信息吗？",
  ["reset_if_title"] = "重置接口统计信息",
  ["reset_ifs_message"] = "您确定要重置所有接口统计信息吗？",
  ["reset_ifs_title"] = "重置接口统计信息",
  ["restore"] = "恢复",
  ["retry"] = "重试",
  ["role"] = "角色",
  ["running"] = "运行中",
  ["save"] = "保存",
  ["save_configuration"] = "保存配置",
  ["save_settings"] = "保存设置",
  ["score"] = "分数",
  ["score_as_client"] = "作为客户端得分",
  ["score_as_server"] = "作为服务器得分",
  ["scripts"] = "脚本",
  ["search"] = "搜索",
  ["search_host"] = "搜索",
  ["seen_since"] = "发现自从",
  ["sent"] = "发送",
  ["server"] = "服务器",
  ["server_to_client"] = "服务器到客户端",
  ["service"] = "服务",
  ["service_acceptance"] = "服务状态",
  ["service_map"] = "服务地图",
  ["set"] = "设置",
  ["settings"] = "设置",
  ["short_broadcast"] = "B",
  ["short_multicast"] = "M",
  ["shortcuts"] = "快捷键",
  ["show_hide_filters"] = "显示/隐藏过滤器",
  ["showing_x_to_y_rows"] = "显示%{tot}行中的%{x}至%{y}",
  ["similarity"] = "相似度",
  ["speed"] = "速度",
  ["srv_port"] = "Srv端口",
  ["ssh"] = "SSH",
  ["start"] = "开始",
  ["status"] = "状态",
  ["stop"] = "停止",
  ["success"] = "成功",
  ["summary"] = "概括",
  ["system"] = "系统",
  ["system_alerts_status"] = "警告",
  ["system_interface"] = "系统接口",
  ["system_interfaces_status"] = "接口",
  ["system_status"] = "系统",
  ["talkers"] = "Talkers",
  ["tcp_flags"] = "TCP标志",
  ["tcp_stats"] = "TCP统计",
  ["telemetry"] = "遥测",
  ["threshold"] = "阈值",
  ["throughput"] = "吞吐量",
  ["time"] = "时间",
  ["time_in"] = "在",
  ["timeframe"] = "时间段",
  ["tls"] = "TLS",
  ["tls_certificate"] = "TLS证书",
  ["today"] = "今天",
  ["toggle_dark_theme"] = "切换暗黑主题",
  ["toggle_white_theme"] = "切换高亮主题",
  ["too_many_exports"] = "流导出丢失",
  ["too_many_flows"] = "流太多",
  ["too_many_hosts"] = "主机过多",
  ["tools"] = "工具",
  ["total"] = "总计",
  ["total_alerts"] = "警告总数",
  ["total_incoming_alerted_flows"] = "作为服务器的警告流总数",
  ["total_incoming_unreachable_flows"] = "作为服务器的不可达流量总端口",
  ["total_outgoing_alerted_flows"] = "作为客户端的警告流总数",
  ["total_outgoing_unreachable_flows"] = "作为客户端的不可达流量总端口",
  ["total_retransmissions_rcvd"] = "接收的总重传",
  ["total_retransmissions_sent"] = "发送的总重传",
  ["total_score"] = "总得分",
  ["traffic"] = "流量",
  ["traffic_policy"] = "流量策略",
  ["undo"] = "撤消",
  ["unicast_only"] = "仅单播",
  ["unknowers"] = "未知流量容量",
  ["unknown"] = "未知",
  ["unlimited"] = "&infin;",
  ["unreachable"] = "不可达",
  ["uploaders"] = "上传量",
  ["used_by"] = "使用者",
  ["value"] = "值",
  ["values"] = "值",
  ["version"] = "您的版本是%{vers}。",
  ["via"] = "经由",
  ["victim"] = "是被攻击者",
  ["views"] = "查看",
  ["vlan"] = "VLAN",
  ["vlan_id"] = "VLAN",
  ["volume"] = "容量",
  ["volume_size"] = "容量大小",
  ["warning"] = "警告",
  ["week"] = "周",
  ["welcome"] = "欢迎",
  ["welcome_to"] = "欢迎使用%{product}",
  ["wrong_date_range"] = "无效的日期范围,请进行更改。",
  ["yes"] = "是",
  [" active_monitoring_stats"] = {
    [" active_monitoring"] = "活动监控",
    [" active_monitoring_description"] = "监控特定主机的状态和响应时间",
    [" add_icmp"] = "添加ICMP监视器",
    [" add_pool"] = "新的活动监控池",
    [" add_record"] = "添加活动的监控记录",
    [" alert_status"] = "警告状态",
    [" alerted"] = " 警告",
    [" am_note_http"] = "测量HTTP（S）使用HTTP和HTTPS检索网页。",
    [" am_note_icmp"] = " ICMP在Windows上不可用。",
    [" app_connect_descr"] = " \"应用连接\"时间是完成TLS握手所花费的时间。",
    [" availability"] = "可用性",
    [" bad_action_param"] = "错误的\"操作\"参数",
    [" config_reset_confirm"] = "您是否确认要放弃当前配置并将其重置为默认值？",
    [" confirm_delete"] = "您是否确认要删除此记录？",
    [" default_interface"] = "默认接口",
    [" download_speed"] = "下载速度",
    [" edit_record"] = "编辑活动监控记录",
    [" empty_action"] = "出了点问题（空操作）。请重新加载页面,然后重试。",
    [" host_add_ok"] = "主机%{host}已成功添加！",
    [" host_delete_ok"] = "主机%{host}已成功删除！",
    [" host_edit_ok"] = "主机%{host}已成功编辑！",
    [" host_exists"] = "主机\"%{host} \"已经存在",
    [" host_not_exists"] = "找不到主机\"%{host}\"",
    [" http_download"] = "吞吐量",
    [" icmp_continuous"] = "连续ICMP",
    [" icmp_continuous_v6"] = "连续ICMP（IPv6）",
    [" invalid_combination"] = "无效的测量/主机组合",
    [" invalid_host"] = "指定了无效的主机",
    [" jitter"] = "平均RTT/抖动",
    [" last_measurement"] = "上次测量",
    [" measurement"] = " 测量",
    [" measurement_time"] = "测量时间",
    [" msec"] = " ms",
    [" no_updates_yet"] = "尚无更新",
    ["not_alerted"] = "没有警告",
    ["note3"] = "该插件会向配置的主机生成流量,以执行测量。",
    ["note_alert"] = "当计算出的测量值超过阈值设置时,将触发警告。",
    ["note_availability"] = "可用性百分比显示最后一天测量值在配置的阈值内的时间百分比。",
    ["note_periodicity_change"] = "更改周期会导致删除旧的时间序列数据。",
    ["other_http_descr"] = " \"名称查询\" 和 \"其他\"的总和是完成HTTP请求所花费的总时间。",
    ["other_https_descr"] = "\"应用连接\",\"名称查询\"和\"其他\"的总和是完成HTTPS请求所花费的总时间。",
    ["pool"] = "池",
    ["rtt_vs_jitter"] = "平均RTT/抖动",
    ["speedtest"] = "速度测试",
    ["throughput"] = "吞吐量",
    ["upload_speed"] = "上传速度",
    ["validation"] = {
      ["abort_add"] = "中止添加活动监控记录吗？",
      ["abort_edit"] = "中止编辑活动监控记录吗？",
    },
  },
  ["about"] = {
    [" startup_line"] = "启动行",
    [" suggest_feature"] = "建议一个功能",
    [" system"] = "系统",
    [" system_id"] = "系统ID",
    [" telegram"] = "帮助和新闻",
    [" telemetry_data_opt_out_msg"] = "通过将加密的<a href=\"%{tel_url}\">匿名遥测数据</a>发送到<a href=\"%{ntop_org}\"> ntop来为项目做出贡献 </a>。",
    [" too_many_exports"] = "重新启动%{product},移除<code> -F </code>选项或使用更快的存储空间。",
    [" upgrade_to_professional"] = "升级到Pro /Enterprise版本",
    [" uptime"] = "正常运行时间",
    [" user_scripts"] = "用户脚本",
    [" version"] = "版本",
    [" you_have_too_many_flows"] = "%{product}的流量过多,导致丢包和CPU高负载。请调整-X或单击下面的按钮以将当前数量翻倍。然后重启%{product}",
    [" you_have_too_many_hosts"] = "%{product}的主机过多,导致丢包和CPU高负载。请调整-x或单击下面的按钮以将当前数量翻倍。然后重启%{product}",
    [" you_need_to_sync_remote_probe_time"] = "由于<a href=\"%{url}\">时钟漂移</A>太大,您必须同步ntopng和远程探针主机的时钟。这将影响收集的流数据。 ",
    ["about"] = "关于",
    ["about_x"] = "关于%{product}",
    ["active"] = "活动的",
    ["alert_defines"] = "警告和流状态定义",
    ["alert_queries"] = "查询",
    ["alerts_dropped"] = "丢弃",
    ["alerts_stored"] = "已储存",
    ["built_on"] = "构建在",
    ["callback_directories"] = "检查目录",
    ["checks"] = "检查",
    ["configure_dhcp_range"] = "配置DHCP范围",
    ["contribute_to_project"] = "贡献",
    ["copyright"] = "版权",
    ["cpu_load"] = "CPU负载",
    ["cpu_states"] = "CPU状态",
    ["create_license"] = "我们建议您根据您收到的说明创建<code> ntopng.license </code>文件,以避免在Redis清理时丢失许可证",
    ["create_license_l"] = "请注意,您还需要创建<code> /etc/ntopng.license </code>文件,因为该文件被其他配套应用程序使用。",
    ["data_directory"] = "数据",
    ["datadir_warning"] = "请<A HREF=\"https://www.ntop.org/support/faq/migrate-the-data-directory-in-ntopng/\">迁移数据</A>",
    ["defs_directories"] = "定义目录",
    ["dhcp_monitoring_title"] = "监控DHCP主机",
    ["dhcp_range_missing_warning"] = "配置<a href='%{dhcp_url}'> DHCP范围</a>,以使<a href='%{url}'>%{name} </a>首选项生效。",
    ["directories"] = "目录",
    ["dropped_store_queries_alerts"] = "查询/存储/丢弃警告",
    ["flow_callbacks_directory"] = "流",
    ["forced_community_notification"] = "Community版使用<code>-community </code>强制安装了ntop许可证。 请删除此选项以启用已安装的许可证。",
    ["forced_community_notification_short"] = "Community版强制使用--Community",
    ["host_callbacks_directory"] = "主机",
    ["host_identifier_warning"] = "此接口正在监控DHCP主机。 请考虑将<a href='%{url}'>%{name} </a>首选项更改为\" MAC地址\"。",
    ["idle"] = "空闲",
    ["interface_callbacks_directory"] = "接口",
    ["iowait"] = "iowait",
    ["last_log"] = "最新日志跟踪",
    ["licence"] = "许可证",
    ["licence_expires_in"] = "许可在%{time}后到期",
    ["licence_generation"] = "单击上面的URL生成您的专业版本许可证,或在<a class='ntopng-external-link' href=\"%{purchase_url}\">电子商店购买 <i class =\"fas fa-external-link-alt\"> </i></a>。<br>如果您是非营利组织,研究机构或教育机构,请阅读<a class='ntopng-external-link' href=\"%{universities_url}\">此处</a> <i class = \" fas fa-external-link-alt \"> </i>。",
    ["licence_status"] = "许可证",
    ["maintenance"] = "维护",
    ["maintenance_expired"] = "%{product}维护已过期",
    ["maintenance_left"] = "直到%{_ until} [还剩%{days_left}天]",
    ["maxmind"] = "包括对<a class='ntopng-external-link' href=\"%{maxmind_url}\"> MaxMind <i class = \" fas fa-external-link-alt \"> </i> </a>和<a class='ntopng-external-link' href='https： //db-ip.com'>DB-IP <i class = \" fas fa-external-link-alt \"> </i> </a>地理位置数据库的支持。",
    ["network_callbacks_directory"] = "网络",
    ["new_major_available"] = "新的%{product}稳定版本（v.%{version}）可在<a href=\"%{url}\">下载</a>：请更新。",
    ["ntop_blog"] = "ntop博客",
    ["platform"] = "平台",
    ["ram_memory"] = "RAM",
    ["readme_and_manual"] = "用户手册",
    ["report_issue"] = "报告问题",
    ["runtime_status"] = "运行状态",
    ["save_licence"] = "保存许可证",
    ["scripts_directory"] = "脚本",
    ["server_time"] = "服务器时间",
    ["specify_licence"] = "在此处指定您的ntopng许可",
  },
  ["activities_page"] = {
    ["host_activity"] = "主机活动",
  },
  ["activity_descriptions"] = {
    ["5min"] = "执行5分钟的任务。 使用RRD时,还会写入主机时间序列数据。",
    ["daily"] = "执行日常任务。",
    ["dequeue_flows_for_hooks"] = "一项关键活动,该活动识别闲置的哈希表条目并清除它们,在流上执行用户脚本并导出流。",
    ["discover"] = "扫描网络中的设备。 如果未请求/计划扫描,则此活动不执行任何操作。",
    ["hourly"] = "执行小时任务。",
    ["housekeeping"] = "检查警告以存储在磁盘上或进行通知,执行n2disk提取,并在需要时重新加载类别列表。",
    ["minute"] = "执行分钟的任务并写入分钟级时间序列。 使用InfluxDB时,还会写入主机时间序列数据。",
    ["second"] = "写入接口时间序列和其他高频数据。",
    ["stats_update"] = "计算活动流量元素的吞吐量。",
    ["timeseries"] = "检查缓冲的时间序列点（Point）并将其导出到InfluxDB。",
  },
  ["alert"] = {
    ["alert_error"] = {
      ["configuration"] = {
        ["no_attacker"] = "攻击者请求,但没有设置攻击者。",
        ["no_status_key"] = "状态密钥请求,但未找到状态密钥。",
        ["no_victim"] = "被攻击者请求,但没有设置被攻击者。",
      },
    },
    ["anomalies"] = {
      ["active_flows_as_client"] = "作为客户端的活动流",
      ["active_flows_as_server"] = "作为服务器的活动流",
      ["contacts"] = "通信行为",
      ["score_as_client"] = "作为客户端得分",
      ["score_as_server"] = "作为服务器得分",
    },
  },
  ["alert_behaviors"] = {
    ["l7_traffic_behavior"] = "流量 RX/TX",
    ["score_behavior"] = "分数",
    ["traffic_rx_behavior"] = "流量 RX",
    ["traffic_tx_behavior"] = "流量 TX",
  },
  ["alert_entities"] = {
    ["am_host"] = "活动监控",
    ["asn"] = "ASN",
    ["flow"] = "流",
    ["host"] = "主机",
    ["host_pool"] = "主机池",
    ["interface"] = "接口",
    ["l7"] = "L7 协议",
    ["mac"] = "设备",
    ["network"] = "本地网络",
    ["other"] = "其他",
    ["snmp_device"] = "SNMP设备",
    ["system"] = "系统",
    ["test"] = "测试",
    ["user"] = "用户",
  },
  ["alert_messages"] = {
    [" anomalous_tcp_flags"] = "%{entity}的%{sent_or_rcvd}相对于SYN，TCP RST标志太多[比率：%{ratio}%%]",
    ["a_new_device_has_connected"] = "新设备<a href=\"%{url}\">%{device} </a>已连接到网络。",
    ["attack_mitigation_via_snmp_failure"] = "无法在SNMP设备<a href=\"%{url}\">上设置接口<a href=\"%{port_url}\">%{port} </a>管理员状态%{device} </a>到%{admin_down}：%{granularity} <b>%{metric} </b>与%{entity} [%{value}%{op}%{threshold}]交叉。确保SNMP设备已配置（有效）写community 。",
    ["attack_mitigation_via_snmp_success"] = "接口<a href=\"%{port_url}\">%{port} </a> SNMP设备<a href=\"%{url}\">%{device上的管理员状态} </a>设置%{admin_down}：%{granularity} <b>%{metric} </b>与%{entity} [%{value}%{op}%{threshold}]交叉",
    ["broadcast_domain_info"] = "由于这些IP似乎属于不同的广播域,因此不太可能看到这些IP之间的ARP通信。请检查主机和网络配置。",
    ["broadcast_domain_too_large"] = "来自<a href=\"%{src_mac_url}\">%{src_mac} </a> /<a href=\"%{spa_url}\">%{spa} < /a>检测到<a href=\"%{dst_mac_url}\">%{dst_mac} </a> /<a href=\"%{tpa_url}\">%{tpa} </a>",
    ["contacted_peers"] = "%{host}%{host_category}通信的Peers太多。作为客户端：[%{value_cli}>%{dyn_threshold_cli}]。作为服务器：[%{value_srv}>%{dyn_threshold_srv }]。",
    ["contacted_peers_as_cli"] = "%{host}%{host_category} [%{value_cli}>%{dyn_threshold_cli}]作为客户端通信的Peers太多。",
    ["contacted_peers_as_srv"] = "%{host}%{host_category} [%{value_srv}>%{dyn_threshold_srv}]作为服务器通信的Peers太多。",
    ["device_has_connected"] = "设备<a href=\"%{url}\">%{device} </a>已连接到网络。",
    ["device_has_disconnected"] = "设备<a href=\"%{url}\">%{device} </a>已与网络断开连接。",
    ["flow_flood_attacker"] = "%{entity}%{host_category}流量泛洪[%{value}＆gt;%{threshold}客户端flow/s,持续约3秒钟]",
    ["flow_flood_victim"] = "%{entity}%{host_category}受到流量泛洪攻击[%{value}＆gt;%{threshold}服务器flow/s,持续约3秒]",
    ["flow_number_anomaly"] = "%{role} 流: %{value} [期望范围: %{lower_bound} ... %{upper_bound}]",
    ["host_alert_dangerous_host"] = "分数超出限制 %{entity}. 危险主机 [分数: %{score}][持续时间: %{duration} mins]",
    ["host_alert_dns_server_contacts"] = "%{entity}连接的DNS服务器太多[%{value}＆gt;%{threshold}服务器]",
    ["host_alert_dns_traffic"] = "DNS流量超出%{entity} [%{value}%{op}%{threshold}]]",
    ["host_alert_ntp_server_contacts"] = " %{entity}连接的NTP服务器太多[%{value}＆gt;%{threshold}服务器]",
    ["host_alert_p2p_traffic"] = "%{entity}P2P流量超出[%{value}%{op}%{threshold}]]",
    ["host_alert_score"] = "%{entity}分数超出[%{value}%{op}%{threshold}]]",
    ["host_alert_smtp_server_contacts"] = "%{entity}连接的SMTP服务器太多[%{value}＆gt;%{threshold}服务器]]",
    ["host_entity"] = "主机%{entity_value}",
    ["host_log"] = "从<a href=\"%{url}\">%{host} </a>（%{facility}）%{line}登录",
    ["host_pool_has_connected"] = "主机池<a href=\"%{url}\">%{pool} </a>已连接到网络。",
    ["host_pool_has_disconnected"] = "主机池<a href=\"%{url}\">%{pool} </a>已与网络断开连接。",
    ["host_remote_to_remote"] = "远程主机<a href=\"%{url}\">%{ip} </a>%{mac}与远程主机通信。< a href = \"%{flow_alerts_url} \">流警告页面</a>。",
    ["http_download_failed"] = "主机%{host} %{numeric_ip}吞吐量检测失败",
    ["iface_alerts_dropped"] = "接口<a href=\"%{url}\">%{iface} </a>正在丢失警告。可能生成了太多警告,或者接收者速度很慢。",
    ["influxdb_dropped_points"] = "Point丢失。无法将它们发送到\"%{influxdb} \"。",
    ["influxdb_migration_msg"] = "您有太多本地主机。请考虑将<a href='%{url}'>从RRD移至InfluxDB </a>以进行时间序列存储,以提高性能。",
    ["influxdb_partial_write"] = "这可以通过在InfluxDB服务器配置中设置\" max-values-per-tag = 0 \"来解决。",
    ["influxdb_write_error"] = "将时间序列数据发送时出错\"%{influxdb} \"：%{err}",
    ["interface_entity"] = "接口%{entity_value}",
    ["ip_outside_dhcp_range_router_ip"] = "[路由器：<a href=\"%{router_url}\">%{router_ip} </a>]",
    ["ip_outsite_dhcp_range"] = "设备<a href=\"%{client_url}\">%{client_mac} </a>的IP地址为<a href=\"%{client_ip_url}\">%{client_ip} </a>不在<a href=\"%{dhcp_url}\">配置的DHCP范围</a>中。[发送方：<a href=\"%{sender_url}\">%{sender_mac} < /a>]",
    ["lldp_arc_added"] = "在SNMP设备<a href=\"%{url1}\">%{node1} </a>和<a href=\"%{url2}\">%{ node2} </a>之间增加了新的链路",
    ["lldp_arc_removed"] = "在SNMP设备<a href=\"%{url1}\">%{node1} </a>和<a href=\"%{url2}\">%{ node2} </a>之间删除了链路",
    ["mac_ip_association_change"] = "IP%{ip}将关联从<a href=\"%{old_mac_url}\">%{old_mac} </a>更改为<a href=\"%{new_mac_url}\"> %{new_mac} </a>",
    ["measurement_too_high_msg"] = "对于%{host}%{numeric_ip} [%{am_value}%{unit}>%{threshold}%{unit}],测量值过高。",
    ["measurement_too_low_msg"] = "对于%{host}%{numeric_ip} [%{am_value}%{unit} <%{threshold}%{unit}],测量值太低。",
    ["misbehaving_flows_ratio"] = "%{entity}的%{sent_or_rcvd}异常流量与总流量之比[Ratio：%{ratio}%%],",
    ["network_entity"] = "网络%{entity_value}",
    ["nfq_flushed"] = "接口 <a href=\"%{url}\">%{name}</a> 数据包队列已刷新. 队列 %{pct}%%填充了%{tot}数据包和%{dropped}丢失.",
    ["ngi_trust_event"] = "在%{time}检测到设备<a href=\"%{mac_url}\">%{mac} </a>的异常活动。[最后状态：%{last_state}] [未更改由于：%{state_unchanged_since}] [异常等级：%{abnormality_grade}]",
    ["ngi_trust_event_released"] = "设备<a href=\"%{mac_url}\">%{mac} </a>状态在%{time}恢复正常。[最后状态：%{last_state}] [自以下日期以来未更改：%{state_unchanged_since}] [异常等级：%{abnormality_grade}]",
    ["no_influxdb"] = "请<A HREF=\"%{url}\">启用时间序列转储到InfluxDB </A>,然后重新加载此页面",
    ["ntopng_anomalous_termination"] = "在异常终止（<a href=\"%{url}\">错误报告</a>）之后开始",
    ["ntopng_start"] = "已开始",
    ["ntopng_stop"] = "已停止",
    ["ntopng_update"] = "软件更新后开始",
    ["open_files_limit_too_small"] = "Ntopng检测到MySQL可以打开的最大文件数可能太小。 由于诸如[打开文件'./ntopng/flowsv6#P#p22.MYD'时资源不足（错误代码：24-打开的文件太多）] [23]之类的错误,这可能导致流数据丢失。 确保增加open_files_limit,或者,如果您只是想忽略此警告,请从首选项中禁用该检查。",
    ["periodic_activity_not_executed"] = "定期活动\"%{script} \"已排队,但尚未执行。 没有可用于执行的线程。 [自%{pending_since}起待处理]",
    ["periodicity_update_ended"] = "定期流结束：%{host_info1} <i class = \" fas fa-exchange-alt fa-lg \" aria-hidden = \" true \" data-original-title = \" \" title = \" \" > </i>%{host_info2} [%{l7_proto}]%{info}",
    ["periodicity_update_new"] = "%{host_info1} <i class = \" fas fa-exchange-alt fa-lg \" aria-hidden = \" true \" data-original-title = \" \" title = \" \"> </i >%{host_info2} [%{l7_proto}]%{info}现在是周期性的（<i class = \" fas fa-wave-square \" title = \" Frequency \"> </i>%{frequency} sec ）",
    ["ping_host_unreachable_v3"] = "主机%{host}%{numeric_ip}无法访问。",
    ["potentially_dangerous_protocol_description"] = "由于使用了过时或易受攻击的协议,因此该流量可能对网络有害。",
    ["proto_bytes_quotas"] = "%{proto}字节",
    ["proto_time_quotas"] = "%{proto}时间",
    ["remote_to_local_insecure_proto"] = "远程到本地不安全协议[类别：%{ndpi_category}] [Breed：%{ndpi_breed}]",
    ["remote_to_remote"] = "远程%{entity}%{host_category}连接了远程主机",
    ["score_number_anomaly"] = "检测到主机%{host}的得分异常为%{role} [得分：%{score}] [下限：%{lower_bound}] [上限：%{upper_bound}]",
    ["shell_script_executed"] = "响应警告而执行的Shell脚本'%{script_exec_comm}'：'%{alert_type}' ",
    ["slow_periodic_activity"] = "定期活动\"%{script} \"运行时间过长[超过%{max_duration}]或执行得太晚（在队列中被阻止）。",
    ["slow_purge"] = "在%{iface}上<a href=\"%{url}\">哈希表空闲条目</a>清除太慢。 这可能导致较高的内存利用率,数据准确性丢失和警告丢失。 [%{idle}%%＆gt%{max_idle}%%]",
    ["speedtest_failed"] = "速度测试执行失败",
    ["subject_quota_exceeded"] = "主机池<a href=\"%{url}\">%{pool} </a>超出了%{subject}配额[%{value}＆gt; %{quota}]",
    ["suspicious_dga_domain"] = "可疑DGA域[%{domain}]",
    ["syn_flood_attacker"] = "%{entity}SYN泛洪[%{value}＆gt; %{threshold}SYN/s,持续约3秒钟]",
    ["syn_flood_victim"] = "%{entity}受到SYN Flood攻击[%{value}＆gt; 收到%{threshold}SYN/s,持续约3秒钟]",
    ["syn_scan_attacker"] = "%{entity}是SYN扫描攻击者，发送[%{value}＆gt; %{threshold}个SYN]",
    ["syn_scan_victim"] = "%{entity}被SYN扫描，收到[%{value}＆gt; %{threshold}个SYN]",
    ["test_failed"] = "测试失败",
    ["threshold_crossed"] = "%{granularity} <b>%{metric} </b>与%{entity}%{host_category}交叉[%{value}%{op}%{threshold}]",
    ["too_many_drops"] = "%{iface}丢包太多[%{perc_drops}%%＆gt%{max_drops}%%]",
    ["too_many_flows"] = "%{iface}的流过多。 请增加%{option}命令行选项。",
    ["too_many_flows_details"] = "您是否想将当前最大流增加一倍？",
    ["too_many_flows_err"] = "无法将最大流加倍。 请从配置文件中调整-X并重新启动ntopng。",
    ["too_many_flows_title"] = "双倍最大流",
    ["too_many_hosts"] = "%{iface}的主机过多。请增加%{option}命令行选项。",
    ["too_many_hosts_details"] = "您想将当前的最大主机数量加倍吗？",
    ["too_many_hosts_err"] = "无法将最大主机数加倍。 请从配置文件中调整-X并重新启动ntopng。",
    ["too_many_hosts_title"] = "双倍最大主机数量",
    ["unknown_contacted_peers"] = "%{host}%{host_category}连接的Peers太多。",
    ["x_alerts"] = "%{num}个警告",
  },
  ["alerts_dashboard"] = {
    ["abort_add_filter"] = "中止添加过滤器？",
    ["active_flows_anomaly"] = "异常活动流",
    ["add_filter"] = "添加过滤器",
    ["alert"] = "警告",
    ["alert_counts"] = "计数",
    ["alert_duration"] = "持续时间",
    ["alert_periodicity_update"] = "流周期已更改",
    ["alert_severity"] = "严重性",
    ["alert_snmp_device_reset_description"] = "SNMP 设备 <a href=\"%{url}\">%{device}</a>已重新启动",
    ["alert_snmp_device_reset_title"] = "SNMP 设备重启",
    ["alert_snmp_interface_mac_changed_description"] = "MAC <a href=\"%{mac_url}\">%{mac}</a> 发现于SNMP设备 <a href=\"%{url}\">%{device}</a> 接口 <a href=\"%{port_url}\">%{port}</a> 之前在 SNMP 设备 <a href=\"%{prev_device_url}\">%{prev_device}</a> 接口 <a href= \"%{prev_port_url}\">%{prev_port}</a>",
    ["alert_snmp_interface_mac_changed_title"] = "SNMP接口MAC已更改",
    ["alert_type"] = "类型",
    ["alert_unexpected_behavior_description"] = "当从指定的%{type}/s识别出异常行为时触发警报。启用/禁用警报后需要重新启动。",
    ["alert_unexpected_behavior_title"] = "异常%{type}行为",
    ["alerts"] = "警告",
    ["alerts_dashboard"] = "警告仪表板",
    ["alerts_ts"] = "进程警告时间序列",
    ["alerts_ts_description"] = "生成进程警告时间序列",
    ["all_hosts"] = "所有主机",
    ["anomalous_tcp_flags"] = "异常TCP标志",
    ["attack_mitigation_snmp_description"] = "当客户端分数超过指定阈值时,将主机SNMP访问端口管理状态设置为关闭",
    ["attack_mitigation_snmp_title"] = "通过SNMP缓解攻击",
    ["blacklisted_country"] = "黑名单国家",
    ["blacklisted_country_descr"] = "当主机连接或被与指定国家连接时触发警告",
    ["blacklisted_flow"] = "黑名单流",
    ["blocked_flow"] = "流风险",
    ["broadcast_domain_too_large"] = "广播域",
    ["client_and_server_countries_blacklisted"] = "客户端国家 %{cli_country} 和服务器国家 %{srv_country} 被列入黑名单",
    ["client_country_blacklisted"] = "客户国家 %{country} 被列入黑名单",
    ["connection_issues"] = "TCP问题",
    ["contacted_peers_description"] = "当主机与客户端或服务器连接数量异常时触发警告",
    ["contacted_peers_title"] = "连接Peer警告",
    ["critical"] = "关键",
    ["custom_period"] = "自定义周期",
    ["debug"] = "调试",
    ["device_connection"] = "设备连接",
    ["device_disconnection"] = "设备断开",
    ["dns_anomaly"] = "DNS异常",
    ["edit_filter"] = "编辑过滤器",
    ["emergency"] = "紧急情况",
    ["engaged_for_longest"] = "过去持续时间最长的参与警报",
    ["error"] = "错误",
    ["experimental_checks"] = "实验性",
    ["experimental_checks_description"] = "启用实验性警告检查：TCP标志、异常流量比率",
    ["external_alert"] = "外部警告",
    ["fail2ban"] = "已执行Fail2Ban命令",
    ["five_mins"] = "最后5分钟",
    ["flow_alert_origins"] = "流量警告来源",
    ["flow_alert_targets"] = "流量警告目的",
    ["flow_anomaly"] = "流量异常",
    ["flow_checks_calls_drops_description"] = "%{num_drops} <a href='%{url}'>流检测</a> 未被调用，一些行为异常流可能会被忽视。",
    ["flow_flood"] = "流量泛洪",
    ["flow_flood_attacker"] = "流量泛洪攻击者",
    ["flow_flood_victim"] = "流泛洪被攻击者",
    ["flow_logger"] = "流记录器",
    ["flow_logger_descr"] = "将新流记录到控制台/文件",
    ["flow_low_goodput"] = "低Goodput率",
    ["flow_misbehaviour"] = "流异常行为",
    ["flow_risk"] = "流风险",
    ["flow_user_scripts_calls_drops_description"] = "%{num_drops} <a href='%{url}'>流用户脚本</a> 未被调用。一些行为异常流可能会被忽视。",
    ["ghost_network_detected"] = "检测到 Ghost 网络",
    ["ghost_network_detected_description"] = "子网 %{network} 不属于 <a href='%{url}'>%{entity} 网络</a>。",
    ["ghost_networks"] = "Ghost网络",
    ["ghost_networks_description"] = "在网络接口上检测到未知网络时触发警告。",
    ["host_alert_dns_server_contacts"] = "DNS服务器通信",
    ["host_alert_ntp_server_contacts"] = "NTP服务器通信",
    ["host_alert_smtp_server_contacts"] = "SMTP服务器通信",
    ["host_log"] = "主机日志",
    ["host_pool_connection"] = "主机池连接",
    ["host_pool_disconnection"] = "主机池断开连接",
    ["host_score_description"] = "计算主机分数并在超过指定阈值时触发警告。",
    ["http_rcvd_sent_ratio"] = "[HTTP Rcvd 响应，/Sent 请求。 %{ratio}%%＜ %{threshold}%%][响应：%{replies}][请求：%{requests}]",
    ["http_sent_rcvd_ratio"] = "[HTTP Sent 响应，/Rcvd 请求。 %{ratio}%%＜ %{threshold}%%][响应：%{replies}][请求：%{requests}]",
    ["icmp_anomaly"] = "ICMP异常",
    ["iec_invalid_transition"] = "IEC104 无效转换",
    ["iec_unexpected_type_id"] = "IEC104 异常类型 ID",
    ["inactivity"] = "不活动",
    ["influxdb_dropped_points"] = "InfluxDB丢失Points",
    ["influxdb_error"] = "InfluxDB错误",
    ["influxdb_export_failure"] = "InfluxDB导出失败",
    ["influxdb_monitor"] = "InfluxDB监控",
    ["influxdb_monitor_description"] = "监控InfluxDB运行状况",
    ["info"] = "信息",
    ["internal_issues"] = "内部问题",
    ["involving_msg"] = "相关的流警告",
    ["known_proto_on_non_std_port"] = "在非标准端口上的应用程序",
    ["known_proto_on_non_std_port_full"] = "在非标准端口 %{port} 上检测到已知应用程序 %{app}",
    ["last_day"] = "最后1天",
    ["last_hour"] = "最后1小时",
    ["last_minute"] = "最后1分钟",
    ["lateral_movement"] = "横向运动",
    ["lateral_movement_descr"] = "检测到新的服务%{info} %{href}",
    ["list_download_failed"] = "列表下载失败",
    ["list_download_succeeded"] = "列表下载成功",
    ["local_only"] = "仅限本地主机",
    ["local_origin_remote_target"] = "本地源 - 远程目标",
    ["login_failed"] = "登录失败",
    ["mac_ip_association_change"] = "IP/MAC更改",
    ["malicious_signature_detected"] = "检测到恶意签名 [Client JA3 Hash: %{ja3_hash}]",
    ["many_replies"] = "%{count} %{what} 响应",
    ["many_requests"] = "%{count} %{what} 请求",
    ["memory_ts"] = "内存时间序列",
    ["memory_ts_description"] = "生成进程内存时间序列",
    ["misbehaving_flows_ratio"] = "行为异常的流太多",
    ["misconfigured_app"] = "错误配置的应用程序",
    ["misconfigured_dhcp_range"] = "配置错误的DHCP范围",
    ["ndpi_dns_suspicious_traffic_title"] = "可疑的DNS流量",
    ["ndpi_http_invalid_content_title"] = "无效的HTTP内容",
    ["ndpi_http_numeric_ip_host_title"] = "HTTP numeric IP 主机",
    ["ndpi_http_suspicious_header_title"] = "HTTP可疑Header",
    ["ndpi_http_suspicious_url_title"] = "HTTP可疑URL",
    ["ndpi_http_suspicious_user_agent_title"] = "HTTP可疑用户代理",
    ["ndpi_malformed_packet_title"] = "畸形数据包",
    ["ndpi_malicious_ja3_title"] = "可能是恶意的JA3",
    ["ndpi_malicious_sha1"] = "恶意SHA1证书",
    ["ndpi_risky_asn_title"] = "有风险的ASN",
    ["ndpi_risky_domain_title"] = "有风险域名",
    ["ndpi_smb_insecure_version_title"] = "不安全的SMB版本",
    ["ndpi_ssh_obsolete_server_title"] = "过期的SSH",
    ["ndpi_suspicious_dga_domain_title"] = "可疑的DGA域",
    ["ndpi_tls_certificate_expired_title"] = "TLS证书已过期",
    ["ndpi_tls_certificate_mismatch_title"] = "TLS证书不匹配",
    ["ndpi_tls_missing_sni_title"] = "缺少TLS SNI",
    ["ndpi_tls_not_carrying_https_title"] = "TLS不携带HTTPS",
    ["ndpi_tls_obsolete_version_title"] = "过期的TLS版本",
    ["ndpi_tls_selfsigned_certificate_title"] = "TLS证书自签名",
    ["ndpi_tls_suspicious_esni_usage_title"] = "使用可疑的TLS ESNI",
    ["ndpi_tls_weak_cipher_title"] = "薄弱的TLS密码",
    ["ndpi_unsafe_protocol_title"] = "不安全的协议",
    ["ndpi_url_possible_rce_injection_title"] = "可能的 RCE",
    ["ndpi_url_possible_sql_injection_title"] = "可能的SQL注入",
    ["ndpi_url_possible_xss_title"] = "可能的XSS",
    ["new_device"] = "新设备",
    ["nfq_flushed"] = "数据包队列刷新",
    ["ngi_trust_event"] = "NG 信任事件",
    ["no_alert"] = "没有警告",
    ["no_alerts"] = "没有警告",
    ["none"] = "空",
    ["not_engaged"] = "非参与",
    ["notice"] = "通知",
    ["one_day"] = "最后一天",
    ["one_hour"] = "最后一天",
    ["one_min"] = "最后一天",
    ["one_reply"] = "%{count} %{what} 响应",
    ["one_request"] = "%{count} %{what} 请求",
    ["out_of_x_total_packets"] = "超出%{tot} 数据包",
    ["periodic_activity_not_executed"] = "未执行定期活动",
    ["periodic_activity_not_executed_descr"] = "当周期性活动排队等待执行但由于所有工作线程都忙而未执行时触发警告。",
    ["periodicity_changed"] = "周期性已更改",
    ["periodicity_changed_descr"] = "周期服务更改%{info} %{href}",
    ["ping_issues"] = "Ping",
    ["possible_bot_flow_description"] = "以Bot作为服务器的可能流",
    ["possible_bot_flow_title"] = "可能的Bot",
    ["potentially_dangerous_protocol"] = "潜在危险协议",
    ["process"] = "Process",
    ["quota_exceeded"] = "超出配额",
    ["remote_access_alert_descr"] = "远程访问结束 [持续时间：%{sec} 秒]",
    ["remote_access_description"] = "远程访问会话结束时触发警告",
    ["remote_access_title"] = "远程访问",
    ["remote_connection_alert_descr"] = "主机 %{host} 有 %{connections} 个使用远程访问协议的活动流",
    ["remote_connection_description"] = "当主机至少有一个使用远程访问协议的活动流时触发警告",
    ["remote_connection_title"] = "远程连接",
    ["remote_only"] = "仅限远程主机",
    ["remote_origin_local_target"] = "远程源 - 本地目标",
    ["remote_to_local_insecure_proto"] = "远程到本地不安全协议",
    ["remote_to_remote"] = "远程到远程",
    ["score_anomaly"] = "分数异常",
    ["score_threshold_cross"] = "分数阈值交叉",
    ["search_criteria"] = "仪表板设置",
    ["select_filter"] = "过滤器",
    ["server_country_blacklisted"] = " %{country}国家服务器被列入黑名单",
    ["shell_script"] = "已执行端点(Endpoint)Shell脚本",
    ["sites_behaviour_description"] = "当主机通信意外数量异常时触发警告",
    ["slow_periodic_activity"] = "慢周期活动",
    ["slow_periodic_activity_descr"] = "当周期性活动执行时间过长时触发警告。",
    ["slow_purge"] = "缓慢空闲清除",
    ["snmp_device_is_unresponsive"] = "SNMP设备<a href=\"%{url}\">%{device}</a> 没有响应",
    ["snmp_device_reset_script_description"] = "当检测到SNMP设备重启时触发警告",
    ["snmp_interface_mac_changed_script_description"] = "当MAC在接口或设备之间移动时触发警告",
    ["snmp_interface_mac_changed_script_title"] = "SNMP接口MAC已更改",
    ["snmp_interfaces_rsi"] = "SNMP 接口 RSI",
    ["snmp_interfaces_rsi_descr"] = "检测接口流量的变化并触发警告（实验性）",
    ["snmp_port_changed_duplex_status"] = "SNMP设备<a href=\"%{url}\">%{device}</a> 上的接口 <a href=\"%{port_url}\">%{port}</a> 更改了其双工状态到 %{new_op}",
    ["snmp_port_changed_operational_status"] = "SNMP设备<a href=\"%{url}\">%{device}</a> 上的接口 <a href=\"%{port_url}\">%{port}</a> 将其状态更改为 %{new_op}",
    ["snmp_port_duplexstatus_change"] = "接口双工状态变化",
    ["snmp_port_duplexstatus_change_description"] = "当接口双工状态改变时触发警告",
    ["snmp_port_errors"] = "接口错误",
    ["snmp_port_errors_description"] = "当接口丢弃/错误计数器增加时触发警告",
    ["snmp_port_errors_increased"] = "SNMP设备<a href=\"%{url}\">%{device}</a> 上的接口 <a href=\"%{port_url}\">%{port}</a> 上的丢弃/错误计数器增加",
    ["snmp_port_load_threshold_exceeded"] = "超出接口负载阈值",
    ["snmp_port_load_threshold_exceeded_message"] = "在 SNMP 设备 <a href=\"%{url}\">%{device}</a> 上的接口 <a href=\"%{port_url}\">%{port}</a> 上超出负载阈值 [阈值：%{threshold}%%][输入负载：%{in_load}%%][输出负载：%{out_load}%%]",
    ["snmp_port_status_change"] = "接口操作状态变化",
    ["snmp_port_status_change_description"] = "当接口的操作状态发生变化时触发警告",
    ["starting_on"] = "开始",
    ["submit"] = "更新仪表板",
    ["suspicious_activity"] = "可疑活动",
    ["suspicious_device_protocol"] = "可疑设备协议",
    ["binary_application_transfer"] = "可疑文件传输",
    ["binary_application_transfer_url"] = "%{type_icon} 检测到可疑文件传输 [%{url}]",
    ["tcp_syn_flood"] = "TCP SYN 泛洪",
    ["tcp_syn_flood_attacker"] = "TCP SYN 泛洪攻击者",
    ["tcp_syn_flood_victim"] = "TCP SYN 泛洪被攻击者",
    ["tcp_syn_scan"] = "TCP SYN 扫描",
    ["tcp_syn_scan_attacker"] = "TCP SYN 扫描攻击者",
    ["tcp_syn_scan_victim"] = "TCP SYN 扫描被攻击者",
    ["threashold_cross"] = "阈值交叉",
    ["tls_old_protocol_version"] = "过期的 TLS 版本 [版本：%{version}]",
    ["tls_protocol_version"] = "TLS 版本：%{version}",
    ["too_many_drops"] = "丢包",
    ["too_many_flows"] = "过多流警告",
    ["too_many_flows_description"] = "当活动流数量过高时触发警告",
    ["too_many_hosts"] = "主机过多警告",
    ["too_many_hosts_description"] = "当活动主机数量过高时触发警告",
    ["top_addresses"] = "Top Mac",
    ["top_alerts"] = "Top 警告",
    ["top_cli"] = "Top 客户端",
    ["top_hosts"] = "Top 主机",
    ["top_networks"] = "Top 网络",
    ["top_srv"] = "Top 服务器",
    ["total_alerts"] = "警告总数",
    ["trailing_msg"] = "时间窗口",
    ["trailing_msg_compact"] = "窗口",
    ["unexpected_host_behavior_description"] = "%{host} [%{type_of_behaviour}][值：%{value}][期望范围：%{lower_bound} ... %{upper_bound}]",
    ["unexpected_host_behaviour_act_flows_title"] = "异常主机活动流",
    ["unexpected_host_behaviour_contacts_title"] = "异常主机通信",
    ["unexpected_host_behaviour_score_title"] = "异常主机评分",
    ["unresponsive_device"] = "无响应设备",
    ["user_activity"] = "用户活动",
    ["user_scripts_calls_drops"] = "用户脚本呼叫中断",
    ["warning"] = "警告",
    ["web_mining"] = "Web挖掘",
    ["x_lost"] = "%{lost} 丢包",
    ["x_ooo"] = "%{ooo} 乱序",
    ["x_retx"] = "%{retx} 重传",
  },
  ["alerts_thresholds_config"] = {
    ["active_local_hosts"] = "本地主机警告",
    ["active_local_hosts_threshold_descr"] = "当活动本地主机的数量超过阈值时触发警告",
    ["activity_time"] = "活动时间警告",
    ["alert_active_description"] = "当活动时间Delta超过阈值时触发警告",
    ["alert_bytes_description"] = "当第 2 层字节Delta（发送 + 接收）超过阈值时触发警告",
    ["alert_bytes_title"] = "流量警告",
    ["alert_dns_description"] = "当DNS流量的第 2 层字节Delta（发送 + 接收）超过阈值时触发警告",
    ["alert_flows_description"] = "当流Delta（作为客户端 + 作为服务器）超过阈值时触发警告",
    ["alert_flows_title"] = "流警告",
    ["alert_idle_description"] = "当空闲时间（自上次看到数据包以来的时间）超过阈值时触发警告",
    ["alert_network_egress_description"] = "当发送字节Delta超过阈值时触发警告",
    ["alert_network_ingress_description"] = "当接收字节Delta超过阈值时触发警告",
    ["alert_network_inner_description"] = "当内部字节Delta超过阈值时触发警告",
    ["alert_p2p_description"] = "当P2P流量的第2层字节Delta（发送 + 接收）超过阈值时触发警告",
    ["alert_packets_description"] = "当数据包Delta（发送 + 接收）超过阈值时触发警告",
    ["alert_packets_title"] = "数据包警告",
    ["alert_slow_purge_threshold"] = "空闲哈希表条目警告",
    ["alert_slow_purge_threshold_descr"] = "当哈希表中空闲条目占条目总数的百分比超过阈值时触发警告",
    ["alert_throughput_description"] = "当平均吞吐量（发送 + 接收）超过阈值时触发警告",
    ["common_thresholds_local_sources"] = "%{source} 常用配置",
    ["daily"] = "每天",
    ["dns_contacts_description"] = "当连接不同DNS服务器的数量超过阈值时触发警告",
    ["dns_contacts_title"] = "DNS服务器连接警告",
    ["dns_traffic"] = "DNS流量警告",
    ["egress_traffic"] = "出口流量警告",
    ["every_5_minutes"] = "每5分钟",
    ["every_minute"] = "每分钟",
    ["flows_anomaly_description"] = "检测活动流中的异常",
    ["flows_anomaly_title"] = "流异常",
    ["hourly"] = "每小时",
    ["idle_time"] = "空闲时间警告",
    ["ingress_traffic"] = "入口流量警告",
    ["inner_traffic"] = "内部流量警告",
    ["manual"] = "手册",
    ["note_add_custom_scripts"] = "%{product} 在这些 <a href=\"%{url}\"> <i class=\"fas fa-folder\"></i>目录</a>中查找自定义脚本。",
    ["note_checks_on_active_hosts"] = "仅对活动主机执行检查。",
    ["note_consecutive_checks"] = "不一定要对连续期间执行连续检查。 例如,如果主机处于空闲状态,则在它再次变为活动状态之前不会检查其阈值。",
    ["note_control_threshold_checks_periods"] = "这些选项卡中列出的回调会定期执行。",
    ["note_create_custom_scripts"] = "创建用于执行回调程序的自定义脚本的文档可在以下 <a class='ntopng-external-link' href=\"%{url}\">链路 <i class=\"fas fa-external-link-alt\"> </i></a>。",
    ["note_thresholds_expressed_as_delta"] = "一些回调程序计算一个Delta。 该Delta是两个相同数量的连续执行之间差异。",
    ["notes"] = "注意事项",
    ["ntp_contacts_description"] = "当连接的NTP服务器数量超过阈值时触发警告",
    ["ntp_contacts_title"] = "NTP服务器连接警告",
    ["p2p_traffic"] = "P2P流量警告",
    ["score_anomaly_description"] = "检测主机得分异常",
    ["score_anomaly_title"] = "分数异常",
    ["smtp_contacts_description"] = "当连接的SMTP服务器的数量超过阈值时触发警告",
    ["smtp_contacts_title"] = "SMTP 服务器连接警告",
    ["threshold_type"] = "回调",
    ["thresholds_single_source"] = "%{source} %{alt_name} 配置",
    ["throughput"] = "吞吐量警告",
  },
  ["appliance"] = {
    ["capture_interfaces"] = "捕获接口",
    ["enable_wifi"] = "启用 Wi-Fi",
    ["enable_wifi_descr"] = "启用或禁用 Wi-Fi 接口。",
    ["management"] = "管理接口",
    ["ssid"] = "SSID",
    ["ssid_descr"] = "Wi-Fi 名称（服务集标识符）。",
    ["wpa_passphrase"] = "密码",
    ["wpa_passphrase_descr"] = "WPA 密码（最少 8 个字符且少于 64 个）。",
  },
  ["arp_page"] = {
    ["arp_type"] = "ARP类型",
  },
  ["as_details"] = {
    ["as"] = "自治系统",
    ["as_parameter_missing_message"] = "自治系统参数丢失（内部错误？）",
    ["as_timeseries_enable_message"] = "可以从 <A HREF=\"%{url}\">%{icon_flask} Preferences</A> 启用自治系统时间序列。 需要几分钟才能看到第一个数据点。",
    ["no_available_data_for_as"] = "自治系统%{asn} 没有可用数据。",
  },
  ["as_stats"] = {
    ["autonomous_systems"] = "自治系统",
  },
  ["blog_feed"] = {
    ["blog_updates"] = "博客更新",
    ["blog_updates_info"] = "在以下位置发布新博客时收到通知：<a href='https://www.ntop.org/blog' target='_blank'>https://ntop.org/blog</a>",
    ["news_from_blog"] = "来自<b>ntop博客</b>的新闻",
    ["nothing_to_show"] = "没有什么新消息。明天试试！",
  },
  ["capabilities"] = {
    ["active_monitoring"] = "活动监控",
    ["alerts"] = "警告",
    ["checks"] = "检查",
    ["developer"] = "开发者",
    ["flowdevices"] = "流设备",
    ["notifications"] = "通知",
    ["pools"] = "池",
    ["preferences"] = "首选项",
    ["snmp"] = "SNMP",
    ["user_scripts"] = "用户脚本",
  },
  ["categories_page"] = {
    ["search_application"] = "搜索应用",
    ["target"] = "目标",
  },
  ["category_lists"] = {
    ["category_lists"] = "类别列表",
    ["connection_time_out"] = "%{duration} 秒后服务器连接超时",
    ["disable_some_list"] = "请禁用某些列表以解决此问题。",
    ["download_succeeded"] = "列表'%{name}'下载成功",
    ["edit_list"] = "编辑类别列表",
    ["enabled"] = "启用",
    ["error_occurred"] = "下载<a href=\"%{url}\">list</a> '%{name}'时出错：%{err}",
    ["http_code"] = "[HTTP 代码：%{err_code}]",
    ["last_update"] = "最后更新",
    ["list_will_be_updated"] = "列表 '%{name}' 将在30秒内更新。",
    ["loading_stats"] = "上次重新加载（在 %{when} ）：已加载 %{num_hosts} 个域、%{num_ips} 个 IP 地址、%{duration} 中的 %{num_ja3} 个 JA3 签名。",
    ["ready_for_update"] = "准备更新",
    ["server_returned_error"] = "服务器返回错误",
    ["too_many_hosts_loaded"] = "达到域名上限（%{limit}个域名）",
    ["too_many_ips_loaded"] = "已达到IP上限（%{limit}个地址）",
    ["too_many_ja3_loaded"] = "已达到JA3 名限制（%{limit}个签名）",
    ["update_frequency"] = "更新频率",
    ["update_now"] = "现在更新",
  },
  ["checks"] = {
    ["categories"] = "类别",
    ["category"] = "类别",
    ["category_internals"] = "内部",
    ["category_internals_descr"] = "%{product}的内部功能（例如，内存管理、主机和流生命周期）",
    ["category_network"] = "网络",
    ["category_network_descr"] = "网络行为和异常（例如，流量高于某个阈值，TCP未按预期工作）",
    ["category_other"] = "其他",
    ["category_other_descr"] = "未分类脚本或无法包含在任何其他类别中的脚本的默认类别",
    ["category_security"] = "网络安全",
    ["category_security_descr"] = "安全行为和异常情况（例如，与黑名单主机的通信、TCP和UDP扫描）",
    ["category_system"] = "系统",
    ["category_system_descr"] = "%{product}运行Top系统功能（例如，磁盘空间已满、负载过高）",
    ["disable_all"] = "全部禁用",
    ["disable_all_checks"] = "禁用全部检测",
    ["disable_all_message"] = "禁用当前配置的所有检查。",
    ["elephant_flows_descr"] = "> %{l2r_bytes} (L2R), > %{r2l_bytes} (R2L)",
    ["exceptions"] = "例外情况: %{exceptions}",
    ["long_lived_flows_descr"] = "> %{duration}",
    ["note_apply_to_default"] = "<b>默认</b>配置也'应用于'任何其他配置中不包括的池。",
    ["note_configsets"] = "在每个配置的基础上配置和启用/禁用检查。可以创建多个配置，并且每个配置可以\"应用\"到多个池。",
    ["note_what_are_checks"] = "<a href=\"%{checks_url}\">Checks</a> %{checks_external}定期或在某个事件发生时执行。 ",
    ["stop_recording_after"] = " 在%{duration}以后停止流量记录",
    ["hint"] = {
      ["body"] = "配置异常<a href='%{link_DHCP}' target='_about'>DHCP</a>, <a href='%{link_SMTP}' target='_about'>SMTP</a>, <a href='%{link_DNS}' target='_about'>DNS</a>, <a href='%{link_NTP}' target='_about'>NTP</a> 服务器。",
      ["title"] = "异常服务器",
    },
  },
  ["conf_backup"] = {
    ["conf_backup"] = "备份配置",
    ["conf_restore"] = "恢复配置",
  },
  ["config_scripts"] = {
    ["applied_to"] = "应用于",
    ["config_apply"] = "配置应用",
    ["config_clone"] = "克隆配置",
    ["config_clone_message"] = "输入克隆名称",
    ["config_delete"] = "删除配置",
    ["config_delete_message"] = "确认要删除这个配置吗？",
    ["config_entities"] = "配置实体",
    ["config_export"] = "导出配置",
    ["config_import"] = "导入配置",
    ["config_name"] = "配置名称",
    ["config_rename"] = "配置重命名",
    ["config_rename_message"] = "输入配置的新名称",
    ["config_reset"] = "重置配置",
    ["config_reset_confirm"] = "确认要删除所有配置并将默认配置重置为其初始值吗？",
    ["config_search"] = "搜索配置：",
    ["config_x"] = "配置 %{product}",
    ["empty_target_message"] = "选择一个或多个目标！",
    ["empty_value_message"] = "输入框不能为空！",
    ["invalid_characters"] = "名称包含无效字符！",
    ["invalid_file"] = "不是有效的配置文件",
    ["max_input_length"] = "名称最多只能有 16 个字符！",
    ["no_file"] = "未选择配置文件",
    ["no_pools_applied"] = "没有池",
    ["select_interface"] = "选择接口",
    ["select_network"] = "选择网络",
    ["type_targets"] = "类型目标",
    ["type_targets_example"] = "输入以逗号分隔的目标。 即：192.168.1.20,192.168.0.0/16",
    ["actions"] = {
      ["apply_to"] = "应用",
      ["clone"] = "克隆",
      ["delete"] = "删除",
      ["edit"] = "编辑",
      ["rename"] = "重命名",
    },
    ["granularities"] = {
      ["flow"] = "流",
      ["host"] = "主机",
      ["interface"] = "接口",
      ["network"] = "网络",
      ["snmp_device"] = "SNMP 设备",
      ["syslog"] = "syslog",
      ["system"] = "系统",
    },
  },
  ["configsets"] = {
    ["bad_target"] = "错误的配置目标：\"%{target}\"",
    ["duplicate_target"] = "目标 \"%{target}\" 被分配给配置 \"%{confname1}\" 和 \"%{confname2}\"",
    ["error_exists"] = "配置名称\"%{name}\"已被使用",
    ["unknown_check"] = "未知用户脚本 \"%{check}\"",
    ["unknown_id"] = "未找到 ID 为 %{confid} 的配置",
    ["unknown_user_script"] = "未知用户脚本 \"%{user_script}\"",
    ["wrong_args_ex_list"] = "排除列表的参数不正确。 检查两个排除列表是否不同。",
  },
  ["contacts_page"] = {
    ["client_address"] = "客户端地址",
    ["client_contacts_initiator"] = "客户端连接（发起人）",
    ["contacts"] = "连接",
    ["no_client_contacts_so_far"] = "目前没有客户端连接",
    ["no_contacts_message"] = "没有此主机的连接",
    ["no_server_contacts_so_far"] = "目前没有服务器连接",
    ["server_address"] = "服务器地址",
    ["server_contacts_receiver"] = "服务器连接（接收方）",
  },
  ["containers_stats"] = {
    ["all_containers"] = "所有容器",
    ["all_pods"] = "所有Pods",
    ["avg_rtt"] = "平均 RTT",
    ["avg_rtt_as_client"] = "作为客户端的平均 RTT",
    ["avg_rtt_as_server"] = "作为服务器的平均 RTT",
    ["avg_rtt_variance"] = "平均 RTT 差异",
    ["avg_rtt_variance_as_client"] = "作为客户端的平均 RTT 差异",
    ["avg_rtt_variance_as_server"] = "作为服务器的平均 RTT 差异",
    ["client_container"] = "客户端容器",
    ["client_pod"] = "客户端 Pod",
    ["client_rtt"] = "客户端 RTT",
    ["container"] = "容器",
    ["containers"] = "容器",
    ["containers_list"] = "容器清单",
    ["containers_of_pod"] = "Pod '%{pod}' 的容器",
    ["pod"] = "Pod",
    ["pods"] = "Pods",
    ["pods_list"] = "Pods清单",
    ["server_container"] = "服务器容器",
    ["server_pod"] = "服务端Pod",
    ["server_rtt"] = "服务器RTT",
  },
  ["country_details"] = {
    ["country"] = "国家",
    ["country_parameter_missing_message"] = "缺少国家/地区参数（内部错误？）",
    ["no_available_stats_for_country"] = "国家 %{country} 没有可用的统计数据",
  },
  ["custom_categories"] = {
    ["add"] = "添加",
    ["add_application"] = "添加应用程序",
    ["add_custom_app"] = "添加自定义应用程序",
    ["add_to_categories"] = "添加主机规则",
    ["app_delete_error"] = "无法删除 \"%{app}\" 应用程序。请重新启动 %{product} 以确保没有挂起的更改。",
    ["app_deleted"] = "应用程序 \"%{app}\" 已被删除。 请重新启动 %{product} 以应用更改。",
    ["application_exists"] = "应用程序\"%{app}\" 已经存在！",
    ["application_name"] = "应用名称",
    ["application_not_exists"] = "应用程序\"%{app}\" 不存在！",
    ["apps_and_categories"] = "应用程序和类别",
    ["custom_host_category"] = "自定义主机规则",
    ["custom_hosts"] = "自定义规则",
    ["delete_app"] = "删除应用",
    ["delete_app_confirm"] = "<b>警告</b>：删除应用程序是一种破坏性操作,可能会影响已定义的其他自定义应用程序的历史数据。 确认要删除 %{app} 吗？",
    ["domain_names_substrings"] = "域名被解释为要匹配的子字符串。<br>例如 \"%{s1}\" 将匹配 \"%{s2}\" 和 \"%{s3}\"",
    ["each_host_separate_line"] = "每个规则必须放在单独的行上",
    ["edit_custom_rules"] = "编辑自定义规则",
    ["edit_hosts"] = "编辑规则",
    ["example_port_range"] = "端口范围示例：\"%{example1}\", \"%{example2}\"",
    ["host_domain_or_cidr"] = "规则可以是域名或 IPv4 地址/CIDR 网络地址",
    ["host_domain_or_port"] = "规则可以是域名、IPv4 地址或 TCP/UDP 端口范围",
    ["invalid_rule"] = "无效规则已被忽略：%{rule}",
    ["new_app_added"] = "应用程序 \"%{app}\" 已配置。 请重新启动 %{product} 以开始使用它。",
    ["non_empty_list_required"] = "必须至少指定一项规则。",
    ["option_needed"] = "自定义应用程序配置当前已禁用,因为配置中缺少 <i>%{option}</i> 参数。 查看<a href=\"%{url}\">文档</a>了解更多详情。",
    ["protos_reboot_necessary"] = "已写入应用程序规则。 请重新启动 %{product} 以应用它们。",
    ["protos_unexpected_error"] = "更新应用程序规则时出现意外错误。 查看 %{product} 日志以获取更多详细信息。",
    ["select_url_category"] = "请选择添加域的位置：",
    ["similar_host_found"] = "类似于 \"%{host}\" 的规则已经包含在 %{category} 类别中",
    ["the_following_is_a_list_of_hosts"] = "以下是 %{category} 类别的自定义规则列表",
    ["the_following_is_a_list_of_hosts_app"] = "以下是 %{application} 应用程序的自定义规则列表",
    ["the_following_url_will_be_added"] = "将添加以下域：",
  },
  ["dashboard"] = {
    ["actual_traffic"] = "实际流量",
    ["community_dashboard"] = "Community仪表板",
    ["gateways_users"] = "网关和用户",
    ["lastday_app_traffic"] = "最后一天的Top应用程序流量视图",
    ["lastday_traffic"] = "网络接口：最后一天视图",
    ["realtime_app_traffic"] = "实时Top应用流量",
    ["realtime_traffic"] = "网络接口：实时流量",
    ["slow_dashboard_message"] = "%{product} 响应延迟太多。 考虑设置更高的<a href='%{url}'>实时统计刷新率</a>。",
    ["top_local_talkers"] = "Top本地Talker",
    ["top_remote_destinations"] = "Top远程目标",
    ["traffic_dashboard"] = "流量仪表盘",
  },
  ["datasources"] = {
    ["alias"] = "别名",
    ["alias_example"] = "即数据源别名",
    ["families"] = "家族",
    ["key"] = "key",
    ["metrics"] = "指标",
    ["new_source"] = "新源",
    ["origin"] = "源",
    ["private"] = "私有",
    ["public"] = "公共",
    ["retention"] = "保留",
    ["schema"] = "schema",
    ["scope"] = "范围",
    ["select_family"] = "选择一个家族",
    ["select_metrics"] = "选择指标",
    ["select_schema"] = "选择schema",
    ["select_schema_first"] = "首先选择一个schema",
    ["select_series"] = "选择一个系列",
    ["series"] = "系列",
    ["source_exists"] = "已经有这样的源了！",
    ["tags"] = "标签",
    ["widget_bound"] = "有绑定到此数据源的部件！",
    ["modal"] = {
      ["add_datasource"] = "添加数据源",
      ["delete_datasource"] = "删除数据源：%{datasource}",
      ["delete_description"] = "您确定要删除数据源：%{datasource}？",
      ["edit_datasource"] = "编辑数据源：%{datasource}",
    },
    ["validation"] = {
      ["abort_add"] = "中止添加数据源？",
      ["abort_edit"] = "中止编辑数据源？",
    },
  },
  ["db_explorer"] = {
    ["abort"] = "中止",
    ["all"] = "所有",
    ["app_between_talkers_pairs"] = "成对talker之间的应用",
    ["app_flows_between"] = "%{app} 应用程序在 %{peer1} 和 %{peer2} 之间流动",
    ["app_talkers"] = "%{app} takler",
    ["app_talkers_with"] = "taker %{peer}使用%{app}",
    ["application_flows"] = "应用程序流",
    ["applications_between"] = "%{peer1} 和 %{peer2} 之间的应用",
    ["average_throughput"] = "平均吞吐量",
    ["client_server_breakdown"] = "客户端vs服务器流量细分",
    ["client_server_host"] = "客户端/服务器主机",
    ["dns_query"] = "DNS查询",
    ["download_flows"] = "下载流",
    ["download_flows_limit"] = "最多 1000 条记录。",
    ["empty_query_response"] = "空查询响应（数据库关闭或查询终止/超时？）",
    ["extract_pcap"] = "提取 pcap",
    ["filter_bpf"] = "过滤器 (BPF)",
    ["first_last_seen"] = "首次/最后出现",
    ["flow_peers"] = "流peers",
    ["get_proto_talkers"] = "获取使用此应用程序taker",
    ["historical_data_explorer"] = "历史浏览器",
    ["host_name"] = "主机名",
    ["host_talkers"] = "%{host} talker",
    ["hosts_talking_proto_with"] = "主机与%{host}通信%{proto}",
    ["http_host"] = "HTTP 主机",
    ["http_method"] = "HTTP 方法",
    ["interface_name"] = "实例名称",
    ["last_hour_flows"] = "过去一小时内看到的流量",
    ["ms_client"] = "%{client} 毫秒（客户端）",
    ["ms_server"] = "%{server} 毫秒（服务器）",
    ["network_latency_breakdown"] = "网络延迟细分",
    ["observation_period"] = "观察期",
    ["ok_request_sent"] = "OK,请求已发送",
    ["packet_rate"] = "平均数据包率",
    ["pcap_requests_and_status"] = "Pcap 请求和状态",
    ["pcaps"] = "Pcaps",
    ["protocol_flows_between"] = "%{proto} 应用程序在 %{peer1} 和 %{peer2} 之间流动",
    ["protocol_id"] = "应用程序id",
    ["query_in_progress"] = "数据库查询正在进行中,等待接收结果...",
    ["request_failed"] = "请求失败",
    ["response_code"] = "响应代码",
    ["search_flows"] = "搜索流",
    ["selected_saved"] = "选择保存",
    ["server_name"] = "服务器名称",
    ["summary"] = "概括",
    ["talkers_with_this_host"] = "与此主机通信的Talker",
    ["task_id"] = "任务 ID",
    ["this_flow_has_been_reset"] = "此流已重置,可能服务器应用程序已关闭",
    ["this_flow_is_active"] = "此流处于活动状态",
    ["this_flow_is_completed"] = "此流已完成,即将到期",
    ["top_flows_ipv4"] = "Top IPv4 流 [%{date_from} - %{date_to}]",
    ["top_flows_ipv6"] = "Top IPv6 流 [%{date_from} - %{date_to}]",
    ["top_proto_ipv4_flows"] = "Top %{proto} IPv4 流",
    ["top_proto_ipv6_flows"] = "Top %{proto} IPv6 流",
    ["total_flows"] = "总流数",
    ["total_packets"] = "总包数",
    ["total_traffic"] = "总字节数",
    ["traffic_rate"] = "平均流量速率",
    ["traffic_received"] = "接收的字节数",
    ["traffic_sent"] = "发送的字节数",
    ["traffic_sent_to_cli"] = "发送给客户端的字节数",
    ["traffic_sent_to_srv"] = "发送到服务器的字节数",
    ["traffic_volume"] = "流量容量",
    ["unable_to_find_flow"] = "找不到指定的流",
    ["unsave"] = "移除",
    ["vlan_id"] = "VLAN Id",
  },
  ["delete_data"] = {
    ["delete_active_interface_confirmation"] = "您确定要删除与 %{ifname} 关联的所有持久数据吗？<br><br><b><b>注意</b>：</b>将对 %{product} 执行数据删除。重启 %{product} 使删除生效。",
    ["delete_active_interface_data_failed"] = "无法删除活动接口数据。",
    ["delete_active_interface_data_ok"] = "%{ifname} 的数据已成功标记为删除。 重启 %{product} 使删除生效。",
    ["delete_confirmation"] = "您确定要删除 %{host} 的所有持久数据吗？",
    ["delete_failed"] = "无法删除 %{host} 的数据。",
    ["delete_inactive_interfaces_confirmation"] = "以下接口处于不活动状态：%{interfaces_list}您确定要删除它们的所有持久数据吗？",
    ["delete_inactive_interfaces_data_failed"] = "无法删除不活动接口数据。",
    ["delete_inactive_interfaces_data_ok"] = "已成功删除不活动接口的数据。",
    ["delete_ok"] = "%{host} 的数据已成功删除。",
    ["msg_err_invalid_host"] = "无法识别指定主机的格式。",
    ["msg_err_no_fs_data"] = "未找到指定主机的持久数据。",
    ["msg_err_unable_to_delete_dir"] = "无法删除数据目录。",
    ["msg_err_unable_to_delete_ts_data"] = "无法删除时间序列数据。",
    ["msg_err_unknown"] = "出现未知错误。",
    ["node_nindex_flows"] = "警告：流量转储启用 nIndex,不会删除历史流量信息。",
    ["note_persistent_data"] = "删除所有持久数据。 数据包括流量计数器、时间序列、流、警告和 Redis 缓存。",
    ["restart_product_toast"] = "重启 %{product} 使活动接口的删除生效。",
  },
  ["details"] = {
    ["additional_mac_address"] = "附加 MAC 地址",
    ["address_resolution_protocol"] = "地址解析协议",
    ["ago"] = "前",
    ["alerts"] = "警告",
    ["arp_replies"] = "ARP 响应",
    ["arp_requests"] = "ARP 请求",
    ["as_client"] = "作为客户端",
    ["as_server"] = "作为服务器",
    ["contacts_blacklisted"] = "列入黑名单的主机的总流量",
    ["details"] = "详情",
    ["device_id"] = "设备ID",
    ["device_port"] = "设备端口",
    ["device_type"] = "设备类型",
    ["first_last_seen"] = "首先/最后查看",
    ["first_observed_on"] = "首先发现在",
    ["flow_snmp_localization"] = "流 SNMP 本地化",
    ["flows_dropped_by_bridge"] = "由于流量策略而拦截的流",
    ["flows_non_packet_iface"] = "最近活动流量/总计/警告总数",
    ["flows_packet_iface"] = "流量：活动/总计/警告/端口未到达",
    ["fragments"] = "片段",
    ["further_host_names_information"] = "其他主机名",
    ["goodput"] = "Goodput",
    ["host_pool"] = "主机池",
    ["host_purged"] = "此主机不再处于活动状态,应该已从内存中清除",
    ["host_snmp_localization"] = "SNMP 本地化",
    ["input_device_port"] = "入口接口 ID",
    ["keep_alive"] = "保持活动",
    ["label_blacklisted_host"] = "列入黑名单",
    ["label_dhcp"] = "DHCP",
    ["label_dhcp_server"] = "DHCP服务器",
    ["label_dns_server"] = "DNS服务器",
    ["label_local_host"] = "本地",
    ["label_ntp_server"] = "NTP服务器",
    ["label_private_ip"] = "私有",
    ["label_remote"] = "远程",
    ["label_short_local_host"] = "L",
    ["label_short_private_ip"] = "P",
    ["label_short_remote"] = "R",
    ["label_smtp_server"] = "SMTP 服务器",
    ["label_system_ip"] = "系统 IP",
    ["lost"] = "丢失",
    ["misbehaving_flows_reasons"] = "流异常行为原因",
    ["note"] = "注意",
    ["note_hosts_located_snmp_device"] = "主机位于使用 <A HREF=%{url}>Bridge MIB</A> 的 SNMP 设备中。",
    ["observation_point_id"] = "发现Point Id",
    ["out_of_order"] = "乱序",
    ["output_device_port"] = "出口接口 ID",
    ["peers"] = "Peers：活动",
    ["rcvd"] = "接收",
    ["retransmissions"] = "重传",
    ["router_access_point_mac_address"] = "（路由器/接入点）MAC 地址",
    ["sent_vs_received_traffic_breakdown"] = "发送vs接收流量细分",
    ["server_contacts"] = "通信的服务器",
    ["show_hosts"] = "显示主机",
    ["source"] = "源",
    ["source_id"] = "源 Id",
    ["tcp_packets_sent_analysis"] = "TCP 数据包发送分析",
    ["total_traffic"] = "总流量",
    ["traffic_sent_received"] = "发送/接收的流量",
    ["vlan_id"] = "VLAN ID",
    ["whois_lookup"] = "Whois 查询",
  },
  ["developer_section"] = {
    ["datasources_list"] = "数据源",
    ["widgets_list"] = "部件",
  },
  ["device_protocols"] = {
    ["alert"] = "触发警告",
    ["device_protocol_policy"] = "设备应用程序策略",
    ["device_protocols"] = "设备应用程序",
    ["devtype_as_proto_client"] = "%{devtype} 作为 <i>%{proto}</i> 客户端",
    ["devtype_as_proto_server"] = "%{devtype} 作为 <i>%{proto}</i> 服务器",
    ["filter_device_protocols"] = "%{filter} 设备应用程序",
    ["ok"] = "可接受",
  },
  ["device_types"] = {
    ["iot"] = "IoT",
    ["laptop"] = "笔记本电脑",
    ["multimedia"] = "多媒体",
    ["nas"] = "NAS",
    ["networking"] = "路由器/交换机",
    ["phone"] = "手机",
    ["printer"] = "打印机",
    ["tablet"] = "平板电脑",
    ["tv"] = "电视",
    ["unknown"] = "未知",
    ["video"] = "录像",
    ["wifi"] = "无线网络",
    ["workstation"] = "计算机",
  },
  ["dhcp"] = {
    ["delete_range"] = "删除DHCP范围",
    ["delete_range_confirm"] = "确认要删除配置的DHCP范围 %{range} 吗？",
    ["dhcp"] = "DHCP",
    ["dhcp_alert_note"] = "当分配的 IP 地址超出配置的DHCP范围时,将生成警告",
    ["dhcp_configuration_note"] = "此页面列出了为此接口配置的DHCP范围。 这些应该与实际使用的DHCP服务器配置相匹配",
    ["invalid_ip_address"] = "无效的 IP 地址",
    ["no_dhcp_ranges"] = "未配置DHCP范围。",
  },
  ["discover"] = {
    ["device"] = "设备",
    ["discovered_devices"] = "发现 %{manuf} %{sys} %{dev} 设备",
    ["err_unable_to_arp_discovery"] = "无法启动 ARP 网络发现",
    ["error_no_discovery_cached"] = "尚未缓存任何发现数据。",
    ["error_unable_to_decode_json"] = "无法解码 JSON 缓存发现数据。",
    ["ghost_icon_descr"] = "%{ghost_icon} 图标突出显示了Ghost主机（即它们不属于接口 IP 地址网络）",
    ["network_discovery"] = "网络发现",
    ["network_discovery_datetime"] = "上次网络发现",
    ["network_discovery_not_enabled"] = "正在进行网络发现",
    ["too_many_devices_descr"] = "发现的设备太多。 结果已经修剪为 %{max_devices}",
  },
  ["dns_page"] = {
    ["dns_breakdown"] = "DNS故障",
    ["dns_query_sent_distribution"] = "DNS查询发送分布",
    ["dns_query_sent_vs_rcvd_distribution"] = "DNS查询发送/接收分布",
    ["doh_dot_server_uses"] = "使用",
    ["doh_dot_servers"] = "已连接 <A class='ntopng-external-link' href=\"https://en.wikipedia.org/wiki/DNS_over_HTTPS\" target=_blank>DoH <i class='fas fa-external-link-alt'></i></A> /<A class='ntopng-external-link' href=\"https://en.wikipedia.org/wiki/DNS_over_TLS\" target=_blank>DoT <i class='fas fa-external-link-alt'></i></A> 服务器",
    ["error_replies"] = "错误响应",
    ["note"] = "注意",
    ["note_dns_ratio"] = "理想情况下,请求与响应DNS比率应为 1（每个请求一个响应）。 当远低于该值时,则存在值得调查的问题,因为这意味着收到的响应数量远低于预期,这可能表明我们正在使用无响应的DNS解析器或它们配置错误（例如,它们已被转移到 另一个IP）。",
    ["positive_replies"] = "积极响应",
    ["queries"] = "查询",
    ["ratio"] = "比率",
    ["rcvd"] = "接收",
    ["replies"] = "响应",
    ["reply_breakdown"] = "响应详细",
    ["request_vs_reply"] = "请求VS响应率",
  },
  ["dscp_page"] = {
    ["statistics"] = "优先级",
    ["statistics_received"] = "已接收",
    ["statistics_sent"] = "发送",
  },
  ["edit_user_script"] = {
    ["author"] = "作者",
    ["category"] = "类别",
    ["delete_alert_exclusions"] = "删除警报排除将导致再次触发新警报。确认要删除的警报排除",
    ["delete_all_alert_exclusions"] = "删除%{host}检查排除项",
    ["delete_all_alert_exclusions_message"] = "确认要删除所有%{host}配置的警报排除？",
    ["description"] = "描述",
    ["excluded_host_name"] = "排除主机名称",
    ["exclusion_list"] = "检查排除项",
    ["exclusion_list_host_x"] = "检查排除项：主机%{Host}",
    ["hooks_config"] = "hooks配置",
    ["success_message"] = "用户脚本 <b>%{script_name}</b> 配置已成功保存！",
    ["hooks_name"] = {
      ["5mins"] = "5分钟",
      ["all"] = "所有",
      ["day"] = "天",
      ["hour"] = "小时",
      ["min"] = "分钟",
    },
  },
  ["email_alert_endpoint"] = {
    ["carbon_copy"] = "抄送",
    ["email_recipient"] = "电子邮件收件人",
    ["email_sender"] = "电子邮件发件人",
    ["smtp_password"] = "SMTP密码",
    ["smtp_server"] = "SMTP服务器",
    ["smtp_username"] = "SMTP 户名",
    ["validation"] = {
      ["empty_SMTP_server"] = "请填写 IPv4/IPv6/主机地址。",
      ["empty_email"] = "请填写电子邮件地址。",
      ["invalid_SMTP_server"] = "请输入有效的 IPv4/IPv6/主机地址。",
      ["invalid_email"] = "请输入有效的电子邮件地址（如name@domain.com）。",
    },
  },
  ["endpoint_notifications"] = {
    ["actions"] = "操作",
    ["add_new_endpoint"] = "添加新端点",
    ["add_new_recipient"] = "添加新收件人",
    ["add_to_all"] = "绑定到所有池",
    ["associated_to"] = "绑定到",
    ["associated_to_endpoints"] = "绑定到端点",
    ["associated_to_recipients"] = "绑定到收件人",
    ["bad_endpoint"] = "检索端点配置失败",
    ["bad_recipient_params"] = "错误的收件人参数",
    ["create_endpoint_first"] = "请先创建一个端点！",
    ["edit_endpoint"] = "编辑端点",
    ["edit_endpoints"] = "编辑端点",
    ["edit_recipient"] = "编辑收件人",
    ["empty_endpoints"] = "未定义端点。",
    ["empty_template"] = "此收件人没有要填写的参数。",
    ["endpoint"] = "端点",
    ["endpoint_config_already_existing"] = "已经有一个同名的配置！",
    ["endpoint_list"] = "端点",
    ["endpoint_recipient_already_existing"] = "已经有一个同名的收件人",
    ["enpoint_recipients_list"] = "收件人",
    ["fill_pct"] = "填充",
    ["get_users_recipient_failed"] = "抱歉,但收件人用户加载失败。 请稍后再试。",
    ["name"] = "名称",
    ["notifications"] = "通知",
    ["num_drops"] = "丢失",
    ["num_uses"] = "使用",
    ["recipient_category"] = "类别过滤器",
    ["recipient_list"] = "收件人",
    ["recipient_name"] = "名称",
    ["recipient_select"] = "端点",
    ["recipient_severity"] = "最低严重程度",
    ["recipient_type"] = "收件人列表",
    ["recipient_users"] = "池使用收件人 %{recipient} ",
    ["remove_endpoint"] = "删除端点",
    ["remove_endpoint_message"] = "确认要删除 '%{endpoint}' 通知端点吗？ <span class='count'>%{recipients_count} 个收件人也将被<b>删除</b>。</span>",
    ["remove_recipient"] = "删除收件人",
    ["remove_recipient_message"] = "确认要删除 '%{recipient}' 收件人吗？",
    ["select_type"] = "选择端点类型",
    ["server_error"] = "出错，稍后再试。",
    ["status"] = "节点状态",
    ["test_recipient"] = "检测",
    ["testing_recipient"] = "测试收件人",
    ["time_since_last_use"] = "自上次使用以来的时间",
    ["timed_out"] = "收件人测试已超时！",
    ["type"] = "类型",
    ["used_by_recipients"] = "被收件人使用",
    ["working_recipient"] = "测试成功！",
    ["zero_users_recipient"] = "没有池使用此收件人。 从 %{link} 页面添加池。",
    ["factory_reset"] = {
      ["body"] = "您确定要重置 <b>%{reset_element}</b> 吗？<br>重置为出厂默认设置<b>您将永久丢失所有已配置的 %{reset_element}</b>。",
      ["title"] = "重置配置：%{reset_element}",
    },
    ["hints"] = {
      ["bind_pools"] = {
        ["body"] = "最后一步！ 将收件人绑定到池以开始向他们发送警告。",
        ["title"] = "绑定收件人",
      },
      ["create_endpoint"] = {
        ["action"] = "创建",
        ["body"] = "创建一个<a target='_blank' href='%{link}'>端点</a>以开始向外部发送警告。",
        ["title"] = "创建端点",
      },
      ["create_recipients"] = {
        ["body"] = "创建新的<a target='_blank' href='%{link}'>收件人</a>以开始向他们发送警告。",
        ["title"] = "创建收件人",
      },
    },
    ["recipients"] = {
      ["cc"] = "抄送",
      ["email_recipient"] = "电子邮件收件人",
    },
    ["severities"] = {
      ["alert"] = "警告",
      ["critical"] = "关键",
      ["debug"] = "调试",
      ["emergency"] = "紧急",
      ["error"] = "错误",
      ["info"] = "信息",
      ["notice"] = "通知",
      ["warning"] = "警告",
    },
    ["validation"] = {
      ["abort_add_endpoint"] = "中止添加新端点？",
      ["abort_add_recipient"] = "中止添加新端点收件人？",
      ["abort_edit_endpoint"] = "中止端点的编辑？",
      ["abort_edit_recipient"] = "中止端点收件人的编辑？",
    },
  },
  ["entity_thresholds"] = {
    ["dns_request_reply_ratio_description"] = "当响应/请求比率低于阈值时触发警告",
    ["dns_request_reply_ratio_title"] = "DNS响应/请求比率",
    ["flow_flood_description"] = "当新的客户端/服务器 Flows/sec 超过阈值时触发警告",
    ["flow_flood_title"] = "流泛洪警告",
    ["flow_victim_description"] = "当新服务器 Flows/sec 超过阈值时触发警告",
    ["flow_victim_title"] = "流量泛洪被攻击者警告",
    ["http_request_reply_ratio_description"] = "当响应/请求比率低于阈值时触发警告",
    ["http_request_reply_ratio_title"] = "HTTP 响应/请求比率",
    ["syn_flood_description"] = "当发送/接收的 SYN 数/秒超过阈值时触发警告",
    ["syn_flood_title"] = "SYN 泛洪警告",
    ["syn_scan_description"] = "当发送/接收的 SYN 数/分钟（无响应）超过阈值时触发警告",
    ["syn_scan_title"] = "SYN 扫描警告",
    ["syn_scan_victim_description"] = "当接收到的 SYNs/min（无响应）数量超过阈值时触发警告",
    ["syn_scan_victim_title"] = "SYN 扫描被攻击者警告",
    ["syn_victim_description"] = "当接收到的 SYNs/sec 数量超过阈值时触发警告",
    ["syn_victim_title"] = "SYN 泛洪被攻击者警告",
  },
  ["error_page"] = {
    ["go_back"] = "返回",
    ["greeting"] = "检测到错误",
    ["presence"] = "这里不被支持！",
  },
  ["event_exporters"] = {
    ["event_device_x"] = "事件探针 %{device}",
    ["event_exporter_ip"] = "事件探针 IP",
    ["event_exporters"] = "事件探针",
  },
  ["export_data"] = {
    ["export_json_data"] = "导出 JSON 数据",
    ["note_active_hosts"] = "仅返回活动主机。",
    ["note_maximum_number"] = "可下载主机的最大数量为 32,768。",
  },
  ["external_stats"] = {
    ["capture_drops"] = "丢包",
    ["capture_packets"] = "捕获的数据包",
    ["signatures_failed"] = "签名失败",
    ["signatures_loaded"] = "签名已加载",
    ["title"] = "外部统计",
  },
  ["field_units"] = {
    ["bytes"] = "字节",
    ["contacts"] = "连接",
    ["flow_sec"] = "flow/s",
    ["flows"] = "流",
    ["hosts"] = "主机",
    ["mbits"] = "Mbps",
    ["packets"] = "数据包",
    ["percentage"] = "%%",
    ["score"] = "分数",
    ["seconds"] = "秒",
    ["syn_min"] = "SYN/min",
    ["syn_sec"] = "SYN/s",
  },
  ["flow_alerts_explorer"] = {
    ["alert_tcp_no_data_exchanged_description"] = "TCP 流结束,没有数据交换",
    ["alert_tcp_no_data_exchanged_title"] = "没有数据交换",
    ["alert_unexpected_dhcp_title"] = "发现异常DHCP服务器",
    ["alert_unexpected_dns_title"] = "发现异常DNS服务器",
    ["alert_unexpected_ntp_title"] = "发现异常 NTP 服务器",
    ["alert_unexpected_smtp_title"] = "发现异常 SMTP 服务器",
    ["alert_zero_tcp_window_title"] = "TCP 零窗口",
    ["by_target_port"] = "按目标端口",
    ["flow_alert_origin"] = "警告来源",
    ["flow_alert_target"] = "警告目标",
    ["label"] = "流警告探测器",
    ["origins"] = "源",
    ["search"] = "搜索流警告",
    ["status_unexpected_dhcp_description"] = "发现异常DHCP服务器：%{server}",
    ["status_unexpected_dns_description"] = "发现异常DNS服务器：%{server}",
    ["status_unexpected_ntp_description"] = "发现异常 NTP 服务器：%{server}",
    ["status_unexpected_smtp_description"] = "发现异常 SMTP 服务器：%{server}",
    ["status_zero_tcp_window_description"] = "报告的 TCP 零窗口",
    ["summary_cli2srv"] = "总源到目标流量",
    ["summary_n_origins"] = "总源",
    ["summary_n_targets"] = "总目标",
    ["summary_srv2cli"] = "总目标到源流量",
    ["summary_total"] = "总流量警告",
    ["targets"] = "目标",
    ["type_alerts_by_type"] = "按类型分类的流量警告",
    ["type_explorer"] = "类型探测器",
    ["visual_explorer"] = "可视化探测器",
  },
  ["flow_callbacks"] = {
    ["allowed_servers_description"] = "服务器IP的逗号分隔值。 示例：173.194.76.109,52.97.232.242",
    ["allowed_servers_title"] = "允许的服务器",
    ["application_detected"] = "检测到应用程序",
    ["asn_behavior_check_list"] = "ASN 分析",
    ["asn_behavior_check_list_example"] = "要分析的ASN（名称或ID）的列表，用逗号分隔",
    ["callback"] = "用户脚本",
    ["callback_config"] = "配置",
    ["callback_elapsed_time"] = "执行时间",
    ["callback_elapsed_time_avg"] = "flow/s",
    ["callback_enabled"] = "启用",
    ["callback_function"] = "Hook",
    ["callback_function_duration"] = "持续时间",
    ["callback_function_duration_fmt_long"] = "%{num_calls} 在 %{time} 内调用 [%{speed} Calls/sec]",
    ["callback_function_duration_fmt_short"] = "%{time}",
    ["callback_function_num_flows"] = "流",
    ["callback_function_throughput"] = "处理速度",
    ["callback_latest_run"] = "最新运行",
    ["dns_esclusion_list"] = "排除的域",
    ["dns_esclusion_list_example"] = "要从无效查询中排除的域列表,以逗号分隔",
    ["idle"] = "空闲",
    ["iec104_description"] = "当检测到无效的 IEC 转换时触发警告",
    ["iec104_title"] = "IEC 无效转换",
    ["iec104_unexpected_type_id_allowed_type_ids_description"] = "逗号分隔IEC 60870-5-104 TypeID 。 示例：1,2,3,4",
    ["iec104_unexpected_type_id_allowed_type_ids_title"] = "允许的类型 ID",
    ["iec104_unexpected_type_id_description"] = "在 IEC 104 协议中检测到异常 TypeID 时触发警告",
    ["iec104_unexpected_type_id_title"] = "IEC 异常类型 ID",
    ["l7_behavior_check_list"] = "应用程序分析",
    ["l7_behavior_check_list_example"] = "要分析的应用程序（名称或ID）的列表，用逗号分隔",
    ["last_calls_per_sec"] = "最新 calls/s",
    ["last_duration"] = "最后时长",
    ["last_num_calls"] = "最新通话次数",
    ["low_goodput_description"] = "当流量吞吐量过低时触发警告",
    ["low_goodput_title"] = "低Goodput",
    ["network_behavior_check_list"] = "网络分析",
    ["network_behavior_check_list_example"] = "要分析的网络列表，用逗号分隔",
    ["no_callbacks_available"] = "没有可用的回调。",
    ["no_callbacks_available_disabled_alerts"] = "没有可用的回调。 回调需要从<a href=\"%{url}\"><i class=\"fas fa-flask\"></i>首选项</a>中启用警告",
    ["no_callbacks_defined"] = "未找到回调",
    ["note_add_custom_scripts"] = "%{product} 在这些 <a href=\"%{url}\"> <i class=\"fas fa-folder\"></i> 目录</a>中查找自定义脚本。",
    ["note_create_custom_scripts"] = "创建用于执行回调的自定义脚本的文档可在以下 <a class='ntopng-external-link' href=\"%{url}\">link <i class=\"fas fa-external-link-alt\"> </i></a>。",
    ["note_flow_application_detected"] = "\"应用程序检测\" 回调在检测到应用程序后立即执行。 这发生在数据包接口的流的前 12 个数据包内,或者在 ZMQ 接口接收到流之后立即发生。",
    ["note_flow_idle"] = "\"空闲\" 回调在流终活动并且空闲时执行。",
    ["note_flow_lifecycle"] = "回调是在流生命周期中的特定时刻执行的。",
    ["note_flow_periodic_update"] = "\"定期更新\" 回调在流活动超过五分钟后定期执行。",
    ["note_flow_staus_changed"] = "\"状态已更改\" 每次流中的状态更改时都会执行回调。 例如,当看到重传等连接问题时。",
    ["note_scripts_list"] = "可用的 %{product} 脚本的完整列表可以在 <a href=\"%{url}\">脚本</a> 页面中找到。",
    ["notes"] = "注意",
    ["periodic_update"] = "定期更新",
    ["status_changed"] = "状态已更改",
    ["status_zero_tcp_window_description"] = "报告的 TCP 零窗口",
    ["status_zero_tcp_window_description_c2s"] = "报告的客户端 TCP 零窗口",
    ["status_zero_tcp_window_description_s2c"] = "报告的服务器 TCP 零窗口",
    ["tcp_no_data_exchanged_description"] = "当 TCP 流结束且没有数据交换时触发警告",
    ["tcp_no_data_exchanged_title"] = "没有数据交换",
    ["unexpected_dhcp_description"] = "检测到不允许的DHCP服务器时触发警告",
    ["unexpected_dhcp_title"] = "异常 DHCP",
    ["unexpected_dns_description"] = "检测到不允许的DNS服务器时触发警告",
    ["unexpected_dns_title"] = "异常 DNS",
    ["unexpected_ntp_description"] = "检测到不允许的 NTP 服务器时触发警告",
    ["unexpected_ntp_title"] = "异常 NTP 服务器",
    ["unexpected_smtp_description"] = "检测到不允许的 SMTP 服务器时触发警告",
    ["unexpected_smtp_title"] = "异常 SMTP 服务器",
    ["zero_tcp_window_description"] = "当流 TCP 窗口为零时触发警告",
    ["zero_tcp_window_title"] = "零 TCP 窗口",
  },
  ["flow_callbacks_config"] = {
    ["blacklisted"] = "黑名单流",
    ["blacklisted_description"] = "当检测到列入黑名单的主机或域时触发警告",
    ["data_exfiltration"] = "数据泄露",
    ["data_exfiltration_description"] = "当检测到可能的数据泄露活动时触发警告",
    ["dev_proto_not_allowed"] = "不允许设备应用",
    ["dev_proto_not_allowed_description"] = "当检测到设备的异常应用程序时触发警告。 可以在<a href='%{url}'>处</a>配置规则",
    ["dev_proto_not_allowed_nedge_description"] = "当检测到 <i>未分配</i> 设备的不允许的应用程序时触发警告。 可以在<a href='%{url}'>处</a>配置策略",
    ["dns_data_exfiltration"] = "DNS数据泄露",
    ["dns_data_exfiltration_description"] = "检测到DNS数据泄露活动时触发警告",
    ["elephant_flows"] = "大数据流",
    ["elephant_flows_description"] = "当流交换超过配置的字节量时触发警告。 远程到本地方向表示本地主机从远程主机接收的流量。",
    ["ext_alert"] = "外部警告",
    ["ext_alert_description"] = "从外部端点接收流量警告（例如 Suricata）",
    ["flow_risk"] = "流风险",
    ["flow_risk_description"] = "根据检测到的 nDPI 流风险触发警告",
    ["flow_risk_list"] = "nDPI 流风险：",
    ["invalid_dns_query"] = "无效的DNS查询",
    ["invalid_dns_query_description"] = "当检测到可能的恶意DNS查询时触发警告",
    ["lateral_movement_description"] = "检测到新的本地服务时触发警报。启用流量行为首选项以便此脚本运行。",
    ["long_lived"] = "长期流",
    ["long_lived_description"] = "当流持续超过配置的持续时间时触发警告",
    ["malicious_signature"] = "恶意JA3签名",
    ["malicious_signature_description"] = "检测到列入黑名单的 JA3 签名时触发警告",
    ["nedge_blocked"] = "拦截的流",
    ["nedge_blocked_description"] = "由于配置的策略而拦截流时触发警告",
    ["no_input"] = "无需配置",
    ["not_purged"] = "未清除",
    ["not_purged_description"] = "检测的流清除逻辑中的错误",
    ["old_tls_version"] = "旧 TLS 版本",
    ["old_tls_version_description"] = "当检测到旧的（可能不安全的）TLS 版本时触发警告",
    ["periodicity_changed_description"] = "定期服务更改时触发警报。启用流量行为首选项以便此脚本运行。",
    ["periodicity_changed_title"] = "周期变化检测",
    ["possible_bot_flow_description"] = "当服务器可能Bot时触发警告",
    ["possible_bot_flow_title"] = "可能的Bot",
    ["potentially_dangerous_protocol"] = "潜在危险的协议",
    ["potentially_dangerous_protocol_description"] = "检测到潜在危险协议时触发警告",
    ["remote_to_local_insecure_proto_description"] = "当远程服务器使用不安全的协议通信本地主机时触发警告",
    ["remote_to_local_insecure_proto_title"] = "远程到本地不安全协议",
    ["remote_to_remote"] = "远程到远程流",
    ["remote_to_remote_description"] = "当流具有远程主机作为客户端和服务器时触发警告",
    ["score"] = "流得分",
    ["suspicious_countries"] = "可疑国家",
    ["suspicious_countries_description"] = "当客户端和服务器中至少有一个来自可疑国家时触发警告",
    ["suspicious_tcp_probing"] = "可疑的 TCP 探测",
    ["suspicious_tcp_probing_description"] = "当检测到可疑的 TCP 探测时触发警告",
    ["suspicious_tcp_syn_probing"] = "可疑的 TCP SYN 探测",
    ["suspicious_tcp_syn_probing_description"] = "当检测到可疑的 TCP SYN 探测时触发警告",
    ["tcp_connection_refused"] = "TCP 连接被拒绝",
    ["tcp_connection_refused_description"] = "当检测到 TCP 连接被拒绝时触发警告",
    ["tcp_issues_generic"] = "TCP 重传问题",
    ["tcp_issues_generic_description"] = "检测 TCP 重传、乱序和丢包问题",
    ["tls_cert_expired"] = "TLS 证书已过期",
    ["tls_cert_expired_description"] = "检测到过期的 TLS 证书时触发警告",
    ["tls_cert_mismatch"] = "TLS 证书问题",
    ["tls_cert_mismatch_description"] = "检测到不匹配的 TLS 证书时触发警告",
    ["tls_cert_selfsigned"] = "TLS 证书自签名",
    ["tls_cert_selfsigned_description"] = "检测到自签名 TLS 证书时触发警告",
    ["tls_unsafe_ciphers"] = "不安全TLS密码",
    ["tls_unsafe_ciphers_description"] = "当检测到不安全的 TLS 密码时触发警告",
    ["udp_unidirectional"] = "单向UDP流",
    ["udp_unidirectional_description"] = "检测到单向 UDP 流时触发警告",
    ["web_mining"] = "Web挖掘",
    ["web_mining_description"] = "切换为检测到来自/到已知执行加密货币挖掘的主机的流量时生成的警报",
  },
  ["flow_details"] = {
    ["acceptable_label"] = "可接受",
    ["actual_peak_throughput"] = "实际/峰值吞吐量",
    ["additional_alert_type"] = "其他警告",
    ["additional_flow_elements"] = "附加流元素",
    ["alert_type"] = "流状态",
    ["alerted_flows"] = "流警告",
    ["alerted_flows_severity"] = "警告流严重性",
    ["application_latency"] = "应用程序延迟",
    ["as_prev_next"] = "AS 前/后",
    ["as_src_dst"] = "ASN [客户端/服务器]",
    ["average_cpu_load"] = "平均 CPU 负载",
    ["bittorrent_hash"] = "BitTorrent哈希",
    ["blacklisted_flow"] = "客户端、服务器或域被列入黑名单",
    ["blacklisted_flow_detailed"] = "黑名单 %{who}",
    ["c_ip_addresses"] = "C IP 地址",
    ["call_canceled"] = "呼叫取消",
    ["call_completed"] = "呼叫完成",
    ["call_error"] = "呼叫错误",
    ["call_id"] = "Call-ID",
    ["call_initiator"] = "呼叫者",
    ["call_quality_indicators"] = "通话质量指标",
    ["call_started"] = "呼叫开始",
    ["call_state"] = "呼叫状态",
    ["called_party"] = "被呼叫者",
    ["cancel_bye_failure_reason_cause"] = "取消/挂断/失败造成",
    ["certificates_not_match"] = "证书不匹配",
    ["client_container_information"] = "客户端容器信息",
    ["client_process_information"] = "客户端进程信息",
    ["client_requested"] = "客户请求",
    ["client_tls_supported_versions"] = "客户端支持的 TLS 协议",
    ["computed_as_tcp_window_size_rtt"] = "计算为 TCP 窗口大小/RTT",
    ["container_id"] = "容器ID",
    ["could_not_add_host_to_category"] = "无法将主机 \"%{host}\" 添加到 \"%{category}\"。",
    ["data_exfiltration"] = "数据泄露",
    ["desirable_label"] = "满意的",
    ["dns_data_exfiltration"] = "DNS数据泄露",
    ["dns_invalid_query"] = "无效的DNS查询",
    ["dns_query"] = "DNS查询",
    ["docker"] = "docker",
    ["docker_name"] = "名称",
    ["drop_flow_traffic_btn"] = "丢弃流",
    ["dropped_packets"] = "丢包",
    ["dtmf_tones_sent"] = "通话期间发送的 DTMF",
    ["dump_flow_traffic"] = "转储流量",
    ["elephant_exceeded"] = "[本地到远程：%{l2r}][远程到本地：%{r2l}]",
    ["elephant_flow"] = "大数据流",
    ["elephant_flow_descr"] = "大数据流是那些传输超过一定配置数据量的流。 管理员可以从首选项配置大数据流。",
    ["elephant_flow_l2r"] = "大数据流（本地到远程流量）",
    ["elephant_flow_r2l"] = "大数据流（远程到本地流量）",
    ["entropy"] = "熵",
    ["error_and_higher"] = "错误和更高",
    ["existing_rules_note"] = "现有规则可以在 <a href=\"%{url}\">%{name}</a> 页面找到。",
    ["external_alert"] = "检测到 %{scope} 警告：%{message} [%{maker}]",
    ["first_last_flow_sequence"] = "第一个/最后一个流序列",
    ["flow_active_msg"] = "流处于活动状态。",
    ["flow_alerted"] = "警告",
    ["flow_anomalies"] = "检测到的流风险",
    ["flow_blocked_by_bridge"] = "由于配置的策略而拦截流",
    ["flow_cannot_be_found_message"] = "找不到此流。",
    ["flow_completed_msg"] = "流已关闭。",
    ["flow_connecting_msg"] = "流量正在连接。",
    ["flow_details"] = "流详细",
    ["flow_emitted"] = "网络接口收到警告时发出的流",
    ["flow_low_goodput"] = "低Goodput率[%{ratio}%%]",
    ["flow_marker"] = "流量标记",
    ["flow_peer_roles_inaccurate_msg"] = "流处于活动状态,但是,尚未看到流的开头,并且Peers（客户端/服务器）可能不准确",
    ["flow_peers_client_server"] = "流 Peers [客户端/服务器]",
    ["flow_quota"] = "流配额",
    ["flow_reset_by_resetter_msg"] = "由 %{resetter} 重置流。",
    ["flow_reset_msg"] = "此流已重置",
    ["flow_score"] = "流分数细分",
    ["flow_shapers"] = "流Shaper",
    ["flow_traffic_is_dropped"] = "流流量被丢弃",
    ["forward"] = "向前",
    ["gsm_cell_identifier"] = "GSM 小区标识符",
    ["gsm_cell_lac"] = "GSM 小区 LAC（位置区号）",
    ["host_successfully_added_to_category"] = "主机 \"%{host}\" 已成功添加到 <a href=\"%{url}\">%{category}</a> 类别。 只有新的流会受到规则的影响。",
    ["http_method"] = "HTTP 方法",
    ["icmp_info"] = "ICMP信息",
    ["ids_alert"] = "检测到 %{scope} 警告：%{message} [%{maker}]",
    ["iec104_latency"] = "I-S 型 Ack 延迟（平均/标准偏差）",
    ["iec104_mask"] = "<A HREF=https://infosys.beckhoff.com/english.php?content=../content/1033/tcplclibiec870_5_104/html/tcplclibiec870_5_104_objref_overview.htm&id>TypeIDs</A> <i class='-fas fa-external 链路-alt'></i>",
    ["iec104_msg_breakdown"] = "消息细分",
    ["iec104_msg_loss"] = "消息丢失/重传",
    ["iec104_transitions"] = "类型 ID 转换",
    ["imsi"] = "IMSI（国际移动用户身份）",
    ["io_wait_time_percentage"] = "I/O 等待时间百分比",
    ["ip_packet_analysis"] = "IP数据包分析",
    ["jitter"] = "抖动",
    ["k8s"] = "Kubernetes",
    ["k8s_name"] = "名称",
    ["k8s_ns"] = "命名空间",
    ["k8s_pod"] = "Pod",
    ["known_proto_on_non_std_port"] = "在非标准端口上发现已知应用程序",
    ["longlived_exceeded"] = "超过 %{amount}",
    ["longlived_flow"] = "长期流",
    ["longlived_flow_descr"] = "长期流是指持续时间超过特定配置时间的流。 管理员可以从首选项配置大数据流。",
    ["looks_like_idle_flow_message"] = "这看起来像一个<font color=red>空闲流</font>,周期性传输只是为了让它保持活力。",
    ["lost_packets"] = "丢包",
    ["low_goodput"] = "低Goodput",
    ["malicious_ja3_signature"] = "可能是恶意的 %{cli_or_srv} 签名 [JA3: <a href=\"%{url}\">%{signature}</a> %{icon}]",
    ["max_estimated_tcp_throughput"] = "最大（估计）TCP 吞吐量",
    ["max_packet_interarrival_time"] = "最大数据包到达间隔时间",
    ["mean_rtt"] = "平均 RTT",
    ["memory_actual_peak"] = "内存实际/峰值",
    ["mibehaving_flows"] = "行为异常的流",
    ["normal"] = "正常",
    ["not_purged"] = "没有从内存中清除",
    ["not_recommended_label"] = "不建议",
    ["notice_and_lower"] = "通知和降低",
    ["nsapi"] = "NSAPI",
    ["ongoing_call"] = "持续通话",
    ["packet_inter_arrival_time"] = "数据包到达间隔时间 <br>[ Min /Avg /Max ]",
    ["payload_type"] = "有效载荷类型",
    ["potentially_dangerous_protocol"] = "潜在危险协议",
    ["process_information_report_warning"] = "进程信息报告是有限的，除非您将ntopng与<A HREF='%{url}'>nProbe</A>和sprobe插件一起使用",
    ["process_pid_name"] = "进程 PID/名称",
    ["profile_name"] = "配置文件名称",
    ["pseudo_mos"] = "（伪）MOS",
    ["r_factor"] = "R因子",
    ["reach_connection_label"] = "到达连接",
    ["remote_to_remote"] = "远程客户端和远程服务器",
    ["response_code"] = "响应代码",
    ["reverse"] = "翻转",
    ["round_trip_time"] = "最小/最大 RTT",
    ["rtp_codecs"] = "RTP 编解码器",
    ["rtp_flow"] = "RTP流",
    ["rtp_protocol_information"] = "RTP协议信息",
    ["rtp_stream_peers"] = "RTP 流peers",
    ["rtp_transit_in_out"] = "RTP 传输输入/输出",
    ["rtt_breakdown"] = "RTT 时间细分",
    ["rtt_distance"] = "客户端/服务器估计距离",
    ["rtt_jitter"] = "RTT抖动",
    ["rule_type"] = "规则类型",
    ["sac_service_area_code"] = "SAC（服务区号）",
    ["score_category_network"] = "网络",
    ["score_category_security"] = "网络安全",
    ["server_container_information"] = "服务器容器信息",
    ["server_name"] = "服务器名称",
    ["server_process_information"] = "服务器进程信息",
    ["sip_call_id"] = "SIP 呼叫 ID",
    ["sip_protocol_information"] = "SIP 协议信息",
    ["slow_application_header"] = "缓慢的应用程序header",
    ["slow_data_exchange"] = "缓慢的数据交换",
    ["slow_tcp_connection"] = "缓慢的TCP 连接",
    ["son_of_father_process"] = "<a href ='%{url}'>%{proc_father_name} [pid: %{proc_father_pid}]</a> 的子进程",
    ["ssh_signature"] = "SSH 签名",
    ["suspicious_client_device_protocol"] = "客户端应用程序。 <a href=\"%{url}\">%{devtype} 应用程序策略</a>不允许使用 \"%{proto}\"",
    ["suspicious_device_protocol"] = "可疑设备协议",
    ["suspicious_server_device_protocol"] = "<a href=\"%{url}\">%{devtype} 应用程序策略</a>不允许的服务器应用程序\"%{proto}\"",
    ["suspicious_tcp_probing"] = "可疑的 TCP 探测",
    ["suspicious_tcp_syn_probing"] = "可疑的 TCP SYN 探测",
    ["sync_source_id"] = "同步源 ID",
    ["tcp_connection_issues"] = "TCP 连接问题",
    ["tcp_packet_analysis"] = "TCP数据包分析",
    ["tcp_severe_connection_issues"] = "严重的 TCP 问题",
    ["throughput_trend"] = "吞吐量趋势",
    ["tls_certificate"] = "TLS 证书",
    ["tls_certificate_expired"] = "TLS 证书已过期",
    ["tls_certificate_invalid"] = "TLS 证书无效",
    ["tls_certificate_mismatch"] = "TLS 证书不匹配",
    ["tls_certificate_selfsigned"] = "TLS 证书自签名",
    ["tls_certificate_validity"] = "TLS 证书有效性",
    ["tls_old_protocol_version"] = "过期的 TLS 版本",
    ["tls_server_names"] = "服务器名称",
    ["tls_unsafe_ciphers"] = "不安全的 TLS 密码",
    ["tos"] = "<A target='_blank' class='ntopng-external-link' href='https://en.wikipedia.org/wiki/Differentiated_services'>DSCP <i class='fas fa-external-link-alt'></i></A> /<A class='ntopng-external-link' target='_blank' href='https://en.wikipedia.org/wiki/Explicit_Congestion_Notification'>ECN <i class='fas fa-external-link-alt'></i></A> [ 客户端/服务器 ]",
    ["udp_unidirectional"] = "单向 UDP 流量",
    ["unknown_status"] = "未知状态 (%{status})",
    ["url"] = "URL",
    ["user_name"] = "用户名",
    ["vm_page_faults"] = "虚拟机页面错误",
    ["warning"] = "警告",
    ["web_mining_detected"] = "该网站以在客户端设备上挖掘加密货币而闻名",
  },
  ["flow_devices"] = {
    ["Probes"] = "探针",
    ["active_sflow"] = "活动 sFlow 设备",
    ["all_device_ports"] = "所有 %{device} 端口",
    ["all_flow_devices"] = "所有流设备",
    ["all_probes"] = "所有探针",
    ["all_sflow_devices"] = "所有 sFlow 设备",
    ["duplex"] = "双工",
    ["enable_flow_ratio"] = "启用Flow/SNMP 比率",
    ["exporter_ip"] = "探针 IP",
    ["exporters"] = "探针",
    ["flow_device_x"] = "探针 %{device}",
    ["flow_device_x_interface_y"] = "探针 %{device} /%{interface}",
    ["flow_exporter_devices"] = "探针设备",
    ["flow_ratio_snmp_instructions"] = "要启用Flow/SNMP 比率,此流设备必须支持 SNMP,并且必须<a href='%{href}'>添加</a>到 SNMP 设备。",
    ["flow_ratio_timeseries_instructions"] = "要启用 Flow/SNMP 比率,必须启用 <b>SNMP 和流设备时间序列</b>。",
    ["full_duplex"] = "全双工",
    ["half_duplex"] = "半双工",
    ["in_errors"] = "错误",
    ["note_condition_1"] = "1)设备 %{device} 必须支持 SNMP,并且必须在 <a href=\"%{url}\">SNMP</a> 设备页面中进行配置",
    ["note_condition_2"] = "2)必须从 <A HREF=\"%{url}\"><i class=\"fas fa-flask\"></i> Preferences</A> 启用 SNMP 和流量设备时间序列",
    ["note_observation_points_fields1"] = "可以从<a href=\"%{url}\"><i class=\"fas fa flask\"></i>首选项启用观测点时间序列。需要几分钟才能看到第一个数据点",
    ["note_observation_points_fields2"] = "检查配置新观察点的方法，请检查<a href=\"%{url}\"><i class=\"fas fa external link alt\"></i>此处</a>。需要重新启动。",
    ["note_snmp_device"] = "仅当在<a href=\"%{url}\">SNMP设备</a>中找到探针IP相同的SNMP设备时,才会显示SNMP相关信息。",
    ["note_zmq_fields1"] = "流量设备是使用 ZMQ 字段%%EXPORTER_IPV4_ADDRESS 和%%NPROBE_IPV4_ADDRESS 确定的。 当两个字段都可用时,前者优先于后者",
    ["note_zmq_fields2"] = "流设备接口使用 ZMQ 字段%%INPUT_SNMP 和%%OUTPUT_SNMP 确定",
    ["note_zmq_fields3"] = "sFlow设备使用通过ZMQ接收的sFlow 字段设备IP（主题'Counter'）确定",
    ["note_zmq_fields4"] = "sFlow设备接口使用通过ZMQ（主题'Counter'）接收的字段ifIndex确定",
    ["out_errors"] = "输出错误",
    ["promisc"] = "混杂",
    ["ratio_not_ready"] = "计算流量比...",
    ["remember_counter_ratio"] = "可以按照以下步骤启用 Counter/SNMP 比率：",
    ["remember_ratio"] = "可以按照以下步骤启用 Flow/SNMP 比率：",
    ["sflow_device_x"] = "sFlow 设备 %{device}",
    ["sflow_snmp_ratio"] = "计数器/SNMP 比率",
    ["sflow_snmp_ratio_descr"] = "计数器/SNMP 比率的计算方法是将使用 sFlow 计数器样本获得的总接口流量除以最近 30 分钟内通过 SNMP 读取的同一接口的总流量",
    ["snmp_description"] = "SNMP 描述",
    ["snmp_devmodel"] = "SNMP 设备型号",
    ["snmp_devname"] = "SNMP 设备名称",
    ["snmp_ifindex"] = "SNMP索引",
    ["snmp_ifname"] = "SNMP接口",
    ["snmp_location"] = "SNMP 位置",
    ["snmp_no_device_found"] = "找不到匹配的 SNMP 设备",
    ["snmp_polling"] = "Flow/SNMP 比率需要至少 30 分钟的 SNMP 和流量数据",
    ["snmp_ratio"] = "Flow/SNMP 比率",
    ["snmp_ratio_descr"] = "Flow/SNMP 比率的计算方法是将使用流获得的总接口流量除以通过 SNMP 读取的同一接口的总流量。 由于在基于流的分析中不考虑非 IP 和第 2 层标头,<br>典型的比率值介于 0.8 和 0.9 之间（即 0.9 表示通过 SNMP 观察到的 90% 的接收流量已在流中报告） .<br>比率是使用最近 5 分钟内的流量计算得出的,因为定期 SNMP 以5分钟为间隔执行。 比率仅在满足以下条件时计算",
    ["this_page_contains_couters"] = "此页面包含通过 sFlow 收集的接口计数器。 如果配置了SNMP,还可以看到本设备的<A HREF=\"%{url}\">SNMP视图</A>",
    ["timeseries_descr"] = "可以从 <a href=\"%{url}\"><i class=\"fas fa-flask\"></i> Preferences</a> 启用流设备时间序列。 需要几分钟才能看到第一个数据点",
    ["tot_bytes"] = "总字节数",
    ["tot_flows"] = "总流数",
  },
  ["flow_fields_description"] = {
    ["appl_latency_ms"] = "应用程序延迟（毫秒）,又名服务器响应时间",
    ["application_id"] = "收集的应用程序 ID（Cisco 或 IXIA）",
    ["application_name"] = "Palo Alto应用程序 ID",
    ["biflow_direction"] = "1=启动器,2=反向启动器",
    ["bittorrent_hash"] = "BITTORRENT 哈希",
    ["client_nw_latency_ms"] = "网络 RTT/2 客户端 <-> nprobe（毫秒）",
    ["community_id"] = "Community ID",
    ["cumulative_icmp_type"] = "ICMP 类型数据包的累积 OR",
    ["dhcp_client_ip"] = "DHCP分配的客户端 IPv4 地址",
    ["dhcp_client_mac"] = "DHCP客户端MAC",
    ["dhcp_client_name"] = "DHCP客户端名称",
    ["dhcp_message_type"] = "DHCP报文类型",
    ["dhcp_remote_id"] = "DHCP代理远程 ID",
    ["dhcp_subscriber_id"] = "DHCP用户 ID",
    ["diameter_clr_cancel_type"] = "diameter取消类型",
    ["diameter_clr_flags"] = "diameter CLR 标志",
    ["diameter_exp_res_result_code"] = "diameter响应实验结果代码",
    ["diameter_exp_res_vendor_id"] = "diameter响应实验结果供应商 ID",
    ["diameter_hop_by_hop_id"] = "diameter 逐跳标识符",
    ["diameter_req_msg_type"] = "diameter请求消息类型",
    ["diameter_req_origin_host"] = "diameter 原始主机请求",
    ["diameter_req_user_name"] = "diameter 请求用户名",
    ["diameter_rsp_msg_type"] = "diameter响应消息类型",
    ["diameter_rsp_origin_host"] = "diameter 源主机响应",
    ["diameter_rsp_result_code"] = "diameter响应结果代码",
    ["direction"] = "流向 [0=src->dst, 1=dst->src]",
    ["dns_num_answers"] = "DNS响应的数量",
    ["dns_query"] = "DNS查询",
    ["dns_query_id"] = "DNS查询事务 ID",
    ["dns_query_type"] = "DNS查询类型（例如 1=A,2=NS..）",
    ["dns_response"] = "DNS响应",
    ["dns_ret_code"] = "DNS返回码（例如 0=无错误）",
    ["dns_ttl_answer"] = "第一个 A 记录的 TTL（如果有）",
    ["dns_tx_id"] = "DNS事务ID",
    ["dot1qCustomerVlanId"] = "dot1Q 客户 Vlan Id",
    ["dot1q_dst_vlan"] = "目的VLAN（QinQ外层VLAN）",
    ["dot1q_src_vlan"] = "源VLAN（QinQ的外层VLAN）",
    ["downstream_session_id"] = "下游会话标识符（例如 L2TP）或 0 如果未知",
    ["downstream_tunnel_id"] = "下游隧道标识符（例如 GTP TEID、VXLAN VNI）或 0（如果未知）",
    ["dst_as"] = "目的BGP AS",
    ["dst_as_map"] = "DST_AS 的组织名称",
    ["dst_as_path_1"] = "目的 AS 路径位置 1",
    ["dst_as_path_10"] = "目的 AS 路径位置 10",
    ["dst_as_path_2"] = "目的 AS 路径位置 2",
    ["dst_as_path_3"] = "目的 AS 路径位置 3",
    ["dst_as_path_4"] = "目的 AS 路径位置 4",
    ["dst_as_path_5"] = "目的 AS 路径位置 5",
    ["dst_as_path_6"] = "目的 AS 路径位置 6",
    ["dst_as_path_7"] = "目的 AS 路径位置 7",
    ["dst_as_path_8"] = "目的 AS 路径位置 8",
    ["dst_as_path_9"] = "目的 AS 路径位置 9",
    ["dst_father_proc_name"] = "Dst父进程名",
    ["dst_father_proc_pid"] = "Dst父进程PID",
    ["dst_fragments"] = "分片数据包数量 dst->src",
    ["dst_ip_city"] = "Dst IP所在城市",
    ["dst_ip_country"] = "Dst IP所在国家",
    ["dst_ip_lat"] = "Dst IP所在纬度",
    ["dst_ip_long"] = "Dst IP所在的经度",
    ["dst_proc_actual_memory"] = "Dst 进程实际内存（字节）",
    ["dst_proc_average_cpu_load"] = "Dst 进程平均负载 (%% * 100)",
    ["dst_proc_name"] = "Dst 进程名称",
    ["dst_proc_num_page_faults"] = "Dst进程数页面错误",
    ["dst_proc_pctg_iowait"] = "Dst进程iowait时间%%(%%*100)",
    ["dst_proc_peak_memory"] = "Dst进程峰值内存（字节）",
    ["dst_proc_pid"] = "Dst进程PID",
    ["dst_proc_uid"] = "Dst 进程 UID",
    ["dst_proc_user_name"] = "dst 进程用户名",
    ["dst_to_src_avg_throughput"] = "Dst 到 Src 平均吞吐量 (bps)",
    ["dst_to_src_max_throughput"] = "Dst 到 Src 最大吞吐量 (bps)",
    ["dst_to_src_min_throughput"] = "Dst 到 Src 最小吞吐量 (bps)",
    ["dst_tos"] = "TOS/DSCP (dst->src)",
    ["dst_vlan"] = "目的VLAN（QinQ内层VLAN）",
    ["duration_in"] = "客户端到服务器流持续时间（毫秒）",
    ["duration_out"] = "客户端到服务器流持续时间（毫秒）",
    ["engine_id"] = "流切换引擎的id",
    ["engine_type"] = "流切换引擎",
    ["exporter_ipv4_address"] = "探针 IPv4 地址",
    ["exporter_ipv6_address"] = "探针 IPv6 地址",
    ["file_gaps"] = "文件间隙",
    ["file_id"] = "文件ID",
    ["file_name"] = "文件名",
    ["file_size"] = "文件大小",
    ["file_state"] = "文件状态",
    ["file_stored"] = "文件存储",
    ["firewall_event"] = "防火墙事件 0=忽略,1=流创建,2=流删除,3=流被拒绝,4=流警告,5=流更新",
    ["first_switched"] = "第一个流pkt的SysUptime（毫秒）",
    ["flow_active_timeout"] = "流缓存条目的活动超时",
    ["flow_duration_microseconds"] = "流持续时间 (微妙)",
    ["flow_duration_milliseconds"] = "流持续时间（毫秒）",
    ["flow_end_microseconds"] = "第一个流数据包-微妙（epoch）",
    ["flow_end_milliseconds"] = "最后一个流数据包-毫秒（epoch）",
    ["flow_end_reason"] = "流结束原因",
    ["flow_end_sec"] = "最后一个流数据包的秒数（epoch）",
    ["flow_id"] = "串行流标识符",
    ["flow_inactive_timeout"] = "流缓存条目的不活动超时",
    ["flow_proto_port"] = "标识流应用程序的 L7 端口,如果未知则为 0",
    ["flow_server_name"] = "流服务器名称（如果知道）",
    ["flow_start_microseconds"] = "第一个流数据包-微妙（epoch）",
    ["flow_start_milliseconds"] = "第一个流数据包-毫秒（epoch）",
    ["flow_start_sec"] = "第一个流数据包的秒数（epoch）",
    ["flow_user_name"] = "隧道的流用户名（如果知道）",
    ["flow_verdict"] = "流判定",
    ["frame_length"] = "原始 L2 帧长",
    ["ftp_command"] = "FTP客户端命令",
    ["ftp_command_ret_code"] = "FTP客户端命令返回码",
    ["ftp_login"] = "FTP客户端登录",
    ["ftp_password"] = "FTP客户端密码",
    ["gtpv0_apn_name"] = "GTPv0 APN 名称",
    ["gtpv0_end_user_ip"] = "GTPv0 最终用户 IP 地址",
    ["gtpv0_end_user_msisdn"] = "GTPv0 最终用户 MSISDN",
    ["gtpv0_rai_cell_lac"] = "GTPv0 小区位置区号",
    ["gtpv0_rai_cell_rac"] = "GTPv0 小区路由区号",
    ["gtpv0_rai_mcc"] = "GTPv0 移动国家代码",
    ["gtpv0_rai_mnc"] = "GTPv0 移动网络代码",
    ["gtpv0_req_msg_type"] = "GTPv0 请求消息类型",
    ["gtpv0_response_cause"] = "GTPv0 操作原因",
    ["gtpv0_rsp_msg_type"] = "GTPv0 响应消息类型",
    ["gtpv0_tid"] = "GTPv0 隧道标识符",
    ["gtpv1_apn_name"] = "GTPv1 APN 名称",
    ["gtpv1_c2s_teid_ctrl"] = "GTPv1 客户端->服务器 TunnelId 控制",
    ["gtpv1_c2s_teid_data"] = "GTPv1 客户端->服务器 TunnelId 数据",
    ["gtpv1_end_user_imei"] = "GTPv1 最终用户 IMEI",
    ["gtpv1_end_user_imsi"] = "GTPv1 最终用户 IMSI",
    ["gtpv1_end_user_ip"] = "GTPv1 最终用户 IP 地址",
    ["gtpv1_end_user_msisdn"] = "GTPv1 最终用户 MSISDN",
    ["gtpv1_rai_lac"] = "GTPv1 RAI 位置区号",
    ["gtpv1_rai_mcc"] = "GTPv1 RAI 移动国家代码",
    ["gtpv1_rai_mnc"] = "GTPv1 RAI 移动网络代码",
    ["gtpv1_rai_rac"] = "GTPv1 RAI 路由区号",
    ["gtpv1_rat_type"] = "GTPv1 RAT 类型",
    ["gtpv1_req_msg_type"] = "GTPv1 请求消息类型",
    ["gtpv1_response_cause"] = "GTPv1 运行原因",
    ["gtpv1_rsp_msg_type"] = "GTPv1 响应消息类型",
    ["gtpv1_s2c_teid_ctrl"] = "GTPv1 服务器->客户端 TunnelId 控制",
    ["gtpv1_s2c_teid_data"] = "GTPv1 服务器->客户端 TunnelId 数据",
    ["gtpv1_uli_cell_ci"] = "GTPv1 ULI 小区 CI",
    ["gtpv1_uli_cell_lac"] = "GTPv1 ULI 小区位置区号",
    ["gtpv1_uli_mcc"] = "GTPv1 ULI 移动国家代码",
    ["gtpv1_uli_mnc"] = "GTPv1 ULI 移动网络代码",
    ["gtpv1_uli_sac"] = "GTPv1 ULI SAC",
    ["gtpv2_apn_name"] = "GTPv2 APN 名称",
    ["gtpv2_c2s_s1u_gtpu_ip"] = "GTPv2 客户端->Svr S1U GTPU IP",
    ["gtpv2_c2s_s1u_gtpu_teid"] = "GTPv2 客户端->Svr S1U GTPU TEID",
    ["gtpv2_c2s_s5_s8_gtpc_ip"] = "GTPv2 客户端->Svr S5/S8 GTPC IP",
    ["gtpv2_c2s_s5_s8_gtpu_ip"] = "GTPv2 客户端->Srv S5/S8 PGW GTPU IP",
    ["gtpv2_c2s_s5_s8_gtpu_teid"] = "GTPv2 客户端->Srv S5/S8 PGW GTPU TEID",
    ["gtpv2_c2s_s5_s8_sgw_gtpu_ip"] = "GTPv2 客户端->Srv S5/S8 SGW GTPU IP",
    ["gtpv2_c2s_s5_s8_sgw_gtpu_teid"] = "GTPv2 客户端->Srv S5/S8 SGW GTPU TEID",
    ["gtpv2_end_user_imei"] = "GTPv2 最终用户 IMEI",
    ["gtpv2_end_user_imsi"] = "GTPv2 最终用户 IMSI",
    ["gtpv2_end_user_msisdn"] = "GTPv2 最终用户 MSISDN",
    ["gtpv2_pdn_ip"] = "GTPV2 PDN IP 地址",
    ["gtpv2_rat_type"] = "GTPv2 RAT 类型",
    ["gtpv2_req_msg_type"] = "GTPv2 请求消息类型",
    ["gtpv2_response_cause"] = "GTPv2 运行原因",
    ["gtpv2_rsp_msg_type"] = "GTPv2 响应消息类型",
    ["gtpv2_s2c_s1u_gtpu_ip"] = "GTPv2的Srv->客户端S1U GTPU IP",
    ["gtpv2_s2c_s1u_gtpu_teid"] = "GTPv2 Srv->客户端 S1U GTPU TEID",
    ["gtpv2_s2c_s5_s8_gtpc_ip"] = "GTPv2 Svr->客户端 S5/S8 GTPC IP",
    ["gtpv2_s2c_s5_s8_gtpu_ip"] = "GTPv2的Srv->客户端S5 /S8 PGW GTPU IP",
    ["gtpv2_s2c_s5_s8_gtpu_teid"] = "GTPv2 Srv->客户端 S5/S8 PGW GTPU TEID",
    ["gtpv2_s2c_s5_s8_sgw_gtpu_ip"] = "GTPv2的Srv->客户端S5 /S8 SGW GTPU IP",
    ["gtpv2_s2c_s5_s8_sgw_gtpu_teid"] = "GTPv2 Srv->客户端 S5/S8 SGW GTPU TEID",
    ["gtpv2_s5_s8_gtpc_teid"] = "GTPv2 S5/S8 SGW GTPC TEIDs",
    ["gtpv2_uli_cell_id"] = "GTPv2 小区标识符",
    ["gtpv2_uli_cell_tac"] = "GTPv2的跟踪区域代码",
    ["gtpv2_uli_mcc"] = "GTPv2 移动国家代码",
    ["gtpv2_uli_mnc"] = "GTPv2 移动网络代码",
    ["httpMessageVersion"] = "HTTP 消息版本",
    ["httpRequestHost"] = "HTTP 请求主机",
    ["httpRequestMethod"] = "HTTP 请求方法",
    ["httpRequestTarget"] = "HTTP 请求目标",
    ["httpStatusCode"] = "HTTP 状态码",
    ["httpUserAgent"] = "HTTP 用户代理",
    ["http_host"] = "HTTP 主机名（IXIA 主机名）",
    ["http_length"] = "HTTP 内容长度",
    ["http_method"] = "HTTP方法",
    ["http_mime"] = "HTTP Mime 类型",
    ["http_protocol"] = "HTTP 协议",
    ["http_referer"] = "HTTP 引用",
    ["http_ret_code"] = "HTTP 返回码（例如 200、304...）",
    ["http_site"] = "没有主机名的HTTP服务器",
    ["http_ua"] = "HTTP 用户代理",
    ["http_url"] = "HTTP URL (IXIA URI)",
    ["http_via"] = "HTTP 通过",
    ["http_x_forwarded_for"] = "HTTP X-Forwarded-For",
    ["icmp_ipv4_code"] = "ICMP代码",
    ["icmp_ipv4_type"] = "ICMP类型",
    ["icmp_type"] = "ICMP 类型 * 256 + ICMP 代码",
    ["imap_login"] = "邮件发件人",
    ["in_bytes"] = "传入流字节（src->dst）",
    ["in_dst_mac"] = "目的 MAC 地址",
    ["in_pkts"] = "传入流数据包（src->dst）",
    ["in_src_mac"] = "源 MAC 地址",
    ["in_src_osi_sap"] = "OSI 源 SAP（仅限 OSI 流量）",
    ["ingress_vrfid"] = "入口 VRF ID",
    ["input_snmp"] = "输入接口 SNMP idx",
    ["ipDSCP"] = "IP DSCP",
    ["ipPayloadLenght"] = "IP有效载荷长度",
    ["ipPrecendence"] = " IP优先级",
    ["ip_lenght_maximum"] = "IP最大长度",
    ["ip_lenght_minimum"] = "IP最小长度",
    ["ip_protocol_version"] = "[4=IPv4][6=IPv6]",
    ["ipfix_sampling_algorithm"] = "采样算法",
    ["ipv4_dst_addr"] = "IPv4 目标地址",
    ["ipv4_dst_mask"] = "IPv4 目的地址子网掩码 (/<bits>)",
    ["ipv4_next_hop"] = "IPv4 下一跳地址",
    ["ipv4_src_addr"] = "IPv4 源地址",
    ["ipv4_src_mask"] = "IPv4 源子网掩码 (/<bits>)",
    ["ipv6_dst_addr"] = "IPv6 目的地址",
    ["ipv6_dst_mask"] = "IPv6 目标掩码",
    ["ipv6_next_hop"] = "IPv6 下一跳地址",
    ["ipv6_src_addr"] = "IPv6 源地址",
    ["ipv6_src_mask"] = "IPv6 源掩码",
    ["l4_dst_port"] = "4层目的端口",
    ["l4_dst_port_map"] = "4层目的端口符号名",
    ["l4_src_port"] = "IPv4 源端口",
    ["l4_src_port_map"] = "4层源端口符号名",
    ["l4_srv_port"] = "4层服务器端口",
    ["l4_srv_port_map"] = "4层服务器端口符号名",
    ["l7_proto"] = "第 7 层应用程序（数字）",
    ["l7_proto_name"] = "七层应用名称",
    ["l7_risk_score"] = "nDPI 流风险分数",
    ["last_switched"] = "最后一个流pkt的SysUptime（毫秒）",
    ["layer2FrameDeltaCount"] = "L2 帧Delta计数",
    ["layer2FrameTotalCount"] = "L2 帧总数",
    ["layer2OctetDeltaSumOfSquares"] = "L2 Octets Delta 平方和",
    ["layer2OctetTotalSumOfSquares"] = "L2 Octets总平方和",
    ["longest_flow_pkt"] = "流的最长数据包（字节）",
    ["max_ttl"] = "最大流量TTL",
    ["maximumLayer2TotalLength"] = "最大 L2 总长度",
    ["min_ttl"] = "最小流量TTL",
    ["minimumLayer2TotalLength"] = "最小 L2 总长度",
    ["mpls_label_1"] = "位置 1 处的 MPLS 标签",
    ["mpls_label_10"] = "位置 10 处的 MPLS 标签",
    ["mpls_label_2"] = "位置 2 处的 MPLS 标签",
    ["mpls_label_3"] = "位置 3 处的 MPLS 标签",
    ["mpls_label_4"] = "位置 4 处的 MPLS 标签",
    ["mpls_label_5"] = "位置 5 处的 MPLS 标签",
    ["mpls_label_6"] = "位置 6 处的 MPLS 标签",
    ["mpls_label_7"] = "位置 7 处的 MPLS 标签",
    ["mpls_label_8"] = "位置 8 处的 MPLS 标签",
    ["mpls_label_9"] = "位置 9 处的 MPLS 标签",
    ["mysql_appl_latency_usec"] = "MySQL 请求->响应延迟（usec）",
    ["mysql_db"] = "正在使用的 MySQL 数据库",
    ["mysql_query"] = "MySQL查询",
    ["mysql_response"] = "MySQL服务器响应",
    ["mysql_server_version"] = "MySQL服务器版本",
    ["mysql_username"] = "用户名",
    ["netbios_query_name"] = "NETBIOS 查询名称",
    ["netbios_query_os"] = "NETBIOS 查询操作系统",
    ["netbios_query_type"] = "NETBIOS 查询类型",
    ["netbios_response"] = "NETBIOS 查询响应",
    ["nprobe_ipv4_address"] = "主机IPv4地址是 nProbe的地址 ",
    ["num_pkts_1024_to_1514_bytes"] = "# IP 大小 > 1024 且 <= 1514 的数据包",
    ["num_pkts_128_to_256_bytes"] = "# IP 大小 > 128 且 <= 256 的数据包",
    ["num_pkts_256_to_512_bytes"] = "# IP 大小 > 256 且 < 512 的数据包",
    ["num_pkts_512_to_1024_bytes"] = "# IP 大小 > 512 且 < 1024 的数据包",
    ["num_pkts_over_1514_bytes"] = "# IP 大小 > 1514 的数据包",
    ["num_pkts_ttl_128_160"] = "# TTL > 128 且 <= 160 的数据包",
    ["num_pkts_ttl_160_192"] = "# TTL > 160 且 <= 192 的数据包",
    ["num_pkts_ttl_192_224"] = "# TTL > 192 且 <= 224 的数据包",
    ["num_pkts_ttl_224_255"] = "# TTL > 224 且 <= 255 的数据包",
    ["num_pkts_ttl_2_5"] = "# TTL > 1 和 TTL <= 5 的数据包",
    ["num_pkts_ttl_32_64"] = "# TTL > 32 且 <= 64 的数据包",
    ["num_pkts_ttl_5_32"] = "# TTL > 5 和 TTL <= 32 的数据包",
    ["num_pkts_ttl_64_96"] = "# TTL > 64 且 <= 96 的数据包",
    ["num_pkts_ttl_96_128"] = "# TTL > 96 且 <= 128 的数据包",
    ["num_pkts_ttl_eq_1"] = "# TTL = 1 的数据包",
    ["num_pkts_up_to_128_bytes"] = "# IP 大小 <= 128 的数据包",
    ["observation_point_id"] = "观察点 ID",
    ["observation_point_type"] = "观察点类型",
    ["ooorder_in_pkts"] = "乱序 TCP 流数据包数 (dst->src)",
    ["ooorder_out_pkts"] = "乱序 TCP 流数据包数 (src->dst)",
    ["oracle_query"] = "Oracle查询",
    ["oracle_query_duration"] = "Oracle 查询持续时间（毫秒）",
    ["oracle_rsp_code"] = "Oracle 响应代码",
    ["oracle_rsp_string"] = "Oracle 响应字符串",
    ["oracle_username"] = "Oracle用户名",
    ["out_bytes"] = "传出流字节 (dst->src) [别名为%%DST_TO_SRC_BYTES]",
    ["out_dst_mac"] = "目标 MAC 地址,可能由观察点之后的中间功能设备修改",
    ["out_dst_osi_sap"] = "OSI 目标 SAP（仅限 OSI 流量）",
    ["out_pkts"] = "传出流数据包 (dst->src) [别名为%%DST_TO_SRC_PKTS]",
    ["out_src_mac"] = "源 MAC 地址,可能由观察点之后的中间功能设备修改",
    ["output_snmp"] = "输出接口 SNMP idx",
    ["packet_section_offset"] = "数据包部分偏移",
    ["packets_observed"] = "看到的数据包总数",
    ["packets_selected"] = "选择采样的包数",
    ["payload_hash"] = "初始流有效载荷hash",
    ["plugin_name"] = "此流使用的插件名称（如果有）",
    ["pop_user"] = "POP3用户登录",
    ["post_napt_dst_transport_port"] = "NAPT后目的传输端口",
    ["post_napt_src_transport_port"] = "NAPT后源传输端口",
    ["post_nat_dst_ipv4_addr"] = "NAT后目的IPv4地址",
    ["post_nat_src_ipv4_addr"] = "NAT后源IPv4地址",
    ["protocol"] = "IP协议字节",
    ["protocol_map"] = "IP协议名称",
    ["radius_acct_in_octets"] = "RADIUS 计算输入Octets",
    ["radius_acct_in_pkts"] = "RADIUS 计算输入数据包",
    ["radius_acct_out_octets"] = "RADIUS 计算输出Octets",
    ["radius_acct_out_pkts"] = "RADIUS 计算输出数据包",
    ["radius_acct_session_id"] = "RADIUS 计算会话名称",
    ["radius_acct_status_type"] = "RADIUS 计算状态类型",
    ["radius_called_station_id"] = "RADIUS 被呼叫站 ID",
    ["radius_calling_station_id"] = "RADIUS 呼叫站 ID",
    ["radius_framed_ip_addr"] = "RADIUS 帧 IP",
    ["radius_nas_identifier"] = "RADIUS NAS 标识符",
    ["radius_nas_ip_addr"] = "RADIUS NAS IP 地址",
    ["radius_req_msg_type"] = "RADIUS 请求消息类型",
    ["radius_rsp_msg_type"] = "RADIUS 响应消息类型",
    ["radius_user_imei"] = "RADIUS 用户 MSISDN（扩展）",
    ["radius_user_imsi"] = "RADIUS 用户 IMSI（扩展）",
    ["radius_user_name"] = "RADIUS 用户名（仅访问）",
    ["retransmitted_in_bytes"] = "重传的 TCP 流字节数 (src->dst)",
    ["retransmitted_in_pkts"] = "重传的 TCP 流数据包数（src->dst）",
    ["retransmitted_out_bytes"] = "重传的 TCP 流字节数 (dst->src)",
    ["retransmitted_out_pkts"] = "重传的 TCP 流包数 (dst->src)",
    ["rtp_dtmf_tones"] = "通话期间发送的 DTMF 音（如果有）",
    ["rtp_first_seq"] = "第一个流 RTP 序列号",
    ["rtp_first_ts"] = "第一个流 RTP 时间戳",
    ["rtp_in_jitter"] = "RTP 抖动 (ms * 1000)",
    ["rtp_in_max_delta"] = "连续 pkts (src->dst) 之间的最大Delta (ms*100)",
    ["rtp_in_mos"] = "RTP 伪 MOS（值 * 100）（src->dst）",
    ["rtp_in_payload_type"] = "RTP负载类型",
    ["rtp_in_pkt_drop"] = "抖动缓冲区丢弃的数据包（src->dst）",
    ["rtp_in_pkt_lost"] = "流中丢失的数据包 (src->dst)",
    ["rtp_in_r_factor"] = "RTP 伪 R_FACTOR（值 * 100）（src->dst）",
    ["rtp_in_transit"] = "RTP 传输（值 * 100）（src->dst）",
    ["rtp_last_seq"] = "最后一个流 RTP 序列号",
    ["rtp_last_ts"] = "后一个流最 RTP 时间戳",
    ["rtp_mos"] = "RTP伪MOS（值*100）（双向平均）",
    ["rtp_out_jitter"] = "RTP 抖动 (ms * 1000)",
    ["rtp_out_max_delta"] = "连续 pkts (dst->src) 之间的最大Delta (ms*100)",
    ["rtp_out_mos"] = "RTP 伪 MOS（值 * 100）（dst->src）",
    ["rtp_out_payload_type"] = "RTP负载类型",
    ["rtp_out_pkt_drop"] = "抖动缓冲区丢弃的数据包 (dst->src)",
    ["rtp_out_pkt_lost"] = "流中丢失的数据包 (dst->src)",
    ["rtp_out_r_factor"] = "RTP 伪 R_FACTOR（值 * 100）（dst->src）",
    ["rtp_out_transit"] = "RTP 传输（值 * 100）（dst->src）",
    ["rtp_r_factor"] = "RTP 伪 R_FACTOR（值 * 100）（平均两个方向）",
    ["rtp_rtt"] = "RTP 往返时间（毫秒）",
    ["rtp_sip_call_id"] = "此 RTP 流对应的 SIP CALL-ID",
    ["rtp_ssrc"] = "RTP 同步源 ID",
    ["s1ap_cause_enb_to_mme"] = "从 ENB 到 MME 的 S1AP 原因",
    ["s1ap_detailed_cause_enb_to_mme"] = "从ENB到MME的S1AP详细原因",
    ["s1ap_enb_ue_s1ap_id"] = "S1AP ENB 标识符",
    ["s1ap_mme_ue_s1ap_id"] = "S1AP MME 标识符",
    ["s1ap_msg_emm_type_enb_to_mme"] = "从 ENB 到 MME 的 S1AP EMM 消息类型",
    ["s1ap_msg_emm_type_mme_to_enb"] = "从 MME 到 ENB 的 S1AP EMM 消息类型",
    ["s1ap_msg_esm_type_enb_to_mme"] = "从 ENB 到 MME 的 S1AP ESM 消息类型",
    ["s1ap_msg_esm_type_mme_to_enb"] = "从 MME 到 ENB 的 S1AP ESM 消息类型",
    ["sampled_packet_id"] = "采样包 ID",
    ["sampled_packet_size"] = "采样包大小",
    ["sampling_algorithm"] = "采样类型（确定性/随机）",
    ["sampling_interval"] = "采样率",
    ["sampling_population"] = "采样数",
    ["sampling_size"] = "要采样的数据包数",
    ["selector_id"] = "采样器 ID",
    ["selector_name"] = "采样器名称",
    ["server_nw_latency_ms"] = "网络 RTT/2 nprobe <-> 服务器（毫秒）",
    ["shortest_flow_pkt"] = "流的最短数据包（字节）",
    ["sip_bye_ok_time"] = "BYE OK 的 SIP 时间（epoch）",
    ["sip_bye_time"] = "BYE的SIP时间（epoch）",
    ["sip_c_ip"] = "SIP C IP 地址",
    ["sip_call_id"] = "SIP 呼叫 ID",
    ["sip_call_state"] = "SIP 呼叫状态",
    ["sip_called_party"] = "SIP 被呼叫者",
    ["sip_calling_party"] = "SIP 呼叫者",
    ["sip_cancel_ok_time"] = "CANCEL OK 的 SIP 时间（epoch）",
    ["sip_cancel_time"] = "CANCEL 的 SIP 时间（epoch）",
    ["sip_invite_failure_time"] = "INVITE FAILURE的SIP时间（epoch）",
    ["sip_invite_ok_time"] = "INVITE OK 的 SIP 时间（epoch）",
    ["sip_invite_time"] = "INVITE 的 SIP 时间（epoch）",
    ["sip_reason_cause"] = "SIP Cancel/Bye/Failure 原因",
    ["sip_response_code"] = "SIP失败响应码",
    ["sip_ringing_time"] = "RINGING 的 SIP 时间（epoch）",
    ["sip_rtp_codecs"] = "SIP RTP 编解码器",
    ["sip_rtp_ipv4_dst_addr"] = "SIP RTP 流目的 IP",
    ["sip_rtp_ipv4_src_addr"] = "SIP RTP流源IP",
    ["sip_rtp_l4_dst_port"] = "SIP RTP 流目的端口",
    ["sip_rtp_l4_src_port"] = "SIP RTP 流源端口",
    ["sip_trying_time"] = "Trying的SIP时间（epoch）",
    ["smtp_mail_from"] = "邮件发件人",
    ["smtp_rcpt_to"] = "邮件收件人",
    ["src_as"] = "源BGP AS",
    ["src_as_map"] = "SRC_AS 的组织名称",
    ["src_as_path_1"] = "Src AS 路径位置 1",
    ["src_as_path_10"] = "Src AS 路径位置 10",
    ["src_as_path_2"] = "Src AS 路径位置 2",
    ["src_as_path_3"] = "Src AS 路径位置 3",
    ["src_as_path_4"] = "Src AS 路径位置 4",
    ["src_as_path_5"] = "Src AS 路径位置 5",
    ["src_as_path_6"] = "Src AS 路径位置 6",
    ["src_as_path_7"] = "Src AS 路径位置 7",
    ["src_as_path_8"] = "Src AS 路径位置 8",
    ["src_as_path_9"] = "Src AS 路径位置 9",
    ["src_father_proc_name"] = "src父进程名",
    ["src_father_proc_pid"] = "src父进程PID",
    ["src_fragments"] = "数量分片的数据包 src->dst",
    ["src_ip_city"] = "src IP所在城市 src IP所在城市",
    ["src_ip_country"] = "src IP所在国家",
    ["src_ip_lat"] = "src IP所在纬度",
    ["src_ip_long"] = "src IP所在的经度",
    ["src_proc_actual_memory"] = "src进程实际内存（字节）",
    ["src_proc_average_cpu_load"] = "src进程平均负载 (%% * 100)",
    ["src_proc_name"] = "src进程名",
    ["src_proc_num_page_faults"] = "src进程NUM页面错误",
    ["src_proc_pctg_iowait"] = "src进程iowait时间%%(%%*100)",
    ["src_proc_peak_memory"] = "src进程峰值内存（字节）",
    ["src_proc_pid"] = "src进程PID",
    ["src_proc_uid"] = "src进程UID",
    ["src_proc_user_name"] = "src进程用户名",
    ["src_to_dst_avg_throughput"] = "src 到 dst 平均吞吐量 (bps)",
    ["src_to_dst_max_throughput"] = "src 到 dst 最大吞吐量 (bps)",
    ["src_to_dst_min_throughput"] = "src 到 dst 最小吞吐量 (bps)",
    ["src_tos"] = "TOS/DSCP (src->dst)",
    ["src_vlan"] = "源VLAN（QinQ内层VLAN）",
    ["ssdp_host"] = "SSDP主机",
    ["ssdp_method"] = "SSDP方法",
    ["ssdp_server"] = "SSDP服务器",
    ["ssdp_type"] = "SSDP类型",
    ["ssdp_usn"] = "SSDP USN",
    ["suricata_app_proto"] = "Suricata 应用程序协议",
    ["suricata_flow_id"] = "Suricata 流 ID",
    ["tcpAckNum"] = "TCP 确认号",
    ["tcpSeqNum"] = "TCP 序列号",
    ["tcpUrgentPtr"] = "TCP 紧急指针",
    ["tcpWindowSize"] = "TCP 窗口大小",
    ["tcp_flags"] = "所有流 TCP 标志的累积",
    ["tcp_win_max_in"] = "最大 TCP 窗口 (src->dst)",
    ["tcp_win_max_out"] = "最大 TCP 窗口 (dst->src)",
    ["tcp_win_min_in"] = "最小 TCP 窗口 (src->dst)",
    ["tcp_win_min_out"] = "最小 TCP 窗口 (dst->src)",
    ["tcp_win_mss_in"] = "TCP 最大段大小 (src->dst)",
    ["tcp_win_mss_out"] = "TCP 最大段大小 (dst->src)",
    ["tcp_win_scale_in"] = "TCP 窗口Scale (src->dst)",
    ["tcp_win_scale_out"] = "TCP 窗口Scale (dst->src)",
    ["tls_cert_after"] = "TLS 证书之后",
    ["tls_cert_dn"] = "TLS 证书 DN",
    ["tls_cert_not_before"] = "不是以前的 TLS 证书",
    ["tls_cert_sha1"] = "TLS 证书指纹",
    ["tls_cert_sn"] = "TLS 证书 S/N",
    ["tls_cert_subject"] = "TLS 证书主题",
    ["tls_server_name"] = "TLS 服务器名称",
    ["tls_version"] = "TLS 版本",
    ["total_bytes_exp"] = "导出的总字节数",
    ["total_flows_exp"] = "导出流的总数",
    ["total_pkts_exp"] = "导出的总流量包",
    ["untunneled_ipv4_dst_addr"] = "非隧道 IPv4 目的地址",
    ["untunneled_ipv4_src_addr"] = "非隧道 IPv4 源地址",
    ["untunneled_ipv6_dst_addr"] = "非隧道 IPv6 目的地址",
    ["untunneled_ipv6_src_addr"] = "非隧道 IPv6 源地址",
    ["untunneled_l4_dst_port"] = "非隧道 IPv4 目的端口",
    ["untunneled_l4_src_port"] = "非隧道 IPv4 源端口",
    ["untunneled_protocol"] = "非隧道 IP 协议字节",
    ["upstream_session_id"] = "上游会话标识符（例如 L2TP）或 0 （如果未知）",
    ["upstream_tunnel_id"] = "上游隧道标识符（例如 GTP TEID、VXLAN VNI）或 0（如果未知）",
    ["user_name"] = "用户名",
  },
  ["flow_risk"] = {
    ["malicious_signature_detected"] = "可能是客户端恶意 JA3 签名",
    ["ndpi_binary_application_transfer"] = "可疑文件传输",
    ["ndpi_desktop_or_file_sharing_session"] = "桌面或文件共享会话",
    ["ndpi_dns_suspicious_traffic"] = "可疑的DNS流量",
    ["ndpi_http_invalid_content"] = "无效的HTTP内容",
    ["ndpi_http_numeric_ip_host"] = "HTTP 数字 IP 主机",
    ["ndpi_http_suspicious_header"] = "HTTP 可疑标头",
    ["ndpi_http_suspicious_url"] = "HTTP 可疑 URL",
    ["ndpi_http_suspicious_user_agent"] = "HTTP 可疑用户代理",
    ["ndpi_known_protocol_on_non_standard_port"] = "非标准端口上的已知应用",
    ["ndpi_malformed_packet"] = "畸形数据包",
    ["ndpi_malicious_ja3"] = "可能的恶意签名 (JA3)",
    ["ndpi_malicious_sha1_certificate"] = "恶意 SHA1 TLS 证书",
    ["ndpi_no_risk"] = "没有风险",
    ["ndpi_risky_asn"] = "有风险的AS",
    ["ndpi_risky_domain"] = "风险域名",
    ["ndpi_smb_insecure_version"] = "SMB 不安全版本",
    ["ndpi_ssh_obsolete_client_version_or_cipher"] = "过期的 SSH 客户端版本或密码",
    ["ndpi_ssh_obsolete_server_version_or_cipher"] = "过期的 SSH 服务器版本或密码",
    ["ndpi_suspicious_dga_domain"] = "可疑的 DGA 域名",
    ["ndpi_tls_cert_validity_too_long"] = "TLS证书有效期太长",
    ["ndpi_tls_certificate_expired"] = "TLS 证书已过期 [%{sni}][%{notBefore} - %{notAfter}]",
    ["ndpi_tls_certificate_mismatch"] = "TLS 证书不匹配",
    ["ndpi_tls_missing_sni"] = "缺少 TLS SNI",
    ["ndpi_tls_not_carrying_https"] = "TLS（可能）不携带 HTTPS",
    ["ndpi_tls_obsolete_version"] = "过期的 TLS 版本",
    ["ndpi_tls_selfsigned_certificate"] = "TLS 证书自签名",
    ["ndpi_tls_suspicious_esni_usage"] = "可疑的 TLS ESNI 使用",
    ["ndpi_tls_weak_cipher"] = "薄弱的 TLS 密码",
    ["ndpi_unsafe_protocol"] = "不安全协议",
    ["ndpi_url_possible_rce_injection"] = "可能的远程代码执行 (RCE)",
    ["ndpi_url_possible_sql_injection"] = "可能的 SQL 注入",
    ["ndpi_url_possible_xss"] = "可能的 XSS",
  },
  ["flows_page"] = {
    ["active_flows"] = "%{filter} 活动流",
    ["actual_throughput"] = "实际值",
    ["alert_type_severity"] = "严重性",
    ["alerted"] = "历史警告",
    ["all_alerted"] = "所有警告",
    ["all_ases"] = "所有 AS",
    ["all_categories"] = "所有类别",
    ["all_devices"] = "所有探针",
    ["all_dscp"] = "所有 DSCP",
    ["all_flows"] = "所有流",
    ["all_host_pool"] = "所有主机池",
    ["all_hosts"] = "所有主机",
    ["all_inIfIdx"] = "所有输入接口",
    ["all_ip_versions"] = "所有版本",
    ["all_l4_protocols"] = "所有协议",
    ["all_misbehaving"] = "所有异常行为",
    ["all_outIfIdx"] = "所有输出接口",
    ["all_proto"] = "所有应用",
    ["all_vlan_ids"] = "所有 VLAN",
    ["applications"] = "应用程序",
    ["blocked"] = "拦截",
    ["client_as"] = "客户端AS",
    ["device_ip"] = "探针",
    ["direction"] = "方向",
    ["dscp"] = "DSCP",
    ["flow_exporters"] = "探针",
    ["flows"] = "%{filter} 流",
    ["flows_as_client"] = "作为客户端的流",
    ["flows_as_server"] = "作为服务器的流",
    ["host"] = " [ 主机 %{host} ]",
    ["host_pool_id"] = "主机池 %{key}",
    ["hosts"] = "主机",
    ["icmp_only"] = "仅 ICMP",
    ["idle_flows_not_listed"] = "未列出空闲流。",
    ["inIfIdx"] = "输入接口",
    ["ip_version"] = "IP版本",
    ["ipv4_only"] = "仅 IPv4",
    ["ipv6_only"] = "仅 IPv6",
    ["l4_protocol"] = "协议",
    ["local_cli_remote_srv"] = "本地客户端 - 远程服务器",
    ["local_only"] = "仅本地",
    ["local_srv_remote_cli"] = "本地服务器 - 远程客户端",
    ["misbehaving"] = "行为异常",
    ["multicast"] = "组播/广播",
    ["network"] = " [网络%{network}]",
    ["non_multicast"] = "非组播/非广播",
    ["normal"] = "正常",
    ["one_way_multicast"] = "单向组播/广播",
    ["one_way_non_multicast"] = "单向非多播/非广播",
    ["outIfIdx"] = "输出接口",
    ["port"] = " [ 端口 %{port} ]",
    ["recently_active_flows"] = " %{filter} 最近活动的流量",
    ["remote_only"] = "仅限远程",
    ["server_as"] = "服务器AS",
    ["sflow_devices"] = "sFlow 设备",
    ["source_id"] = "来源 ID",
    ["tcp_only"] = "仅 TCP",
    ["tcp_state"] = "TCP 状态",
    ["tcp_state_closed"] = "关闭",
    ["tcp_state_connecting"] = "正在连接",
    ["tcp_state_established"] = "已建立连接",
    ["tcp_state_reset"] = "重置",
    ["tcp_state_unknown"] = "未知",
    ["tot_bytes"] = "总字节数：",
    ["tot_throughput"] = "总吞吐量：",
    ["total_bytes"] = "总字节数",
    ["total_bytes_rcvd"] = "接收的总字节数",
    ["total_bytes_sent"] = "发送的总字节数",
    ["total_bytes_unknown"] = "未知应用程序总字节数",
    ["udp_only"] = "仅 UDP",
    ["vlan"] = "VLAN",
  },
  ["geo_map"] = {
    ["browser_reported_home_map"] = "浏览器报告的home地图位置",
    ["configure_geo_ip"] = "配置GeoIP",
    ["geo_ip"] = "GeoIP",
    ["geo_map"] = "地理地图",
    ["geolocation_error"] = "地理位置错误",
    ["geolocation_warning"] = "警告",
    ["hosts_geomap"] = "主机地理地图",
    ["latitude"] = "纬度",
    ["longitude"] = "经度",
    ["note"] = "注意",
    ["note_active_flows"] = "在具有公共IP地址的peers之间进行活动流。",
    ["note_compiled_ntopng_with_geolocation"] = "已经编译了带有地理定位的软件并开始使用它。",
    ["note_google_maps_browser_api_key"] = "需要 Google Maps API Key,您可以从 <a href='%{url_google}'>Google Cloud Platform Console</a> 生成一个：<br>请创建一个新项目并选择 Maps JavaScript API -> Create Credentials -> API 密钥。<br>生成后,可以通过<a href='%{url_prefs}'><i class=\"fas fa-flask\"></i>首选项</a提交密钥 >.",
    ["note_html_browser_geolocation"] = "HTML <A HREF=%{url}>browser geolocation</A> 用于根据未知位置显示在地图主机上。",
    ["note_requirements_visualize_maps"] = "为了可视化地图,您必须",
    ["note_working_internet_connection"] = "有可用的 Internet 连接。",
    ["unavailable_geolocation"] = "您的浏览器不支持或禁用地理定位。",
    ["using_default_location"] = "使用默认位置,也许浏览器地理定位已关闭。",
    ["warning_accuracy"] = "地理定位精度取决于地理定位数据库,并且可能是近似的。",
  },
  ["graphs"] = {
    ["95_perc"] = "第95 Perc",
    ["active_devices"] = "活动设备",
    ["active_flows"] = "活动流",
    ["active_host_contacts"] = "活动的主机连接",
    ["active_hosts"] = "活动主机",
    ["active_http_servers"] = "活动HTTP服务器",
    ["active_monitoring"] = "活动监控",
    ["actual_memory"] = "实际内存",
    ["app_connect"] = "应用程序连接",
    ["arp_replies"] = "ARP 响应",
    ["arp_requests"] = "ARP 请求",
    ["arp_rqst_sent_rcvd_rpls"] = "ARP 发送请求/接收响应",
    ["average"] = "平均",
    ["average_traffic"] = "平均flow/s",
    ["avg"] = "平均",
    ["binary_operators_note"] = "有关 EMA、SMA 和 RSI 指标的更多信息,请查看<a href=\"%{url}\">在线文档</a>。",
    ["blocked_flows"] = "拦截的流",
    ["bytes_sent_non_unicast"] = "非单播发送的字节数",
    ["bytes_sent_unicast"] = "单播发送的字节数",
    ["cli_active_flows_anomalies"] = "异常：客户端活动流",
    ["cli_active_flows_behaviour"] = "行为：客户端活动流",
    ["cli_score"] = "作为客户端评分",
    ["cli_score_anomalies"] = "异常：客户端评分",
    ["cli_score_behaviour"] = "行为：客户端评分",
    ["contacts"] = "通信",
    ["custom"] = "自定义",
    ["discarded_probing_bytes"] = "丢弃的探针流量",
    ["discarded_probing_packets"] = "丢弃的探针包",
    ["dns_qry_rcvd_rsp_sent"] = "DNS 接收查询/发送响应",
    ["dns_qry_sent_rsp_rcvd"] = "DNS发送查询/接收响应",
    ["download_chart_data"] = "下载当前图表数据",
    ["download_records"] = "下载记录",
    ["dropped_flows"] = "丢弃的流",
    ["dscp_classes"] = "DSCP 类",
    ["dumped_flows"] = "导出的流",
    ["echo_reply_packets"] = "重复响应数据包",
    ["echo_request_packets"] = "重复请求数据包",
    ["export_flows_as_txt"] = "将流导出为 TXT 文件",
    ["flows_status_overview"] = "%{protocol} 流状态概览",
    ["flows_vs_local_hosts"] = "活动流与本地主机",
    ["flows_vs_traffic"] = "活动流与流量",
    ["get_permanent_link"] = "获取永久链路",
    ["graph_info"] = "图表信息",
    ["host_contacts_behaviour"] = "行为：通信",
    ["host_unreachable_flows"] = "主机不可达流总数",
    ["hosts_anomalies"] = "主机异常",
    ["http_stats"] = "HTTP 统计信息",
    ["iface_score_anomalies"] = "异常: 分数",
    ["iface_score_behavior"] = "行为: 人数",
    ["iface_traffic_anomalies"] = "异常: 流量 RX/TX",
    ["iface_traffic_rx_behavior"] = "行为: 流量 RX",
    ["iface_traffic_tx_behavior"] = "行为: 流量 TX",
    ["influxdb_not_responding"] = "由于 InfluxDB 没有响应,查询已中止。 查询超时可以从 <a href=\"%{url}\">%{flask_icon} Preferences</a> 配置。",
    ["last_ms"] = "最后毫秒",
    ["loc_host_anomalies"] = "本地主机异常",
    ["max"] = "最大",
    ["max_ms"] = "最大毫秒",
    ["max_rtt"] = "最大 RTT 时间",
    ["memory_vs_hosts_flows"] = "进程内存 vs 主机/流",
    ["min_rtt"] = "最小 RTT 时间",
    ["minute_interface_top_stats"] = "接口分钟最高流量统计",
    ["name_lookup"] = "名称查询",
    ["new_flows"] = "新流",
    ["note_ases_rcvd"] = "接收的流量是自治系统收到的流量。",
    ["note_ases_sent"] = "发送的流量是自治系统发送的流量。",
    ["note_ases_traffic"] = "显示的流量由自治系统发送和接收。",
    ["note_chart_zoom"] = "在图表上单击并拖动可放大,双击可缩小。",
    ["note_protocols_shown"] = "未在所选时间范围内看到的应用程序故未列在下拉列表中。",
    ["num_blacklisted_flows"] = "包含黑名单主机的流",
    ["num_calls"] = "调用次数",
    ["num_ms_rtt"] = "RTT 时间 ms",
    ["num_nfq_drops"] = "丢包",
    ["num_nfq_pct"] = "队列填充%%",
    ["packet_drops"] = "丢包",
    ["packets_vs_drops"] = "数据包VS丢包",
    ["peak_memory"] = "峰值内存",
    ["percentile"] = "第 95 个 <a href=\"%{url}\">百分位</a>",
    ["permanent_link"] = "永久链路",
    ["pkt_drops"] = "丢包",
    ["process_alerts"] = "进程警告",
    ["process_memory"] = "进程内存",
    ["query_too_much_time"] = "查询需要很长时间才能完成",
    ["query_too_slow"] = "查询花费了太多时间来完成",
    ["query_was_aborted"] = "查询已中止",
    ["raw_flows"] = "原始流",
    ["raw_proto_flows"] = "原始 %{proto} 流",
    ["red_line_unreachable"] = "红线表示主机无法访问的时间。",
    ["rem_host_anomalies"] = "远程主机异常",
    ["reset_zoom"] = "重置缩放",
    ["right_axis_note"] = "（右轴）",
    ["score"] = "分数",
    ["score_vs_hosts_flows"] = "分数 vs 主机/流",
    ["srv_active_flows_anomalies"] = "异常：服务器活动流",
    ["srv_active_flows_behaviour"] = "行为：服务器活动流",
    ["srv_score"] = "作为服务器得分",
    ["srv_score_anomalies"] = "异常：服务器得分",
    ["srv_score_behaviour"] = "行为：服务器得分",
    ["tcp_finack_packets"] = "TCP FIN+ACK 数据包",
    ["tcp_packets"] = "TCP数据包",
    ["tcp_packets_keep_alive"] = "TCP 数据包 KeepAlive",
    ["tcp_packets_lost"] = "TCP 数据包丢失",
    ["tcp_packets_ooo"] = "TCP 数据包乱序",
    ["tcp_packets_retr"] = "TCP 重传数据包",
    ["tcp_retr_ooo_lost"] = "TCP 重传乱序和丢失",
    ["tcp_rst_packets"] = "TCP RST 数据包",
    ["tcp_rx_stats"] = "TCP 接收统计",
    ["tcp_syn_packets"] = "TCP SYN 数据包",
    ["tcp_syn_vs_tcp_synack"] = "TCP SYN 与 SYN+ACK 数据包",
    ["tcp_synack_packets"] = "TCP SYN+ACK 数据包",
    ["tcp_tx_stats"] = "TCP 发送统计",
    ["the_select_timeframe_too_small_top_talkers"] = "所选的时间范围太小,无法确定最常发言的人。 请选择至少 1 分钟的帧。",
    ["top_checks"] = "按持续时间排列的Top%{subdir}脚本",
    ["top_clients"] = "Top %{protocol} 客户端",
    ["top_interfaces"] = "Top接口",
    ["top_l7_contacts"] = "Top %{protocol} L7 连接",
    ["top_ndpi_categories"] = "Top类别",
    ["top_ndpi_protocols"] = "Top应用",
    ["top_profiles"] = "Top Profiles",
    ["top_receivers"] = "Top 接收者",
    ["top_senders"] = "Top 发送者",
    ["top_servers"] = "Top %{protocol} 服务器",
    ["top_user_scripts"] = "按持续时间排列的Top %{subdir} 的脚本",
    ["tot_ms"] = "总ms",
    ["total_alerted_flows"] = "总警告流",
    ["total_unreachable_flows"] = "总端口不可达流量",
    ["traffic_rcvd"] = "流量接收",
    ["traffic_rxtx"] = "流量接收/发送",
    ["traffic_sent"] = "流量发送",
    ["traffic_vs_errors"] = "流量 vs 错误",
    ["trend"] = "趋势",
    ["udp_packets"] = "UDP数据包",
    ["udp_sent_unicast_vs_non_unicast"] = "UDP 发送单播与非单播",
    ["unreachable_host"] = "无法访问的主机",
    ["zmq_flow_coll_drops"] = "流收集丢失",
    ["zmq_flow_coll_udp_drops"] = "收集套接字丢失",
    ["zmq_msg_rcvd_vs_drops"] = "收集与丢弃的 ZMQ 消息",
    ["zmq_received_flows"] = "收集到的 ZMQ 流",
    ["metrics_prefixes"] = {
      ["broadcast_bytes"] = "广播字节",
      ["drops"] = "丢失",
      ["num_devices"] = "设备数量",
      ["num_flows"] = "流数量",
      ["num_hosts"] = "主机数",
      ["num_http_hosts"] = "HTTP主机数量",
      ["num_ms_rtt"] = "RTT毫秒",
      ["packets"] = "数据包",
    },
    ["metrics_suffixes"] = {
      ["by_as"] = "%{metric} 由 AS",
      ["egress"] = "出口",
      ["ingress"] = "入口",
      ["inner"] = "内部",
      ["rcvd"] = "接收",
      ["sent"] = "发送",
    },
  },
  ["hash_table"] = {
    ["AggregatedFlowHash"] = "聚合流",
    ["AutonomousSystemHash"] = "自治系统(AS)",
    ["CountriesHash"] = "国家",
    ["FlowHash"] = "流量",
    ["HostHash"] = "主机",
    ["MacHash"] = "MAC",
    ["VlanHash"] = "VLAN",
  },
  ["host_config"] = {
    ["active_monitor_enable"] = "当前已禁用活动监控。 它可以在<a href=\"%{url}\">处</a>启用。",
    ["drop_all_host_traffic"] = "丢弃所有主机流量",
    ["hide_from_top"] = "隐藏Top talker",
    ["hide_host_from_top_descr"] = "不要将主机 %{host} 计入Top talker计算中",
    ["host_alias"] = "主机别名",
    ["host_pool"] = "主机池",
    ["host_traffic_policy"] = "主机流量策略",
    ["modify_host_pool_policy_btn"] = "修改主机池策略",
    ["trigger_alerts_for_host"] = "为主机 %{host} 触发警告",
    ["trigger_host_alerts"] = "触发主机警告",
    ["unblock_host_traffic"] = "取消拦截主机流量",
  },
  ["host_details"] = {
    ["active_alerted_flows"] = "活动警告流",
    ["behavioural_anomalies"] = "行为计数器异常",
    ["blocked_traffic"] = "拦截的流量",
    ["client_score"] = "客户",
    ["disabled_alert_type_help"] = "关闭基于流量状态的流量警告生成",
    ["host"] = "主机",
    ["host_cannot_be_found_message"] = "找不到主机 %{host}。",
    ["host_parameter_missing_message"] = "缺少主机参数（内部错误？）",
    ["host_restore_in_progress"] = "正在恢复主机 %{host}...",
    ["host_traffic_blocked_quota"] = "某些主机流量由于超出配额被拦截",
    ["host_traffic_blocked_quota_and_shaper"] = "某些主机流量由于超出配额被拦截和Shaper拦截",
    ["host_traffic_blocked_shaper"] = "某些主机流量被拦截Shaper拦截",
    ["inactive_host_x"] = "%{host} [不活动]",
    ["list_of_general_purpose"] = "通用设备：%{list}",
    ["list_of_special_purpose"] = "专用设备：%{list}",
    ["multiple_selection"] = "可以通过按 CTRL 键选择多个元素",
    ["reset_blacklisted_stats"] = "重置列入黑名单的主机统计信息",
    ["reset_blacklisted_stats_confirm"] = "确认要重置 \"%{host}\" 列入黑名单的统计信息吗？",
    ["reset_blacklisted_stats_note"] = "<b>注意</b>：这些信息在主机地图\"黑名单主机流\"中可见",
    ["reset_host_stats"] = "重置主机统计",
    ["reset_host_stats_confirm"] = "确认要重置 \"%{host}\" 运行时统计信息吗？",
    ["reset_host_stats_note"] = "<b>注意</b>：此主机的历史图表在重置后可能会有一个缺口。",
    ["reset_stats_in_progress"] = "主机统计将在 5 秒内重置",
    ["restore_from_cache_message_v1"] = "主机 %{host} 处于不活动状态。 点击<a href=%{js_code}>这里</a>从缓存中恢复它。",
    ["server_score"] = "服务器",
    ["snmp"] = "SNMP",
    ["status_ignore"] = "禁用流状态警告",
    ["trace_debug_host_info"] = "主机：%{hostinfo},VLAN：%{vlan}",
    ["trace_debug_host_ip"] = "主机：%{hostip},VLAN：%{vlan}",
    ["trace_debug_restored_host_info"] = "恢复的主机信息",
    ["unknown_device_type"] = "未知设备类型",
  },
  ["host_pools"] = {
    ["active_pool_members"] = "活动池成员",
    ["add_host_pool_member"] = "添加主机池成员",
    ["alias_or_manufacturer"] = "别名/制造商",
    ["and_associated_members"] = "其历史数据和任何相关成员",
    ["cannot_delete_cp"] = "如果有任何捕获用户关联,则无法删除池",
    ["change_member_pool"] = "更改池成员",
    ["change_pool"] = "切换池",
    ["children_safe"] = "儿童安全",
    ["cidr"] = "CIDR",
    ["click_here_host_pool"] = "单击此处转到主机池页面",
    ["config_export"] = "导出配置",
    ["config_import"] = "导入配置",
    ["config_import_message"] = "注意：具有相同名称的主机池将被替换。",
    ["confirm_delete_pool"] = "确认要删除主机池吗",
    ["confirm_empty_pool"] = "确认要从主机池中删除所有成员吗",
    ["confirm_remove_member"] = "确认要移除成员吗",
    ["confirm_reset_pool_quotas"] = "确认要重置 \"%{pool}\" 池计数器吗？",
    ["confirm_reset_quotas"] = "确认要重置所有配额计数器吗？",
    ["create_new_quotas_here"] = "您可以在<a href=\"%{url}\">处</a>创建新的配额。",
    ["create_pool_hint"] = "您可以从\"管理池\"选项卡创建新池。",
    ["create_pools"] = "管理池",
    ["delete_pool"] = "删除池",
    ["duplicate_member"] = "重复的成员地址",
    ["duplicate_pool"] = "重复的池名称",
    ["edit_host_pool_member"] = "编辑成员",
    ["edit_host_pools"] = "编辑主机池",
    ["empty_pool"] = "空池",
    ["enforce_quotas_per_pool_member"] = "每个成员配额",
    ["enforce_shapers_per_pool_member"] = "每个成员Shaper",
    ["from_pool"] = "从主机池",
    ["host_members"] = "主机池成员",
    ["host_pools"] = "主机池",
    ["import_hosts"] = "导入主机池成员",
    ["import_hosts_desc"] = "上传包含要填写的主机池成员的 CSV 文件。",
    ["import_hosts_title"] = "导入主机池成员：%{pool}",
    ["invalid_member"] = "无效的成员地址格式",
    ["ip_address"] = "IP地址",
    ["ip_filter"] = "IPv4/IPv6",
    ["ipv4"] = "IPv4",
    ["ipv6"] = "IPv6",
    ["mac_address"] = "MAC地址",
    ["mac_filter"] = "MAC地址",
    ["manage_pools"] = "管理池成员身份",
    ["manufacturer_filter"] = "制造商：%{manufacturer}",
    ["max_members_message"] = "使用此版本的 ntopng,您只能为每个主机池分配 %{maxnum} 成员。 升级到企业版以消除此限制。",
    ["max_pools_message"] = "使用此版本的 ntopng,您只能创建 %{maxnum} 个主机池。 升级到企业版以消除此限制。",
    ["member_address"] = "成员地址",
    ["member_exists"] = "成员 \"%{member_name}\" 未添加。 它已分配给池 \"%{member_pool}\"。",
    ["member_filter"] = "成员：%{member}",
    ["member_type"] = "成员类型",
    ["members_limit_reached"] = "已达到成员限制",
    ["network"] = "网络",
    ["network_normalized"] = "网络 \"%{network}\" 具有非零主机标识符。 使用 \"%{network_normalized}\"。",
    ["new_host_pool"] = "新主机池",
    ["new_pool"] = "新池",
    ["no_hosts_defined"] = "未定义主机池。",
    ["no_pools_defined"] = "未定义主机池。",
    ["per_member_quotas"] = "设置每个成员配额选项后,每个主机都有单独的配额计数。 未设置时,所有主机流量将作为一个整体计入配额限制",
    ["per_member_shapers"] = "设置每个成员的shapers选项后,每个主机都将拥有自己的 shapers。 未设置时,属于同一池的所有主机将共享相同的Shaper",
    ["policy"] = "当前策略",
    ["pool"] = "池名",
    ["pool_name"] = "池名",
    ["pool_name_pattern_error"] = "请仅输入小写字符、数字或下划线",
    ["remove_member"] = "移除成员",
    ["remove_member_pool"] = "您确定要从池 '%{pool}' 中删除成员：'%{member}' 吗？",
    ["reset_quotas"] = "重置配额",
    ["search_member"] = "搜索成员",
    ["select_host_pool"] = "当前<b>主机池</b>",
    ["select_member_type"] = "选择成员类型",
    ["select_new_pool"] = "从下面的下拉列表中选择一个新池以将成员 \"%{member}\" 分配给：",
    ["select_policy"] = "选择新策略",
    ["specify_member_address"] = "请指定 IPv4/IPv6 地址或网络或 MAC 地址",
    ["specify_pool_name"] = "指定池名称",
    ["type_valid"] = "输入有效的 %input",
    ["view"] = "查看",
    ["validation"] = {
      ["abort_add_member"] = "中止添加新的主机池成员？",
      ["abort_add_pool"] = "中止添加新池？",
      ["abort_edit_pool"] = "中止编辑池？",
    },
  },
  ["hosts_comparison"] = {
    ["hosts_parameter_missing_message"] = "缺少主机参数（内部错误？）",
  },
  ["hosts_map_page"] = {
    ["active_alert_flows"] = "活动警告流",
    ["alerted_flows"] = "总警告流量",
    ["all_flows"] = "所有的流",
    ["blacklisted_flows_hosts"] = "包含黑名单主机的流",
    ["dns_bytes"] = "DNS流量",
    ["dns_queries"] = "DNS查询与响应",
    ["ntp_packets"] = "NTP流量",
    ["score"] = "主机得分",
    ["syn_distribution"] = "SYN分布",
    ["syn_vs_rst"] = "SYN vs RST",
    ["syn_vs_synack"] = "SYN vs SYNACK",
    ["tcp_bytes_sent_vs_rcvd"] = "S发送/接收的 TCP 字节",
    ["tcp_pkts_sent_vs_rcvd"] = "发送/接收的 TCP 数据包",
    ["traffic_ratio"] = "流量比率",
    ["unreach_flows"] = "不可达流",
    ["labels"] = {
      ["af_c"] = "作为客户端的警告流",
      ["af_s"] = "作为服务器的警告流",
      ["b_ratio"] = "字节比率 [-1 = 下载,0 = 混合,1 = 上传]",
      ["blacklisted_as_client"] = "包含黑名单主机的客户端流数量",
      ["blacklisted_as_server"] = "包含黑名单主机的服务器流数量",
      ["client_score"] = "作为攻击者得分",
      ["dns_p_r"] = "收到肯定的DNS响应",
      ["dns_r"] = "收到的DNS字节数",
      ["dns_s"] = "发送的DNS查询",
      ["dns_sent"] = "发送的DNS字节数",
      ["f_a_c"] = "作为客户端的警告流",
      ["f_a_s"] = "作为服务器的警告流",
      ["f_c"] = "作为客户端的流",
      ["f_s"] = "作为服务器的流",
      ["ntp_r"] = "收到的 NTP 数据包",
      ["ntp_s"] = "发送的 NTP 数据包",
      ["p_ratio"] = "数据包比率 [-1 = 下载,0 = 混合,1 = 上传]",
      ["rst_r"] = "收到的 RST 数量",
      ["sa_r"] = "收到的 SYN/ACK 数量",
      ["server_score"] = "作为被攻击者得分",
      ["syn_r"] = "收到的 SYN 数量",
      ["syn_s"] = "发送的 SYN 数量",
      ["tcp_b_r"] = "接收到的 TCP 字节数",
      ["tcp_b_s"] = "发送的 TCP 字节数",
      ["tcp_p_r"] = "收到的 TCP 数据包",
      ["tcp_p_s"] = "发送的 TCP 数据包",
      ["uf_c"] = "作为客户端无法访问的流",
      ["uf_s"] = "作为服务器无法访问的流",
    },
  },
  ["hosts_stats"] = {
    ["all"] = "All",
    ["all_hosts"] = "所有主机",
    ["as_geolocation"] = "AS 地理定位",
    ["as_info"] = "AS信息",
    ["as_path"] = "AS路径",
    ["as_prefixes"] = "AS 前缀",
    ["asn_title"] = "对于 AS %{asn}",
    ["bgp_updates"] = "BGP 更新",
    ["blacklisted"] = "列入黑名单",
    ["blacklisted_hosts_only"] = "仅列入黑名单的主机",
    ["blocking_traffic_policy_popup_msg"] = "主机流量受拦截流量策略的影响",
    ["broadcast_domain_hosts_only"] = "仅广播域主机",
    ["country_title"] = "国家 %{country}",
    ["filter_hosts"] = "过滤主机",
    ["filtered"] = "过滤",
    ["filtered_hosts_only"] = "流量被拦截的主机",
    ["host_pool"] = "主机池 %{pool_name}",
    ["hosts"] = "主机",
    ["hosts_list"] = "主机列表",
    ["hosts_page_title"] = "%{all} %{traffic_type} %{local_remote} %{protocol} %{network_cidr} %{network} %{ip_version} %{os} 主机%{country_asn_or_mac} %{vlan}",
    ["idle_hosts_not_listed"] = "未列出空闲主机。",
    ["in_network"] = "%{network}",
    ["ipver_title"] = "IPv%{version_num}",
    ["label_blacklisted_host"] = "列入黑名单",
    ["label_broadcast_domain_host"] = "广播域",
    ["label_local_host"] = "本地",
    ["label_remote_host"] = "远程",
    ["local"] = "本地",
    ["local_hosts_only"] = "仅本地主机",
    ["location"] = "位置",
    ["mac_title"] = " MAC %{mac}",
    ["more_info_about_as_popup_msg"] = "有关 AS 的更多信息",
    ["pool_title"] = "对于池 %{poolname}",
    ["remote"] = "远程",
    ["remote_hosts_only"] = "仅远程主机",
    ["source_id"] = "源 ID",
    ["traffic_type_all"] = "所有主机",
    ["traffic_type_one_way"] = "单向流量",
    ["traffic_type_two_ways"] = "双向流量",
    ["user"] = "用户 %{pool_name}",
    ["user_title"] = "对于用户 %{poolname}",
    ["vlan_title"] = "VLAN %{vlan}",
  },
  ["http_page"] = {
    ["distribution"] = "分布",
    ["http_queries"] = "发送的HTTP查询",
    ["http_responses"] = "收到的HTTP响应",
    ["method"] = "方法",
    ["other_method"] = "其他方法",
    ["requests"] = "请求",
    ["requests_served"] = "服务的请求",
    ["response_code"] = "响应码",
    ["response_code_1xx"] = "1xx（信息）",
    ["response_code_2xx"] = "2xx（成功）",
    ["response_code_3xx"] = "3xx（重定向）",
    ["response_code_4xx"] = "4xx（客户端错误）",
    ["response_code_5xx"] = "5xx（服务器错误）",
    ["responses"] = "响应",
    ["traffic_received"] = "收到的流量",
    ["traffic_sent"] = "流量发送",
    ["virtual_hosts"] = "虚拟主机",
  },
  ["http_servers_stats"] = {
    ["actual_requests"] = "实际请求",
    ["bytes_received"] = "接收的字节数",
    ["bytes_sent"] = "发送的字节数",
    ["http_server_ip"] = "HTTP 服务器 IP",
    ["http_servers"] = "服务器 HTTP",
    ["http_virtual_host"] = "HTTP 虚拟主机",
    ["local_http_servers"] = "本地HTTP服务器",
    ["total_requests"] = "请求总数",
  },
  ["http_status_code"] = {
    ["bad_request"] = "指定的 URL 包含无效/危险字符",
    ["enterprise_only"] = "需要企业版许可证",
    ["forbidden"] = "禁止页面",
    ["internal_error"] = "内部服务器错误",
    ["not_found"] = "找不到请求的页面",
    ["pro_only"] = "需要专业版/企业版许可证",
    ["shut_start"] = "此时无法处理请求,可能正在启动或关闭",
  },
  ["icmp_page"] = {
    ["icmp_code"] = "代码",
    ["icmp_message"] = "ICMP 消息",
    ["icmp_port_unreachable_extra"] = "%{unreach_protocol} 端口 %{unreach_port} 在 %{unreach_host} 上无法访问",
    ["icmp_type"] = "类型",
    ["last_rcvd_peer"] = "最后的接收peer",
    ["last_sent_peer"] = "最后的发送peer",
    ["packets_received"] = "收到的数据包",
    ["packets_sent"] = "发送的数据包",
  },
  ["if_stats_config"] = {
    ["companion_interface"] = "配套接口",
    ["custom_name"] = "自定义名称",
    ["custom_name_popup_msg"] = "为接口指定别名",
    ["discard_probing_traffic"] = "丢弃的探针流量",
    ["dump_flows_to_database"] = "转储流到数据库",
    ["gw_macs"] = "基于 MAC 地址的流量方向",
    ["gw_macs_description"] = "这用于根据提供的 MAC 地址（逗号分隔列表）计算流量方向（入口或出口）,因为在某些情况下（从流量镜像或 pcap 捕获流量时）不可能知道流量方向。 流向到配置的 MAC 地址的流量被视为出口流量。",
    ["gw_macs_example"] = "例如 %{example}",
    ["hide_from_top_networks"] = "从Top网络中隐藏",
    ["hide_from_top_networks_descr"] = "例如 %{example}",
    ["interface_network_discovery"] = "定期进行接口网络发现",
    ["interface_speed"] = "接口速度",
    ["interface_speed_measure_unit"] = "(Mbit/s)",
    ["interface_speed_popup_msg"] = "指定最大接口速度",
    ["interface_top_talkers_creation"] = "创建接口Top Talkers",
    ["is_mirrored_traffic"] = "镜像流量",
    ["referesh_rate_measure_unit"] = "(sec)",
    ["scaling_factor"] = "入口数据包采样率",
    ["scaling_factor_popup_msg"] = "这应该与您的捕获接口采样率相匹配",
    ["show_dyn_iface_traffic"] = "复制分解的流量",
    ["show_dyn_iface_traffic_note"] = "启用此选项,定向到动态或自定义子接口的流量仍会显示在主接口中",
    ["trigger_interface_alerts"] = "触发接口警告",
  },
  ["if_stats_networks"] = {
    ["no_broadcast_domains"] = "没有通过检查ARP流量推断出的广播域。",
    ["no_ip_addresses_read"] = "没有从配置中读取 IP 地址。",
    ["note_ghost_bcast_domains"] = "%{ghost_icon} 突出显示Ghost广播域（即不属于任何接口 IP 地址网络）。",
    ["note_iface_addresses"] = "从接口配置中读取 IP 地址。",
    ["note_iface_bcast_domains"] = "通过检查 ARP 流量来推断广播域。",
  },
  ["if_stats_overview"] = {
    ["active"] = "活动",
    ["alert_events"] = "警告事件",
    ["all_counters"] = "所有计数器",
    ["blocked_flows"] = "拦截的流",
    ["bridged_traffic"] = "桥接流量",
    ["buffer_full"] = "缓冲区已满",
    ["collected_flows"] = "收集的流量",
    ["collected_logs"] = "收集的日志",
    ["conntrack_flow_entries"] = "Conntrack 流条目",
    ["counter_anomalies"] = "计数器异常",
    ["direct_mode"] = "直接模式",
    ["discarded_probing_traffic"] = "丢弃的探针流量",
    ["dispatched_logs"] = "已调度事件",
    ["dropped_alerts_info"] = "由于警报队列已满而无法写入磁盘的未参与警报数。",
    ["dropped_flows"] = "丢弃的流",
    ["dropped_packets"] = "丢包",
    ["drops_only"] = "仅丢弃",
    ["dumped_files"] = "转储文件",
    ["dumped_packets"] = "转储的数据包",
    ["egress_packets"] = "出口数据包",
    ["expired_maintenance"] = "过期的",
    ["export_disabled"] = "在运行时首选项中禁用流导出",
    ["exported_flows"] = "导出的流量",
    ["family"] = "家族",
    ["flows_export_statistics"] = "流量出口统计",
    ["handled_logs"] = "检测到的事件",
    ["has_traffic_directions"] = " RX /TX方向",
    ["host_correlations"] = "主机关联",
    ["id"] = "Id",
    ["iface_switch_active_msg"] = "所选接口 <b>%{interface}</b> [ifid: %{ifid}] 现在处于活动状态",
    ["iface_switch_empty_session_msg"] = "空会话",
    ["iface_switch_error_msg"] = "切换接口时出错",
    ["in_path_interface"] = "In-Path 接口（线路凹凸）",
    ["ingress_packets"] = "入口数据包",
    ["ingress_traffic"] = "入口流量",
    ["interface_ip"] = "接口IP",
    ["interface_name"] = "接口",
    ["interface_rx_updates"] = "接口更新",
    ["malformed_logs"] = "格式错误的日志",
    ["nf"] = "网络过滤器",
    ["nf_enobufs"] = "无缓冲空间",
    ["nf_handle_packet_failed"] = "排队数据包处理失败",
    ["nf_queue_dropped"] = "丢弃的数据包（队列已满）",
    ["nf_queue_pct"] = "队列填充水平",
    ["nf_queue_total"] = "排队的数据包",
    ["nf_queue_user_dropped"] = "丢弃的数据包（无法发送到用户空间）",
    ["no_license"] = "空",
    ["note"] = "注意",
    ["note_drop_ifstats_dynamic"] = "丢弃在主接口可用",
    ["note_drops_sflow"] = "丢弃仅针对 sFlow 显示,因为 NetFlow 不会报告它们。<br><A HREF=https://www.ietf.org/rfc/rfc3176.txt>sFlow</A> drops 指的是sFlow代理检测到标记为要采样的数据包由于缺乏资源而被放弃的次数。丢弃计数器报告自代理上次重置以来检测到的丢弃总数。一个高的丢弃率表明管理代理无法以硬件产生的速度来处理样本。增加sFlow采样率将减少丢弃率。注意：一个不能检测到丢包的代理将总是报告为零。",
    ["note_packets"] = "在以太网网络中,每个数据包都有一个 <A HREF=\"https://en.wikipedia.org/wiki/Ethernet_frame\">24 字节的开销</A> [前导（7 字节）,帧开始（1 字节）、CRC（4 个字节）和 <A HREF=\"http://en.wikipedia.org/wiki/Interframe_gap\">IFG</A>（12 个字节）]。 此类开销需要计入接口流量,但不会添加到 IP 地址之间交换的流量中。 这是因为此类数据会增加接口负载,但无法计入主机交换的流量中,因此预计主机和接口流量值之间的差异很小。",
    ["note_probe_zmq_drops_export_queue_full"] = "远程探针已成功处理这些流,但由于导出队列已满而无法将它们排入队列以进行导出。",
    ["note_probe_zmq_drops_flow_collection_drops"] = "nProbe 丢弃的收集流数据包 (NetFlow v9) 或丢弃的流 (NetFlow v5/IPFIX)",
    ["note_probe_zmq_drops_flow_collection_udp_socket_drops"] = "在 nProbe 收集套接字上丢弃的流数据包",
    ["note_probe_zmq_timeout_lifetime"] = "nProbe 公布的流生命周期持续时间和根据收集的流计算的实际生命周期。",
    ["note_remote_probe_time"] = "ntopng 时间和远程 nProbe 时间之间的时差：理想情况下应该接近于零。 如果太大请同步nProbe/ntopng时间",
    ["paused"] = "暂停",
    ["probe_ip"] = "探针IP",
    ["probe_public_ip"] = "探针公共IP",
    ["probe_throughput"] = "nProbe 吞吐量",
    ["probe_timeout_idle"] = "空闲超时",
    ["probe_timeout_lifetime"] = "生存期超时",
    ["probe_zmq_drops_export_queue_full"] = "队列已满丢弃的流",
    ["probe_zmq_drops_flow_collection_drops"] = "流收集丢包",
    ["probe_zmq_drops_flow_collection_udp_socket_drops"] = "收集套接字丢包",
    ["probe_zmq_num_endpoints"] = "活动的探针 ZMQ 端点",
    ["probe_zmq_num_flow_exports"] = "导出的流量",
    ["public_probe_ip"] = "公共探针 IP",
    ["received_traffic"] = "收到的流量",
    ["remote_flow_lifetime"] = "流生存期",
    ["remote_probe"] = "nProbe",
    ["remote_probe_collecting_from_x_devices"] = "正在收集来自 <a href=\"%{url}\">%{num} nProbe的流</a>.",
    ["remote_probe_collector_mode"] = "收集模式",
    ["remote_probe_edition"] = "编辑",
    ["remote_probe_license"] = "License",
    ["remote_probe_maintenance"] = "维护",
    ["remote_probe_showing_first_x_devices"] = "显示前 %{num}个探针.",
    ["remote_probe_time"] = "nProbe 时间漂移",
    ["reset_counters"] = "重置计数器",
    ["rrd_timeseries"] = "RRD时间序列",
    ["send_error"] = "发送错误",
    ["sflow_counter_updates"] = "计数器更新",
    ["shaped_filtered_packets"] = "整形（Shaped）/过滤数据包",
    ["snmp"] = "SNMP",
    ["state"] = "状态",
    ["syslog_statistics"] = "系统日志收集统计",
    ["traffic_breakdown"] = "流量细分",
    ["traffic_statistics"] = "流量统计",
    ["unhandled_flows"] = "未处理的流",
    ["unhandled_logs"] = "不支持的事件",
    ["zmq_avg_msg_flows"] = "平均 Flows/Msg",
    ["zmq_encryption_alias"] = "******************",
    ["zmq_encryption_public_key"] = "ZMQ 加密公钥",
    ["zmq_encryption_public_key_note"] = "如果您使用 nProbe 作为探针,请将 --zmq-encryption-key '%{key}' 添加到 nProbe 配置中。",
    ["zmq_message_drops"] = "丢弃的 ZMQ 消息",
    ["zmq_message_rcvd"] = "收集到的 ZMQ 消息",
    ["zmq_rx_statistics"] = "ZMQ统计",
  },
  ["index_page"] = {
    ["applications"] = "应用程序",
    ["asns"] = "ASNs",
    ["dashboard"] = "仪表盘",
    ["hosts"] = "主机",
    ["live_update"] = "实时更新",
    ["minute"] = "分钟",
    ["never"] = "从不",
    ["no_packet_warning"] = "尚未在接口 %{ifname} 上接收到数据包。<p>请等待 %{countdown} 秒,直到此页面重新加载。",
    ["refresh"] = "刷新",
    ["refresh_frequency"] = "刷新频率",
    ["refresh_graph_popup_msg"] = "刷新图表",
    ["seconds"] = "秒",
    ["senders"] = "发送方",
    ["top_flow_talkers"] = "Top流量talker",
  },
  ["infrastructure_dashboard"] = {
    ["actions"] = "操作",
    ["add_instance"] = "添加 %{product} 实例",
    ["chart"] = "图表",
    ["edit_instance"] = "编辑 %{product} 实例",
    ["fetching_data"] = "获取数据",
    ["information"] = "信息",
    ["infrastructure"] = "基础设施",
    ["infrastructure_dashboard"] = "基础设施",
    ["name"] = "名称",
    ["not_polled_yet"] = "尚未轮询",
    ["ntopng_instance"] = "%{product} 实例",
    ["remove_instance"] = "确认要删除 '%{instance}' 实例吗？",
    ["status"] = "状态",
    ["url"] = "URL",
    ["collected_stats"] = {
      ["alerts"] = "警告",
      ["download"] = "下载",
      ["engaged_alerts"] = "参与警告",
      ["flow_alerts"] = "流警告",
      ["flows"] = "流",
      ["hosts"] = "主机",
      ["last_update"] = "最后更新",
      ["throughput"] = "吞吐量",
      ["upload"] = "上传",
    },
    ["modal"] = {
      ["bandwidth_threshold"] = "带宽阈值",
      ["failed_login"] = "认证失败,请检查用户认证Token是否已更改！",
      ["hint_token"] = "可以从用户配置文件页面生成Token。",
      ["insert_instance"] = "%{product}-实例",
      ["instance_alias"] = "实例别名",
      ["instance_url"] = "URL",
      ["password"] = "密码",
      ["rtt_threshold"] = "RTT阈值",
      ["seconds"] = "秒",
      ["successfull_login"] = "远程实例响应成功！",
      ["test_auth"] = "测试",
      ["testing_authentication"] = "测试认证",
      ["threshold_hint"] = "触发警告的最低阈值",
      ["token"] = "用户认证Token",
      ["unknown_host"] = "认证失败,请检查网址是否正确！",
      ["url_hint"] = "远程实例 URL",
      ["username"] = "用户名",
    },
    ["validation"] = {
      ["abort_add"] = "中止添加新实例？",
      ["abort_edit"] = "中止编辑当前实例？",
    },
  },
  ["input_item_list"] = {
    ["dns_input_list_description"] = "允许的DNSIP 的逗号分隔值。 示例：192.168.1.1",
    ["dns_input_list_title"] = "允许的 DNS",
    ["ntp_input_list_description"] = "允许的 NTP IP 的逗号分隔值。 示例：192.168.1.1",
    ["ntp_input_list_title"] = "允许的 NTP",
    ["smtp_input_list_description"] = "允许的 SMTP IP 的逗号分隔值。 示例：192.168.1.1",
    ["smtp_input_list_title"] = "允许的 SMTP",
  },
  ["internals"] = {
    ["alert_drops"] = "警告丢失",
    ["alert_drops_descr"] = "警告丢失时触发警告。 这可能是由于生成的警告过多或收件人速度慢。",
    ["all_hash_tables"] = "所有哈希表",
    ["all_periodic_activities"] = "所有定期活动",
    ["all_user_script_targets"] = "所有用户脚本目标",
    ["any_periodic_activity_issue"] = "所有定期活动",
    ["chart_script_duration"] = "%{script} 持续时间",
    ["chart_script_rrds"] = "%{script} TS 统计",
    ["check"] = "用户脚本",
    ["check_target"] = "目标",
    ["checks"] = "检查",
    ["degraded_performance"] = "性能下降",
    ["duration"] = "持续时间",
    ["flow_calls_stats"] = "flow.lua 调用统计",
    ["flow_lua_duration"] = "flow.lua 持续时间",
    ["hash_entries"] = "哈希条目",
    ["hash_table"] = "哈希表",
    ["hash_table_utilization"] = "哈希表利用率",
    ["hash_tables"] = "哈希表",
    ["high_idle_entries"] = "大量空闲条目",
    ["iface_hash_tables"] = "%{iface} 哈希表",
    ["iface_periodic_scripts"] = "%{iface} 定期活动",
    ["in_progress_since"] = "正在进行中自从",
    ["last_duration_ms"] = "最后时长",
    ["last_num_calls"] = "最后调用次数",
    ["last_start_time"] = "最后开始",
    ["last_start_time_ago"] = "%{time}以前",
    ["lua"] = "Lua",
    ["max_duration_ms"] = "最长持续时间",
    ["max_duration_secs"] = "最长持续时间",
    ["missed_idle"] = "未命中空闲",
    ["missed_periodic_update"] = "错过定期更新",
    ["missed_proto_detected"] = "检测到丢失的Proto",
    ["num_calls"] = "%{script} 调用次数",
    ["num_calls_vs_duration"] = "Lua 调用与持续时间",
    ["num_drops"] = "Points丢失",
    ["num_failed_enqueues"] = "入队失败",
    ["num_missed_calls"] = "丢失的调用",
    ["num_writes"] = "Points 写",
    ["pending_periodic_update"] = "待定定期更新",
    ["pending_proto_detected"] = "待定Proto检测",
    ["periodic_activities"] = "定期活动",
    ["periodic_activities_descr"] = "将鼠标悬停在任何列出的活动上以读其说明。",
    ["periodic_activities_last_start_time_descr"] = "上次开始表示上次开始定期活动的时间。",
    ["periodic_activities_max_duration_secs_descr"] = "最大持续时间表示允许脚本执行多长时间。",
    ["periodic_activities_not_shown"] = "仅显示至少已安排一次的定期活动。",
    ["periodic_activities_periodicity_descr"] = "频率表示定期活动被安排执行的频率。",
    ["periodic_activities_rrd_descr"] = "TS 代表时间序列。",
    ["periodic_activities_tot_not_executed_descr"] = "未执行-统计周期性活动未计划执行的次数，原因可能是该活动已在运行（运行缓慢）或已计划（但没有线程可执行）。",
    ["periodic_activities_tot_running_slow_descr"] = "运行缓慢-统计周期性活动花费很长时间才能完成的次数。",
    ["periodic_activity"] = "定期活动",
    ["periodic_activity_issues"] = "问题",
    ["periodicity"] = "定期",
    ["queue"] = "队列",
    ["queued"] = "排队",
    ["queues"] = "队列",
    ["rrd_drops"] = "TS丢包",
    ["script_deadline_exceeded"] = "运行缓慢",
    ["script_deadline_exceeded_descr"] = "脚本花费了很长时间才能完成",
    ["script_duration"] = "%{script} 持续时间",
    ["script_not_executed"] = "未执行",
    ["script_not_executed_descr"] = "脚本已被调度但尚未执行",
    ["script_num_calls"] = "%{script} 调用次数",
    ["script_stats"] = "%{script} 与总数",
    ["sleeping"] = "睡眠",
    ["state_active"] = "处于活动状态的条目",
    ["state_idle"] = "处于空闲状态的条目",
    ["state_idle_vs_active"] = "空闲条目与活动条目",
    ["status_description"] = "周期性活动可以处于以下状态之一：",
    ["status_queued_descr"] = "活动已安排,但尚未运行",
    ["status_running_descr"] = "该活动目前正在执行中",
    ["status_sleeping_descr"] = "活动已完成,等待下次安排",
    ["successful_calls"] = "成功通话",
    ["system_alert_drops_descr"] = "由于内部警告队列中没有空间,因此在警告丢失时触发警告。 这可能是由于生成的警告过多。",
    ["system_iface_periodic_scripts"] = "系统定期活动",
    ["time_exceeded"] = "超时",
    ["time_usage"] = "时间利用率",
    ["timeseries_queue_length"] = "TS 队列长度",
    ["timeseries_queued_points"] = "排队Point",
    ["timeseries_writes"] = "TS 写入",
    ["tot_not_executed"] = "未执行",
    ["tot_running_slow"] = "运行缓慢",
    ["total_duration"] = "总 %{subdir} 脚本持续时间",
    ["total_flow_duration"] = "总时长",
    ["total_num_calls"] = "总 %{subdir} 脚本调用次数",
    ["total_stats"] = "总 %{subdir} 脚本统计",
    ["user_script"] = "用户脚本",
    ["user_script_target"] = "目标",
    ["user_scripts"] = "用户脚本",
    ["work_completion"] = "完成",
  },
  ["invalid_filters"] = {
    ["double_arg"] = "给了两个相同的参数 \"%{args}\"",
    ["empty"] = "空过滤器列表",
    ["few_args"] = "不正确的参数,需要一个过滤器及其值,但只给出了一个 \"%{args}\"",
    ["incorrect_args"] = "参数不正确,其中一个参数不受支持或不正确\"%{args}\"",
    ["invalid_subdir"] = "给出的子目录无效",
  },
  ["ites_page"] = {
    ["contacts"] = "连接",
    ["current_sites"] = "当前站点",
    ["last_5_minutes_sites"] = "最后5分钟站点",
    ["sites"] = "站点",
    ["time_range"] = "时间范围：",
    ["top_sites_not_enabled_message"] = "为了跟踪访问过的站点,从 <a href=\"%{url}\"><i class=\"fas fa-flask 为本地主机启用创建访问最多的网站\"></i> 首选项</a>。",
    ["top_sites_not_seen"] = "当前本地主机尚未访问任何网站。",
    ["top_visited_sites"] = "最常访问的站点",
  },
  ["license_page"] = {
    ["agreement"] = "有效License",
    ["license"] = "License",
    ["not_valid"] = "无效",
    ["status"] = "状态",
    ["valid"] = "有效的",
  },
  ["live_capture"] = {
    ["active_live_captures"] = "实时捕获",
    ["note"] = "捕获主机 %{hostinfo} 的实时流量。 流将在 1 分钟或 10 万个数据包后自动停止。",
  },
  ["live_traffic"] = {
    ["error_generic"] = "一般错误",
    ["error_not_found"] = "未找到",
    ["error_not_granted"] = "请求未获批准",
    ["modal_vlan_tagged_with_bpf_confirmation"] = "此主机带有 VLAN 标记。 提交的 BPF 过滤器要正确匹配数据包,必须包含 VLAN 51。可以在过滤器中将 VLAN 指定为 <br> <pre>vlan %{vlan} 和 <i>...rest of the filter...< /i></pre>",
    ["modal_vlan_tagged_with_bpf_continue"] = "继续",
    ["modal_vlan_tagged_with_bpf_title"] = "主机 pcap 下载",
  },
  ["local_flow_matrix"] = {
    ["error_no_local_hosts"] = "找不到本地主机",
    ["local_flow_matrix"] = "局部流矩阵",
    ["local_hosts_active_flows_matrix"] = "本地主机活动流矩阵",
  },
  ["local_hosts_stats"] = {
    ["criteria"] = "标准",
    ["looking_glass"] = "镜子",
  },
  ["locales"] = {
    ["cn"] = "中文",
    ["cz"] = "捷克语",
    ["de"] = "德语",
    ["en"] = "英语",
    ["it"] = "意大利语",
    ["jp"] = "日语",
    ["pt"] = "葡萄牙语",
  },
  ["login"] = {
    ["add_captive_portal_user"] = "添加捕获门户用户",
    ["add_web_user"] = "添加网络用户",
    ["auth_success"] = "认证成功",
    ["auth_token"] = "用户认证Token",
    ["blacklisted_ip_notice"] = "身份验证尝试失败太多次。请在5分钟后重试。",
    ["change_password"] = "更改密码",
    ["confirm_password"] = "确认密码",
    ["device_label"] = "设备标签",
    ["donation"] = "如果您觉得 %{product} 有用,请通过小额<A href=\"%{donation_url}\">捐赠</A>来支持我们。 您的资金将有助于运行和促进该项目的发展。 谢谢你。",
    ["enter_credentials"] = "请输入您访问此网络的凭据。",
    ["generate_token"] = "生成Token",
    ["informative_captive_join"] = "访问Internet",
    ["informative_captive_portal_tos"] = "我已阅读并同意<a target='_blank' href='%{url}'>服务条款</a>。",
    ["internet_redirecting"] = "我们正在将您重定向到 Internet...",
    ["license"] = "%{product} 在 <a href=\"%{license_url}\">%{license}</a> 下发布。",
    ["links"] = "<A target=\"_blank\" href=\"https://www.ntop.org/support/documentation/documentation/\">用户指南</A> | <A target=\"_blank\" HREF=\"https://www.ntop.org/community/\">社区</A> | <A target=\"_blank\" HREF=\"https://www.ntop.org/support/need-help-2/commercial-support/\">技术支持</A> | <A target=\"_blank\" HREF=\"https://www.ntop.org/support/documentation/faq/\">FAQ</A> | <A target=\"_blank\" HREF=\"https://github.com/ntop\">Code</A> | <A target=\"_blank\" href=\"https://www.ntop.org/support/need-help-2/contact-us/\">Contact Us</A>",
    ["logging_out"] = "注销...",
    ["login"] = "登录",
    ["logout"] = "退出",
    ["logout_message"] = "您确定要退出吗？",
    ["logout_user_x"] = "退出 %{user}",
    ["must_change_password"] = "必须更改默认管理员密码。 请在下方输入新密码。",
    ["password"] = "密码",
    ["password_mismatch"] = "密码不匹配",
    ["password_not_valid"] = "请指定不同的密码",
    ["password_pattern_error"] = "请输入至少 5 个有效字符。",
    ["password_ph"] = "密码（默认:admin）",
    ["passwords_dont_match"] = "密码不匹配。",
    ["unable_to_login"] = "无法登录？",
    ["unsupported_browser"] = "您的浏览器不受支持。<br>请考虑使用最新的 Firefox 或 Chrome。",
    ["username"] = "用户名",
    ["username_ph"] = "用户名（默认:admin）",
    ["web_users"] = "网络用户",
    ["welcome_to"] = "欢迎来到 %{product}",
    ["wrong-credentials"] = "登录失败：用户名或密码无效。",
  },
  ["mac_details"] = {
    ["and_n_more_hosts"] = "<a href='%{host_url}'>%{host_label}</a> 和 <a href='%{url}'>%{num} 更多主机</a>",
    ["and_one_more_host"] = "<a href='%{host_url}'>%{host_label}</a> 和 <a href='%{url}'>多一个主机</a>",
    ["json_error_inactive"] = "JSON 错误：不活动的主机已清除或 %{product} 已终止？",
    ["mac"] = "MAC",
    ["mac_cannot_be_found_message"] = "找不到 MAC %{mac}。",
    ["mac_host"] = "<a href='%{host_url}'>%{host_label}</a>",
    ["mac_parameter_missing_message"] = "MAC 参数丢失（内部错误？）",
    ["reset_mac_stats"] = "重置 MAC 统计",
    ["reset_mac_stats_note"] = "<b>注意</b>：这个mac地址的历史图表在重置后可能会有一个缺口。",
    ["reset_stats_in_progress"] = "MAC 统计将在 5 秒内重置",
  },
  ["mac_stats"] = {
    ["all_devices"] = "全部",
    ["all_layer_2_devices"] = "所有设备",
    ["all_manufacturers"] = "所有制造商",
    ["arp_total"] = "ARP",
    ["dev_layer_2_devices"] = "%{device_type} 设备",
    ["dhcp_only"] = "仅 DHCP",
    ["filter_macs"] = "过滤 MAC",
    ["idle_devices_not_listed"] = "未列出空闲设备。",
    ["layer_2_dev_devices"] = "%{device_type} 设备",
    ["layer_2_devices_with_manufacturer"] = "%{manufacturer} %{title}",
    ["layer_2_source_devices"] = "%{device_type} 设备",
    ["manufacturer"] = "制造商",
    ["source_macs"] = "仅源Mac",
  },
  ["manage_configurations"] = {
    ["active_monitoring"] = "活动监控",
    ["after_reset_request"] = "<b>重启</b> %{product} 使配置恢复出厂设置生效。",
    ["checks"] = "检测",
    ["config_import"] = "导入 <span class='item'>{{ type }}</span> 配置",
    ["everything"] = "整个 %{product} 配置（包括用户、首选项和下面的所有配置）",
    ["export_policy"] = "导出 nProbe IPS 策略",
    ["infrastructure_instances"] = "基础设施实例",
    ["manage_configuration"] = "管理配置",
    ["manage_configurations"] = "管理配置",
    ["notifications"] = "端点和收件人",
    ["pool_endpoint_recipients"] = "池（包括 SNMP、活动监控、用户脚本、端点和收件人）",
    ["snmp"] = "SNMP",
    ["user_scripts"] = "用户脚本",
    ["factory_reset"] = {
      ["body"] = "将配置重置为出厂设置。 所有自定义更改都将丢失。",
      ["title"] = "重置 <span class='selected-item'>%{reset_element}</span> 配置",
    },
    ["import_modal"] = {
      ["all_title"] = "导入 %{product} 配置",
      ["title"] = "导入 <span class='selected-item'>%{import_element}</span> 配置",
    },
    ["messages"] = {
      ["import_all_success"] = "配置导入成功。 请<b>重启 %{product}</b> 以使导入的配置生效。",
      ["import_success"] = "已成功导入所选配置！",
      ["reset_all_success"] = "恢复出厂设置请求成功发出。 请<b>重启 %{product}</b> 以使配置恢复出厂设置生效。",
      ["reset_success"] = "所选配置已成功重置！",
    },
  },
  ["manage_data"] = {
    ["all_hosts"] = "全部",
    ["delete"] = "删除主机数据",
    ["delete_active_interface"] = "删除接口数据",
    ["delete_inactive_interfaces"] = "删除不活动接口数据",
    ["delete_system_interface_data"] = "删除系统接口数据",
    ["delete_tab"] = "删除",
    ["export"] = "导出主机数据",
    ["export_tab"] = "导出",
    ["hosts"] = "主机",
    ["inactive_interfaces_data"] = "不活动接口",
    ["inactive_interfaces_description"] = "管理<b>不活动接口</b>的数据,您可以删除关于这些接口的缓存信息。<br><br>配置不活动接口时启用下面的按钮。",
    ["ip_or_mac_address"] = "IP 或 MAC 地址或 /24",
    ["local_hosts"] = "本地",
    ["mac_or_ip_required"] = "请填写有效的 IP 或 MAC 地址",
    ["manage_data"] = "管理数据",
    ["remote_hosts"] = "远程",
    ["single"] = "单一",
    ["system_interface_data"] = "系统接口",
    ["system_interface_description"] = "管理<b>系统接口</b>的缓存数据,您可以删除数据以释放一些空间。<br><br>下面的按钮在没有删除待处理请求时启用。",
    ["system_interface_note"] = "系统接口数据包括位于\"系统\"菜单项下的所有数据,例如 CPU、InfluxDB 和 SNMP 数据。",
  },
  ["manage_users"] = {
    ["add_new_user"] = "添加新用户",
    ["administrator"] = "管理员",
    ["allow_pcap_download"] = "允许 pcap 下载",
    ["allow_pcap_download_descr"] = "允许用户下载实时流量和 pcap",
    ["allowed_interface"] = "允许的接口",
    ["allowed_networks"] = "允许的网络",
    ["allowed_networks_descr"] = "此用户可以查看的逗号分隔的网络列表。 例子：",
    ["allowed_passwd_charset"] = "允许的字符是 ISO 8859-1 (latin1) 大小写字母、数字和特殊符号",
    ["any_interface"] = "任何接口",
    ["authentication_lifetime"] = "身份验证生命周期",
    ["change_user_password"] = "更改用户密码",
    ["change_user_preferences"] = "更改用户首选项",
    ["confirm_delete_user"] = "您确定要删除 %{user} 吗？",
    ["enable_captive_portal"] = "<a href=\"%{url}\">启用捕获门户</a> 允许您定义捕获门户用户",
    ["expires_after"] = "到期后",
    ["group"] = "分组",
    ["host_pool_id"] = "主机池 ID",
    ["manage"] = "管理",
    ["manage_user_x"] = "管理用户 %{user}",
    ["manage_users"] = "用户",
    ["new_password"] = "新密码",
    ["new_password_confirm"] = "确认新密码",
    ["no_host_pools"] = "未定义主机池。 请在<a href='%{url}'>处</a>创建一个",
    ["non_privileged_user"] = "非高级用户",
    ["old_password"] = "旧密码",
    ["the_auth_can_be_perpetual"] = "身份验证可以是永久的,也可以在指定时间后过期",
    ["the_host_pool_associated"] = "认证成功后与用户关联的主机池",
    ["token"] = "Token",
    ["token_not_generated"] = "Token尚未生成",
    ["unlimited"] = "无限制",
    ["user_role"] = "用户角色",
    ["web_captive_users"] = "网络/捕获门户用户",
  },
  ["metrics"] = {
    ["days"] = "天",
    ["hours"] = "小时",
    ["mins"] = "分钟",
    ["minutes"] = "分钟",
    ["secs"] = "秒",
  },
  ["ndpi_categories"] = {
    ["Advertisement"] = "广告",
    ["Allowed_Site"] = "允许的站点",
    ["Antimalware"] = "反恶意软件",
    ["Banned_Site"] = "被禁站点",
    ["Chat"] = "聊天",
    ["Cloud"] = "云",
    ["Collaborative"] = "协作",
    ["DataTransfer"] = "数据传输",
    ["Database"] = "数据库",
    ["Download-FileTransfer-FileSharing"] = "下载-WeTransfer-文件共享",
    ["Email"] = "电子邮件",
    ["FileSharing"] = "文件共享",
    ["Game"] = "游戏",
    ["Malware"] = "恶意软件",
    ["Media"] = "媒体",
    ["Mining"] = "挖掘",
    ["Music"] = "音乐",
    ["Network"] = "网络",
    ["Productivity"] = "生产率",
    ["RPC"] = "RPC",
    ["RemoteAccess"] = "远程访问",
    ["Shopping"] = "购物",
    ["Site_Unavailable"] = "站点不可用",
    ["SocialNetwork"] = "社交网络",
    ["SoftwareUpdate"] = "软件更新",
    ["Streaming"] = "流",
    ["System"] = "系统",
    ["Unspecified"] = "未指定",
    ["VPN"] = "VPN",
    ["Video"] = "视频",
    ["VoIP"] = "VoIP",
    ["Web"] = "Web",
  },
  ["ndpi_page"] = {
    ["custom_applications"] = "自定义应用程序",
    ["live_flows_count"] = "实时流计数",
    ["note"] = "注意",
    ["note_historical_per_protocol_traffic"] = "历史 %{what} 流量数据可以通过 <a href='%{url}'>%{flask_icon} Preferences</a> 启用。",
    ["note_live_flows_chart"] = "此图表仅描述 TCP 连接。",
    ["note_possible_probing_alert"] = "像 %{icon} 这样的图标表示可能的探针（或应用程序服务器关闭）警告,因为主机已接收到特定应用程序的流量而没有发回任何数据。 您可以使用<A HREF='%{url}'>历史报告</A>来深入了解这个问题。",
    ["note_protocol_usage_time"] = "应用程序使用时间是按离散的时隙间隔计算的。",
    ["num_contacted_services_as_client"] = "客户通信服务",
    ["num_contacted_services_as_client_descr"] = "（TLS、HTTP 和DNS名称）",
    ["overview"] = "%{what} 概述",
    ["rcvd"] = "接收",
    ["received_only"] = "仅接收",
    ["sent"] = "发送",
    ["sent_only"] = "仅发送",
    ["total_since_startup"] = "总计（自启动以来）",
    ["unable_to_find_host"] = "无法找到 %{host_ip}（数据已过期？）",
  },
  ["nedge"] = {
    ["a_member_address_is"] = "成员地址是为用户分配设备的规则。 以下顺序适用：",
    ["a_zero_quota_is_unlimited"] = "零配额意味着无限流量。",
    ["active_user"] = "活动用户",
    ["add_gateway"] = "添加网关",
    ["add_member"] = "添加成员",
    ["add_port_forwarding_rule"] = "添加端口转发规则",
    ["add_routing_policy"] = "添加路由策略",
    ["add_some_gateways_first"] = "没有定义网关。 请在<a href=\"%{url}\">处</a>定义网关。",
    ["add_static_lease"] = "添加静态租约",
    ["add_static_route"] = "添加静态路由",
    ["add_system_user"] = "添加系统用户",
    ["add_user_member"] = "添加用户成员",
    ["applied_policy"] = "适用策略",
    ["apply_and_reboot"] = "应用并重启",
    ["apply_and_restart_self"] = "应用并重启",
    ["apply_configuration"] = "应用配置",
    ["apply_configuration_and_reboot"] = "您将应用新的配置。网络接口将根据指定的设置重新配置,系统将重新启动。错误的设置会导致系统失去 IP 连接。仅当您非常确信设置正确时才继续配置。",
    ["apply_configuration_and_restart_self"] = "您将应用新配置。这将导致 %{product} 重新启动,并可能导致此 Web GUI 在几秒钟内无响应。",
    ["apply_the_initial_device_configuration"] = "应用初始设备配置。",
    ["authentication_gui_and_captive_portal"] = "身份验证方法适用于 %{product} gui 和 <a href=\"%{url}\">Captive Portal</a> 访问。",
    ["bandwidth_allocation"] = "带宽分配",
    ["bind_to_any"] = "任何接口",
    ["bind_to_lan"] = "LAN 接口",
    ["cannot_rename_gateway"] = "无法重命名网关。",
    ["cannot_rename_policy"] = "无法重命名策略。",
    ["cannot_rename_static_route"] = "无法重命名静态路由。",
    ["captive_portal_other_methods"] = "捕获门户使用<a href=\"%{url}\">身份验证首选项</a>中定义的身份验证方法（例如 Radius）。",
    ["categories_policies"] = "类别策略",
    ["configure_routing_policies"] = "配置路由策略",
    ["configured_members"] = "已配置成员",
    ["confirm_delete_gateway"] = "确认要删除网关\"%{gw}\"吗？",
    ["confirm_delete_policy"] = "确认要删除策略\"%{policy}\"吗？",
    ["confirm_delete_port_forwarding_rule"] = "确认要删除指定的规则吗？",
    ["confirm_delete_static_lease"] = "确认要删除\"%{hostname}\"静态租约吗？",
    ["confirm_delete_static_route"] = "确认要删除静态路由\"%{sr}\"吗？",
    ["confirm_remove_member_from_user"] = "确认要从用户 %{user} 中删除成员 %{member} 吗？",
    ["could_not_add_port_forwarding"] = "无法添加端口转发规则",
    ["custom"] = "自定义",
    ["custom_datetime"] = "当前时间",
    ["custom_datetime_descr"] = "手动设置系统的日期和时间。",
    ["custom_policy"] = "自定义策略",
    ["data_reset"] = "数据重置",
    ["data_reset_msg"] = "确认要重置所有设备数据吗？数据包括存储的时间序列、流量、警告、Redis 缓存和主机计数器。设备配置和系统设置不会受到重置的影响。<br> 这将导致 %{product} 重新启动,并可能导致此 Web GUI 在几秒钟内无响应。",
    ["date_time"] = "日期和时间",
    ["default_categories"] = "默认类别策略",
    ["default_categories_with_a_defined_policy"] = "默认自定义类别策略",
    ["default_categories_with_a_defined_quota"] = "默认配额类别策略",
    ["default_categories_with_policy_x"] = "默认 %{x} 类别策略",
    ["default_gateway"] = "默认网关",
    ["default_policy"] = "默认策略",
    ["default_protocols"] = "默认应用策略",
    ["default_protocols_with_a_defined_policy"] = "默认自定义应用程序策略",
    ["default_protocols_with_a_defined_quota"] = "默认配额应用策略",
    ["default_protocols_with_policy_x"] = "默认 %{x} 应用策略",
    ["default_shaper"] = "通过",
    ["default_shaper_descr"] = "接口速度的默认带宽百分比。",
    ["delete_gateway"] = "删除网关",
    ["delete_policy"] = "删除策略",
    ["delete_port_forwarding_rule"] = "删除端口转发规则",
    ["delete_static_lease"] = "删除静态租约",
    ["delete_static_route"] = "删除静态路由",
    ["device_protocol_policy_has_higher_priority"] = "设备策略的优先级高于应用程序策略。",
    ["device_protocols_blocked_warning"] = "流量将<b>不会</b>被阻止,因为 %{device_protocols_policies} 被禁用",
    ["device_protocols_policies"] = "设备策略",
    ["dhcp_disabled_warning"] = "如果您的网络中没有其他可用的DHCP服务器,那么您的客户端将无法自动连接。",
    ["dhcp_first_ip"] = "第一个 IP 地址",
    ["dhcp_first_ip_descr"] = "DHCP地址池的第一个IP地址",
    ["dhcp_last_ip"] = "最后一个 IP 地址",
    ["dhcp_last_ip_descr"] = "DHCP地址池的最后一个IP地址",
    ["dhcp_leases"] = "DHCP租约",
    ["dhcp_server"] = "DHCP服务器",
    ["dhcp_server_description"] = "启用DHCP服务器。",
    ["dhcp_server_not_enabled"] = "DHCP服务器未启用。您可以在<a href=\"%{url}\">处</a>启用它。",
    ["dns_configuration"] = "DNS配置",
    ["dns_server_preset"] = "DNS服务器预设",
    ["dns_server_preset_descr"] = "您可以从列表中选择优先DNS服务器或指定您的自定义DNS服务器。",
    ["download_description"] = "接口的最大下载速度。",
    ["download_speed"] = "下载速度",
    ["edit_gateway"] = "编辑网关",
    ["edit_quotas"] = "编辑配额",
    ["edit_routing_policy"] = "编辑路由策略",
    ["edit_static_route"] = "编辑静态路由",
    ["edit_user"] = "编辑用户",
    ["edit_users"] = "编辑用户",
    ["enable_device_protocols_policies"] = "设备应用策略",
    ["enable_interface"] = "启用接口",
    ["enable_interface_descr"] = "启用或禁用网络接口。",
    ["enable_nat"] = "启用 NAT",
    ["enable_nat_descr"] = "如果启用,客户端的 IP 地址将被伪装成 WAN 接口的 IP 地址。",
    ["enforce_global_dns"] = "强制执行全局 DNS",
    ["enforce_global_dns_description"] = "启用DNS伪造以强制设备使用指定的全局 DNS。",
    ["enternal_port_overlap"] = "外部端口与分配给 %{ip} 的端口 %{port} 重叠",
    ["external_port"] = "外部端口 (WAN)",
    ["external_port_or_range"] = "外部端口或端口范围",
    ["factory_reset"] = "恢复出厂设置",
    ["factory_reset_msg"] = "确认要将设备重置为出厂配置吗？任何配置和数据都将丢失。",
    ["filter_policies"] = "过滤策略",
    ["gateway_exists"] = "网关 %{gw} 已经存在。",
    ["gateway_interface_descr"] = "指定用于确定网关IP地址的网络接口",
    ["gateway_name"] = "网关名称",
    ["gateway_type"] = "模式",
    ["gateway_type_address_descr"] = "指定网关的IP地址。",
    ["gateway_x"] = "网关 %{gw}",
    ["gateways"] = "网关",
    ["gateways_status"] = "网关状态",
    ["global_dns"] = "全局 DNS",
    ["global_shapers_rates"] = "全局带宽控制",
    ["high_priority"] = "高优先级",
    ["higher_priority"] = "更高的优先级",
    ["hostname"] = "主机名",
    ["hostname_lease_exists"] = "已存在主机名为 \"%{hostname}\" 的静态租约。",
    ["interfaces_configuration"] = "接口配置",
    ["internal_ip"] = "内部 IP (LAN)",
    ["internal_port"] = "内部端口 (LAN)",
    ["internal_port_number"] = "内部端口号",
    ["internal_port_overlap"] = "内部端口与 %{internal_port} 重叠已经映射到外部端口 %{external_port}",
    ["invalid_dhcp_range"] = "DHCP范围无效。请确保DHCPIP 地址在 LAN 网络范围内,否则您的设备将无法上网。",
    ["invalid_port_range"] = "无效的端口范围",
    ["ip_address_takes_preced"] = "IP 地址优先于网络地址",
    ["ip_lease_exists"] = "一个静态租约与 ip \"%{ip}\" 已经存在。",
    ["lan"] = "LAN",
    ["lan_ip_addr"] = "局域网 IP 地址",
    ["lan_ip_addr_descr"] = "指定LAN接口IP地址。",
    ["lan_recovery_ip"] = "LAN恢复IP",
    ["lan_recovery_ip_descr"] = "到达 LAN 接口上 %{product} 设备的附加 IP 地址。",
    ["lan_recovery_warning"] = "请确保这不会与任何其他定义的网络冲突。",
    ["link_not_ready"] = "链路未就绪",
    ["linkdown"] = "链路Down",
    ["low_priority"] = "低优先级",
    ["lower_priority"] = "更低优先级",
    ["mac_address_takes_preced"] = "mac 地址优先于 IP 地址",
    ["mac_lease_exists"] = "mac 的静态租约 \"%{mac}\" 已经存在。",
    ["max_bps_per_host"] = "每个主机最大 %{bps}",
    ["max_perc"] = "最大 %{perc}%%",
    ["max_rrt"] = "最大 RTT",
    ["max_rrt_description"] = "到监视器地址的最大往返时间。如果测得的 RTT 超过此值,则认为网关关闭。",
    ["maximum_enforced"] = "最大强制执行",
    ["maximum_enforced_descr"] = "如果设置为最大强制,则无论可用带宽如何,都将强制执行最大带宽分配。",
    ["member_address"] = "成员地址",
    ["member_exists"] = "未添加成员 %{member},因为它已分配给用户 %{existing_user}",
    ["members"] = "成员",
    ["members_for_user_x"] = "用户 %{user} 的成员",
    ["min_perc"] = "最小 %{perc}%%",
    ["minimum_guaranteed"] = "最低保证",
    ["minimum_guaranteed_descr"] = "如果设置为最小保证,则只保证最小带宽分配。",
    ["mode"] = "模式",
    ["nedge"] = "nEdge",
    ["network_conf_dhcp"] = "DHCP",
    ["network_conf_iface_descr"] = "指定接口IP地址模式。",
    ["network_conf_iface_gw"] = "接口默认网关的IP地址（例如192.168.1.1）。",
    ["network_conf_iface_ip"] = "接口的IP地址（例如 192.168.1.10）。",
    ["network_conf_iface_nmask"] = "接口的子网掩码（例如 255.255.255.0）。",
    ["network_conf_iface_title"] = "%{ifrole} 接口 %{ifname}",
    ["network_conf_static"] = "静态",
    ["network_conf_vlan_trunk"] = "VLAN Trunk",
    ["no_gateways_defined"] = "未定义网关。",
    ["no_interfaces_available"] = "没有可用的接口。",
    ["no_routing_policies_defined"] = "没有定义路由策略",
    ["no_rules"] = "无规则",
    ["no_static_leases"] = "无静态租约",
    ["no_static_routes_defined"] = "没有定义静态路由。",
    ["ntp_sync"] = "NTP 同步",
    ["ntp_sync_descr"] = "切换与 NTP 服务器的同步。",
    ["per_ip_rates"] = "每个主机带宽控制",
    ["ping_address"] = "监控地址",
    ["ping_address_description"] = "要监控的地址以验证网关状态。",
    ["please_fill_out_this_field"] = "请填写此字段。",
    ["policy_due_to_source"] = "%{policy} 由于 %{source}",
    ["policy_name"] = "策略名称",
    ["port_forwarding"] = "端口转发",
    ["power_off"] = "关机",
    ["power_off_confirm"] = "您要关闭设备电源吗？",
    ["priority_x"] = "优先级 %{x}",
    ["product_status"] = "%{product} 状态",
    ["protocol_policy_has_higher_priority"] = "应用程序策略的优先级高于类别策略。",
    ["protocols_policies"] = "应用程序策略",
    ["reboot"] = "重启",
    ["reboot_corfirm"] = "你想重启设备吗？",
    ["remote_assistance"] = "远程协助",
    ["remove_all_members"] = "删除所有成员",
    ["remove_all_members_confirm"] = "确认要删除用户 %{user} 的所有成员吗？",
    ["reset_and_reboot"] = "重置并重启",
    ["reset_and_restart_self"] = "重置并重启",
    ["restrict_weg_gui_access_description"] = "限制对该管理 Web GUI 的访问,使其只能从 LAN 接口（更安全）或系统的任何接口（不太安全）访问。",
    ["restrict_weg_gui_access_title"] = "管理 Web GUI 访问",
    ["router"] = "路由器",
    ["router_mode_requires_enterprise"] = "路由器模式仅在 nedge 企业版中可用。",
    ["routing"] = "路由",
    ["routing_policies"] = "路由策略",
    ["routing_policy"] = "路由策略",
    ["routing_policy_exists"] = "路由策略\"%{policy}\" 已经存在。",
    ["safe_search_description"] = "使用儿童安全DNS阻止不当内容。",
    ["search_categories"] = "搜索类别",
    ["search_protocols"] = "搜索应用程序",
    ["security"] = "网络安全",
    ["set_the_device_mode"] = "设置设备操作模式。",
    ["setup_apply"] = "应用",
    ["setup_config_edited_descr"] = "应用更改使新配置生效或放弃以继续使用当前配置。",
    ["setup_config_edited_title"] = "系统配置已编辑！",
    ["setup_discard"] = "丢弃",
    ["setup_mode"] = "操作模式",
    ["setup_network"] = "网络",
    ["shapers"] = "带宽控制",
    ["single_port_router"] = "单端口路由器",
    ["slow_pass_shaper_descr"] = "接口速度的慢速通道带宽百分比。",
    ["slower_pass_shaper_descr"] = "接口速度的更慢速通道带宽百分比。",
    ["specify_a_protocol"] = "指定一个应用程序",
    ["static_dhcp_leases"] = "DHCP服务器静态租约",
    ["static_route_address_descr"] = "指定静态路由的目的地。",
    ["static_route_exists"] = "静态路由 %{sr} 已经存在。",
    ["static_route_is_local_descr"] = "指定这是否是到本地网络的静态路由。到本地网络的静态路由中的所有主机都被视为本地主机。",
    ["static_route_is_local_title"] = "本地网络静态路由",
    ["static_route_name"] = "静态路由名称",
    ["static_route_netmask_descr"] = "指定要应用于静态路由目的地的网络掩码。",
    ["static_route_via_descr"] = "指定用作静态路由下一跳的 IP 地址。",
    ["static_route_x"] = "静态路由 %{sr}",
    ["static_routes"] = "静态路由",
    ["static_routes_in_policies"] = "静态路由应用于（并优先于）任何定义的<a href=\"%{url}\">路由策略</a>。",
    ["status_activate"] = "激活",
    ["status_connected"] = "已连接",
    ["status_connecting"] = "正在连接",
    ["status_deactivate"] = "停用",
    ["status_disabled"] = "已禁用",
    ["status_disconnected"] = "断开连接",
    ["status_down"] = "Down",
    ["status_failed"] = "失败",
    ["status_inactive"] = "不活动",
    ["status_unreachable"] = "无法到达",
    ["status_up"] = "Up",
    ["system_setup"] = "系统设置",
    ["system_users"] = "系统用户",
    ["test_policies"] = "测试策略",
    ["test_policies_page_description"] = "此页面可用于验证哪些策略将根据当前配置应用于 %{user} 设备。",
    ["the_fallback_policy"] = "<a href=\"%{url}\">回退策略</a>在给定应用程序及其相应类别的用户和全局策略都设置为\"默认\"。",
    ["the_not_assigned_user_contains"] = "\"未分配\" 用户可用于为未分配给任何用户的设备定义策略。",
    ["the_primary_dns_server"] = "主DNS服务器",
    ["the_secondary_dns_server"] = "辅助DNS服务器",
    ["timezone"] = "时区",
    ["timezone_descr"] = "设置系统的时区。",
    ["upload_description"] = "接口的最大上传速度。",
    ["upload_speed"] = "上传速度",
    ["user"] = "用户",
    ["user_categories"] = "用户 %{user} 的类别策略",
    ["user_categories_with_a_defined_policy"] = "用户 %{user} 的自定义类别策略",
    ["user_categories_with_a_defined_quota"] = "用户 %{user} 的配额类别策略",
    ["user_categories_with_policy_x"] = "%{x} 用户 %{user} 的类别策略",
    ["user_category_policies"] = "用户类别策略",
    ["user_device_protocols"] = "%{user} 设备的应用程序策略",
    ["user_protocols"] = "用户 %{user} 的应用策略",
    ["user_protocols_with_a_defined_policy"] = "用户 %{user} 的自定义应用程序策略",
    ["user_protocols_with_a_defined_quota"] = "用户 %{user} 的配额应用策略",
    ["user_protocols_with_policy_x"] = "%{x} 用户 %{user} 的应用程序策略",
    ["users_list"] = "用户列表",
    ["view_quotas"] = "查看配额",
    ["wan"] = "WAN",
    ["wan_interface"] = "WAN 接口",
    ["with_quota"] = "有配额",
    ["x_shaper_per_host_descr"] = "%{shaper} 最大每台主机带宽分配。0 表示无限制。",
    ["you_can_assign_routing_policy"] = "您可以将路由策略分配给<a href=\"%{url}\">用户</a>。",
    ["you_can_assign_shapers_to_users_protocols"] = "您可以将带宽策略分配给<a href=\"%{url}\">用户</a>特定应用程序或将它们全局设置为<a href=\"%{url2}\">后备策略</a>。",
    ["you_can_set_static_dhcp_lease_here"] = "您可以在<a href=\"%{url}\">处</a>配置静态DHCP租约。",
    ["you_can_use_gateways_in_policies"] = "您可以在<a href=\"%{url}\">路由策略</a>中指定网关优先级。",
    ["policy_source_labels"] = {
      ["policy_source_category"] = "类别策略",
      ["policy_source_default"] = "未知",
      ["policy_source_device_protocol"] = "设备应用策略",
      ["policy_source_pool"] = "默认策略",
      ["policy_source_protocol"] = "应用策略",
    },
  },
  ["network_alert_config"] = {
    ["trigger_alerts_for_network"] = "触发网络 %{network} 的警告",
    ["trigger_network_alerts"] = "触发网络警告",
  },
  ["network_details"] = {
    ["network"] = "网络",
    ["network_alias"] = "网络别名",
    ["network_parameter_missing_message"] = "网络参数丢失（内部错误？）",
    ["no_available_stats_for_network"] = "网络 %{network} 没有可用的统计数据",
  },
  ["network_stats"] = {
    ["network"] = "网络",
    ["network_list"] = "网络列表",
    ["network_name"] = "网络名称",
    ["networks_traffic_with_ipver"] = "%{networks} IPv%{ipver} 流量",
    ["note_aliases_not_configurable"] = "别名是不可编辑的,如使用选项 -m 静态指定的。",
    ["note_broader_network"] = "更广泛的网络将不包括在小型网络中定义的主机。",
    ["note_overlapping_networks"] = "如果您定义了重叠网络：",
    ["note_see_both_network_entries"] = "您将在上表中看到两个网络条目。",
  },
  ["nindex"] = {
    ["bytes"] = "字节",
    ["cli_asn"] = "Cli ASN",
    ["cli_port"] = "cli 端口",
    ["client"] = "客户端",
    ["client_name"] = "客户端名称",
    ["extract_flow_info"] = "操作",
    ["first_seen"] = "开始",
    ["flow_risk"] = "流量风险",
    ["info"] = "信息",
    ["l4proto"] = "协议",
    ["l7cat"] = "L7 类别",
    ["l7proto"] = "L7协议",
    ["last_seen"] = "结束",
    ["max_results"] = "最大结果",
    ["observation_point_id"] = "Obs. Point ID",
    ["packets"] = "Pkts",
    ["probe_ip"] = "探针 IP",
    ["query_performed"] = "在执行中的查询",
    ["run_query"] = "运行查询",
    ["select_note"] = "对所有字段使用 * 或以逗号分隔的字段列表 (<field1>,<field2>,...)",
    ["server_name"] = "服务器名称",
    ["sever"] = "服务器",
    ["src2dst_dscp"] = "客户端 DSCP",
    ["srv_asn"] = "Srv ASN",
    ["srv_port"] = "Srv 端口",
    ["status"] = "状态",
    ["throughput"] = "吞吐量",
    ["time_range"] = "时间范围",
    ["vlan_id"] = "VLAN",
    ["where_note"] = "使用&lt;field&gt;&lt;operator&gt;&lt;value&gt;条件,其中&lt;operator&gt;可以是&lt;, &gt;, =, !=. AND/OR 运算符也可以组合条件. 请为字符串字段使用引号（例如 INFO='www.ntop.org'）",
    ["tags"] = {
      ["cli_asn"] = "客户端 ASN",
      ["cli_ip"] = "客户端 IP",
      ["cli_port"] = "客户端端口",
      ["flow_risk"] = "流量风险",
      ["l4proto"] = "协议",
      ["l7_proto"] = "应用程序",
      ["l7cat"] = "应用类别",
      ["l7proto"] = "应用程序",
      ["observation_point_id"] = "观察 Point ID",
      ["probe_ip"] = "探针 IP",
      ["role"] = "角色",
      ["roles"] = "角色",
      ["src2dst_dscp"] = "客户端 DSCP",
      ["srv_ip"] = "服务器IP",
      ["srv_port"] = "服务器端口",
      ["status"] = "状态",
      ["vlan_id"] = "VLAN",
    },
  },
  ["os_details"] = {
    ["no_available_stats_for_os"] = "OS %{os} 没有可用的统计数据",
    ["os"] = "操作系统",
    ["os_parameter_missing_message"] = "OS 参数丢失（内部错误？）",
  },
  ["os_stats"] = {
    ["hosts_by_operating_system"] = "按操作系统划分的主机",
  },
  ["packets_page"] = {
    ["arp_distribution"] = "ARP 分布",
    ["ip_version_distribution"] = "IP 版本分布",
    ["lost"] = "丢失",
    ["out_of_order"] = "乱序",
    ["received_distribution"] = "接收数据分布",
    ["retransmissions"] = "重传",
    ["sent_distribution"] = "发送数据分布",
    ["sent_vs_rcvd_distribution"] = "发送 vs 接收分布",
    ["size_distribution"] = "大小分布",
    ["tcp_flags_distribution"] = "TCP 标志分布",
    ["tcp_flags_vs_arp_distribution"] = "TCP 标志/ARP 分布",
    ["tcp_packets_analysis"] = "TCP 数据包分析",
    ["version_vs_flags_distribution"] = "IP 版本 vs TCP 标志分布",
  },
  ["peers_page"] = {
    ["host"] = "主机",
    ["no_active_flows_message"] = "没有观察到指定主机的活动流",
    ["top_peer_protocol"] = "Top Peer应用程序",
    ["top_peers_for_host"] = "Top %{hostkey} Peers",
    ["traffic_volume"] = "流量容量",
  },
  ["periodic_activities_descr"] = {
    ["5min.lua"] = "迭代活动主机并写入其时间序列。以5分钟的粒度执行检查。",
    ["daily.lua"] = "执行旧数据清理。以每日粒度执行检查。",
    ["dequeue_flows_for_hooks.lua"] = "处理流、主机和其他哈希条目的内存生命周期。执行流检查。将流转储到数据库。",
    ["discover.lua"] = "运行请求的和定期的网络发现。",
    ["hourly.lua"] = "以每小时的粒度执行检查",
    ["housekeeping.lua"] = "检查 n2disk 提取作业。检查并重新加载列表。",
    ["minute.lua"] = "编写扩展接口时间序列这样的应用程序。为检查、pods、容器编写时间序列。以分钟的粒度执行检查。",
    ["notifications.lua"] = "处理警告通知,将它们分派给配置的接收者。",
    ["periodic_user_scripts.lua"] = "执行周期性的检查。",
    ["second.lua"] = "写入基本接口时间序列,例如总流量。",
    ["stats_update.lua"] = "更新系统统计信息（例如 CPU 负载）。",
    ["timeseries.lua"] = "将时间序列数据写入 InfluxDB。不使用 InfluxDB 时无效。",
  },
  ["periodicity_map_page"] = {
    ["no_periodicity"] = "没有找到<b>主机</b>。",
    ["node_descr"] = "主机用圆圈表示：<font color=#28a745>Local</font>、<font color=#6c757d>Remote</font>、<font color=#007bff>Multicast</font >, <font color=#17a2b8>广播</font> 主机。<li>双击节点可跳转到主机页面。使用鼠标滚轮放大/缩小。</li>",
  },
  ["please_wait_page"] = {
    ["command"] = "命令",
    ["database"] = "数据库",
    ["host"] = "主机",
    ["id"] = "id",
    ["info"] = "信息",
    ["operations_on_database_msg"] = "当前对数据库执行的操作如下：",
    ["state"] = "状态",
    ["time"] = "时间",
    ["user"] = "用户",
    ["waiting_for_db_msg"] = "正在进行数据库架构迁移,正在等待数据库 <b>%{dbname}</b> 开始运行。一旦数据库准备就绪,您将被重定向。取决于现有数据库的大小数据库,此操作可能需要很长时间。迁移仅执行一次且仅在主要版本升级期间执行。如有疑问,请通过 <a href='mailto:info@ntop.org'>info@ntop.org 联系开发人员</a>.",
  },
  ["plugins_overview"] = {
    ["action_view"] = "在代码编辑器中打开用户脚本源代码",
    ["alert_key"] = "警告Key",
    ["alert_key_string"] = "警告Key字符串",
    ["alert_name"] = "警告名称",
    ["edition_only"] = "仅%{edition}",
    ["filtered"] = "过滤",
    ["filters"] = "过滤器",
    ["has_attacker"] = "已知攻击者",
    ["has_victim"] = "已知被攻击者",
    ["hooks"] = "hooks",
    ["loaded_plugins"] = "加载的插件",
    ["plugin"] = "插件",
    ["reload_plugins"] = "重新加载插件",
    ["script"] = "脚本",
    ["source_location"] = "源位置",
    ["status_key"] = "流状态Key",
    ["type"] = "类型",
  },
  ["policy"] = {
    ["abort_add_rule"] = "终止添加规则?",
    ["add_rule"] = "添加规则",
    ["autoconfigure"] = "自动配置",
    ["autoconfigure_note"] = "自动配置功能通过检测池中主机常用的主机/协议，自动为池创建建议的配置。请注意，在使用此功能时，默认策略设置为Drop（除非已设置默认策略）。",
    ["autoconfigure_sites_note"] = "为了填充主机规则，自动配置功能要求在\"设置\">\"首选项\">\"应用程序\"下启用访问次数最多的站点。为获得最佳效果，应至少在收集足够数据前一天启用它。",
    ["continent"] = "洲",
    ["continent_rules"] = "洲规则",
    ["continent_rules_descr"] = "在此配置洲例外规则。",
    ["country"] = "国家",
    ["country_rules"] = "国家规则",
    ["country_rules_descr"] = "在此配置国家例外规则。",
    ["default"] = "默认",
    ["default_policy"] = "默认策略",
    ["default_policy_descr"] = "默认策略定义与池匹配的流量的默认操作。可以通过添加以下规则来配置默认策略的例外（即使用与默认策略相反的策略）。",
    ["drop"] = "Drop",
    ["edit_rule"] = "编辑规则",
    ["host"] = "主机",
    ["host_rules"] = "主机规则",
    ["host_rules_descr"] = "在此配置主机名（例如，用于TLS、DNS...协议）例外规则。",
    ["l7_category"] = "L7 应用程序类别",
    ["l7_category_rules"] = "L7 应用程序类别规则",
    ["l7_category_rules_descr"] = "在此配置流量类别例外规则。",
    ["l7_proto"] = "L7 协议",
    ["l7_proto_rules"] = "L7 协议规则",
    ["l7_proto_rules_descr"] = "在此配置L7应用程序协议例外规则。",
    ["marker"] = "操作",
    ["pass"] = "pass",
    ["remove_rules"] = "移除规则",
    ["risk"] = "风险",
    ["risk_rules"] = "风险规则",
    ["risk_rules_descr"] = "基于nDPI计算的流风险在这里配置策略。",
    ["rule"] = "规则",
    ["rules"] = "策略规则",
    ["traffic_policy_for"] = "流量策略： <a href='%{url}'>%{pool} <i class='fas fa-users'></i></a>",
    ["markers"] = {
      ["drop"] = "Drop",
      ["pass"] = "Pass",
      ["slow"] = "Slow Pass",
      ["slower"] = "Slower Pass",
    },
  },
  ["policy_presets"] = {
    ["business"] = "业务",
    ["business_description"] = "公司员工的预设",
    ["child"] = "儿童",
    ["child_description"] = "儿童预设。阻止社交网络并限制游戏时间",
    ["default"] = "默认",
    ["guests"] = "访客",
    ["guests_description"] = "访客预设。只允许基本导航",
    ["no_obfuscation"] = "无混淆",
    ["no_obfuscation_description"] = "防止流量混淆的预设",
    ["walled_garden"] = "围墙",
    ["walled_garden_description"] = "阻止 VPN 和社交网络",
  },
  ["pool_details"] = {
    ["host_pool"] = "主机池",
    ["host_pool_timeseries_enable_message"] = "可以从 <A HREF=\"%{url}\">%{icon_flask} 首选项</A> 启用主机池时间序列。需要几分钟时间才能看到第一个数据点。",
    ["no_available_data_for_host_pool_message"] = "主机池'%{pool_name}'没有可用数据。",
    ["pool_parameter_missing_message"] = "参数丢失（内部错误？）",
  },
  ["pool_stats"] = {
    ["host_pool_list"] = "主机池列表",
  },
  ["pools"] = {
    ["config_import"] = "导入配置",
    ["config_import_message"] = "<b>注意</b>：同名的池将被替换。",
    ["configuration"] = "检查配置",
    ["edit_host_pool"] = "编辑主机池",
    ["edit_pools"] = "编辑池",
    ["edit_recipients"] = "编辑收件人",
    ["family"] = "家族",
    ["id"] = "ID",
    ["insert_name"] = "例如：MyPool",
    ["members"] = "成员",
    ["name"] = "名称",
    ["name_requirements"] = "名称必须长于 1 个字符且不能为空。",
    ["no_recipients"] = "没有为上述池配置收件人。",
    ["not_assigned"] = "未分配",
    ["pool"] = "池",
    ["pools"] = "池",
    ["recipients"] = "收件人",
    ["remove_pool"] = "确认确定要删除 '%{pool}' 池吗？",
    ["select_config"] = "选择用户脚本配置",
    ["some_recipients"] = "为上述池配置收件人${recipients}。",
    ["unbounded_members"] = "全部未绑定",
    ["pool_name"] = {
      ["active_monitoring"] = "活动监控",
      ["flow"] = "流",
      ["host"] = "主机",
      ["host_pool"] = "主机池",
      ["interface"] = "接口",
      ["local_network"] = "本地网络",
      ["mac"] = "设备",
      ["snmp_device"] = "SNMP设备",
      ["system"] = "系统",
    },
    ["pool_names"] = {
      ["active_monitoring"] = "活动监控",
      ["all"] = "全部",
      ["flow"] = "流",
      ["host"] = "主机",
      ["host_pool"] = "主机池",
      ["interface"] = "接口",
      ["local_network"] = "本地网络",
      ["mac"] = "设备",
      ["snmp_device"] = "SNMP 设备",
      ["system"] = "系统",
    },
    ["pool_names_suffix"] = {
      ["active_monitoring"] = "活动监控池",
      ["interface"] = "接口池",
      ["local_network"] = "本地网络池",
      ["snmp"] = "SNMP池",
    },
  },
  ["port_details"] = {
    ["active_flows_for_host_and_port"] = "%{host}:%{port} 的活动流",
    ["active_flows_on_port"] = "端口 %{port} 上的活动流",
    ["active_flows_on_port_symbolic"] = "端口 %{port} 上的活动流 [%{symbolic_port}]",
    ["l4_port"] = "L4 端口",
  },
  ["ports_page"] = {
    ["client_ports"] = "客户端端口",
    ["num_contacted_ports"] = "唯一端口连接总数",
    ["num_contacted_ports_as_client"] = "远程通信端口（作为客户端）",
    ["num_host_contacted_ports_as_server"] = "本地通信端口（作为服务器）",
    ["server_ports"] = "服务器端口",
  },
  ["prefs"] = {
    ["active_local_host_cache_interval_description"] = "连续活动本地主机缓存转储之间的间隔。默认值：1 小时。",
    ["active_local_host_cache_interval_title"] = "活动本地主机缓存间隔",
    ["admin_group_description"] = "要作为管理员进行身份验证，用户必须属于的组名。",
    ["admin_group_title"] = "LDAP 管理员组",
    ["alert_entity"] = "警告实体",
    ["alerts_notifications"] = "端点",
    ["alerts_retention"] = "警告保留",
    ["alerts_settings"] = "警告设置",
    ["all"] = "全部",
    ["auth_methods_order"] = "验证方法检查顺序与上面列出的顺序相同，例如，如果同时启用了LDAP和local auth，则首先检查LDAP，如果验证失败，则接下来检查local auth。",
    ["authentication"] = "身份验证",
    ["authentication_duration"] = "身份验证持续时间",
    ["authentication_duration_descr"] = "经过身份验证的用户的会话终止并需要重新身份验证之前的时间。更改只会影响新创建的会话。",
    ["authentication_duration_title"] = "经过身份验证的会话持续时间",
    ["authentication_midnight_expiration_descr"] = "在午夜切换会话终止,强制所有经过身份验证的用户在午夜后重新进行身份验证。",
    ["authentication_midnight_expiration_title"] = "经过身份验证的会话在午夜终止",
    ["behavior_timeseries"] = "行为时间序列",
    ["behaviour"] = "行为分析",
    ["behaviour_analysis_learning_period_description"] = "配置行为流量分析的学习周期。",
    ["behaviour_analysis_learning_period_title"] = "学习周期",
    ["behaviour_analysis_status_during_learning_description"] = "当服务映射正在学习时新发现的服务的默认状态。",
    ["behaviour_analysis_status_during_learning_title"] = "学习期间的服务状态",
    ["behaviour_analysis_status_post_learning_description"] = "服务 Map 完成学习后新发现的服务的默认状态。",
    ["behaviour_analysis_status_post_learning_title"] = "学习后服务状态",
    ["bind_dn_description"] = "绑定 LDAP 服务器的专有名称。示例：",
    ["bind_dn_title"] = "LDAP 绑定 DN",
    ["bind_pwd_description"] = "绑定用于与 LDAP 服务器进行身份验证的密码。",
    ["bind_pwd_title"] = "LDAP 绑定认证密码",
    ["both"] = "两者",
    ["cache_settings"] = "缓存设置",
    ["captive_portal_url_description"] = "在成功的强制网络门户身份验证后,您的客户将被重定向到的 URL。",
    ["captive_portal_url_title"] = "重定向网址",
    ["client_x509_auth"] = "HTTPS 客户端认证",
    ["client_x509_auth_descr"] = "根据客户端的 X.509 证书切换客户端的身份验证。当客户端 X.509 通用名称 (CN) 等于 %{product} 用户之一时,客户端将被验证为%{product} 用户。更改此设置需要重新启动 %{product}。",
    ["client_x509_auth_title"] = "客户端 X.509 证书认证",
    ["could_not_contact_influxdb"] = "无法连接 InfluxDB 数据库：%{msg}",
    ["data_retention"] = "数据保留",
    ["data_retention_descr"] = "保留转储流量、历史时间序列和Top-talkers的天数。默认：30 天。",
    ["default_snmp_community_description"] = "当尝试访问尚未通过 SNMP 设备页面配置的选定本地主机的 SNMP MIB 时,将使用默认 SNMP Community。",
    ["default_snmp_community_title"] = "默认 SNMP Community",
    ["default_snmp_proto_version_description"] = "默认的SNMP协议版本。",
    ["default_snmp_proto_version_title"] = "默认 SNMP 版本",
    ["default_snmp_timeout_description"] = "SNMP 代理响应超时（秒）。",
    ["default_snmp_timeout_title"] = "SNMP 代理响应超时",
    ["device_identifier"] = "设备标识符",
    ["device_identifier_descr"] = "通常设备通过 MAC 地址来标识。但是,如果设备在路由器后面,则需要 IP 标识。",
    ["disable_alerts_generation_description"] = "向内部数据库和通知收件人发出警报。",
    ["disable_alerts_generation_title"] = "发出警报",
    ["dns"] = "DNS",
    ["dns_service"] = "服务",
    ["dump_settings"] = "流量转储设置",
    ["duration"] = "持续时间",
    ["dynamic_interfaces_creation_description"] = "ntopng可以使用一个标准来分解传入的流量。当选择了分解条件时，ntopng将使用该条件值将传入的流量转移到动态创建的接口。例如，当选择'VLAN Id'标准时，将为观察到的每个VLAN Id创建一个动态接口，传入的流量将根据VLAN Id值转移到一个动态接口。",
    ["dynamic_interfaces_creation_note_0"] = "'VLAN Id' 分解支持物理接口以及通过 ZMQ 接收的流。其他分解标准仅支持 ZMQ 流,对物理接口无效。",
    ["dynamic_interfaces_creation_note_1"] = "标准更改不会影响现有接口。<b>需要重启 ntopng 才能使更改生效</b>。",
    ["dynamic_interfaces_creation_note_2"] = "在非 sflow 设备上使用‘入口接口’标准时,%%​​INPUT_SNMP 必须出现在 nprobe 模板中。",
    ["dynamic_interfaces_creation_note_3"] = "'接口' 分解标准将任何流添加到两个动态接口。%%INPUT_SNMP 的值用于选择第一个动态接口,而%%OUTPUT_SNMP 的值用于选择第二个。",
    ["dynamic_interfaces_creation_note_4"] = "通过自定义流量分解选项禁用动态流量分解（将其设置为\"空\"）,也可以使用类似 BPF 的过滤器进行自定义流量分解。",
    ["dynamic_interfaces_creation_title"] = "动态流量分解",
    ["dynamic_network_interfaces"] = "动态网络接口",
    ["elasticsearch_notification"] = "Elasticsearch 通知",
    ["elasticsearch_send_error"] = "无法连接到 Elasticsearch。[响应代码：%{code}][响应：%{resp}]。",
    ["elasticsearch_sent_successfully"] = "Elasticsearch 连接正常。",
    ["elasticsearch_unable_to_send_alerts"] = "无法向 Elasticsearch 发送警告",
    ["elasticsearch_unsupported_version"] = "不支持的 Elasticsearch 版本 %{version}。至少需要版本 7。",
    ["elephant_flow_local_to_remote_bytes_description"] = "流在被视为大数据流之前可以上传的数据量。",
    ["elephant_flow_local_to_remote_bytes_title"] = "大数据流量阈值（本地到远程）",
    ["elephant_flow_remote_to_local_bytes_description"] = "流在被视为大数据流之前可以下载的数据量。",
    ["elephant_flow_remote_to_local_bytes_title"] = "大数据流量阈值（远程到本地）",
    ["email_notification"] = "邮件通知",
    ["email_notification_password_description"] = "用于身份验证的 SMTP 服务器密码（可选）。",
    ["email_notification_password_title"] = "密码",
    ["email_notification_recipient_description"] = "收件人邮箱地址。",
    ["email_notification_recipient_title"] = "收件人",
    ["email_notification_sender_description"] = "发件人电子邮件地址。",
    ["email_notification_sender_title"] = "发件人",
    ["email_notification_server_description"] = "发送邮件的 SMTP 服务器。例如,如果您使用 GMail,则需要指定 smtps://smtp.gmail.com:465。请注意,GMail 还需要 <a href ='https://myaccount.google.com/lesssecureapps'>访问不太安全的应用程序</a>被启用。",
    ["email_notification_server_title"] = "SMTP 服务器",
    ["email_notification_username_description"] = "用于身份验证的 SMTP 服务器用户名（可选）。",
    ["email_notification_username_title"] = "用户名",
    ["email_send_error"] = "电子邮件发送错误：%{msg}。请参阅<a href='%{url}'>在线文档</a>。",
    ["email_sent_successfully"] = "邮件发送成功",
    ["errors"] = "错误",
    ["errors_and_warnings"] = "错误和警告",
    ["ewma_alpha_percent_description"] = "指数加权移动平均 (EWMA) alpha 系数,以百分比表示。用于估计自治系统往返时间。",
    ["ewma_alpha_percent_title"] = "EWMA alpha 系数（百分比）",
    ["expert_view"] = "专家视图",
    ["featured_dns_description"] = "以下是安全DNS服务器的列表。您可以将它们用作全局DNS服务器或作为儿童安全提供商。",
    ["featured_dns_title"] = "特色DNS服务器",
    ["flow_interface"] = "接口",
    ["flow_max_idle_description"] = "流被视为空闲的不活动时间。仅对数据包接口有效。对于 ZMQ 接口,超时是自动确定的。默认值：1 分钟。",
    ["flow_max_idle_title"] = "流空闲超时",
    ["flows_dump"] = "流转储",
    ["flows_dump_limited_days"] = "在此 ntopng 版本中,Flow 转储限制为 %{days} 天。升级到企业版以取消此限制。",
    ["global_dns_description"] = "用于将客户端DNS请求重定向到非儿童安全 主机池的DNS服务器。如果为空,则禁用重定向并使用默认 DNS。",
    ["global_dns_title"] = "全局主DNS",
    ["gui"] = "用户界面",
    ["host_mask"] = "屏蔽主机 IP 地址",
    ["http_auth"] = "HTTP 认证",
    ["http_auth_server_description"] = "用于验证用户身份的HTTP服务器的 URL。",
    ["http_auth_server_title"] = "HTTP 服务器",
    ["idle_timeout_settings"] = "空闲超时设置",
    ["iec60870_learning_period_description"] = "配置IEC60870流量分析的学习周期。默认：6小时。",
    ["iec60870_learning_period_title"] = "IEC60870 学习周期",
    ["ignored_interfaces_description"] = "当选择\"入口接口\"作为分解标准时，不要向动态接口添加任何在此列表中具有%%INPUT_SNMP的流。选择\"接口\"分解时，不要添加到第一个 (第二个) 动态接口在此列表中有其%%INPUT_SNMP (%%OUTPUT_SNMP) 的任何流。指定一个或多个逗号分隔的 SNMP 接口 ID。更改需要 %{product} 重新启动。",
    ["ignored_interfaces_title"] = "忽略的接口",
    ["incompatible_influxdb_version_found"] = "不兼容的InfluxDB 版本：需要 \"%{required}\", found \"%{found}\"。请从<a href='%{url}'>官方安装更新版本网站</a>。",
    ["influxdb_auth_description"] = "启用 InfluxDB 身份验证。",
    ["influxdb_auth_title"] = "InfluxDB 身份验证",
    ["influxdb_create_error"] = "无法创建\"%{db}\" 数据库：%{msg}",
    ["influxdb_dbname_description"] = "用于时间序列存储的数据库。不会迁移现有数据。",
    ["influxdb_dbname_title"] = "InfluxDB 数据库",
    ["influxdb_password_description"] = "使用 InfluxDB 数据库进行身份验证的密码。",
    ["influxdb_query_timeout_description"] = "InfluxDB 查询的最长持续时间（以秒为单位）。超过此持续时间的查询将被中止。",
    ["influxdb_query_timeout_title"] = "查询超时",
    ["influxdb_retention_error"] = "无法在数据库 \"%{db}\" 上设置保留策略：%{msg}",
    ["influxdb_storage_note"] = "<b>注意</b>：数据将按照 %{interval} 的间隔清除,这对应于 InfluxDB <a href='%{url}'>shard duration</a> .",
    ["influxdb_url_description"] = "指向正在运行的 InfluxDB 实例的 URL。",
    ["influxdb_url_title"] = "InfluxDB URL",
    ["influxdb_username_description"] = "使用 InfluxDB 数据库进行身份验证的用户名。",
    ["ingress_egress_flow_interface"] = "入口/出口接口",
    ["ingress_flow_interface"] = "仅入接口",
    ["ingress_vrf_id"] = "入口 VRF Id",
    ["interfaces_timeseries"] = "接口时间序列",
    ["internal_log"] = "内部日志",
    ["is_influxdb_running"] = "InfluxDB 正在运行吗？",
    ["l2_devices_timeseries"] = "设备时间序列",
    ["last_seen"] = "上次发现",
    ["ldap_authentication"] = "LDAP 认证",
    ["ldap_server_address_description"] = "LDAP 服务器的 IP 地址和端口（例如 ldaps://localhost:636）。默认值：\"ldap://localhost:389\".",
    ["ldap_server_address_title"] = "LDAP 服务器地址",
    ["learning_period_description"] = "学习周期服务所花费的时间,以小时为单位。默认值：1 小时。",
    ["learning_period_title"] = "服务学习期",
    ["license"] = "许可证",
    ["local_auth"] = "本地认证",
    ["local_host_cache_duration_description"] = "从缓存中删除缓存的本地主机的时间。默认值：1 小时。",
    ["local_host_cache_duration_title"] = "本地主机缓存持续时间",
    ["local_host_mask"] = "屏蔽本地主机",
    ["local_host_max_idle_description"] = "本地主机被视为空闲（秒）之前的不活动时间。空闲的本地主机被转储到缓存中,以便它们的计数器可以在它们再次变为活动状态时恢复。计数器包括但不限于, 数据包和字节总数和每个第 7 层应用程序。默认值：5 分钟。",
    ["local_host_max_idle_title"] = "本地主机空闲超时",
    ["local_hosts_cache_settings"] = "本地主机缓存设置",
    ["local_hosts_timeseries"] = "本地主机时间序列",
    ["logging"] = "日志",
    ["longlived_flow_duration_description"] = "流被视为长期流的最短持续时间。",
    ["longlived_flow_duration_title"] = "长期流持续时间",
    ["max_entity_alerts_description"] = "每个系列（主机、网络、接口等）警告的最大数量。警告会定期检查,如果实际警告数量超过最大数量,将删除最早的警告以保留警告数量等于最大配置数量的 80%。",
    ["max_entity_alerts_title"] = "最大警告数",
    ["max_num_bytes_per_tiny_flow_description"] = "流必须被视为微小流的最大字节数。默认值：64。",
    ["max_num_bytes_per_tiny_flow_title"] = "每个微小流的最大字节数",
    ["max_num_packets_per_tiny_flow_description"] = "一个流必须被认为是一个微小流的最大数据包数。默认值：3。",
    ["max_num_packets_per_tiny_flow_title"] = "每个微小流的最大数据包数",
    ["max_num_secs_before_delete_alert_description"] = "警告被删除前的最大天数。所有早于该时间段的警告将在启动时和由 ntopng 定期自动删除",
    ["max_num_secs_before_delete_alert_title"] = "删除前的最大天数",
    ["max_ui_strlen_description"] = "缩短长度超过指定最大字符数的字符串。",
    ["max_ui_strlen_title"] = "最大显示字符串长度",
    ["mgmt_acl_description"] = "一个以逗号分隔的 IPv4 网络列表。必须明确允许或禁止每个网络,分别使用前导 \"+\" 或 \"-\" 符号。最长匹配原则用于重叠网络。示例：\"+127.0.0.0/8\" 使接口只能从本地主机访问,\"+0.0.0.0/0,-192.168.0.0/16\" 使除192.168.0.0/16之外的所有人都可以访问接口。必须重新启动%{product}更改才能生效。",
    ["mgmt_acl_title"] = "访问控制列表",
    ["misc"] = "Misc",
    ["multiple_ldap_account_type_description"] = "选择你的账户类型",
    ["multiple_ldap_account_type_title"] = "LDAP 账户类型",
    ["multiple_timeseries_database_description"] = "用于存储和检索时间序列数据的驱动程序。",
    ["multiple_timeseries_database_title"] = "时间序列驱动",
    ["mysql"] = "MySQL",
    ["n2disk_license_description"] = "连续流量记录需要 n2disk 许可证,请在 <a href=\"%{purchase_url}\">e-shop</a> 购买。如果您是非营利组织、研究或教育机构机构请读<a href=\"%{universities_url}\">this</a>。",
    ["n2disk_license_description_enterprise_l"] = "ntopng Enterprise L 已经解锁了 n2disk 1G,因此在这种情况下不需要额外的许可证。",
    ["n2disk_license_systemid"] = "SystemID: %{systemid}",
    ["n2disk_license_title"] = "n2disk 许可证密钥",
    ["n2disk_license_version"] = "n2disk 版本：%{version}",
    ["n2n_supernode_description"] = "<a class='ntopng-external-link' href=\"%{url}\">n2n的地址<i class=\"fas fa-external-link-alt\"></i></a>超级节点。设置您自己的超级节点以获得完全私有的远程访问。<br><b>注意</b>：此更改仅在远程协助重启后才会生效。",
    ["n2n_supernode_title"] = "超级节点",
    ["network_discovery"] = "网络发现",
    ["network_discovery_interval_description"] = "连续发现网络之间的间隔。默认值：15 分钟。",
    ["network_discovery_interval_title"] = "活动网络发现间隔",
    ["network_interfaces"] = "网络接口",
    ["no_host_mask"] = "不要屏蔽主机",
    ["non_local_host_max_idle_description"] = "远程主机被视为空闲后的不活动时间。默认值：1 分钟。",
    ["non_local_host_max_idle_title"] = "远程主机空闲超时",
    ["none"] = "空",
    ["note_timeseries_resolution_disabled"] = "<b>注意</b>：这是禁用的,因为 \"%{pref}\" 未设置为默认值。",
    ["note_update_frequency_disabled"] = "<b>注意</b>：由于 \"%{pref}\" 请求,此功能已禁用。",
    ["observation_points"] = "观察 Points",
    ["other_timeseries"] = "其他时间序列",
    ["per_category"] = "每个类别",
    ["per_protocol"] = "每个应用程序",
    ["policy_target_type"] = "策略目标类型",
    ["policy_target_type_description"] = "指定您希望应用的策略目标类型。",
    ["posix"] = "posix",
    ["preferences"] = "首选项",
    ["primary_dns"] = "主DNS",
    ["probe_ip_address"] = "探针IP",
    ["probe_ip_and_ingress_iface_idx"] = "探针IP + 入口接口",
    ["protocols"] = "应用程序",
    ["radius_admin_group_description"] = "当 RADIUS 返回的 Filter-Id Attribute-Value 的值与此字段中指定的值匹配时,用户将被验证为管理员。如果不匹配,则用户被验证为非高级用户。",
    ["radius_admin_group_title"] = "RADIUS 管理员 Filter-Id",
    ["radius_auth"] = "RADIUS 认证",
    ["radius_secret_descroption"] = "用于与 RADIUS 服务器进行身份验证的共享密钥。",
    ["radius_secret_title"] = "RADIUS Secret",
    ["radius_server_description"] = "RADIUS 服务器的主机和端口（例如 %{example}）。",
    ["radius_server_title"] = "RADIUS 服务器地址",
    ["recording"] = "流量主机",
    ["remote_host_mask"] = "屏蔽远程主机",
    ["report"] = "报告",
    ["restart_needed"] = "请重新启动 %{product} 以应用更改",
    ["runtime_prefs"] = "运行时首选项",
    ["safe_search_dns_description"] = "用于执行<a href='%{url}'>儿童安全</a> 查询的DNS服务器。<br>可以为主机池中的各个主机池启用儿童安全配置页面。默认值：208.67.222.123。",
    ["safe_search_dns_title"] = "儿童安全 DNS",
    ["samaccount"] = "sAMAccount",
    ["search_path_description"] = "用于搜索用户的根路径。",
    ["search_path_title"] = "LDAP搜索路径",
    ["search_preferences"] = "搜索首选项",
    ["secondary_dns"] = "第二DNS",
    ["secondary_dns_description"] = "当全局主DNS服务器不可用时使用的后备DNS服务器。",
    ["secondary_dns_title"] = "全局第二DNS",
    ["security_alerts"] = "安全警告",
    ["send_test_elasticsearch"] = "测试连接",
    ["send_test_mail"] = "发送测试邮件",
    ["send_test_slack"] = "发送测试消息",
    ["send_test_webhook"] = "发送测试消息",
    ["sender_username_description"] = "设置slack通知发件人的用户名",
    ["sender_username_title"] = "通知发件人用户名",
    ["simple_view"] = "简单视图",
    ["slack_channel"] = "Slack 通道",
    ["slack_channel_names"] = "通道名称",
    ["slack_channel_names_descr"] = "配置发送每个警告的 Slack 通道。通道必须存在才能正确显示警告。",
    ["slack_integration"] = "Slack集成",
    ["slack_notification_severity_preference_description"] = "错误（仅错误）、错误和警告（错误和警告，无信息）, 所有 (各种警告都会被通知).",
    ["slack_notification_severity_preference_title"] = "通知严重性",
    ["slack_send_error"] = "Slack 消息发送错误。",
    ["slack_sent_successfully"] = "一条 Slack 消息已发送到 #%{channel} 通道。",
    ["slack_webhook_description"] = "将你的通知发送到这个 slack URL",
    ["slack_webhook_title"] = "webhook通知",
    ["snmp"] = "SNMP",
    ["snmp_alerts"] = "SNMP 警告",
    ["snmp_port_load_threshold_description"] = "设置阈值（作为端口速度的百分比）以在端口负载超过指定限制（平均 5 分钟）时触发警告。",
    ["snmp_port_load_threshold_title"] = "接口负载阈值警告",
    ["snmp_timeseries_config_link"] = "可以从 <a href=\"%{url}\">SNMP 部分</a>启用 SNMP 时间序列。",
    ["stats_reset"] = "统计重置",
    ["status_alerts"] = "状态警告",
    ["successfully_connected_influxdb"] = "在 InfluxDB(%{version}) 上成功初始化数据库 \"%{db}\"",
    ["syslog_alert_format_description"] = "选择系统日志警告输出格式。",
    ["syslog_alert_format_json"] = "JSON",
    ["syslog_alert_format_plaintext"] = "纯文本",
    ["syslog_alert_format_title"] = "系统日志警告格式",
    ["syslog_notification"] = "系统日志通知",
    ["syslog_sent_successfully"] = "一条消息已被记录",
    ["system_probes_timeseries"] = "探针时间序列",
    ["telemetry"] = "遥测",
    ["telemetry_contribute"] = "贡献",
    ["telemetry_do_not_contribute"] = "不贡献",
    ["telemetry_email_description"] = "如果您希望在检测到遥测数据异常时 ntop 团队联系您,请提供电子邮件地址。如果您不提供此地址,ntop 将无法识别您的身份。",
    ["telemetry_email_title"] = "通信方式",
    ["timeseries"] = "时间序列",
    ["timeseries_database"] = "Timeseries 数据库",
    ["timeseries_resolution_resolution_description_2"] = "本地主机的连续时间序列数据点之间的间隔",
    ["timeseries_resolution_resolution_title"] = "时间序列分辨率",
    ["toggle_access_log_description"] = "在数据转储目录中切换HTTP访问日志的创建。设置将在下次 %{product} 启动时生效。",
    ["toggle_access_log_title"] = "启用HTTP访问日志",
    ["toggle_active_local_host_cache_enabled_description"] = "切换活动本地主机缓存条目的创建。定期缓存活动本地主机可用于保护主机计数器免受故障（例如,断电）。这对于很少空闲的本地主机尤其重要因为它保证他们的计数器将在指定的时间间隔后被缓存。",
    ["toggle_active_local_host_cache_enabled_title"] = "活动本地主机缓存",
    ["toggle_alert_syslog_description"] = "切换警告记录到系统日志。",
    ["toggle_alert_syslog_title"] = "切换系统日志通知",
    ["toggle_alerts_notifications_description"] = "切换到第三方端点的警告报告。",
    ["toggle_alerts_notifications_title"] = "对第三方端点的警告",
    ["toggle_asn_rrds_description"] = "为自治系统切换字节和应用程序时间序列的创建。",
    ["toggle_asn_rrds_title"] = "自治系统",
    ["toggle_autologout_description"] = "切换已过期会话的web界面用户的自动登出",
    ["toggle_autologout_title"] = "自动登出",
    ["toggle_autoupdates_description"] = "切换夜间自动 %{product} 更新。请注意,成功更新后 %{product} 会自动重启。",
    ["toggle_autoupdates_title"] = "自动更新",
    ["toggle_behaviour_analysis_description"] = "启用主机和接口行为流量分析。需要重启才能使此首选项生效。",
    ["toggle_behaviour_analysis_title"] = "开启流量行为分析",
    ["toggle_captive_portal_description"] = "启用网络捕获门户以验证网络用户。",
    ["toggle_captive_portal_title"] = "捕获门户",
    ["toggle_country_rrds_description"] = "切换为国家/地区创建字节时间序列。",
    ["toggle_country_rrds_title"] = "国家",
    ["toggle_dst_with_post_nat_dst_description"] = " IPv4 目标地址 (%%IPV4_DST_ADDR) 和端口 (%%L4_DST_PORT) 替换为它们的 post-nat 值 (%%POST_NAT_DST_IPV4_ADDR 和%%POST_NAPT_DST_TRANSPORT_PORT)。",
    ["toggle_dst_with_post_nat_dst_title"] = "使用 Post-Nat 目标 IPv4 地址和端口",
    ["toggle_elasticsearch_notification_description"] = "将警告通知切换到 Elasticsearch。",
    ["toggle_elasticsearch_notification_title"] = "切换 Elasticsearch 通知",
    ["toggle_elephant_flows_alerts_description"] = "切换检测到大数据流时生成的警告。这对于检测不需要的行为（例如数据泄露）很有用。",
    ["toggle_elephant_flows_alerts_title"] = "大数据流警告",
    ["toggle_email_notification_description"] = "切换通过电子邮件警告通知。",
    ["toggle_email_notification_title"] = "切换电子邮件通知",
    ["toggle_emit_flow_alerts_description"] = "您可以禁用流量警告的生成以节省 CPU 周期。在这种情况下,分数将被更新但不会触发流量警告。",
    ["toggle_emit_flow_alerts_title"] = "发出流量警告",
    ["toggle_emit_host_alerts_description"] = "您可以禁用主机警告的生成以节省 CPU 周期。在这种情况下,分数将被更新但不会触发主机警告。",
    ["toggle_emit_host_alerts_title"] = "发出主机警告",
    ["toggle_enable_runtime_flows_dump_description"] = "切换到配置数据库的流转储。",
    ["toggle_enable_runtime_flows_dump_title"] = "流量转储",
    ["toggle_flow_rrds_description"] = "切换通过 ZMQ 接收的远程设备每个端口的字节时间序列的创建（例如 sFlow/NetFlow/SNMP）。<br>对于非 sFlow 设备,%%INPUT_SNMP 和%%OUTPUT_SNMP 必须出现在 nprobe 模板中。",
    ["toggle_flow_rrds_title"] = "流探针",
    ["toggle_host_mask_description"] = "出于隐私原因,可能需要屏蔽主机 IP 地址。例如,如果您是 ISP,您不应该知道哪些本地地址正在访问远程主机。",
    ["toggle_host_mask_title"] = "屏蔽主机 IP 地址",
    ["toggle_host_pools_log_description"] = "切换与主机池相关的日志的创建。日志包括主机与其主机池的关联和解除关联。",
    ["toggle_host_pools_log_title"] = "启用主机池事件日志",
    ["toggle_host_tskey_description"] = "用于本地广播域中主机的数据和时间序列转储的密钥。在DHCP网络中,通常需要选择\"MAC 地址\"。",
    ["toggle_host_tskey_title"] = "本地广播域主机标识符",
    ["toggle_http_auth"] = "切换HTTP身份验证",
    ["toggle_http_auth_descr"] = "通过HTTP身份验证器切换 gui 身份验证。",
    ["toggle_ids_alert_description"] = "切换入侵检测系统生成的警告。通过系统日志查看 Suricata 集成。",
    ["toggle_ids_alert_title"] = "IDS 警告",
    ["toggle_informative_captive_portal_description"] = "不要对用户进行身份验证。在让用户访问互联网之前只显示一个信息页面。",
    ["toggle_informative_captive_portal_title"] = "信息",
    ["toggle_internals_rrds_description"] = "切换<a href=\"%{url}\">内部</a>时间序列的创建,例如哈希表条目、周期性活动持续时间和写入的时间序列点数、插件持续时间和通话次数。",
    ["toggle_internals_rrds_title"] = "内部",
    ["toggle_ldap_anonymous_bind_description"] = "启用匿名绑定。",
    ["toggle_ldap_anonymous_bind_title"] = "LDAP 匿名绑定",
    ["toggle_ldap_auth"] = "切换 LDAP 身份验证",
    ["toggle_ldap_auth_descr"] = "通过 LDAP 切换 gui 身份验证。",
    ["toggle_ldap_referrals_description"] = "指定客户端是否应自动遵循 LDAP 服务器返回的引用。",
    ["toggle_ldap_referrals_title"] = "关注推荐",
    ["toggle_local_auth"] = "切换本地身份验证",
    ["toggle_local_auth_descr"] = "根据 %{product} 定义的用户切换 gui 身份验证。",
    ["toggle_local_host_cache_enabled_description"] = "为空闲的本地主机切换缓存条目的创建。缓存的本地主机计数器在检测到额外的主机流量时自动恢复到以前的值。",
    ["toggle_local_host_cache_enabled_title"] = "空闲本地主机缓存",
    ["toggle_local_hosts_one_way_ts_description"] = "为单向流量切换时间序列的创建。单向流量通常由远程主机执行探针生成,因此建议禁用此功能。",
    ["toggle_local_hosts_one_way_ts_title"] = "单向流量时间序列",
    ["toggle_local_hosts_ts_creation_description"] = "启用完整的主机时间序列创建（完整）,将其限制为字节和分数（轻）,或将其关闭。",
    ["toggle_local_hosts_ts_creation_title"] = "主机时间序列",
    ["toggle_log_to_file_description"] = "在数据转储目录中切换 ntopng.log 的创建,而不是使用 syslog。设置将在下一次 %{product} 启动时生效。",
    ["toggle_log_to_file_title"] = "启用跟踪日志",
    ["toggle_logging_level_description"] = "选择运行时日志级别。",
    ["toggle_logging_level_title"] = "日志级别",
    ["toggle_longlived_flows_alerts_description"] = "切换检测到长期流时生成的警告。这对于检测不需要的行为（例如数据泄露）很有用。",
    ["toggle_longlived_flows_alerts_title"] = "长期流警告",
    ["toggle_midnight_stats_reset_description"] = "在午夜切换自动重置统计信息。这包括主机和 MAC 地址统计信息。",
    ["toggle_midnight_stats_reset_title"] = "午夜统计重置",
    ["toggle_mysql_check_open_files_limit_description"] = "由于 MySQL open_files_limit 的定期检查,启用 MySQL 警告生成。<br>open_files_limit 检查可用于检测何时打开的 MySQL 文件数量很高,这可能导致数据库填写错误。",
    ["toggle_mysql_check_open_files_limit_title"] = "启用 MySQL 警告",
    ["toggle_ndpi_flows_rrds_description"] = "为每个第 7 层应用程序的总流数切换时间序列的创建。",
    ["toggle_ndpi_flows_rrds_title"] = "第 7 层总流量",
    ["toggle_ndpi_timeseries_creation_description"] = "切换第 7 层应用程序时间序列的创建。为每个应用程序创建时间序列需要更多的磁盘空间和额外的 I/O,通常不需要。",
    ["toggle_ndpi_timeseries_creation_title"] = "第 7 层应用程序",
    ["toggle_network_discovery_description"] = "使用多种技术（包括 ARP 扫描、MDNS和 SSDP）切换网络设备的定期发现。<p><b><b>注意</b>：</b>发现可以<u >仅</u>在物理接口（即没有 ZMQ）上启用,我们可以在其中向网络发送/接收流量（即只读接口,例如连接到端口镜像的那些接口将不起作用）。",
    ["toggle_network_discovery_title"] = "活动网络发现",
    ["toggle_os_rrds_description"] = "为操作系统创建字节时间序列。",
    ["toggle_os_rrds_title"] = "操作系统",
    ["toggle_pools_rrds_description"] = "为定义的主机池创建字节和应用程序时间序列。",
    ["toggle_pools_rrds_title"] = "主机池",
    ["toggle_radius_auth"] = "切换 RADIUS 身份验证",
    ["toggle_radius_auth_descr"] = "通过 RADIUS 服务器切换 gui 身份验证。",
    ["toggle_send_telemetry_data_description"] = "通过向<a href=\"%{ntop_org}\">ntop发送加密的<a href=\"%{url}\">匿名遥测数据</a>来为项目做出贡献.org</a>。数据仅涉及 %{product} 的状态（例如,正常运行时间、状态、崩溃报告）。任何有关受监控流量的信息都不会发送。",
    ["toggle_send_telemetry_data_title"] = "遥测数据",
    ["toggle_shaping_directions_description"] = "启用此选项可以为入口和出口流量设置不同的shaping策略。",
    ["toggle_shaping_directions_title"] = "切片Shaping方向",
    ["toggle_slack_notification_description"] = "通过 slack 切换警告通知。请参阅<a href='%{url}'>处</a>的集成说明。",
    ["toggle_slack_notification_title"] = "切换<a href=\"%{url}\">Slack</a>通知",
    ["toggle_snmp_alerts_port_duplexstatus_change_description"] = "当接口双工状态改变时触发警告",
    ["toggle_snmp_alerts_port_duplexstatus_change_title"] = "接口双工状态变化警告",
    ["toggle_snmp_alerts_port_errors_description"] = "当接口上的丢弃或错误计数器增加时生成的切换警告",
    ["toggle_snmp_alerts_port_errors_title"] = "接口丢弃/错误警告",
    ["toggle_snmp_alerts_port_status_change_description"] = "切换接口操作状态改变时生成的警告",
    ["toggle_snmp_alerts_port_status_change_title"] = "接口操作状态变化警告",
    ["toggle_snmp_debug_description"] = "切换 SNMP 设备的调试。启用此选项会将额外信息打印到 ntopng 日志中。",
    ["toggle_snmp_debug_title"] = "SNMP 调试",
    ["toggle_snmp_port_admin_status_description"] = "切换接口管理员状态。切换是即时的,但显示的实际状态会在下一次轮询期间刷新。",
    ["toggle_snmp_port_admin_status_title"] = "接口管理员状态",
    ["toggle_snmp_rrds_description"] = "切换为 SNMP 设备的每个端口创建字节时间序列。将为每个设备端口创建一个包含入口/出口字节的时间序列。",
    ["toggle_snmp_rrds_title"] = "SNMP 设备时间序列",
    ["toggle_src_with_post_nat_src_description"] = "将 IPv4 源地址 (%%IPV4_SRC_ADDR) 和端口 (%%L4_SRC_PORT) 替换为它们的 post-nat 值（%%POST_NAT_SRC_IPV4_ADDR 和%%POST_NAPT_SRC_TRANSPORT_PORT）。",
    ["toggle_src_with_post_nat_src_title"] = "使用 Post-Nat 源 IPv4 地址和端口",
    ["toggle_system_probes_timeseries_description"] = "切换为系统探针（例如活动监控和 Redis 监控）创建时间序列。",
    ["toggle_system_probes_timeseries_title"] = "探针",
    ["toggle_theme_description"] = "在主题之间切换。",
    ["toggle_theme_title"] = "主题",
    ["toggle_thpt_content_description"] = "选择要在流量报告中显示的吞吐量单位。",
    ["toggle_thpt_content_title"] = "吞吐量单位",
    ["toggle_tiny_flows_dump_description"] = "切换微小流的转储,即具有少量数据包或字节的流。减少数据库中的流基数,加速填写和搜索。调整微小流有助于限制流基数,同时不降低可见性转储的信息。",
    ["toggle_tiny_flows_dump_title"] = "微小流转储",
    ["toggle_top_sites_description"] = "为本地主机切换访问最多的网站的创建。Top站点是使用<a href=\"%{url}\">启发式</a>创建的,维护不超过 20 个站点每个本地主机。启发式完全在内存中运行,不需要与磁盘进行任何交互。每 5 分钟清除一次Top站点。可以通过定期轮询 JSON 为每个本地主机创建访问最多的网站的历史档案<i class=\"fas fa-download fa-sm\"></i> 下载连接可从主机详情页面访问。这些站点的应用协议有：TLS、QUIC、HTTP 和 DNS",
    ["toggle_top_sites_title"] = "访问最多的网站",
    ["toggle_traffic_rrd_creation_description"] = "切换创建字节和数据包时间序列。",
    ["toggle_traffic_rrd_creation_title"] = "流量",
    ["toggle_users_rrds_description"] = "为定义的用户切换创建字节和应用程序时间序列。",
    ["toggle_users_rrds_title"] = "用户",
    ["toggle_vlan_rrds_description"] = "切换创建 VLAN 的字节和应用程序时间序列。",
    ["toggle_vlan_rrds_title"] = "VLAN",
    ["toggle_webhook_notification_description"] = "通过 webhook (HTTP) 切换警告通知。",
    ["toggle_webhook_notification_title"] = "切换 Webhook 通知",
    ["topk_heuristic_precision_description"] = "在聚合历史上排名靠前的主机、国家等时使用启发式方法来构建流量报告。在构建长期报告时很有用。",
    ["topk_heuristic_precision_title"] = "Top-K 启发式",
    ["traffic_behaviour"] = "流量行为",
    ["traffic_bridging"] = "流量桥接",
    ["traffic_shaping"] = "流量整形(Shaping)",
    ["ts_resolution_note2"] = "降低分辨率会导致现有数据出现缺口。建议<a target=_blank href=\"%{url}\">删除数据库</a> %{external_icon}。 ",
    ["updates"] = "更新",
    ["user_authentication"] = "用户身份验证",
    ["user_group_description"] = "作为非高级用户进行身份验证用户必须属于的组名。",
    ["user_group_title"] = "LDAP 用户组",
    ["vlan"] = "VLAN ID",
    ["web_user_interface"] = "Web 用户界面",
    ["webhook_notification"] = "Webhook 通知",
    ["webhook_notification_severity_preference_description"] = "错误（仅限错误）、错误和警告（错误和警告，无信息）, 所有 (各种警告都会被通知).",
    ["webhook_notification_severity_preference_title"] = "通知严重性",
    ["webhook_password_description"] = "HTTP 认证时使用的密码（可选）。",
    ["webhook_send_error"] = "向 webhook 发送消息时出错。",
    ["webhook_sent_successfully"] = "消息已发送到 webhook",
    ["webhook_sharedsecret_description"] = "设置一个（用户定义的）secret 由接收者验证（可选）。",
    ["webhook_sharedsecret_title"] = "共享的 Secret",
    ["webhook_url_description"] = "以 JSON 格式向此 URL 发送通知。",
    ["webhook_url_title"] = "通知网址",
    ["webhook_username_description"] = "HTTP 身份验证时使用的用户名（可选）。",
    ["wifi"] = "无线",
    ["zmq_interfaces"] = "ZMQ 接口",
  },
  ["processes_stats"] = {
    ["active_processes_title"] = "活动进程：实时视图",
    ["active_since"] = "活动自",
    ["flows_count"] = "流量计数",
    ["legend"] = "Legend",
    ["lines"] = "行",
    ["missing_pid_name_message"] = "缺少pid名称",
    ["no_traffic_detected"] = "此进程未检测到流量,流已过期或进程已终止。",
    ["processes_timeline_title"] = "进程时间线",
    ["stack"] = "堆栈",
    ["timeline"] = "时间线",
    ["top_hosts"] = "Top主机",
    ["top_hosts_traffic"] = "Top主机流量",
    ["traffic_rcvd"] = "流量接收",
    ["traffic_sent"] = "流量发送",
    ["type"] = "类型",
  },
  ["queue"] = {
    ["activeFlowsToDump"] = "活动流转储",
    ["hookFlowEnd"] = "flowEnd流用户脚本Hook",
    ["hookPeriodicUpdate"] = "periodicUpdate 流用户脚本Hook",
    ["hookProtocolDetected"] = "protocolDetected流用户脚本Hook",
    ["idleFlowsToDump"] = "空闲流转储",
  },
  ["queue_description"] = {
    ["activeFlowsToDump"] = "等待转储的活动流的队列",
    ["hookFlowEnd"] = "空闲流排队等待执行它们的 flowEnd 用户脚本Hook",
    ["hookPeriodicUpdate"] = "活动流正在等待他们的periodicUpdate 用户脚本Hook",
    ["hookProtocolDetected"] = "新流正在等待执行他们的 protocolDetected 用户脚本Hook",
    ["idleFlowsToDump"] = "等待转储空闲流的队列",
  },
  ["report"] = {
    ["application_breakdown"] = "应用程序细分",
    ["applications"] = "应用程序",
    ["approximated_data"] = "在选择非常大的时间间隔时，可以进行轻微的数据近似。",
    ["begin_date_time"] = "开始日期/时间",
    ["date"] = "%{year}-%{month}-%{year}",
    ["end_date_time"] = "结束日期/时间",
    ["filter_report"] = "过滤报告",
    ["generate"] = "生成",
    ["invalid_begin"] = "无效的开始",
    ["invalid_begin_date"] = "开始日期无效",
    ["invalid_begin_end"] = "无效的开始/结束",
    ["invalid_to"] = "无效到",
    ["local_networks"] = "本地网络",
    ["local_remote"] = "本地/远程",
    ["network_interface"] = "网络接口",
    ["period"] = "间隔",
    ["please_check_format"] = "请检查其格式",
    ["please_choose_valid"] = "请选择一个有效的开始/结束日期和时间",
    ["please_choose_valid_date_and_time"] = "请选择一个有效的日期和时间",
    ["receivers"] = "接收者",
    ["remote_local"] = "远程/本地",
    ["report_for_subject"] = "报告 - %{num_min} - %{subject} 开始 %{dt}",
    ["reports_professional_only"] = "报告仅在专业版中可用",
    ["senders"] = "Senders",
    ["starting"] = "开始",
    ["submit_filter"] = "提交过滤器",
    ["symbolic_host_name_at_visualization"] = "符号主机名在可视化过程中被解析。",
    ["toggle_all"] = "全部切换",
    ["top_asn"] = "Top ASN",
    ["top_countries"] = "Top国家",
    ["top_local_hosts"] = "Top本地主机",
    ["top_local_os"] = "Top本地操作系统",
    ["top_networks"] = "Top网络",
    ["top_non_local_os"] = "Top非本地操作系统",
    ["top_remote_hosts"] = "Top远程主机",
    ["top_talkers"] = "Top Talkers",
    ["total_traffic"] = "总流量",
    ["traffic_anomalies"] = "流量异常",
    ["traffic_anomalies_local_hosts"] = "本地主机异常",
    ["traffic_anomalies_remote_hosts"] = "远程主机异常",
    ["traffic_report"] = "流量报告",
    ["use_server_timezone"] = "使用服务器时区",
  },
  ["rest_consts"] = {
    ["ADD_POOL_FAILED"] = "无法使用提交的参数添加池",
    ["ADD_POOL_FAILED_TOO_MANY_POOLS"] = "创建的池太多。考虑将ntopng升级到Enterprise M或以上，以创建更多的池。",
    ["ADD_POOL_FAILED_TOO_MANY_POOLS_ENTERPRISE"] = "创建太多池",
    ["BAD_CONTENT"] = "错误内容",
    ["BAD_FORMAT"] = "格式错误",
    ["BIND_POOL_MEMBER_ALREADY_BOUND"] = "无法绑定已经绑定到另一个池的成员",
    ["BIND_POOL_MEMBER_FAILED"] = "无法将 sumbitted 成员绑定到池中",
    ["CONFIGURATION_FILE_MISMATCH"] = "上传的文件不是有效的配置文件！",
    ["DELETE_POOL_FAILED"] = "无法删除包含已提交参数的池",
    ["EDIT_POOL_FAILED"] = "无法使用提交的参数编辑池",
    ["INFRASTRUCTURE_INSTANCE_ALREADY_EXISTING"] = "重复的实例。",
    ["INFRASTRUCTURE_INSTANCE_CHECK_AUTH_FAILED"] = "授权失败。无效的Token",
    ["INFRASTRUCTURE_INSTANCE_CHECK_FAILED"] = "无法执行请求。无法到达目的地址。",
    ["INFRASTRUCTURE_INSTANCE_CHECK_INVALID_RESPONSE"] = "收到无效响应。需要 JSON",
    ["INFRASTRUCTURE_INSTANCE_CHECK_NOT_FOUND"] = "未找到",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_ALIAS"] = "实例名称不能为空。",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_BANDWIDTH_THRESHOLD"] = "带宽阈值不能为空！",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_ID"] = "实例ID不能为空。",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_RTT_THRESHOLD"] = "RTT 阈值不能为空！",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_TOKEN"] = "实例的Token不能为空。",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_URL"] = "必须定义实例的 URL。",
    ["INFRASTRUCTURE_INSTANCE_SAME_ALIAS"] = "已存在具有此别名的实例。",
    ["INFRASTRUCTURE_INSTANCE_SAME_TOKEN"] = "一个实例已经绑定了这个Token。",
    ["INFRASTRUCTURE_INSTANCE_SAME_URL"] = "已经定义了具有此 URL 的实例",
    ["INTERNAL_ERROR"] = "内部错误",
    ["INVALID_ARGUMENTS"] = "无效参数",
    ["INVALID_HOST"] = "无效主机",
    ["INVALID_INTERFACE"] = "无效接口",
    ["NAME_RESOLUTION_FAILED"] = "无法将主机名解析为 IP 地址",
    ["NOT_FOUND"] = "未找到",
    ["NOT_GRANTED"] = "未授权",
    ["NO_SNMP_DEVICE_DISCOVERED"] = "未发现 SNMP 设备",
    ["OK"] = "OK",
    ["PARTIAL_IMPORT"] = "配置已成功导入,但有些元素已经存在且未被替换。",
    ["POOL_ADDED_SUCCESSFULLY"] = "池添加成功",
    ["POOL_DELETED_SUCCESSFULLY"] = "池删除成功",
    ["POOL_EDITED_SUCCESSFULLY"] = "池编辑成功",
    ["POOL_MEMBER_BOUND_SUCCESSFULLY"] = "池成员绑定成功",
    ["POOL_NOT_FOUND"] = "未找到池",
    ["SNMP_DEVICE_ADDED_SUCCESSFULLY"] = "SNMP 设备添加成功",
    ["SNMP_DEVICE_ALREADY_ADDED"] = "SNMP 设备已添加",
    ["SNMP_DEVICE_DELETED_SUCCESSFULLY"] = "SNMP 设备删除成功",
    ["SNMP_DEVICE_EDITED_SUCCESSFULLY"] = "SNMP 设备编辑成功",
    ["SNMP_DEVICE_INTERFACE_STATUS_CHANGE_FAILED"] = "SNMP 设备接口状态更改失败",
    ["SNMP_DEVICE_UNREACHABLE"] = "SNMP 设备无应答",
    ["WIDGETS_MISSING_DATASOURCES"] = "缺少数据源",
    ["WIDGETS_MISSING_DATASOURCE_PARAMS"] = "缺少数据源参数",
    ["WIDGETS_MISSING_DATASOURCE_TYPE"] = "缺少数据源类型",
    ["WIDGETS_MISSING_TRANSFORMATION"] = "缺少转换",
    ["WIDGETS_UNKNOWN_DATASOURCE_TYPE"] = "未知数据源类型",
  },
  ["restart"] = {
    ["confirm"] = "您现在要重启 %{product} 吗？",
    ["restart"] = "重启",
    ["restart_now"] = "立即重启",
    ["restart_product"] = "重启 %{product}",
    ["restart_required"] = "需要重启",
    ["restarting"] = "正在重新启动 %{product}...（这应该需要几秒钟）",
  },
  ["scripts_list"] = {
    ["alert_severity"] = "警告严重性",
    ["all"] = "全部",
    ["are_you_sure"] = "您的编辑尚未保存！您确定要关闭此对话框吗？",
    ["attention"] = "注意",
    ["back_scripts_page"] = "返回脚本",
    ["blacklisted_country"] = "请填写以逗号分隔的国家代码值。（即：IT,FR,DE,UK）",
    ["config"] = "配置",
    ["exclusion_list"] = "逗号分隔的 IP 地址列表。此列表中的主机不会触发此警告。",
    ["exclusion_list_title"] = "排除列表",
    ["filter_dropdown"] = "过滤类别",
    ["items_list_comma"] = "除 ',' 外不允许使用特殊字符",
    ["missing_configset"] = "未找到请求的配置集！",
    ["previous_page"] = "点击这里返回上一页！",
    ["reset_default"] = "重置为出厂值",
    ["script_search"] = "搜索脚本：",
    ["scripts_x"] = "脚本 %{subdir} - %{config}",
    ["select_operator"] = "请选择运算符！",
    ["template_not_found"] = "模板未找到！",
    ["actions"] = {
      ["edit"] = "编辑",
      ["edit_script"] = "编辑脚本",
      ["view"] = "视图",
      ["view_src_script"] = "查看源脚本",
    },
    ["templates"] = {
      ["blacklisted_country_list"] = "黑名单国家列表",
      ["elephant_flows_l2r"] = "大数据流阈值（本地到远程）",
      ["elephant_flows_r2l"] = "大数据流阈值（远程到本地）",
      ["excluded_applications"] = "排除的应用程序和类别",
      ["flow_duration_threshold"] = "流量持续时间阈值",
      ["template_not_implemented"] = "模板尚未实现！",
    },
  },
  ["service_map_page"] = {
    ["allowed"] = "允许",
    ["denied"] = "拒绝",
    ["fetch_error"] = "获取服务映射状态时出错。",
    ["filter_hosts"] = "过滤器",
    ["graph_view"] = "图表视图",
    ["learning"] = "服务地图正在<b>学习</b>...",
    ["learning_table"] = "服务地图正在<b>学习</b>,点击重新加载表格按钮查看新变化...",
    ["no_services"] = "未找到<b>服务</b>。",
    ["node_descr"] = "主机用圆圈表示：<font color=#28a745>Local</font>、<font color=#6c757d>Remote</font>、<font color=#007bff>Multicast</font >, <font color=#17a2b8>广播</font> 主机。<li>弧表示两个主机之间的通信：<font color=#28a745>allowed</font>,<font color=red>denied</font >, 和 <font color=#ffc107> 未定</font> 服务状态。<li>双击节点跳转到主机页面。使用鼠标滚轮放大/缩小。</li>",
    ["note_1"] = "在<b>学习</b>期间,不能更改服务状态。",
    ["note_2"] = "<b>Undecided</b> (<kbd class='bg-warning text-dark'><i class='fas fa-hourglass-half'></i> </kbd>) 状态需要用户确认。一旦服务状态得到确认,就可以<b>允许</b> (<kbd class='bg-success'><i class='fas fa- check'></i></kbd>) 或 <b>Denied</b>（<kbd class='bg-danger'><i class='fas fas fa-times'></i></i></kbd>)",
    ["protocols"] = "所有协议",
    ["set_state"] = "将 %{label} 服务状态设置为：",
    ["status"] = "状态",
    ["table_view"] = "表格视图",
    ["undecided"] = "未定",
    ["aging"] = {
      ["aging"] = "所有时间",
      ["day"] = "最后一天",
      ["month"] = "自上个月以来",
      ["week"] = "自上周以来",
    },
  },
  ["severity_groups"] = {
    ["group_error_or_higher"] = "错误或更高",
    ["group_none"] = "空",
    ["group_notice_or_lower"] = "通知或更低",
    ["group_warning"] = "警告",
  },
  ["sflows_stats"] = {
    ["active_flows"] = "活动流",
    ["client_peer"] = "客户端peer",
    ["client_process"] = "客户端进程",
    ["server_peer"] = "服务器peer",
    ["server_process"] = "服务器进程",
    ["total_bytes"] = "总字节数",
  },
  ["shaping"] = {
    ["applied_to"] = "应用于",
    ["bandwidth_manager"] = "带宽管理器",
    ["confirm_delete_all_policies"] = "确认要删除所有\"%{pool_name}\"池策略",
    ["confirm_delete_policy"] = "确认要删除吗",
    ["confirm_delete_shaper"] = "确认要删除shaper吗",
    ["daily_time"] = "每日时间（当前/配额）",
    ["daily_time_quota"] = "每日时间配额",
    ["daily_traffic"] = "每日流量（当前/配额）",
    ["daily_traffic_quota"] = "每日流量配额",
    ["delete_all_policies"] = "删除所有策略",
    ["delete_policy"] = "删除策略",
    ["delete_shaper"] = "删除Shaper",
    ["for_dropping_all_traffic"] = "用于丢弃所有流量",
    ["for_no_shaping"] = "对于没有Shaper",
    ["initial_clone_protocols"] = "初始应用规则将被克隆",
    ["initial_empty_protocols"] = "初始应用规则将为空",
    ["manage_policies"] = "管理策略",
    ["no_quota_applied"] = "无配额",
    ["no_quota_data"] = "未设置配额",
    ["no_quota_traffic"] = "目前没有配额相关的流量",
    ["no_shapers_available"] = "没有可用的Shaper",
    ["note_default_pool_config"] = "策略和配额仅适用于用户定义的主机池。选择不同的池或在<a href='%{url}'>处</a>创建一个新池。",
    ["note_drop_core"] = "删除一些核心协议可能会对其他协议产生副作用。例如,如果您阻止 DNS,<br>符号主机名将不再解析,因此只能与数字 IP 通信。",
    ["note_quota_unlimited"] = "将流量和时间配额设置为 0 以实现无限流量。",
    ["note_target_type"] = "您可以通过<a href=\"%{url}\">流量桥接首选项</a>专家视图中的策略目标类型字段修改要在目标列中显示的内容。",
    ["note_traffic_categories"] = "您可以从<a href=\"%{url}\">应用程序页面</a>配置类别。",
    ["notes"] = "注意",
    ["policy_from_pool"] = "来自池的策略",
    ["protocol_families"] = "应用程序家族",
    ["protocol_policy"] = "流量策略",
    ["protocols"] = "应用程序",
    ["protocols_policies"] = "应用程序策略",
    ["see_quotas_here"] = "请访问主机池<a href='%{url}'>配额页面</a>,了解活动主机池配额的完整概述。",
    ["select_to_clone"] = "选择一个现有网络来克隆应用程序规则",
    ["set_max_rate_to"] = "设置最大速率为",
    ["shaper0_message"] = "Shaper 0 是用于没有定义 shaper 的本地主机的默认 shaper",
    ["shaper_id"] = "shaper_id",
    ["shapers_in_use_message"] = "Shapers 只能在没有应用于任何网络的情况下被删除",
    ["traffic_from"] = "流量来自",
    ["traffic_to"] = "流量到",
  },
  ["show_alerts"] = {
    ["1_day"] = "1天",
    ["1_hour"] = "1小时",
    ["1_month"] = "1个月",
    ["1_week"] = "1周",
    ["1_year"] = "1年",
    ["30_min"] = "30分钟",
    ["5_min"] = "5分钟",
    ["6_months"] = "6个月",
    ["acknowledge_alert"] = "确认警告",
    ["acknowledged"] = "确认的",
    ["alarmable"] = "警告",
    ["alert_actions"] = "操作",
    ["alert_count"] = "计数",
    ["alert_counts"] = "计数",
    ["alert_datetime"] = "日期/时间",
    ["alert_description"] = "描述",
    ["alert_duration"] = "持续时间",
    ["alert_severity"] = "严重性",
    ["alert_severity_and_type"] = "严重性和类型",
    ["alert_type"] = "警告类型",
    ["alerts"] = "警告",
    ["alerts_are_disabled_message"] = "警告已禁用。请检查首选项页面以启用它们。",
    ["alerts_queries"] = "警告查询",
    ["alerts_to_purge"] = "清除警告",
    ["alerts_to_purge_x"] = "%{filter} 要清除的警告",
    ["cli_country"] = "客户端国家",
    ["confirm_acknowledge_alert"] = "你确定想承认指定的警报吗？",
    ["confirm_acknowledge_alerts"] = "Stored alerts matching the specified criteria will be acknowledged. Continue?",
    ["confirm_delete_alert"] = "您确定要删除指定的警告吗",
    ["confirm_delete_alerts"] = "符合指定条件的存储警告将被删除。继续吗？",
    ["confirm_delete_filtered_alerts"] = "符合指定禁用条件的存储警告将被删除。",
    ["confirm_external_link"] = "您正在被转到: ",
    ["confirm_filter_alert"] = "将阻止触发符合指定禁用条件的警告。",
    ["confirm_label_alerts"] = "符合指定标准的已存储警报将被确认。继续吗？",
    ["confirm_release_alert"] = "确认要释放指定的警告吗？",
    ["daily"] = "每日",
    ["delete_alert"] = "删除警告",
    ["delete_alerts_configuration"] = "删除配置",
    ["delete_all"] = "全部删除",
    ["delete_config_btn"] = "删除 %{conf}s 通用配置",
    ["delete_config_message"] = "确认要删除%{conf}s的常用%{granularity}配置吗",
    ["detected_alerts"] = "检测到的警告",
    ["disable_alerts"] = "禁用",
    ["disable_alerts_message"] = "确认要在 %{entity_value} 上禁用 \"%{type}\" 警告吗？",
    ["disable_alerts_title"] = "排除检查",
    ["disabled_alerts"] = "禁用警告",
    ["disabled_alerts_not_available"] = "该功能仅在ntpng Pro或以上版本中可用。",
    ["download_alerts"] = "下载警告",
    ["dropped_alerts"] = "丢弃的警告",
    ["enable_alerts"] = "启用",
    ["enable_alerts_message"] = "确认要在 %{entity_value} 上启用 \"%{type}\" 警告吗？",
    ["enable_alerts_title"] = "启用警告",
    ["engaged"] = "参与",
    ["engaged_alerts"] = "参与警告",
    ["exclude_alert_err"] = "排除警告时出错,请检查参数并重试",
    ["expand_action"] = "扩展",
    ["explorer"] = "探索",
    ["filter_alert"] = "禁用警告",
    ["filters"] = "过滤器",
    ["five_minutes"] = "五分钟",
    ["flow_alerts"] = "流量警告",
    ["flow_checks_drops_descr"] = "当某些<a href='%{url}'>流检查</a>未被执行时，触发警报。",
    ["flow_checks_drops_title"] = "流检查丢弃警告",
    ["flow_user_scripts_drops_descr"] = "当某些 <a href='%{url}'>Flows 用户脚本</a>未执行时触发警告。",
    ["flow_user_scripts_drops_title"] = "流用户脚本丢弃警告",
    ["flush_data"] = "刷新数据",
    ["host_delete_config_btn"] = "删除主机%{host}配置",
    ["host_delete_config_confirm"] = "确认要删除主机的 %{granularity} 配置",
    ["hourly"] = "每小时",
    ["iface_delete_config_btn"] = "删除接口 %{iface} 配置",
    ["iface_delete_config_confirm"] = "确认要删除接口的 %{granularity} 配置",
    ["iface_engaged_dropped_alerts"] = "参与vs丢弃警告",
    ["in_the_selected_time_frame"] = "在选定的时间段内",
    ["interface_drops_threshold"] = "接口丢包警告",
    ["interface_drops_threshold_descr"] = "丢弃的数据包占接口观察数据包总数的最大百分比",
    ["label_alert"] = "标签警告",
    ["last_day"] = "最后一天",
    ["last_hour"] = "最后一小时",
    ["last_minute"] = "最后一分钟",
    ["minute"] = "分钟",
    ["network_delete_config_btn"] = "删除网络%{network}配置",
    ["network_delete_config_confirm"] = "确认要删除网络的 %{granularity} 配置",
    ["no_recorded_alerts_message"] = "没有记录的警告",
    ["num_ignored_alerts"] = "忽略的警告",
    ["older_1_day_ago"] = "早于1天前",
    ["older_1_hour_ago"] = "早于1小时前",
    ["older_1_month_ago"] = "早于1个月前",
    ["older_1_week_ago"] = "早于1周前",
    ["older_1_year_ago"] = "早于1年前",
    ["older_30_minutes_ago"] = "早于30分钟前",
    ["older_5_minutes_ago"] = "早于5分钟前",
    ["older_6_months_ago"] = "早于6个月前",
    ["older_than"] = "早于",
    ["past"] = "过去",
    ["past_alerts"] = "过去的警告",
    ["past_flow"] = "过去的流",
    ["purge_all_alerts"] = "清除所有警告",
    ["purge_num_alerts"] = "清除 [%{num_alerts} 个警告]",
    ["purge_subj_alerts"] = "清除 %{subj} 警告",
    ["purge_subj_alerts_confirm"] = "确认要清除所有 %{subj} 警告吗？",
    ["release_alert"] = "发布警告",
    ["release_alert_action"] = "发布",
    ["remove_filters"] = "移除过滤器",
    ["reset_alert_database"] = "刷新警告数据",
    ["reset_alert_database_message"] = "此操作将删除任何与警告相关的数据,包括<ul><li>运行时警告状态</li><li>持久警告 SQLite 存储</li></ul>请转到用户如果您还想重置警告配置,请使用脚本设置。<br><br>确认要继续吗",
    ["srv_country"] = "服务器国家",
    ["startup"] = "启动",
    ["stored_alerts"] = "存储警告",
    ["trigger_alert_descr"] = "启用在此警告器上触发的警告",
    ["trigger_host_alert_descr"] = "触发 %{host} 生成的警告",
    ["trigger_iface_alert_descr"] = "触发 %{iface} 生成的警告",
    ["trigger_network_alert_descr"] = "触发 %{network} 生成的警告",
    ["presets"] = {
      ["30_min"] = "最后30分钟",
      ["5_min"] = "最后5分钟",
      ["day"] = "最后一天",
      ["hour"] = "最后一小时",
      ["month"] = "最后一个月",
      ["presets"] = "预设",
      ["week"] = "上周",
      ["year"] = "去年",
    },
  },
  ["slack_alert_endpoint"] = {
    ["notification_webhook"] = "通知webhook",
    ["sender_username"] = "发件人用户名",
  },
  ["snmp"] = {
    ["AlertDescription"] = "警告描述",
    ["ConsoleDisplayBufferText"] = "显示文本",
    ["GeneralSerialNumber"] = "序列号",
    ["InputCurrentLevel"] = "当前等级",
    ["InputDimUnit"] = "Dim Unit",
    ["InputMaxCapacity"] = "最大容量",
    ["InputMediaDimFeedDirDeclared"] = "InputMediaDimFeedDirDeclared",
    ["InputMediaDimXFeedDirDeclared"] = "InputMediaDimXFeedDirDeclared",
    ["InputName"] = "输入名称",
    ["MarkerColorantValue"] = "标记着色剂",
    ["MarkerLifeCount"] = "标记生命计数",
    ["MarkerSuppliesDescription"] = "标记消耗描述",
    ["MarkerSuppliesLevel"] = "标记消耗等级",
    ["MarkerSuppliesMaxCapacity"] = "标记消耗最大容量",
    ["PrinterCurrentStatus"] = "当前状态",
    ["PrinterOperationalStatus"] = "操作状态",
    ["absolute_stats"] = "自启动以来的统计数据",
    ["actual_stats"] = "自上次轮询以来的统计数据",
    ["add_device"] = "添加",
    ["add_devices"] = "添加设备",
    ["add_snmp_devices"] = "添加 SNMP 设备",
    ["alert_topology"] = "没有节点可显示！",
    ["alerted_interfaces"] = "有错误的接口",
    ["all_interfaces"] = "所有接口",
    ["all_snmp_interface_types"] = "所有类型",
    ["all_snmp_statuses"] = "所有状态",
    ["and_one_more_interface"] = "还有<a href='%{url}'>1 个接口</a>",
    ["authentication_passphrase"] = "身份验证密码",
    ["authentication_protocol"] = "身份验证协议",
    ["bound_interface_description"] = "将网络接口绑定到 SNMP 接口有助于比较 ntopng 监控的网络流量与 SNMP 报告的网络流量",
    ["caching_in_progress"] = "设备 SNMP 正在进行中。这可能需要 5 分钟。页面将定期刷新。",
    ["contact"] = "连接",
    ["cpuIdle"] = "CPU 空闲",
    ["cpuSystem"] = "CPU 系统",
    ["cpuUser"] = "CPU 用户",
    ["delete_all_devices"] = "全部删除",
    ["delete_unresponsive_devices"] = "删除无响应",
    ["device_example"] = "示例：%{example}",
    ["device_ip"] = "设备IP",
    ["device_model"] = "设备型号",
    ["device_polling"] = "设备轮询",
    ["device_polling_desc"] = "定期执行 SNMP 漫游以收集设备数据,包括设备接口流量和状态。",
    ["device_responsiveness"] = "设备过滤器",
    ["device_status"] = "设备状态",
    ["devices_ added"] = "设备已添加",
    ["disable_port_monitor"] = "禁用端口监视器",
    ["discover_devices"] = "发现设备",
    ["edit_device"] = "编辑",
    ["edit_snmp_devices"] = "编辑 SNMP 设备",
    ["enable_port_monitor"] = "启用端口监视器",
    ["errors_filter"] = "错误过滤器",
    ["ifName"] = "接口名称",
    ["ifindex"] = "索引",
    ["in_bytes"] = "以字节为单位",
    ["in_discards"] = "在丢弃中",
    ["in_errors"] = "错误",
    ["interface_device"] = "接口 %{interface} [%{device}]",
    ["interfaces_with_discards"] = "有丢弃的接口",
    ["interfaces_with_errors"] = "有错误的接口",
    ["interfaces_with_errors_or_discards"] = "有错误/丢弃的接口",
    ["last_change"] = "最后一次更改",
    ["lldpRemChassisId"] = "远程设备",
    ["lldpRemPortDesc"] = "远程端口描述",
    ["lldpRemPortId"] = "远程端口标识符",
    ["lldpRemSysDesc"] = "远程系统描述",
    ["lldpRemSysName"] = "远程系统名称",
    ["lldp_topology_changed"] = "LLDP 拓扑已更改",
    ["lldp_topology_changed_description"] = "检测 SNMP 网络拓扑的变化并触发警告",
    ["lldp_topology_changed_title"] = "LLDP 拓扑监视器",
    ["low_port_speed"] = "低端口速度",
    ["memAvailReal"] = "可用内存",
    ["memTotalReal"] = "总内存",
    ["memTotalSwap"] = "总交换",
    ["multiple_macs_non_trunk"] = "非中继端口上的多个 MAC",
    ["no_answer"] = "主机 %{host} 无应答：请检查 SNMP 是否已启动并且Community设置为正确值",
    ["no_host_discovered"] = "在 %{host}/%{cidr} 中没有发现主机",
    ["no_polling_yet"] = "未轮询",
    ["none"] = "空",
    ["note"] = "注意",
    ["object_id"] = "ObjectID",
    ["out_bytes"] = "输出字节数",
    ["per_interface_settings"] = "这些设置只影响接口 \"%{iface}\"。查看 <a href=\"%{url}\">%{global}</a> 以获得全局配置。",
    ["port_macs"] = "MAC 地址",
    ["port_macs_short"] = "MACs",
    ["privacy_passphrase"] = "私有密码",
    ["privacy_protocol"] = "私有协议",
    ["prune_unresponsive_snmp_devices"] = "删除无响应的 SNMP 设备",
    ["prune_unresponsive_snmp_devices_confirm"] = "确认要删除所有无响应的 SNMP 设备吗？",
    ["remove_all_snmp_devices"] = "删除所有 SNMP 设备",
    ["remove_all_snmp_devices_confirm"] = "确认要删除所有的 SNMP 设备吗？",
    ["remove_snmp_device"] = "删除 SNMP 设备",
    ["remove_snmp_device_confirm"] = "确认要删除SNMP设备吗",
    ["responsiveness_all"] = "所有设备",
    ["responsiveness_responsive"] = "响应设备",
    ["responsiveness_unresponsive"] = "无响应设备",
    ["select_pool"] = "选择一个池",
    ["snmp_all_interfaces"] = "所有接口",
    ["snmp_also_seen_on"] = "也发现",
    ["snmp_and_n_more_interfaces"] = "和<a href='%{url}'>%{count} 个接口</a>",
    ["snmp_authnopriv"] = "Auth. w/o Privacy",
    ["snmp_authpriv"] = "Auth. with Privacy",
    ["snmp_community"] = "community",
    ["snmp_device"] = "SNMP 设备",
    ["snmp_device_does_not_respond"] = "SNMP 设备没有响应。",
    ["snmp_device_last_poll"] = "自上次轮询以来的时间",
    ["snmp_device_last_poll_duration"] = "上次轮询持续时间",
    ["snmp_device_polling_disabled"] = "周期性设备轮询被禁用。显示的信息可能丢失或过时。轮询可以从 <a href=\"%{url}\"><i class=\"fas fa-flask 启用\"></i> 首选项</a>。",
    ["snmp_device_x"] = "%{device} (%{ip})",
    ["snmp_devices"] = "SNMP 设备",
    ["snmp_error_unable_to_set_interface_admin_status"] = "无法切换管理员状态。请确保 SNMP 设备已配置（有效）写入community。",
    ["snmp_host"] = "主机",
    ["snmp_interface"] = "SNMP 接口",
    ["snmp_interface_type"] = "接口类型",
    ["snmp_interface_x"] = "接口：%{interface}",
    ["snmp_interfaces"] = "接口",
    ["snmp_last_poll_errors"] = "有错误的接口 (lLast Poll)",
    ["snmp_level"] = "安全级别",
    ["snmp_lldp_descr"] = "当<A HREF=\"https://en.wikipedia.org/wiki/Link_Layer_Discovery_Protocol\">LLDP</A> 协议在被监控设备上启用并且可读时,设备拓扑信息通过SNMP可用。",
    ["snmp_lldp_node_color"] = "节点颜色代表设备型号。",
    ["snmp_lldp_zoom_descr"] = "点击节点探索设备,使用鼠标滚轮缩放,拖动点击空白区域。",
    ["snmp_monitor_disable_confirm"] = "确认要禁用 SNMP 设备 %{device} 上的端口状态监控吗？",
    ["snmp_monitor_enable_confirm"] = "确认要在 SNMP 设备 %{device} 上启用端口状态监控吗？只要端口状态改变,就会生成警告。",
    ["snmp_netsnmp"] = "CPU/内存",
    ["snmp_noauthnopriv"] = "空",
    ["snmp_note_bridge_mib"] = "在桥 MIB 上执行定期 SNMP walk以填充表。",
    ["snmp_note_bridge_mib_status"] = "显示的状态是与 mac 地址关联的 dot1qTpFdbStatus 桥 MIB 状态。",
    ["snmp_note_periodic_interfaces_polling"] = "执行定期 SNMP walk以定期 5 分钟间隔更新接口。",
    ["snmp_note_thpt_calc"] = "最近两次连续SNMP漫游的总流量（输入+输出）的吞吐量占比。",
    ["snmp_note_thpt_span"] = "最近两次连续的 SNMP walk在 %{last} 和 %{prev} 执行。",
    ["snmp_pool"] = "池",
    ["snmp_port_monitor"] = "SNMP 端口监视器",
    ["snmp_printer"] = "打印机",
    ["snmp_read_community"] = "读Community",
    ["snmp_timeseries_descr"] = "SNMP 设备时间序列可以从<a href=\"%{url}\"><i class=\"fas fa-flask\"></i>首选项</a>启用. 需要几分钟才能看到第一个数据点",
    ["snmp_topology"] = "拓扑",
    ["snmp_username"] = "用户名",
    ["snmp_version"] = "SNMP 版本",
    ["snmp_write_community"] = "写Community",
    ["stats_since"] = "统计自",
    ["status_dormant"] = "休眠",
    ["status_down"] = "Down",
    ["status_lowerlayerdown"] = "下层",
    ["status_notpresent"] = "不存在",
    ["status_unknown"] = "Unkonwn",
    ["status_up"] = "Up",
    ["status_up_in_use"] = "Up (在使用)",
    ["testing"] = "测试",
    ["tp_fbd_status_invalid"] = "无效",
    ["tp_fbd_status_learned"] = "学习",
    ["tp_fbd_status_mgmt"] = "mgmt",
    ["tp_fbd_status_other"] = "其他",
    ["tp_fbd_status_self"] = "自身",
    ["trunk"] = "trunk",
    ["unable_to_delete_snmp_device"] = "无法删除 SNMP 设备 %{device}",
    ["unresponsive_device"] = "SNMP walk 太慢。显示的数据可能已过时或丢失。检查网络连接和设备响应。将在 15 分钟后执行新的扫描尝试。",
    ["view_device"] = "查看设备",
    ["descriptions"] = {
      ["add_pool"] = "新的 SNMP 池",
      ["authentication_passphrase"] = "SNMPv3 认证密码",
      ["authentication_protocol"] = "SNMPv3 认证协议",
      ["host"] = "SNMP 代理 IP 地址",
      ["level"] = "SNMPv3 安全级别",
      ["pool"] = "要绑定的SNMP池",
      ["privacy_passphrase"] = "SNMPv3 加密密码",
      ["privacy_protocol"] = "SNMPv3 加密协议",
      ["read_community"] = "SNMP v1/v2c 读Community",
      ["username"] = "SNMPv3 认证用户名",
      ["version"] = "SNMP 协议版本",
      ["write_community"] = "SNMP v1/v2c 写Community",
    },
    ["validation"] = {
      ["abort_add_snmp_devices"] = "中止添加新的 SNMP 设备？",
      ["abort_edit_snmp_devices"] = "中止编辑 SNMP 设备？",
      ["invalid_host"] = "请输入 IPv4 地址或 IPv6 地址。",
      ["read_community_empty"] = "读Community不能为空。",
    },
  },
  ["snmp_page"] = {
    ["guide_snmp_page_message"] = "访问页面 <a href='%{url}'> SNMP</a> 将此主机添加到已配置的 SNMP 设备列表中。",
    ["not_configured_as_snmp_device_message"] = "主机 %{host_ip} 尚未配置为 SNMP 设备。",
    ["showing_snmp_mib_info_default_community_message"] = "显示使用默认Community'%{community}'检索的SNMP MIB信息",
    ["snmp_device_not_found"] = "未发现SNMP设备 \"%{host}\" : 也许它已从已配置的 SNMP 设备列表中删除？",
    ["trying_to_retrive_message"] = "尝试使用默认Community '%{community}' 检索主机 SNMP MIB",
    ["unable_to_retrive_snmp_default_community_message"] = "无法使用默认Community '%{community}' 检索主机 SNMP MIB。",
  },
  ["snmp_ratio_errors"] = {
    ["flowdevice_timeseries_not_existing"] = "不可用。",
    ["flowdevice_timeseries_query_failed"] = "时间序列不可读",
    ["nan_ratio"] = "比率不是数字。",
    ["negative_ratio"] = "负比率。",
    ["ok"] = "OK",
    ["snmp_timeseries_not_enabled"] = "时间序列已禁用。",
    ["snmp_timeseries_not_existing"] = "不可用。",
    ["snmp_timeseries_query_failed"] = "时间序列不可读。",
    ["snmp_timeseries_zero_total"] = "无数据。",
    ["timeseries_start_time_mismatch"] = "时间序列未对齐。",
  },
  ["sprobe_page"] = {
    ["active_memory"] = "活动内存",
    ["latency"] = "延迟",
    ["processes"] = "进程",
    ["processes_traffic_tree"] = "进程流量树",
    ["show_more_info"] = "鼠标悬停显示进程信息或双击显示更多信息。",
    ["system_interactions"] = "系统交互",
    ["top_processes"] = "Top进程",
    ["top_users"] = "Top用户",
    ["tree"] = "树",
    ["users"] = "用户",
  },
  ["sub_interfaces"] = {
    ["add_sub_if"] = "添加子接口",
    ["add_sub_if_success"] = "子接口创建成功。需要重启 ntopng 才能生效。",
    ["collection_filter_description"] = "也可以使用 NetFlow 字段,包括 INPUT_SNMP、OUTPUT_SNMP、SRC_VLAN、DST_VLAN。非数字值应提供单引号。",
    ["confirm_delete"] = "您确定要删除 %{name} 吗？",
    ["delete"] = "删除子接口",
    ["disaggregation"] = "自定义流量分解",
    ["empty_filter"] = "未指定过滤器",
    ["filter_description"] = "支持扩展的 nBPF 格式。除了标准的 BPF 语法之外,还可以使用特殊的 keyworks,如 <i>local</i> 和 <i>remote</i>（结合<i>src</i> 和 <i>dst</i> 限定符）。",
    ["filter_examples"] = "过滤示例",
    ["invalid_name_format"] = "接口名称只能包含字母、数字、破折号。",
    ["invalid_name_length"] = "接口名称太短（5 个或更多字符）",
    ["invalid_xnbpf"] = "无效的 nBPF 过滤器",
    ["note"] = "ntopng 可以使用类似 BPF 的过滤器来分解传入的流量,将其转移到逻辑子接口。如果过滤器重叠,可以将相同的流量转移到多个接口。",
    ["sub_if_name"] = "子接口名称",
    ["traffic_filter_nbpf"] = "流量过滤器（nBPF 格式）",
    ["traffic_filter_xnbpf"] = "流量过滤器（扩展 nBPF 格式）",
  },
  ["syslog"] = {
    ["add"] = "添加日志Producer",
    ["bad_action_param"] = "糟糕\"操作\"参数",
    ["confirm_delete"] = "确认要移除这个Producer吗？",
    ["edit"] = "编辑日志Producer",
    ["empty_action"] = "出现问题（空操作）。重新加载页面并重试。",
    ["host_add_ok"] = "主机 %{host} 添加成功！",
    ["host_delete_ok"] = "主机 %{host} 已成功删除！",
    ["host_edit_ok"] = "主机 %{host} 已成功编辑！",
    ["host_exists"] = "主机\"%{host}\"已经存在",
    ["host_not_exists"] = "主机\"%{host}\"未找到",
    ["ip_or_device"] = "IP 或设备名称",
    ["ip_or_device_note"] = "日志Producer IP 应与系统日志消息中显示的 IP 或设备名称匹配。示例：<br />Thu Apr 10 16:30:00 2020;<strong>10.0.0.1< /strong>; <181>4 月 10 日星期四 16:30:00 <strong>sslvpn-device-01</strong> 日志服务器：...",
    ["producer_host"] = "日志ProducerIP",
    ["producer_type"] = "日志Producer类型",
    ["producers"] = "Syslog 日志Producer",
    ["syslog"] = "系统日志",
  },
  ["system_alerts_stats"] = {
    ["internal_alerts_queue"] = "内部队列",
    ["notes_enq_deq"] = "'填充级别' 是队列警告的比例,参考队列大小。'丢弃' 是由于队列已满而丢弃的警告比例,参考警告总数",
    ["notes_internal_alerts_queue"] = "内部队列与 SQLite 结合使用来处理内核中生成的警告并将它们显示在 UI 中",
    ["queues_status"] = "填充级别/丢弃",
  },
  ["system_interfaces_stats"] = {
    ["local_hosts"] = "本地主机",
    ["remote_hosts"] = "远程主机",
  },
  ["system_stats"] = {
    ["cpu_states_monitor"] = "CPU 状态监视器",
    ["cpu_states_monitor_description"] = "监控 CPU 的状态（例如,用户、系统、空闲和 iowait）",
    ["disk_monitor"] = "磁盘监视器",
    ["disk_monitor_description"] = "监控可用磁盘空间",
    ["dropped_points"] = "丢失Points",
    ["export_failures"] = "失败的导出",
    ["export_retries"] = "重试导出",
    ["exported_points"] = "总Points",
    ["exported_vs_dropped_points"] = "总Points vs 丢失Points",
    ["exports"] = "总导出",
    ["exports_label"] = "导出",
    ["health"] = "健康",
    ["high_series_cardinality"] = "高基数会导致数据库变慢",
    ["infludb_retried_vs_failed_exports"] = "重试/失败的导出",
    ["influxdb_health_green"] = "OK",
    ["influxdb_health_green_descr"] = "导出到 InfluxDB 工作正常。",
    ["influxdb_health_red"] = "导出失败",
    ["influxdb_health_red_descr"] = "InfluxDB 无法访问或速度太慢,无法跟上，丢Point。",
    ["influxdb_health_yellow"] = "暂时无法导出",
    ["influxdb_health_yellow_descr"] = "目前无法导出到 InfluxDB。正在重试。",
    ["influxdb_note_docs"] = "ntopng <a target='_blank' href='%{url}'>用户指南</a>中提供了包含所有显示字段详细说明的文档。",
    ["influxdb_storage"] = "存储(%{dbname})",
    ["last_ip"] = "最后一个 IP",
    ["series_cardinality"] = "系列基数",
    ["short_desc_influxdb_cardinality"] = "表明 InfluxDB 写Point的难度有多大。",
    ["short_desc_influxdb_dropped_points"] = "由于 InfluxDB 写入失败而丢失的总Point数。",
    ["short_desc_influxdb_exported_points"] = "成功写入 InfluxDB 的总Point数。",
    ["short_desc_influxdb_exports"] = "成功写入 InfluxDB 的总数。",
    ["short_desc_influxdb_health"] = "当前 InfluxDB 导出健康状况。",
    ["short_desc_influxdb_ram_memory"] = "InfluxDB 从操作系统获得的总内存。",
    ["short_desc_influxdb_storage_utilization"] = "InfluxDB 使用的总磁盘空间。",
    ["value"] = "值",
    ["write_througput"] = "写入吞吐量",
    ["redis"] = {
      ["command_hits"] = "%{cmd} 调用",
      ["commands"] = "命令",
      ["redis_health_green"] = "OK",
      ["redis_health_green_descr"] = "Redis工作正常。",
      ["redis_health_red"] = "持久化问题",
      ["redis_health_red_descr"] = "Redis无法持久化数据。预计 %{product} 会出现故障。",
      ["redis_keys"] = "数据库密钥",
      ["redis_monitor"] = "Redis监视器",
      ["redis_monitor_description"] = "监控Redis健康",
      ["short_desc_redis_health"] = "当前Redis健康状况。",
      ["short_desc_redis_keys"] = "Redis数据库中的key总数。",
      ["short_desc_redis_ram_memory"] = "Redis从操作系统获得的总内存。",
      ["top_redis_commands"] = "Top命令调用",
      ["tot_calls"] = "总通话数",
    },
  },
  ["tags"] = {
    ["acknowledged"] = "确认的",
    ["alert_id"] = "警告类型",
    ["begin_time"] = "开始时间",
    ["cli_ip"] = "客户端",
    ["cli_port"] = "客户端端口",
    ["end_time"] = "结束；时间",
    ["flow_risk"] = "流风险",
    ["ip"] = "主机",
    ["l4proto"] = "协议",
    ["l7_proto"] = "应用程序",
    ["l7cat"] = "L7应用程序类别",
    ["l7proto"] = "应用程序",
    ["network_name"] = "网络",
    ["role"] = "角色",
    ["roles"] = "角色",
    ["severity"] = "严重性",
    ["srv_ip"] = "服务器",
    ["srv_port"] = "服务器端口",
    ["subtype"] = "主题",
  },
  ["telemetry_page"] = {
    ["crash_report"] = "崩溃报告",
    ["crash_report_descr"] = "在 %{product} 异常终止时发送崩溃报告。崩溃报告包含 %{product} 信息,例如版本、构建架构、操作系统和启动选项。没有发送任何有关受监控流量的信息。以下是崩溃报告的示例：",
    ["last_data_sent"] = "最后发送的数据",
    ["send_telemetry_data"] = "通过发送遥测数据做出贡献",
    ["telemetry_data"] = "遥测数据",
    ["telemetry_data_change_preference"] = "<a href='%{url}'>更改</a>",
    ["telemetry_data_no_consent"] = "没有表示偏好",
  },
  ["top_host_sites"] = {
    ["0"] = "0:00 - 0:59",
    ["1"] = "1:00 - 1:59",
    ["10"] = "10:00 - 10:59",
    ["11"] = "11:00 - 11:59",
    ["12"] = "12:00 - 12:59",
    ["13"] = "13:00 - 13:59",
    ["14"] = "14:00 - 14:59",
    ["15"] = "15:00 - 15:59",
    ["16"] = "16:00 - 16:59",
    ["17"] = "17:00 - 17:59",
    ["18"] = "18:00 - 18:59",
    ["19"] = "19:00 - 19:59",
    ["2"] = "2:00 - 2:59",
    ["20"] = "20:00 - 20:59",
    ["21"] = "21:00 - 21:59",
    ["22"] = "22:00 - 22:59",
    ["23"] = "23:00 - 23:59",
    ["3"] = "3:00 - 3:59",
    ["4"] = "4:00 - 4:59",
    ["5"] = "5:00 - 5:59",
    ["6"] = "6:00 - 6:59",
    ["7"] = "7:00 - 7:59",
    ["8"] = "8:00 - 8:59",
    ["9"] = "9:00 - 9:59",
    ["current"] = "当前",
    ["day"] = "前一天",
    ["last_5_min"] = "最后5分钟",
    ["note_protocol_used"] = "提供信息的应用协议：HTTP、TLS、QUIC、DNS。",
    ["note_time_range"] = "时间范围从当前数据到已知的最早数据排序。",
    ["note_tot_entries"] = "每台主机最多提供20条记录。",
    ["note_yesterday_data"] = "数据最多保存24小时,也就是说你最多可以看到昨天的数据。因此,当前时间之后进入菜单的任何小时都是昨天的数据。",
  },
  ["top_hosts"] = {
    ["note"] = "此页面将每隔几秒刷新一次以报告Top主机活动。<p>当主机产生流量时,会显示一个垂直条,以便您可以看到最活跃的主机。",
    ["top_hosts_local"] = "Top主机（本地）",
  },
  ["topk_heuristic"] = {
    ["precision"] = {
      ["aggressive"] = "Aggressive",
      ["disabled"] = "禁用",
      ["less_accurate"] = "更低准确度",
      ["more_accurate"] = "更高准确度",
    },
  },
  ["traffic_behaviour"] = {
    ["allowed"] = "允许",
    ["denied"] = "拒绝",
    ["learning"] = "学习",
    ["undecided"] = "未定",
  },
  ["traffic_page"] = {
    ["flow_distribution"] = "流分布",
    ["flow_duration"] = "总流持续时间",
    ["flow_frequency"] = "新的流创建频率",
    ["hosts_contacts_cardinality"] = "唯一主机连接总数",
    ["l4_proto_overview"] = "协议概述",
    ["no_traffic_observed_message"] = "没有观察到指定主机的流量",
    ["num_contacted_hosts_as_client"] = "通信的主机（作为客户端）",
    ["num_host_contacts_as_server"] = "通信的主机（作为服务器）",
    ["rcvd"] = "接收",
  },
  ["traffic_profiles"] = {
    ["advanced_filter_examples"] = "高级过滤器示例",
    ["all_profiles"] = "所有配置文件",
    ["confirm_delete_profile"] = "您确定要删除配置文件吗",
    ["delete_profile"] = "删除配置文件",
    ["destination_network"] = "目标网络",
    ["duplicate_profile"] = "重复的配置文件名称",
    ["edit_traffic_profiles"] = "流量概况",
    ["enter_profile_filter"] = "输入配置文件过滤器",
    ["enter_profile_name"] = "输入配置文件名称",
    ["facebook_traffic"] = "Facebook",
    ["host_http_https"] = "主机 HTTP/HTTPS",
    ["host_traffic"] = "主机",
    ["http_traffic"] = "HTTP",
    ["invalid_bpf"] = "无效的 BPF 过滤器",
    ["max_profiles_num"] = "使用此版本的 ntopng,您只能创建 %{maxnum} 个配置文件。升级到企业版以消除此限制",
    ["no_profiles"] = "未设置配置文件",
    ["note"] = "注意",
    ["note_0"] = "流量配置文件名称允许使用字母数字字符、空格和下划线",
    ["note_1"] = "流量配置文件应用于流。每个流最多可以有一个配置文件,因此在多个配置文件匹配的情况下,只选择第一个",
    ["profile_name"] = "配置文件名称",
    ["see_ndpi_protos"] = "(协议ID和名称都支持,使用%{option}打印所有nDPI支持的协议)",
    ["simple_filter_examples"] = "简单过滤器示例",
    ["source_ethernet"] = "源 MAC 地址",
    ["traffic_between"] = "主机之间的流量",
    ["traffic_filter_bpf"] = "流量过滤器（nBPF 格式）",
    ["traffic_from_to"] = "从一个主机到另一个主机的流量",
    ["traffic_profiles"] = "流量概况",
  },
  ["traffic_recording"] = {
    ["about_to_download_flow"] = "即将提取从 <b>%{date_begin}</b> 到 <b>%{date_end}</b> 匹配所选流的流量。将下载 PCAP 文件。",
    ["about_to_extract"] = "即将从 <b>%{date_begin}</b> 提取流量到 <b>%{date_end}</b>",
    ["active_since"] = "活动自",
    ["archive"] = "存档",
    ["completed"] = "已完成",
    ["continuous_recording"] = "连续流量记录",
    ["continuous_recording_and_flows"] = "连续流量记录和流量可见性",
    ["delete_all_jobs"] = "删除所有作业",
    ["delete_all_jobs_confirm"] = "确认要删除所有提取作业吗？活动作业将被停止。",
    ["delete_job"] = "删除作业",
    ["delete_job_confirm"] = "确认要删除 Job %{job_id} 和相关的 Pcap 文件吗？",
    ["disabled"] = "禁用",
    ["disk_space"] = "最大磁盘空间",
    ["disk_space_note"] = "用于记录磁盘流量的最大磁盘空间。",
    ["disk_space_note_in_use"] = "%{in_use} GB 已被此实例使用。",
    ["download"] = "下载",
    ["download_nth_pcap"] = "下载 Pcap #{0}",
    ["dump_window"] = "转储窗口",
    ["edit_filter"] = "编辑过滤器",
    ["err_alloc"] = "内存分配错误",
    ["err_filter"] = "不支持的过滤器",
    ["err_open"] = "无法打开转储",
    ["err_stuck"] = "卡住或中断",
    ["err_unknown"] = "意外错误",
    ["ext_interface"] = "外部接口",
    ["extract_action"] = "提取",
    ["extract_now"] = "立即提取",
    ["extracted_bytes"] = "提取的字节数",
    ["extracted_pa​​ckets"] = "提取的数据包",
    ["extraction_scheduled"] = "已经安排了一个新的提取作业,请在文件准备好后立即检查 %{page} 页面以下载 pcap。此次提取的作业 ID 为 ",
    ["failed"] = "失败",
    ["failure"] = "失败",
    ["failure_note"] = "请检查日志跟踪并尝试重新启动服务。",
    ["filter_bpf"] = "过滤器（nBPF 格式）",
    ["filter_examples"] = "过滤示例",
    ["global_settings_note"] = "可以从<a href=\"%{url}\">流量记录首选项</a>中配置全局设置（例如许可证密钥）。",
    ["invalid_bpf"] = "无效过滤器",
    ["job_date_time"] = "工作创建日期/时间",
    ["job_id"] = "作业 ID",
    ["jobs"] = "工作",
    ["logs"] = "流量记录日志",
    ["max_extracted_pcap_bytes_description"] = "从记录数据中提取的 pcap 文件的最大大小。如果提取超过此大小,输出将被拆分为多个文件。",
    ["max_extracted_pcap_bytes_title"] = "最大提取文件大小",
    ["missing_data_msg"] = "流量记录已停止或重新开始。可能会丢失一些数据。",
    ["missing_parameters"] = "缺少必要参数",
    ["msg_err_missing_timeline"] = "无法确定当前运行的 n2disk 的时间线。只有当 n2disk 是最新的并且使用选项 --timeline-dir 运行时才能执行提取。",
    ["msg_err_timeline_path_not_existing"] = "在系统中找不到时间线路径。",
    ["msg_err_unable_to_access_timeline"] = "无法读取记录数据，权限不足。确保运行ntopng的用户与运行n2disk的用户在同一组中。",
    ["msg_err_unknown"] = "未知状态。",
    ["msg_external_providers_detected"] = "检测到外部流量记录提供者。ntopng可以自动将此接口绑定到外部提供者。您可以使用此页面让ntopng负责记录,或者您可以<a href=\"%{url} \"><i class=\"fas fa-cog\"></i> 配置</a> 一个外部流量记录提供者。",
    ["msg_ok"] = "准备提取",
    ["multiple_extracted_files"] = "流量提取生成了多个 %{mb} MB 的 pcap 文件,可以从首选项中更改此限制。单击下面的链路下载 pcap 文件。",
    ["no_data_in_interval"] = "磁盘上没有（或部分）所选时间间隔内的pcap数据。请注意pcap数据每分钟转储一次（最近一分钟不可用）并删除旧数据（请检查当前<A HREF=\"%{page}\">状态页面</A>中的转储窗口。请尝试选择不同的时间间隔。",
    ["no_file"] = "还没有转储文件",
    ["no_recorded_data"] = "在指定的时间范围内没有记录数据。流量记录仪需要一些时间来写入最近的数据。",
    ["not_found"] = "文件未找到",
    ["not_granted"] = "请求未获准",
    ["note_dump"] = "在以太网网络中,每个数据包都有一个 <A HREF=\"https://en.wikipedia.org/wiki/Ethernet_frame\">24 个字节的开销</A> [preamble,开始frame, CRC, IFG]。这样的开销被计入接口流量,但不会添加到转储到磁盘的流量中。因此,此页面中的字节数可能与接口图表中报告的字节数不同。",
    ["packet_dumps"] = "数据包转储",
    ["pcap_download"] = "下载 pcap",
    ["pcap_download_message"] = "开始解压并检查 %{page} 页面以在文件准备好后立即下载 pcap。",
    ["pcap_extract"] = "提取 pcap",
    ["processing"] = "正在处理",
    ["queue_as_job"] = "队列作为作业",
    ["recording"] = "记录中",
    ["restart"] = "重启",
    ["restart_service"] = "重启服务",
    ["settings"] = "设置",
    ["start_extraction"] = "开始提取",
    ["stop_job"] = "停止工作",
    ["stop_job_confirm"] = "确认要停止作业 %{job_id} 吗？",
    ["stopped"] = "停止",
    ["storage"] = "存储",
    ["storage_dir"] = "存储目录",
    ["storage_directory_config"] = "可以通过在 %{product} 配置文件中指定 <i>%{option}</i> 选项来更改存储目录路径。",
    ["storage_utilization"] = "存储利用率",
    ["storage_utilization_pcap"] = "pcap 存储利用率",
    ["traffic_extraction_jobs"] = "流量提取作业",
    ["traffic_extractions"] = "提取",
    ["traffic_on_disk"] = "磁盘流量",
    ["traffic_recording"] = "流量记录",
    ["traffic_recording_provider"] = "流量记录提供者",
    ["traffic_recording_settings"] = "流量记录设置",
    ["traffic_recording_status"] = "流量记录状态",
    ["unsupported_fields"] = "请注意,所选图表提供了提取工具不完全支持的流量视图,提取的流量将包含比预期更多的流量。",
    ["waiting"] = "等待",
    ["waiting_nospace"] = "正在等待（磁盘没有空间）",
  },
  ["traffic_report"] = {
    ["current_day"] = "今天",
    ["current_month"] = "本月",
    ["current_week"] = "本周",
    ["daily"] = "每日",
    ["header_daily"] = "每日报告",
    ["header_monthly"] = "每月报高",
    ["header_weekly"] = "每周报告",
    ["monthly"] = "每月",
    ["previous_day"] = "作天",
    ["previous_month"] = "上个月",
    ["previous_week"] = "上周",
    ["weekly"] = "每周",
  },
  ["tree_map"] = {
    ["hosts_treemap"] = "主机Tree Map",
  },
  ["unknown_devices"] = {
    ["active_only"] = "仅活动",
    ["all_devices"] = "所有设备",
    ["assign_device_pool"] = "将设备分配给池",
    ["assign_pool"] = "分配到池",
    ["device_name"] = "设备名称",
    ["devices_only_note"] = "如果没有使用此类 MAC 地址定义的池成员,则设备被视为未分配。基于 IP 的成员地址在此处不适用。",
    ["filter_devices"] = "过滤设备",
    ["first_seen"] = "第一次发现",
    ["inactive_only"] = "仅不活动",
    ["last_seen"] = "最后一次发现",
    ["no_pools"] = "必须创建主机池才能将未知设备分配。",
    ["select_pool"] = "请从下面的下拉列表中选择一个主机池以将 '%{mac}' 分配给：",
    ["unassigned_devices"] = "未分配的主机池设备",
  },
  ["updates"] = {
    ["available"] = "新",
    ["check"] = "检查更新",
    ["checking"] = "正在检查更新..",
    ["install"] = "安装更新",
    ["install_confirm"] = "您现在要更新并重启 ntopng 吗？",
    ["installing"] = "正在安装更新..",
    ["maintenance_expired"] = "请注意本产品维护已过期,升级后需要更新license。",
    ["no_updates"] = "无可用更新",
    ["update_failure_message"] = "更新过程中出现问题,请重试或手动尝试",
  },
  ["user_activity"] = {
    ["add_ntopng_user"] = "用户 '%{user}' 添加了新用户 %{new_user}。",
    ["add_pool"] = "用户 '%{user}' 创建了新池 <a href=\"%{pool_url}\">%{pool_name}</a>.",
    ["alerts_deleted"] = "用户 '%{user}' 清除了 '%{status}' 警告。",
    ["bind_pool_member"] = "用户 '%{user}' 添加了成员​​ %{member} 到池 <a href=\"%{pool_url}\">%{pool_name}</a>。",
    ["changed_preference"] = "用户 '%{user}' 更改了 %{pref} 的设置。",
    ["delete_ntopng_user"] = "用户 '%{user}' 删除了用户 %{old_user}.",
    ["delete_pool"] = "用户 '%{user}' 删除了池 <a href=\"%{pool_url}\">%{pool_name}</a>.",
    ["deleted_all_interfaces_data"] = "用户 '%{user}' 删除了所有接口的数据。",
    ["deleted_host_data"] = "用户 '%{user}' 删除了接口 %{ifname} 上主机 %{host} 的数据。",
    ["deleted_inactive_interfaces_data"] = "用户 '%{user}' 删除了所有不活动接口的数据。",
    ["deleted_interface_data"] = "用户 '%{user}' 删除了接口 %{ifname} 的数据。",
    ["deleted_network_data"] = "用户 '%{user}' 删除了接口 %{ifname} 上网络 %{network} 的数据。",
    ["disabled_preference"] = "用户 '%{user}' 禁用首选项 %{pref}.",
    ["edit_pool"] = "用户 '%{user}' 编辑了池 <a href=\"%{pool_url}\">%{pool_name}</a>.",
    ["enabled_preference"] = "用户 '%{user}' 已启用首选项 %{pref}.",
    ["exported_data"] = "用户 '%{user}' 导出了 '%{mode}' 主机的 JSON 数据。",
    ["exported_data_host"] = "用户 '%{user}' 为 '%{mode}' 主机 %{host} 导出了 JSON 数据。",
    ["file_downloaded"] = "用户 '%{user}' 下载了文件 '%{file}'。",
    ["flows_downloaded"] = "用户 '%{user}' 下载了从 %{from} 到 %{to} 的流数据。",
    ["host_json_downloaded"] = "用户 '%{user}' 为主机 %{host} 下载了 JSON 数据。",
    ["live_capture"] = "用户 '%{user}' 在接口 %{ifname} 上捕获了实时流量。",
    ["live_capture_host"] = "用户 '%{user}' 在接口 %{ifname} 上捕获了主机 %{host} 的实时流量。",
    ["live_capture_host_with_filter"] = "用户 '%{user}' 在接口 %{ifname} 上捕获了主机 %{host} 匹配过滤器 '%{filter}' 的实时流量。",
    ["live_capture_with_filter"] = "用户 '%{user}' 在接口 %{ifname} 上捕获了匹配过滤器 '%{filter}' 的实时流量。",
    ["live_extraction"] = "用户 '%{user}' 从接口 %{ifname} 下载了从 %{from} 到 %{to} 的流量。",
    ["live_extraction_with_filter"] = "用户 '%{user}' 从接口 %{ifname} 下载了从 %{from} 到 %{to} 匹配过滤器 '%{filter}' 的流量。",
    ["login_not_authorized"] = "用户 '%{user}' 登录被拒绝。",
    ["login_successful"] = "用户 '%{user}' 已登录。",
    ["password_changed"] = "用户 '%{pwd_user}' 的密码由 '%{user}' 更改,ip 为 '%{ip}'。",
    ["recording_disabled"] = "用户 '%{user}' 在接口 %{ifname} 上禁用了流量记录。",
    ["recording_enabled"] = "用户 '%{user}' 在接口 %{ifname} 上启用了流量记录。",
    ["set_host_alias"] = "用户 '%{user}' 将主机 %{host} 的别名设置为 %{alias}。",
    ["set_pool_config"] = "用户 '%{user}' 导入的池配置。",
    ["set_scripts_config"] = "用户 '%{user}' 导入的用户脚本配置。",
    ["snmp_device_ added"] = "用户 '%{user}' 添加了 SN​​MP 设备 %{ip}。",
    ["snmp_device_deleted"] = "用户 '%{user}' 删除了 SNMP 设备 %{ip}。",
    ["unknown_activity"] = "用户 '%{user}' 做了一些未知活动 (%{scope}),请报告。",
    ["unknown_activity_function"] = "用户 '%{user}' 做了一些未知活动 (%{name}),请报告。",
    ["user_ added"] = "用户 '%{add_user}' 由 '%{user}' 添加。",
    ["user_deleted"] = "用户 '%{del_user}' 已被 '%{user}' 删除。",
  },
  ["user_info"] = {
    ["missing_user_name_message"] = "缺少用户名",
    ["processes"] = "进程",
    ["processes_overview"] = "流概览",
    ["top_l4_protocols"] = "Top协议",
    ["top_l7_protocols"] = "Top应用",
  },
  ["user_scripts"] = {
    ["categories"] = "类别",
    ["category"] = "类别",
    ["category_internals"] = "内部结构",
    ["category_internals_descr"] = "%{product} 的内部功能（例如,内存管理以及主机和流生命周期）",
    ["category_network"] = "网络",
    ["category_network_descr"] = "网络行为和异常（例如,流量超过某个阈值,TCP 未按预期工作）",
    ["category_other"] = "其他",
    ["category_other_descr"] = "未分类脚本或不能包含在任何其他类别中的脚本的默认类别",
    ["category_security"] = "网络安全",
    ["category_security_descr"] = "安全行为和异常（例如,来自或与黑名单主机的通信,TCP 和 UDP 扫描）",
    ["category_system"] = "系统",
    ["category_system_descr"] = "%{product} 在其上运行的系统的功能（例如,磁盘空间已满,负载过高）",
    ["disable_all"] = "全部禁用",
    ["disable_all_message"] = "禁用当前配置的所有用户脚本。",
    ["disable_all_user_scripts"] = "禁用所有用户脚本",
    ["elephant_flows_descr"] = "> %{l2r_bytes} (L2R), > %{r2l_bytes} (R2L)",
    ["exceptions"] = "例外：%{exceptions}",
    ["long_lived_flows_descr"] = "> %{duration}",
    ["note_apply_to_default"] = "<b>默认</b> 配置也\"应用于\"任何未包含在任何其他配置中的池。",
    ["note_configsets"] = "用户脚本是在每个配置的基础上配置和启用/禁用的。可以创建多个配置,每个配置都可以\"应用于\"多个池。",
    ["note_what_are_user_scripts"] = "<a href=\"%{user_scripts_url}\">用户脚本</a> %{user_scripts_external} 会定期或在特定事件发生时执行。",
    ["stop_recording_after"] = "在 %{duration} 后停止记录",
    ["hint"] = {
      ["body"] = "配置异常 <a href='%{link_DHCP}' target='_about'>DHCP</a>, <a href='%{link_SMTP}' target='_about'>SMTP</a> a>、<a href='%{link_DNS}' target='_about'>DNS</a>、<a href='%{link_NTP}' target='_about'>NTP</a> 服务器。",
      ["title"] = "异常服务器",
    },
  },
  ["users"] = {
    ["add_new_user"] = "添加新用户",
    ["add_user"] = "添加用户",
    ["assigned_devices"] = "分配的设备",
    ["cat_protocols"] = "%{cat} 应用程序",
    ["categories"] = "类别",
    ["client_policy"] = "设备作为客户端",
    ["confirm_delete_user"] = "确认要删除用户\"%{user}\"吗？",
    ["delete_user"] = "删除用户",
    ["delete_user_error"] = "无法删除用户。请查看程序日志了解更多详情。",
    ["devices"] = "设备",
    ["edit"] = "编辑",
    ["edit_global_policies"] = "编辑默认策略",
    ["fallback_policy"] = "回退策略",
    ["full_name"] = "全名",
    ["global"] = "全局",
    ["global_policies"] = "默认策略",
    ["no_users"] = "没有用户",
    ["num_protocols"] = "应用程序数量",
    ["policy"] = "策略",
    ["protocols"] = "应用程序",
    ["reset_to_defaults"] = "重置为默认值",
    ["reset_to_defaults_confirm"] = "确认要将设备类型 \"%{devtype}\" 策略重置为默认值吗？",
    ["server_policy"] = "设备作为服务器",
    ["settings"] = "设置",
    ["total_daily_time_quota"] = "每日总时间配额",
    ["total_daily_traffic_quota"] = "每日总流量配额",
    ["unchanged"] = "不变",
    ["user_exists"] = "用户\"%{user}\"已经存在",
    ["user_x"] = "用户：%{user}",
    ["users"] = "用户",
    ["shapers"] = {
      ["default"] = "默认",
      ["drop"] = "Drop",
      ["pass"] = "Pass",
      ["slow"] = "Slow Pass",
      ["slower"] = "Slower Pass",
    },
  },
  ["validation"] = {
    ["FAILED_HTTP_REQUEST"] = "请求失败,请稍后再试。",
    ["empty_name"] = "请输入名称。",
    ["invalid_input"] = "请尊重输入模式。",
    ["missing_field"] = "请填写输入。",
  },
  ["vlan_details"] = {
    ["no_available_stats_for_vlan_message"] = "VLAN %{vlan_id} 没有可用的统计数据。请等待几分钟,让 %{product} 收集新的统计数据。",
    ["vlan_id_parameter_missing_or_invalid_message"] = "Vlan_Id 参数丢失或无效",
  },
  ["vlan_stats"] = {
    ["VLAN"] = "VLANs",
    ["vlan_id"] = "VLAN ID",
  },
  ["webhook_alert_endpoint"] = {
    ["password"] = "密码",
    ["shared_secret"] = "共享secret",
    ["url"] = "URL",
    ["username"] = "用户名",
  },
  ["widgets_page"] = {
    ["actions"] = "操作",
    ["begin_time"] = "开始时间",
    ["datasource"] = "数据源",
    ["end_time"] = "结束时间",
    ["interface"] = "接口",
    ["key"] = "key",
    ["name"] = "名称",
    ["params"] = "参数",
    ["type"] = "类型",
    ["widgets"] = "小部件",
    ["modal"] = {
      ["add_widget"] = "添加小部件",
      ["delete_description"] = "确认确定要删除 <b>%{widget}</b> 吗？页面中的小部件也将被禁用！",
      ["delete_widget"] = "删除小部件：%{widget}",
      ["edit_widget"] = "编辑小部件：%{widget}",
      ["embed_widget"] = "嵌入小部件标签",
      ["embed_widget_desc_1"] = "复制并粘贴这个 <b>HTML</b> 标签到您的页面中",
      ["embed_widget_desc_2"] = "请确保在您的页面中包含 ntop 小部件插件 <code>ntop-widgets.js</code>。",
    },
    ["validation"] = {
      ["abort_add"] = "中止小部件的添加？",
      ["abort_edit"] = "中止小部件的编辑？",
    },
  },
}

return lang
