local lang = {
  ["abort"] = "Cancella",
  ["acknowledge"] = "Conferma",
  ["acknowledge_alerts"] = "Conferma Allarmi",
  ["acknowledged_all"] = "Tutti",
  ["acknowledged_only"] = "Conferma Solamente",
  ["action"] = "Azione",
  ["actions"] = "Azioni",
  ["active_flows"] = "Flussi Attivi",
  ["activities"] = "Attività",
  ["activity"] = "Attività",
  ["add"] = "Aggiungi",
  ["admin_status"] = "Admin",
  ["advanced"] = "Avanzate",
  ["advanced_charts_test"] = "Test Grafici Avanzati",
  ["aggregation"] = "Aggregazione",
  ["alias"] = "Alias",
  ["all"] = "Tutto",
  ["all_hosts"] = "Tutti Gli Hosts",
  ["allowed"] = "Permesso",
  ["analysis"] = "Analisi",
  ["and"] = "e",
  ["and_x_more"] = "e %{num} altri",
  ["answer"] = "Risposta",
  ["any"] = "qualsiasi",
  ["app_name"] = "Nome Applicazione",
  ["application"] = "Applicazione",
  ["applications"] = "Applicazioni",
  ["applications_long"] = "Applicazioni",
  ["apply"] = "Applica",
  ["are_you_sure"] = "Sei Sicuro?",
  ["arp"] = "ARP",
  ["arp_type"] = "Tipo ARP",
  ["as"] = "AS",
  ["as_client"] = "come Client",
  ["as_number"] = "Numero AS",
  ["as_server"] = "come Server",
  ["asn"] = "ASN",
  ["asn_id"] = "ASN %{id}",
  ["attacker"] = "È un attaccante",
  ["auto_refresh_descr"] = "Clicca le frecce per abilitare/disabilitare l'Auto-Refresh (abilitato durante la rotazione). Clicca il Cog vicino alle frecce per aprire le preferenze degli allarmi",
  ["autolayout_description"] = "Auto-layout potrebbe cambiare drasticamente il layout attuale del network.",
  ["availability"] = "Availability",
  ["available"] = "Disponibile",
  ["avg_client_nw_latency"] = "Latenza Media del Network Client",
  ["avg_server_nw_latency"] = "Latenza Media del Network Server",
  ["backup"] = "Backup",
  ["begin"] = "Inizio",
  ["big_endian"] = "Anno/Mese/Giorno",
  ["bind"] = "Connetti",
  ["breakdown"] = "Ripartizione",
  ["bridge"] = "Bridge",
  ["broadcast"] = "Broadcast",
  ["broadcast_domain"] = "Dominio di Broadcast",
  ["broadcast_domain_too_large_description"] = "Triggera un alllarme quando viene rilevato traffico ARP tra due MACS addresses appartenenti a domini broadcast differenti",
  ["broadcast_domain_too_large_title"] = "Dominio broadcast troppo grande",
  ["broadcast_traffic"] = "Traffico di broadcast",
  ["browse_config"] = "Sfoglia",
  ["busy"] = "Occupato",
  ["bytes"] = "Bytes",
  ["cancel"] = "Cancella",
  ["categories"] = "Categorie",
  ["category"] = "Categoria",
  ["change"] = "Cambia",
  ["change_number_of_rows"] = "Cambia il numero di righe per pagina",
  ["change_policy"] = "Cambia la politica di un Host Pool",
  ["chart"] = "Grafico",
  ["cli_port"] = "Porta Cli",
  ["click_to_expand"] = "Click per espandere",
  ["client"] = "Client",
  ["client_and_server"] = "Client e Server",
  ["client_requested_server_name"] = "Nome Servere Richiesto",
  ["client_tls_supported_versions"] = "Protocolli TLS supportati dal Client",
  ["client_to_server"] = "Da Client a Server",
  ["clone"] = "Clona",
  ["close"] = "Chiudi",
  ["code"] = "Codice ICMP",
  ["comment"] = "Commento",
  ["configure"] = "Configura",
  ["confirm"] = "Conferma",
  ["confirm_delete_obs_point"] = "Sei sicuro di voler eliminare i punti di osservazione selezionati?",
  ["copied"] = "Copiato",
  ["count"] = "Conta",
  ["countries"] = "Paesi",
  ["create"] = "Crea",
  ["current_filter"] = "Escludi Check",
  ["current_filter_for"] = "Escludi tutti",
  ["dark"] = "Dark",
  ["data_flush_confirm"] = "Sei sicuro di voler svuotare tutti i dati?",
  ["date_from"] = "Data/Ora d'Inizio:",
  ["date_to"] = "Dta/Ora di Fine:",
  ["day"] = "Giorno",
  ["default"] = "Default",
  ["default_serie_name"] = "Serie 1",
  ["define"] = "Definisci",
  ["delete"] = "Elimina",
  ["delete_alerts"] = "Elimina Allarmi",
  ["delete_disabled_alerts"] = "Elimina Allarmi",
  ["delete_obs_point"] = "Elimina punti di osservazioneDelete Observation Point",
  ["description"] = "Descrizione",
  ["destination"] = "Destinazione",
  ["developer"] = "Sviluppatore",
  ["device_protocols_description"] = "In questa pagina è possibile configurare la lista di protocolli applicativi accettabili per tipo di dispositivo e direzione (dispositivo usato come client o server, rispetto al protocollo applicativo). Abilitando lo user script Applicazione Non Permessa per Device nella pagina di configurazione degli user script, vengono generati degli allarmi quando un protocollo anomalo viene rilevato.",
  ["devices"] = "Dispositivi",
  ["difference"] = "Differenza",
  ["disable"] = "Disabilita",
  ["disable_check_title"] = "Disabilita le selezioni correnti",
  ["disabled"] = "Disabilitato",
  ["discarded"] = "Scartati",
  ["dismiss"] = "Ignora",
  ["dns"] = "DNS",
  ["does_not_have"] = "non contiene",
  ["domain"] = "Dominio",
  ["double_num_flows_hosts"] = "Numero in virgola mobile",
  ["download"] = "Download",
  ["download_icon"] = "<i class='fas fa-arrow-down'></i>",
  ["download_x"] = "Download %{what}",
  ["downloaders"] = "Volume in Download",
  ["drilldown"] = "Drilldown",
  ["dscp"] = "DSCP",
  ["dscp_dst2src"] = "Server DSCP",
  ["dscp_src2dst"] = "Client DSCP",
  ["duplexstatus"] = "Duplex Status",
  ["duration"] = "Durata",
  ["edit"] = "Modifica",
  ["edit_check_title"] = "Modifica Opzioni Di Controllo",
  ["edit_configuration"] = "Modifica Configurazione",
  ["egress"] = "Uscita",
  ["empty"] = "Vuoto",
  ["enable"] = "Abilita",
  ["enable_auto_refresh"] = "Auto-Refresh",
  ["enable_check_title"] = "Abilita Le Opzioni Selezionate",
  ["enable_them"] = "Abilitali",
  ["enabled"] = "Abilitato",
  ["end"] = "Fine",
  ["enterpriseOnly"] = "Questa funzione è disponibile solo nell'edizione Enterprise di ntopng",
  ["error"] = "Errore",
  ["error_no_search_results"] = "Nessun risultato trovato. Si prega di modificare i criteri di ricerca.",
  ["error_not_granted"] = "Richiesta non permessa",
  ["error_rrd_cannot_complete_dump"] = "Non è stato possibile completare la scrittura degli RRD degli host locali. Il disco è lento oppure ci sono troppi host locali?",
  ["error_rrd_low_resolution"] = "Si sta richiedendo di recuperare i dati a una risoluzione inferiore rispetto a quella disponibile sull'RRD, il che comporterà dati non validi.<br>Se desideri ancora dati con tale granularità, ti preghiamo di sintonizzare le <a href=\"%{prefs}\">Impostazioni sulle serie temporali</a>",
  ["errors"] = "Errori",
  ["expired_csrf"] = "Impossibile effettuare l'azione richiesta. La pagina è scaduta. Effettuare l'operazione dopo aver ricaricato la pagina.",
  ["export"] = "Esporta",
  ["external_link"] = "Link esterno",
  ["external_link_url"] = "<button id='copyHttpUrl' class='btn btn-light btn-sm border ms-1' style='cursor: pointer;'><i class='fas fa-copy'></i></button> <a class='ntopng-external-link' href='https://%{url}' target='_blank'><i class='fas fa-external-link-alt'></i> %{url_name}</a>",
  ["factory_reset"] = "Factory Reset",
  ["filter"] = "Filtro",
  ["filter_by"] = "Filtra in base",
  ["fingerprint_note"] = "Il nome dell'applicazione è dettato solamente se ntopng è configurato per ricevere eBPF da ZMQ",
  ["first_seen"] = "Prima Visita",
  ["flow"] = "Flusso",
  ["flow_alerts_search_criteria"] = "Criteri di Ricerca per gli Allarmi sui Flussi",
  ["flow_count_per_l7"] = "Numero Flussi (Applicazioni)",
  ["flow_dump_not_working"] = "Controlla i log del servizio per maggiori informazioni. Potrebbero esserci problemi coi permessi o potrebbe non esserci spazio a sufficienza.",
  ["flow_dump_not_working_title"] = "Il dump dei flussi non sta funzionando",
  ["flow_search_criteria"] = "Criteri di Ricerca del Flusso",
  ["flow_search_from_aggregated"] = "(Aggregato)",
  ["flow_search_results"] = "Risultati di Ricerca Flusso",
  ["flow_size"] = "Dimensione Del Flusso",
  ["flows"] = "Flussi",
  ["flush_map_data"] = "Svuota Dati",
  ["forbidden"] = "Vietato",
  ["free"] = "Libero",
  ["frequency"] = "Frequenza",
  ["from"] = "da",
  ["full"] = "Full",
  ["gateway"] = "Gateway",
  ["geolocation_unavailable"] = "Leggere <a href='%{url}' target='%{target}'>queste istruzioni<i class='%{icon}'></i></a> per abilitarla.",
  ["geolocation_unavailable_title"] = "Geolocalizzazione non disponibile",
  ["granularity"] = "Granularità",
  ["has"] = "contiene",
  ["has_attacker"] = "Ha Un Attaccante",
  ["has_victim"] = "Ha Una Vittima",
  ["hassh_fingerprint"] = "Fingerprint HASSH",
  ["health"] = "Stato",
  ["help"] = "Aiuto",
  ["highest_avg_dst_score"] = "Media Del Miglior Score Di Un Flusso (Servers)",
  ["highest_avg_flow_size_per_l7"] = "Media Flusso Di Dimensione Magiore (Applicazioni)",
  ["highest_avg_l7_score"] = "Media Del Miglior Score Di Un Flusso (Applicazioni)",
  ["highest_avg_protocol_score"] = "Media Del Miglior Score Di Un Flusso(Protocolli)",
  ["highest_avg_src_score"] = "Media Del Miglior Score Di Un Flusso (Clients)",
  ["historical"] = "Dati Storici",
  ["host"] = "Host %{host}",
  ["host_address"] = "Indirizzo dell'Host",
  ["host_notes"] = "Note Host",
  ["hostname"] = "Nome Host",
  ["hosts"] = "Host",
  ["hosts_map"] = "Mappa Degli Hosts",
  ["hour"] = "Ora",
  ["howto_multiselect"] = "Per selezionare più elementi tenere premuto il tasto <kbd>Ctrl</kbd> (o <kbd>Cmd</kbd> su Macs).",
  ["http"] = "HTTP",
  ["http_s"] = "HTTP(S)",
  ["https"] = "HTTPS",
  ["icmp"] = "ICMP",
  ["icmpv4"] = "ICMPv4",
  ["icmpv6"] = "ICMPv6",
  ["id"] = "ID",
  ["iface_download"] = "%{iface} download",
  ["iface_upload"] = "%{iface} upload",
  ["import"] = "Importa",
  ["import_export_config"] = "Importa/Esporta Configurazione",
  ["incomingflows"] = "Numero Flussi in Entrata",
  ["info"] = "Informazioni",
  ["ingress"] = "Ingresso",
  ["interface"] = "Interfaccia",
  ["interface_id"] = "Id Interfaccia",
  ["interface_ifname"] = "Interfaccia %{ifname}",
  ["interfaces"] = "Interfacce",
  ["invalid_value"] = "Valore Non Valido",
  ["ip_address"] = "Indirizzo IP",
  ["ipv4"] = "IPv4",
  ["ipv6"] = "IPv6",
  ["ja3_client_fingerprint"] = "Fingerprint JA3 Client",
  ["ja3_fingerprint"] = "Fingerprint JA3",
  ["jump_to_chart"] = "Vista Grafico",
  ["jump_to_table"] = "Vista Tabella",
  ["key"] = "Chiave",
  ["l7_id"] = "App %{id}",
  ["l7_proto"] = "Protocollo L7",
  ["label"] = "Etichetta",
  ["label_alerts"] = "Etichetta Allarmi",
  ["language"] = "Lingua",
  ["last_method"] = "Metodo",
  ["last_query"] = "Richiesta",
  ["last_query_type"] = "Tipo Di Richiesta",
  ["last_return_code"] = "Codice Di Ritorno",
  ["last_seen"] = "Ultima Volta Visto",
  ["last_url"] = "URL",
  ["latency"] = "Latenza",
  ["layer_2"] = "Indirizzi Mac",
  ["legenda"] = "Legenda",
  ["level"] = "Livello",
  ["light"] = "Light",
  ["little_endian"] = "Giorno/Mese/Anno",
  ["loading"] = "Caricamento...",
  ["local_traffic"] = "Traffico Locale",
  ["lua_c_api"] = "Lua/C API",
  ["mac_address"] = "Indirizzo Mac",
  ["maps"] = "Mappe",
  ["max_rate"] = "Rate Massima",
  ["memory"] = "Memoria",
  ["middle_endian"] = "Mese/Giorno/Anno",
  ["missing_x_parameter"] = "Parametro \"%{param}\" mancante",
  ["model"] = "Modello",
  ["month"] = "Mese",
  ["mtu"] = "MTU",
  ["multicast"] = "Multicast",
  ["name"] = "Nome",
  ["name_txt"] = "Nome Testo",
  ["nation"] = "Nazione",
  ["netmask"] = "Netmask",
  ["network_id"] = "Network %{id}",
  ["networks"] = "Reti",
  ["new"] = "Nuovo",
  ["next"] = "Successivo",
  ["no"] = "No",
  ["noTraffic"] = "Alcun traffico è stato riportato per la specifica selezione di data/ora",
  ["no_attacker_no_victim"] = "Nessun No Attacker/Victim",
  ["no_data_available"] = "Nessun Dato Disponibile",
  ["no_flows"] = "Nessun Flusso",
  ["no_ports"] = "Nessuna Porta",
  ["no_results_found"] = "Nessun risultato trovato",
  ["no_talkers_for_the_host"] = "Non ci sono talker per l'host corrente",
  ["notAfter"] = "Validità Certificato TLS Cert. Fino A",
  ["notBefore"] = "Validità Certificato TLS Cert. Da",
  ["not_admin"] = "Richiesti privilegi d'amministratore",
  ["note_flow_search_allowed_networks"] = "Le seguenti reti sono consentite per l'utente: %{nets}.",
  ["note_flow_search_allowed_networks_applications"] = "Gli host che non appartengono a reti consentite non saranno visualizzati",
  ["note_flow_search_allowed_networks_counter"] = "I contatori nei risultati della ricerca sono totali ma solamente i flussi con client e server nelle reti consentite saranno mostrati.",
  ["note_flow_search_allowed_networks_ipv6"] = "I flussi IPv6 non sono visualizzati quando ci sono reti consentite IPv6 non-nulle.",
  ["note_flow_search_allowed_networks_talkers"] = "Gli host che non appartengono a reti consentite non saranno visualizzati",
  ["notes"] = "NOTA:",
  ["num_different_ports"] = "%{num} porte. Nessuna fa più del %{threshold}%% di traffico.",
  ["num_uses"] = "Numero Utilizzi",
  ["observation_point_alias"] = "Alias Punti di Osservazione",
  ["observation_point_id"] = "ID Punto Di Osservazione",
  ["observationpoints"] = "Punti Di Osservazione",
  ["observations"] = "Osservazioni",
  ["obsolete_mysql"] = "Supporto MySQL deprecato",
  ["obsolete_mysql_message"] = "ntopng sta salvando i dati del flusso in MySQL, ma l'esportazione MySQL è deprecata. Perfavore consultare <a href=\\\"https://www.ntop.org/guides/ntopng/clickhouse.html\\\">ClickHouse</a> e migrare i dati.",
  ["obsolete_nindex"] = "Il Supporto nIndex è deprecato",
  ["obsolete_nindex_message"] = "ntopng sta salvando i dati del flusso in nindex, ma l'esportazione MySQL è deprecata. Perfavore consultare <a href=\\\"https://www.ntop.org/guides/ntopng/clickhouse.html#migration-from-nindex\\\">e migrare i dati</a> to <a href=\\\"https://www.ntop.org/guides/ntopng/clickhouse.html\\\">ClickHouse</a>.",
  ["off"] = "Off",
  ["offline"] = "Offline (Nessuna Connessione Internet)",
  ["open_documentation"] = "Apri nella Documentazione",
  ["operating_systems"] = "Sistemi Operativi",
  ["optional_field"] = "(Campo opzionale)",
  ["os"] = "OS",
  ["other"] = "Altro",
  ["others"] = "Altri",
  ["outgoingflows"] = "Numero Flussi in Uscita",
  ["overview"] = "Panoramica",
  ["packets"] = "Pacchetti",
  ["passive"] = "Passivo",
  ["peers"] = "Peers",
  ["percentage"] = "Percentuale",
  ["perform_autolayout"] = "Performa Auto-Layout",
  ["periodicity"] = "Periodicità",
  ["periodicity_map"] = "Mappa Periodicità",
  ["pkts"] = "Pkts",
  ["pkts_distribution"] = "Distribuzione Pacchetti",
  ["plugin_browser"] = "Visualizzatore Plugin: %{plugin_name}",
  ["plugin_contents"] = "File",
  ["plugins"] = "Plugins",
  ["pollers"] = "Pollers",
  ["port"] = "Porta",
  ["ports"] = "Interfacce",
  ["posted"] = "Pubblicato il",
  ["predominant_direction"] = "Direzione Principale",
  ["previous"] = "Precedente",
  ["proto_info"] = "Info Protocollo",
  ["protocol"] = "Protocollo",
  ["protocol_categories"] = "Categorie dei Protocolli",
  ["protocols"] = "Protocolli",
  ["purged_error_message"] = "Molto probabilmente è scaduto e %{product} lo ha eliminato dalla memoria. È possibile modificare le impostazioni del timeout d'inattività dalle <a href=\"%{url}\"><i class=\"fas fa-flask\"></i> Impostazioni</a>.",
  ["push_notification"] = "Notifiche Push",
  ["quotas"] = "Quote",
  ["ram_available"] = "Disponibile",
  ["ram_total"] = "Totale",
  ["ram_used"] = "Usata",
  ["reason"] = "Motivo",
  ["received"] = "Ricevuto",
  ["recipient"] = "Destinatario",
  ["records_to_show"] = "Records da mostrare",
  ["redirect"] = "Reindirizzare",
  ["refresh"] = "Aggiorna",
  ["reload_graph"] = "Ricarica Grafico",
  ["reload_map"] = "Ricarica Mappa",
  ["remote_hosts"] = "Remote Hosts",
  ["remote_networks"] = "Reti Remote",
  ["remote_probe_clock_drift"] = "Orologio del probe remoto non allineato",
  ["remove"] = "Rimuovi",
  ["request_failed_message"] = "La richiesta è fallita!",
  ["reset"] = "Reset",
  ["reset_drops_if_message"] = "Sei sicuro di voler resettare il contatore della perdita di pacchetti nell'interfaccia?",
  ["reset_drops_if_title"] = "Resetta Le Statistiche Di Perdita Pacchetti Nell'Interfaccia",
  ["reset_drops_ifs_message"] = "Sei Sicuro Di Voler Resettare I Contatori Di Perdite Di Pacchetti Di Tutte Le Interfacce?",
  ["reset_drops_ifs_title"] = "Resetta Le Statistiche Di Perdita Pacchetti Nelle Interfacce",
  ["reset_if_message"] = "Sei Sicuro Di Voler Resettare Le Statistiche Delle Interfacce?",
  ["reset_if_title"] = "Resetta Le Statistiche Delle Interfacce",
  ["reset_ifs_message"] = "Sei Sicuro Di Voler Resettare Le Statistiche Di Tutte Le Interfacce?",
  ["reset_ifs_title"] = "Resetta Le Statistiche Di Tutte Le Interfacce",
  ["restore"] = "Ripristina",
  ["retry"] = "Riprova",
  ["role"] = "Ruolo",
  ["running"] = "In esecuzione",
  ["save"] = "Salva",
  ["save_configuration"] = "Salva Configurazione",
  ["save_settings"] = "Salva Impostazioni",
  ["score"] = "Score",
  ["score_as_client"] = "Score Come Client",
  ["score_as_server"] = "Score Come Server",
  ["score_distribution"] = "Distribuzione Score",
  ["score_not_accounted"] = "Non Contabilizzato",
  ["scripts"] = "Scripts",
  ["search"] = "Cerca",
  ["search_host"] = "Cerca",
  ["seen_since"] = "Attivo da",
  ["sent"] = "Inviato",
  ["server"] = "Server",
  ["server_name"] = "Nome Server",
  ["server_names"] = "Nomi Server",
  ["server_to_client"] = "Da Server a Client",
  ["service"] = "Servizio",
  ["service_acceptance"] = "Stato Servizio",
  ["service_map"] = "Mappa Servizi",
  ["set"] = "Imposta",
  ["settings"] = "Impostazioni",
  ["short_broadcast"] = "B",
  ["short_multicast"] = "M",
  ["shortcuts"] = "Shortcuts",
  ["show_hide_filters"] = "Mostra/Nascondi Filtri",
  ["showing_x_to_y_rows"] = "Visualizzate righe dalla %{x} alla %{y} di %{tot} totali",
  ["similarity"] = "Somiglianze",
  ["speed"] = "Velocità",
  ["srv_port"] = "Porta Srv",
  ["ssh"] = "SSH",
  ["ssid"] = "SSID",
  ["start"] = "Avvia",
  ["status"] = "Stato",
  ["stop"] = "Ferma",
  ["success"] = "Operazione Completata",
  ["summary"] = "Sommario",
  ["system"] = "Sistema",
  ["system_alerts_status"] = "Allarmi",
  ["system_interface"] = "Interfaccia Sistema",
  ["system_interfaces_status"] = "Stato delle Interfacce",
  ["system_status"] = "Stato del Sistema",
  ["talkers"] = "Talkers",
  ["tcp_flags"] = "Flags TCP",
  ["tcp_stats"] = "Statistiche TCP",
  ["telemetry"] = "Telemetria",
  ["threshold"] = "Soglia",
  ["throughput"] = "Throughput",
  ["time"] = "Ora",
  ["time_in"] = "in",
  ["timeframe"] = "Periodo",
  ["timeline_test"] = "Timeline Test",
  ["tls"] = "TLS",
  ["tls_certificate"] = "Certificato TLS",
  ["tls_version"] = "Versione TLS",
  ["today"] = "Oggi",
  ["toggle_dark_theme"] = "Attiva/Disattiva Dark Theme",
  ["toggle_white_theme"] = "Attiva/Disattiva White Theme",
  ["too_many_exports"] = "Troppe perdite di esportazione flusso",
  ["too_many_flows"] = "Troppi flussi",
  ["too_many_hosts"] = "Troppi host",
  ["tools"] = "Strumenti",
  ["top_cli_per_proto"] = "Top Clients per Applicazione",
  ["top_dst_asn"] = "Migliori Sistemi Autonomi (Servers)",
  ["top_dst_score"] = "Miglior Score Totale (Servers)",
  ["top_hosts_per_proto_in_flows"] = "Top Hosts per Applicazione (Flussi)",
  ["top_l4_proto"] = "Protocolli Migliori",
  ["top_l7_cat"] = "Categorie Migliori",
  ["top_l7_proto"] = "Applicazioni Migliori",
  ["top_src_asn"] = "Migliori Sistemi Autonomi (Clients)",
  ["top_src_score"] = "Miglior Score Totale (Clients)",
  ["total"] = "Totale",
  ["total_alerts"] = "Allarmi Totali",
  ["total_incoming_alerted_flows"] = "Totale Flussi Allarmati come Server",
  ["total_incoming_unreachable_flows"] = "Totale Flussi Unreachable in Ingresso",
  ["total_outgoing_alerted_flows"] = "Totale Flussi Allarmati come Client",
  ["total_outgoing_unreachable_flows"] = "Totale Flussi Unreachable in Uscita",
  ["total_retransmissions_rcvd"] = "Totale Ritrasmissioni Ricevute",
  ["total_retransmissions_sent"] = "Totale Ritrasmissioni Inviate",
  ["total_score"] = "Score Totale",
  ["traffic"] = "Traffico",
  ["traffic_cli_to_srv"] = "Cli <i class='fas fa-arrow-right'></i> Srv",
  ["traffic_policy"] = "Politica del Traffico",
  ["traffic_srv_to_cli"] = "Srv <i class='fas fa-arrow-right'></i> Cli",
  ["type"] = "Tipo ICMP",
  ["unable_to_copy_to_clickboard"] = "Impossibile copiare nella clipboard",
  ["undo"] = "Annulla",
  ["unicast_only"] = "Solo Unicast",
  ["unknowers"] = "Volume del Traffico Sconosciuto",
  ["unknown"] = "Sconosciuto",
  ["unlimited"] = "&infin;",
  ["unreachable"] = "Non Raggiungibile",
  ["upload"] = "Upload",
  ["upload_icon"] = "<i class='fas fa-arrow-up'></i>",
  ["uploaders"] = "Volume in Upload",
  ["used_by"] = "Usato da",
  ["value"] = "Valore",
  ["values"] = "Valori",
  ["version"] = "La tua versione è %{vers}.",
  ["via"] = "Via",
  ["victim"] = "È Una Vittima",
  ["views"] = "Viste",
  ["vlan"] = "VLAN",
  ["vlan_id"] = "VLAN",
  ["volume"] = "Volume",
  ["volume_size"] = "Volume",
  ["warning"] = "Attenzione",
  ["week"] = "Settimana",
  ["welcome"] = "Benvenuto",
  ["welcome_to"] = "Benvenuto su %{product}",
  ["wrong_date_range"] = "Finestra temporale invalida, per favore cambiala.",
  ["yes"] = "Si",
  ["about"] = {
    ["about"] = "Informazioni",
    ["about_x"] = "Informazioni su %{product}",
    ["active"] = "attivo",
    ["alert_defines"] = "Definizioni degli Allarmi",
    ["alert_queries"] = "Queries",
    ["alerts_dropped"] = "Perse",
    ["alerts_stored"] = "Salvate",
    ["built_on"] = "Costruito su",
    ["callback_directories"] = "Cartelle degli User Script",
    ["checks"] = "Checks",
    ["configure_dhcp_range"] = "Configura range DHCP",
    ["contribute_to_project"] = "Contribuisci",
    ["copyright"] = "Copyright",
    ["cpu_load"] = "Carico CPU",
    ["cpu_states"] = "Stati della CPU",
    ["create_license"] = "E' consigliabile creare il file <code>ntopng.license</code> come da istruzioni ricevute",
    ["create_license_l"] = "E' necessario creare il file <code>/etc/ntopng.license</code> poichè usato da altre applicazioni del bundle",
    ["data_directory"] = "Dati",
    ["datadir_warning"] = "Informativa <A HREF=\"https://www.ntop.org/support/faq/migrate-the-data-directory-in-ntopng/\">migrazione dati</A>",
    ["defs_directories"] = "Cartelle delle Definizioni",
    ["dhcp_monitoring_title"] = "Monitoraggio Host DHCP",
    ["dhcp_range_missing_warning"] = "La preferenza <a href='%{url}'>%{name}</a> non ha effetto. Configurare il <a href='%{dhcp_url}'>range DHCP</a>.",
    ["directories"] = "Cartelle",
    ["dropped_store_queries_alerts"] = "Allarmi Queries/Salvate/Perse",
    ["flow_checks_directory"] = "Flusso",
    ["forced_community_notification"] = "Edizione Community forzata con --community con una licenza installata. Rimuovere questa opzione per utilizzare la licenza installata",
    ["forced_community_notification_short"] = "Edizione Community edition forzata con --community",
    ["host_checks_directory"] = "Host",
    ["host_identifier_warning"] = "Questa interfaccia monitora host DHCP. Cambiare la preferenza <a href='%{url}'>%{name}</a> a \"Mac Address\".",
    ["idle"] = "idle",
    ["interface_callbacks_directory"] = "Interfaccia",
    ["iowait"] = "iowait",
    ["last_log"] = "Traccia dell'ultimo Log",
    ["licence"] = "Licenza",
    ["licence_expires_in"] = "La licenza scadrà tra %{time}",
    ["licence_generation"] = "Clicca sull'URL sopra per generare la tua licenza Professional, o <br> compra una licenza nell'<a href=\"%{purchase_url}\">e-shop</a>. Se sei un'organizzazione no-profit, di ricerca o di studio<br>leggi <a href=\"%{universities_url}\">questo</a>.",
    ["licence_status"] = "Licenza",
    ["maintenance"] = "Manutenzione: Fino a %{_until} [%{days_left} giorni residui]",
    ["maintenance_expired"] = "La manutenzione su %{product} è scaduta",
    ["maintenance_left"] = "Fino a %{_until} [%{days_left} giorni restanti]",
    ["maxmind"] = "Questo prodotto supporta i databases di geolocalizzazione forniti da <a class='ntopng-external-link' href=\"%{maxmind_url}\">MaxMind <i class=\"fas fa-external-link-alt\"></i></a> e <a class='ntopng-external-link' href='https://db-ip.com'>DB-IP <i class=\"fas fa-external-link-alt\"></i></a>.",
    ["network_callbacks_directory"] = "Reti",
    ["new_major_available"] = "Una nuova versione stabile di %{product} (v.%{version}) è disponibile per il <a href=\"%{url}\">download</a>: è consigliato aggiornare",
    ["ntop_blog"] = "Blog di ntop",
    ["platform"] = "Piattaforma",
    ["ram_memory"] = "RAM",
    ["readme_and_manual"] = "Manuale Utente",
    ["remote_probe_times"] = "L'ora locale %{local_time} e l'ora della sonda remota è %{remote_time}",
    ["report_issue"] = "Segnala un Problema",
    ["runtime_status"] = "Stato in Tempo Reale",
    ["save_licence"] = "Salva Licenza",
    ["scripts_directory"] = "Scripts",
    ["server_time"] = "Tempo del Server",
    ["specify_licence"] = "Specifica qui la tua licenza ntopng",
    ["startup_line"] = "Linea D'avvio",
    ["suggest_feature"] = "Suggerisci una nuova funzionalità",
    ["system"] = "Sistema",
    ["system_id"] = "System Id",
    ["telegram"] = "Aiuto e Notizie",
    ["telemetry_data_opt_out_msg"] = "Modifiche alla scelta attuale possono essere fatte in qualsiasi momento dalle <a href=\"%{url}\"><i class=\"fas fa-flask\"></i>Preferenze</a>.",
    ["too_many_exports"] = "Riavvia %{product} rimozione in corso <code>-F</code> opzione o usare storage più veloce.",
    ["upgrade_to_professional"] = "Aggiorna alla versione Professional",
    ["uptime"] = "Attivo da",
    ["version"] = "Versione",
    ["you_have_too_many_flows"] = "Ci sono troppi host/flussi per la configurazione di %{product} e questo potrebbe causare perdite di pacchetti e alto carico sulla CPU. Riavvia %{product} aumentando il valore dei parametri -x e -X",
    ["you_have_too_many_hosts"] = "Troppi host per %{product}. Questo può causare perdita di pacchetti è alto carico. Aumentare con -x/-X e riavviare %{product}",
    ["you_need_to_sync_remote_probe_time"] = "E' necessario sincronizzare il clock di ntopng con quello dei probe remoti.",
  },
  ["active_monitoring_stats"] = {
    ["active_monitoring"] = "Monitoraggio Attivo",
    ["active_monitoring_description"] = "Monitora lo stato e il tempo di risposta di host specifici",
    ["add_icmp"] = "Aggiungi Monitor ICMP",
    ["add_pool"] = "Nuovo Pool Monitoraggio Attivo",
    ["add_record"] = "Aggiungi Record di Monitoraggio",
    ["alert_status"] = "Stato Allarmi",
    ["alerted"] = "Allarmati",
    ["am_note_http"] = "Le misurazioni http e https scaricano una pagina web utilizzando HTTP e HTTPS",
    ["am_note_icmp"] = "Le misurazioni icmp e icmp6 fanno un ping dell'host utilizzando ICMP e ICMPv6",
    ["app_connect_descr"] = "Il valore di \"Connessione App\" indica il tempo necessaro a completare l'handshake TLS.",
    ["availability"] = "Availability",
    ["bad_action_param"] = "Parametro \"action\" non valido",
    ["checking_host"] = "Controllo host...",
    ["confirm_delete"] = "Vuoi veramente eliminare questo record?",
    ["default_interface"] = "Interfaccia Default",
    ["download_speed"] = "Velocità di Download",
    ["edit_record"] = "Modifica Record di Monitoraggio",
    ["empty_action"] = "Qualcosa è andato male (\"action\" vuota). Ricarica la pagina e riprova.",
    ["host_add_ok"] = "L'host %{host} è stato aggiunto con successo!",
    ["host_delete_ok"] = "L'host %{host} è stato eliminato con successo!",
    ["host_edit_ok"] = "L'host %{host} è stato modificato con successo!",
    ["host_exists"] = "Host \"%{host}\" già esistente",
    ["host_not_exists"] = "Host \"%{host}\" non trovato",
    ["http_download"] = "Throughput",
    ["icmp_continuous"] = "ICMP Continuo (IPv4)",
    ["icmp_continuous_v6"] = "ICMP Continuo (IPv6)",
    ["invalid_combination"] = "Combinazione measurement/host non valida",
    ["invalid_host"] = "L'host specificato non è valido",
    ["jitter"] = "RTT / Jitter Medi",
    ["last_measurement"] = "Ultima Misurazione",
    ["measurement"] = "Misurazione",
    ["measurement_time"] = "Tempo di Misurazione",
    ["msec"] = "ms",
    ["no_updates_yet"] = "Nessun aggiornamento disponibile",
    ["not_alerted"] = "Non Allarmati",
    ["note3"] = "Questo plugin genera traffico verso gli host configurati per misurarne la performance",
    ["note_alert"] = "Un'alert viene generata quando il valore registrato eccede la soglia configurata",
    ["note_alert_dashed"] = "L'elemento tratteggiato indica l'attuale ora in corso di misurazione",
    ["note_availability"] = "L'availability mostra la frazione del tempo, nell'ultimo giorno, in cui la misura è rientrata entro la soglia configurata.",
    ["note_periodicity_change"] = "Cambiare la periodicità causa la cancellazione delle vecchie serie temporali",
    ["other_http_descr"] = "La somma del \"Name Lookup\" e di \"Altro\" costituisce il tempo totale in di completamento della risposta HTTP.",
    ["other_https_descr"] = "La somma di \"Connessione App\", \"Name Lookup\" and \"Altro\" costituisce il tempo totale in di completamento della risposta HTTPS.",
    ["pool"] = "Pool",
    ["rtt_vs_jitter"] = "RTT / Jitter Medio",
    ["speedtest"] = "Speedtest",
    ["throughput"] = "Throughput",
    ["upload_speed"] = "Velocità di Upload",
    ["validation"] = {
      ["abort_add"] = "Annulla?",
      ["abort_edit"] = "Annulla?",
    },
  },
  ["activities_page"] = {
    ["host_activity"] = "Attività Host",
  },
  ["activity_descriptions"] = {
    ["5min"] = "Esegue i task ai 5 minuti. Quando RRD è abilitato, scrive anche le serie temporali degli host.",
    ["daily"] = "Esegue i task giornalieri.",
    ["dequeue_flows_for_hooks"] = "Identifica le entry idle nelle hash table e le elimina dalla memoria, esegue gli user script sui flussi, esporta i flussi sul database configurato.",
    ["discover"] = "Scansiona la rete per cercare dispositivi. L'attività non fa niente se nessuno scan è stato richiesto o programmato.",
    ["hourly"] = "Esegue i task orari.",
    ["housekeeping"] = "Emette notifiche per gli allarmi e le salva su disco, effettua i task di estrazione di n2disk, ricarica le liste quando necessario.",
    ["minute"] = "Esegue i task al minuto, scrive le serie temporali al minuto. Quando InfluxDB è abilitato, scrive le serie temporali degli host.",
    ["second"] = "Scrive le serie temporali delle interfacce.",
    ["stats_update"] = "Calcola il throughput degli host, delle reti, e di altri elementi di traffico.",
    ["timeseries"] = "Preleva e scrive le serie temporali accodate per la scrittura.",
  },
  ["alert"] = {
    ["alert_error"] = {
      ["configuration"] = {
        ["no_attacker"] = "Attaccante richiesto, nessun attaccante impostato.",
        ["no_status_key"] = "Status key richiesta, nessuna statusk key trovata.",
        ["no_victim"] = "Vittima richiesta, nessuna vittima impostata.",
      },
    },
    ["anomalies"] = {
      ["active_flows_as_client"] = "Comportamento Dei Flussi Attivi Come Client",
      ["active_flows_as_server"] = "Comportamento Dei Flussi Attivi Come Server",
      ["contacts"] = "Comportamento Contatti",
      ["score_as_client"] = "Comportamento Dei Flussi Attivi Come Client",
      ["score_as_server"] = "Comportamento Dei Flussi Attivi Come Server",
    },
  },
  ["alert_behaviors"] = {
    ["l7_traffic_behavior"] = "Traffico RX/TX",
    ["score_behavior"] = "Score",
    ["traffic_rx_behavior"] = "Traffico RX",
    ["traffic_tx_behavior"] = "Traffico TX",
  },
  ["alert_details"] = {
    ["cannot_be_found_message"] = "Questo Allarme Non È Ritrovabile.",
  },
  ["alert_entities"] = {
    ["am_host"] = "Monitoraggio Attivo",
    ["asn"] = "ASN",
    ["flow"] = "Flusso",
    ["host"] = "Host",
    ["host_pool"] = "Host Pool",
    ["interface"] = "Interfaccia",
    ["l7"] = "Protocollo L7",
    ["mac"] = "Dispositivo",
    ["network"] = "Network Locale",
    ["other"] = "Altro",
    ["snmp_device"] = "Dispositivo SNMP",
    ["system"] = "Sistema",
    ["test"] = "Test",
    ["user"] = "Utente",
  },
  ["alert_messages"] = {
    ["a_new_device_has_connected"] = "Il nuovo dispositivo <a href=\"%{url}\">%{device}</a> si è connesso alla rete.",
    ["alert_ids_ips_jail_add"] = "Host %{host} aggiunto alla lista degli host pool vietati alle %{when}",
    ["alert_ids_ips_jail_remove"] = "Host %{host} rimosso alla hosts pool vietati alle %{when}",
    ["alert_port_too_many_macs"] = "Troppi MACs su su porta non-trunk <a href=\\\"%{url}\\\">%{ip}</a> Id Interfaccia <a href=\\\"%{port_url}\\\">%{port}</a> [%{value} %{op} %{threshold} MACs]",
    ["anomalous_tcp_flags"] = "%{entity} ha %{sent_or_rcvd} troppi TCP RST rispetto ai SYN [Rapporto: %{ratio}%%]",
    ["attack_mitigation_via_snmp_failure"] = "Impossibile cambiare l'admin status dell'interfaccia <a href=\"%{port_url}\">%{port}</a> del dispositivo SNMP <a href=\"%{url}\">%{device}</a> a %{admin_down}: %{granularity} <b>%{metric}</b> oltrepassata da %{entity} [%{value} %{op} %{threshold}]. Controllare la write community configurata per il dispositivo SNMP.",
    ["attack_mitigation_via_snmp_success"] = "Admin status dell'interfaccia <a href=\"%{port_url}\">%{port}</a> del dispositivo SNMP <a href=\"%{url}\">%{device}</a> cambiato a %{admin_down}: %{granularity} <b>%{metric}</b> oltrepassata da %{entity} [%{value} %{op} %{threshold}]",
    ["broadcast_domain_info"] = "E' improbabile del traffico ARP tra questi due IP poichè sembrano appartenere a domini di broadcast differenti. Controllare configurazioni di host e reti.",
    ["broadcast_domain_too_large"] = "Rilevato traffico ARP da <a href=\"%{src_mac_url}\">%{src_mac}</a>/<a href=\"%{spa_url}\">%{spa}</a> a <a href=\"%{dst_mac_url}\">%{dst_mac}</a>/<a href=\"%{tpa_url}\">%{tpa}</a>. Questo traffico ARP è inatteso in quanto questi IP sembrano appartenere a differenti domini di broadcast. Controllare la configurazione della rete e degli host.",
    ["contacted_peers"] = "Troppi Peers contattai da %{host} %{host_category}. Come client: [%{value_cli} > %{dyn_threshold_cli}]. Come server: [%{value_srv} > %{dyn_threshold_srv}].",
    ["contacted_peers_as_cli"] = "Troppi Peers contattati come client da %{host} %{host_category} [%{value_cli} > %{dyn_threshold_cli}].",
    ["contacted_peers_as_srv"] = "Troppi Peers contattati come server da %{host} %{host_category} [%{value_srv} > %{dyn_threshold_srv}].",
    ["device_has_connected"] = "Il dispositivo <a href=\"%{url}\">%{device}</a> si è connesso alla rete.",
    ["device_has_disconnected"] = "Il dispositivo <a href=\"%{url}\">%{device}</a> si è disconnesso dalla rete.",
    ["flow_flood_attacker"] = "%{entity} è un flooder di flussi [%{value} &gt; %{threshold} flussi]",
    ["flow_flood_victim"] = "%{entity} sta subendo un flood di flussi [%{value} &gt; %{threshold} flows]",
    ["flow_number_anomaly"] = "%{role} Flussi: %{value} [Range aspettato: %{lower_bound} ... %{upper_bound}]",
    ["host_alert_countries_contacts"] = "Troppe nazioni contattate da %{entity} [%{value} &gt; %{threshold} nazioni",
    ["host_alert_dangerous_host"] = "Limite del punteggio superato di %{entity} [Score: %{score}]",
    ["host_alert_dns_server_contacts"] = "Troppi server DNS contattati da %{entity} [%{value} &gt; %{threshold} servers]",
    ["host_alert_dns_traffic"] = "Traffico DNS superato di %{entity} [%{value} %{op} %{threshold}]",
    ["host_alert_domain_names_contacts"] = "Troppi Domain Names contattati [Soglia: %{threshold}][Contattati: %{contacts}]",
    ["host_alert_ntp_server_contacts"] = "Troppi server NTP contattati da%{entity} [%{value} &gt; %{threshold} servers]",
    ["host_alert_ntp_traffic"] = "Traffico NTP superato di %{entity} [%{value} %{op} %{threshold}]",
    ["host_alert_p2p_traffic"] = "Traffico P2P superato di %{entity} [%{value} %{op} %{threshold}]",
    ["host_alert_pkt_threshold"] = "Soglia pacchetto superata di %{entity} [%{value} %{op} %{threshold}]",
    ["host_alert_score"] = "Score superato di %{entity} [%{value} %{op} %{threshold}]",
    ["host_alert_smtp_server_contacts"] = "Troppi server SMTP contattati da %{entity} [%{value} &gt; %{threshold} servers]",
    ["host_entity"] = "host %{entity_value}",
    ["host_log"] = "Log da <a href=\"%{url}\">%{host}</a> (%{facility}) %{line}",
    ["host_pool_has_connected"] = "L'host pool <a href=\"%{url}\">%{pool}</a> si è connesso alla rete.",
    ["host_pool_has_disconnected"] = "L'host pool <a href=\"%{url}\">%{pool}</a> si è disconnesso dalla rete.",
    ["host_remote_to_remote"] = "L'host remoto <a href=\"%{url}\">%{ip}</a> %{mac} ha contattato un host remoto. I flussi remoto-a-remoto sono disponibili nella pagina degli <a href=\"%{flow_alerts_url}\">allarmi dei flussi</a>.",
    ["http_download_failed"] = "Host %{host} %{numeric_ip} Controllo del throughput non riuscito.",
    ["icmp_flood_attacker"] = "%{entity} è un flooder ICMP [%{value} &gt; %{threshold} client ICMP Flussi/sec per ~3 sec]",
    ["icmp_flood_victim"] = "%{entity} è sotto attacco flood ICMP [%{value} &gt; %{threshold} server ICMP Flussi/sec per ~3 sec]",
    ["iface_alerts_dropped"] = "%{num_dropped} allarmi persi sull'interfaccia <a href=\"%{url}\">%{iface}</a>.",
    ["influxdb_dropped_points"] = "Punti persi. Impossibile esportarli a \"%{influxdb}\".",
    ["influxdb_migration_msg"] = "Troppi local host. Passare a <a href='%{url}'>InfluxDB</a> per migliorare la performance.",
    ["influxdb_partial_write"] = "Questo può essere risolto impostando \"max-values-per-tag = 0\" nella configurazione del server InfluxDB.",
    ["influxdb_write_error"] = "Si è verificato un errore nell'invio delle serie temporali a \"%{influxdb}\": %{err}",
    ["interface_entity"] = "interfaccia %{entity_value}",
    ["ip_outside_dhcp_range_router_ip"] = "[Router: <a href=\"%{router_url}\">%{router_ip}</a>]",
    ["ip_outsite_dhcp_range"] = "Il dispositivo <a href=\"%{client_url}\">%{client_mac}</a> ha ricevuto l'indirizzo IP <a href=\"%{client_ip_url}\">%{client_ip}</a> il quale non rientra nel <a href=\"%{dhcp_url}\">range DHCP configurato</a>. [Dispositivo che ha dato l'IP: <a href=\"%{sender_url}\">%{sender_mac}</a>]",
    ["lldp_arc_added"] = "Nuovo link aggiunto tra dispositivi SNMP <a href=\"%{url1}\">%{node1}</a> e <a href=\"%{url2}\">%{node2}</a>",
    ["lldp_arc_removed"] = "Link rimosso tra dispositivi SNMP <a href=\"%{url1}\">%{node1}</a> e <a href=\"%{url2}\">%{node2}</a>",
    ["mac_ip_association_change"] = "IP %{ip} ha cambiato associazione da <a href=\"%{old_mac_url}\">%{old_mac}</a> a <a href=\"%{new_mac_url}\">%{new_mac}</a>",
    ["measurement_too_high_msg"] = "Misurazione troppo alto per l'%{host} %{numeric_ip} [%{am_value} %{unit} > %{threshold} %{unit}].",
    ["measurement_too_low_msg"] = "Misurazione troppo bassa per l'%{host} %{numeric_ip} [%{am_value} %{unit} < %{threshold} %{unit}].",
    ["misbehaving_flows_ratio"] = "%{entity} ha %{sent_or_rcvd} troppi flussi misbehaving rispetto al totale dei flussi [Rapporto: %{ratio}%%]",
    ["network_entity"] = "network %{entity_value}",
    ["nfq_flushed"] = "La coda dei pacchetti dell'interfaccia <a href=\"%{url}\">%{name}</a> è stata svuotata. Coda %{pct}%% piena con %{tot} pacchetti e %{dropped} persi.",
    ["ngi_trust_event"] = "Attività anormale rilevata per il dispositivo <a href=\\\"%{mac_url}\\\">%{mac}</a> alle %{time}. [Ultimo stato: %{last_state}][Non modificato da: %{state_unchanged_since}][Grado di anormalità: %{abnormality_grade}]",
    ["ngi_trust_event_released"] = "Dispositivo <a href=\\\"%{mac_url}\\\">%{mac}</a> lo stato è tornato normale alle %{time}. [Ultimo stato: %{last_state}][Non modificato da: %{state_unchanged_since}][Grado di anormalità: %{abnormality_grade}]",
    ["no_clickhouse"] = "ClickHouse non abilitato.",
    ["no_influxdb"] = "Si prega di <A HREF=\"%{url}\">impostare InfluxDB come database</A> per le serie temporali e quindi ricaricare questa pagina",
    ["ntopng_anomalous_termination"] = "Avvio dopo terminazione anonima (<a href=\"%{url}\">riporta bug</a>)",
    ["ntopng_start"] = "Avviato",
    ["ntopng_stop"] = "Fermato",
    ["ntopng_update"] = "Avviato dopo un aggiornamento software",
    ["open_files_limit_too_small"] = "Ntopng ha rilevato che il numero massimo di file che MySQL può aprire è potenzialmente troppo piccolo. Questo potrebbe causare la pardita di dati sui flussi per via di errori quali [Out of resources when opening file './ntopng/flowsv6#P#p22.MYD' (Errcode: 24 - Too many open files)][23]. Assicurati di aumentare open_files_limit o, se vuoi semplicemente ignorare questo avvertimento, disabilita la relativa opzione dalle impostazioni.",
    ["periodic_activity_not_executed"] = "L'attività periodica \"%{script}\" è stata accodata ma non eseguita. Nessun thread disponibile per l'esecuzione. [In attesa da %{pending_since}]",
    ["periodicity_update_ended"] = "Il flusso periodico è terminato: %{host_info1} <i class=\\\"fas fa-exchange-alt fa-lg\\\" aria-hidden=\\\"true\\\" data-original-title=\\\"\\\" title=\\\"\\\"></i> %{host_info2} [%{l7_proto}] %{info}",
    ["periodicity_update_new"] = "%{host_info1} <i class=\\\"fas fa-exchange-alt fa-lg\\\" aria-hidden=\\\"true\\\" data-original-title=\\\"\\\" title=\\\"\\\"></i> %{host_info2} [%{l7_proto}] %{info} &egrave; adesso periodico (<i class=\\\"fas fa-wave-square\\\" title=\\\"Frequency\\\"></i> %{frequenza} sec)",
    ["ping_host_unreachable_v3"] = "L'Host %{host} %{numeric_ip} non è raggiungibile.",
    ["potentially_dangerous_protocol_description"] = "Questo flusso può esser dannoso per la rete in quanto utilizza una applicazione vecchia o vulnerabile.",
    ["proto_bytes_quotas"] = "di bytes %{proto}",
    ["proto_time_quotas"] = "di tempo %{proto}",
    ["remote_to_local_insecure_proto"] = "Protocollo Insuro Remoto verso Locale [Categoria: %{ndpi_category}][Tipo: %{ndpi_breed}]",
    ["remote_to_remote"] = "Host remoto %{entity} ha contattato un host remoto",
    ["score_number_anomaly"] = "%{role} Score: %{score} [Limiti attesi: %{lower_bound} ... %{upper_bound} <i class='fas fa-question-circle' title='Questo comportamento viene calcolato automaticamente analiizando il comportamento dell'host nel tempo come spiegato nella guida utente `Basic Concept->Alerts->Behavior Alerts` section'></i>][Cybersecurity Score: %{cat_sec}% | Network Score: %{cat_net}%][Flow Alerts: <a href='%{flow_params}'><i class='fas fa-search-plus fa-lg'></i></a>][Host Alerts Passati: <a href='%{host_params_historical}'><i class='fas fa-search-plus fa-lg'></i></a>][Host Alerts Attivi: <a href='%{host_params_engaged}'><i class='fas fa-search-plus fa-lg'></i></a>]",
    ["score_number_anomaly_threshold"] = "%{metric} superata di %{entity} [%{value} > %{threshold}]",
    ["score_threshold"] = "Score superato di %{entity} as %{cli_or_srv} [%{value} > %{threshold}]",
    ["shell_script_executed"] = "Shell script '%{script_exec_comm}' eseguito in risposta ad un allarme: '%{alert_type}",
    ["slow_periodic_activity"] = "L'attività periodica \"%{script}\" sta girando da troppo tempo [più di %{max_duration}] oppure è stata eseguita in ritardo (era bloccata in coda).",
    ["slow_purge"] = "L'eliminazione delle <a href=\"%{url}\">entry idle nella hash table</a> di %{iface} è lenta. Questo può causare un utilizzo troppo elevato di memoria, perdita di accuratezza, e perdita di allarmi. [&gt %{max_idle}%%]",
    ["speedtest_failed"] = "Speedtest fallito",
    ["subject_quota_exceeded"] = "L'host pool <a href=\"%{url}\">%{pool}</a> ha superato la quota %{subject} [%{value} &gt; %{quota}]",
    ["suspicious_dga_domain"] = "Dominio DGA sospetto %{href} [ %{domain} %{info}]",
    ["syn_flood_attacker"] = "%{entity} è un SYN flooder [%{value} &gt; %{threshold} SYN]",
    ["syn_flood_victim"] = "%{entity} sta ricevendo un SYN flood [%{value} &gt; %{threshold} SYN]",
    ["syn_scan_attacker"] = "%{entity} sta facendo un SYN Scan [%{value} &gt; %{threshold} SYN]",
    ["syn_scan_victim"] = "%{entity} sta ricevendo un SYN Scan [%{value} &gt; %{threshold} SYN]",
    ["test_failed"] = "Test Fallito",
    ["threshold_crossed"] = "Soglia <b>%{metric}</b>/%{granularity} oltrepassata da %{entity} [%{value} %{op} %{threshold}]",
    ["too_many_drops"] = "%{iface} ha troppi pacchetti persi [&gt %{max_drops}%%]",
    ["too_many_flows"] = "%{iface} ha troppi flussi. Aumentare con l'opzione %{option}",
    ["too_many_flows_details"] = "Vuoi raddoppiare il numero massimo di flussi?",
    ["too_many_flows_err"] = "Impossibile raddoppiare il numero massimo di flussi. Perfavore modificare -X dal file di configurazione e riavviare ntopng.",
    ["too_many_flows_title"] = "Numero massimo di flussi doppio",
    ["too_many_hosts"] = "%{iface} ha troppi host. Aumentare con l'opzione %{option}",
    ["too_many_hosts_details"] = "Vuoi raddoppiare il numero massimo di host correnti?",
    ["too_many_hosts_err"] = "Impossibile raddoppiare il numero massimo di Host. Perfavore modificare -X dal file di configurazione e riavviare ntopng.",
    ["too_many_hosts_title"] = "Numero massimo di host doppio",
    ["unknown_contacted_peers"] = "Troppi Peers contattati da %{host} %{host_category}.",
    ["x_alerts"] = "%{num} allarmi",
  },
  ["alerts_dashboard"] = {
    ["active_flows_anomaly"] = "Anomalie Flussi Attivi",
    ["add_filter"] = "Aggiungi filtro",
    ["alert"] = "Allarme",
    ["alert_counts"] = "Conteggio",
    ["alert_duration"] = "Durata",
    ["alert_ids_ips_jail_add"] = "Host bloccato aggiunto",
    ["alert_ids_ips_jail_remove"] = "Host bloccato rimosso",
    ["alert_periodicity_update"] = "Periodicità cambiata",
    ["alert_port_too_many_macs_title"] = "Troppi MACs in Non-Trunk",
    ["alert_severity"] = "Severità",
    ["alert_snmp_device_reset_description"] = "Il dispositivo SNMP <a href=\"%{url}\">%{device}</a> è stato riavviato",
    ["alert_snmp_device_reset_title"] = "Dispositivo SNMP riavviato",
    ["alert_snmp_interface_mac_changed_description"] = "Il MAC <a href=\"%{mac_url}\">%{mac}</a> visto sul dispositivo SNMP <a href=\"%{url}\">%{device}</a> interfaccia <a href=\"%{port_url}\">%{port}</a> era stato precedentemente visto sul dispositivo SNMP <a href=\"%{prev_device_url}\">%{prev_device}</a> interfaccia <a href=\"%{prev_port_url}\">%{prev_port}</a>",
    ["alert_snmp_interface_mac_changed_title"] = "Cambiamento di posizione SNMP di un MAC",
    ["alert_type"] = "Tipo",
    ["alert_unexpected_behavior_description"] = "Fai scattare un allarme quando un comportamento anomalo è identificato  the specified %{type}/s. <b>Restart is required</b> dopo aver abilitato/disabilitato l'allarme.",
    ["alert_unexpected_behavior_title"] = "Comportamento %{type} Inaspettato",
    ["alerts"] = "Allarmi",
    ["alerts_dashboard"] = "Dashboard degli Allarmi",
    ["alerts_ts"] = "Serie temporali degli allarmi del processo",
    ["alerts_ts_description"] = "Genera serie temporali sugli allarmi del processo",
    ["all_hosts"] = "Tutti gli Hosts",
    ["anomalous_tcp_flags"] = "Flag TCP anomali",
    ["attack_mitigation_snmp_description"] = "Imposta l'admin status SNMP della access port a cui è connesso l'host quando il client score eccede la soglia configurata",
    ["attack_mitigation_snmp_title"] = "Mitigazione Attacchi via SNMP",
    ["binary_application_transfer"] = "Trasferimento File Sospetto",
    ["binary_application_transfer_url"] = "%{type_icon} Trasferimento file sospetto rilevato [%{url}]",
    ["blacklisted_country"] = "Paese in blacklist",
    ["blacklisted_country_descr"] = "Emetti un allarme quando gli host sono contattati o contattano certi paesi",
    ["blacklisted_flow"] = "Flusso Blacklistato",
    ["blocked_flow"] = "Flusso Bloccato",
    ["broadcast_domain_too_large"] = "Dominio di broadcast",
    ["checks_calls_drops"] = "Perse Chiamate agli Checks Calls",
    ["clickhouse_monitor"] = "ClickHouse monitor",
    ["clickhouse_monitor_description"] = "Monitoraggio della qualità di ClickHouse",
    ["client_and_server_countries_blacklisted"] = "Il paese del client %{cli_country} e il paese del server %{srv_country} sono in blacklist",
    ["client_country_blacklisted"] = "Il paese del client %{country} è in blacklist",
    ["connection_issues"] = "Problemi di connessione",
    ["contacted_peers_description"] = "Genera un allarme quando un host contatta un numero inaspettato di clients o servers",
    ["contacted_peers_title"] = "Allarme Peers Contattati",
    ["critical"] = "Critico",
    ["custom_period"] = "Periodo Personalizzato",
    ["dangerous_host_description"] = "Genera un allarme e aggiungi un host alla lista della pool bloccati per 30 minuti se la soglia impostata viene supera per più di 5 minuti consecutivi. Agli host nella host pool bloccati viene impedito di generare traffico quando viene usato nProbe in modalità IPS.",
    ["dangerous_host_title"] = "Host Pericoloso",
    ["debug"] = "Debug",
    ["device_connection"] = "Connessione Dispositivo",
    ["device_disconnection"] = "Disconnessione Dispositivo",
    ["dns_anomaly"] = "Anomalia DNS",
    ["element"] = "Elemento",
    ["emergency"] = "Emergenza",
    ["engaged_for_longest"] = "Allarmi Passati rimasti attivi per più tempo",
    ["error"] = "Errore",
    ["experimental_checks"] = "Sperimentali",
    ["experimental_checks_description"] = "Abilita controlli sperimentali: flag TCP, rapporto flussi anomalo",
    ["external_alert"] = "Allarmi di sorgenti esterne",
    ["fail2ban"] = "Comando Fail2Ban eseguito",
    ["five_mins"] = "Ultimi 5 Minuti",
    ["flow_alert_origins"] = "Sorgenti d'Allarme del flusso",
    ["flow_alert_targets"] = "Destinazioni d'Allarme del flusso",
    ["flow_anomaly"] = "Anomalia di flusso",
    ["flow_checks_calls_drops_description"] = "%{num_drops} <a href='%{url}'>checks sui flussi</a> non sono stati eseguiti.",
    ["flow_flood"] = "Flows Flood",
    ["flow_flood_attacker"] = "Flows Flood Attacccante",
    ["flow_flood_victim"] = "Flows Flood Vittima",
    ["flow_logger"] = "Logger dei Flussi",
    ["flow_logger_descr"] = "Scrive i flussi su console o su file",
    ["flow_low_goodput"] = "Low Goodput Ratio",
    ["flow_misbehaviour"] = "Flusso Misbehaving",
    ["flow_risk"] = "Flow Risk",
    ["ghost_network_detected"] = "Rete fantasma rilevata",
    ["ghost_network_detected_description"] = "La sottorete %{network} non appartiene ad alcuna delle <a href='%{url}'>%{entity} reti</a>.",
    ["ghost_networks"] = "Reti Fantasma",
    ["ghost_networks_description"] = "Emetti un allarme quando vengono rilevate reti sconosciute",
    ["host_alert_countries_contacts"] = "Allarme nazioni contattate",
    ["host_alert_dns_server_contacts"] = "DNS Server Contacts",
    ["host_alert_dns_traffic"] = "Traffico DNS",
    ["host_alert_domain_names_contacts"] = "Triggera un alarme quando il numero di Domain Names contattati supera una certa soglia",
    ["host_alert_ntp_server_contacts"] = "NTP Server Contacts",
    ["host_alert_ntp_traffic"] = "Traffico NTP",
    ["host_alert_p2p_traffic"] = "Traffico P2P",
    ["host_alert_smtp_server_contacts"] = "SMTP Server Contacts",
    ["host_log"] = "Host Log",
    ["host_pool_connection"] = "Connessione Host Pool",
    ["host_pool_disconnection"] = "Disconnessione Host Pool",
    ["host_score_description"] = "Calcola lo score dell'host ed emetti un allarme quando esso oltrepassa la soglia specificata.",
    ["http_rcvd_sent_ratio"] = "[HTTP Rcvd Repls. / Sent Reqs. %{ratio}%% &lt; %{threshold}%%][Repls: %{replies}][Reqs: %{requests}]",
    ["http_sent_rcvd_ratio"] = "[HTTP Sent Repls. / Rcvd Reqs. %{ratio}%% &lt; %{threshold}%%][Repls: %{replies}][Reqs: %{requests}]",
    ["icmp_anomaly"] = "Anomalia ICMP",
    ["icmp_flood"] = "ICMP Flood",
    ["iec_invalid_transition"] = "IEC104 Invalid Transition",
    ["iec_unexpected_type_id"] = "TypeId Inaspettato IEC104",
    ["inactivity"] = "Inattività",
    ["influxdb_dropped_points"] = "Punti persi su InfluxDB",
    ["influxdb_error"] = "Errore InfluxDB",
    ["influxdb_export_failure"] = "Fallimento durante l'esportazione a InfluxDB",
    ["influxdb_monitor"] = "Monitor InfluxDB",
    ["influxdb_monitor_description"] = "Monitor Stato di InfluxDB",
    ["info"] = "Info",
    ["internal_issues"] = "Issue Interne",
    ["involving_msg"] = "Allarmi sui Flussi che coinvolgono",
    ["known_proto_on_non_std_port"] = "Applicazione Nota su Porta Non-Standard",
    ["known_proto_on_non_std_port_full"] = "Applicazione %{app} rilevata sulla porta non-standard %{port}",
    ["last_day"] = "Ultimo Giorno",
    ["last_hour"] = "Ultima Ora",
    ["last_minute"] = "Ultimo Minuto",
    ["lateral_movement"] = "Movimento Laterale",
    ["lateral_movement_descr"] = "Rilevato servizio negato %{info} %{href}",
    ["list_download_failed"] = "Download Lista Fallito",
    ["list_download_succeeded"] = "Download Lista OK",
    ["local_only"] = "Solo Hosts Locali",
    ["local_origin_remote_target"] = "Origine Locale - Destinazione Remota",
    ["login_failed"] = "Login fallito",
    ["mac_ip_association_change"] = "Cambio IP/MAC",
    ["malicious_signature_detected"] = "Signature possibilmente malevola",
    ["many_replies"] = "%{count} %{what} risposte",
    ["many_requests"] = "%{count} %{what} richieste",
    ["memory_ts"] = "Serie temporali della memoria del processo",
    ["memory_ts_description"] = "Genera serie temporali della memoria del processo",
    ["misbehaving_flows_ratio"] = "Troppi flussi misbehaving",
    ["misconfigured_app"] = "Problema di Configurazione",
    ["misconfigured_dhcp_range"] = "Range DHCP malconfigurato",
    ["ndpi_dns_suspicious_traffic_title"] = "Traffico DNS sospetto",
    ["ndpi_http_invalid_content_title"] = "Contenuto HTTP invalido",
    ["ndpi_http_numeric_ip_host_title"] = "IP host numerico HTTP",
    ["ndpi_http_suspicious_url_title"] = "Header HTTP sospetto",
    ["ndpi_http_suspicious_user_agent_title"] = "User-Agent HTTP Sospetto",
    ["ndpi_malformed_packet_title"] = "Pacchetto Malformato",
    ["ndpi_malicious_ja3_title"] = "Probabile JA3 pericoloso",
    ["ndpi_malicious_sha1"] = "Certificato SHA1 pericoloso",
    ["ndpi_risky_asn_title"] = "ASN rischioso",
    ["ndpi_risky_domain_title"] = "Dominio Rischioso",
    ["ndpi_smb_insecure_version_title"] = "Versione SMB Non Sicura",
    ["ndpi_ssh_obsolete_client_title"] = "Client SSH Obsoleto",
    ["ndpi_ssh_obsolete_server_title"] = "Server SSH Obsoleto",
    ["ndpi_suspicious_dga_domain_title"] = "Dominio DGA Sospetto",
    ["ndpi_tls_cert_validity_too_long_title"] = "Validità Certificato TLS Troppo Lunga",
    ["ndpi_tls_certificate_expired_title"] = "Certificato TLS Scaduto",
    ["ndpi_tls_certificate_mismatch_title"] = "Certificato TLS Non Corrispondente",
    ["ndpi_tls_missing_sni_title"] = "TLS SNI Mancante",
    ["ndpi_tls_not_carrying_https_title"] = "TLS che non trasporta HTTPS",
    ["ndpi_tls_obsolete_version_title"] = "Versione TLS Obsoleta",
    ["ndpi_tls_selfsigned_certificate_title"] = "Certificato TLS Auto-Segnato",
    ["ndpi_tls_suspicious_esni_usage_title"] = "Utilizzo Sospetto TLS ESNI",
    ["ndpi_tls_weak_cipher_title"] = "Cipher TLS Debole",
    ["ndpi_unsafe_protocol_title"] = "Protocollo Non Sicuro",
    ["ndpi_url_possible_rce_injection_title"] = "Probabile RCE",
    ["ndpi_url_possible_sql_injection_title"] = "Probabile iniezione SQL",
    ["ndpi_url_possible_xss_title"] = "Probabile XSS",
    ["new_device"] = "Nuovo Dispositivo",
    ["nfq_flushed"] = "Coda dei Pacchetti Svuotata",
    ["ngi_trust_event"] = "NGI Trust Event",
    ["no_alert"] = "Nessun Allarme",
    ["no_alerts"] = "Nessun Allarme",
    ["none"] = "Nulla",
    ["not_engaged"] = "Non in Corso",
    ["notice"] = "Notizia",
    ["one_day"] = "Ultimo Giorno",
    ["one_hour"] = "Ultima Ora",
    ["one_min"] = "Ultimo Minuto",
    ["one_reply"] = "%{count} %{what} risposta",
    ["one_request"] = "%{count} %{what} richiesta",
    ["out_of_x_total_packets"] = "out of %{tot} Packets",
    ["periodic_activity_not_executed"] = "Attività periodica non eseguita",
    ["periodic_activity_not_executed_descr"] = "Emette un allarme quando una attività periodica è accodata ma non viene eseguita perché tutti i thread sono già occupati",
    ["periodicity_changed"] = "Periodicità",
    ["periodicity_changed_descr"] = "periodicità cambiata %{info} %{href}",
    ["periodicity_is_aperiodic_descr"] = "Servizio non più peiodico %{info} %{href}",
    ["periodicity_is_periodic_descr"] = "Servizio Periodico %{info} %{href}",
    ["ping_issues"] = "Ping",
    ["possible_bot_flow_description"] = "Probabile Flusso con Bot come server",
    ["possible_bot_flow_title"] = "Probabile Bot",
    ["potentially_dangerous_protocol"] = "Applicazione potenzialmente pericolosa",
    ["process"] = "Processo",
    ["quota_exceeded"] = "Quota Superata",
    ["remote_access_alert_descr"] = "Accesso remoto terminato [durata: %{sec} sec]",
    ["remote_access_description"] = "Genera un allarme quando una sessione di Accesso Remoto è terminata",
    ["remote_access_title"] = "Acccesso Remoto",
    ["remote_connection_alert_descr"] = "Host %{host} ha %{connections} flussi attivi che usano un protocollo di accesso remoto",
    ["remote_connection_description"] = "Genera un allarme quando un host ha almeno un flusso attivo che usa un protocollo di accesso remoto",
    ["remote_connection_title"] = "Connessione Remota",
    ["remote_only"] = "Solo Hosts Remoti",
    ["remote_origin_local_target"] = "Origine Remota - Destinazione Locale",
    ["remote_to_local_insecure_proto"] = "Protocollo non sicuro da Remoto a Locale",
    ["remote_to_remote"] = "Remoto verso Remoto",
    ["score_anomaly"] = "Anomalia nel punteggio",
    ["score_threshold_cross"] = "Soglia Di Score Superata",
    ["search_criteria"] = "Impostazioni Dashboard",
    ["server_country_blacklisted"] = "Il paese del server %{country} è in blacklist",
    ["shell_script"] = "Endpoint Shell Script Executed",
    ["sites_behaviour_description"] = "Genera un allarme quando un host contatta un numero inaspettato di hosts",
    ["slow_periodic_activity"] = "Attività Periodica Lenta",
    ["slow_periodic_activity_descr"] = "Emette un allarme quando una attività periodica impiega troppo tempo per completare la sua esecuzione",
    ["slow_purge"] = "Eliminazione lenta delle Entry Idle",
    ["snmp_device_is_unresponsive"] = "Il dispositivo SNMP <a href=\"%{url}\">%{device}</a> non risponde",
    ["snmp_device_reset_script_description"] = "Emette un allarme quando è stato rilevato il restart di un dispositivo SNMP",
    ["snmp_interface_mac_changed_script_description"] = "Emette un allarme quando un MAC address ha cambiato interfaccia o dispositivo SNMP",
    ["snmp_interface_mac_changed_script_title"] = "Cambiamento di posizione SNMP di un MAC",
    ["snmp_interfaces_rsi"] = "RSI Interfacce SNMP",
    ["snmp_interfaces_rsi_descr"] = "Emette un allarme quando si rileva il cambiamento di traffico sulle interfacce SNMP (sperimentale)",
    ["snmp_port_changed_duplex_status"] = "L'interfaccia <a href=\"%{port_url}\">%{port}</a> del dispositivo SNMP <a href=\"%{url}\">%{device}</a> ha cambiato il suo duplex status in %{new_op}",
    ["snmp_port_changed_operational_status"] = "L'interfaccia <a href=\"%{port_url}\">%{port}</a> sul dispositivo SNMP <a href=\"%{url}\">%{device}</a> è ha cambiato in suo stato in %{new_op}",
    ["snmp_port_duplexstatus_change"] = "Cambio Duplex Status di Interfaccia SNMP",
    ["snmp_port_duplexstatus_change_description"] = "Emette un allarme quando una interfaccia di un dispositivo SNMP cambia il suo duplex status",
    ["snmp_port_errors"] = "Errori Interfaccia SNMP",
    ["snmp_port_errors_description"] = "Emette un allarme quando aumentano i contatori di discard o errori di una interfaccia di un dispositivo SNMP",
    ["snmp_port_errors_increased"] = "Contatori di discard/errori aumentati per l'interfaccia <a href=\"%{port_url}\">%{port}</a> del dispositivo SNMP <a href=\"%{url}\">%{device}</a>",
    ["snmp_port_load_threshold_exceeded"] = "Traffico Interfaccia SNMP",
    ["snmp_port_load_threshold_exceeded_message"] = "Il traffico dell'interfaccia <a href=\"%{port_url}\">%{port}</a> del dispositivo SNMP <a href=\"%{url}\">%{device}</a> ha superato la soglia configurata [soglia percentuale: %{threshold}%%][Ingresso: %{in_load}%%][Uscita: %{out_load}%%]",
    ["snmp_port_status_change"] = "Cambio Stato Porta",
    ["snmp_port_status_change_description"] = "Emette un allarme quando una interfaccia di un dispositivo SNMP cambia il suo operational status",
    ["starting_on"] = "partendo da",
    ["submit"] = "Aggiorna Dashboard",
    ["suspicious_activity"] = "Attività Sospetta",
    ["suspicious_device_protocol"] = "Protocollo Dispositivo Sospetto",
    ["tcp_syn_flood"] = "SYN Flood TCP",
    ["tcp_syn_flood_attacker"] = "TCP SYN Flood Attacker",
    ["tcp_syn_flood_victim"] = "TCP SYN Flood Victim",
    ["tcp_syn_scan"] = "TCP SYN Scan",
    ["tcp_syn_scan_attacker"] = "TCP SYN Scan Attacker",
    ["tcp_syn_scan_victim"] = "TCP SYN Scan Victim",
    ["threashold_cross"] = "Superamento Soglia",
    ["tls_old_protocol_version"] = "Versione TLS Obsoleta [Versione: %{version}]",
    ["tls_protocol_version"] = "Versione TLS: %{version}",
    ["too_many_drops"] = "Perdite di Pacchetti",
    ["too_many_flows"] = "Allarme per Troppi Flussi",
    ["too_many_flows_description"] = "Emette un allarme quando il numero di flussi attivi è troppo alto",
    ["too_many_hosts"] = "Allarme per Troppi Host",
    ["too_many_hosts_description"] = "emette un allarme quando il numero di host attivi à troppo alto",
    ["top_addresses"] = "Migliori MAC addresses",
    ["top_alerts"] = "Allarmi Principali",
    ["top_cli"] = "Clients Principali",
    ["top_hosts"] = "Hosts Principali",
    ["top_networks"] = "Networks Principali",
    ["top_srv"] = "Servers Principali",
    ["total_alerts"] = "Allarmi Totali",
    ["trailing_msg"] = "Finestra Temporale",
    ["trailing_msg_compact"] = "Window",
    ["unexpected_behavior_anomaly_description"] = "%{entity} <a href=\\\"%{href}\\\"><i class=\\\"fas fa-lg fa-chart-area\\\"></i></a> [%{type_of_behavior}: %{value} | Expected Range: %{lower_bound} ... %{upper_bound}]",
    ["unexpected_host_behavior_description"] = "%{host} [%{type_of_behaviour}][Value: %{value}][Expected Range: %{lower_bound} ... %{upper_bound}]",
    ["unexpected_host_behaviour_act_flows_title"] = "Comportamento Inaspettato Dei Flussi Attivi Di Un Host",
    ["unexpected_host_behaviour_contacts_title"] = "Comportamento Inaspettato Dei Contatti Di Un Host",
    ["unexpected_host_behaviour_score_title"] = "Comportamento Inaspettato Del Score Di Un Host",
    ["unresponsive_device"] = "Dispositivo Non Risponde",
    ["user_activity"] = "Attività Utente",
    ["warning"] = "Attenzione",
    ["web_mining"] = "Mining Web",
    ["x_lost"] = "%{lost} Perso",
    ["x_ooo"] = "%{ooo} Fuori Ordine",
    ["x_retx"] = "%{retx} Ritrasmissione",
    ["tooltips"] = {
      ["top_addresses"] = "Mac Addresses con più allarmi",
      ["top_alerts"] = "Alerts più visti",
      ["top_hosts"] = "Hosts con più allarmi",
      ["top_networks"] = "Networks con più allarmi",
    },
  },
  ["alerts_thresholds_config"] = {
    ["active_local_hosts"] = "Host Locali",
    ["active_local_hosts_threshold_descr"] = "Numero di host locali attivi",
    ["activity_time"] = "Tempo d'Attività",
    ["alert_active_description"] = "Tempo d'attività dall'ultimo controllo.",
    ["alert_bytes_description"] = "Bytes di differenza livello 2 (inviati + ricevuti)",
    ["alert_bytes_title"] = "Allarme sul Traffico",
    ["alert_dns_description"] = "Bytes di differenza livello 2 (inviati + ricevuti) per il traffico DNS rilevato",
    ["alert_flows_description"] = "Differenza di flussi (come client + come server)",
    ["alert_flows_title"] = "Allarme sui Flussi",
    ["alert_idle_description"] = "Tempo d'inattività dall'ultimo pacchetto visto (secondi)",
    ["alert_network_egress_description"] = "Differenza di bytes in uscita",
    ["alert_network_ingress_description"] = "Differenza di bytes in ingresso",
    ["alert_network_inner_description"] = "Differenza di bytes interni",
    ["alert_ntp_description"] = "Genera un allarme quando la differenza di bytes Layer 2 (inviati + ricevuti) per traffico NTP supera la soglia",
    ["alert_p2p_description"] = "Differenza di bytes di livello 2 (inviati + ricevuti) per il traffico peer-to-peer rilevato",
    ["alert_packets_description"] = "Differenza di pacchetti(inviati + ricevuti)",
    ["alert_packets_title"] = "Allarme sui Pacchetti",
    ["alert_pkt_description"] = "Genera un allarme quando la differenza di bytes (inviati + ricevuti) supera la soglia",
    ["alert_pkt_title"] = "Allarme Pacchetti",
    ["alert_slow_purge_threshold"] = "Allarme per le Entry Idle nelle Hash Table",
    ["alert_slow_purge_threshold_descr"] = "Emette un allarme quando la percentuale di entry idle rispetto al totale delle entry di una hash table eccede la soglia configurata",
    ["alert_throughput_description"] = "Throughput medio (inviato + ricevuto) [Mbps]",
    ["common_thresholds_local_sources"] = "Soglie Comuni %{source}",
    ["countries_contacts_description"] = "Emette un allarme quando il numero di nazioni differenti contattate eccede la soglia",
    ["countries_contacts_title"] = "Allarme sulle nazioni contattate",
    ["daily"] = "Giornalmente",
    ["dns_contacts_description"] = "Genera un allarme quando il numero di server DNS contattati supera la soglia",
    ["dns_contacts_title"] = "Allarme contatti servers DNS",
    ["dns_traffic"] = "Traffico DNS",
    ["domain_names_contacts_description"] = "Genera un allarme quando il numero di Domain Names contattati è maggiore di una certa soglia",
    ["domain_names_contacts_title"] = "Allarme Contatti Domain Names",
    ["egress_traffic"] = "Traffico in Uscita",
    ["every_5_minutes"] = "Ogni 5 Minuti",
    ["every_minute"] = "Ogni Minuto",
    ["flows_anomaly_description"] = "Rileva anomalie nel numero di flussi correnti",
    ["flows_anomaly_title"] = "Anomalia di flussi",
    ["host_alert_pkt_threshold"] = "Soglia pacchetti",
    ["hourly"] = "Ogni Ora",
    ["idle_time"] = "Tempo d'Inattività",
    ["ingress_traffic"] = "Traffico in Ingresso",
    ["inner_traffic"] = "Traffico Interno",
    ["manual"] = "Manuale",
    ["note_add_custom_scripts"] = "%{product} carica gli script dalle seguenti <a href=\"%{url}\"> <i class=\"fas fa-folder\"></i> cartelle</a>.",
    ["note_checks_on_active_hosts"] = "I controlli sono effettuati solo sugli host attivi. Impostare una condizione 'minore di' implica comunque maggiore di zero.",
    ["note_consecutive_checks"] = "I controlli consecutivi non sono necessariamente eseguiti in periodi consecutivi. Per esempio se un host diventa inattivo, le sue soglie non saranno controllate finchè non torna attivo.",
    ["note_control_threshold_checks_periods"] = "Le soglie elencate in questi tab sono controllate periodicamente. Usa i tab per scegliere la frequenza di controllo.",
    ["note_create_custom_scripts"] = "La documentazione per la creazione di plugin e custom script è disponibile al seguente <a class='ntopng-external-link' href=\"%{url}\">link <i class=\"fas fa-external-link-alt\"></i></a>.",
    ["note_thresholds_expressed_as_delta"] = "Le soglie espresse come differenza sono riferite alla differenza tra due misurazioni avvenute alla frequenza di controllo soglia impostata.",
    ["notes"] = "NOTE",
    ["ntp_contacts_description"] = "Genera un allarme quando il numero di server NTP contattati supera una certa soglia",
    ["ntp_contacts_title"] = "Allarme Contatti server NTP",
    ["ntp_traffic"] = "Allarme Traffico NTP",
    ["p2p_traffic"] = "Traffico P2P",
    ["score_anomaly_description"] = "Rileva anomalie nel punteggio di un Host",
    ["score_anomaly_title"] = "Score anomalia",
    ["score_threshold_description"] = "Genera un allarme quando il punteggio di un host supera la soglai",
    ["score_threshold_title"] = "Soglia del punteggio superato",
    ["smtp_contacts_description"] = "Genera un allarme quando il numero di servers SMTP contattati supera la soglia",
    ["smtp_contacts_title"] = "Allarme Contatti Server SMTP",
    ["threshold_type"] = "Tipo di Soglia",
    ["thresholds_single_source"] = "%{source} %{alt_name} Soglie",
    ["throughput"] = "Volume d'Uscita",
  },
  ["appliance"] = {
    ["capture_interfaces"] = "Cattura Interfacce",
    ["enable_wifi"] = "Abilita Wi-Fi",
    ["enable_wifi_descr"] = "Abilita o disabilita l'interfaccia Wi-Fi.",
    ["management"] = "Pannello di Controllo Interfaccia",
    ["ssid"] = "SSID",
    ["ssid_descr"] = "Nome Wi-Fi (SSID).",
    ["wpa_passphrase"] = "Passphrase",
    ["wpa_passphrase_descr"] = "WPA passphrase (minimo 8 caratteri e meno di 64).",
  },
  ["arp_page"] = {
    ["arp_type"] = "Tipo ARP",
  },
  ["as_details"] = {
    ["as"] = "Sistema Autonomo",
    ["as_parameter_missing_message"] = "Parametro di Sistema Autonomo mancante (errore interno ?)",
    ["as_timeseries_enable_message"] = "Le serie temporali dell'AS possono essere abilitate dalle <A HREF=\"%{url}\">%{icon_flask} Preferenze</A>. Sono necessari pochi minuti per vedere i punti del primo dato.",
    ["no_available_data_for_as"] = "Nessun dato disponibile per l'AS %{asn}.",
  },
  ["as_stats"] = {
    ["autonomous_systems"] = "Autonomous Systems",
  },
  ["blog_feed"] = {
    ["blog_updates"] = "Aggioramenti dal Blog",
    ["blog_updates_info"] = "Ricevi notifiche quando ci sono nuovi articoli in: <a href='https://www.ntop.org/blog' target='_blank'>https://ntop.org/blog</a>",
    ["news_from_blog"] = "Novità dal <b>Blog ntop</b>",
    ["nothing_to_show"] = "Nulla da mostrare. Riprova domani!",
  },
  ["breed"] = {
    ["acceptable"] = "Protocollo Accettabile",
    ["dangerous"] = "Pericoloso",
    ["fun"] = "Protocollo Divertimento",
    ["safe"] = "Protocollo Sicuro",
    ["unsafe"] = "Protocollo Non Sicuro",
  },
  ["capabilities"] = {
    ["active_monitoring"] = "Monitoraggio Attivo",
    ["alerts"] = "Allarmi",
    ["checks"] = "Script Utente",
    ["developer"] = "Sviluppatore",
    ["flowdevices"] = "Device a Flussi",
    ["historical_flows"] = "Flussi Storici",
    ["notifications"] = "Notifiche",
    ["pools"] = "Pool",
    ["preferences"] = "Preferenze",
    ["snmp"] = "SNMP",
  },
  ["categories_page"] = {
    ["search_application"] = "Cerca Protocollo",
    ["target"] = "Obiettivo",
  },
  ["category_lists"] = {
    ["category_lists"] = "Liste per le Categorie",
    ["connection_time_out"] = "Connessione andata in timeout dopo %{duration} secondi.",
    ["disable_some_list"] = "Disabilitare alcune liste per risolvere il problema.",
    ["download_succeeded"] = "Lista '%{name}' scaricata con successo",
    ["edit_list"] = "Modifica Liste per le Categorie",
    ["enabled"] = "Abilitata",
    ["error_occurred"] = "Errore durante il download di '%{name}': %{err}",
    ["http_code"] = "[Codice HTTP: %{err_code}]",
    ["last_update"] = "Ultimo Aggiornamento",
    ["list_will_be_updated"] = "La lista '%{name}' sarà aggiornata in circa 30 secondi",
    ["loading_stats"] = "Ultimo aggiornamento (%{when}): caricati %{num_hosts} domini, %{num_ips} indirizzi IP, %{num_ja3} signature JA3 in %{duration} .",
    ["ready_for_update"] = "Pronto per l'aggiornamento",
    ["server_returned_error"] = "Il server ha ritornato un errore",
    ["too_many_hosts_loaded"] = "Il limite per i domini è stato raggiunto (massimo %{limit})",
    ["too_many_ips_loaded"] = "Il limite per gli indirizzi IP è stato raggiunto (massimo %{limit})",
    ["too_many_ja3_loaded"] = "Il limite per le signature JA3 è stato raggiunto (massimo %{limit})",
    ["update_frequency"] = "Frequenza di aggiornamento",
    ["update_now"] = "Aggiorna Adesso",
  },
  ["checks"] = {
    ["categories"] = "Categorie",
    ["category"] = "Categoria",
    ["category_ids_ips"] = "Rilevamento e Prevenzione Intrusione",
    ["category_ids_ips_descr"] = "Controlla e valuta il comportamento di hosts e li aggiunge alla pool degli hosts bloccati quando considerato sospetto. Quando ntopng viene usato in combinazione con nProbe IPS, gli hosts sospetti sono bloccati e viene impedito di generare traffico.",
    ["category_internals"] = "Internals",
    ["category_internals_descr"] = "Funzionalità interne di ntopng (e.g., gestione della memoria e dei cicli di vita di flussi e host)\"  lang.checks.category_network = \"Rete",
    ["category_network"] = "Rete",
    ["category_network_descr"] = "Anomalie sul comportamento delle reti Network behaviors and anomalies (e.g., traffic above a certain threshold, TCP not working as expected)",
    ["category_other"] = "Altro",
    ["category_other_descr"] = "Categoria di default per gli script che non possono essere inclusi in altre categorie",
    ["category_security"] = "Sicurezza",
    ["category_security_descr"] = "Anomalie e comportamenti inerenti la sicurezza (e.g, contatti da o verso un host in blacklist, TCP o UDP scan)",
    ["category_system"] = "Sistema",
    ["category_system_descr"] = "Funzionalità del sistema sul quale sta girando ntopng (e.g, disco pieno, carico troppo elevato).",
    ["disable_all"] = "Disabilita tutti",
    ["disable_all_checks"] = "Disabilita Tutti gli Script Utente",
    ["disable_all_message"] = "Disabilita tutti gli script utente per la configurazione corrente.",
    ["elephant_flows_descr"] = "> %{l2r_bytes} (L2R), > %{r2l_bytes} (R2L)",
    ["exceptions"] = "Eccezioni: %{exceptions}",
    ["factory_reset_all_checks"] = "Ripristino Impostazioni Di Fabbrica dei Check",
    ["factory_reset_all_message"] = "Vuoi resettare  la configurazione delle selezioni?",
    ["long_lived_flows_descr"] = "> %{duration}",
    ["note_apply_to_default"] = "La configurazione di <b>default</b> è 'Applicata a' ogni pool che non è incluso in ogni altra configurazione più specifica.",
    ["note_configsets"] = "Gli user script sono configurati e abilitati/disabilitati in base alla configurazione. Più configurazioni possono esser create, e ogni configurazione più essere 'Applicata a' pool differenti.",
    ["note_what_are_checks"] = "Gli <a href=\"%{checks_url}\">User Script</a> %{checks_external} sono eseguiti periodicamente oppure quando un certo evento si verifica.",
    ["stop_recording_after"] = "Interrompere registrazione dopo %{duration}",
    ["hint"] = {
      ["body"] = "Configure unexpected <a href='%{link_DHCP}' target='_about'>DHCP</a>, <a href='%{link_SMTP}' target='_about'>SMTP</a>, <a href='%{link_DNS}' target='_about'>DNS</a>, <a href='%{link_NTP}' target='_about'>NTP</a> servers.",
      ["title"] = "Server Inaspettati",
    },
  },
  ["conf_backup"] = {
    ["conf_backup"] = "Backup Configurazione",
    ["conf_restore"] = "Ripristino Configurazione",
  },
  ["config_scripts"] = {
    ["applied_to"] = "Applicata ad",
    ["config_apply"] = "Applica Configurazione",
    ["config_clone"] = "Clona Configurazione",
    ["config_clone_message"] = "Digita un nome per la configurazione clonata",
    ["config_delete"] = "Elimina Configurazione",
    ["config_delete_message"] = "Vuoi veramente cancellare questa configurazione?",
    ["config_entities"] = "Entità di Configurazione",
    ["config_export"] = "Esporta Configurazione",
    ["config_import"] = "Importa Configurazione",
    ["config_name"] = "Nome Configurazione",
    ["config_rename"] = "Rinomina Configurazione",
    ["config_rename_message"] = "Digita il nuovo nome per la configurazione",
    ["config_reset"] = "Reset Configurazione",
    ["config_reset_confirm"] = "La seguente operazione cancellerà tutte le configurazioni e ripristinerà tutti i valori di default. Procedere?",
    ["config_search"] = "Cerca Configurazione:",
    ["config_x"] = "Config %{product}",
    ["empty_target_message"] = "Selezionare uno o più target!",
    ["empty_value_message"] = "L'input non può esser vuoto",
    ["invalid_characters"] = "Il nome contiene caratteri non validi",
    ["invalid_file"] = "Non è un file di configurazione valido",
    ["max_input_length"] = "Il nome non può avere più di 16 caratteri",
    ["no_file"] = "Nessun file di configurazione selezionato",
    ["no_pools_applied"] = "Nessun pool",
    ["select_interface"] = "Seleziona interfacce",
    ["select_network"] = "Seleziona reti",
    ["type_targets"] = "Digita i targets:",
    ["type_targets_example"] = "Digita i targets separati da virgola, ad esempio: 192.168.1.20,192.168.0.0/16",
    ["actions"] = {
      ["apply_to"] = "Applica a",
      ["clone"] = "Clona",
      ["delete"] = "Cancella",
      ["edit"] = "Modifica",
      ["rename"] = "Rinomina",
    },
    ["granularities"] = {
      ["flow"] = "Flussi",
      ["host"] = "Hosts",
      ["interface"] = "Interfacce",
      ["network"] = "Reti",
      ["snmp_device"] = "Dispositivi SNMP",
      ["syslog"] = "Syslog",
      ["system"] = "Sistema",
    },
  },
  ["configsets"] = {
    ["bad_target"] = "Target non valido: \"%{target}",
    ["duplicate_target"] = "Il target \"%{target}\" è assegnato a due configurazioni: \"%{confname1}\" e \"%{confname2}",
    ["error_exists"] = "Il nome \"%{name}\" è già in uso",
    ["unknown_check"] = "User script sconosciuto \"%{check}",
    ["unknown_id"] = "Nessuna configurazione trovata con id %{confid}",
    ["wrong_args_ex_list"] = "I parametri delle eccezioni degli allarmi non sono corretti. Controllare se sono presenti duplicati.",
  },
  ["contacts_page"] = {
    ["client_address"] = "Indirizzo client",
    ["client_contacts_initiator"] = "Contatti del Client (Iniziatore)",
    ["contacts"] = "Contatti",
    ["no_client_contacts_so_far"] = "Nessun contatto client fino ad ora",
    ["no_contacts_message"] = "Nessun contatto per questo host",
    ["no_server_contacts_so_far"] = "Nessun contatto server fino ad ora",
    ["server_address"] = "Indirizzo Server",
    ["server_contacts_receiver"] = "Contatti del Server (Ricevente)",
  },
  ["containers_stats"] = {
    ["all_containers"] = "Tutti i Container",
    ["all_pods"] = "Tutti i Pod",
    ["avg_rtt"] = "RTT Medio",
    ["avg_rtt_as_client"] = "RTT Medio come Client",
    ["avg_rtt_as_server"] = "RTT Medio come Server",
    ["avg_rtt_variance"] = "Varianza RTT Medio",
    ["avg_rtt_variance_as_client"] = "Varianza RTT Medio come Client",
    ["avg_rtt_variance_as_server"] = "Varianza RTT Medio come Server",
    ["client_container"] = "Client Container",
    ["client_pod"] = "Client Pod",
    ["client_rtt"] = "Client RTT",
    ["container"] = "Container",
    ["containers"] = "Container",
    ["containers_list"] = "Lista Container",
    ["containers_of_pod"] = "Container del Pod '%{pod}'",
    ["pod"] = "Pod",
    ["pods"] = "Pod",
    ["pods_list"] = "Lista Pod",
    ["server_container"] = "Server Container",
    ["server_pod"] = "Server Pod",
    ["server_rtt"] = "Server RTT",
  },
  ["country_details"] = {
    ["country"] = "Nazione",
    ["country_parameter_missing_message"] = "Parametro nazione mancante (errore interno?)",
    ["no_available_stats_for_country"] = "Non ci sono statistiche per la nazione %{country}",
  },
  ["country_stats"] = {
  },
  ["custom_categories"] = {
    ["add"] = "Aggiungi",
    ["add_application"] = "Aggiungi Applicazione",
    ["add_custom_app"] = "Aggiungi Applicazione Custom",
    ["add_to_categories"] = "Aggiungi alle Categorie",
    ["app_delete_error"] = "Impossibile eliminare \"%{app}\". Riavviare %{product}.",
    ["app_deleted"] = "Applicazione \"%{app}\" eliminata. Riavviare %{product}.",
    ["application_exists"] = "L'applicazione \"%{app}\" esiste già!",
    ["application_name"] = "Nome Applicazione",
    ["application_not_exists"] = "L'applicazione \"%{app}\" non esiste!",
    ["apps_and_categories"] = "Applicazioni e Categorie",
    ["custom_host_category"] = "Categorie Host Personalizzate",
    ["custom_hosts"] = "Host Personalizzati",
    ["delete_app"] = "Elimina Applicazione",
    ["delete_app_confirm"] = "<b>Attenzione</b>: Eliminare un'applicazione è un'operazione distruttiva che può avere effetto sui dai storici. Cancellare %{app}?",
    ["domain_names_substrings"] = "I nomi di dominio vengono interpretati come sottostringhe durante il confronto.<br>Ad es. \"%{s1}\" corrisponde a \"%{s2}\" e \"%{s3}",
    ["each_host_separate_line"] = "Gli host devono essere inseriti su linee separate",
    ["edit_custom_rules"] = "Modifica Regole Custom",
    ["edit_hosts"] = "Modifica Host",
    ["example_port_range"] = "Esempio intervallo porte: \"%{example1}\", \"%{example2}",
    ["host_domain_or_cidr"] = "Gli host possono essere sia nomi di dominio che indirizzi IPv4 o indirizzi di rete in formato CIDR",
    ["host_domain_or_port"] = "Le regole possono contenere domini, indirizzi IPv4 e range di porte TCP/UDP",
    ["invalid_rule"] = "Ignorata regola non valida: %{rule}",
    ["new_app_added"] = "L'applicazione \"%{app}\" è stata configurata. Riavviare %{product}.",
    ["non_empty_list_required"] = "Almeno una regola deve essere specificata.",
    ["option_needed"] = "Le applicazioni custom sono disabilitate perché l'opzione <i>%{option}</i> manca dalla configurazione. Controllare la <a href=\"%{url}\">documentazione</a> per maggiori informazioni.",
    ["protos_reboot_necessary"] = "Le regole per le applicazioni sono state scritte. Riavviare %{product}.",
    ["protos_unexpected_error"] = "Error durante l'aggiornamento delle regole per le applicazioni. Controllare il log di %{product} per dettagli aggiuntivi.",
    ["select_url_category"] = "Si prega di selezionare la categoria alla quale assegnare il dominio:",
    ["similar_host_found"] = "un host simile a \"%{host}\" è già presente nella categoria %{category}",
    ["the_following_is_a_list_of_hosts"] = "Quella seguente è una lista di host personalizzati per la categoria %{category}",
    ["the_following_is_a_list_of_hosts_app"] = "La seguente è una lista di regole custom per l'applicazione %{application}",
    ["the_following_url_will_be_added"] = "Il dominio seguente sarà aggiunto:",
  },
  ["dashboard"] = {
    ["actual_traffic"] = "Traffico Corrente",
    ["community_dashboard"] = "Dashboard Community",
    ["gateways_users"] = "Gateway e Utenti",
    ["lastday_app_traffic"] = "Top Traffico Applicativo Ultimo Giorno",
    ["lastday_traffic"] = "Interfacce di Rete: Vista dell'Ultimo Giorno",
    ["realtime_app_traffic"] = "Traffico Applicativo in Tempo Reale",
    ["realtime_traffic"] = "Interfacce di Rete: Traffico in Tempo Reale",
    ["slow_dashboard_message"] = "Too much latency in the %{product} response. Consider setting an higher <a href='%{url}'>Realtime Stats Refresh Rate</a>.",
    ["top_local_talkers"] = "Top Talker Locali",
    ["top_remote_destinations"] = "Top Destinazioni Remote",
    ["traffic_dashboard"] = "Dashboard di Traffico",
  },
  ["datasources"] = {
    ["alias"] = "Alias",
    ["alias_example"] = "i.e. Datasource Alias",
    ["families"] = "Famiglie",
    ["key"] = "Chiave",
    ["metrics"] = "Metriche",
    ["new_source"] = "Nuova Sorgente",
    ["origin"] = "Origine",
    ["private"] = "Privato",
    ["public"] = "Pubblico",
    ["retention"] = "Retention",
    ["schema"] = "Schema",
    ["scope"] = "Scope",
    ["select_family"] = "Seleziona una famiglia",
    ["select_metrics"] = "Seleziona una metrica",
    ["select_schema"] = "Seleziona uno schema",
    ["select_schema_first"] = "Seleziona uno schema",
    ["select_series"] = "Seleziona una serie temporale",
    ["series"] = "Serie Temporale",
    ["source_exists"] = "Sorgente già configurata!",
    ["tags"] = "Tags",
    ["widget_bound"] = "Ci sono dei widget collegati a questa sorgente dati!",
    ["modal"] = {
      ["add_datasource"] = "Aggiungere sorgente dati",
      ["delete_datasource"] = "Elimina sorgente dati: %{datasource}",
      ["delete_description"] = "Sei sicuro di voler eliminare la sorgente dati: %{datasource}?",
      ["edit_datasource"] = "Modifica Sorgente Dati: %{datasource}",
    },
    ["validation"] = {
      ["abort_add"] = "Cancellare l'aggiunta di una sorgente dati?",
      ["abort_edit"] = "Cancellare la modifica della sorgente dati?",
    },
  },
  ["datatable"] = {
    ["abort_add_filter"] = "Annullare l'aggiunta del filtro?",
    ["add_filter"] = "Aggiungi Filtro",
    ["edit_filter"] = "Modifica Filtro",
    ["no_records"] = "Non Sono Presenti Records",
    ["number_of_records"] = "Numero di records",
    ["select_filter"] = "Filtri",
  },
  ["db_explorer"] = {
    ["abort"] = "Annulla",
    ["all"] = "tutti",
    ["app_between_talkers_pairs"] = "applicazioni tra le coppie di host",
    ["app_flows_between"] = "Flussi applicativi %{app} tra %{peer1} e %{peer2}",
    ["app_talkers"] = "Talkers %{app}",
    ["app_talkers_with"] = "%{app} talkers con %{peer}",
    ["application_flows"] = "Flussi Applicativi",
    ["applications_between"] = "Applicazioni tra %{peer1} e %{peer2}",
    ["average_throughput"] = "Thpt medio",
    ["client_server_breakdown"] = "Suddivisione del Traffico tra Client e Server",
    ["client_server_host"] = "Host Client/Server",
    ["dns_query"] = "Query DNS",
    ["download_flows"] = "Scarica flussi",
    ["download_flows_limit"] = "Massimo 1000 risultati.",
    ["empty_query_response"] = "Risposta vuota alla query (database non accessibile o timeout richiesta/?)",
    ["extract_pcap"] = "Estrai pcap",
    ["filter_bpf"] = "Filtro (BPF)",
    ["first_last_seen"] = "Visto Prima/Ultima volta",
    ["flow_peers"] = "Peers del Flusso",
    ["get_proto_talkers"] = "Ottieni host che usano questo protocollo",
    ["historical_data_explorer"] = "Esploratore di Dati Storici",
    ["historical_flow"] = "Flussi Storici",
    ["host_name"] = "Nome host",
    ["host_talkers"] = "Talkers %{host} ",
    ["hosts_talking_proto_with"] = "Hosts che parlano %{proto} con {host}",
    ["http_host"] = "Host HTTP",
    ["http_method"] = "Metodo HTTP",
    ["interface_name"] = "Nome dell'istanza",
    ["last_hour_flows"] = "Flussi visualizzati nell'ultima ora",
    ["ms_client"] = "%{client} ms (client)",
    ["ms_server"] = "%{server} ms (server)",
    ["network_latency_breakdown"] = "Suddivisione della Latenza di Rete",
    ["observation_period"] = "Periodo d'Osservazione",
    ["ok_request_sent"] = "OK, richiesta inviata",
    ["packet_rate"] = "Velocità media del Pacchetto",
    ["pcap_requests_and_status"] = "Richieste e Stati Pcap",
    ["pcaps"] = "Pcaps",
    ["protocol_flows_between"] = "%{proto} flussi di protocollo tra %{peer1} e %{peer2}",
    ["protocol_id"] = "Id Protocollo",
    ["query_in_progress"] = "La query sul database è in corso, in attesa di ricevere i risultati...",
    ["request_failed"] = "Richiesta fallita",
    ["response_code"] = "Codice di Risposta",
    ["search_flows"] = "Cerca Flussi",
    ["selected_saved"] = "Seleziona salvati",
    ["server_name"] = "Nome del Server",
    ["summary"] = "Sommario",
    ["talkers_with_this_host"] = "Talkers con questo host",
    ["task_id"] = "Id del Task",
    ["this_flow_has_been_reset"] = "Questo flusso è stato resettato e probabilmente l'applicazione server non è funzionante",
    ["this_flow_is_active"] = "Il flusso è attivo",
    ["this_flow_is_completed"] = "Il flusso è completo e presto sarà scartato",
    ["top_flows_ipv4"] = "Top Flussi IPv4 [%{date_from} - %{date_to}]",
    ["top_flows_ipv6"] = "Top Flussi IPv6 [%{date_from} - %{date_to}]",
    ["top_proto_ipv4_flows"] = "Top Flussi IPv4 %{proto}",
    ["top_proto_ipv6_flows"] = "Top Flussi IPv6 %{proto}",
    ["tos"] = "DSCP [ Client / Server ]",
    ["total_flows"] = "Flussi Totali",
    ["total_packets"] = "Pacchetti Totali",
    ["total_traffic"] = "Bytes Totali",
    ["traffic_rate"] = "Velocità media del Traffico",
    ["traffic_received"] = "Bytes Ricevuti",
    ["traffic_sent"] = "Bytes Inviati",
    ["traffic_sent_to_cli"] = "Bytes Inviati ai Client",
    ["traffic_sent_to_srv"] = "Bytes Inviati ai Server",
    ["traffic_volume"] = "Volume di Traffico",
    ["unable_to_find_flow"] = "Impossibile trovare il flusso specificato",
    ["unsave"] = "Rimuovi",
    ["vlan_id"] = "VLAN Id",
  },
  ["db_search"] = {
    ["bytes"] = "Bytes",
    ["cli_asn"] = "Cli ASN",
    ["cli_location"] = "Posizione Client",
    ["cli_nw_latency"] = "Latenza Network del Client",
    ["cli_port"] = "Cli Port",
    ["client"] = "Client",
    ["client_name"] = "Nome Client",
    ["dst2src_dscp"] = "Server DSCP",
    ["dst2src_tcp_flags"] = "Server TCP Flags",
    ["extract_flow_info"] = "Azioni",
    ["first_seen"] = "Inizio",
    ["flow_risk"] = "Rischio Del Flusso",
    ["info"] = "Info",
    ["l4proto"] = "Protocollo",
    ["l7cat"] = "Categoria L7",
    ["l7proto"] = "Applicazionione",
    ["last_seen"] = "Fine",
    ["max_results"] = "Numero massimo di risultati",
    ["observation_point_id"] = "ID Punto di Osservazione",
    ["packets"] = "Pkts",
    ["probe_ip"] = "Esportatore di flussi",
    ["query"] = "Query",
    ["query_copied"] = "Query SQL copiata nella clipboard!",
    ["query_performed"] = "Query eseguita in",
    ["run_query"] = "Esegui query",
    ["select_note"] = "Usa * per tutti i campi o una lista dei campi di un comma-separated (<field1>,<field2>,...)",
    ["server"] = "Server",
    ["server_name"] = "Nome Server",
    ["src2dst_dscp"] = "Client DSCP",
    ["src2dst_tcp_flags"] = "Client TCP Flags",
    ["srv_asn"] = "Srv ASN",
    ["srv_location"] = "Posizione Server",
    ["srv_nw_latency"] = "Latenza Network del Server",
    ["srv_port"] = "Srv Port",
    ["status"] = "Stato",
    ["throughput"] = "Thpt",
    ["time_range"] = "Range temporale",
    ["vlan_id"] = "VLAN",
    ["where_note"] = "Usa &lt;field&gt; &lt;operator&gt; &lt;value&gt; conditions, where &lt;operator&gt; could be &lt;, &gt;, =, !=. AND/OR sono permessi per combinare le condizioni. Si raccomanda di usare i doppi apici per i campi stringa (es. INFO='www.ntop.org')",
    ["tags"] = {
      ["acknowledged"] = "Acknowledged",
      ["alert_id"] = "Tipo di Allarme",
      ["cli_asn"] = "Client ASN",
      ["cli_ip"] = "IP del Client",
      ["cli_location"] = "Posizione Client",
      ["cli_mac"] = "Client MAC",
      ["cli_name"] = "Nome del Client",
      ["cli_network"] = "Client Network",
      ["cli_nw_latency"] = "Latenza Network Client",
      ["cli_port"] = "Client Port",
      ["dscp_id"] = "DSCP ID",
      ["dst2src_dscp"] = "Server DSCP",
      ["dst2src_tcp_flags"] = "Srv TCP Flags",
      ["flow_risk"] = "Rischio del Flusso",
      ["info"] = "Info",
      ["ip"] = "IP",
      ["ip_version"] = "Versione IP",
      ["l4proto"] = "Protocollo",
      ["l7_proto"] = "Applicazione",
      ["l7cat"] = "Category",
      ["l7proto"] = "Applicazione",
      ["name"] = "Nome host",
      ["observation_point_id"] = "ID punto di osservazione",
      ["probe_ip"] = "Esportatore di flusso",
      ["role"] = "Ruoloi",
      ["role_cli_srv"] = "Client/Server",
      ["roles"] = "Ruoli",
      ["score"] = "Score",
      ["severity"] = "Importanza",
      ["src2dst_dscp"] = "Client DSCP",
      ["src2dst_tcp_flags"] = "Cli TCP Flags",
      ["srv_asn"] = "Server ASN",
      ["srv_ip"] = "Server IP",
      ["srv_location"] = "Posizione Server",
      ["srv_mac"] = "Server MAC",
      ["srv_name"] = "Nome Server",
      ["srv_network"] = "Network del Server",
      ["srv_nw_latency"] = "Latenza Network del Server",
      ["srv_port"] = "Server Port",
      ["status"] = "Stato",
      ["subtype"] = "Argomento",
      ["traffic"] = "Bytes",
      ["vlan_id"] = "VLAN",
    },
  },
  ["delete_data"] = {
    ["delete_active_interface_confirmation"] = "Sei sicuro di vole eliminare tutti i dati di %{ifname}? L' eliminazione viene effettuata durante la partenza di %{product}. Riavvia %{product} per eseguire la cancellazione.",
    ["delete_active_interface_data_failed"] = "Impossibile eliminare i dati delle interfacce.",
    ["delete_active_interface_data_ok"] = "I dati di %{ifname} sono stati marcati per l'eliminazione. Riavvia %{product} per eseguire l'eliminazione.",
    ["delete_confirmation"] = "Sei sicuro di voler eliminare tutti i dati persistenti dell'host %{host}?",
    ["delete_failed"] = "Impossibile eliminare i dati dell'host %{host}.",
    ["delete_inactive_interfaces_confirmation"] = "Le seguenti interfacce sono inattive:%{interfaces_list}Sei sicuro di voler eliminare tutti i loro dati persistenti?",
    ["delete_inactive_interfaces_data_failed"] = "Impossibile eliminare i dati delle interfacce inattive.",
    ["delete_inactive_interfaces_data_ok"] = "I dati delle interfacce inattive sono stati eliminati con successo.",
    ["delete_ok"] = "I dati dell'host %{host} sono stati eliminati con successo.",
    ["msg_err_invalid_host"] = "Il formato dell'host specificato non è corretto.",
    ["msg_err_no_fs_data"] = "Non è stato trovato alcun dato persistente per l'host specificato.",
    ["msg_err_unable_to_delete_dir"] = "Impossibile eliminare la data directory.",
    ["msg_err_unable_to_delete_ts_data"] = "Impossibile eliminare i dati delle timeseries.",
    ["msg_err_unknown"] = "Errore sconosciuto.",
    ["node_nindex_flows"] = "nIndex è configurato per il dump dei flussi. I flussi dumpati non saranno eliminati.",
    ["note_persistent_data"] = "Tutti i dati persistenti saranno eliminati. I dati includono contatori di traffico, serie temporali, flussi, allarmi, e cache Redis.",
    ["restart_product_toast"] = "Riavviare %{product} per eliminare le effettive interfacce attive.",
  },
  ["details"] = {
    ["additional_mac_address"] = "Indirizzo MAC Aggiuntivo",
    ["address_resolution_protocol"] = "Protocollo di Risoluzione Indirizzo",
    ["ago"] = "fa",
    ["alerts"] = "Allarmi",
    ["arp_replies"] = "Risposte ARP",
    ["arp_requests"] = "Richieste ARP",
    ["as_client"] = "Come Client",
    ["as_server"] = "Come Server",
    ["contacts_blacklisted"] = "Flussi Totali con Hosts Blacklistati",
    ["details"] = "Dettagli",
    ["device_port"] = "Porta Dispositivo",
    ["device_type"] = "Tipo di Dispositivo",
    ["first_last_seen"] = "Visto Prima/Ultima volta",
    ["first_observed_on"] = "Osservato per la Prima Volta il",
    ["flow_snmp_localization"] = "Localizazione Flusso SNMP",
    ["flow_verdict"] = "Verdetto del Flusso",
    ["flows_dropped_by_bridge"] = "Flussi Bloccati a causa delle Politiche di Traffico",
    ["flows_non_packet_iface"] = "Flussi Attivi / Totali",
    ["flows_packet_iface"] = "Flussi: Attivi / Totali / A Basso Goodput / Non Raggiungibili",
    ["fragments"] = "Frammenti",
    ["further_host_names_information"] = "Ulteriori nomi",
    ["goodput"] = "Goodput",
    ["host_pool"] = "Host Pool",
    ["host_purged"] = "Questo host non &egrave; pi&ugrave; attivo ed &egrave; stato eliminato dalla memoria",
    ["host_snmp_localization"] = "Localizazione Host SNMP",
    ["input_device_port"] = "Porta di Input Dispositivo",
    ["keep_alive"] = "KeepAlive",
    ["label_blacklisted_host"] = "Host Blacklistato",
    ["label_dhcp"] = "DHCP",
    ["label_dhcp_server"] = "Server DHCP",
    ["label_dns_server"] = "Server DNS",
    ["label_local_host"] = "Host Locale",
    ["label_ntp_server"] = "Server NTP",
    ["label_private_ip"] = "IP Privato",
    ["label_remote"] = "Remoto",
    ["label_short_local_host"] = "L",
    ["label_short_private_ip"] = "P",
    ["label_short_remote"] = "R",
    ["label_smtp_server"] = "Server SMTP",
    ["label_system_ip"] = "Sistema IP",
    ["lost"] = "Persi",
    ["misbehaving_flows_reasons"] = "Motivi per Flussi Misbehaving",
    ["more_then_one_device"] = " (Più di un Probe è stato rilevato per questo Host)",
    ["note"] = "NOTE",
    ["note_hosts_located_snmp_device"] = "Gli host sono individuati nei dispositivi SNMP usando il <A HREF=%{url}>Bridge MIB</A>.",
    ["observation_point_id"] = "Id Punto di Osservazione",
    ["out_of_order"] = "Fuori Ordine",
    ["output_device_port"] = "Porta di Output Dispositivo",
    ["peers"] = "Peer: Attivi",
    ["probes_ipv4_address"] = "IPv4 Address dell'ultimo Probe",
    ["rcvd"] = "Rcvt",
    ["retransmissions"] = "Ritrasmissioni",
    ["router_access_point_mac_address"] = "Indirizzo MAC (Router/AccessPoint)",
    ["sent_vs_received_traffic_breakdown"] = "Breakdown Traffico Inviato vs Ricevuto",
    ["server_contacts"] = "Server Contattati",
    ["show_hosts"] = "Mostra Host",
    ["source"] = "Origine",
    ["source_id"] = "Id Sorgente",
    ["tcp_packets_sent_analysis"] = "Analisi Pacchetti TCP Inviati",
    ["total_traffic"] = "Traffico Totale",
    ["traffic_sent_received"] = "Traffico Inviato / Ricevuto",
    ["vlan_id"] = "VLAN ID",
    ["whois_lookup"] = "Ricerca Whois",
  },
  ["developer_section"] = {
    ["datasources_list"] = "Sorgenti",
    ["widgets_list"] = "Widget",
  },
  ["device_protocols"] = {
    ["alert"] = "Genera Allarme",
    ["device_protocol_policy"] = "Politica per i Protocolli dei Dispositivi",
    ["device_protocols"] = "Protocolli dei Dispositivi",
    ["devtype_as_proto_client"] = "%{devtype} come client <i>%{proto}</i>",
    ["devtype_as_proto_server"] = "%{devtype} come server <i>%{proto}</i>",
    ["filter_device_protocols"] = "Protocolli dei Dispositivi %{filter}",
    ["ok"] = "Accettabile",
  },
  ["device_types"] = {
    ["iot"] = "IoT",
    ["laptop"] = "Portatile",
    ["multimedia"] = "Multimedia",
    ["nas"] = "NAS",
    ["networking"] = "Router/Switch",
    ["phone"] = "Telefono",
    ["printer"] = "Stampante",
    ["tablet"] = "Tablet",
    ["tv"] = "TV",
    ["unknown"] = "Sconosciuto",
    ["video"] = "Video",
    ["wifi"] = "Rete Wireless",
    ["workstation"] = "Computer",
  },
  ["dhcp"] = {
    ["delete_range"] = "Elimina range DHCP",
    ["delete_range_confirm"] = "Eliminare il range DHCP configurato %{range} ?",
    ["dhcp"] = "DHCP",
    ["dhcp_alert_note"] = "Emette un allarme quando viene assegnato un indirizzo IP al di fuori dei range DHCP configurati",
    ["dhcp_configuration_note"] = "Questa pagina elenca tutti i range DHCP configurati per questa interfaccia. Questi range DHCP dovrebbero corrispondere a quelli configurati sul server DHCP attualmente in uso",
    ["invalid_ip_address"] = "Indirizzo IP non valido",
    ["no_dhcp_ranges"] = "Nessun range DHCP configurato",
  },
  ["discover"] = {
    ["device"] = "Dispositivo",
    ["discovered_devices"] = "Dispositivi %{manuf} %{sys} %{dev} Scoperti",
    ["err_unable_to_arp_discovery"] = "Impossibile avviare la Scansione di Rete ARP",
    ["error_no_discovery_cached"] = "Nessun dato di scansione è stato trovato nella cache.",
    ["error_unable_to_decode_json"] = "Impossibile decodificare la cache JSON dei dati scoperti.",
    ["ghost_icon_descr"] = "L'icona %{ghost_icon} evidenzia gli host fantasma (ovvero gli host che non appartengono alla rete IP dell'interfaccia)",
    ["network_discovery"] = "Scansione di Rete",
    ["network_discovery_datetime"] = "Ultima Scansione di Rete",
    ["network_discovery_not_enabled"] = "Una Scansione di Rete è in corso. La pagina si aggiornerà periodicamente.",
    ["too_many_devices_descr"] = "Troppi dispositivi trovati. Alcuni dispositivi potrebbero esser stati scartati poiché il massimo numero di dispositivi gestiti è %{max_devices}",
  },
  ["dns_page"] = {
    ["dns_breakdown"] = "Breakdown DNS",
    ["dns_query_sent_distribution"] = "Distribuzione Query DNS inviate",
    ["dns_query_sent_vs_rcvd_distribution"] = "Distribuzione query DNS Inviate vs Ricevute",
    ["doh_dot_server_uses"] = "Uses",
    ["doh_dot_servers"] = "Contacted <A class=\\'ntopng-external-link\\' href=\\\"https://en.wikipedia.org/wiki/DNS_over_HTTPS\\\" target=_blank>DoH <i class=\\'fas fa-external-link-alt\\'></i></A> / <A class=\\'ntopng-external-link\\' href=\\\"https://en.wikipedia.org/wiki/DNS_over_TLS\\\" target=_blank>DoT <i class=\\'fas fa-external-link-alt\\'></i></A> Servers",
    ["error_replies"] = "Risposte d'Errore",
    ["note"] = "NOTE",
    ["note_dns_ratio"] = "Idealmente il rapporto DNS di richieste vs risposte dovrebbe essere 1 (una risposta per ogni richiesta). Quando il valore è molto più basso, ci sono problemi che vale la pena investigare perché significa che il numero di risposte ricevute è molto più basso del previsto e questo può indicare che stiamo usando risolutori DNS che non rispondono o che sono mal configurati (ad es. sono stati migrati ad un altro IP).",
    ["positive_replies"] = "Risposte Positive",
    ["queries"] = "Richieste",
    ["ratio"] = "Rapporto",
    ["rcvd"] = "Rcvt",
    ["replies"] = "Risposte",
    ["reply_breakdown"] = "Breakdown Risposte",
    ["request_vs_reply"] = "Rapporto Richiesta vs Risposta",
  },
  ["dscp_page"] = {
    ["statistics"] = "Classi di precedenza",
    ["statistics_received"] = "Ricevuti",
    ["statistics_sent"] = "Spediti",
  },
  ["edit_check"] = {
    ["author"] = "Autore",
    ["category"] = "Categoria",
    ["delete_alert_exclusions"] = "Deleting alert exclusions will cause new alerts to be triggered again. Do you really want to delete alert exclusions for",
    ["delete_all_alert_exclusions"] = "Delete %{host} Check Exclusions",
    ["delete_all_alert_exclusions_message"] = "Do you really want to delete all %{host} configured alert exclusions?",
    ["description"] = "Descrizione",
    ["excluded_host"] = "Host Esclusi",
    ["excluded_host_name"] = "Nome Host Escluso",
    ["exclusion_list"] = "Controlla Esclusioni",
    ["exclusion_list_host_x"] = "Controlla Esclusioni: Host %{host}",
    ["hooks_config"] = "Configurazione Hooks",
    ["success_message"] = "The User Script <b>%{script_name}</b> configuration has been saved successfully!",
    ["hooks_name"] = {
      ["5mins"] = "5 Minuti",
      ["all"] = "Tutti",
      ["day"] = "Giornaliero",
      ["hour"] = "Orario",
      ["min"] = "Minuto",
    },
  },
  ["email_alert_endpoint"] = {
    ["carbon_copy"] = "CC",
    ["email_recipient"] = "Destinatario",
    ["email_sender"] = "Mittente",
    ["smtp_password"] = "Password SMTP",
    ["smtp_server"] = "Server SMTP",
    ["smtp_username"] = "Username SMTP",
    ["validation"] = {
      ["empty_SMTP_server"] = "Inserire un indirizzo IPv4/IPv6 oppure un Host.",
      ["empty_email"] = "Inserire un indirizzo email valido.",
      ["invalid_SMTP_server"] = "Inserire un indirizzo IPv4/IPv6 oppure un Host valido.",
      ["invalid_email"] = "Inserire un indirizzo email valido (i.e. name@domain.com).",
    },
  },
  ["endpoint_notifications"] = {
    ["actions"] = "Azioni",
    ["add_new_endpoint"] = "Aggiungi Nuovo Endpoint",
    ["add_new_recipient"] = "Aggiungi Nuovo Recipient",
    ["add_to_all"] = "Collega a Tutte le Pools",
    ["associated_to"] = "Associato A",
    ["associated_to_endpoints"] = "Endpoint Associati",
    ["associated_to_recipients"] = "Recipient Associati",
    ["bad_endpoint"] = "Errore durante il recupero della configurazione dell'endpoint",
    ["bad_recipient_params"] = "Parametri del destinatario errati",
    ["bind_to_num_pools"] = "Pools",
    ["bind_to_pools"] = "Connesso a",
    ["categories"] = "Categorie",
    ["create_endpoint_first"] = "E' necessario creare un endpoint prima!",
    ["edit_endpoint"] = "Modifica Endpoint",
    ["edit_endpoints"] = "Modifica Endpoints",
    ["edit_recipient"] = "Modifica Recipient",
    ["empty_endpoints"] = "Nessun Endpoint configurato.",
    ["empty_template"] = "Questo recipient non ha parametri.",
    ["endpoint"] = "Endpoint",
    ["endpoint_config_already_existing"] = "Nome già esistente!",
    ["endpoint_list"] = "Endpoint",
    ["endpoint_recipient_already_existing"] = "Nome già esistente!",
    ["enpoint_recipients_list"] = "Recipient",
    ["fill_pct"] = "Riempi",
    ["get_users_recipient_failed"] = "Caricamento fallito.",
    ["internal_error"] = "Errore Interno",
    ["min_severity"] = "Severità minima",
    ["name"] = "Nome",
    ["no_recipients"] = "<i class='text-danger fas fa-exclamation-triangle'></i> No Recipient configured",
    ["notifications"] = "Notifiche",
    ["num_drops"] = "Persi",
    ["num_uses"] = "Volte Usato",
    ["recipient_category"] = "Filtro Categorie",
    ["recipient_list"] = "Recipient",
    ["recipient_name"] = "Nome",
    ["recipient_select"] = "Endpoint",
    ["recipient_severity"] = "Severità Minima",
    ["recipient_type"] = "Lista Recipient",
    ["recipient_users"] = "Pool che usano il Recipient %{recipient}",
    ["remove_endpoint"] = "Elimina Endpoint",
    ["remove_endpoint_message"] = "Eliminare '%{endpoint}'? Saranno eliminati anche <span class='count'>%{recipients_count}.</span>",
    ["remove_recipient"] = "Elimina Recipient",
    ["remove_recipient_message"] = "Eliminare '%{recipient}'?",
    ["select_type"] = "Selezionare un Tipo",
    ["server_error"] = "Qualcosa è andato storto.",
    ["status"] = "Stato dei Nodi",
    ["test_recipient"] = "Test",
    ["testing_recipient"] = "Test in corso",
    ["time_since_last_use"] = "Tempo dall'Ultimo Utilizzo",
    ["timed_out"] = "Timeout!",
    ["type"] = "Tipo",
    ["used_by_recipients"] = "Usato da",
    ["working_recipient"] = "Test OK!",
    ["zero_users_recipient"] = "Non ci sono pool che usano questo recipient. Aggiungere pool dalla pagina %{link}.",
    ["factory_reset"] = {
      ["body"] = "Vuoi veramente resetttare <b>%{reset_element}</b>?<br>Resettando, tutti gli %{reset_element} configurati saranno persi.",
      ["title"] = "Reset Configurazione: %{reset_element}",
    },
    ["hints"] = {
      ["bind_pools"] = {
        ["body"] = "Ultimo passo! Associa i destinatari ai pool per iniziare a inviare loro avvisi.",
        ["title"] = "Connetti Destinatario",
      },
      ["create_endpoint"] = {
        ["action"] = "Crea",
        ["body"] = "Crea un <a target='_blank' href='%{link}'>endpoint</a> per inviare allarmi.",
        ["title"] = "Crea un Endpoint",
      },
      ["create_recipients"] = {
        ["body"] = "Crea un nuovo <a target='_blank' href='%{link}'>recipient</a> per iviargli allarmi",
        ["title"] = "Crea Destinatari",
      },
    },
    ["recipients"] = {
      ["cc"] = "CC",
      ["email_recipient"] = "Recipient Email",
    },
    ["severities"] = {
      ["alert"] = "Alert",
      ["critical"] = "Critico",
      ["debug"] = "Debug",
      ["emergency"] = "Emergenza",
      ["error"] = "Errore",
      ["info"] = "Info",
      ["notice"] = "Notizia",
      ["warning"] = "Attenzione",
    },
    ["validation"] = {
      ["abort_add_endpoint"] = "Annullare?",
      ["abort_add_recipient"] = "Annullare?",
      ["abort_edit_endpoint"] = "Annullare?",
      ["abort_edit_recipient"] = "Annullare?",
    },
  },
  ["entity_thresholds"] = {
    ["dns_request_reply_ratio_description"] = "Genera un allarme quando il rapporto risposte/richieste è sotto la soglia",
    ["dns_request_reply_ratio_title"] = "Rapporto Risposte/Richieste DNS.",
    ["flow_flood_description"] = "Genera un allarme quando un nuovo client/server flussi/sec supera la soglia",
    ["flow_flood_title"] = "Flow Flood Alert",
    ["flow_victim_description"] = "Genera un allarme quando il nuovo server flussi/sec supera la soglia",
    ["flow_victim_title"] = "Flow Flood Victim Alert",
    ["http_request_reply_ratio_description"] = "Genera un allarme qunado il rapporto risposte/richieste è sotto la soglia",
    ["http_request_reply_ratio_title"] = "Rapporto richieste/risposte HTTP",
    ["icmp_flood_description"] = "Genera un allarme quando il numero di flussi/sec ICMP di inviati/ricevuti supera la soglia",
    ["icmp_flood_title"] = "ICMP Flood Alert",
    ["syn_flood_description"] = "Genera un allarme quando il numero di SYNs/sec inviati/ricevuti supera la soglia",
    ["syn_flood_title"] = "SYN Flood Alert",
    ["syn_scan_description"] = "Genera un allarme quando il numero di SYNs/min inviati/ricevuti (con nessuna risposta) supera la soglia",
    ["syn_scan_title"] = "Allarme Scan SYN",
    ["syn_scan_victim_description"] = "Genera un allarme quando il numero di SYNs/min (con nessuna risposta) supera la soglia",
    ["syn_scan_victim_title"] = "SYN Scan Victim Alert",
    ["syn_victim_description"] = "Genera un allarme quando il numero di SYNs/sec ricevuti supera la soglia",
    ["syn_victim_title"] = "SYN Flood Victim Alert",
  },
  ["error_page"] = {
    ["go_back"] = "Torna indietro",
    ["greeting"] = "&Egrave; stato rilevato un errore",
    ["presence"] = "Non dovresti essere qui!",
  },
  ["event_exporters"] = {
    ["event_device_x"] = "Esportatori di Eventi %{device}",
    ["event_exporter_ip"] = "IP dell'Esporatore di Eventi",
    ["event_exporters"] = "Esportatori di Eventi",
  },
  ["export_data"] = {
    ["export_json_data"] = "Esporta i dati JSON",
    ["note_active_hosts"] = "Solo gli hosts attivi sono ritornati.",
    ["note_maximum_number"] = "Il massimo numero di hosts scaricabili è 32,768.",
  },
  ["external_stats"] = {
    ["capture_drops"] = "Pacchetti Persi",
    ["capture_packets"] = "Pacchetti Catturati",
    ["signatures_failed"] = "Signature Fallite",
    ["signatures_loaded"] = "Signatures Caricate",
    ["title"] = "Statistiche Allarmi Esterni",
  },
  ["field_units"] = {
    ["bytes"] = "Byte",
    ["contacts"] = "Contatti",
    ["flow_sec"] = "Flussi/sec",
    ["flows"] = "Flussi",
    ["hosts"] = "Host",
    ["icmp_flow_sec"] = "Flussi/sec ICMP",
    ["macs"] = "MACs",
    ["mbits"] = "Mbps",
    ["packets"] = "Pacchetti",
    ["percentage"] = "%%",
    ["score"] = "Score",
    ["seconds"] = "Sec",
    ["syn_min"] = "SYNs/min",
    ["syn_sec"] = "SYNs/sec",
  },
  ["flow_alerts_explorer"] = {
    ["alert_tcp_no_data_exchanged_description"] = "Flusso TCP terminato con nessun dato scambiato",
    ["alert_tcp_no_data_exchanged_title"] = "Nessun dato scambiato",
    ["alert_unexpected_dhcp_title"] = "Trovato Server DHCP Inaspettato",
    ["alert_unexpected_dns_title"] = "Trovato Server DNS Inaspettato",
    ["alert_unexpected_ntp_title"] = "Trovato Server NTP Inaspettato",
    ["alert_unexpected_smtp_title"] = "Trovato Server SMTP Inaspettato",
    ["alert_zero_tcp_window_title"] = "TCP Zero Window",
    ["by_target_port"] = "In base alla Porta di Destinazione",
    ["flow_alert_origin"] = "Origine Allarme",
    ["flow_alert_target"] = "Destinazione Allarme",
    ["label"] = "Esploratore Allarmi sui Flussi",
    ["origins"] = "Sorgenti",
    ["search"] = "Cerca Allarmi sui Flussi",
    ["status_unexpected_dhcp_description"] = "Trovato Server DHCP Inaspettato: %{server}",
    ["status_unexpected_dns_description"] = "Trovato Server DNS Inaspettato: %{server}",
    ["status_unexpected_ntp_description"] = "Trovato Server NTP Inaspettato: %{server}",
    ["status_unexpected_smtp_description"] = "Trovato Server SMTP Inaspettato: %{server}",
    ["status_zero_tcp_window_description"] = "Reported TCP Zero Window",
    ["summary_cli2srv"] = "Traffico Totale da Origine a Destinazione",
    ["summary_n_origins"] = "Sorgenti Totali",
    ["summary_n_targets"] = "Destinazioni Totali",
    ["summary_srv2cli"] = "Traffico Totale da Desinazione a Origine",
    ["summary_total"] = "Allarmi Totali sui Flussi",
    ["targets"] = "Destinazioni",
    ["type_alerts_by_type"] = "Allarmi di Flusso per tipo",
    ["type_explorer"] = "Esplora per Tipo",
    ["visual_explorer"] = "Esploratore Visuale",
  },
  ["flow_checks"] = {
    ["allowed_server_names_description"] = "Valori di server IP separati da Comma separated values of server IPs. Example: 173.194.76.109,my.example.com",
    ["allowed_servers_description"] = "Comma separated values of server IPs. Example: 173.194.76.109,52.97.232.242",
    ["allowed_servers_title"] = "Servers Permessi",
    ["application_detected"] = "Applicazione Rilevata",
    ["asn_behavior_check_list"] = "ASN da analizzare",
    ["asn_behavior_check_list_example"] = "Una lista di nomi o ID ASN da analizzare, separato da virgole",
    ["callback"] = "User Script",
    ["callback_config"] = "Configurazione",
    ["callback_elapsed_time"] = "Tempo di Esecuzione",
    ["callback_elapsed_time_avg"] = "Flussi/sec",
    ["callback_enabled"] = "Abilitata",
    ["callback_function"] = "Hook",
    ["callback_function_duration"] = "Durata",
    ["callback_function_duration_fmt_long"] = "%{num_calls} chiamate in %{time} [%{speed} Chiamate/sec]",
    ["callback_function_duration_fmt_short"] = "%{time}",
    ["callback_function_num_flows"] = "Flussi",
    ["callback_function_throughput"] = "Velocità di Processamento",
    ["callback_latest_run"] = "Ultima Esecuzione",
    ["dns_esclusion_list"] = "Lista di Esclusione",
    ["dns_esclusion_list_example"] = "Lista di domini (separati da virgola) da escludere per la generazione di allarmi DNS",
    ["idle"] = "Idle",
    ["iec104_description"] = "Genera un allarme quando viene rilevata una transizione IEC invalida",
    ["iec104_title"] = "Transizione IEC Invalida",
    ["iec104_unexpected_type_id_allowed_type_ids_description"] = "Valori IEC divisi da virgole IEC 60870-5-104 TypeIDs. Esempio: 1,2,3,4",
    ["iec104_unexpected_type_id_allowed_type_ids_title"] = "TypeIDs Permessi",
    ["iec104_unexpected_type_id_description"] = "Genera un allarme quando viene rilevato un TypeID inaspettato nel protocollo IEC 104",
    ["iec104_unexpected_type_id_title"] = "TypeID IEC Inaspettato",
    ["l7_behavior_check_list"] = "Applicazione da analizzare",
    ["l7_behavior_check_list_example"] = "Una lista di applicazioni (nome o ID) da analizzare, separato da virgole",
    ["last_calls_per_sec"] = "Ultime Chiamate/sec",
    ["last_duration"] = "Ultima Durata",
    ["last_num_calls"] = "Ultimo Numero di Chiamate",
    ["low_goodput_description"] = "Genera un allarme quando il flusso goodput è troppo basso",
    ["low_goodput_title"] = "Goodput Basso",
    ["network_behavior_check_list"] = "Networks da analizzare",
    ["network_behavior_check_list_example"] = "Una lista di Networks da analizzare, separati da virgole",
    ["no_callbacks_available"] = "Non ci sono callback disponibili.",
    ["no_callbacks_available_disabled_alerts"] = "Callback non disponibili. Le callback hanno bisogno che gli allarmi siano abilitati dalle <a href=\"%{url}\"><i class=\"fas fa-flask\"></i> Preferenze</a>",
    ["no_callbacks_defined"] = "Nessuna callback trovata",
    ["note_add_custom_scripts"] = "%{product} carica gli script dalle seguenti <a href=\"%{url}\"> <i class=\"fas fa-folder\"></i> cartelle</a>.",
    ["note_create_custom_scripts"] = "La documentazione per la creazione di plugin e custom script è disponibile al seguente <a class='ntopng-external-link' href=\"%{url}\">link <i class=\"fas fa-external-link-alt\"></i></a>.",
    ["note_flow_application_detected"] = "Le callback \"Applicazione Rilevata\" sono eseguite subito dopo che l'applicazione del flusso è stata rilevata. Questo avviene entro i primi 12 pacchetti di ogni flusso oppure immediatamente quando un flusso è ricevuto via ZMQ.",
    ["note_flow_idle"] = "Le callback \"Idle\" sono eseguite quando un flusso è terminato ed è quindi considerato idle.",
    ["note_flow_lifecycle"] = "Le callback sono eseguite a momenti ben definiti durante il ciclo di vita di ciascun flusso.",
    ["note_flow_periodic_update"] = "Le callback \"Aggiornamento Periodico\" sono eseguite periodicamente su ciascun flusso attivo da almeno 5 minuti.",
    ["note_flow_staus_changed"] = "Le callback \"Cambiamento di Stato\" sono eseguite ogni volta che un flusso ha un cambiamento di stato. Ad esempio quando ci sono problemi di connessione o ritrasmissioni.",
    ["note_scripts_list"] = "La lista completa di tutti gli script di %{product} è disponibile alla pagina <a href=\"%{url}\">scripts</a>.",
    ["notes"] = "NOTE",
    ["periodic_update"] = "Aggiornamento Periodico",
    ["status_changed"] = "Cambiamento di Stato",
    ["status_zero_tcp_window_description"] = "Reported TCP Zero Window",
    ["status_zero_tcp_window_description_c2s"] = "Reported client TCP zero window",
    ["status_zero_tcp_window_description_s2c"] = "Reported server TCP zero window ",
    ["tcp_no_data_exchanged_description"] = "Genera un allarme quando un flusso TCP finisce con nessun dato scambiato",
    ["tcp_no_data_exchanged_title"] = "Nessun Dato Scambiato",
    ["unexpected_dhcp_description"] = "Genera un allarme quando un server DHCP non permesso viene trovato",
    ["unexpected_dhcp_title"] = "DHCP Inaspettato",
    ["unexpected_dns_description"] = "Genera un allarme quando un server DNS non permesso viene rilevato",
    ["unexpected_dns_title"] = "DNS Inaspettato",
    ["unexpected_ntp_description"] = "Genera un allarme quando un server NTP non permesso viene rilevato",
    ["unexpected_ntp_title"] = "Server NTP Inaspettato",
    ["unexpected_smtp_description"] = "Genera un allarme quando un server SMTP non permesso viene rilevato",
    ["unexpected_smtp_title"] = "Server SMTP Inaspettato",
    ["zero_tcp_window_description"] = "Genera un allarme quando la finestra di un flusso TCP è zero",
    ["zero_tcp_window_title"] = "Zero TCP Window",
  },
  ["flow_checks_config"] = {
    ["blacklisted"] = "Flusso in Blacklist",
    ["blacklisted_description"] = "Emette un allarme quando viene rilevato un host o un dominio in blacklist",
    ["broadcast_non_udp_traffic_description"] = "Genera un allarme quando un host contatta un indirizzo Broadcast usa un protocollo non-UDP",
    ["broadcast_non_udp_traffic_title"] = "Traffico Broadcast Non-UDP",
    ["data_exfiltration"] = "Data Exfiltration",
    ["data_exfiltration_description"] = "Emette un allarme per possibili data exfiltration",
    ["dev_proto_not_allowed"] = "Applicazione Non Permessa per Device",
    ["dev_proto_not_allowed_description"] = "Emette un allarme quando viene rilevata una applicazione non permessa per un dato device. Le applicazioni permesse possono esser configurate da <a href='%{url}'>questa pagina</a>",
    ["dev_proto_not_allowed_nedge_description"] = "Emetti un allarme quando viene rilevata un'applicazione non autorizzata per un dispositivo <i>Non Assegnato</i>. Le policy possono essere configurate <a href='%{url}'>qua</a>",
    ["dns_data_exfiltration"] = "Data Exfiltration via DNS",
    ["dns_data_exfiltration_description"] = "Emette un allarme quando si rileva una data exfiltration via DNS",
    ["elephant_flows"] = "Flussi Elephant",
    ["elephant_flows_description"] = "Emette un allarme quando un flusso scambia più della quantità di dati massima configurata",
    ["ext_alert"] = "Allarme da Sorgenti Esterne",
    ["ext_alert_description"] = "Riceve allarmi sui flussi da sorgenti esterne (e.g. Suricata)",
    ["flow_risk"] = "Flow Risk",
    ["flow_risk_description"] = "Emetti allarmi in base ai flow risk di nDPI",
    ["flow_risk_list"] = "flow risks nDPI:",
    ["invalid_dns_query"] = "Allarme Query DNS Non Valida",
    ["invalid_dns_query_description"] = "Emette un allarme quando una query DNS potenzialmente malevola viene rilevata",
    ["lateral_movement_description"] = "Genera un allarme quando un servizio inaspettato viene rilevato. Abilita l'impostazionedel comportamento del traffico in modo da far eseguire questo script.",
    ["lateral_movement_title"] = "Rilevato un Movimento Laterale",
    ["long_lived"] = "Flussi Long-Lived",
    ["long_lived_description"] = "Emette un allarme quando un flusso dura più della durata massima configurata",
    ["malicious_signature"] = "Signature Malevole",
    ["malicious_signature_description"] = "Emette un allarme quando una signature potenzialmente malevola viene rilevata",
    ["nedge_blocked"] = "Flussi Bloccati",
    ["nedge_blocked_description"] = "Emetti un allarme quando un flusso è bloccato a causa delle policy configurate",
    ["no_input"] = "Nessuna configurazione necessaria",
    ["not_purged"] = "Ciclo di Vita dei Flussi",
    ["not_purged_description"] = "Emette un allarme quando sono rilevati bug od altri problemi relativi al ciclo di vita dei flussi",
    ["old_tls_version"] = "Versione TLS Vecchia",
    ["old_tls_version_description"] = "Emette un allarme quando si rileva l'utilizzo di una versione vecchia e potenzialmente insicura del TLS",
    ["periodicity_changed_description"] = "Trigger an alert when a periodic service changes. Enable traffic behaviour preference in order for this script to operate.",
    ["periodicity_changed_title"] = "Periodicity Changed Detection",
    ["possible_bot_flow_description"] = "Genera un allarme quando il server è probabilmento un Bot",
    ["possible_bot_flow_title"] = "Probabile Bot",
    ["potentially_dangerous_protocol"] = "Applicazione Potenzialmente Pericolosa",
    ["potentially_dangerous_protocol_description"] = "Emette un allarme quando si rileva traffico di una applicazione potenzialmente pericolosa",
    ["remote_to_local_insecure_proto_description"] = "Genera un allarme quando un server remoto contattat un host locale usando un protocollo non sicuro",
    ["remote_to_local_insecure_proto_title"] = "Protocollo non sicuro da Remoto a Locale",
    ["remote_to_remote"] = "Flusso Remoto a Remoto",
    ["remote_to_remote_description"] = "Emette un allarme quando si rileva un flusso in cui sia il client che il server sono host remoti",
    ["score"] = "Score del Flusso",
    ["suspicious_countries"] = "Paesi Sospetti",
    ["suspicious_countries_description"] = "Emette un allarme quando il client o il server di un flusso appartiene ad un paese sospetto",
    ["suspicious_tcp_probing"] = "Probing TCP Sospetto",
    ["suspicious_tcp_probing_description"] = "Genera un allarme quando viene rilevato probing TCP sospetto",
    ["suspicious_tcp_syn_probing"] = "Probing TCP SYN Sospetto",
    ["suspicious_tcp_syn_probing_description"] = "Genera un allarme quando viene rilevato probing TCP SYN sospetto",
    ["tcp_connection_refused"] = "Connessione TCP Rifiutata",
    ["tcp_connection_refused_description"] = "Genera un allarme quando viene rifiutata una connessione TCP",
    ["tcp_issues_generic"] = "Problemi Sequenza TCP",
    ["tcp_issues_generic_description"] = "Rileva pacchetti ritrasmessi, persi e fuori ordine nel TCP",
    ["tls_cert_expired"] = "Certificato TLS Scaduto",
    ["tls_cert_expired_description"] = "Emette un allarme quango si rileva l'utilizzo di un certificato TLS scaduto",
    ["tls_cert_mismatch"] = "Problemi Certificato TLS",
    ["tls_cert_mismatch_description"] = "Emette un allarme quando si rileva un certificato TLS non corrispondente a quanto chiesto dal client.",
    ["tls_cert_selfsigned"] = "Certificato TLS Self-signed",
    ["tls_cert_selfsigned_description"] = "Emetti un allarme quando viene rilevato un certificato TLS self-signed",
    ["tls_unsafe_ciphers"] = "Cifrari TLS Non Sicuri",
    ["tls_unsafe_ciphers_description"] = "Emette un allarme quando si rileva l'utilizzo di cifrari insicuri all'interno del TLS",
    ["udp_unidirectional"] = "Flusso UDP Unidirezionale",
    ["udp_unidirectional_description"] = "Emette un allarme quando si rileva un flusso UDP unidirezionale",
    ["web_mining"] = "Web Mining",
    ["web_mining_description"] = "Emette un allarme quando si rileva traffico potenzialmente inerente al mining di criptovalute.",
  },
  ["flow_details"] = {
    ["acceptable_label"] = "Accettabile",
    ["actual_peak_throughput"] = "Throughput Attuale / Picco",
    ["additional_alert_type"] = "Altri Stati del Flusso",
    ["additional_flow_elements"] = "Elementi Aggiuntivi del Flusso",
    ["alert_type"] = "Stato Flusso",
    ["alerted_flows"] = "Flussi Allarmati",
    ["alerted_flows_severity"] = "Severità Flussi Allarmati",
    ["application_latency"] = "Latenza Applicazione",
    ["as_prev_next"] = "AS Prec / Succ",
    ["as_src_dst"] = "AS Sorgente / Destinatario",
    ["average_cpu_load"] = "Carico medio CPU",
    ["bittorrent_hash"] = "Hash BitTorrent",
    ["blacklisted_flow"] = "Client o server blacklistato (o entrambi)",
    ["blacklisted_flow_detailed"] = "%{who} in blacklist",
    ["broadcast_non_udp_traffic"] = "Rilevato Traffico Broadcast che usa un protocollo non-UDP [Cli MAC: %{mac}] [Cli IP: %{ip}]",
    ["c_ip_addresses"] = "C IP Addresses",
    ["call_canceled"] = "Chiamata Cancellata",
    ["call_completed"] = "Chiamata Completata",
    ["call_error"] = "Errore di Chiamata",
    ["call_id"] = "Call-ID",
    ["call_initiator"] = "Peer Chiamante",
    ["call_quality_indicators"] = "Indicatori Qualità Chiamata",
    ["call_started"] = "Chiamata Iniziata",
    ["call_state"] = "Stato Chiamata",
    ["called_party"] = "Peer Chiamato",
    ["cancel_bye_failure_reason_cause"] = "Cancel/Bye/Failure Reason Cause",
    ["certificates_not_match"] = "I certificati non corrispondono",
    ["client_container_information"] = "Informazioni sul Client Container",
    ["client_process_information"] = "Informazioni Processo Client",
    ["client_requested"] = "Il client ha richiesto",
    ["client_tls_supported_versions"] = "Protocolli TLS supportati dal Client",
    ["computed_as_tcp_window_size_rtt"] = "Calcolato come una grandezza di finestra TCP / RTT",
    ["container_id"] = "Container ID",
    ["could_not_add_host_to_category"] = "Impossibile aggiungere l'host \"%{host}\" alla categoria \"%{category}\".",
    ["data_exfiltration"] = "Data Exfiltration",
    ["desirable_label"] = "Auspicabile",
    ["dns_data_exfiltration"] = "Data Exfiltration via DNS",
    ["dns_invalid_query"] = "Richiesta DNS non valida",
    ["dns_query"] = "Coda DNS",
    ["docker"] = "Docker",
    ["docker_name"] = "Nome",
    ["drop_flow_traffic_btn"] = "Blocca il Traffico del Flusso",
    ["dropped_packets"] = "Pacchetti Bloccati",
    ["dtmf_tones_sent"] = "Toni DTMF inviati durante la chiamata",
    ["dump_flow_traffic"] = "Scarica il Traffico del Flusso",
    ["elephant_exceeded"] = "Oltre %{vol}",
    ["elephant_flow"] = "Flusso Elephant",
    ["elephant_flow_descr"] = "I flussi elephant sono quelli che scambiano più di una certa quantità di dati configurata.",
    ["elephant_flow_l2r"] = "Flusso Elephant (da Locale a Remoto)",
    ["elephant_flow_r2l"] = "Flusso Elephant (da Remoto a Locale)",
    ["entropy"] = "Entropia",
    ["error_and_higher"] = "Errore o Più",
    ["existing_rules_note"] = "Le regole esistenti possono esser trovate alla pagina <a href=\"%{url}\">%{name}</a>.",
    ["external_alert"] = "Rilevato allarme %{scope}: %{msg} [%{maker}]",
    ["first_last_flow_sequence"] = "Prima / Ultima Sequenza Flusso",
    ["flow_active_msg"] = "Questo flusso è attivo",
    ["flow_alerted"] = "Flusso Allarmato",
    ["flow_anomalies"] = "Flow Risk Rilevati",
    ["flow_blocked_by_bridge"] = "Flusso bloccato a causa di una politica configurata",
    ["flow_cannot_be_found_message"] = "Questo flusso non può essere trovato.",
    ["flow_completed_msg"] = "Questo flusso è completo e scadrà presto",
    ["flow_connecting_msg"] = "Connessione in corso.",
    ["flow_details"] = "Dettagli Flusso",
    ["flow_emitted"] = "Flusso emesso quando l'interfaccia di rete era allarmata",
    ["flow_issues"] = "Problemi",
    ["flow_low_goodput"] = "Rapporto goodput basso [%{ratio}%%]",
    ["flow_marker"] = "Marcatore Flusso",
    ["flow_peer_roles_inaccurate_msg"] = "Non è stato visto l'inizio del flusso: i ruoli dei peer (client/server) possono essere imprecisi",
    ["flow_peers_client_server"] = "Peer dei Flussi [ Client / Server ]",
    ["flow_quota"] = "Quota del Flusso",
    ["flow_reset_by_resetter_msg"] = "Questo flusso è stato resettato da %{resetter}",
    ["flow_reset_msg"] = "Questo flusso è stato resettato",
    ["flow_score"] = "Score del Flusso",
    ["flow_score_breakdown"] = "Ripartizione per categoria di punteggio",
    ["flow_shapers"] = "Shapers del Flusso",
    ["flow_traffic_is_dropped"] = "Traffico del Flow Scartato",
    ["forward"] = "Inoltra",
    ["gsm_cell_identifier"] = "Identificatore di Cella GSM",
    ["gsm_cell_lac"] = "GSM Cell LAC (Location Area Code)",
    ["host_successfully_added_to_category"] = "Host \"%{host}\" aggiunto con successo alla categoria <a href=\"%{url}\">%{category}</a>. Questa regola sarà applicata solo ai nuovi flussi.",
    ["http_method"] = "Metodo HTTP",
    ["icmp_info"] = "Info ICMP",
    ["ids_alert"] = "Rilevato allarme %{scope}: %{msg} [%{maker}]",
    ["iec104_latency"] = "Latenza Tipo I-S Ack (Media / Deviazione Std)",
    ["iec104_mask"] = "<A class='ntopng-external-link' href='https://en.wikipedia.org/wiki/IEC_60870-5'>IEC 60870-5-104 <i class='fas fa-external-link-alt'></i></A> <A class='ntopng-external-link' href=https://infosys.beckhoff.com/english.php?content=../content/1033/tcplclibiec870_5_104/html/tcplclibiec870_5_104_objref_overview.htm&id>TypeIDs <i class='fas fa-external-link-alt'></i></A>",
    ["iec104_msg_breakdown"] = "Ripartizione dei Messaggi",
    ["iec104_msg_loss"] = "Messaggi persi Lost / Ritrasmissione",
    ["iec104_transitions"] = "Transizioni Type ID",
    ["imsi"] = "IMSI (International mobile Subscriber Identity)",
    ["io_wait_time_percentage"] = "Percentuale di Attesa I/O",
    ["ip_packet_analysis"] = "Analisi Pacchetti IP",
    ["jitter"] = "Jitter",
    ["k8s"] = "Kubernetes",
    ["k8s_name"] = "Nome",
    ["k8s_ns"] = "Namespace",
    ["k8s_pod"] = "Pod",
    ["known_proto_on_non_std_port"] = "Trovata applicazione conosciuta su porta non-standard",
    ["longlived_exceeded"] = "Oltre %{amount}",
    ["longlived_flow"] = "Flusso Long-Lived",
    ["longlived_flow_descr"] = "I flussi long-lived sono quelli che durano più di una certa durata massima configurata.",
    ["looks_like_idle_flow_message"] = "Questo sembra un <font color=red>flusso inattivo</font> con trasmissioni periodiche utilizzate solo per tenerlo vivo.",
    ["lost_packets"] = "Pacchetti Persi",
    ["low_goodput"] = "Goodput Basso",
    ["malicious_ja3_signature"] = "Signature del %{cli_or_srv} potenzialmente malevola [JA3: <a href=\"%{url}\">%{signature}</a> %{icon}]",
    ["max_estimated_tcp_throughput"] = "Massimo (Stimato) Throughput TCP",
    ["max_packet_interarrival_time"] = "Massimo Tempo d'Intervallo di Pacchetto",
    ["mean_rtt"] = "RTT Medio",
    ["memory_actual_peak"] = "Memoria Attuale / Picco",
    ["mibehaving_flows"] = "Flussi Misbehaving",
    ["normal"] = "Normale",
    ["not_recommended_label"] = "Non Raccomandato",
    ["notice_and_lower"] = "Numero",
    ["now_purged"] = "Il flusso non sembra essere attivo e dovrebbe esser stato rimosso dalla memoria.",
    ["nsapi"] = "NSAPI",
    ["ongoing_call"] = "Chiamata in Corso",
    ["packet_inter_arrival_time"] = "Intervallo d'Arrivo tra i Pacchetti <br>[ Min / Medio / Max ]",
    ["payload_type"] = "Tipo di Payload",
    ["potentially_dangerous_protocol"] = "Applicazione Potenzialmente pericolosa",
    ["process_information_report_warning"] = "Il report sulle informazioni di processo è limitato a meno che non si usi ntopng con <A HREF='%{url}'>nProbe</A> ed il plugin sprobe",
    ["process_pid_name"] = "PID/Nome Processo",
    ["profile_name"] = "Nome Profilo",
    ["pseudo_mos"] = "(Pseudo) MOS",
    ["r_factor"] = "Fattore-r",
    ["reach_connection_label"] = "Raggiungi Connessione",
    ["remote_to_remote"] = "Client remoto e server remoto",
    ["response_code"] = "Codice di Risposta",
    ["reverse"] = "Inverso",
    ["round_trip_time"] = "Tempo di andata e ritorno",
    ["rtp_codecs"] = "Codecs RTP",
    ["rtp_flow"] = "Flusso RTP",
    ["rtp_protocol_information"] = "Informazione Protocollo RTP",
    ["rtp_stream_peers"] = "Stream Peers RTP",
    ["rtp_transit_in_out"] = "Transito RTP IN / OUT",
    ["rtt_breakdown"] = "Breakdown del Round-Trip Time",
    ["rtt_distance"] = "Distanza Stimata Client/Server",
    ["rtt_jitter"] = "RTT Jitter",
    ["rule_type"] = "Tipo di Regola",
    ["sac_service_area_code"] = "SAC (Service Area Code)",
    ["score_category_network"] = "Rete",
    ["score_category_security"] = "Sicurezza",
    ["server_container_information"] = "Informazioni sul Server Container",
    ["server_name"] = "Nome Server",
    ["server_process_information"] = "Informazioni Processo Server",
    ["sip_call_id"] = "SIP Call-ID",
    ["sip_protocol_information"] = "Informazione Protocollo SIP",
    ["slow_application_header"] = "Intestazione Applicazione Lenta",
    ["slow_data_exchange"] = "Scambio Dati Lento",
    ["slow_tcp_connection"] = "Connessione TCP Lenta",
    ["son_of_father_process"] = "figlio di <a href ='%{url}'>%{proc_father_pid}/%{proc_father_name}</a>",
    ["ssh_signature"] = "Firma SSH",
    ["suspicious_client_device_protocol"] = "Protocollo client \"%{proto}\" non permesso dalla <a href=\"%{url}\">politica del protocollo per %{devtype}</a> configurata",
    ["suspicious_device_protocol"] = "Applicazione del Device Sospetta",
    ["suspicious_server_device_protocol"] = "Protocollo server \"%{proto}\" non permesso dalla <a href=\"%{url}\">politica del protocollo per %{devtype}</a> configurata",
    ["suspicious_tcp_probing"] = "Probing TCP Sospetto",
    ["suspicious_tcp_syn_probing"] = "Probing TCP SYN sospetto",
    ["sync_source_id"] = "Sincronizza Sorgente ID",
    ["tcp_connection_issues"] = "Problemi di Connesione TCP (Ritrasmissioni, OOO, Persi)",
    ["tcp_packet_analysis"] = "Analisi Pacchetto TCP",
    ["tcp_severe_connection_issues"] = "Gravi problemi di connessione TCP (pacchetti ritrasmessi, fuori ordine, persi)",
    ["throughput_trend"] = "Trend del Volume",
    ["tls_certificate"] = "Certificato TLS",
    ["tls_certificate_expired"] = "Certificato TLS Scaduto",
    ["tls_certificate_invalid"] = "Certificato TLS Non Valido",
    ["tls_certificate_mismatch"] = "Il certificato TLS non corrisponde",
    ["tls_certificate_selfsigned"] = "Certificato TLS Self-signed",
    ["tls_certificate_validity"] = "Validità Certificato TLS",
    ["tls_old_protocol_version"] = "Versione Protocollo TLS Obsoleta",
    ["tls_server_names"] = "Nomi Server",
    ["tls_unsafe_ciphers"] = "Cifratura TLS non Sicura",
    ["tos"] = "<A class='ntopng-external-link' target='_blank' href='https://en.wikipedia.org/wiki/Differentiated_services'>DSCP <i class='fas fa-external-link-alt'></i></A> / <A class='ntopng-external-link' target='_blank' href='https://en.wikipedia.org/wiki/Explicit_Congestion_Notification'>ECN <i class='fas fa-external-link-alt'></i></A> [ Client / Server ]",
    ["udp_unidirectional"] = "Traffico UDP Unidirezionale",
    ["unknown_status"] = "Stato sconosciuto (%{status})",
    ["url"] = "URL",
    ["user_name"] = "Nome Utente",
    ["vm_page_faults"] = "Errori di Pagina VM",
    ["warning"] = "Attenzione",
    ["web_mining_detected"] = "Questo sito è conosciuto per fare mining di criptovalute sui dispositivi utente",
  },
  ["flow_devices"] = {
    ["active_sflow"] = "Dispositivi sFlow Attivi",
    ["all_device_ports"] = "Tutte le Porte di %{device}",
    ["all_exporters"] = "Esportatore di tutti i flussi",
    ["all_flow_devices"] = "Tutti i Dispositivi a Flusso",
    ["all_sflow_devices"] = "Tutti i Dispositivi sFlow",
    ["collection"] = "Collezione",
    ["curr_hosts"] = "Hosts",
    ["curr_throughput"] = "Throughput",
    ["duplex"] = "Duplex",
    ["enable_flow_ratio"] = "Abilita Ratio Flow/SNMP",
    ["exporter_devices"] = "Esportatore di Flussi dei Dispusitivi",
    ["exporter_ip"] = "IP dell'Esportatore di Flusso",
    ["exporters"] = "Esportatori",
    ["flow_device_x"] = "Dispositivo a Flusso %{device}",
    ["flow_device_x_interface_y"] = "Esportatore flusso %{device} / %{interface}",
    ["flow_ratio_snmp_instructions"] = "Per abilitare il ratio Flow/SNMP, questo dispositivo deve supportare SNMP e deve essere <a href='%{href}'>aggiunta</a> ai dispositivi SNMP.",
    ["flow_ratio_timeseries_instructions"] = "Per abilitare il ratio Flow/SNMP, le serie temporali di <b>SNMP e Flow probe</b> devono essere abilitate.",
    ["full_duplex"] = "Full-Duplex",
    ["half_duplex"] = "Half-Duplex",
    ["in_errors"] = "Errori In",
    ["note_condition_1"] = "Il dispositivo %{device} deve supportare SNMP e deve essere configurato nella pagina dei <a href=\"%{url}\">dispositivi SNMP</a>",
    ["note_condition_2"] = "Le serie temporali per i dispositivi SNMP e a flussi devono essere abilitati dalle <A HREF=\"%{url}\"><i class=\"fas fa-flask\"></i> Preferenze</A> (Vista Esperta)",
    ["note_observation_points_fields1"] = "Observation Points timeseries can be enabled from the <a href=\\\"%{url}\\\"><i class=\\\"fas fa-flask\\\"></i> Preferences</a>. Few minutes are necessary to see the first data points",
    ["note_observation_points_fields2"] = "To check the way to configure a new observation point check <a class=\\'ntopng-external-link\\' href=\\\"%{url}\\\"><i class=\\\"fas fa-external-link-alt\\\"></i> Here</a>. Restart is needed.",
    ["note_observation_points_fields3"] = "Il Throughput è aggiornato con traffico recentemente attivo dello scorso minuto.",
    ["note_snmp_device"] = "Le informazioni inerenti all'SNMP sono mostrate solamente quando una device SNMP con lo stesso indirizzo IP dell'esportatore è presente tra le <a href=\"%{url}\">device SNMP</a>.",
    ["note_zmq_fields1"] = "I dispositivi a flusso sono determinati utilizzando i campi ZMQ %%EXPORTER_IPV4_ADDRESS e %%NPROBE_IPV4_ADDRESS. Quando entrambi i campi sono disponibili, il primo ha precedenza sul secondo",
    ["note_zmq_fields2"] = "Le interfacce dei dispositivi a flusso sono determinate usando i campi ZMQ %%INPUT_SNMP e %%OUTPUT_SNMP",
    ["note_zmq_fields3"] = "I dispositivi sFlow sono determinati dal campo sFlow probeIP ricevuto tramite ZMQ (topic 'counter')",
    ["note_zmq_fields4"] = "Le interfacce dei dispositivi sFlow sono determinate usando il campo ifIndex ricevuto tramite ZMQ (topic 'counter')",
    ["observation_point"] = "Punto di Osservazione",
    ["observation_point_x"] = "Punto di Osservazione %{observation_point}",
    ["observation_points"] = "Punti di Osservazione",
    ["out_errors"] = "Errori Out",
    ["promisc"] = "Promisc",
    ["ratio_not_ready"] = "Calcolo del ratio in corso...",
    ["remember_ratio"] = "Il rapporto Flusso/SNMP può essere attivato seguendo i seguenti passi:",
    ["sflow_device_x"] = "Dispositivo sFlow %{device}",
    ["sflow_snmp_ratio"] = "Rapporto Counters/SNMP",
    ["sflow_snmp_ratio_descr"] = "Il rapporto Counters/SNMP viene calcolato dividendo il totale del traffico dell'interfaccia come riportato nei counter sample sFlow, per il totale del traffico della stessa interfaccia letto via SNMP.",
    ["snmp_description"] = "Descrizione SNMP",
    ["snmp_devmodel"] = "Modello Dispositivo SNMP",
    ["snmp_devname"] = "Nome Dispositivo SNMP",
    ["snmp_ifindex"] = "Indice SNMP",
    ["snmp_ifname"] = "Interfaccia SNMP",
    ["snmp_location"] = "Posizione SNMP",
    ["snmp_no_device_found"] = "Nessun dispositivo SNMP trovato",
    ["snmp_polling"] = "Il ratio Flow/SNMP ha bisogno di almeno 30 minuti di dati SNMP e flussi",
    ["snmp_ratio"] = "Rapporto Flusso/SNMP",
    ["snmp_ratio_descr"] = "Il rapporto Flusso/SNMP è un numero tra 0 e 1 che indica quanto i flussi ricevuto rappresentino il traffico totale. Così come accade nell'analisi basata sui flussi, intestazioni non-IP e di Livello 2 non sono considerate, <br>valori di rapporto normali sono comprese tra 0.8 e 0.9 (es. 0.9 significa che il 90%% del traffico ricevuto osservato tramite SNMP è stato riportato in flussi).<br>Il rapporto è calcolato sul traffico degli ultimi 5 minuti, intervallo uguale alla frequenza di poll delle device SNMP. Il calcolo del rapporto necessita delle seguenti condizioni",
    ["this_page_contains_couters"] = "Questa pagina riporta i contatori delle interfacce così come sono state raccolte via sFlow. Se SNMP è configurato, si può anche vedere la <A HREF=\"%{url}\">vista SNMP</A> di questo dispositivo",
    ["timeseries_descr"] = "Le serie temporali per i dispositivi a flusso possono essere abilitate dalle <a href=\"%{url}\"><i class=\"fas fa-flask\"></i> Preferenze</a>. Sono necessari alcuni minuti per vedere i primi punti di dati",
    ["tot_bytes"] = "Bytes Totali",
    ["tot_flows"] = "Flussi Totali",
    ["tot_traffic"] = "Traffico",
  },
  ["flow_fields_description"] = {
    ["appl_latency_ms"] = "Latenza applicazione (msec), a.k.a. tempo di risposta server",
    ["application_id"] = "ID applicazione raccolta (Cisco o IXIA)",
    ["application_name"] = "Palo Alto App-Id",
    ["biflow_direction"] = "1 = iniziatore, 2 = Iniziatoreinverso",
    ["bittorrent_hash"] = "Hash BITTORRENT",
    ["client_nw_latency_ms"] = "Network RTT/2 client <-> nprobe (msec)",
    ["community_id"] = "Community ID",
    ["cumulative_icmp_type"] = "Cumulativo OR di pacchetti di tipo ICMP",
    ["device_name"] = "Nome del Dispositivo",
    ["dhcp_client_ip"] = "Indirizzo IPv4 del client assegnato dal DHCP",
    ["dhcp_client_mac"] = "Indirizzo MAC del client DHCP",
    ["dhcp_client_name"] = "Nome client DHCP",
    ["dhcp_message_type"] = "Tipo di messaggio DHCP",
    ["dhcp_remote_id"] = "ID remoto agente DHCP",
    ["dhcp_subscriber_id"] = "Id subscriber DHCP",
    ["diameter_clr_cancel_type"] = "DIAMETER Cancellation Type",
    ["diameter_clr_flags"] = "DIAMETER CLR Flags",
    ["diameter_exp_res_result_code"] = "DIAMETER Response Experimental Result Code",
    ["diameter_exp_res_vendor_id"] = "DIAMETER Response Experimental Result Vendor Id",
    ["diameter_hop_by_hop_id"] = "DIAMETER Hop by Hop Identifier",
    ["diameter_req_msg_type"] = "DIAMETER Request Msg Type",
    ["diameter_req_origin_host"] = "DIAMETER Origin Host Request",
    ["diameter_req_user_name"] = "DIAMETER Request User Name",
    ["diameter_rsp_msg_type"] = "DIAMETER Response Msg Type",
    ["diameter_rsp_origin_host"] = "DIAMETER Origin Host Response",
    ["diameter_rsp_result_code"] = "DIAMETER Response Result Code",
    ["direction"] = "Direzione flusso [0=src->dst, 1=dst->src]",
    ["dns_num_answers"] = "DNS # di risposta restituitas",
    ["dns_query"] = "Richiesta DNS",
    ["dns_query_id"] = "ID transazione richiesta DNS",
    ["dns_query_type"] = "Tipo di richiesta DNS (e.g. 1=A, 2=NS..)",
    ["dns_response"] = "Responso DNS(s)",
    ["dns_ret_code"] = "Codice di ritorno DNS (e.g. 0=nessun errore)",
    ["dns_ttl_answer"] = "TTL del primo record A (se presente)",
    ["dns_tx_id"] = "DNS Transaction ID",
    ["dot1qCustomerVlanId"] = "dot1Q Customer Vlan Id",
    ["dot1q_dst_vlan"] = "VLAN Destinazione (VLAN esterna in QinQ)",
    ["dot1q_src_vlan"] = "VLAN Sorgente (VLAN esterna in QinQ)",
    ["downstream_session_id"] = "Identificatore sessione downstream (e.g. L2TP) o 0 se sconosciuto",
    ["downstream_tunnel_id"] = "Identificatore del tunnel downstream (e.g. GTP TEID, VXLAN VNI) o 0 se sconosciuto",
    ["dst_as"] = "Destinazione BGP AS",
    ["dst_as_map"] = "Organization name for DST_AS",
    ["dst_as_path_1"] = "Percorso Destinazione AS posizione 1",
    ["dst_as_path_10"] = "Percorso Destinazione AS posizione 10",
    ["dst_as_path_2"] = "Percorso Destinazione AS posizione 2",
    ["dst_as_path_3"] = "Percorso Destinazione AS posizione 3",
    ["dst_as_path_4"] = "DPercorso Destinazione AS posizione 4",
    ["dst_as_path_5"] = "DPercorso Destinazione AS posizione 5",
    ["dst_as_path_6"] = "Percorso Destinazione AS posizione 6",
    ["dst_as_path_7"] = "Percorso Destinazione AS posizione 7",
    ["dst_as_path_8"] = "Percorso Destinazione AS posizione 8",
    ["dst_as_path_9"] = "Percorso Destinazione AS posizione 9",
    ["dst_father_proc_name"] = "Dst father process name",
    ["dst_father_proc_pid"] = "Dst father process PID",
    ["dst_fragments"] = "Numero di pacchetti frammentati dest->sorg",
    ["dst_intf_name"] = "Nome Interfaccia Server",
    ["dst_ip_city"] = "Città in cui si trova l'IP di destinazione",
    ["dst_ip_country"] = "Nazione in cui cui si trova l'IP di destinazione",
    ["dst_ip_lat"] = "Latitudine in cui si trova l'IP di destinazione",
    ["dst_ip_long"] = "Longitudine in cui si trova l'IP di destinazione",
    ["dst_os_name"] = "Nome OS Server",
    ["dst_proc_actual_memory"] = "Dst process actual memory (bytes)",
    ["dst_proc_average_cpu_load"] = "Dst process avg load (%% * 100)",
    ["dst_proc_name"] = "Dst process name",
    ["dst_proc_num_page_faults"] = "Dst process num pagefaults",
    ["dst_proc_package_name"] = "Server Process Package",
    ["dst_proc_pctg_iowait"] = "Src process iowait time %% (%% * 100)",
    ["dst_proc_peak_memory"] = "Dst process peak memory (bytes)",
    ["dst_proc_pid"] = "Dst process PID",
    ["dst_proc_uid"] = "Dst process UID",
    ["dst_proc_user_name"] = "Dst process user name",
    ["dst_sw_version"] = "Versione Software del Server",
    ["dst_to_src_avg_throughput"] = "Troughput medio da destinazione a sorgente (bps)",
    ["dst_to_src_max_throughput"] = "Massimo throughput da destinazione a sorgente (bps)",
    ["dst_to_src_min_throughput"] = "Minimo throughput da destinazione a sorgente (bps)",
    ["dst_tos"] = "TOS/DSCP (dest->sorg)",
    ["dst_user"] = "Username del Server",
    ["dst_vlan"] = "VLAN Destinazione (VLAN interna in QinQ)",
    ["duration_in"] = "Durata stream CLient a Server (msec)",
    ["duration_out"] = "Durata stream CLient a Server (msec)",
    ["engine_id"] = "Id del meccanismo di commutazione di flusso",
    ["engine_type"] = "Meccanismo di commutazione di flusso",
    ["exporter_ipv4_address"] = "Indirizzo IPv4 dell'esportatore del flusso",
    ["exporter_ipv6_address"] = "Indirizzo IPv6 dell'esportatore del flusso",
    ["file_gaps"] = "Gap File",
    ["file_id"] = "ID del File",
    ["file_name"] = "Nome del File",
    ["file_size"] = "Dimensione del File",
    ["file_state"] = "Stato del File",
    ["file_stored"] = "File Salvato",
    ["firewall_event"] = "Evento Firewall 0=ignora, 1=flusso creato, 2=flusso eliminato, 3=flusso non accettato, 4=allarme flusso, 5=aggiorna flusso",
    ["first_switched"] = "SysUptime (msec) del primo pacchetto del flusso",
    ["flow_active_timeout"] = "Timeout d'attività per le entry di cache del flusso",
    ["flow_duration_microseconds"] = "Durata flusso (usec)",
    ["flow_duration_milliseconds"] = "Durata flusso (msec)",
    ["flow_end_microseconds"] = "uSec (epoca) del primo pacchetto del flusso",
    ["flow_end_milliseconds"] = "Msec (epoca) dell'ultimo pacchetto del flusso",
    ["flow_end_reason"] = "Motivo per la fine del flusso",
    ["flow_end_sec"] = "Secondi (epoca) dell'ultimo pacchetto del flusso",
    ["flow_id"] = "Identificatore seriale del flusso",
    ["flow_inactive_timeout"] = "Timeout d'inattività per le entry di cache del flusso",
    ["flow_proto_port"] = "Porta L7 che identifica il protocollo del flusso o 0 se sconosciuto",
    ["flow_server_name"] = "Nome flusso del server (se conosciuto)",
    ["flow_start_microseconds"] = "uSec (epoca) del primo pacchetto del flusso",
    ["flow_start_milliseconds"] = "Msec (epoca) del primo pacchetto del flusso",
    ["flow_start_sec"] = "Secondi (epoca) del primo pacchetto del flusso",
    ["flow_user_name"] = "Nome utente del flusso del tunnel (se conosciuto)",
    ["flow_verdict"] = "Verdetto del Flusso",
    ["frame_length"] = "Lungezza originale del frame L2",
    ["ftp_command"] = "Comando client FTP",
    ["ftp_command_ret_code"] = "Codice di ritorno del comando FTP client",
    ["ftp_login"] = "Login client FTP",
    ["ftp_password"] = "Password client FTP",
    ["gtpv0_apn_name"] = "GTPv0 APN Name",
    ["gtpv0_end_user_ip"] = "GTPv0 End User IP Address",
    ["gtpv0_end_user_msisdn"] = "GTPv0 End User MSISDN",
    ["gtpv0_rai_cell_lac"] = "GTPv0 Cell Location Area Code",
    ["gtpv0_rai_cell_rac"] = "GTPv0 Cell Routing Area Code",
    ["gtpv0_rai_mcc"] = "GTPv0 Mobile Country Code",
    ["gtpv0_rai_mnc"] = "GTPv0 Mobile Network Code",
    ["gtpv0_req_msg_type"] = "GTPv0 Request Msg Type",
    ["gtpv0_response_cause"] = "GTPv0 Cause of Operation",
    ["gtpv0_rsp_msg_type"] = "GTPv0 Response Msg Type",
    ["gtpv0_tid"] = "GTPv0 Tunnel Identifier",
    ["gtpv1_apn_name"] = "GTPv1 APN Name",
    ["gtpv1_c2s_teid_ctrl"] = "GTPv1 Client->Server TunnelId Control",
    ["gtpv1_c2s_teid_data"] = "GTPv1 Client->Server TunnelId Data",
    ["gtpv1_end_user_imei"] = "GTPv1 End User IMEI",
    ["gtpv1_end_user_imsi"] = "GTPv1 End User IMSI",
    ["gtpv1_end_user_ip"] = "GTPv1 End User IP Address",
    ["gtpv1_end_user_msisdn"] = "GTPv1 End User MSISDN",
    ["gtpv1_rai_lac"] = "GTPv1 RAI Location Area Code",
    ["gtpv1_rai_mcc"] = "GTPv1 RAI Mobile Country Code",
    ["gtpv1_rai_mnc"] = "GTPv1 RAI Mobile Network Code",
    ["gtpv1_rai_rac"] = "GTPv1 RAI Routing Area Code",
    ["gtpv1_rat_type"] = "GTPv1 RAT Type",
    ["gtpv1_req_msg_type"] = "GTPv1 Request Msg Type",
    ["gtpv1_response_cause"] = "GTPv1 Cause of Operation",
    ["gtpv1_rsp_msg_type"] = "GTPv1 Response Msg Type",
    ["gtpv1_s2c_teid_ctrl"] = "GTPv1 Server->Client TunnelId Control",
    ["gtpv1_s2c_teid_data"] = "GTPv1 Server->Client TunnelId Data",
    ["gtpv1_uli_cell_ci"] = "GTPv1 ULI Cell CI",
    ["gtpv1_uli_cell_lac"] = "GTPv1 ULI Cell Location Area Code",
    ["gtpv1_uli_mcc"] = "GTPv1 ULI Mobile Country Code",
    ["gtpv1_uli_mnc"] = "GTPv1 ULI Mobile Network Code",
    ["gtpv1_uli_sac"] = "GTPv1 ULI SAC",
    ["gtpv2_apn_name"] = "GTPv2 APN Name",
    ["gtpv2_c2s_s1u_gtpu_ip"] = "GTPv2 Client->Svr S1U GTPU IP",
    ["gtpv2_c2s_s1u_gtpu_teid"] = "GTPv2 Client->Svr S1U GTPU TEID",
    ["gtpv2_c2s_s5_s8_gtpc_ip"] = "GTPv2 Client->Svr S5/S8 GTPC IP",
    ["gtpv2_c2s_s5_s8_gtpu_ip"] = "GTPv2 Client->Srv S5/S8 PGW GTPU IP",
    ["gtpv2_c2s_s5_s8_gtpu_teid"] = "GTPv2 Client->Srv S5/S8 PGW GTPU TEID",
    ["gtpv2_c2s_s5_s8_sgw_gtpu_ip"] = "GTPv2 Client->Srv S5/S8 SGW GTPU IP",
    ["gtpv2_c2s_s5_s8_sgw_gtpu_teid"] = "GTPv2 Client->Srv S5/S8 SGW GTPU TEID",
    ["gtpv2_end_user_imei"] = "GTPv2 End User IMEI",
    ["gtpv2_end_user_imsi"] = "GTPv2 End User IMSI",
    ["gtpv2_end_user_msisdn"] = "GTPv2 End User MSISDN",
    ["gtpv2_pdn_ip"] = "GTPV2 PDN IP Address",
    ["gtpv2_rat_type"] = "GTPv2 RAT Type",
    ["gtpv2_req_msg_type"] = "GTPv2 Request Msg Type",
    ["gtpv2_response_cause"] = "GTPv2 Cause of Operation",
    ["gtpv2_rsp_msg_type"] = "GTPv2 Response Msg Type",
    ["gtpv2_s2c_s1u_gtpu_ip"] = "GTPv2 Srv->Client S1U GTPU IP",
    ["gtpv2_s2c_s1u_gtpu_teid"] = "GTPv2 Srv->Client S1U GTPU TEID",
    ["gtpv2_s2c_s5_s8_gtpc_ip"] = "GTPv2 Svr->Client S5/S8 GTPC IP",
    ["gtpv2_s2c_s5_s8_gtpu_ip"] = "GTPv2 Srv->Client S5/S8 PGW GTPU IP",
    ["gtpv2_s2c_s5_s8_gtpu_teid"] = "GTPv2 Srv->Client S5/S8 PGW GTPU TEID",
    ["gtpv2_s2c_s5_s8_sgw_gtpu_ip"] = "GTPv2 Srv->Client S5/S8 SGW GTPU IP",
    ["gtpv2_s2c_s5_s8_sgw_gtpu_teid"] = "GTPv2 Srv->Client S5/S8 SGW GTPU TEID",
    ["gtpv2_s5_s8_gtpc_teid"] = "GTPv2 S5/S8 SGW GTPC TEIDs",
    ["gtpv2_uli_cell_id"] = "GTPv2 Cell Identifier",
    ["gtpv2_uli_cell_tac"] = "GTPv2 Tracking Area Code",
    ["gtpv2_uli_mcc"] = "GTPv2 Mobile Country Code",
    ["gtpv2_uli_mnc"] = "GTPv2 Mobile Network Code",
    ["httpMessageVersion"] = "Versione Messaggio HTTP",
    ["httpRequestHost"] = "Host richiesta HTTP",
    ["httpRequestMethod"] = "Metodo Richiesta HTTP",
    ["httpRequestTarget"] = "Target Richiesta HTTP",
    ["httpStatusCode"] = "HTTP Status Code",
    ["httpUserAgent"] = "HTTP User Agent",
    ["http_host"] = "HTTP Host Name (IXIA Host Name)",
    ["http_length"] = "HTTP Content-Length",
    ["http_method"] = "HTTP METHOD",
    ["http_mime"] = "HTTP Mime Type",
    ["http_protocol"] = "HTTP",
    ["http_referer"] = "HTTP Referer",
    ["http_ret_code"] = "HTTP return code (e.g. 200, 304...)",
    ["http_site"] = "HTTP server without host name",
    ["http_ua"] = "HTTP User Agent",
    ["http_url"] = "HTTP URL (IXIA URI)",
    ["http_via"] = "HTTP Via",
    ["http_x_forwarded_for"] = "HTTP X-Forwarded-For",
    ["icmp_ipv4_code"] = "Codice ICMP",
    ["icmp_ipv4_type"] = "Tipo ICMP",
    ["icmp_type"] = "Tipo ICMP * 256 + codice ICMP",
    ["imap_login"] = "Mail sender",
    ["in_bytes"] = "Bytes del flusso in ingresso (src->dst)",
    ["in_dst_mac"] = "Indirizzo MAC di destinazione",
    ["in_pkts"] = "Pacchetti del flusso in ingresso (src->dst)",
    ["in_src_mac"] = "Indirizzo MAC Sorgente",
    ["in_src_osi_sap"] = "OSI Source SAP (Solo Traffico OSI)",
    ["ingress_vrfid"] = "ID VRF d'ingresso",
    ["input_snmp"] = "Indice dell'interfaccia SNMP di Input",
    ["ipDSCP"] = "IP DSCP",
    ["ipPayloadLenght"] = "Lunghezza Payload IP",
    ["ipPrecendence"] = "Precedenza IP",
    ["ip_lenght_maximum"] = "Lunghezza Massima dell'IP",
    ["ip_lenght_minimum"] = "Lunghezza Minima dell'IP",
    ["ip_protocol_version"] = "[4=IPv4][6=IPv6]",
    ["ipfix_sampling_algorithm"] = "Algoritmo di sampling",
    ["ipv4_dst_addr"] = "Indirizzo destinazione IPv4",
    ["ipv4_dst_mask"] = "Maschera di rete destinazione IPv4 (/<bits>)",
    ["ipv4_next_hop"] = "Next hop IPv4",
    ["ipv4_src_addr"] = "Indirizzo sorgente IPv4",
    ["ipv4_src_mask"] = "Maschera di rete sorgente IPv4 (/<bits>)",
    ["ipv6_dst_addr"] = "Indirizzo destinazione IPv6",
    ["ipv6_dst_mask"] = "MAschera destinazione IPv6",
    ["ipv6_next_hop"] = "Indirizzo next hop IPv6",
    ["ipv6_src_addr"] = "Indirizzo sorgente IPv6",
    ["ipv6_src_mask"] = "Maschera sorgente IPv6",
    ["l4_dst_port"] = "Porta di destinazione Layer 4",
    ["l4_dst_port_map"] = "Nome simbolico porta di destinazione Layer 4",
    ["l4_src_port"] = "Porta IPv4 Sorgente",
    ["l4_src_port_map"] = "Nome simbolico porta sorgente Layer 4",
    ["l4_srv_port"] = "Porta server Layer 4",
    ["l4_srv_port_map"] = "Nome simbolico porta server Layer 4",
    ["l7_proto"] = "Protocollo Layer 7 (numerico)",
    ["l7_proto_name"] = "Nome protocollo Layer 7",
    ["l7_risk_score"] = "Score Rischio flusso nDPI",
    ["last_switched"] = "SysUptime (msec) dell'ultimo pacchetto del flusso",
    ["layer2FrameDeltaCount"] = "L2 Frame Delta Count",
    ["layer2FrameTotalCount"] = "L2 Frame Total Count",
    ["layer2OctetDeltaSumOfSquares"] = "L2 Octet Delta Sum Of Squares",
    ["layer2OctetTotalSumOfSquares"] = "L2 Octet Total Sum Of Squares",
    ["longest_flow_pkt"] = "Il pacchetto più lungo (byte) del flusso",
    ["max_ttl"] = "TTL Massimo del flusso",
    ["maximumLayer2TotalLength"] = "Lunghezza Massima Totale L2",
    ["min_ttl"] = "TTL Minimo del flusso",
    ["minimumLayer2TotalLength"] = "Lunghezza Minima Totale L2",
    ["mpls_label_1"] = "Etichetta MPLS alla posizione 1",
    ["mpls_label_10"] = "Etichetta MPLS alla posizione 10",
    ["mpls_label_2"] = "Etichetta MPLS alla posizione 2",
    ["mpls_label_3"] = "Etichetta MPLS alla posizione 3",
    ["mpls_label_4"] = "Etichetta MPLS alla posizione 4",
    ["mpls_label_5"] = "Etichetta MPLS alla posizione 5",
    ["mpls_label_6"] = "Etichetta MPLS alla posizione 6",
    ["mpls_label_7"] = "Etichetta MPLS alla posizione 7",
    ["mpls_label_8"] = "Etichetta MPLS alla posizione 8",
    ["mpls_label_9"] = "Etichetta MPLS alla posizione 9",
    ["mysql_appl_latency_usec"] = "MySQL request->response latecy (usec)",
    ["mysql_db"] = "MySQL database in use",
    ["mysql_query"] = "MySQL Query",
    ["mysql_response"] = "MySQL server response",
    ["mysql_server_version"] = "MySQL server version",
    ["mysql_username"] = "MySQL username",
    ["netbios_query_name"] = "NETBIOS Query Name",
    ["netbios_query_os"] = "NETBIOS Query OS",
    ["netbios_query_type"] = "NETBIOS Query Type",
    ["netbios_response"] = "NETBIOS Query Response",
    ["nprobe_ipv4_address"] = "Indirizzo IPv4 dell'host dove nProbe è in esecuzione",
    ["num_pkts_1024_to_1514_bytes"] = "# pacchetti la cui dimensione IP è > 1024 e <= 1514",
    ["num_pkts_128_to_256_bytes"] = "# pacchetti la cui dimensione IP è > 128 e <= 256",
    ["num_pkts_256_to_512_bytes"] = "# pacchetti la cui dimensione IP è > 256 e < 512",
    ["num_pkts_512_to_1024_bytes"] = "# pacchetti la cui dimensione IP è > 512 e < 1024",
    ["num_pkts_over_1514_bytes"] = "# pacchetti la cui dimensione IP è > 1514",
    ["num_pkts_ttl_128_160"] = "# pacchetti con TTL > 128 e <= 160",
    ["num_pkts_ttl_160_192"] = "# pacchetti con TTL > 160 e <= 192",
    ["num_pkts_ttl_192_224"] = "# pacchetti con TTL > 192 e <= 224",
    ["num_pkts_ttl_224_255"] = "# pacchetti con TTL > 224 e <= 255",
    ["num_pkts_ttl_2_5"] = "# pacchetti con TTL > 1 e TTL <= 5",
    ["num_pkts_ttl_32_64"] = "# pacchetti con TTL > 32 e <= 64",
    ["num_pkts_ttl_5_32"] = "# pacchetti con TTL > 5 e TTL <= 32",
    ["num_pkts_ttl_64_96"] = "# pacchetti con TTL > 64 e <= 96",
    ["num_pkts_ttl_96_128"] = "# pacchetti con TTL > 96 e <= 128",
    ["num_pkts_ttl_eq_1"] = "# pacchetti con TTL = 1",
    ["num_pkts_up_to_128_bytes"] = "# pacchetti la cui dimensione IP è <= 128",
    ["observation_point_id"] = "Id del Punto d'Osservazione",
    ["observation_point_type"] = "Tupo di Punto d'Osservazione",
    ["ooorder_in_pkts"] = "Numero di pacchetti di flusso TCP fuori ordine (destinazione->sorgente)",
    ["ooorder_out_pkts"] = "Numero di pacchetti di flusso TCP fuori ordine (sorgente->destinazione)",
    ["oracle_query"] = "Oracle Query",
    ["oracle_query_duration"] = "Oracle Query Duration (msec)",
    ["oracle_rsp_code"] = "Oracle Response Code",
    ["oracle_rsp_string"] = "Oracle Response String",
    ["oracle_username"] = "Oracle Username",
    ["out_bytes"] = "Bytes del flusso in uscita (dst->src) [Alias a  %%DST_TO_SRC_BYTES]",
    ["out_dst_mac"] = "Indirizzo MAC di destinazione, potenzialmente modificato da una funzione middlebox dopo il Punto d'Osservazione",
    ["out_dst_osi_sap"] = "OSI Destination SAP (Solo Traffico OSI)",
    ["out_pkts"] = "Pacchetti del flusso in uscita (dst->src) [Alias a %%DST_TO_SRC_PKTS]",
    ["out_src_mac"] = "Indirizzo MAC Sorgente, potenzialmente modificato da una funzione middlebox dopo il Punto d'Osservazione",
    ["output_snmp"] = "Indice dell'interfaccia SNMP di Output",
    ["packet_section_offset"] = "Offset della sezione del pacchetto",
    ["packets_observed"] = "Numero totale di pacchetti visualizzati",
    ["packets_selected"] = "Numero di pacchetti selezionati per la campionatura",
    ["payload_hash"] = "Initial flow payload hash",
    ["plugin_name"] = "Nome del plugin utilizzato da questo flusso (se presente)",
    ["policy_action"] = "Politica delle Azioni",
    ["policy_id"] = "Id Politica",
    ["pop_user"] = "POP3 user login",
    ["post_napt_dst_transport_port"] = "Post NAPT Destination Transport Port",
    ["post_napt_src_transport_port"] = "Post NAPT Source Transport Port",
    ["post_nat_dst_ipv4_addr"] = "Indirizzo IPv4 destinazione dopo il NAT",
    ["post_nat_src_ipv4_addr"] = "Indirizzo IPv4 sorgente dopo il NAT",
    ["protocol"] = "Bytes del Protocollo IP",
    ["protocol_map"] = "Nome Protocollo IP",
    ["radius_acct_in_octets"] = "RADIUS Accounting Input Octets",
    ["radius_acct_in_pkts"] = "RADIUS Accounting Input Packets",
    ["radius_acct_out_octets"] = "RADIUS Accounting Output Octets",
    ["radius_acct_out_pkts"] = "RADIUS Accounting Output Packets",
    ["radius_acct_session_id"] = "RADIUS Accounting Session Name",
    ["radius_acct_status_type"] = "RADIUS Accounting Status Type",
    ["radius_called_station_id"] = "RADIUS Called Station Id",
    ["radius_calling_station_id"] = "RADIUS Calling Station Id",
    ["radius_framed_ip_addr"] = "RADIUS Framed IP",
    ["radius_nas_identifier"] = "RADIUS NAS Identifier",
    ["radius_nas_ip_addr"] = "RADIUS NAS IP Address",
    ["radius_req_msg_type"] = "RADIUS Request Msg Type",
    ["radius_rsp_msg_type"] = "RADIUS Response Msg Type",
    ["radius_user_imei"] = "RADIUS User MSISDN (Extension)",
    ["radius_user_imsi"] = "RADIUS User IMSI (Extension)",
    ["radius_user_name"] = "RADIUS User Name (Access Only)",
    ["retransmitted_in_bytes"] = "Numero di byte di flusso TCP ritrasmessi (sorgente->destinazione)",
    ["retransmitted_in_pkts"] = "Numero di pacchetti di flusso TCP ritrasmessi(sorgente->destinazione)",
    ["retransmitted_out_bytes"] = "Numero di byte di flusso TCP ritrasmessi(destinazione->sorgente)",
    ["retransmitted_out_pkts"] = "Numero di pacchetti di flusso TCP ritrasmessi (destinazione->sorgente)",
    ["rtp_dtmf_tones"] = "DTMF tones sent (if any) during the call",
    ["rtp_first_seq"] = "First flow RTP Seq Number",
    ["rtp_first_ts"] = "First flow RTP timestamp",
    ["rtp_in_jitter"] = "RTP jitter (ms * 1000)",
    ["rtp_in_max_delta"] = "Max delta (ms*100) between consecutive pkts (src->dst)",
    ["rtp_in_mos"] = "RTP pseudo-MOS (value * 100) (src->dst)",
    ["rtp_in_payload_type"] = "RTP payload type",
    ["rtp_in_pkt_drop"] = "Packet discarded by Jitter Buffer (src->dst)",
    ["rtp_in_pkt_lost"] = "Packet lost in stream (src->dst)",
    ["rtp_in_r_factor"] = "RTP pseudo-R_FACTOR (value * 100) (src->dst)",
    ["rtp_in_transit"] = "RTP Transit (value * 100) (src->dst)",
    ["rtp_last_seq"] = "Last flow RTP Seq Number",
    ["rtp_last_ts"] = "Last flow RTP timestamp",
    ["rtp_mos"] = "RTP pseudo-MOS (value * 100) (average both directions)",
    ["rtp_out_jitter"] = "RTP jitter (ms * 1000)",
    ["rtp_out_max_delta"] = "Max delta (ms*100) between consecutive pkts (dst->src)",
    ["rtp_out_mos"] = "RTP pseudo-MOS (value * 100) (dst->src)",
    ["rtp_out_payload_type"] = "RTP payload type",
    ["rtp_out_pkt_drop"] = "Packet discarded by Jitter Buffer (dst->src)",
    ["rtp_out_pkt_lost"] = "Packet lost in stream (dst->src)",
    ["rtp_out_r_factor"] = "RTP pseudo-R_FACTOR (value * 100) (dst->src)",
    ["rtp_out_transit"] = "RTP Transit (value * 100) (dst->src)",
    ["rtp_r_factor"] = "RTP pseudo-R_FACTOR (value * 100) (average both directions)",
    ["rtp_rtt"] = "RTP Round Trip Time (ms)",
    ["rtp_sip_call_id"] = "SIP call-id corresponding to this RTP stream",
    ["rtp_ssrc"] = "RTP Sync Source ID",
    ["s1ap_cause_enb_to_mme"] = "S1AP Cause from ENB to MME",
    ["s1ap_detailed_cause_enb_to_mme"] = "S1AP Detailed Cause from ENB to MME",
    ["s1ap_enb_ue_s1ap_id"] = "S1AP ENB Identifier",
    ["s1ap_mme_ue_s1ap_id"] = "S1AP MME Identifier",
    ["s1ap_msg_emm_type_enb_to_mme"] = "S1AP EMM Message Type from ENB to MME",
    ["s1ap_msg_emm_type_mme_to_enb"] = "S1AP EMM Message Type from MME to ENB",
    ["s1ap_msg_esm_type_enb_to_mme"] = "S1AP ESM Message Type from ENB to MME",
    ["s1ap_msg_esm_type_mme_to_enb"] = "S1AP ESM Message Type from MME to ENB",
    ["sampled_packet_id"] = "Id pacchetto campionato",
    ["sampled_packet_size"] = "Dimensione campionata del pacchetto",
    ["sampling_algorithm"] = "Tipo campionamento (deterministico/random)",
    ["sampling_interval"] = "Frequenza di campionamento",
    ["sampling_population"] = "Sampling population",
    ["sampling_size"] = "Numero di pacchetti da campionare",
    ["selector_id"] = "Id selettore",
    ["selector_name"] = "Nome del campionatore",
    ["server_nw_latency_ms"] = "Network RTT/2 nprobe <-> server (msec)",
    ["shortest_flow_pkt"] = "Il pacchetto più corto (byte) del flusso",
    ["sip_bye_ok_time"] = "SIP time (epoch) of BYE OK",
    ["sip_bye_time"] = "SIP time (epoch) of BYE",
    ["sip_c_ip"] = "SIP C IP adresses",
    ["sip_call_id"] = "SIP call-id",
    ["sip_call_state"] = "SIP Call State",
    ["sip_called_party"] = "SIP Called party",
    ["sip_calling_party"] = "SIP Call initiator",
    ["sip_cancel_ok_time"] = "SIP time (epoch) of CANCEL OK",
    ["sip_cancel_time"] = "SIP time (epoch) of CANCEL",
    ["sip_invite_failure_time"] = "SIP time (epoch) of INVITE FAILURE",
    ["sip_invite_ok_time"] = "SIP time (epoch) of INVITE OK",
    ["sip_invite_time"] = "SIP time (epoch) of INVITE",
    ["sip_reason_cause"] = "SIP Cancel/Bye/Failure reason cause",
    ["sip_response_code"] = "SIP failure response code",
    ["sip_ringing_time"] = "SIP time (epoch) of RINGING",
    ["sip_rtp_codecs"] = "SIP RTP codecs",
    ["sip_rtp_ipv4_dst_addr"] = "SIP RTP stream dest IP",
    ["sip_rtp_ipv4_src_addr"] = "SIP RTP stream source IP",
    ["sip_rtp_l4_dst_port"] = "SIP RTP stream dest port",
    ["sip_rtp_l4_src_port"] = "SIP RTP stream source port",
    ["sip_trying_time"] = "SIP time (epoch) of Trying",
    ["smtp_mail_from"] = "Mail sender",
    ["smtp_rcpt_to"] = "Mail recipient",
    ["src_as"] = "Sorgente BGP AS",
    ["src_as_map"] = "Organization name for SRC_AS",
    ["src_as_path_1"] = "Percorso Sorgente AS posizione 1",
    ["src_as_path_10"] = "Percorso Sorgente AS posizione 10",
    ["src_as_path_2"] = "Percorso Sorgente AS posizione 2",
    ["src_as_path_3"] = "Percorso Sorgente AS posizione 3",
    ["src_as_path_4"] = "Percorso Sorgente AS posizione 4",
    ["src_as_path_5"] = "Percorso Sorgente AS posizione 5",
    ["src_as_path_6"] = "Percorso Sorgente AS posizione 6",
    ["src_as_path_7"] = "Percorso Sorgente AS posizione 7",
    ["src_as_path_8"] = "Percorso Sorgente AS posizione 8",
    ["src_as_path_9"] = "Percorso Sorgente AS posizione 9",
    ["src_father_proc_name"] = "Src father process name",
    ["src_father_proc_pid"] = "Src father process PID",
    ["src_fragments"] = "Numero di pacchetti frammentati sorg->dest",
    ["src_intf_name"] = "Nome Interfaccia Client",
    ["src_ip_city"] = "Città in cui cui si trova l'IP sorgente",
    ["src_ip_country"] = "Nazione in cui cui si trova l'IP sorgente",
    ["src_ip_lat"] = "Latitudine in cui si trova l'IP di sorgente",
    ["src_ip_long"] = "Longitudine in cui si trova l'IP di sorgente",
    ["src_os_name"] = "Nome OS del Client",
    ["src_proc_actual_memory"] = "Src process actual memory (bytes)",
    ["src_proc_average_cpu_load"] = "Src process avg load (%% * 100)",
    ["src_proc_name"] = "Src process name",
    ["src_proc_num_page_faults"] = "Src process num pagefaults",
    ["src_proc_package_name"] = "Client Process Package",
    ["src_proc_pctg_iowait"] = "Src process iowait time %% (%% * 100)",
    ["src_proc_peak_memory"] = "Src process peak memory (bytes)",
    ["src_proc_pid"] = "Src process PID",
    ["src_proc_uid"] = "Src process UID",
    ["src_proc_user_name"] = "Src process user name",
    ["src_sw_version"] = "Versione Software del Client",
    ["src_to_dst_avg_throughput"] = "Throughput medio da sorgente a destinazione (bps)",
    ["src_to_dst_max_throughput"] = "Massimo throughput da sorgente a destinazione (bps)",
    ["src_to_dst_min_throughput"] = "Minimo throughput da sorgente a destinazione (bps)",
    ["src_tos"] = "TOS/DSCP (sorg->dest)",
    ["src_user"] = "Client Username",
    ["src_vlan"] = "VLAN Sorgente (VLAN interna in QinQ)",
    ["ssdp_host"] = "SSDP Host",
    ["ssdp_method"] = "SSDP Method",
    ["ssdp_server"] = "SSDP Server",
    ["ssdp_type"] = "SSDP Type",
    ["ssdp_usn"] = "SSDP USN",
    ["suricata_app_proto"] = "Suricata Application Protocol",
    ["suricata_flow_id"] = "Suricata Flow ID",
    ["tcpAckNum"] = "Numero Ack TCP",
    ["tcpSeqNum"] = "Numero Sequenze TCP",
    ["tcpUrgentPtr"] = "TCP Urgent Ptr",
    ["tcpWindowSize"] = "Dimensione finestra TCP",
    ["tcp_flags"] = "Cumulativo di tutte le Flags TCP del flusso",
    ["tcp_win_max_in"] = "Max TCP Window (sorgente->destinazione)",
    ["tcp_win_max_out"] = "Max TCP Window (dst->src)",
    ["tcp_win_min_in"] = "Finestra Min TCP Window (sorgente->destinazione)",
    ["tcp_win_min_out"] = "Min TCP Window (dst->src)",
    ["tcp_win_mss_in"] = "TCP Max Segment Size (sorgente->destinazione)",
    ["tcp_win_mss_out"] = "TCP Max Segment Size (dst->src)",
    ["tcp_win_scale_in"] = "TCP Window Scale (sorgente->destinazione)",
    ["tcp_win_scale_out"] = "TCP Window Scale (dst->src)",
    ["tls_cert_after"] = "Certificato TLS: After",
    ["tls_cert_dn"] = "Certificato TLS: DN",
    ["tls_cert_not_before"] = "Certificato TLS: Not Before",
    ["tls_cert_sha1"] = "Fingerprint del Certificato TLS",
    ["tls_cert_sn"] = "Certificato TLS: S/N",
    ["tls_cert_subject"] = "Certificato TLS: Subject",
    ["tls_server_name"] = "Nome server TLS",
    ["tls_version"] = "Versione TLS",
    ["total_bytes_exp"] = "Bytes totali esportati",
    ["total_flows_exp"] = "Numero totale di flussi esportati",
    ["total_pkts_exp"] = "Totale pacchetti flusso esportati",
    ["untunneled_ipv4_dst_addr"] = "Indirizzo di destinazione IPv4 non tunneled",
    ["untunneled_ipv4_src_addr"] = "Indirizzo di origine IPv4 non tunneled",
    ["untunneled_ipv6_dst_addr"] = "Indirizzo di destinazione IPv6 non tunneled",
    ["untunneled_ipv6_src_addr"] = "Indirizzo di origine IPv6 non tunneled",
    ["untunneled_l4_dst_port"] = "Porta di destinazione IPv4 non tunneled",
    ["untunneled_l4_src_port"] = "Porta sorgente IPv4 non tunneled",
    ["untunneled_protocol"] = "Byte di protocollo IP non incanalato",
    ["upstream_session_id"] = "Identificatore di sessione upstream (e.g. L2TP) o 0 se sconosciuto",
    ["upstream_tunnel_id"] = "Identificatore del tunnel upstream (e.g. GTP TEID, VXLAN VNI) o 0 se sconosciuto",
    ["user_name"] = "Nome Utente",
  },
  ["flow_risk"] = {
    ["malicious_signature_detected"] = "Signature JA3 del CLient Probabilmente Maliziosa",
    ["ndpi_binary_application_transfer"] = "Trasferimento File Sospetto",
    ["ndpi_clear_text_credentials"] = "Credenziali in chiaro",
    ["ndpi_desktop_or_file_sharing_session"] = "Sessione Desktop o File Sharing",
    ["ndpi_dns_suspicious_traffic"] = "Traffico DNS Sospetto",
    ["ndpi_http_invalid_content"] = "Contenuto HTTP invalido",
    ["ndpi_http_numeric_ip_host"] = "Host HTTP Numerico",
    ["ndpi_http_suspicious_header"] = "Header HTTP Sospetto",
    ["ndpi_http_suspicious_url"] = "URL HTTP Sospetto",
    ["ndpi_http_suspicious_user_agent"] = "User-Agent HTTP Sospetto",
    ["ndpi_known_protocol_on_non_standard_port"] = "Applicazione Nota su Porta Non-Standard",
    ["ndpi_malformed_packet"] = "Pacchetto malformato",
    ["ndpi_malicious_ja3"] = "Probabile Signature Maliziosa (JA3)",
    ["ndpi_malicious_sha1_certificate"] = "Certificato SHA1 TLS Malizioso",
    ["ndpi_no_risk"] = "Nessun rischio",
    ["ndpi_risky_asn"] = "Rischioso come",
    ["ndpi_risky_domain"] = "Dominio Rischioso",
    ["ndpi_smb_insecure_version"] = "Versione SMB Non Sicura",
    ["ndpi_ssh_obsolete_client_version_or_cipher"] = "Versione Client SSH Obsoleta",
    ["ndpi_ssh_obsolete_server_version_or_cipher"] = "Versione Server SSH Obsoleta",
    ["ndpi_suspicious_dga_domain"] = "Nome a dominio sospetto DGA",
    ["ndpi_tls_cert_validity_too_long"] = "Validità certificato TLS maggiore di 13 mesi",
    ["ndpi_tls_certificate_expired"] = "Certificato TLS Scaduto",
    ["ndpi_tls_certificate_mismatch"] = "Certificato TLS Errato",
    ["ndpi_tls_fatal_alert"] = "Allarme TLS Fatale",
    ["ndpi_tls_missing_sni"] = "TLS SNI Mancante",
    ["ndpi_tls_not_carrying_https"] = "TLS (probabilmente) non contentente HTTPS",
    ["ndpi_tls_obsolete_version"] = "Versione TLS Obsoleta",
    ["ndpi_tls_selfsigned_certificate"] = "Certificato TLS Self-signed",
    ["ndpi_tls_suspicious_esni_usage"] = "Uso sospetto del TLS ESNI",
    ["ndpi_tls_suspicious_extension"] = "Estensione TLS Sospetta",
    ["ndpi_tls_uncommon_alpn"] = "TLS ALPN non comune",
    ["ndpi_tls_weak_cipher"] = "Cifrario TLS Debole",
    ["ndpi_unsafe_protocol"] = "Applicazione non Sicura",
    ["ndpi_url_possible_rce_injection"] = "Possibile Remote Code Execution (RCE)",
    ["ndpi_url_possible_sql_injection"] = "Possibile SQL Injection",
    ["ndpi_url_possible_xss"] = "Possibile XSS",
  },
  ["flows_page"] = {
    ["active_flows"] = "Flussi %{filter} Attivi",
    ["actual_throughput"] = "Thpt Corrente",
    ["alert_type_severity"] = "Severità",
    ["alerted"] = "Allarmato",
    ["all_alerted"] = "Tutti gli Allarmati",
    ["all_ases"] = "Tutti gli AS",
    ["all_categories"] = "Tutte le Categorie",
    ["all_devices"] = "Tutti gli Esportatori di Flussi",
    ["all_dscp"] = "Tutti DSCP",
    ["all_flows"] = "Tutti i Flussi",
    ["all_host_pool"] = "Tutte Host Pools",
    ["all_hosts"] = "Tutti gli hosts",
    ["all_inIfIdx"] = "Tute le Interfacce di Input",
    ["all_ip_versions"] = "Tutte le Versioni",
    ["all_l4_protocols"] = "Tutti i Protocolli",
    ["all_misbehaving"] = "Tutti i Misbehaving",
    ["all_networks"] = "Tutti Networks",
    ["all_outIfIdx"] = "Tutte le Interfacce di Output",
    ["all_proto"] = "Tutti i Protocolli",
    ["all_vlan_ids"] = "Tutte Le VLAN",
    ["applications"] = "Applicazioni",
    ["blocked"] = "Bloccato",
    ["client_as"] = "AS Client",
    ["device_ip"] = "Esportatore di Flussi",
    ["direction"] = "Direzione",
    ["dscp"] = "DSCP",
    ["exporters"] = "Esportatori",
    ["flows"] = "Flussi %{filter}",
    ["flows_as_client"] = "Flussi come Client",
    ["flows_as_server"] = "Flussi come Server",
    ["host"] = " [ Host %{host} ]",
    ["host_pool_id"] = "Host Pool %{key}",
    ["hosts"] = "Host",
    ["icmp_only"] = "Solo ICMP",
    ["idle_flows_not_listed"] = "Flussi idle non listati.",
    ["inIfIdx"] = "Interfaccia di Input",
    ["ip_version"] = "Versione IP",
    ["ipv4_only"] = "Solo IPv4",
    ["ipv6_only"] = "Solo IPv6",
    ["l4_protocol"] = "Protocollo",
    ["local_cli_remote_srv"] = "Client Locale - Server Remoto",
    ["local_only"] = "Solo Locali",
    ["local_srv_remote_cli"] = "Server Locale - Client Remoto",
    ["misbehaving"] = "Misbehaving",
    ["multicast"] = "Multicast/Broadcast",
    ["network"] = " [ Rete %{network} ]",
    ["networks"] = "Networks",
    ["non_multicast"] = "Non-Multicast/Non-Broadcast",
    ["normal"] = "Normale",
    ["one_way_multicast"] = "Multicast a Singola Direzione/Broadcast",
    ["one_way_non_multicast"] = "A Singola Direzione Non-Multicast/Non-Broadcast",
    ["outIfIdx"] = "Interfaccia di Output",
    ["port"] = " [ Porta %{port} ]",
    ["recently_active_flows"] = "Flussi Attivi %{filter}",
    ["remote_only"] = "Solo Remoti",
    ["server_as"] = "AS Server",
    ["sflow_devices"] = "Dispositivi sFlow",
    ["source_id"] = "Id Sorgente",
    ["tcp_only"] = "Solo TCP",
    ["tcp_state"] = "Stato TCP",
    ["tcp_state_closed"] = "Chiuso",
    ["tcp_state_connecting"] = "Connessione in Corso",
    ["tcp_state_established"] = "Connesso",
    ["tcp_state_reset"] = "Resettato",
    ["tcp_state_unknown"] = "Sconosciuto",
    ["tot_bytes"] = "Bytes Totali: ",
    ["tot_throughput"] = "Throughput Totale: ",
    ["total_bytes"] = "Bytes Totali",
    ["total_bytes_rcvd"] = "Byte Totali Ricevuti",
    ["total_bytes_sent"] = "Byte Totali Trasmessi",
    ["total_bytes_unknown"] = "Totale dei Byte di Applicazioni Sconosciute",
    ["udp_only"] = "Solo UDP",
    ["vlan"] = "VLAN",
  },
  ["geo_map"] = {
    ["browser_reported_home_map"] = "Il browser ha riportato la posizione base sulla mappa",
    ["configure_geo_ip"] = "Configura Geo IP",
    ["geo_ip"] = "Geo IP",
    ["geo_map"] = "Mappa Geografica",
    ["geolocation_error"] = "Errore di Geolocalizzazione",
    ["geolocation_warning"] = "Avvertimento",
    ["hosts_geomap"] = "Mappa Geografica degli Hosts",
    ["latitude"] = "Latitudine",
    ["longitude"] = "Longitudine",
    ["note"] = "NOTE",
    ["note_active_flows"] = "Avere flussi attivi tra i peer con indirizzi IP pubblici.",
    ["note_compiled_ntopng_with_geolocation"] = "Aver compilato il software con la geolocalizzazione ed averlo avviato con quella.",
    ["note_google_maps_browser_api_key"] = "Una chiave browser per l'API di Google Maps può essere necesaria. Unformazioni precise sulla generazione di chiavi <a href='%{url_google}'>possono essere trovate qui</a>.<br>Una volta generata, la chiave può essere caricata attraverso le <a href='%{url_prefs}'><i class=\"fas fa-flask\"></i> Impostazioni</a>.",
    ["note_html_browser_geolocation"] = "HTML la <A HREF=%{url}>geolocalizzazione browser</A> è usata per individusare sulla mappa gli host basati su posizioni sconosciute.",
    ["note_requirements_visualize_maps"] = "Per visualizzare le mappe devi",
    ["note_working_internet_connection"] = "Avere una connessione ad Internet funzionante.",
    ["unavailable_geolocation"] = "La geolocalizzazione non è supportata dal tuo browser o è disabilitata.",
    ["using_default_location"] = "E' in uso la posizione di default.",
    ["warning_accuracy"] = "L'accuratezza dipende dal database di geolocalizzazione e può essere approssimata",
  },
  ["graphs"] = {
    ["95_perc"] = "95o Perc",
    ["95_perc_rcvd"] = "Ricevuto 95th Perc",
    ["95_perc_sent"] = "Inviato 95th Perc",
    ["active_devices"] = "Dispositivi Attivi",
    ["active_flows"] = "Flussi Attivi",
    ["active_host_contacts"] = "Contatti Host Attivi",
    ["active_hosts"] = "Hosts Attivi",
    ["active_http_servers"] = "Servers HTTP Attivi",
    ["active_monitoring"] = "Monitoraggio Attivo",
    ["actual_memory"] = "Memoria Corrente",
    ["app_connect"] = "Connessione App",
    ["arp_replies"] = "Risposte ARP",
    ["arp_requests"] = "Richieste ARP",
    ["arp_rqst_sent_rcvd_rpls"] = "Richieste ARP Trasmesse / Rispose Ricevute",
    ["average"] = "Media",
    ["average_traffic"] = "Traffico/sec medio",
    ["avg"] = "Media",
    ["avg_rcvd"] = "Media Ricevuto",
    ["avg_sent"] = "Media Inviato",
    ["binary_operators_note"] = "Per ulteriori informazioni sugli indicatori EMA, SMA e RSI consultare la <a href=\"%{url}\">documentazione online</a>.",
    ["blocked_flows"] = "Flussi Bloccati",
    ["bytes_sent_non_unicast"] = "Byte Trasmessi Non Unicast",
    ["bytes_sent_unicast"] = "Byte Trasmessi Unicast",
    ["cli_active_flows_anomalies"] = "Anomalie: Flussi del Client Attivi",
    ["cli_active_flows_behaviour"] = "Comportamento: Flussi del Client Attivi",
    ["cli_score"] = "Score come client",
    ["cli_score_anomalies"] = "Anomalie: Score Client",
    ["cli_score_behaviour"] = "Comportamento: Score Client",
    ["contacts"] = "Contatti",
    ["custom"] = "Altro",
    ["discarded_probing_bytes"] = "Traffico di Probing Scartato",
    ["discarded_probing_packets"] = "Pacchetti di Probing Scartati",
    ["dns_qry_rcvd_rsp_sent"] = "Query DNS Ricevute / Risposte Trasmesse",
    ["dns_qry_sent_rsp_rcvd"] = "Query DNS Trasmesse / Risposte Ricevute",
    ["download_chart_data"] = "Scarica i dati del grafico",
    ["download_records"] = "Records di Download",
    ["dropped_flows"] = "Flussi Persi",
    ["dscp_classes"] = "Classi DSCP",
    ["dumped_flows"] = "Flussi Esportati",
    ["echo_reply_packets"] = "Pacchetti ICMP Echo Reply",
    ["echo_request_packets"] = "Pacchetti ICMP Echo Request",
    ["export_flows_as_txt"] = "Esporta flussi come file TXT",
    ["flows_status_overview"] = "Sommario Stato dei Flussi per %{protocol}",
    ["flows_vs_local_hosts"] = "Flussi Attivi vs Host Locali",
    ["flows_vs_traffic"] = "Flussi Attivi vs Traffico",
    ["get_permanent_link"] = "Ottieni link permanente",
    ["graph_info"] = "Info sul Grafico",
    ["host_contacts_behaviour"] = "Comportamento: Contatti",
    ["host_unreachable_flows"] = "Totale Flussi ICMP con Host Unreachable",
    ["hosts_anomalies"] = "Anomalie Hosts",
    ["http_stats"] = "Statistiche HTTP",
    ["iface_score_anomalies"] = "Anomalie: Score",
    ["iface_score_behavior"] = "Comportamento: Score",
    ["iface_traffic_anomalies"] = "Anomalie: Traffico RX/TX",
    ["iface_traffic_rx_behavior"] = "Comportamento: Traffico RX",
    ["iface_traffic_tx_behavior"] = "Comportamento: Traffico TX",
    ["influxdb_not_responding"] = "La query è stata cancellata poiché InfluxDB non sta rispondendo. Il timeout delle query può esser configurato dalle <a href=\"%{url}\">%{flask_icon} Preferenze</a> .",
    ["last_ms"] = "Ultimo ms",
    ["loc_host_anomalies"] = "Anomalie Hosts Locali",
    ["lower_bound"] = "Limite inferiore",
    ["max"] = "Max",
    ["max_ms"] = "Massimo ms",
    ["max_rtt"] = "Tempo RTT Massimo",
    ["memory_vs_hosts_flows"] = "Memoria del Processs vs Host/Flussi",
    ["min_rtt"] = "Tempo RTT Minimo",
    ["minute_interface_top_stats"] = "Statistiche sul Top Traffico per Minuto",
    ["name_lookup"] = "Risoluzione Nome",
    ["new_flows"] = "Nuovi Flussi",
    ["note_ases_rcvd"] = "Il traffico ricevuto è il traffico ricevuto dal sistema autonomo.",
    ["note_ases_sent"] = "Il traffico inviato è il traffico inviato dal sistema autonomo.",
    ["note_ases_traffic"] = "Il traffico mostrato è inviato e ricevuto dal sistema autonomo.",
    ["note_chart_zoom"] = "Fare click e trascinare sul grafico per effettuare lo zoom.",
    ["note_protocols_shown"] = "I protocolli non visti nel lasso di tempo selezionato non sono elencati nel dropdown.",
    ["num_blacklisted_flows"] = "Flussi con hosts Blacklistati",
    ["num_calls"] = "Numero Chiamate",
    ["num_ms_rtt"] = "Round-Trip Time ms",
    ["num_nfq_drops"] = "Pacchetti Droppati",
    ["num_nfq_pct"] = "Riempimento Coda %%",
    ["packet_drops"] = "Perdite Pacchetti",
    ["packets_vs_drops"] = "Pacchetti vs Pacchetti Persi",
    ["peak_memory"] = "Memoria di Picco",
    ["percentile"] = "95o <a href=\"%{url}\">Percentile</a>",
    ["permanent_link"] = "Link Permanente",
    ["pkt_drops"] = "Perdite Pct",
    ["process_alerts"] = "Allarme del Processo",
    ["process_memory"] = "Memoria del Processo",
    ["query_too_much_time"] = "La richiesta sta impiegando troppo tempo.",
    ["query_too_slow"] = "La richiesta ha preso troppo tempo.",
    ["query_was_aborted"] = "La richiesta è stata annullata.",
    ["raw_flows"] = "Flussi Raw",
    ["raw_proto_flows"] = "Flussi Raw %{proto}",
    ["red_line_unreachable"] = "Una linea rossa indica il tempo in cui l'host non era raggiungibile",
    ["rem_host_anomalies"] = "Anomalie Hosts Remoti",
    ["reset_zoom"] = "Reset Zoom",
    ["right_axis_note"] = "(asse destro)",
    ["score"] = "Score",
    ["score_vs_hosts_flows"] = "Score rispetto a Hosts/Flussi",
    ["srv_active_flows_anomalies"] = "Anomalie: Flussi Attivi del Server",
    ["srv_active_flows_behaviour"] = "Comportamento: Flussi Attivi del Server",
    ["srv_score"] = "Score come Server",
    ["srv_score_anomalies"] = "Anomalie: Score Server",
    ["srv_score_behaviour"] = "Comportamento: Score Server",
    ["tcp_finack_packets"] = "Pacchetti TCP FIN+ACK",
    ["tcp_packets"] = "Pacchetti TCP",
    ["tcp_packets_keep_alive"] = "Pacchetti TCP KeepAlive",
    ["tcp_packets_lost"] = "Pacchetti TCP Persi",
    ["tcp_packets_ooo"] = "Pacchetti TCP Fuori Ordine",
    ["tcp_packets_retr"] = "Pacchetti TCP Ritrasmessi",
    ["tcp_retr_ooo_lost"] = "TCP Ritrasmessi Fuori Ordine e Persi",
    ["tcp_rst_packets"] = "Pacchetti TCP RST",
    ["tcp_rx_stats"] = "Statistiche TCP Ricevuto",
    ["tcp_syn_packets"] = "Pacchetti TCP SYN",
    ["tcp_syn_vs_tcp_synack"] = "Pacchetti TCP SYN vs SYN+ACK",
    ["tcp_synack_packets"] = "Pacchetti TCP SYN+ACK",
    ["tcp_tx_stats"] = "Statistiche TCP Trasmesso",
    ["the_select_timeframe_too_small_top_talkers"] = "L'intervallo selezionato è troppo piccolo per determinare i top talker. Seleziona almeno un intervallo di 1 minuto.",
    ["top_checks"] = "Top %{subdir} Script per Durata",
    ["top_clients"] = "Top Client %{protocol}",
    ["top_interfaces"] = "Top Interfacce",
    ["top_l7_contacts"] = "Top Contatti L7",
    ["top_local_receivers"] = "Migliori Ricevitori Locali",
    ["top_local_senders"] = "Migliori Trasmittenti Locali",
    ["top_ndpi_categories"] = "Top Categorie",
    ["top_ndpi_protocols"] = "Top Protocolli",
    ["top_profiles"] = "Top Profili",
    ["top_receivers"] = "Top Destinatari",
    ["top_senders"] = "Top Mittenti",
    ["top_servers"] = "Top Server %{protocol}",
    ["tot_ms"] = "Totale ms",
    ["total_alerted_flows"] = "Totale Flussi Allarmati",
    ["total_unreachable_flows"] = "Totale Flussi ICMP Port Unreachable",
    ["traffic"] = "Traffico",
    ["traffic_nfq"] = "Traffico Totale/Userspace",
    ["traffic_rcvd"] = "Traffico RX",
    ["traffic_rxtx"] = "Traffico RX/TX",
    ["traffic_sent"] = "Traffico TX",
    ["traffic_userspace"] = "Userspace",
    ["traffic_vs_errors"] = "Traffico vs Errori",
    ["trend"] = "Trend",
    ["udp_packets"] = "Pacchetti UDP",
    ["udp_sent_unicast_vs_non_unicast"] = "UDP trasmesso Unicast vs Non Unicast",
    ["unreachable_host"] = "Host Non Raggiungibile",
    ["upper_bound"] = "Banda Superiore",
    ["zmq_flow_coll_drops"] = "Drop Collezionamento Flussi",
    ["zmq_flow_coll_udp_drops"] = "Drop sul Socket di Collezionamento",
    ["zmq_msg_rcvd_vs_drops"] = "Messaggi ZMQ Collezionati vs Persi",
    ["zmq_received_flows"] = "Pacchetti ZMQ Ricevuti",
    ["metrics_prefixes"] = {
      ["broadcast_bytes"] = "Bytes di Broadcast",
      ["drops"] = "Perdite",
      ["num_devices"] = "Num Dispositivi",
      ["num_flows"] = "Num Flussi",
      ["num_hosts"] = "Num Host",
      ["num_http_hosts"] = "Num Host HTTP",
      ["num_ms_rtt"] = "RTT ms",
      ["packets"] = "Pacchetti",
    },
    ["metrics_suffixes"] = {
      ["by_as"] = "%{metric} per AS",
      ["egress"] = "In Uscita",
      ["ingress"] = "In Ingresso",
      ["inner"] = "Interno",
      ["rcvd"] = "rcvt",
      ["sent"] = "inviati",
    },
  },
  ["hash_table"] = {
    ["AggregatedFlowHash"] = "Flussi Aggregati",
    ["AutonomousSystemHash"] = "Autonomous Systems",
    ["CountriesHash"] = "Paesi",
    ["FlowHash"] = "Flussi",
    ["HostHash"] = "Hosts",
    ["MacHash"] = "MACs",
    ["VlanHash"] = "VLANs",
  },
  ["host_config"] = {
    ["active_monitor_enable"] = "Il Monitoraggio Attivo permette di monitorare lo stato e la latenza degli host. Può essere abilitato <a href=\"%{url}\">qui</a>.",
    ["drop_all_host_traffic"] = "Scarta Tutto il Traffico dell'Host",
    ["hide_from_top"] = "Nascondi da Top Talker",
    ["hide_host_from_top_descr"] = "Ignora l'host %{host} nel calcolo dei top talker",
    ["host_alias"] = "Alias Host",
    ["host_pool"] = "Host Pool",
    ["host_traffic_policy"] = "Politica di Traffico dell'host",
    ["modify_host_pool_policy_btn"] = "Modifica la Politica dell'host pool",
    ["trigger_alerts_for_host"] = "Genera allarmi per l'host %{host}",
    ["trigger_host_alerts"] = "Genera allarmi per l'host",
    ["unblock_host_traffic"] = "Sblocca Traffico Host",
  },
  ["host_details"] = {
    ["active_alerted_flows"] = "Flussi Attivi Allarmati",
    ["behavioural_anomalies"] = "Contatore Anomalie Comportamentali",
    ["blocked_traffic"] = "Traffico Bloccato",
    ["client_score"] = "Client",
    ["disabled_alert_type_help"] = "Spegne la generazione di allarmi basata sullo stato dei flussi",
    ["host"] = "Host",
    ["host_cannot_be_found_message"] = "L'host %{host} non può essere trovato.",
    ["host_parameter_missing_message"] = "Parametro host mancante (errore interno ?)",
    ["host_restore_in_progress"] = "Ripristino host %{host}...",
    ["host_traffic_blocked_quota_and_shaper"] = "Una parte di traffico host è stato bloccato da un superamtneo di quota e da uno shaper",
    ["host_traffic_blocked_shaper"] = "Del traffico dell'host è stato bloccato da uno shaper bloccante",
    ["inactive_host_x"] = "%{host} [Inattivo]",
    ["list_of_general_purpose"] = "Dispositivi general purpose: %{list}",
    ["list_of_special_purpose"] = "Dispositivi special purpose: %{list}",
    ["multiple_selection"] = "Più elementi possono essere selezionati premendo il tasto CTRL",
    ["notes"] = "Note",
    ["reset_blacklisted_stats"] = "Reset Statistiche Hosts Blacklisted",
    ["reset_blacklisted_stats_confirm"] = "Sei sicuro di voler resettare le statistiche \\\"%{host}\\\" blacklisted?",
    ["reset_blacklisted_stats_note"] = "<b>NOTE</b>: queste info sono visibili nella host map 'Flussi con Blacklisted Host",
    ["reset_host_stats"] = "Reset delle Statistiche dell'Host",
    ["reset_host_stats_confirm"] = "Resettare le statistiche dell'host \"%{host}\"?",
    ["reset_host_stats_note"] = "<b>NOTA</b>: i grafici storici dell'host potrebbero avere un gap dopo il reset",
    ["reset_stats_in_progress"] = "Le statistiche dell'host saranno resettate entro 5 secondi",
    ["restore_from_cache_message_v1"] = "L'host %{host} è inattivo. Clicca <a href=%{js_code}> qui</a> per ripristinarlo dalla cache.",
    ["server_score"] = "Server",
    ["snmp"] = "SNMP",
    ["status_ignore"] = "Allarme sul Flow Status Disabilitate",
    ["trace_debug_host_info"] = "Host:%{hostinfo}, Vlan: %{vlan}",
    ["trace_debug_host_ip"] = "Host:%{hostip}, Vlan: %{vlan}",
    ["trace_debug_restored_host_info"] = "Info sull'Host Ripristinato",
    ["unknown_device_type"] = "Tipo di Dispositivo Sconosciuto",
  },
  ["host_pools"] = {
    ["active_pool_members"] = "Membri attivi del Pool",
    ["add_host_pool_member"] = "Aggiungi Membro",
    ["alias_or_manufacturer"] = "Alias / Produttore",
    ["and_associated_members"] = "i relativi dati RRD e tutti i membri assocaiti",
    ["cannot_delete_cp"] = "Un pool non può essere eliminato se c'è un utente Captive Portal associato",
    ["change_member_pool"] = "Cambia membro Pool",
    ["change_pool"] = "Cambia Pool",
    ["children_safe"] = "Child Safe",
    ["cidr"] = "CIDR",
    ["click_here_host_pool"] = "Clicca qui per passare alla pagina degli Host Pool",
    ["config_export"] = "Esporta Configurazione",
    ["config_import"] = "Importa Configurazione",
    ["config_import_message"] = "NOTA: gli host pool con lo stesso nome saranno rimpiazzati",
    ["confirm_delete_pool"] = "Vuoi veramente eliminare l'host pool",
    ["confirm_empty_pool"] = "Vuoi veramente eliminare tutti i membri dall'host pool",
    ["confirm_remove_member"] = "Vuoi veramente eliminare il membro",
    ["confirm_reset_pool_quotas"] = "Vuoi veramente resettare tutti i contatori del pool \"%{pool}\"?",
    ["confirm_reset_quotas"] = "Vuoi veramente resettare tutti i contatori delle quote?",
    ["create_new_quotas_here"] = "Puoi definire nuove quote <a href=\"%{url}\">qui</a>.",
    ["create_pool_hint"] = "Puoi creare nuovi pool dal menu Gestici Pool.",
    ["create_pools"] = "Gestisci Pool",
    ["delete_pool"] = "Elimina Pool",
    ["duplicate_member"] = "Duplica l'indirizzo del membro",
    ["duplicate_pool"] = "Duplica il nome del pool",
    ["edit_host_pool_member"] = "Modifica Membri",
    ["edit_host_pools"] = "Modifica gli Host Pool",
    ["empty_pool"] = "Svuota Pool",
    ["enforce_quotas_per_pool_member"] = "Quota per Membro",
    ["enforce_shapers_per_pool_member"] = "Shapers per Membro",
    ["from_pool"] = "dall'host pool",
    ["host_members"] = "Membri Host Pool",
    ["host_pools"] = "Host Pool",
    ["import_hosts"] = "Importare Membri Host Pools",
    ["import_hosts_desc"] = "Upload un file CSV contente i membri della host pool da inserire.",
    ["import_hosts_title"] = "Importa i membri della Host Pool: %{pool}",
    ["invalid_member"] = "Formato indirizzo membro non valido",
    ["ip_address"] = "IP Address",
    ["ip_filter"] = "IPv4/IPv6",
    ["ipv4"] = "IPv4",
    ["ipv6"] = "IPv6",
    ["mac_address"] = "MAC Address",
    ["mac_filter"] = "MAC Address",
    ["manage_pool"] = "Gestisci Pool",
    ["manage_pools"] = "Gestisci la Membership del Pool",
    ["manufacturer_filter"] = "Produttore: %{manufacturer}",
    ["max_members_message"] = "Con questa versione di ntopng puoi assegnare fino a %{maxnum} membri ad ogni host pool. Aggiorna alla versione Enterprise per rimuovere questo limite",
    ["max_pools_message"] = "Con questa versione di ntopng puoi creare fino a %{maxnum} host pool. Aggiorna alla versione Enterprise per rimuovere questo limite.",
    ["member_address"] = "Indirizzo membro",
    ["member_exists"] = "il membro \"%{member_name}\" non è stato aggiunto perchè è già assegnato al pool \"%{member_pool}\".",
    ["member_filter"] = "Membro: %{member}",
    ["member_type"] = "Tipo",
    ["members_limit_reached"] = "limite membri raggiunto",
    ["network"] = "Rete",
    ["network_normalized"] = "la rete \"%{network}\" ha un identificatore host diverso da 0. Verrà invece utilizzata la rete \"%{network_normalized}\".",
    ["new_host_pool"] = "Nuovo Host Pool",
    ["new_pool"] = "Nuovo Pool",
    ["no_hosts_defined"] = "Nessun host pool definito.",
    ["no_pools_defined"] = "Nessun Host Pool definito.",
    ["per_member_quotas"] = "Quando la quota per membro è impostata, ogni gost avrà un conteggio quota separato. Quando la quota per membro non è impostata, tutto il traffico degli host sarà contato assieme per il raggoungimento della quota limite",
    ["per_member_shapers"] = "Quando l'opzione shapers per membro è impostata, ogni host avrà i suoi shaper. Quando l'opzione non è impostata, tutti gli host appartenenti allo stesso pool condividerrano gli stessi shaper",
    ["policy"] = "Politica Corrente",
    ["pool"] = "Nome Pool",
    ["pool_name"] = "Nome Pool",
    ["pool_name_pattern_error"] = "Inserisci solo caratteri minuscoli, numeri o trattino basso",
    ["remove_member"] = "Rimuovi Membro",
    ["remove_member_pool"] = "Eliminare il membro '%{member}' dal pool '%{pool}'?",
    ["reset_quotas"] = "Resetta Quote",
    ["search_member"] = "Cerca Membro",
    ["select_host_pool"] = "<b>Host Pool</b> Attuale",
    ["select_member_type"] = "Seleziona un tipo",
    ["select_new_pool"] = "Seleziona un nuovo pool dal dropdown sottostante per assegnarvi il membro \"%{member}",
    ["select_policy"] = "Seleziona la nuova politica",
    ["specify_member_address"] = "Specifica un indirizzo IPv4/IPv6, una rete o un indirizzo MAC",
    ["specify_pool_name"] = "Specifica il nome del pool",
    ["type_valid"] = "Inserire un %input valido",
    ["view"] = "Vedi",
    ["validation"] = {
      ["abort_add_member"] = "Annulla?",
      ["abort_add_pool"] = "Annulla?",
      ["abort_edit_pool"] = "Annulla?",
    },
  },
  ["hosts_comparison"] = {
    ["hosts_parameter_missing_message"] = "Il parametro degli Hosts è mancante (errore interno ?)",
  },
  ["hosts_map_page"] = {
    ["active_alert_flows"] = "Flussi allertati attivi",
    ["alerted_flows"] = "Totale flussi allertati",
    ["all_flows"] = "Tutti i flussi",
    ["blacklisted_flows_hosts"] = "Flussi con hosts Blacklisted",
    ["dns_bytes"] = "Traffico DNS",
    ["dns_queries"] = "Queries vs Risposte DNS",
    ["ntp_packets"] = "Traffico NTP",
    ["score"] = "Score Host",
    ["syn_distribution"] = "Distribuzione SYN",
    ["syn_vs_rst"] = "SYN vs RST",
    ["syn_vs_synack"] = "SYN vs SYNACK",
    ["tcp_bytes_sent_vs_rcvd"] = "Bytes TCP Inviati/Ricevuti",
    ["tcp_pkts_sent_vs_rcvd"] = "Pacchetti TCP Inviati/Ricevuti",
    ["traffic_ratio"] = "Rapporto Traffico",
    ["unreach_flows"] = "Flussi non Raggiungibili",
    ["labels"] = {
      ["af_c"] = "Flussi allertati come Client",
      ["af_s"] = "Flussi allertati come Server",
      ["b_ratio"] = "Rapporto Bytes [-1 = Download, 0 = Mixed, 1 = Upload]",
      ["blacklisted_as_client"] = "Numero Flussi Client con Blacklisted Hosts",
      ["blacklisted_as_server"] = "Numero Flussi Server con Blacklisted Hosts",
      ["client_score"] = "Score come attacante",
      ["dns_p_r"] = "Risposte Positive DNS ricevute",
      ["dns_r"] = "Bytes DNS Ricevuti",
      ["dns_s"] = "Queries DNS Inviate",
      ["dns_sent"] = "Bytes DNS Inviati",
      ["f_a_c"] = "Flussi allertati come Client",
      ["f_a_s"] = "Flussi allertati come Server",
      ["f_c"] = "Flussi come Client",
      ["f_s"] = "Flussi come Server",
      ["ntp_r"] = "Pacchetti NTP Ricevuti",
      ["ntp_s"] = "Pacchetti NTP Inviati",
      ["p_ratio"] = "Rapporto Pacchetti [-1 = Download, 0 = Mixed, 1 = Upload]",
      ["rst_r"] = "# di RST Ricevuti",
      ["sa_r"] = "# di SYN/ACK Ricevuti",
      ["server_score"] = "Score come vittima",
      ["syn_r"] = "# di SYN Ricevuti",
      ["syn_s"] = "# di SYN Inviati",
      ["tcp_b_r"] = "Bytes TCP Ricevuti",
      ["tcp_b_s"] = "Bytes TCP Inviati",
      ["tcp_p_r"] = "Pacchetti TCP Ricevuti",
      ["tcp_p_s"] = "Pacchetti TCP Inviati",
      ["uf_c"] = "Flussi non raggiungibili come Client",
      ["uf_s"] = "Flussi non raggiungibili come Server",
    },
  },
  ["hosts_stats"] = {
    ["all"] = "Tutti gli",
    ["all_hosts"] = "Tutti gli Host",
    ["as_geolocation"] = "Geolocalizzazione AS",
    ["as_info"] = "Info AS",
    ["as_path"] = "Path AS",
    ["as_prefixes"] = "Prefissi AS",
    ["asn_title"] = "per AS %{asn}",
    ["bgp_updates"] = "Aggiornamenti BGP",
    ["blacklisted"] = "Blacklistati",
    ["blacklisted_hosts_only"] = "Solo Hosts Blacklistati",
    ["blocking_traffic_policy_popup_msg"] = "Il traffico host è influenzato da una Politica di Traffico bloccante",
    ["broadcast_domain_hosts_only"] = "Solo Hosts nel Dominio di Broadcast",
    ["country_title"] = "per Nazione %{country}",
    ["filter_hosts"] = "Filtra Host",
    ["filtered"] = "Filtrati",
    ["filtered_hosts_only"] = "Solo Host Con Traffico Bloccato",
    ["host_pool"] = "Host Pool %{pool_name}",
    ["hosts"] = "Host",
    ["hosts_list"] = "Lista Host",
    ["hosts_page_title"] = "%{all} Host %{traffic_type} %{local_remote} %{protocol} %{os} %{ip_version} %{country_asn_or_mac} %{network} %{vlan}",
    ["idle_hosts_not_listed"] = "Gli host non sono elencati.",
    ["in_network"] = "in %{network}",
    ["ipver_title"] = "IPv%{version_num}",
    ["label_blacklisted_host"] = "Host Blacklisted",
    ["label_broadcast_domain_host"] = "Dominio di Broadcast",
    ["label_local_host"] = "Host Locale",
    ["label_remote_host"] = "Host Remoto",
    ["local"] = "Locali",
    ["local_hosts_only"] = "Solo Host Locali",
    ["location"] = "Posizione",
    ["mac_title"] = "con Mac %{mac}",
    ["more_info_about_as_popup_msg"] = "Più informazioni sull'AS",
    ["pool_title"] = "per il Pool %{poolname}",
    ["probe_traffic"] = "[Probe %{device_ip}]",
    ["remote"] = "Remoti",
    ["remote_hosts_only"] = "Solo Host Remoti",
    ["source_id"] = "Id Sorgente",
    ["traffic_type_all"] = "Tutti gli Host",
    ["traffic_type_one_way"] = "Traffico a Singola Direzione",
    ["traffic_type_two_ways"] = "Traffico Bidirezionale",
    ["user"] = "Utente %{pool_name}",
    ["user_title"] = "per l'Utente %{poolname}",
    ["vlan_title"] = "VLAN %{vlan}",
  },
  ["http_page"] = {
    ["distribution"] = "Distribuzione",
    ["http_queries"] = "Richieste HTTP Effettuate",
    ["http_responses"] = "Risposte HTTP Ricevute",
    ["method"] = "Metodo",
    ["other_method"] = "Altro Metodo",
    ["requests"] = "Richieste",
    ["requests_served"] = "Richieste Servite",
    ["response_code"] = "Codice di Risposta",
    ["response_code_1xx"] = "1xx (Informativo)",
    ["response_code_2xx"] = "2xx (Successo)",
    ["response_code_3xx"] = "3xx (Reindirizzamento)",
    ["response_code_4xx"] = "4xx (Errore Client)",
    ["response_code_5xx"] = "5xx (Errore Server)",
    ["responses"] = "Risposte",
    ["traffic_received"] = "Traffico Ricevuto",
    ["traffic_sent"] = "Traffico Inviato",
    ["virtual_hosts"] = "Host Virtuali",
  },
  ["http_servers_stats"] = {
    ["actual_requests"] = "Richieste Effettive",
    ["bytes_received"] = "Bytes Ricevuti",
    ["bytes_sent"] = "Bytes Inviati",
    ["http_server_ip"] = "IP del Server HTTP",
    ["http_servers"] = "Server HTTP",
    ["http_virtual_host"] = "Host Virtuale HTTP",
    ["local_http_servers"] = "Server HTTP locali",
    ["total_requests"] = "Richieste Totali",
  },
  ["http_status_code"] = {
    ["bad_request"] = "L'URL specificato contiene caratteri invalidi/pericolosi",
    ["enterprise_only"] = "È richiesta la licenza Enterprise per accedere a questa pagina!",
    ["forbidden"] = "Pagina proibita!",
    ["internal_error"] = "Errore interno del server",
    ["not_found"] = "La pagina richiesta non è stata trovata.",
    ["pro_only"] = "È richiesta la licenza Professional/Enterprise per accedere a questa pagina!",
    ["shut_start"] = "Impossibile soddisfare la richiesta in questo momento (probabile booting up o shutting down)",
  },
  ["icmp_page"] = {
    ["icmp_code"] = "Codice",
    ["icmp_message"] = "Messaggio ICMP",
    ["icmp_port_unreachable_extra"] = "%{unreach_protocol} porta %{unreach_port} irraggiungibile su %{unreach_host}",
    ["icmp_type"] = "Tipo",
    ["last_rcvd_peer"] = "Ultimo Peer Ricevuto",
    ["last_sent_peer"] = "Ultimo Peer Inviato",
    ["packets_received"] = "Pacchetti Ricevuti",
    ["packets_sent"] = "Pacchetti Inviati",
  },
  ["if_stats_config"] = {
    ["companion_interface"] = "Interfaccia Compagna",
    ["custom_name"] = "Nome Personalizzato",
    ["custom_name_popup_msg"] = "Specifica un alias per l'interfaccia",
    ["discard_probing_traffic"] = "Scarta il Traffico di Probing",
    ["dump_flows_to_database"] = "Salva i flussi sul Database",
    ["gw_macs"] = "Direzioni basate su traffico MAC Address",
    ["gw_macs_description"] = "Questo è usato per calcolare la direzione del traffico Viene utilizzato per calcolare la direzione del traffico (in entrata o in uscita) in base agli indirizzi MAC forniti (elenco separato da virgole) poiché in alcuni casi (quando si acquisisce traffico da un mirror del traffico o da un pcap) non è possibile conoscere il traffico direzione. Il traffico diretto agli indirizzi MAC configurati è considerato traffico in uscita.<br><b>Nota:</b><br>Nel caso in cui non sia configurato alcun indirizzo MAC, la direzione del traffico viene impostata utilizzando il traffico degli host locali e remoti (-m).",
    ["gw_macs_example"] = "e.g. %{example}",
    ["hide_from_top_networks"] = "Reti Top Nascoste",
    ["hide_from_top_networks_descr"] = "es. %{example}",
    ["interface_network_discovery"] = "Scansione Periodica delle rete tramite l'interfaccia",
    ["interface_speed"] = "Velocità Interfaccia (Mbit/s)",
    ["interface_speed_measure_unit"] = "(Mbit/s)",
    ["interface_speed_popup_msg"] = "Specifica la massima velocità dell'interfaccia",
    ["interface_top_talkers_creation"] = "Crea Top Talker per l'interfaccia",
    ["is_mirrored_traffic"] = "Traffico proveniente da un mirror",
    ["referesh_rate_measure_unit"] = "(sec)",
    ["scaling_factor"] = "Frequenza di Campionamento Pacchetti in Ingresso",
    ["scaling_factor_popup_msg"] = "Dovrebbe corrispondere alla velocità di campionamento dell'interfaccia di cattura",
    ["show_dyn_iface_traffic"] = "Duplica Traffico Disaggregato",
    ["show_dyn_iface_traffic_note"] = "Abilitando questa opzione il traffico destinato a interfacce dinamiche o sotto-interfacce viene duplicato e mantenuto anche sulla interfaccia master",
    ["trigger_interface_alerts"] = "Genera Allarmi per l'interfaccia",
  },
  ["if_stats_networks"] = {
    ["no_broadcast_domains"] = "Nessun dominio di trasmissione dedotto dall'ispezione del traffico ARP.",
    ["no_ip_addresses_read"] = "Nessun indirizzo IP letto dalla configurazione.",
    ["note_ghost_bcast_domains"] = "L'icona %{ghost_icon} identifica domini di broadcast fantasma, ovvero non appartenenti al alcuna delle reti a cui appartengono le interfacce di rete",
    ["note_iface_addresses"] = "Gli indirizzi IP sono letti dalla configurazione dell'interfaccia",
    ["note_iface_bcast_domains"] = "I domini di broadcast sono determinati dal traffico ARP",
  },
  ["if_stats_overview"] = {
    ["active"] = "Attivo",
    ["alert_events"] = "Eventi Allarmi",
    ["all_counters"] = "Tutti i Contatori",
    ["blocked_flows"] = "Flussi Bloccati",
    ["bridged_traffic"] = "Traffico Bridged",
    ["buffer_full"] = "Buffer Pieno",
    ["collected_flows"] = "Flussi Collezionati",
    ["collected_logs"] = "Logs Collezionati",
    ["conntrack_flow_entries"] = "Flussi all'interno di Conntrack",
    ["counter_anomalies"] = "Contatore Anomalie",
    ["direct_mode"] = "Direct Mode",
    ["discarded_probing_traffic"] = "Traffic di Probing Scartato",
    ["dispatched_logs"] = "Events Consegnati",
    ["dropped_alerts_info"] = "Numero di allarmi che non è stato possibile scrivere su disco poiché la coda degli allarmi era piena.",
    ["dropped_flows"] = "Flussi Persi",
    ["dropped_packets"] = "Pacchetti Persi",
    ["drops_only"] = "Solo le Perdite",
    ["dumped_files"] = "File Scritti",
    ["dumped_packets"] = "Pacchetti Scritti",
    ["egress_packets"] = "Pacchetti in Uscita",
    ["expired_maintenance"] = "Scaduto",
    ["export_disabled"] = "Esportazione flussi disabilitata dalle preferenze",
    ["exported_flows"] = "Flussi Esportati",
    ["family"] = "Famiglia",
    ["flows_export_statistics"] = "Esporta le Statistiche dei Flussi",
    ["handled_logs"] = "Eventi Rilevati",
    ["has_traffic_directions"] = "con direzioni RX/TX",
    ["host_correlations"] = "Correlazioni Host",
    ["id"] = "Id",
    ["iface_switch_active_msg"] = "L'interfaccia selezionata <b>%{interface}</b> [ifid: %{ifid}] è ora attiva",
    ["iface_switch_empty_session_msg"] = "Sessione vuota",
    ["iface_switch_error_msg"] = "Errora durante il cambio d'interfaccia",
    ["in_path_interface"] = "Interfaccia in-path (Bump in the Wire)",
    ["ingress_packets"] = "Pacchetti in Ingresso",
    ["ingress_traffic"] = "Traffico in Ingresso",
    ["interface_ip"] = "Interfaccia IP",
    ["interface_name"] = "Nome Interfaccia",
    ["interface_rx_updates"] = "Aggiornamenti RX dell'Interfaccia",
    ["malformed_logs"] = "Log Malformati",
    ["nf"] = "Netfilter",
    ["nf_enobufs"] = "Spazio Buffer Esaurito",
    ["nf_handle_packet_failed"] = "Fallimenti Gestione Pacchetti Accodati",
    ["nf_queue_dropped"] = "Paccheti Persi (Coda Piena)",
    ["nf_queue_pct"] = "Livello Riempimento Coda",
    ["nf_queue_total"] = "Pacchetti Accodati",
    ["nf_queue_user_dropped"] = "Paccheti Persi (Impossibile Inviare in spazio Utente)",
    ["no_license"] = "Nessuno",
    ["note"] = "NOTE",
    ["note_drop_ifstats_dynamic"] = "Le perdite sono disponibili nell'interfaccia principale",
    ["note_drops_sflow"] = "Le perdite vengono segnalate solo per sFlow poiché il protocollo NetFlow non le riporta.<br>In <A HREF=https://www.ietf.org/rfc/rfc3176.txt>sFlow</A> le perdite indicano il numero di pacchetti che dovevano essere campionati ma che non lo sono stati a causa di risorse insufficienti. Il numero di perdite riportato è riferito all'ultimo reset dell'agent. Un numero alto di perdite indica che l'agent di gestione non è in grado di processare i campioni alla velocità con cui sono generati dall'hardware. Aumentando la frequenza di campionamento sFLow ridurrà la frequenza di perdita dei pacchetti. Nota: un agent che non è in grado d'identificare le perdite riporterà sempre un valore zero.",
    ["note_packets"] = "Nelle reti ethernet, ogni pacchetto ha un <A HREF=\"https://en.wikipedia.org/wiki/Ethernet_frame\">overhead di 24 byte</A> [preambolo (7 bytes), inizio frame (1 byte), CRC (4 bytes), e <A HREF=\"http://en.wikipedia.org/wiki/Interframe_gap\">IFG</A> (12 bytes)]. Di tale overhead bisogna tener conto nel calcolo del traffico dell'interfaccia, ma non nel traffico scambiato tra diversi indirizzo IP. Questo accade poiché tale traffico contribuisce al carico sull'interfaccia, ma non può essere conteggiato come traffico tra gli host, dunque potrebbero esserci discrepanze tra i valori del traffico degli host e quelli dell'interfaccia.",
    ["note_probe_zmq_drops_export_queue_full"] = "Il probe ha processato con successo questo flussi ma l'esportazione è fallita poichè la coda di esportazione era piena.",
    ["note_probe_zmq_drops_flow_collection_drops"] = "Pacchetti contenenti flussi droppati da nProbe (NetFlow v9) oppure flussi droppati (NetFlow v5/IPFIX)",
    ["note_probe_zmq_drops_flow_collection_udp_socket_drops"] = "Pacchetti droppati sul socket di collezionamento di nProbe",
    ["note_probe_zmq_timeout_lifetime"] = "Tempo di vita dei flussi configurato su nProbe e reale calcolato sui flussi.",
    ["note_remote_probe_time"] = "Differenza, in secondi, tra i tempo di ntopng e dell'nProbe remoto. Idealmente dovrebbe essere prossima allo zero. Se troppo grande, sincronizzare i clock di ntopng e nProbe",
    ["paused"] = "In Pausa",
    ["probe_ip"] = "Probe IP",
    ["probe_public_ip"] = "Probe IP Pubblico",
    ["probe_throughput"] = "Throughput del Probe",
    ["probe_timeout_idle"] = "Timeout d'Inattività",
    ["probe_timeout_lifetime"] = "Timeout d'Attività",
    ["probe_zmq_drops_export_queue_full"] = "Flussi Droppati per Coda Piena",
    ["probe_zmq_drops_flow_collection_drops"] = "Perdita di Flussi nel Collezionamento",
    ["probe_zmq_drops_flow_collection_udp_socket_drops"] = "Drop sul Socket di Collezionamento",
    ["probe_zmq_num_endpoints"] = "Endpoint di Probe ZMQ attivi",
    ["probe_zmq_num_flow_exports"] = "Flussi esportati",
    ["public_probe_ip"] = "Probe IP Pubblico",
    ["received_traffic"] = "Traffico Ricevuto",
    ["remote_flow_lifetime"] = "Durata Flusso Remoto",
    ["remote_probe"] = "Probe Remoto",
    ["remote_probe_collecting_from_x_devices"] = "Raccoglimento da <a href=\\\"%{url}\\\">%{num} nProbe</a>.",
    ["remote_probe_collector_mode"] = "Modalità Collettore",
    ["remote_probe_edition"] = "Edizione",
    ["remote_probe_license"] = "Licensa",
    ["remote_probe_maintenance"] = "Mantenimento",
    ["remote_probe_showing_first_x_devices"] = "Sono mostrati i primi %{num}.",
    ["remote_probe_time"] = "nProbe Time Drift",
    ["reset_counters"] = "Reimposta Contatori",
    ["rrd_timeseries"] = "Serie Temporali RRD",
    ["send_error"] = "Errori d'Invio",
    ["sflow_counter_updates"] = "Aggiornamenti di Contatori sFlow",
    ["shaped_filtered_packets"] = "Pacchetti Shapati/Filtrati",
    ["snmp"] = "SNMP",
    ["state"] = "Stato",
    ["syslog_statistics"] = "Statistiche Collezionamento Syslog",
    ["traffic_breakdown"] = "Breakdown del Traffico",
    ["traffic_statistics"] = "Statistiche Traffico",
    ["unhandled_flows"] = "Flussi Non Gestiti",
    ["unhandled_logs"] = "Eventi Non Supportati",
    ["zmq_avg_msg_flows"] = "Media flussi per Messaggio",
    ["zmq_encryption_alias"] = "******************",
    ["zmq_encryption_public_key"] = "Chiave di Cifratura ZMQ",
    ["zmq_encryption_public_key_note"] = "Se nProbe è usato come esportatore di flussi è necessario aggiungere l'opzione --zmq-encryption-key '%{key}' al file di configurazione di nProbe.",
    ["zmq_message_drops"] = "Messaggi Persi",
    ["zmq_message_rcvd"] = "Messaggi Collezionati",
    ["zmq_rx_statistics"] = "Statistiche ZMQ RX",
  },
  ["index_page"] = {
    ["applications"] = "Applicazioni",
    ["asns"] = "ASN",
    ["dashboard"] = "Dashboard",
    ["hosts"] = "Host",
    ["live_update"] = "Aggiornamento Live",
    ["minute"] = "Minuto",
    ["never"] = "Mai",
    ["no_packet_warning"] = "Nessun pacchetto è stato ancora ricevuto sull'interfaccia %{ifname}.<p>Attendere %{countdown} secondi finchè questa pagina si ricarica.",
    ["refresh"] = "Aggiorna",
    ["refresh_frequency"] = "Frequenza d'aggiornamento",
    ["refresh_graph_popup_msg"] = "Grafico d'aggiornamento",
    ["seconds"] = "Secondi",
    ["senders"] = "Mittenti",
    ["top_flow_talkers"] = "Top Talkers flusso",
  },
  ["infrastructure_dashboard"] = {
    ["actions"] = "Azioni",
    ["add_instance"] = "Aggiungi istanza %{product}",
    ["chart"] = "Grafico",
    ["edit_instance"] = "Modifica istanza %{product}",
    ["fetching_data"] = "Recupero dati",
    ["information"] = "Informazione",
    ["infrastructure"] = "Infrastruttura",
    ["infrastructure_dashboard"] = "Infrastruttura",
    ["name"] = "Nome",
    ["not_polled_yet"] = "Non ancora richiesto",
    ["ntopng_instance"] = "%{product} Istanza",
    ["remove_instance"] = "Vuoi veramente eliminare l'istanza '%{instance}'?",
    ["status"] = "Stato",
    ["url"] = "URL",
    ["collected_stats"] = {
      ["alerts"] = "Allarme",
      ["download"] = "Download",
      ["engaged_alerts"] = "Allarmi Coinvolti",
      ["flow_alerts"] = "Allerta flusso",
      ["flows"] = "Flussi",
      ["hosts"] = "Hosts",
      ["last_update"] = "Ultimo Aggiornamento",
      ["throughput"] = "Throughput",
      ["upload"] = "Upload",
    },
    ["modal"] = {
      ["bandwidth_threshold"] = "Soglia di Banda",
      ["failed_login"] = "L'autenticazione non è riuscita, controllare se il token di autenticazione utente è stato modificato!",
      ["hint_token"] = "Un token può essere generato dalla pagina del profilo utente.",
      ["insert_instance"] = "%{product}-istanza",
      ["instance_alias"] = "Alias Istanza",
      ["instance_url"] = "URL",
      ["password"] = "Password",
      ["rtt_threshold"] = "Soglia RTT",
      ["seconds"] = "sec",
      ["successfull_login"] = "L'istanza remota ha risposto con successo!",
      ["test_auth"] = "Controllare",
      ["testing_authentication"] = "Testing autenticazione",
      ["threshold_hint"] = "Soglia minima per triggherare un allarme",
      ["token"] = "Token di Autenticazione Utente",
      ["unknown_host"] = "L'autenticazione è fallita, perfavore controllare che l'URL sia corretto!",
      ["url_hint"] = "URL dell'istanza remota",
      ["username"] = "Username",
    },
    ["validation"] = {
      ["abort_add"] = "Annullare l'aggiunta di una nuova istanza?",
      ["abort_edit"] = "Annullare la modifica dell'istanza corrente?",
    },
  },
  ["input_item_list"] = {
    ["dns_input_list_description"] = "Valori separati da virgole di IP DNS permessi. Esempio: 192.168.1.1",
    ["dns_input_list_title"] = "DNS permessi",
    ["ntp_input_list_description"] = "Valori separati da virgole di IP NTP permessi. Esempio: 192.168.1.1",
    ["ntp_input_list_title"] = "NTP Permessi",
    ["smtp_input_list_description"] = "Valori separati da virgole di IP SMTP permessi. Esempio: 192.168.1.1",
    ["smtp_input_list_title"] = "SMTP Permessi",
  },
  ["internals"] = {
    ["alert_drops"] = "Perdita di Allarmi",
    ["alert_drops_descr"] = "Allarmi generati da questo script sono state perse.",
    ["all_check_targets"] = "Tutti i Target degli User Script",
    ["all_hash_tables"] = "Tutte le tabelle hash",
    ["all_periodic_activities"] = "Tutte le Attività Periodiche",
    ["any_periodic_activity_issue"] = "Attività con Problemi",
    ["chart_script_duration"] = "Durata %{script}",
    ["chart_script_rrds"] = "Stats TS %{script}",
    ["check"] = "User Script",
    ["check_target"] = "Target",
    ["checks"] = "Checks",
    ["degraded_performance"] = "Performance Degradata",
    ["duration"] = "Durata",
    ["flow_calls_stats"] = "Stat Chiamate flow.lua",
    ["flow_lua_duration"] = "Durata flow.lua",
    ["hash_entries"] = "Entry nella Hash",
    ["hash_table"] = "Tabella Hash",
    ["hash_table_utilization"] = "Utilizzo Tabella Hash",
    ["hash_tables"] = "Tabelle Hash",
    ["high_idle_entries"] = "Elevato numero di entry idle",
    ["iface_hash_tables"] = "Tabelle Hash di %{iface}",
    ["iface_periodic_scripts"] = "Attività Periodiche di %{iface}",
    ["in_progress_since"] = "In Corso Da",
    ["last_duration_ms"] = "Ultima Durata",
    ["last_num_calls"] = "Ultimo Numero di Chiamate",
    ["last_start_time"] = "Ultimo Avvio",
    ["last_start_time_ago"] = "%{time} fa",
    ["lua"] = "Lua",
    ["max_duration_ms"] = "Durata Massima",
    ["max_duration_secs"] = "Durata Massima",
    ["missed_idle"] = "Idle Saltate",
    ["missed_periodic_update"] = "Aggiornamento Periodico Saltate",
    ["missed_proto_detected"] = "Applicazione Rilevata Saltate",
    ["num_calls"] = "Numero Chiamate %{script}",
    ["num_calls_vs_duration"] = "Chiamate Lua vs Durata",
    ["num_drops"] = "Punti Persi",
    ["num_failed_enqueues"] = "Accodamenti Falliti",
    ["num_missed_calls"] = "Chiamate Saltate",
    ["num_writes"] = "Punti Scritti",
    ["pending_periodic_update"] = "Aggiornamento Periodico in Attesa",
    ["pending_proto_detected"] = "Applicazione Rilevata in Attesa",
    ["periodic_activities"] = "Attività Periodiche",
    ["periodic_activities_descr"] = "Passare il mouse sulle attività listate per ottenere la loro descrizione.",
    ["periodic_activities_last_start_time_descr"] = "Ultimo Avvio indica quando l'attività periodica è stata avviata l'ultima volta.",
    ["periodic_activities_max_duration_secs_descr"] = "Durata Massima indica per quanto tempo lo script può eseguire.",
    ["periodic_activities_not_shown"] = "Solamente le attività periodiche schedulate almeno una volta sono mostrate nella tabella.",
    ["periodic_activities_periodicity_descr"] = "La frequenza indica quanto spesso una attività periodica viene schedata per l'esecuzione.",
    ["periodic_activities_rrd_descr"] = "TS sta per Timeseries, ovvero serie temporali.",
    ["periodic_activities_tot_not_executed_descr"] = "Totale Non Eseguito conta il numero di volte in cui una attività periodica non è stata schedata per l'esecuzione perché era già in corso oppure perché era già in coda ma non c'era nessun thread disponibile per l'esecuzione.",
    ["periodic_activities_tot_running_slow_descr"] = "Totale Esecuzione Lenta conta il numero di volte in cui una attività periodica ha richiesto troppo tempo per completare la propria esecuzione.",
    ["periodic_activity"] = "Attività Periodica",
    ["periodic_activity_issues"] = "Problemi",
    ["periodicity"] = "Periodicità",
    ["queue"] = "Coda",
    ["queued"] = "Accodata",
    ["queues"] = "Code",
    ["rrd_drops"] = "Perdita TS",
    ["script_deadline_exceeded"] = "Esecuzione Lenta",
    ["script_deadline_exceeded_descr"] = "Lo script ha impiegato troppo tempo per terminare",
    ["script_duration"] = "Durata %{script}",
    ["script_not_executed"] = "Non Eseguito",
    ["script_not_executed_descr"] = "Lo script è stato schedulato ma non ancora eseguito",
    ["script_num_calls"] = "Numero Chiamate %{script}",
    ["script_stats"] = "%{script} vs Totale",
    ["sleeping"] = "In sleep",
    ["state_active"] = "Entry in stato Attivo",
    ["state_idle"] = "Entry in stato Idle",
    ["state_idle_vs_active"] = "Entry Idle vs Attive",
    ["status_description"] = "Un'attività periodica può essere in uno dei seguenti stati:",
    ["status_queued_descr"] = "L'attività è stata schedulata ma non sta ancora girando",
    ["status_running_descr"] = "L'attività è in esecuzione",
    ["status_sleeping_descr"] = "L'attività ha completato l'esecuzione e sta aspettando di esser nuovamente schedulata",
    ["successful_calls"] = "Chiamate con Successo",
    ["system_alert_drops_descr"] = "attivare un avviso quando gli avvisi vengono eliminati poiché non c'è spazio nella coda degli avvisi interni. Probabilmente, ciò è dovuto a troppi avvisi generati.",
    ["system_iface_periodic_scripts"] = "Attività Periodiche di Sistema",
    ["time_exceeded"] = "Tempo Ecceduto",
    ["time_usage"] = "Utilizzo del Tempo",
    ["timeseries_queue_length"] = "Lunghezza Coda TS",
    ["timeseries_queued_points"] = "Punti in Coda",
    ["timeseries_writes"] = "TS Scritte",
    ["tot_not_executed"] = "Non Eseguito",
    ["tot_running_slow"] = "Esecuzione Lenta",
    ["total_duration"] = "Totale Durata Script %{subdir}",
    ["total_flow_duration"] = "Totale Durata",
    ["total_num_calls"] = "Totale Numero Chiamate Script %{subdir}",
    ["total_stats"] = "Totale Stat Script %{subdir}",
    ["work_completion"] = "Completamento",
  },
  ["invalid_filters"] = {
    ["double_arg"] = "Sono stati dati due parametri identici \\\"%{args}\\",
    ["empty"] = "Svuotare lista dei filtri",
    ["few_args"] = "Parametri errati, è richiesto un filtro e il suo valore ma ne è stato fornito solo uno \\\"%{args}\\",
    ["incorrect_args"] = "Parametri errati, uno dei parametri non è supportato o errato \\\"%{args}\\",
    ["invalid_subdir"] = "Sottocartella Fornita Invalida",
  },
  ["ja3"] = {
    ["client_hash"] = "Hash JA3 Client",
    ["server_cipher"] = "Cipher JA3 Server",
    ["server_hash"] = "Hash JA3 Server",
    ["server_unsafe_cipher"] = "Stato Cipher",
  },
  ["license_page"] = {
    ["agreement"] = "Contratto di licenza",
    ["license"] = "Licenza",
    ["not_valid"] = "Non Valido",
    ["status"] = "Stato",
    ["valid"] = "Valido",
  },
  ["live_capture"] = {
    ["active_live_captures"] = "Cattura Live",
    ["note"] = "Cattura live del traffico dell'host %{hostinfo}. Lo stream viene interrotto automenticamente dopo 1 minuto o 100k pacchetti.",
  },
  ["live_traffic"] = {
    ["error_generic"] = "Errore generico",
    ["error_not_found"] = "Non trovato",
    ["error_not_granted"] = "Richiesta non autorizzata",
    ["modal_vlan_tagged_with_bpf_confirmation"] = "Questo host ha un tag VLAN. Il filtro BPF specificato, per individuare correttamente i suoi pacchetti, deve contenere la VLAN 51. La VLAN si specifica nel filtro come, ad esempio, <pre>vlan %{vlan} and <i>...resto del filtro...</i></pre>",
    ["modal_vlan_tagged_with_bpf_continue"] = "Continua",
    ["modal_vlan_tagged_with_bpf_title"] = "Download pcap dell'Host",
  },
  ["local_flow_matrix"] = {
    ["error_no_local_hosts"] = "Nessun host locale può essere trovato",
    ["local_flow_matrix"] = "Matrice dei Flussi Locali",
    ["local_hosts_active_flows_matrix"] = "Matrice dei Flussi Attivi degli Local Hosts",
  },
  ["local_hosts_stats"] = {
    ["criteria"] = "Criterio",
    ["looking_glass"] = "Lente d'Ingrandimento",
  },
  ["locales"] = {
    ["cn"] = "Cinese",
    ["cz"] = "Ceco",
    ["de"] = "Tedesco",
    ["en"] = "Inglese",
    ["it"] = "Italiano",
    ["jp"] = "Giapponese",
    ["pt"] = "Portoghese",
  },
  ["login"] = {
    ["add_captive_portal_user"] = "Aggiungi Utente Captive Portal",
    ["add_web_user"] = "Aggiungi Utente Web",
    ["auth_success"] = "Autenticazione riuscita",
    ["auth_token"] = "Token Autenticazione Utente",
    ["blacklisted_ip_notice"] = "Troppi tentativi falliti di autenticazione. Riprovare più tardi.",
    ["change_password"] = "Cambia Password",
    ["confirm_password"] = "Conferma Password",
    ["device_label"] = "Etichetta Dispositivo",
    ["enter_credentials"] = "Inserisci le tue credenziali per accedere a questa rete.",
    ["generate_token"] = "Genera Token",
    ["informative_captive_join"] = "Accedi a Internet",
    ["informative_captive_portal_tos"] = "Ho letto e accettato i <a target='_blank' href='%{url}'>termini di servizio</a>.",
    ["internet_redirecting"] = "Ti stiamo reindirizzando a Internet...",
    ["license"] = "%{product} è rilasciato sotto <a href=\"%{license_url}\">%{license}</a>.",
    ["links"] = "<A target=\\\"_blank\\\" href=\\\"https://www.ntop.org/support/documentation/documentation/\\\">User\\'s Guides</A> | <A target=\\\"_blank\\\" HREF=\\\"https://www.ntop.org/community/\\\">Community</A> | <A target=\\\"_blank\\\" HREF=\\\"https://www.ntop.org/support/need-help-2/commercial-support/\\\">Support</A> | <A target=\\\"_blank\\\" HREF=\\\"https://www.ntop.org/support/documentation/faq/\\\">FAQ</A> | <A target=\\\"_blank\\\" HREF=\\\"https://github.com/ntop\\\">Code</A> | <A target=\\\"_blank\\\" href=\\\"https://www.ntop.org/support/need-help-2/contact-us/\\\">Contact Us</A>",
    ["logging_out"] = "Uscendo dal programma...",
    ["login"] = "Accedi",
    ["logout"] = "Esci",
    ["logout_message"] = "Sei sicuro di voler uscire?",
    ["logout_user_x"] = "Disconnetti %{user}",
    ["must_change_password"] = "La password amministratore predefinita deve essere cambiata. Si prega d'inserire una nuova password qui sotto.",
    ["password"] = "Password",
    ["password_mismatch"] = "Le Password non combaciano",
    ["password_not_valid"] = "Specifica una password differente",
    ["password_pattern_error"] = "Inserisci almeno 5 caratteri validi.",
    ["password_ph"] = "Password (default admin)",
    ["passwords_dont_match"] = "Le password non coincidono.",
    ["unable_to_login"] = "Non riesci ad accedere ?",
    ["unsupported_browser"] = "Il browser che usi non è supportato.<br>Si consiglia l'utilizzo di una versione recente di Firefox o Chrome.",
    ["username"] = "Nome Utente",
    ["username_ph"] = "Nome Utente (default admin)",
    ["web_users"] = "Utenti Web",
    ["welcome_to"] = "Benvenuto in %{product}",
    ["wrong-credentials"] = "Login fallito: username o password invalidi.",
  },
  ["mac_details"] = {
    ["and_n_more_hosts"] = "<a href='%{host_url}'>%{host_label}</a> e <a href='%{url}'>%{num} altri host</a>",
    ["and_one_more_host"] = "<a href='%{host_url}'>%{host_label}</a> e <a href='%{url}'> un altro host</a>",
    ["json_error_inactive"] = "Errore JSON: l'host inattivo è stato eliminato oppure %{product} è terminato.",
    ["mac"] = "Mac",
    ["mac_cannot_be_found_message"] = "Il mac %{mac} non può essere trovato.",
    ["mac_host"] = "<a href='%{host_url}'>%{host_label}</a>",
    ["mac_parameter_missing_message"] = "Parametro Mac mancante (errore interno ?)",
    ["reset_mac_stats"] = "Resetta le statistiche del MAC",
    ["reset_mac_stats_note"] = "<b>NOTA</b>: i grafici storici del MAC potrebbero avere un gap dopo il reset.",
    ["reset_stats_in_progress"] = "Le statistiche del MAC saranno resettate entro 5 secondi",
  },
  ["mac_stats"] = {
    ["all_devices"] = "Tutti i Dispositivi",
    ["all_layer_2_devices"] = "Tutti i dispositivi",
    ["all_manufacturers"] = "Tutti i Produttori",
    ["arp_total"] = "ARP",
    ["dev_layer_2_devices"] = "Dispositivi %{device_type}",
    ["dhcp_only"] = "Solo DHCP",
    ["filter_macs"] = "Filtri MAC",
    ["idle_devices_not_listed"] = "I dispositivi idle non sono listati.",
    ["layer_2_dev_devices"] = "Dispositivi %{device_type}",
    ["layer_2_devices_with_manufacturer"] = "%{title} con produttore %{manufacturer}'",
    ["layer_2_source_devices"] = "Dispositivi %{device_type}",
    ["manufacturer"] = "Produttore",
    ["source_macs"] = "Solo Sorgenti",
  },
  ["manage_configurations"] = {
    ["active_monitoring"] = "Monitoraggio Attivo",
    ["after_behavior_changes"] = "<b>Riavviare</b> %{product} per rendere i cambiamenti di comportamento effettivi.",
    ["after_reset_request"] = "<b>Riavvia</b> %{product} per rendere effettivo il reset.",
    ["checks"] = "Script Utente",
    ["config_import"] = "Importa Configurazione <span class='item'>{{ type }}</span>",
    ["everything"] = "Tutta la configurazione di %{product} (inclusi utenti, preferenze, e tutte le configurazioni sotto)",
    ["export_policy"] = "Esportare la politica nProbe IPS",
    ["infrastructure_instances"] = "Istanze infrastrutturali",
    ["manage_configuration"] = "Gestione Configurazione",
    ["manage_configurations"] = "Gestione Configurazioni",
    ["notifications"] = "Endpoint e Recipient",
    ["pool_endpoint_recipients"] = "Pools (include SNMP, monitoraggio attivo, script utente, endpoints e recipients)",
    ["snmp"] = "SNMP",
    ["factory_reset"] = {
      ["body"] = "Reset configurazione. Tutti i cambiamenti saranno persi.",
      ["title"] = "Reset Configurazione <span class='selected-item'>%{reset_element}</span>",
    },
    ["import_modal"] = {
      ["all_title"] = "Importa Configurazione %{product}",
      ["title"] = "Importa Configurazione <span class='selected-item'>%{import_element}</span>",
    },
    ["messages"] = {
      ["import_all_success"] = "Configurazione importata correttamente. Perfavore <b> riavviare %{product}</b> per rendere effettiva la configurazione importata.",
      ["import_success"] = "La configurazione è stata importata con successo!",
      ["reset_all_success"] = "<b>Riavviare %{product}</b> per rendere il reset effettivo.",
      ["reset_success"] = "Reset avvenuto con successo!",
    },
  },
  ["manage_data"] = {
    ["all_hosts"] = "Tutti",
    ["delete"] = "Cancella i Dati degli Host",
    ["delete_active_interface"] = "Cancella i Dati dell'Interfaccia",
    ["delete_inactive_interfaces"] = "Cancella i Dati delle Interfacce Inattive",
    ["delete_system_interface_data"] = "Cancella i dati dell'Interfaccia Sistema",
    ["delete_tab"] = "Cancella",
    ["export"] = "Esporta i Dati degli Host",
    ["export_tab"] = "Esporta",
    ["hosts"] = "Hosts",
    ["inactive_interfaces_data"] = "Interfacce Inattive",
    ["inactive_interfaces_description"] = "Gestisci dati delle <b>Interfacce Inattive</b>.",
    ["ip_or_mac_address"] = "IP o MAC Address o /24",
    ["local_hosts"] = "Locali",
    ["mac_or_ip_required"] = "Si prega di specificare un IP o MAC Address valido",
    ["manage_data"] = "Gestisci Dati",
    ["remote_hosts"] = "Remoti",
    ["single"] = "Singoli",
    ["system_interface_data"] = "Interfaccia Sistema",
    ["system_interface_description"] = "Gestisci dati della <b>Interfaccia Sistema</b>.",
    ["system_interface_note"] = "I dati dell'Interfaccia Sistema includono tutti i dati che si trovano sotto la entry \"Sistema\" del menù, ad esempio la CPU, la memoria, InfluxDB e SNMP.",
  },
  ["manage_users"] = {
    ["add_new_user"] = "Aggiungi Nuovo Utente",
    ["administrator"] = "Amministratore",
    ["allow_pcap_download"] = "Consenti il download di pcap",
    ["allow_pcap_download_descr"] = "Consenti il download di traffico live e pcap",
    ["allowed_interface"] = "Interfaccia Permessa",
    ["allowed_networks"] = "Reti Permesse",
    ["allowed_networks_descr"] = "Lista di reti separate da virgola che l'utente può vedere. Esempio:",
    ["allowed_passwd_charset"] = "Sono permessi caratteri ISO 8859-1 (latin1), lettere maiuscole e minuscole, numeri e simboli speciali",
    ["any_interface"] = "Qualunque Interfaccia",
    ["authentication_lifetime"] = "Durata Autenticazione",
    ["change_user_password"] = "Cambia Password Utente",
    ["change_user_preferences"] = "Cambia Preferenze Utente",
    ["confirm_delete_user"] = "Sei sicuro di voler eliminare %{user}?",
    ["enable_captive_portal"] = "Dopo <a href=\"%{url}\">aver abilitato il Captive Portal</a> sarà possibile definire utenti captive portal",
    ["expires_after"] = "Scade dopo",
    ["group"] = "Gruppo",
    ["host_pool_id"] = "Id Host Pool",
    ["manage"] = "Gestisci",
    ["manage_user_x"] = "Gestisci Utente %{user}",
    ["manage_users"] = "Gestisci Utenti",
    ["new_password"] = "Nuova Password",
    ["new_password_confirm"] = "Conferma Nuova Password",
    ["no_host_pools"] = "Nessun host pool definito. Creane uno <a href='%{url}'>qui</a>",
    ["non_privileged_user"] = "Utente Non Privilegiato",
    ["old_password"] = "Vecchia Password",
    ["the_auth_can_be_perpetual"] = "L'autenticazione può essere duratura o può scadere dopo il periodo di tempo specificato",
    ["the_host_pool_associated"] = "L'host pool a cui l'utente verrà associato dopo essersi autenticato con successo",
    ["token"] = "Token",
    ["token_not_generated"] = "Token non ancora generato",
    ["unlimited"] = "Illimitata",
    ["user_role"] = "Ruolo Utente",
    ["web_captive_users"] = "Utenti Web/Captive Portal",
  },
  ["map_page"] = {
    ["alerted_view"] = "Vista allertata",
    ["allowed"] = "Permesso",
    ["denied"] = "Non Permesso",
    ["fetch_error"] = "Errore durante il recupero dello stato della mappa di servizio.",
    ["filter_hosts"] = "Filtri",
    ["geo_map_notes"] = "I dettagli dell'Host mostrati cliccando sui marker possono essere modificati dalle preferenze <a href=\\\"%{prefs_link}\\\"><i class=\\\"fas fa-cog\\\"></i></a>",
    ["graph_view"] = "Vista Grafico",
    ["learning"] = "The Service Map is <b>learning</b>...",
    ["no_services"] = "Nessun <b>servizio</b> è stato rilevato.",
    ["node_descr"] = "Hosts are represented as circles: <font color=#28a745>Local</font>, <font color=#6c757d>Remote</font>, <font color=#007bff>Multicast</font>, <font color=#17a2b8>Broadcast</font> hosts and <font color=#8004e0>MAC</font> addresses.<li>Arcs represent a communication between two hosts: <font color=#28a745>allowed</font>, <font color=red>denied</font>, and <font color=#ffc107>undecided</font> service status.<li>Double click on a node to jump to the host page. Use the mouse wheel to zoom in/out.</li><li>The <b>Standard View</b> is the normal view of the Service/Periodicity Map, showing which hosts are talking with whom.</li><li>The <b>Alerted View</b> is a going to show only the alerted hosts; the greater the node, the most alerts it has.</li>",
    ["note_1"] = "During the <b>learning</b>, a service status cannot be changed.",
    ["note_2"] = "The services with the <b>Undecided</b> (<kbd class='bg-warning text-dark'><i class='fas fa-hourglass-half'></i></kbd>) status require a confirmation by the user. Once a service status has been confirmed it can be <b>Allowed</b> (<kbd class='bg-success'><i class='fas fa-check'></i></kbd>) or <b>Denied</b> (<kbd class='bg-danger'><i class='fas fas fa-times'></i></kbd>).",
    ["only_mem_hosts"] = "Solo Hosts Attivi",
    ["protocols"] = "Tutti i Protocolli",
    ["set_state"] = "Impostare %{label} lo stato del/dei servizio/i a: ",
    ["standard_view"] = "Vista Standard",
    ["status"] = "Stato",
    ["table_view"] = "Vista a Tabella",
    ["undecided"] = "Indeciso",
    ["aging"] = {
      ["aging"] = "Da sempre",
      ["day"] = "Da ieri",
      ["month"] = "Dal mese scorso",
      ["week"] = "Dalla scoarsa settimana",
    },
  },
  ["metrics"] = {
    ["days"] = "Giorni",
    ["hours"] = "Ore",
    ["mins"] = "Min",
    ["minutes"] = "Minuti",
    ["secs"] = "Sec",
  },
  ["ndpi_categories"] = {
    ["Advertisement"] = "Pubblicità",
    ["Allowed_Site"] = "Sito Permesso",
    ["Antimalware"] = "Antimalware",
    ["Banned_Site"] = "Sito Bannato",
    ["Chat"] = "Chat",
    ["Cloud"] = "Cloud",
    ["Collaborative"] = "Collaborazione",
    ["DataTransfer"] = "Trasferimento Dati",
    ["Database"] = "Database",
    ["Download-FileTransfer-FileSharing"] = "Download/File Sharing",
    ["Email"] = "Email",
    ["FileSharing"] = "File Sharing",
    ["Game"] = "Gioco",
    ["Malware"] = "Malware",
    ["Media"] = "Multimediale",
    ["Mining"] = "Mining",
    ["Music"] = "Musica",
    ["Network"] = "Rete",
    ["Productivity"] = "Produttività",
    ["RPC"] = "RPC",
    ["RemoteAccess"] = "Accesso Remoto",
    ["Shopping"] = "Shopping",
    ["Site_Unavailable"] = "Sito Non Disponibile",
    ["SocialNetwork"] = "Social Network",
    ["SoftwareUpdate"] = "Aggiornamento Software",
    ["Streaming"] = "Streaming",
    ["System"] = "Sistema",
    ["Unspecified"] = "Non Specificato",
    ["VPN"] = "VPN",
    ["Video"] = "Video",
    ["VoIP"] = "VoIP",
    ["Web"] = "Web",
  },
  ["ndpi_page"] = {
    ["custom_applications"] = "Applicazioni Personalizzate",
    ["live_flows_count"] = "Conteggio dei Flussi in Tempo Reale",
    ["note"] = "NOTE",
    ["note_historical_per_protocol_traffic"] = "Lo storico dei dati %{what} di traffico può essere abilitato attraverso le <a href='%{url}'>%{flask_icon} Preferenze</a>.",
    ["note_live_flows_chart"] = "Questa tabella raffigura solo le connessioni TCP.",
    ["note_possible_probing_alert"] = "Un'icona come %{icon} indica un possibile allarme di probing (o applicazione server non attiva) poiché l'host ha ricevuto traffico per uno specifico protocollo applicativo senza inviare alcun dato. Puoi usare i <A HREF='%{url}'>report storici</A> per approfondire questo problema.",
    ["note_protocol_usage_time"] = "Il tempo d'utilizzo del protocollo è calcolato su intervalli discreti di slot.",
    ["num_contacted_services_as_client"] = "Servizi Contattati come Client",
    ["num_contacted_services_as_client_descr"] = "(nomi TLS, HTTP e DNS)",
    ["overview"] = "%{what} Panoramica",
    ["rcvd"] = "Rcvt",
    ["received_only"] = "Solo ricevuto",
    ["sent"] = "Invt",
    ["sent_only"] = "Solo inviato",
    ["total_since_startup"] = "Totale (Dall'avvio)",
    ["unable_to_find_host"] = "Impossibile trovare %{host_ip} (dato scaduto ?)",
  },
  ["nedge"] = {
    ["a_member_address_is"] = "Un indirizzo membro costituisce una regola per assegnare i dispositivi a un utente. Si applica l'ordine seguente:",
    ["a_zero_quota_is_unlimited"] = "Una quota pari a zero implica traffico illimitato.",
    ["active_user"] = "Utente Attivo",
    ["add_gateway"] = "Aggiungi un Gateway",
    ["add_member"] = "Aggiungi un Membro",
    ["add_port_forwarding_rule"] = "Aggiungi Regola di Redirezione Porta",
    ["add_routing_policy"] = "Aggiungi una Policy di Routing",
    ["add_some_gateways_first"] = "Aggiungi Lease Statica",
    ["add_static_route"] = "Aggiungi Route Statica",
    ["add_system_user"] = "Aggiungi Utente di Sistema",
    ["add_user_member"] = "Aggiungi Utente Membro",
    ["applied_policy"] = "Policy Applicata",
    ["apply_and_reboot"] = "Applica e Riavvia",
    ["apply_and_restart_self"] = "Applica e Riparti",
    ["apply_configuration"] = "Applica Configurazione",
    ["apply_configuration_and_reboot"] = "Stai procedendo con l'applicazione di una nuova configurazione. Le interfacce di rete saranno riconfigurate secondo con la configurazione specificata e il sistema verrà riavviato. Una configurazione sbagliata potrebbe causare la perdita di connettività di rete del sistema. Procedi soltanto se sei sicuro della correttezza dalla nuova configurazione.",
    ["apply_configuration_and_restart_self"] = "Stai procedendo con l'applicazione di una nuova configurazione. Questo causerà il riavvio di %{product}, l'interfaccia web potrebbe non rispondere per alcuni secondi.",
    ["apply_the_initial_device_configuration"] = "Applica la configurazione iniziale.",
    ["authentication_gui_and_captive_portal"] = "I metodi di autenticazione si applicano sia alla gui di %{product}, sia all'accesso tramite <a href=\"%{url}\">Captive Portal</a>.",
    ["bandwidth_allocation"] = "Allocazione di banda",
    ["bind_to_any"] = "Qualsiasi Interfaccia",
    ["bind_to_lan"] = "Interfaccia LAN",
    ["cannot_rename_gateway"] = "Non è consentito rinominare il gateway.",
    ["cannot_rename_policy"] = "Non è consentito rinominare la policy.",
    ["captive_portal_other_methods"] = "Il Captive Portal utilizza i metodi di autenticazione definiti nelle <a href=\"%{url}\">Preferenze di Autenticazione</a> (es. Radius).",
    ["categories_policies"] = "Policy di Categorie.",
    ["configure_routing_policies"] = "Configura Policy di Routing",
    ["configured_members"] = "Configura Membri",
    ["confirm_delete_gateway"] = "Confermi la cancellazione del gateway \"%{gw}\" ?",
    ["confirm_delete_policy"] = "Confermi la cancellazione della policy \"%{policy}\" ?",
    ["confirm_delete_port_forwarding_rule"] = "Confermi la cancellazione della regola selezionata?",
    ["confirm_delete_static_lease"] = "Confermi la cancellazione della lease statica per \"%{hostname}\"?",
    ["confirm_delete_static_route"] = "Confermi la cancellazione della route statica \"%{sr}\" ?",
    ["confirm_remove_member_from_user"] = "Confermi la cancellazione del membro %{member} dall'utente %{user}?",
    ["could_not_add_port_forwarding"] = "Impossibile aggiungere la regola di redirezione porta",
    ["custom"] = "Custom",
    ["custom_datetime"] = "Orario Corrente",
    ["custom_datetime_descr"] = "Imposta manualmente la data e l'ora del sistema.",
    ["custom_policy"] = "Policy Custom",
    ["data_reset"] = "Resetta Informazioni",
    ["data_reset_msg"] = "Vuoi resettare tutte le informazioni dei dispositivi? Le informazioni includono: serie temporali, flussi, allarmi, cache di Redis e contatori per gli host. Le configurazioni dei dispositivi e le impostazioni del sistema non verrano resettate.<br> Questa operazione causerà il riavvio di %{product} e potrebbe rendere l'interfaccia web non raggiungibile per alcuni secondi.",
    ["date_time"] = "Data e Ora",
    ["default_categories"] = "Policy di Default per Categoria",
    ["default_categories_with_a_defined_policy"] = "Policy di Default per Categoria Custom",
    ["default_categories_with_a_defined_quota"] = "Quota di Default di Policy per Categoria",
    ["default_categories_with_policy_x"] = "Policy di Default per la Categoria %{x}",
    ["default_gateway"] = "Gateway di Default",
    ["default_policy"] = "Policy di Default",
    ["default_protocols"] = "Policy di Default per Applicazione",
    ["default_protocols_with_a_defined_policy"] = "Policy di Default per Applicazione Custom",
    ["default_protocols_with_a_defined_quota"] = "Quota di Default di Policy per Applicazione",
    ["default_protocols_with_policy_x"] = "Policy di default per l'Applicazione %{x}",
    ["default_shaper"] = "Consentito",
    ["default_shaper_descr"] = "Valore di default per la percentuale di banda rispetto alla velocità dell'interfaccia",
    ["delete_gateway"] = "Elimina Gateway",
    ["delete_policy"] = "Elimina Policy",
    ["delete_port_forwarding_rule"] = "Elimina regola di redirezione porta",
    ["delete_static_lease"] = "Elimina Lease Statica",
    ["delete_static_route"] = "Elimina Route Statica",
    ["device_protocol_policy_has_higher_priority"] = "Le Policy per Dispositivo hanno priorità più alta rispetto alle Policy per Applicazione.",
    ["device_protocols_blocked_warning"] = "Il traffico <b>non</b> sarà bloccato perché le seguenti policy sono disabilitate: %{device_protocols_policies}",
    ["device_protocols_policies"] = "Policy per Dispositivo",
    ["dhcp_disabled_warning"] = "Se nessun altro server DHCP è disponibile sulla rete, i client non saranno in grado di connettersi automaticamente.",
    ["dhcp_first_ip"] = "Primo Indirizzo IP",
    ["dhcp_first_ip_descr"] = "Primo indirizzo IP del pool DHCP",
    ["dhcp_last_ip"] = "Ultimo indirizzo IP",
    ["dhcp_last_ip_descr"] = "Ultimo indirizzo IP del pool DHCP",
    ["dhcp_leases"] = "Lease DHCP",
    ["dhcp_server"] = "Server DHCP",
    ["dhcp_server_description"] = "Abilita server DHCP.",
    ["dhcp_server_not_enabled"] = "Il server DHCP non è abilitato. Attivalo <a href=\"%{url}\">qua</a>.",
    ["dns_configuration"] = "Configurazione DNS",
    ["dns_server_preset"] = "Preset del Server DNS",
    ["dns_server_preset_descr"] = "Seleziona un server DNS dalla lista, oppure specifica un server DNS custom.",
    ["download_description"] = "Velocità massima di download per l'interfaccia.",
    ["download_speed"] = "Velocità di download",
    ["edit_gateway"] = "Modifica Gateway",
    ["edit_quotas"] = "Modifica Quota",
    ["edit_routing_policy"] = "Modifica Policy di Routing",
    ["edit_static_route"] = "Modifica Route Statica",
    ["edit_user"] = "Modifica Utente",
    ["edit_users"] = "Modifica Utenti",
    ["enable_device_protocols_policies"] = "Policy per le Applicazioni del Device",
    ["enable_interface"] = "Abilita interfaccia",
    ["enable_interface_descr"] = "Abilita o disabilita l'interfaccia di rete.",
    ["enable_nat"] = "Abilita NAT",
    ["enable_nat_descr"] = "In caso di opzione abilitata, l'indirizzo IP del dispositivo verrà mascherato con l'indirizzo IP dell'interfaccia WAN.",
    ["enforce_global_dns"] = "Forza DNS globale",
    ["enforce_global_dns_description"] = "Abilita l'opzione per forzare i dispositivi ad utilizzare il DNS globale specificato.",
    ["enternal_port_overlap"] = "La porta esterna coincide con la porta %{port}, assignata all'indirizzo %{ip}",
    ["external_port"] = "Porta Esterna (WAN)",
    ["external_port_or_range"] = "Porta esterna o range di porte esterne",
    ["factory_reset"] = "Factory Reset",
    ["factory_reset_msg"] = "Confermi il reset del dispositivo alla sua configurazione di fabbrica? Tutte le configurazioni andranno perdute.",
    ["filter_policies"] = "Policy per i filtri",
    ["gateway"] = "Gateway",
    ["gateway_exists"] = "Il gateway %{gw} è già presente.",
    ["gateway_interface_descr"] = "Specifica l'interfaccia di rete da utilizzare per determinare l'indirizzo IP del gateway associato",
    ["gateway_name"] = "Nome Gateway",
    ["gateway_type"] = "Modalità",
    ["gateway_type_address_descr"] = "Specifica l'indirizzo IP del gateway.",
    ["gateway_x"] = "Gateway %{gw}",
    ["gateways"] = "Nodi Gateway",
    ["gateways_status"] = "Stato dei Gateway",
    ["global_dns"] = "DNS Globale",
    ["global_shapers_rates"] = "Controllo Globale della Banda",
    ["high_priority"] = "Alta Priorità",
    ["higher_priority"] = "Priorità più Alta",
    ["hostname"] = "Hostname",
    ["hostname_lease_exists"] = "Una lease statica per l'hostname \"%{hostname}\" è già presente.",
    ["interfaces_configuration"] = "Configurazione Interfacce",
    ["internal_ip"] = "IP Interno (LAN)",
    ["internal_port"] = "Porta Interna (LAN)",
    ["internal_port_number"] = "Numero porta interna",
    ["internal_port_overlap"] = "La porta interna coincide con la porta %{internal_port}, già mappata sulla porta esterna %{external_port}",
    ["invalid_dhcp_range"] = "Range DHCP non valido. Assicurarsi che gli indirizzi IP del pool DHCP sono contenuti nel range della LAN, altrimenti i dispositivi non saranno in grado di connettersi alla rete.",
    ["invalid_port_range"] = "Range di porte non valido",
    ["ip_address_takes_preced"] = "Un indirizzo IP ha la precedenza su una rete",
    ["ip_lease_exists"] = "Una lease statica con l'indirizzo \"%{ip}\" esiste già.",
    ["lan"] = "LAN",
    ["lan_ip_addr"] = "Indirizzo IP di LAN",
    ["lan_ip_addr_descr"] = "Specifica l'indirizzo IP dell'interfaccia LAN.",
    ["lan_recovery_ip"] = "IP ripristino LAN",
    ["lan_recovery_ip_descr"] = "Un indirizzo IP aggiuntivo per raggiungere il dispositivo %{product} sull'interfaccia LAN.",
    ["lan_recovery_warning"] = "Assicurati che questo non sia in conflitto con nessun'altra rete definita.",
    ["link_not_ready"] = "Link non pronto",
    ["linkdown"] = "Link inattivo",
    ["low_priority"] = "Bassa priorità",
    ["lower_priority"] = "Priorità più bassa",
    ["mac_address_takes_preced"] = "Un indirizzo mac ha la precedenza su un indirizzo IP",
    ["mac_lease_exists"] = "Una lease statica per l'indirizzo mac \"%{mac}\" esiste già.",
    ["max_bps_per_host"] = "Max %{bps} per host",
    ["max_perc"] = "Max %{perc}%%",
    ["max_rrt"] = "Max RTT",
    ["max_rrt_description"] = "Il massimo Round Trip Time verso l'indirizzo del monitor. Se l'RTT misurato supera questo valore, il gateway viene considerato inattivo.",
    ["maximum_enforced"] = "Valore massimo garantito",
    ["maximum_enforced_descr"] = "Se il valore massimo garantito viene impostato, verrà assegnata l'allocazione massima della larghezza di banda, indipendentemente dalla banda disponibile.",
    ["member_address"] = "Indirizzo membro",
    ["member_exists"] = "Membro %{member} non aggiunto, poiché è già assegnato all'utente %{existing_user}",
    ["members"] = "Membri",
    ["members_for_user_x"] = "Membri dell'utente %{user}",
    ["min_perc"] = "Min %{perc}%%",
    ["minimum_guaranteed"] = "Minimo garantito",
    ["minimum_guaranteed_descr"] = "Se impostato sul minimo garantito, sarà garantita solo l'allocazione minima della larghezza di banda.",
    ["mode"] = "Modalità",
    ["nedge"] = "nEdge",
    ["network_conf_dhcp"] = "DHCP",
    ["network_conf_iface_descr"] = "Specifica la modalità dell'indirizzo IP dell'interfaccia.",
    ["network_conf_iface_gw"] = "Indirizzo IP del gateway predefinito dell'interfaccia (ad esempio 192.168.1.1).",
    ["network_conf_iface_ip"] = "Indirizzo IP dell'interfaccia (ad es. 192.168.1.10).",
    ["network_conf_iface_nmask"] = "Maschera di rete dell'interfaccia (ad es. 255.255.255.0).",
    ["network_conf_iface_title"] = "Interfaccia %{ifrole} %{ifname}",
    ["network_conf_static"] = "Statico",
    ["network_conf_vlan_trunk"] = "Trunk VLAN",
    ["no_gateways_defined"] = "Nessun gateway definito.",
    ["no_interfaces_available"] = "Nessuna interfaccia disponibile.",
    ["no_routing_policies_defined"] = "Nessuna policy di routing definita",
    ["no_rules"] = "Nessuna regola",
    ["no_static_leases"] = "Nessuna lease statica",
    ["no_static_routes_defined"] = "Nessuna route statica definita.",
    ["ntp_sync"] = "Sincronizzazione NTP",
    ["ntp_sync_descr"] = "Attiva o disattiva la sincronizzazione con un server NTP.",
    ["per_ip_rates"] = "Controllo larghezza di banda per host",
    ["ping_address"] = "Monitor Address",
    ["ping_address_description"] = "L'indirizzo da monitorare per verificare lo stato del gateway.",
    ["please_fill_out_this_field"] = "Compila questo campo.",
    ["policy_due_to_source"] = "%{policy} a causa di %{source}",
    ["policy_name"] = "Nome policy",
    ["port_forwarding"] = "Port Forwarding",
    ["power_off"] = "Spegnimento",
    ["power_off_confirm"] = "Vuoi spegnere il dispositivo?",
    ["priority_x"] = "Priorità %{x}",
    ["product_status"] = "Stato %{product}",
    ["protocol_policy_has_higher_priority"] = "Le policy per applicazione hanno una priorità più alta rispetto alle policy per categoria.",
    ["protocols_policies"] = "Policy applicative",
    ["reboot"] = "Riavvia",
    ["reboot_corfirm"] = "Vuoi riavviare il dispositivo?",
    ["remove_all_members"] = "Rimuovi tutti i membri",
    ["remove_all_members_confirm"] = "Confermi la rimozione di tutti i membri dell'utente %{user}?",
    ["reset_and_reboot"] = "Resetta e riavvia",
    ["reset_and_restart_self"] = "Resetta e fai ripartire",
    ["restrict_weg_gui_access_description"] = "Limita l'accesso a questa interfaccia web di gestione, per renderla accessibile solo dall'interfaccia LAN (metodo più sicuro) o da qualsiasi interfaccia del sistema (metodo meno sicuro).",
    ["restrict_weg_gui_access_title"] = "Accesso all'interfaccia web di gestione",
    ["router"] = "Router",
    ["router_mode_requires_enterprise"] = "La modalità router è disponibile solo nell'edizione Enterprise di nedge.",
    ["routing"] = "Routing",
    ["routing_policies"] = "Le policy di routing",
    ["routing_policy"] = "Policy di routing",
    ["routing_policy_exists"] = "La policy di routing \"%{policy}\" esiste già.",
    ["safe_search_description"] = "Usa il \"Child Safe\" DNS per bloccare contenuti inappropriati.",
    ["search_categories"] = "Cerca categorie",
    ["search_protocols"] = "Cerca applicazioni",
    ["security"] = "Sicurezza",
    ["set_the_device_mode"] = "Imposta la modalità operativa del dispositivo.",
    ["setup_apply"] = "Applica",
    ["setup_config_edited_descr"] = "Applica le modifiche per rendere effettiva la nuova configurazione, o annulla per continuare a utilizzare quella corrente.",
    ["setup_config_edited_title"] = "Configurazione del sistema modificata!",
    ["setup_discard"] = "Annulla",
    ["setup_mode"] = "Modalità operativa",
    ["setup_network"] = "Rete",
    ["shapers"] = "Controllo della larghezza di banda",
    ["single_port_router"] = "Router a porta singola",
    ["slow_pass_shaper_descr"] = "La percentuale di larghezza di banda Slow Pass rispetto alla velocità dell'interfaccia.",
    ["slower_pass_shaper_descr"] = "La percentuale di larghezza di banda Slower Pass rispetto alla velocità dell'interfaccia.",
    ["specify_a_protocol"] = "Specifica un'applicazione",
    ["static_dhcp_leases"] = "Lease statiche del server DHCP",
    ["static_route_address_descr"] = "Specifica la destinazione per la route statica.",
    ["static_route_exists"] = "La route statica %{sr} esiste già.",
    ["static_route_is_local_descr"] = "Specifica se si tratta di una route statica verso una rete locale. Tutti gli host in una route statica verso una rete locale vengono trattati come host locali.",
    ["static_route_is_local_title"] = "Route statica della rete locale",
    ["static_route_name"] = "Nome Route statica",
    ["static_route_netmask_descr"] = "Specifica la netmask da applicare alla destinazione per la route statica.",
    ["static_route_via_descr"] = "Specifica l'indirizzo IP da utilizzare come hop successivo per la route statica.",
    ["static_route_x"] = "Route statica %{sr}",
    ["static_routes"] = "Route statiche",
    ["static_routes_in_policies"] = "Le route statiche vengono applicate a (e hanno la precedenza su) una qualsiasi delle <a href=\"%{url}\">Policy di routing</a> definite.",
    ["status_activate"] = "Attiva",
    ["status_connected"] = "Connetti",
    ["status_connecting"] = "Collegamento in corso",
    ["status_deactivate"] = "Disattiva",
    ["status_disabled"] = "Disabilitato",
    ["status_disconnected"] = "Disconnesso",
    ["status_down"] = "Giù",
    ["status_failed"] = "Fallito",
    ["status_inactive"] = "Inattivo",
    ["status_unreachable"] = "Non raggiungibile",
    ["status_up"] = "Su",
    ["system_setup"] = "Configurazione del sistema",
    ["system_users"] = "Utenti del sistema",
    ["test_policies"] = "Policy di test",
    ["test_policies_page_description"] = "Questa pagina può essere utilizzata per verificare quali saranno le policy applicate ai dispositivi dell'utente %{user}, in base alla configurazione corrente.",
    ["the_fallback_policy"] = "La <a href=\"%{url}\">Policy di backup</a> viene applicata quando sia l'utente che la policy globale per una data applicazione e la sua categoria corrispondente sono impostate su \"Predefinito\".",
    ["the_not_assigned_user_contains"] = "L'utente \"Non assegnato\" può essere utilizzato per definire policy per dispositivi che non sono assegnati a nessun utente.",
    ["the_primary_dns_server"] = "Server DNS primario",
    ["the_secondary_dns_server"] = "Server DNS secondario",
    ["timezone"] = "Fuso orario",
    ["timezone_descr"] = "Imposta il fuso orario del sistema.",
    ["upload_description"] = "La velocità massima di upload per l'interfaccia.",
    ["upload_speed"] = "Velocità di upload",
    ["user"] = "Utente",
    ["user_categories"] = "Policy di Categoria per l'Utente %{user}",
    ["user_categories_with_a_defined_policy"] = "Policy di Categoria Custom per l'Utente %{user}",
    ["user_categories_with_a_defined_quota"] = "Policy della Categoria Quota per l'Utente %{user}",
    ["user_categories_with_policy_x"] = "%{x} Policy di Categoria per l'Utente %{user}",
    ["user_category_policies"] = "Policy per la Categoria Utente",
    ["user_device_protocols"] = "Policy di Applicazione per Dispositivi %{user}",
    ["user_protocols"] = "Policy di Applicazione per l'Utente %{user}",
    ["user_protocols_with_a_defined_policy"] = "Policy di Applicazione Custom per l'Utente %{user}",
    ["user_protocols_with_a_defined_quota"] = "Policy di Applicazione Quota per l'Utente %{user}",
    ["user_protocols_with_policy_x"] = "%{x} Policy di Applicazione per l'Utente %{user}",
    ["users_list"] = "Elenco utenti",
    ["view_quotas"] = "Visualizza Quote",
    ["wan"] = "WAN",
    ["wan_interface"] = "Interfaccia WAN",
    ["with_quota"] = "Con Quota",
    ["x_shaper_per_host_descr"] = "L'allocazione della larghezza di banda massima per host di %{shaper}. 0 per illimitato.",
    ["you_can_assign_routing_policy"] = "Puoi assegnare policy di routing agli <a href=\"%{url}\">Utenti</a>.",
    ["you_can_assign_shapers_to_users_protocols"] = "Puoi assegnare criteri di larghezza di banda agli <a href=\"%{url}\">Utenti</a>, o impostarli globalmente come <a href = \"%{url2}\">Policy di backup</a>.",
    ["you_can_set_static_dhcp_lease_here"] = "Puoi configurare le lease DHCP statiche <a href=\"%{url}\">qui</a>.",
    ["you_can_use_gateways_in_policies"] = "Puoi specificare le priorità dei gateway nelle <a href=\"%{url}\">Policy di routing</a>.",
    ["policy_source_labels"] = {
      ["policy_source_category"] = "Policy della Categoria",
      ["policy_source_default"] = "Sconosciuto",
      ["policy_source_device_protocol"] = "Policy di Applicazione del Dispositivo",
      ["policy_source_pool"] = "Policy di default",
      ["policy_source_protocol"] = "Policy dell'Applicazione",
    },
  },
  ["network_alert_config"] = {
    ["trigger_alerts_for_network"] = "Generata allarmi per la Rete %{network}",
    ["trigger_network_alerts"] = "Abilita Allarmi sulle Reti",
  },
  ["network_details"] = {
    ["network"] = "Rete",
    ["network_alias"] = "Alias della Rete",
    ["network_parameter_missing_message"] = "Parametro di rete mancante (errore interno ?)",
    ["no_available_stats_for_network"] = "Nessuna statistica disponibile per la rete %{network}",
  },
  ["network_stats"] = {
    ["network_list"] = "Lista Reti",
    ["network_name"] = "Nome Rete",
    ["networks"] = "Reti",
    ["networks_traffic_with_ipver"] = "%{networks} con traffico IPv%{ipver}",
    ["note_aliases_not_configurable"] = "L'alias non è modificabille come specificato staticamente con l'opzione -m.",
    ["note_broader_network"] = "Le reti più ampie non includeranno host definiti in reti più piccole.",
    ["note_overlapping_networks"] = "NOTA: In caso di overlapping tra reti:",
    ["note_see_both_network_entries"] = "Puoi vedere entrambe le entrate di rete nella tabella sopra.",
  },
  ["os_details"] = {
    ["no_available_stats_for_os"] = "Nessuna statistica disponibile per l'OS %{os}",
    ["os"] = "Sistema Operativo",
    ["os_parameter_missing_message"] = "Parametro OS mancante (internal error?)",
  },
  ["os_stats"] = {
    ["hosts_by_operating_system"] = "Host per Sistema Operativo",
  },
  ["packets_page"] = {
    ["arp_distribution"] = "Distribuzione ARP",
    ["ip_version_distribution"] = "Distribuzione Versione IP",
    ["lost"] = "Persi",
    ["out_of_order"] = "Fuori Ordine",
    ["received_distribution"] = "Distribuzione Ricezione",
    ["retransmissions"] = "Ritransmissioni",
    ["sent_distribution"] = "Distribuzione Invio",
    ["sent_vs_rcvd_distribution"] = "Distribuzione Inviati e Ricevuti",
    ["size_distribution"] = "Distribuzione Dimensione",
    ["tcp_flags_distribution"] = "Distribuzione Flags TCP",
    ["tcp_flags_vs_arp_distribution"] = "Distribuzione TCP Flags / ARP",
    ["tcp_packets_analysis"] = "Analisi Pacchetti TCP",
    ["version_vs_flags_distribution"] = "Distribuzione Versione IP e TCP Flags",
  },
  ["peers_page"] = {
    ["host"] = "Host",
    ["no_active_flows_message"] = "Non sono stati osservati flussi attivi per l'host in questione",
    ["top_peer_protocol"] = "Top Protocollo del Peer",
    ["top_peers_for_host"] = "Top %{hostkey} Peer",
    ["traffic_volume"] = "Volume di Traffico",
  },
  ["periodic_activities_descr"] = {
    ["5min"] = {
      ["lua"] = "Itera gli host attivi e scrive le loro serie temporali. Esegue gli user script ai 5 minuti.",
    },
    ["daily"] = {
      ["lua"] = "Effettua operazioni di pulizia dei dati vecchi. Esegue gli script giornalieri.",
    },
    ["dequeue_flows_for_hooks"] = {
      ["lua"] = "Gestisce il ciclo di vita dei flussi, degli host e di altre hash entry. Esegue gli user script. Dumpa i flussi sul database.",
    },
    ["discover"] = {
      ["lua"] = "Esegue network discovery on-demand e periodiche.",
    },
    ["hourly"] = {
      ["lua"] = "Esegue gli user script orari",
    },
    ["housekeeping"] = {
      ["lua"] = "Processa gli allarmi generati, li inserisce nel database e li propaga verso endpoint esterni. Controlla l'estrazione con n2disk. Controlla e ricarica le liste.",
    },
    ["minute"] = {
      ["lua"] = "Scrive dettagliate serie temporali per l'interfaccia quali ad esempio le applicazioni. Scrive le serie temporali per gli user script, per i pod e per i container. Esegue gli user script al minuto.",
    },
    ["notifications"] = {
      ["lua"] = "Processa gli allarmi, consegnandoli ai recipient configurati.",
    },
    ["periodic_checks"] = {
      ["lua"] = "Esegue gli script periodici.",
    },
    ["second"] = {
      ["lua"] = "Scrive fondamentali serie temporali per l'interfaccia quali il suo traffico totale.",
    },
    ["stats_update"] = {
      ["lua"] = "Aggiorna le statistiche degli host quali il throughput e il traffico totale.",
    },
    ["timeseries"] = {
      ["lua"] = "Scrive le serie temporali su InfluxDB.",
    },
  },
  ["please_wait_page"] = {
    ["command"] = "Comando",
    ["database"] = "Database",
    ["host"] = "Host",
    ["id"] = "Id",
    ["info"] = "Info",
    ["operations_on_database_msg"] = "Le operazioni attualmente eseguite sul database sono lpe seguenti:",
    ["state"] = "Stato",
    ["time"] = "Tempo",
    ["user"] = "Utente",
    ["waiting_for_db_msg"] = "Migrazione dello schema del database in corso, in attesa che il database <b>%{dbname}</b> diventi operativo. Verrai reindirizzato non appena il database sarà pronto. Per database di dimensione importante, questa operazione potrebbe richiedere molto tempo. Le migrazioni vengono eseguite una sola volta e solo durante i principali aggiornamenti della versione. In caso di dubbi, contattare gli sviluppatori a <a href='mailto:info@ntop.org'>info@ntop.org</a>.",
  },
  ["plugins_overview"] = {
    ["action_view"] = "Apri il codice dello user script nell'editor di codice",
    ["alert_key"] = "Chiave dell'Allarme",
    ["alert_key_string"] = "Stringa chiave di avviso",
    ["alert_name"] = "Nome Allarme",
    ["edition_only"] = "Solo %{edition}",
    ["filtered"] = "Filtrato",
    ["filters"] = "Filtri",
    ["has_attacker"] = "Attaccante Conosciuto",
    ["has_victim"] = "Vittima Conosciuta",
    ["hooks"] = "Hooks",
    ["loaded_plugins"] = "Plugin Caricati",
    ["plugin"] = "Plugin",
    ["reload_plugins"] = "Ricarica Plugin",
    ["script"] = "Script",
    ["source_location"] = "Posizione del Sorgente",
    ["status_key"] = "Chiave dello Stato",
    ["type"] = "Tipo",
  },
  ["policy"] = {
    ["abort_add_rule"] = "Annullare aggiunta regola?",
    ["add_rule"] = "Aggiunta regola",
    ["autoconfigure"] = "Auto Configurazione",
    ["autoconfigure_note"] = "La funzionalità di configurazione automatica crea automaticamente una configurazione suggerita per il pool rilevando protocolli/host comunemente utilizzati/contattati dagli host nel pool. Tieni presente che la politica predefinita è impostata su Drop quando si utilizza questa funzionalità (a meno che non sia già stata impostata una politica predefinita).",
    ["autoconfigure_sites_note"] = "Per popolare le regole dell'host, la funzione di configurazione automatica richiede che l'euristica dei siti più visitati sia abilitata in Impostazioni > Preferenze > Applicazioni. Per ottenere i migliori risultati, dovrebbe essere abilitato almeno un giorno prima per raccogliere dati sufficienti.",
    ["continent"] = "Continente",
    ["continent_rules"] = "Regole per Continente",
    ["continent_rules_descr"] = "Configurare qui le regole di eccezioni per continente.",
    ["country"] = "Paese",
    ["country_rules"] = "Regole per Paese",
    ["country_rules_descr"] = "Configurare qui la regole di eccezioni per paese.",
    ["default"] = "Default",
    ["default_policy"] = "Politica Default",
    ["default_policy_descr"] = "La politica predefinita definisce l'azione predefinita per il traffico corrispondente al pool. Eccezioni alla policy di default (cioè utilizzando la policy opposta rispetto a quella di default) possono essere configurate aggiungendo le regole di seguito.",
    ["drop"] = "Rimuovi",
    ["edit_rule"] = "Modifica Regole",
    ["host"] = "Host",
    ["host_rules"] = "Regole per Host",
    ["host_rules_descr"] = "Configurare qui le regole di eccezione nomehost (e.g. usato in protocolli TLS, DNS...).",
    ["l7_category"] = "Categoria L7",
    ["l7_category_rules"] = "Regole per Categoria L7",
    ["l7_category_rules_descr"] = "Configura qui le regole di eccezione della categoria di traffico.",
    ["l7_proto"] = "Protocollo L7",
    ["l7_proto_rules"] = "Regole Protocollo L7",
    ["l7_proto_rules_descr"] = "Configurare qui le eccezioni di regole di protocollo L7/application.",
    ["marker"] = "Azione",
    ["pass"] = "Passa",
    ["remove_rules"] = "Rimuovere Regole",
    ["risk"] = "Rischio",
    ["risk_rules"] = "Regole di Rischio",
    ["risk_rules_descr"] = "Configura qui le politiche basate sul rischio del flusso calcolato da nDPI.",
    ["rule"] = "Regola",
    ["rules"] = "Regole di Politica",
    ["traffic_policy_for"] = "Politiche di traffico per <a href='%{url}'>%{pool} <i class='fas fa-users'></i></a>",
    ["markers"] = {
      ["drop"] = "Rimuovi",
      ["pass"] = "Passa",
      ["slow"] = "Slow Pass",
      ["slower"] = "Slower Pass",
    },
  },
  ["policy_presets"] = {
    ["business"] = "Business",
    ["business_description"] = "Un preset per i dipendenti delle aziende",
    ["child"] = "Bambino",
    ["child_description"] = "Un preset per i bambini. Blocca i social network e limita il tempo di gioco",
    ["default"] = "Default",
    ["guests"] = "Ospiti",
    ["guests_description"] = "Un preset per gli ospiti. Consente solo la navigazione base",
    ["no_obfuscation"] = "Nessun Oscuramento",
    ["no_obfuscation_description"] = "Un preset per prevenire l'oscuramento del traffico",
    ["walled_garden"] = "Walled Garden",
    ["walled_garden_description"] = "Blocca le VPN ed i Social Network",
  },
  ["pool_details"] = {
    ["host_pool"] = "Host Pool",
    ["host_pool_timeseries_enable_message"] = "La serie temporali dell'Host Pool possono essere abilitata dalle <A HREF=\"%{url}\">%{icon_flask} Impostazioni</A>. Sono necessari alcuni minuti per vedere i primi punti di dati.",
    ["no_available_data_for_host_pool_message"] = "Nessun dato disponibile per l'host pool '%{pool_name}'.",
    ["pool_parameter_missing_message"] = "Parametro mancante (errore interno ?)",
  },
  ["pool_stats"] = {
    ["host_pool_list"] = "Lista Host Pool",
  },
  ["pools"] = {
    ["config_import"] = "Importa Configurazione",
    ["config_import_message"] = "<b>Nota</b>: i pool con lo stesso nome saranno sostituiti.",
    ["configuration"] = "Configurazione Script Utente",
    ["edit_host_pool"] = "Modifica Host Pool",
    ["edit_pools"] = "Modifica Pool",
    ["edit_recipients"] = "Modifica recipients",
    ["family"] = "Famiglia",
    ["id"] = "ID",
    ["insert_name"] = "i.e. MyPool",
    ["members"] = "Membri",
    ["name"] = "Nome",
    ["name_requirements"] = "Il nome deve essere lungo almeno 1 carattere e non deve essere vuoto.",
    ["no_recipients"] = "Nessun recipient configurato per il pool.",
    ["not_assigned"] = "Non Assegnato",
    ["pool"] = "Pool",
    ["pools"] = "Pool",
    ["recipients"] = "Recipient",
    ["remove_pool"] = "Eliminare il pool '%{pool}'?",
    ["select_config"] = "Seleziona una Configurazione Script Utente",
    ["some_recipients"] = "${recipients} configurati per il pool.",
    ["unbounded_members"] = "Tutti i non associati",
    ["pool_name"] = {
      ["active_monitoring"] = "Monitoraggio Attivo",
      ["flow"] = "Flusso",
      ["host"] = "Host",
      ["host_pool"] = "Host Pool",
      ["interface"] = "Interfaccia",
      ["local_network"] = "Rete Locale",
      ["mac"] = "Dispositivo",
      ["snmp_device"] = "Dispositivo SNMP",
      ["system"] = "Sistema",
    },
    ["pool_names"] = {
      ["active_monitoring"] = "Monitoraggio Attivo",
      ["all"] = "Tutti",
      ["flow"] = "Flussi",
      ["host"] = "Host",
      ["host_pool"] = "Host Pool",
      ["interface"] = "Interfacce",
      ["local_network"] = "Reti Locali",
      ["mac"] = "Dispositivi",
      ["snmp_device"] = "Dispositivi SNMP",
      ["system"] = "Sistema",
    },
    ["pool_names_suffix"] = {
      ["active_monitoring"] = "Pool Monitoraggio Attivo",
      ["interface"] = "Pool Interface",
      ["local_network"] = "Pool Reti Locali",
      ["snmp"] = "Pool SNMP",
    },
  },
  ["port_details"] = {
    ["active_flows_for_host_and_port"] = "Flussi Attivi per %{host}:%{port}",
    ["active_flows_on_port"] = "Flussi Attivi sulla porta %{port}",
    ["active_flows_on_port_symbolic"] = "Flussi attivi sulla porta %{port} [%{symbolic_port}]",
    ["l4_port"] = "Porta L4",
  },
  ["ports_page"] = {
    ["client_ports"] = "Porte Client",
    ["num_contacted_ports"] = "Numero Porte Contattate",
    ["num_contacted_ports_as_client"] = "Porte Remote Contattate (come Client)",
    ["num_host_contacted_ports_as_server"] = "Porte Locali Contattate (come Server)",
    ["server_ports"] = "Porte Server",
  },
  ["prefs"] = {
    ["active_local_host_cache_interval_description"] = "L'intervallo tra salvataggi consecutivi per gli host locali attivi. Default: 1 ora.",
    ["active_local_host_cache_interval_title"] = "Intervallo di Cache sugli Host Locali",
    ["admin_group_description"] = "Nome del gruppo al quale un utente deve appartenere per autenticarsi come amministratore.",
    ["admin_group_title"] = "Gruppo Amministratore LDAP",
    ["alert_entity"] = "Entità Allarme",
    ["alerts_notifications"] = "Notifiche Allarmi",
    ["alerts_page_refresh_rate__title"] = "Allarme Frequenza di Aggiornamento",
    ["alerts_page_refresh_rate_description"] = "Imposta il timer della frequenza di aggiornamento (> 0, per essere effettivo).",
    ["alerts_retention"] = "Conservazione Allarmi",
    ["alerts_settings"] = "Impostazioni Avvisi",
    ["all"] = "Tutti",
    ["auth_methods_order"] = "L'ordine dei metodi di autenticazione è lo stesso della lista sopra. Ad es. se sia LDAP che l'autenticazione locale sono abilitate, sarà controllato prima LDAP e, se l'autenticazione fallisce, successivamente sarà controllata l'autenticazione locale.",
    ["authentication"] = "Autenticazione",
    ["authentication_duration"] = "Durata dell'Autenticazione",
    ["authentication_duration_descr"] = "Tempo dopo il quale la sessione di un utente autenticato viene terminata ed una nuova autenticazione viene richiesta. I cambiamenti vengono applicati solo alle nuove sessioni.",
    ["authentication_duration_title"] = "Durata delle Sessioni Autenticate",
    ["authentication_midnight_expiration_descr"] = "Attiva/disattiva la terminazione forzata a mezzanotte di tutte le sessioni autenticate attive.",
    ["authentication_midnight_expiration_title"] = "Terminazione delle Sessioni Autenticate a Mezzanotte",
    ["behavior_timeseries"] = "Comportamento Serie Temporali",
    ["behaviour"] = "Comportamento Traffico",
    ["behaviour_analysis_learning_period_description"] = "Configura il periodo di apprendimento per l'analisi comportamentale del traffico.",
    ["behaviour_analysis_learning_period_title"] = "Periodo di apprendimento",
    ["behaviour_analysis_status_during_learning_description"] = "Lo stato predefinito di un nuovo servizio rilevato durante l'apprendimento della mappa dei servizi.",
    ["behaviour_analysis_status_during_learning_title"] = "Stato del Servizio Durante il Periodo di Apprendimento",
    ["behaviour_analysis_status_post_learning_description"] = "Lo stato predefinito di un nuovo servizio rilevato quando la mappa dei servizi ha terminato l'apprendimento.",
    ["behaviour_analysis_status_post_learning_title"] = "Stato del servizio dopo l'apprendimento",
    ["bind_dn_description"] = "Bind Distinguished Name of LDAP server. Example:",
    ["bind_dn_title"] = "Bind DN di LDAP",
    ["bind_pwd_description"] = "Bind password usata per autenticarsi con il server LDAP.",
    ["bind_pwd_title"] = "Bind Password per l'autenticazione LDAP",
    ["both"] = "Entrambi",
    ["cache_settings"] = "Impostazioni Cache",
    ["captive_portal_url_description"] = "L'URL a cui verranno reindirizzati i client dopo un'autenticazione al captive portal avvenuta con successo.",
    ["captive_portal_url_title"] = "URL di reindirizzamento",
    ["client_x509_auth"] = "Autenticazione HTTPS del Client",
    ["client_x509_auth_descr"] = "Abilita l'autenticazione dei client utilizzando il loro certificato X.509. Quando il Common Name (CN) nel certificato del client corrisponde al nome di uno degli utenti di ntopng, il client viene autenticato con quello stesso utente. Cambiamenti a questa preferenza richiedono un riavvio di ntopng.",
    ["client_x509_auth_title"] = "Autenticazione Client con Certificato X.509",
    ["could_not_contact_influxdb"] = "Impossibile contattere il database InfluxDB: %{msg}",
    ["data_retention"] = "Conservazione Dati",
    ["data_retention_descr"] = "Numero di giorni per i quali conservare i flussi dumpati, le serie temprali e i top talker. Default: 30 giorni.",
    ["default_snmp_community_description"] = "La comunità SNMP predefinita viene utilizzata quando si prova a percorrere il MIB SNMP di un dato host locale che non è stato configurato tramite la pagina dei dispositivi SNMP.",
    ["default_snmp_community_title"] = "Comunità SNMP di Default",
    ["default_snmp_proto_version_description"] = "La versione predefinita del protocollo SNMP.",
    ["default_snmp_proto_version_title"] = "Versione SNMP di Default",
    ["default_snmp_timeout_description"] = "Tiemout Risposta del dispositivo SNMP (in secondi).",
    ["default_snmp_timeout_title"] = "Timeout Risposta",
    ["device_identifier"] = "Identificatore Dispositivo",
    ["device_identifier_descr"] = "Normalmente i dispositivi vengono identificati dall'indirizzo MAC. Invece, se i dispositivi sono dietro a un router, l'identificazione basata su IP è consigliabile.",
    ["disable_alerts_generation_description"] = "Abilita la generazione di allarmi.",
    ["disable_alerts_generation_title"] = "Abilita Allarmi",
    ["dns"] = "DNS",
    ["dns_service"] = "Servizio",
    ["dump_settings"] = "Impostazioni Dump",
    ["duration"] = "Durata",
    ["dynamic_interfaces_creation_description"] = "ntopng può usare un criterio per diseggregare il traffico in entrata.Quando viene selezionato un criterio di disaggregazione, ntopng utilizzerà il valore del criterio per dividere il traffico in entrata su interfacce create dinamicamente.Per esempio, quando viene selezionato il criterio VLAN Id, verrà creata un'interfaccia dinamica per ogni VLAN Id osservato, ed il traffico in entrata verrà deviato a un'interfaccia dinamica in base al valore del VLAN Id. ",
    ["dynamic_interfaces_creation_note_0"] = "La disaggregazione VLAN Id è supportata sia per le interfacce fisiche che per i flussi ricevuti su ZMQ. Gli altri criteri di disaggregazione sono supportati solo per i flussi ZMQ e saranno inefficaci per le interfacce fisiche.",
    ["dynamic_interfaces_creation_note_1"] = "I cambi di criterio non interesseranno le interfacce esistenti.",
    ["dynamic_interfaces_creation_note_2"] = "Quando si utilizza il criterio sull'Interfaccia d'Ingresso con dispositivi non a flusso, %%INPUT_SNMP deve apparire nel template di nprobe.",
    ["dynamic_interfaces_creation_note_3"] = "Il criterio di disaggregazione 'Interfaccia' aggiunge un flusso a due interfacce dinamiche. Il valore del campo %%INPUT_SNMP è usato per selezionare la prima interfaccia dinamica, mentre il valore di %%OUTPUT_SNMP è usato per selezionare la seconda.",
    ["dynamic_interfaces_creation_note_4"] = "La disaggregazione custom mediante filtri simil-BPF è disponibile anche disabilitando la Disaggregazione Dinamica del Traffico mediante il tab Disaggregazione Custom Del Traffico.",
    ["dynamic_interfaces_creation_title"] = "Criterio Di Disaggregazione",
    ["dynamic_network_interfaces"] = "Interfacce di rete dinamiche",
    ["elasticsearch_notification"] = "Elasticsearch",
    ["elasticsearch_send_error"] = "Impossibile connettersi a Elasticsearch. [Codice: %{code}][Risposta: %{resp}].",
    ["elasticsearch_sent_successfully"] = "La connessione a Elasticsearch funziona.",
    ["elasticsearch_unable_to_send_alerts"] = "Impossibile inoltrare allarmi a Elasticsearch",
    ["elasticsearch_unsupported_version"] = "Versione %{version} di Elasticsearch non supportata. Need at least version 7.",
    ["elephant_flow_local_to_remote_bytes_description"] = "La quantità di dati che un flusso può inviare da un host locale a un host remoto per essere considerato Flusso Elephant.",
    ["elephant_flow_local_to_remote_bytes_title"] = "Soglia Flussi Voluminosi (da Locale a Remoto)",
    ["elephant_flow_remote_to_local_bytes_description"] = "La quantità di dati che un flusso può trasferire da un host remoto a un host locale per essere considerato Flusso Elephant.",
    ["elephant_flow_remote_to_local_bytes_title"] = "Soglia Flussi Voluminosi (da Remoto a Locale)",
    ["email_notification"] = "Notifiche Email",
    ["email_notification_password_description"] = "La password per l'autenticazione al server SMTP (opzionale).",
    ["email_notification_password_title"] = "Password",
    ["email_notification_recipient_description"] = "L'indirizzo destinatario dell'email.",
    ["email_notification_recipient_title"] = "Destinatario",
    ["email_notification_sender_description"] = "L'indirizzo mittente dell'email.",
    ["email_notification_sender_title"] = "Mittente",
    ["email_notification_server_description"] = "Il server SMTP a cui consegnare l'email. Es. smtp://example.com:25 .",
    ["email_notification_server_title"] = "Server SMTP",
    ["email_notification_username_description"] = "Lo username per l'autenticazione al server SMTP (opzionale).",
    ["email_notification_username_title"] = "Username",
    ["email_send_error"] = "Errore nell'invio dell'email. Controllare <a href='%{url}'>la documentazione online</a> per ulteriori dettagli.",
    ["email_sent_successfully"] = "Email inviata con successo",
    ["enable_alerts_refresh_description"] = "Abilita per aggiornare periodicamente gli avvisi quando un utente sta guardando la pagina degli avvisi.",
    ["enable_alerts_refresh_title"] = "Refresh Allarmi",
    ["errors"] = "Errori",
    ["errors_and_warnings"] = "Errori e avvertenze",
    ["ewma_alpha_percent_description"] = "Coefficiente alpha per la media mobile esponenziale (EWMA), espresso come percentuale. Utilizzato per stimare il round-trip time degli Autonomous System.",
    ["ewma_alpha_percent_title"] = "Coefficiente (percentuale) Alpha EWMA",
    ["expert_view"] = "Vista Esperta",
    ["featured_dns_description"] = "Di seguito è riportato un elenco di server DNS sicuri. Puoi usarli come server DNS globali o come fornitori di ricerca Child Safe.",
    ["featured_dns_title"] = "Server DNS Preferiti",
    ["flow_interface"] = "Interfaccia",
    ["flow_max_idle_description"] = "Tempo d'inattività dopo il quale un flusso viene considerato inattivo. Impostazione valida solo per le interfacce a pacchetto. Per le interfacce ZMQ questo valore viene determinato automaticamente. Default: 1 min.",
    ["flow_max_idle_title"] = "Timeout d'inattività del Flusso",
    ["flow_table"] = "Tabella Flusso",
    ["flow_table_begin_epoch_description"] = "Aggiungi/Rimuovi la colonna Inizio dalla tabella di flusso. Rappresenta la prima vista del flusso e può essere ordinata.",
    ["flow_table_begin_epoch_title"] = "Colonna di inizio della tabella di flusso",
    ["flow_table_probe_order_description"] = "Ordina il filtro sonda per IP o Nome (Utile solo se è disponibile almeno una sonda)",
    ["flow_table_probe_order_title"] = "Ordine della sonda della tabella di flusso",
    ["flow_table_time_description"] = "Tempo del flusso visualizzato nel grafico della tabella di flusso. Scegli tra la durata del flusso e l'ultima volta che hai visto il flusso",
    ["flow_table_time_title"] = "Tempo della tabella di flusso",
    ["flows_dump"] = "Dump Flussi",
    ["flows_dump_limited_days"] = "Il periodo di dump è limitato a %{days} giorni in questa versione di ntopng. Aggiornare alla versione enterprise per rimuovere questo limite.",
    ["global_dns_description"] = "Un server DNS per reindirizzare le richieste DNS dei client per cui non è abilitato il Safe Search. Se vuoto, il reindirizzamento è disabilitato e viene utilizzato il DNS predefinito.",
    ["global_dns_title"] = "DNS Globale Primario",
    ["gui"] = "Interfaccia Utente",
    ["host_mask"] = "Nacondi indirizzi IP host",
    ["http_auth"] = "Autenticazione HTTP",
    ["http_auth_server_description"] = "L'URL del server HTTP da usare per autenticare gli utenti.",
    ["http_auth_server_title"] = "Server HTTP",
    ["idle_timeout_settings"] = "Impostazioni Timeout d'inattività",
    ["iec60870_learning_period_description"] = "Configurare il periodo di apprendimento per l'analisi del traffico IEC60870. Predefinito: 6 ore.",
    ["iec60870_learning_period_title"] = "Periodo di Apprendimento IEC60870",
    ["ignored_interfaces_description"] = "Quando si seleziona 'Interfaccia d'Ingresso' come criterio di disaggregazione, non verrà aggiunto all'interfaccia dinamica alcun flusso avente %%INPUT_SNMP in questa lista. Quando si seleziona 'Interfaccia' come criterio di disaggregazione, non verrà aggiunto alla prima (seconda) interfaccia dinamica alcun flusso avente avente %%INPUT_SNMP (%%OUTPUT_SNMP) in questa lista. Specificare uno o più ID d'interfaccia SNMP, separati da virgola. I cambiamenti richiedono il riavvio di %{product}.",
    ["ignored_interfaces_title"] = "Interfacce ignorate",
    ["incompatible_influxdb_version_found"] = "Versione di InfluxDB non compatibile: richiesta \"%{required}\", trovata \"%{found}. Installare una versione aggiornata dal <a href='%{url}'>sito ufficiale</a>",
    ["influxdb_auth_description"] = "Abilita l'autenticazione su InfluxDB.",
    ["influxdb_auth_title"] = "Autenticazione InfluxDB",
    ["influxdb_create_error"] = "Impossibile creare il database \"%{db}\": %{msg}",
    ["influxdb_dbname_description"] = "Il datatabase da usare per il salvataggio delle serie temporali. I dati esistenti non verranno migrati.",
    ["influxdb_dbname_title"] = "Database InfluxDB",
    ["influxdb_password_description"] = "La password da usare per autenticarsi al database InfluxDB.",
    ["influxdb_query_timeout_description"] = "Durata Massima, in secondi, per le query ad InfluxDB prima che scatti il timeout. Le query che eccedono questa durata vengono annullate automaticamente.",
    ["influxdb_query_timeout_title"] = "Timeout delle Query",
    ["influxdb_retention_error"] = "Impossibile settare la retention policy nel datatbase \"%{db}\": %{msg}",
    ["influxdb_storage_note"] = "<b>NOTA</b>: i dati saranno eliminati ad intervalli di %{interval}, corrispondenti alla <a href='%{url}'>durata di shard</a> di InfluxDB.",
    ["influxdb_url_description"] = "La URL di una instanza avviata di InfluxDB.",
    ["influxdb_url_title"] = "URL InfluxDB",
    ["influxdb_username_description"] = "Il nome utente da usare per autenticarsi al database al database InfluxDB.",
    ["ingress_egress_flow_interface"] = "Interfaccia di ingresso/uscita",
    ["ingress_flow_interface"] = "Interfaccia d'ingresso",
    ["ingress_vrf_id"] = "Id VRF Ingresso",
    ["interfaces_timeseries"] = "Serie Temporali delle Interfacce",
    ["internal_log"] = "Log Interno",
    ["ip_order"] = "IP",
    ["is_influxdb_running"] = "InfluxDB sta girando?",
    ["l2_devices_timeseries"] = "Serie Temporali Dispositivi",
    ["last_seen"] = "Ultima Visita",
    ["ldap_authentication"] = "Autenticazione LDAP",
    ["ldap_server_address_description"] = "IP e porta del server LDAP (es. ldaps://localhost:636). Default: \"ldap://localhost:389\".",
    ["ldap_server_address_title"] = "Indirizzo Server LDAP",
    ["learning_period_description"] = "Tempo, in ore, dedicato all'apprendimento dei servizi periodici. Predefinito: 1 ora.",
    ["learning_period_title"] = "Periodo di Apprendimento del Servizio",
    ["license"] = "Licenza",
    ["local_auth"] = "Autenticazione Locale",
    ["local_host_cache_duration_description"] = "Tempo dopo il quale un host locale è cancellato dalla cache. Default: 1 ora.",
    ["local_host_cache_duration_title"] = "Durata Cache Host Locali",
    ["local_host_mask"] = "Nacondi gli host locali",
    ["local_host_max_idle_description"] = "Tempo d'inattività dopo il quale un host locale è considerato inattivo (sec). Gli host locali inattivi vengono archiviati in una cache, in modo che i contatori possano essere ripristinati nel caso in cui diventino di nuovo attivi. I contatori includono, pacchetti e byte totali e per applicazione Layer-7, ma non solo. Default: 5 min.",
    ["local_host_max_idle_title"] = "Timeout d'inattività Host Locali",
    ["local_hosts_cache_settings"] = "Impostazioni Cache Host Locali",
    ["local_hosts_timeseries"] = "Serie Temporali Host locali",
    ["logging"] = "Logging",
    ["longlived_flow_duration_description"] = "La durata minima di un flusso per essere considerato Long-Lived.",
    ["longlived_flow_duration_title"] = "Durata Flussi Longevi",
    ["max_entity_alerts_description"] = "Il numero massimo di avvisi per famiglia (host, reti, interfacce, ecc.). Gli avvisi vengono controllati periodicamente e, nel caso in cui il numero effettivo di avvisi ecceda il numero massimo, gli avvisi più vecchi verranno eliminati per preservare un numero di avvisi pari all'80%% del numero massimo configurato.",
    ["max_entity_alerts_title"] = "Numero Massimo di Allarmi",
    ["max_num_bytes_per_tiny_flow_description"] = "Il massimo numero di bytes che un flusso deve avere per essere considerato un flusso tiny. Default: 64.",
    ["max_num_bytes_per_tiny_flow_title"] = "Massimo Numero di bytes per Flusso Tiny",
    ["max_num_packets_per_tiny_flow_description"] = "Il massimo numero di pacchetti che un flusso deve avere per essere considerato un flusso tiny. Default: 3.",
    ["max_num_packets_per_tiny_flow_title"] = "Massimo Numero di Pacchetti per Flussi Tiny",
    ["max_num_secs_before_delete_alert_description"] = "Il numero massimo di giorni prima dell'eliminazione di un avviso. Tutti gli avvisi più vecchi di quel periodo verranno automaticamente eliminati sia all'avvio che periodicamente da ntopng",
    ["max_num_secs_before_delete_alert_title"] = "Numero Massimo di Giorni Prima dell'Eliminazione",
    ["max_ui_strlen_description"] = "Accorcia le stringhe più lunghe del massimo numero specificato di caratteri.",
    ["max_ui_strlen_title"] = "Lunghezza Massima della Stringa Visualizzata",
    ["mgmt_acl_description"] = "Una lista di reti IPv4 separate da virgola. Ogni rete deve essere esplicitamente permessa o bloccata tramite un segno \"+\" o \"-\" iniziale, rispettivamente. In caso di reti sovrapposte, vale il principio del match più lungo. Esempi: \"+127.0.0.0/8\" rende l'interfaccia accedibile solo da local host, \"+0.0.0.0/0,-192.168.0.0/16\", rende l'interfaccia accedibile da tutti eccetto 192.168.0.0/16. Bisogna riavviare %{product} per applicare i cambiamenti.",
    ["mgmt_acl_title"] = "Access Control List",
    ["misc"] = "Varie",
    ["multiple_ldap_account_type_description"] = "Scegli il tipo di acoount",
    ["multiple_ldap_account_type_title"] = "Tipi di account LDAP",
    ["multiple_timeseries_database_description"] = "Il driver usato scrivere e leggere dati sulle serie temporali.",
    ["multiple_timeseries_database_title"] = "Driver Serie Temporali",
    ["mysql"] = "MySQL",
    ["n2disk_license_description"] = "La registrazione continua del traffico richiede una licenza per n2disk, puoi acquistarne una sullo <a href=\"%{purchase_url}\">shop online</a>. Se fai parte di un'organizzazione no-profit, di ricerca o un istituto scolastico leggi <a href=\"%{universities_url}\">questo</a>.",
    ["n2disk_license_description_enterprise_l"] = "ntopng Enterprise L sblocca n2disk 1G, quindi nessuna licenza aggiuntiva è richiesta in tala caso.",
    ["n2disk_license_systemid"] = "SystemID: %{systemid}",
    ["n2disk_license_title"] = "Licenza n2disk",
    ["n2disk_license_version"] = "Versione di n2disk: %{version}",
    ["n2n_supernode_description"] = "L'indirizzo del supernodo di <a class='ntopng-external-link' href=\"%{url}\">n2n <i class=\"fas fa-external-link-alt\"></i></a>. Configura il tuo supernode per avere un accesso remoto completamente privato.<br>NOTA: questo cambiamento sarà applicato solo dopo aver riavviato l'Assistenza Remota.",
    ["n2n_supernode_title"] = "Supernodo",
    ["name_order"] = "Nome",
    ["network_discovery"] = "Scansione di Rete",
    ["network_discovery_interval_description"] = "Intervallo tra scansioni di rete consecutive. Default: 15 min.",
    ["network_discovery_interval_title"] = "Intervallo della scansione di rete",
    ["network_interfaces"] = "Interfacce di Rete",
    ["no_host_mask"] = "Non nascondere host",
    ["non_local_host_max_idle_description"] = "Tempo d'inattività dopo il quale un host remoto viene considerato inattivo. Default: 1 min.",
    ["non_local_host_max_idle_title"] = "Tempo d'inattività per gli Host Remoti",
    ["none"] = "Nessuna",
    ["note_timeseries_resolution_disabled"] = "NOTA: questa opzione è disabilitata perchè \"%{pref}\" non è impostato al valore di default.",
    ["note_update_frequency_disabled"] = "<b>NOTA</b>: disabilitato a seguito dei requisiti della preferenza \"%{pref}\".",
    ["observation_points"] = "Punti di Osservazione",
    ["other_timeseries"] = "Altre Serie Temporali",
    ["per_category"] = "Per Categoria",
    ["per_protocol"] = "Per Protocollo",
    ["policy_target_type"] = "Tipo di Politica",
    ["policy_target_type_description"] = "Specifica il tipo di politiche che desideri applicare.",
    ["posix"] = "Posix",
    ["preferences"] = "Preferenze",
    ["primary_dns"] = "DNS Primario",
    ["probe_ip_address"] = "IP del Probe",
    ["probe_ip_and_ingress_iface_idx"] = "Probe IP + Interfaccia Ingress",
    ["protocols"] = "Protocolli",
    ["radius_admin_group_description"] = "L'utente è autenticato come amministratore quando il valore della coppia Filter-Id Attribute-Value restituito da RADIUS corrisponde a quello specificato in questo campo. Se non c'è una corrispondenza, l'utente viene autenticato come non privilegiato.",
    ["radius_admin_group_title"] = "RADIUS Filter-Id Amministratore",
    ["radius_auth"] = "Autenticazione RADIUS",
    ["radius_secret_descroption"] = "La chiave segreta condivisa per l'autenticazione con il server RADIUS.",
    ["radius_secret_title"] = "Chiave Segreta RADIUS",
    ["radius_server_description"] = "Host e porta del server RADIUS (ad es. %{example}).",
    ["radius_server_title"] = "Indirizzo del server RADIUS",
    ["recording"] = "Registrazione Traffico",
    ["remote_host_mask"] = "Nascondi gli host remoti",
    ["report"] = "Report",
    ["restart_needed"] = "Riavviare %{product} per rendere effettivi i cambiamenti",
    ["runtime_prefs"] = "Preferenze in Esecuzione",
    ["safe_search_dns_description"] = "Il server DNS da usare per avere per eseguire una ricerca DNS sicura tramite il servizio <a href='%{url}'>Safe Search</a>.<br>Safe Search può essere abilitato per singoli host pool dalla pagina di configurazione dei host pool. Default: 208.67.222.123.",
    ["safe_search_dns_title"] = "DNS Child Safe",
    ["samaccount"] = "sAMAccount",
    ["search_path_description"] = "Radice del percorso usato per cercare gli utenti.",
    ["search_path_title"] = "LDAP Search Path",
    ["search_preferences"] = "Impostazioni di Ricerca",
    ["secondary_dns"] = "DNS Secondario",
    ["secondary_dns_description"] = "Un server DNS da utilizzare quando il server DNS global primario non è disponibile.",
    ["secondary_dns_title"] = "DNS Globale Secondario",
    ["security_alerts"] = "Allarmi di sicurezza",
    ["send_test_elasticsearch"] = "Test",
    ["send_test_mail"] = "Invia Email di Test",
    ["send_test_slack"] = "Invia Messaggio di Test",
    ["send_test_webhook"] = "Invia un messaggio di test",
    ["sender_username_description"] = "Imposta l'username del mittente delle notifiche slack",
    ["sender_username_title"] = "Username del Mittente delle Notifiche",
    ["simple_view"] = "Vista Semplice",
    ["slack_channel"] = "Canale Slack",
    ["slack_channel_names"] = "Nomi Canali",
    ["slack_channel_names_descr"] = "E' possibile configurare i nomi dei canali Slack in cui sono inviate i vari allarmi. I canali devono esistere affinché l'allarme possa essere visualizzato.",
    ["slack_integration"] = "Integrazione Slack",
    ["slack_notification_severity_preference_description"] = "Errori (solo errori), Errori ed avvertenze (Errori ed avvertenze, no informazioni), Tutto (tutti i tipi di allarmi saranno notificati).",
    ["slack_notification_severity_preference_title"] = "Preferenza della notifica in base alla severità",
    ["slack_send_error"] = "Errore nell'invio del messaggio Slack.",
    ["slack_sent_successfully"] = "Un messaggio è stato inviato al canale Slack #%{channel}.",
    ["slack_webhook_description"] = "Invia la tua notifica a questo URL slack",
    ["slack_webhook_title"] = "Notifiche Webhook",
    ["snmp"] = "SNMP",
    ["snmp_alerts"] = "Allarme SNMP",
    ["snmp_port_load_threshold_description"] = "Imposta una soglia (come percentuale della velocità dell'interfaccia) per emettere allarmi quando il traffico dell'interfaccia supera la soglia durante gli ultimi 5 minuti.",
    ["snmp_port_load_threshold_title"] = "Allarme sul Traffico Dell'Interfaccia SNMP",
    ["snmp_port_too_many_macs_description"] = "Genera un allarme quando il numero di MACs rilevatirilevato su una porta non trunk supera la soglia configurata",
    ["snmp_port_too_many_macs_title"] = "Troppi MACs in Non-Trunk",
    ["snmp_timeseries_config_link"] = "Le serie temporali per l'SNMP possono essere abilitate dalla <a href=\"%{url}\">sezione SNMP</a> delle preferenze sotto la vista esperta.",
    ["stats_reset"] = "Reset Statistiche",
    ["status_alerts"] = "Allarmi di Stato",
    ["successfully_connected_influxdb"] = "Database \"%{db}\" inizializzato con successo su InfluxDB(%{version})",
    ["syslog_alert_format_description"] = "Seleziona il formato degli allarmi da emettere su Syslog.",
    ["syslog_alert_format_json"] = "JSON",
    ["syslog_alert_format_plaintext"] = "Testo",
    ["syslog_alert_format_title"] = "Formato Allarmi su Syslog",
    ["syslog_notification"] = "Notifiche Syslog",
    ["syslog_sent_successfully"] = "Messaggio loggato con successo",
    ["system_probes_timeseries"] = "Serie Temporali dei Probe di Sistema",
    ["teams_send_error"] = "Errore nell'invio del messaggio a Microsoft Teams.",
    ["teams_sent_successfully"] = "Un messaggio è stato inviato a Microsoft Teams",
    ["telemetry"] = "Telemetria",
    ["telemetry_contribute"] = "Contribuisci",
    ["telemetry_do_not_contribute"] = "Non Contribuire",
    ["telemetry_email_description"] = "Fornisci un indirizzo email al quale esser ricontattato dal team di ntop in caso si rilevino anomalie o malfunzionamenti dalla telemetria. ntop non sarà in grado di ricontattarti senza questo indirizzo.",
    ["telemetry_email_title"] = "Contatti",
    ["timeseries"] = "Serie Temporali",
    ["timeseries_database"] = "Database Serie Temporali",
    ["timeseries_resolution_resolution_description_2"] = "L'intervallo tra punti consecutivi per gli host locali",
    ["timeseries_resolution_resolution_title"] = "Risoluzione Serie Temporali",
    ["toggle_access_log_description"] = "Attiva/disattiva la creazione di log d'accesso HTTP nella directory del dump dei dati. I cambiamenti saranno effettuati al prossimo avvio di %{product}.",
    ["toggle_access_log_title"] = "Abilita Log degli Acessi HTTP",
    ["toggle_active_local_host_cache_enabled_description"] = "Abilita la creazione di entrate cache per gli hosts locali attivi. Cachare periodicamente gli host locali attivi può essere utile per proteggere i dati dell'host da possibilifallimenti (es., perdite di corrente). Questo è particolarmente importante per gli host locali che raramente diventano inattivi poiché garantisce che i loro contatori saranno salvati dopo il lasso di tempo specificato.",
    ["toggle_active_local_host_cache_enabled_title"] = "Cache Host locali attivi",
    ["toggle_alert_syslog_description"] = "Abilita il logging di allarmi sul syslog di sistema.",
    ["toggle_alert_syslog_title"] = "Allarmi su Syslog",
    ["toggle_alerts_notifications_description"] = "Abilita/Disabilita notifiche alllarmi su programmmi esterni.",
    ["toggle_alerts_notifications_title"] = "Abilita/Disabilita Notifiche Allarmi",
    ["toggle_asn_rrds_description"] = "Aziona la creazione di serie temporali per traffico e protocolli applicativi per i sistemi autonomi.",
    ["toggle_asn_rrds_title"] = "Autonomous Systems",
    ["toggle_autologout_description"] = "Abilita il logout automatico dall'interfaccia web degli utenti con una sessione scaduta.",
    ["toggle_autologout_title"] = "Logout Automatico",
    ["toggle_autoupdates_description"] = "Abilita gli aggiornamenti automatici notturni. Un riavvio avviene automaticamente dopo ogni aggiornamento.",
    ["toggle_autoupdates_title"] = "Aggiornamenti Automatici",
    ["toggle_behaviour_analysis_description"] = "Abilita analisi comportamento traffico per host e interfacce. E' necessario un riavvio per rendere effettivo questo cambiamento.",
    ["toggle_behaviour_analysis_title"] = "Abilita Analisi Comportamentale del Traffico",
    ["toggle_captive_portal_description"] = "Abilita il captive portal per autenticare gli utenti della rete.",
    ["toggle_captive_portal_title"] = "Captive Portal",
    ["toggle_country_rrds_description"] = "Abilita la creazione delle serie temporali di traffico per le nazioni.",
    ["toggle_country_rrds_title"] = "Nazioni",
    ["toggle_date_type_description"] = "Cambia tra diversi <A class='ntopng-external-link' target='_blank' HREF='https://en.wikipedia.org/wiki/Date_format_by_country'>formati per rappresentare date</A>.",
    ["toggle_date_type_title"] = "Formato Data",
    ["toggle_dst_with_post_nat_dst_description"] = "Sostituisci gli indirizzi IPv4 di destinazione (%%IPV4_DST_ADDR) e le porte (%%L4_DST_PORT) con i lori valori post-nat (%%POST_NAT_DST_IPV4_ADDR e %%POST_NAPT_DST_TRANSPORT_PORT).",
    ["toggle_dst_with_post_nat_dst_title"] = "Utilizza gli indirizzi IP e le porte IPv4 di destinazione post-nat",
    ["toggle_elasticsearch_notification_description"] = "Abilita allarmi verso Elasticsearch.",
    ["toggle_elasticsearch_notification_title"] = "Abilita Allarmi Elasticsearch",
    ["toggle_elephant_flows_alerts_description"] = "Abilita gli avvisi generati quando un flusso elephant viene rilevato. Questo è utile per rilevare comportamenti inattesi (ad es. fuoriuscita di dati).",
    ["toggle_elephant_flows_alerts_title"] = "Avvisi Flussi Voluminosi",
    ["toggle_email_notification_description"] = "Abilita l'invio di allarmi via email.",
    ["toggle_email_notification_title"] = "Abilita Notifiche Email",
    ["toggle_emit_flow_alerts_description"] = "È possibile disabilitare la generazione di avvisi di flusso per salvare i cicli della CPU. In questo caso il punteggio verrà aggiornato ma non verrà attivato alcun avviso di flusso.",
    ["toggle_emit_flow_alerts_title"] = "Emettti un allarme flusso",
    ["toggle_emit_host_alerts_description"] = "È possibile disabilitare la generazione di avvisi host per salvare i cicli della CPU. In questo caso il punteggio verrà aggiornato ma non verrà attivato alcun avviso host.",
    ["toggle_emit_host_alerts_title"] = "Emetti allarmi host",
    ["toggle_enable_runtime_flows_dump_description"] = "Abilita il dump dei flussi verso il database configurato con -F.",
    ["toggle_enable_runtime_flows_dump_title"] = "Dump Flussi",
    ["toggle_flow_rrds_description"] = "Abilita la creazione di serie temporali in byte per ogni porta del dispositivo remoto come ricevuta tramite ZMQ (es. sFlow/NetFlow/SNMP).<br>Per dispositivi non sFlow, %%INPUT_SNMP e %%OUTPUT_SNMP devono apparire nel template di nprobe.",
    ["toggle_flow_rrds_title"] = "Dispositivi a Flusso",
    ["toggle_host_mask_description"] = "Per motivi di privacy potrebbe essere necessario nascondere gli indirizzi IP degli hosts. Per esempio se sei un ISP non sei autorizzato a conoscere quali indirizzi locali stanno accedendo gli hosts remoti.",
    ["toggle_host_mask_title"] = "Nacondi indirizzi IP host",
    ["toggle_host_pools_log_description"] = "Abilita di log collegati agli pool host. I log includono le associazioni e disassociazioni di host ai rispettivi pool.",
    ["toggle_host_pools_log_title"] = "Abilita Log di Eventi Host Pool",
    ["toggle_host_tskey_description"] = "La chiave in base alla quale salvare le serie temporali dell'host all'interno del dominio di broadcast. In una rete DHCP è generalmente desiderabile scegliere 'Mac'.",
    ["toggle_host_tskey_title"] = "Identificatore per gli Host nel Dominio di Broadcast Locale",
    ["toggle_http_auth"] = "Abilita Autenticazione HTTP",
    ["toggle_http_auth_descr"] = "Abilita l'autenticazione sull'interfaccia attraverso un autenticatore HTTP.",
    ["toggle_ids_alert_description"] = "Emetti allarmi generati da Sistemi di Intrusion Detection (IDS). Consultare l'integrazione di Suricata mediante syslog per esempio.",
    ["toggle_ids_alert_title"] = "Allarme IDS",
    ["toggle_informative_captive_portal_description"] = "Non autenticare gli utenti. Mostra solo una pagina informativa prima di permettere l'accesso a Internet.",
    ["toggle_informative_captive_portal_title"] = "Informativo",
    ["toggle_interface_name_only_description"] = "Se abilitato, mostra il nome dell'interfaccia solo nel menu a tendina.",
    ["toggle_interface_name_only_title"] = "Solo il Nome Interfaccia",
    ["toggle_internals_rrds_description"] = "Abilita la creazione di serie temporali <a href=\"%{url}\">internals</a>, e.g., entry nelle hash table, durata delle attività periodiche e numero di punti scritti, durata dei plugin e numero di chiamate.",
    ["toggle_internals_rrds_title"] = "Internals",
    ["toggle_ldap_anonymous_bind_description"] = "Abilita l'anonymous binding di LDAP.",
    ["toggle_ldap_anonymous_bind_title"] = "LDAP Anonymous Binding",
    ["toggle_ldap_auth"] = "Abilita Autenticazione LDAP",
    ["toggle_ldap_auth_descr"] = "Abilita l'autenticazione sull'interfaccia attraverso LDAP.",
    ["toggle_ldap_referrals_description"] = "Specifica se il client deve seguire automaticamente il referral fornito dai server LDAP.",
    ["toggle_ldap_referrals_title"] = "Segui Referral",
    ["toggle_local_auth"] = "Abilita Autenticazione Locale",
    ["toggle_local_auth_descr"] = "Abilita l'autenticazione sull'interfaccia secondo gli utenti definiti in %{product}.",
    ["toggle_local_host_cache_enabled_description"] = "Abilita la creazione di entrate cache per gli host locali inattivi. I contatori di Local hosts Cached sono resettati automaticamente ai loro valori precedenti al rilevamento di traffico host aggiuntivo.",
    ["toggle_local_host_cache_enabled_title"] = "Cache host locali inattivi",
    ["toggle_local_hosts_one_way_ts_description"] = "Abilita la creazione di serie temporali per il traffico one-way. Il traffico one-way è tipicamente generato da host remoti che fanno probing.",
    ["toggle_local_hosts_one_way_ts_title"] = "Serie Temporali One-Way",
    ["toggle_local_hosts_ts_creation_description"] = "Abilita la creazione completa delle serie temporali degli host, la limita ai byte e allo score, oppure la spegne.",
    ["toggle_local_hosts_ts_creation_title"] = "Serie Temporali degli Host",
    ["toggle_log_to_file_description"] = "Attiva/disattiva la creazione di un file di tracciamento delle attivita' (ntopng.log) nella directory del dump dei dati invece che nel syslog. I cambiamenti saranno effettuati al prossimo avvio di %{product}.",
    ["toggle_log_to_file_title"] = "Abilita Creazione File di Tracciamento",
    ["toggle_logging_level_description"] = "Seleziona il livello del logging a tempo d'esecuzione.",
    ["toggle_logging_level_title"] = "Livello Log",
    ["toggle_longlived_flows_alerts_description"] = "Abilita gli avvisi generati quando un flusso longevo viene rilevato. Questo è utile per rilevare comportamenti inattesi (ad es. fuoriuscita di dati).",
    ["toggle_longlived_flows_alerts_title"] = "Avvisi Flussi Longevi",
    ["toggle_midnight_stats_reset_description"] = "Abilita il reset automatico delle statistiche degli host e dei MAC a mezzanotte.",
    ["toggle_midnight_stats_reset_title"] = "Reset Statistiche Notturno",
    ["toggle_mysql_check_open_files_limit_description"] = "Abilita la generazione di allarmi MySQL a causa dei controlli periodici del MySQL open_files_limit.<br>Il controllo di open_files_limit è utile per individuare quando il numero di file MySQL aperti è alto, il che potrebbe condurre a errori di inserimento nel database.",
    ["toggle_mysql_check_open_files_limit_title"] = "Abilita allarmi MySQL",
    ["toggle_ndpi_flows_rrds_description"] = "Abilita la creazione di serie temporali del numero total di flussi per ogni applicazione",
    ["toggle_ndpi_flows_rrds_title"] = "Totale Flussi per Applicazione",
    ["toggle_ndpi_timeseries_creation_description"] = "Abilita la creazione di serie temporatli per le applicazioni Livello-7. Creare una serie temporale per protocollo richiede più spazio disco e I/O extra e, in generale, non è necessario.",
    ["toggle_ndpi_timeseries_creation_title"] = "Applicazioni Livello-7",
    ["toggle_network_discovery_description"] = "Abilita la scansione periodica di dispositivi all'interno della rete, utilizzando diverse tecniche quali scansioni ARP, MDNS e SSDP.",
    ["toggle_network_discovery_title"] = "Abilita Scansione di Rete",
    ["toggle_observation_points_rrd_creation_description"] = "Attiva/disattiva la creazione di byte e serie temporali dei flussi per i punti di osservazione disponibili.",
    ["toggle_observation_points_rrds_description"] = "Attiva/disattiva le serie temporali dei punti di osservazione. Non viene creata alcuna serie temporale se non sono presenti punti di osservazione.<br><b>Nota:</b> le preferenze delle applicazioni delle serie temporali L-7 del punto di osservazione sono le stesse dell'interfaccia; quindi se non viene creata alcuna serie temporale per le applicazioni dell'interfaccia L-7, non viene creata alcuna serie temporale per le applicazioni dei punti di osservazione L-7.",
    ["toggle_observation_points_rrds_title"] = "Punti di osservazione",
    ["toggle_os_rrds_description"] = "Attiva/disattiva la creazione di serie temporali di byte per i sistemi operativi.",
    ["toggle_os_rrds_title"] = "Sistemi Operativi",
    ["toggle_pools_rrds_description"] = "Aziona la creazione di serie temporali per traffico e protocolli applicativi per gli host pool configurati.",
    ["toggle_pools_rrds_title"] = "Host Pool",
    ["toggle_radius_auth"] = "Abilita Autenticazione RADIUS",
    ["toggle_radius_auth_descr"] = "Abilita l'autenticazione sull'interfaccia attraverso un server RADIUS.",
    ["toggle_send_telemetry_data_description"] = "Contribuisci al progetto trasmettendo dati di telemetria cifrati ed anonimi a <a href=\"%{url}\">ntop.org</a>. I dati riguardano il solo stato di %{product} (ad esempio i crash report). Niente che riguarda il traffico monitorato viene mai spedito.",
    ["toggle_send_telemetry_data_title"] = "Telemetria",
    ["toggle_shaping_directions_description"] = "Abilitando questa opzione è possible impostare politiche di shaping differenti per i pachetti d'ingresso e d'uscita.",
    ["toggle_shaping_directions_title"] = "Dividi Le Direzioni dello Shaping",
    ["toggle_slack_notification_description"] = "Aziona la notifica di allarmi attraverso Slack. Vedi le istruzioni d'integrazione <a href='%{url}'>qui</a>.",
    ["toggle_slack_notification_title"] = "Abilita le notifiche di <a href=\"%{url}\">Slack</a>",
    ["toggle_snmp_alerts_port_duplexstatus_change_description"] = "Emetti allarmi quando lo stato duplex di una interfaccia cambia",
    ["toggle_snmp_alerts_port_duplexstatus_change_title"] = "Allarme sul Cambiamento dello Stato Duplex di una Interfaccia.",
    ["toggle_snmp_alerts_port_errors_description"] = "Emetti allarmi generate quando i contatori di discard/error di una interfaccia crescono",
    ["toggle_snmp_alerts_port_errors_title"] = "Allarme sui Contatori Discards/Errori di una Interfaccia",
    ["toggle_snmp_alerts_port_status_change_description"] = "Emetti allarmi generati quando l'Operational State di una Interfaccia cambia.",
    ["toggle_snmp_alerts_port_status_change_title"] = "Allarme sul Cambiamento dell'Operational State di una Interfaccia.",
    ["toggle_snmp_debug_description"] = "Abilita il debugging delle device SNMP. Abilitando questa opzione verranno stampate informazioni aggiuntive nel log di ntopng.",
    ["toggle_snmp_debug_title"] = "Debug SNMP.",
    ["toggle_snmp_port_admin_status_description"] = "Abilita il cambiamento dell'admin status dell'interfaccia. Il cambiamento è immediato ma il refresh dello stato avverrà durante il prossimo poll.",
    ["toggle_snmp_port_admin_status_title"] = "Admin Status dell'Interfaccia",
    ["toggle_snmp_rrds_description"] = "Attiva / disattiva la creazione di serie temporali per i bytes di ogni porta dei dispositivi SNMP. Per ogni porta del dispositivoverrà creato un RRD con bytes di ingresso / uscita.",
    ["toggle_snmp_rrds_title"] = "Serie Temporali dei Dispositivi SNMP",
    ["toggle_src_with_post_nat_src_description"] = "Sostituisci gli indirizzi sorgente IPv4(%%IPV4_SRC_ADDR) e le porte (%%L4_SRC_PORT) con i loro valori post-nat (%%POST_NAT_SRC_IPV4_ADDR e %%POST_NAPT_SRC_TRANSPORT_PORT).",
    ["toggle_src_with_post_nat_src_title"] = "Usa indirizzi IP Post-Nat e porte IPv4",
    ["toggle_system_probes_timeseries_description"] = "Abilita la creazione di serie temporali per i probe di sistema, quali quello per il monitoraggio dell'RTT o per il monitoraggio di Redis.",
    ["toggle_system_probes_timeseries_title"] = "Probes",
    ["toggle_theme_description"] = "Modifica lo stile di ntopng.",
    ["toggle_theme_title"] = "Tema di ntopng",
    ["toggle_thpt_content_description"] = "Seleziona l'unità di throughput da far vedere nei report del traffico.",
    ["toggle_thpt_content_title"] = "Utità di Throughput",
    ["toggle_tiny_flows_dump_description"] = "Attiva / disattiva l'esportazione dei flussi tiny, ovvero dei flussi con pochi pacchetti o byte.Riduce la cardinalità dei flussi nei database, velocizza l'inserimento e la ricerca. L'ottimizzazione dei flussi tiny può aiutare a limitare la cardinalità del flusso senza ridurre la visibilità delle informazioni salvate.",
    ["toggle_tiny_flows_dump_title"] = "Esporta Flussi Tiny",
    ["toggle_top_sites_description"] = "Attiva / disattiva la creazione dei siti Web più visitati per gli host locali. I top siti sono creati usando <a href=\"%{url}\">un' euristica</a> che mantiene non più di 20 siti per host locale. L'euristica funziona completamente in memoria e non richiede alcuna interazione con il disco. I siti principali vengono cancellati ogni 5 minuti. È possibile creare un archivio storico dei siti Web più visitati, per ciascun host locale, effettuando periodicamente il polling del <i class=\"fas fa-download fa-sm\"></i> download link del JSON accessibile dalla pagina dei dettagli host.",
    ["toggle_top_sites_title"] = "Top Siti HTTP",
    ["toggle_traffic_rrd_creation_description"] = "Abilita la creazione delle serie temporali per bytes e pacchetti.",
    ["toggle_traffic_rrd_creation_title"] = "Traffico",
    ["toggle_users_rrds_description"] = "Abilita la creazione di serie temporali per bytes e protocolli applicativi per gli utenti definiti.",
    ["toggle_users_rrds_title"] = "Utenti",
    ["toggle_vlan_rrds_description"] = "Abilita di serie temporali per traffico e protocolli applicativi per VLAN",
    ["toggle_vlan_rrds_title"] = "VLAN",
    ["toggle_webhook_notification_description"] = "Abilita la notifica di allarmi via via webhook (HTTP).",
    ["toggle_webhook_notification_title"] = "Abilita Notifica Allarmi via Webhook",
    ["topk_heuristic_precision_description"] = "Abilita l'utilizzo di un'euristica quando si aggregano i principali host, i paesi, ecc., per creare report di traffico. Utile quando si creano report su lunghi periodi.",
    ["topk_heuristic_precision_title"] = "Euristica Top-K",
    ["traffic_behaviour"] = "Comportamento Traffico",
    ["traffic_bridging"] = "Traffic Bridging",
    ["traffic_shaping"] = "Shaping del Traffico",
    ["ts_resolution_note2"] = "Abbassare la risoluzione puù creare gap nei dati esistenti. Si raccomanda <a target=_blank href=\"%{url}\">la cancellazione del database</a> %{external_icon}.",
    ["updates"] = "Aggiornamenti",
    ["user_authentication"] = "Autenticazione Utente",
    ["user_group_description"] = "Nome del gruppo al quale un utente deve appartenere per autenticarsi come un utente non privilegiato.",
    ["user_group_title"] = "Gruppo Utente LDAP",
    ["vlan"] = "Id VLAN",
    ["web_user_interface"] = "Interfaccia Web Utente",
    ["webhook_notification"] = "Notifica Webhook",
    ["webhook_notification_severity_preference_description"] = "Solo Errori, Errori e Warning, Tutte gli Allarmi.",
    ["webhook_notification_severity_preference_title"] = "Severità delle Notifiche",
    ["webhook_password_description"] = "La password da usare per l'autenticazione HTTP (opzionale).",
    ["webhook_send_error"] = "Errore durante l'invio del messaggio al webhook.",
    ["webhook_sent_successfully"] = "messaggio spedito al webhook.",
    ["webhook_sharedsecret_description"] = "Imposta un segreto definito dall'utente che possa esser validato dal ricevitore (opzionale).",
    ["webhook_sharedsecret_title"] = "Segreto Condiviso",
    ["webhook_url_description"] = "Trasmetti una notifica al seguente URL in formato JSON.",
    ["webhook_url_title"] = "URL per le notifiche",
    ["webhook_username_description"] = "L'username da usare per l'autenticazione HTTP (opzionale).",
    ["wifi"] = "Wireless",
    ["zmq_interfaces"] = "Interfacce ZMQ",
    ["geo_map"] = {
      ["geo_map"] = "Geo Map",
      ["geo_map_customization"] = "Customizzazione Geo Map",
      ["toggle_geo_map_alerted_flows_title"] = "Num. di flussi allertati",
      ["toggle_geo_map_asname_title"] = "Nome del sistema autonomo",
      ["toggle_geo_map_blacklisted_flows_title"] = "Num. Flussi Blacklisted",
      ["toggle_geo_map_custom_description"] = "Attiva o disattiva le informazioni extra specificate nella visualizzazione host della mappa geografica",
      ["toggle_geo_map_host_name_title"] = "Nome Host",
      ["toggle_geo_map_num_flows_title"] = "Num. Flussi",
      ["toggle_geo_map_rxtx_data_title"] = "Rx/Tx data",
      ["toggle_geo_map_score_title"] = "Score",
    },
  },
  ["processes_stats"] = {
    ["active_processes_title"] = "Processi Attivi: Vista in Tempo Reale",
    ["active_since"] = "Attivo Da",
    ["bytes_count"] = "Conteggio Bytes",
    ["legend"] = "Legenda",
    ["lines"] = "Linee",
    ["missing_pid_name_message"] = "Nome pid mancante",
    ["no_traffic_detected"] = "Nessun traffico rilevato per questo processo, il processo del flusso è scaduto, oppure il processo è terminato.",
    ["processes_timeline_title"] = "Linea del tempo dei Processi",
    ["stack"] = "Pila",
    ["timeline"] = "Linea del tempo",
    ["top_hosts"] = "Top Host",
    ["top_hosts_traffic"] = "Top Traffico degli Host",
    ["traffic_rcvd"] = "Traffico Rcvt",
    ["traffic_sent"] = "Traffico Invt",
    ["type"] = "Tipo",
  },
  ["queries"] = {
    ["custom_query"] = "Custom Query",
    ["queries"] = "Queries",
    ["raw_flows"] = "Flussi",
    ["raw_flows_bytes"] = "Flussi (Grafico Bytes)",
    ["raw_flows_records"] = "Flussi",
    ["raw_flows_score"] = "Flussi (Grafico Score)",
    ["top_clients"] = "Migliori Client",
    ["top_l7_contacts"] = "Migliori Contatti L7",
  },
  ["queue"] = {
    ["activeFlowsToDump"] = "Dump Flussi Attivi",
    ["hookFlowEnd"] = "Hook script utente flowEnd",
    ["hookPeriodicUpdate"] = "Hook script utente periodicUpdate",
    ["hookProtocolDetected"] = "Hook script utente protocolDetected",
    ["idleFlowsToDump"] = "Dump Flussi Idle",
  },
  ["queue_description"] = {
    ["activeFlowsToDump"] = "Coda dei flussi attivi in attesa di essere dumpati",
    ["hookFlowEnd"] = "Flussi idle accodati in attesa che il loro hook script utente flowEnd venga eseguito",
    ["hookPeriodicUpdate"] = "Flussi attivi accodati in attesa che il loro hook script utente periodicUpdate venga eseguito",
    ["hookProtocolDetected"] = "Flussi nuovi accodati in attesa che il loro hook script utente protocolDetected venga eseguito",
    ["idleFlowsToDump"] = "Coda dei flussi idle in attesa di essere dumpati",
  },
  ["report"] = {
    ["application_breakdown"] = "Breakdown Applicazione",
    ["applications"] = "Applicazioni",
    ["approximated_data"] = "È possibile eseguire un'approssimazione dei dati minore quando si selezionano intervalli di tempo molto grandi.",
    ["begin_date_time"] = "Data/Ora d'Inizio",
    ["date"] = "%{day}-%{month}-%{year}",
    ["end_date_time"] = "Data/Ora di Fine",
    ["filter_report"] = "Filtra Report",
    ["generate"] = "Genera",
    ["invalid_begin"] = "Inizio Invalido",
    ["invalid_begin_date"] = "Data d'inizio non valida",
    ["invalid_begin_end"] = "Inizio/Fine Invalido",
    ["invalid_to"] = "Data di fine non valida",
    ["local_networks"] = "Networks Locali",
    ["local_remote"] = "Locale/Remoto",
    ["network_interface"] = "Interfaccia di Rete",
    ["period"] = "Intervallo",
    ["please_check_format"] = "Controlla il formato",
    ["please_choose_valid"] = "Scegli una data/ora d'inizio/fine valida",
    ["please_choose_valid_date_and_time"] = "Scegli una data e ora valida",
    ["receivers"] = "Destinatari",
    ["remote_local"] = "remoto/locale",
    ["report_for_subject"] = "Report - %{num_min} - per %{subject} dal %{dt}",
    ["reports_professional_only"] = "I report sono disponibili nella versione Professional",
    ["score_distribution"] = "Distribuzione Score",
    ["senders"] = "Mittenti",
    ["starting"] = "Da",
    ["submit_filter"] = "Conferma Filtro",
    ["symbolic_host_name_at_visualization"] = "I nomi simbolici sono risolti in fase di visualizzazione.",
    ["toggle_all"] = "Cambia Tutti",
    ["top_asn"] = "Top ASN",
    ["top_countries"] = "Top Paesi",
    ["top_local_hosts"] = "Top Hosts Locali",
    ["top_local_os"] = "Top Sistema Operativo Locale",
    ["top_networks"] = "Top Reti",
    ["top_non_local_os"] = "Top Non-Local OS",
    ["top_remote_hosts"] = "Top Hosts Remoti",
    ["top_talkers"] = "Top Hosts",
    ["total_traffic"] = "Traffico Totale",
    ["traffic_anomalies"] = "Anomalie Traffico",
    ["traffic_anomalies_local_hosts"] = "Anomalie Local Hosts",
    ["traffic_anomalies_remote_hosts"] = "Anomalie Remote Hosts",
    ["traffic_report"] = "Report di Traffico",
    ["use_server_timezone"] = "Utilizza la Timezone del Server",
  },
  ["rest_consts"] = {
    ["ADD_POOL_FAILED"] = "Impossibile aggiungere il pool",
    ["ADD_POOL_FAILED_TOO_MANY_POOLS"] = "Troppe pools create. Considera un upgrade ntopng a Enterprise M o migliore per creare pools addizionalior above to create additionl pools.",
    ["ADD_POOL_FAILED_TOO_MANY_POOLS_ENTERPRISE"] = "Troppe pools create.",
    ["BAD_CONTENT"] = "Contenuto non valido",
    ["BAD_FORMAT"] = "Formato non valido",
    ["BIND_POOL_MEMBER_ALREADY_BOUND"] = "Impossibile associare il membro al pool. Membro già appartenente ad un altro pool",
    ["BIND_POOL_MEMBER_FAILED"] = "Impossibile associare il membro al pool",
    ["CONFIGURATION_FILE_MISMATCH"] = "Il file caricato non è valido!",
    ["DELETE_POOL_FAILED"] = "Impossibile eliminare il pool",
    ["DHCP_ACTIVE_LEASES_NOT_NEDGE"] = "Build non è nEdge.",
    ["DHCP_ACTIVE_LEASES_NOT_ROUTING_MODE"] = "L'elenco dei lease DHCP è disponibile solo in modalità di routing.",
    ["EDIT_POOL_FAILED"] = "Impossibile modificare il pool",
    ["INFRASTRUCTURE_INSTANCE_ALREADY_EXISTING"] = "Istanza Duplicata.",
    ["INFRASTRUCTURE_INSTANCE_CHECK_AUTH_FAILED"] = "Autorizzazione fallita. Token non valido",
    ["INFRASTRUCTURE_INSTANCE_CHECK_FAILED"] = "Impossibile eseguire la richiesta. Destinazione irraggiungibile.",
    ["INFRASTRUCTURE_INSTANCE_CHECK_INVALID_RESPONSE"] = "Risposta ricevuta invalida. Aspettato JSON",
    ["INFRASTRUCTURE_INSTANCE_CHECK_NOT_FOUND"] = "Non Trovato",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_ALIAS"] = "Il nome dell'istanza non può essere vuoto.",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_BANDWIDTH_THRESHOLD"] = "La soglia della banda non può essere vuota!",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_ID"] = "L'ID dell'istanza non può essere vuoto.",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_RTT_THRESHOLD"] = "La soglia RTT non può essere vuota!",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_TOKEN"] = "Il token dell'istanza non può essere vuoto.",
    ["INFRASTRUCTURE_INSTANCE_EMPTY_URL"] = "Un URL per l'istanza deve essere definito.",
    ["INFRASTRUCTURE_INSTANCE_SAME_ALIAS"] = "Esiste già un'istanza con questo nome.",
    ["INFRASTRUCTURE_INSTANCE_SAME_TOKEN"] = "Un'istanza è già stata collegata con questo token.",
    ["INFRASTRUCTURE_INSTANCE_SAME_URL"] = "Un'istanza con questo URL è già definita",
    ["INTERNAL_ERROR"] = "Errore",
    ["INVALID_ARGUMENTS"] = "Parametri non validi",
    ["INVALID_HOST"] = "Host non valido",
    ["INVALID_INTERFACE"] = "Interfaccia non valida",
    ["NAME_RESOLUTION_FAILED"] = "Impossibile risolvere l'host in un indirizzo IP",
    ["NOT_FOUND"] = "Non trovato",
    ["NOT_GRANTED"] = "Non autorizzato",
    ["NO_SNMP_DEVICE_DISCOVERED"] = "Nessun dispositivo SNMP trovato",
    ["OK"] = "Successo",
    ["PARTIAL_IMPORT"] = "La configurazione è stata importata con successo, ma alcuni elementi sono stati sostituiti",
    ["POOL_ADDED_SUCCESSFULLY"] = "Pool aggiunto con successo",
    ["POOL_DELETED_SUCCESSFULLY"] = "Pool eliminato con successo",
    ["POOL_EDITED_SUCCESSFULLY"] = "Pool modificafto con successo",
    ["POOL_MEMBER_BOUND_SUCCESSFULLY"] = "Membro associato correttamente",
    ["POOL_NOT_FOUND"] = "Pool non trovato",
    ["SNMP_DEVICE_ADDED_SUCCESSFULLY"] = "Dispositivo SNMP aggiunto con successo",
    ["SNMP_DEVICE_ALREADY_ADDED"] = "Dispositivo SNMP già aggiunto",
    ["SNMP_DEVICE_DELETED_SUCCESSFULLY"] = "Dispositivo SNMP eliminato con successo",
    ["SNMP_DEVICE_EDITED_SUCCESSFULLY"] = "Dispositivo SNMP modificato con successo",
    ["SNMP_DEVICE_INTERFACE_STATUS_CHANGE_FAILED"] = "Cambiamento di stato del dispositivo SNMP fallito",
    ["SNMP_DEVICE_UNREACHABLE"] = "Nessuna risposta dal dispositivo SNMP",
    ["WIDGETS_MISSING_DATASOURCES"] = "Sorgenti dati mancanti",
    ["WIDGETS_MISSING_DATASOURCE_PARAMS"] = "Parametri sorgenti dati mancanti",
    ["WIDGETS_MISSING_DATASOURCE_TYPE"] = "Tipi di sorgenti dati mancanti",
    ["WIDGETS_MISSING_TRANSFORMATION"] = "Trasformazione mancante",
    ["WIDGETS_UNKNOWN_DATASOURCE_TYPE"] = "Tipo di origine dati sconosciuto",
  },
  ["restart"] = {
    ["confirm"] = "Riavviare %{product} adesso?",
    ["just_started"] = "Avviamento in corso. Le statistiche sul traffico verranno prima calcolate in %{when} <a target='_blank' href='%{url}'><i class='fas fa-info-circle'></i></a>.",
    ["restart"] = "Riavvia",
    ["restart_now"] = "Riavvia adesso",
    ["restart_product"] = "Riavvia %{product}",
    ["restart_required"] = "Riavvio Necessario",
    ["restarting"] = "Riavvio in corso... (potrebbero esser necessari alcuni secondi)",
  },
  ["scripts_list"] = {
    ["alert_severity"] = "Severità allarme",
    ["all"] = "Tutti",
    ["are_you_sure"] = "Le modifiche non sono ancora state salvate. Sei veramente sicuro di volere chiudere la finestra?",
    ["attention"] = "Attenzione",
    ["back_scripts_page"] = "Torna agli Scripts",
    ["blacklisted_country"] = "Scrivere i codici dei paesi separati da virgole, e.g., IT,FR,DE,UK",
    ["config"] = "Configurazione",
    ["exclusion_list"] = "Elenco di indirizzi IP separati da virgole. Questo avviso non verrà attivato per gli host all'interno di questo elenco.",
    ["exclusion_list_title"] = "Hosts Esclusi",
    ["filter_dropdown"] = "Filtra Categorie",
    ["items_list_comma"] = "Caratteri speciali non sono ammessi",
    ["missing_configset"] = "La configurazione richiesta non è stata trovata!",
    ["note_packet_interface"] = "Verificare la disponibilità di interfacce a pacchetto",
    ["note_zmq_interface"] = "Verificare la disponibilità per interfacce ZMQ",
    ["previous_page"] = "Ritornare alla pagina precedente!",
    ["reset_default"] = "Ripristina Default",
    ["script_search"] = "Cerca Script:",
    ["scripts_x"] = "Scripts %{subdir} - %{config}",
    ["select_operator"] = "Selezionare un operatore!",
    ["template_not_found"] = "Il template non è stato trovato!",
    ["actions"] = {
      ["edit"] = "Modifica",
      ["edit_script"] = "Modifica Script",
      ["view"] = "Visualizza",
      ["view_src_script"] = "Visualizza Sorgenti Script",
    },
    ["templates"] = {
      ["blacklisted_country_list"] = "Lista Paesi in Blacklist",
      ["elephant_flows_l2r"] = "Soglia Flussi Elephant (Locale a Remoto)",
      ["elephant_flows_r2l"] = "Soglia Flussi Elephant (Remoto a Locale)",
      ["excluded_applications"] = "Escludere applicazioni e categorie",
      ["flow_duration_threshold"] = "Soglia Durata Flusso",
      ["template_not_implemented"] = "Il template non è ancora stato implementato!",
    },
  },
  ["severity_groups"] = {
    ["group_error_or_higher"] = "Errore o Più",
    ["group_none"] = "Nessuno",
    ["group_notice_or_lower"] = "Notizia o Meno",
    ["group_warning"] = "Attenzione",
  },
  ["sflows_stats"] = {
    ["active_flows"] = "Flussi Attivi",
    ["client_peer"] = "Peer Client",
    ["client_process"] = "Processo Client",
    ["server_peer"] = "Peer Server",
    ["server_process"] = "Processo Server",
    ["total_bytes"] = "Bytes Totali",
  },
  ["shaping"] = {
    ["applied_to"] = "Applicato a",
    ["bandwidth_manager"] = "Gestore di Banda",
    ["confirm_delete_all_policies"] = "Vuoi veramente eliminare tutte le politiche del pool \"%{pool_name}",
    ["confirm_delete_policy"] = "Sei sicuro di volerla eliminare",
    ["confirm_delete_shaper"] = "Vuoi veramente eliminare lo shaper ",
    ["daily_time"] = "Tempo giornaliero (Corrente / Quota)",
    ["daily_time_quota"] = "Quota sul Tempo Giornaliero",
    ["daily_traffic"] = "Traffico Giornaliero (Corrente / Quota)",
    ["daily_traffic_quota"] = "Quota del traffico giornaliero",
    ["delete_all_policies"] = "Elimina tutte le politiche",
    ["delete_policy"] = "Elimina Politica",
    ["delete_shaper"] = "Elimina Shaper",
    ["for_dropping_all_traffic"] = "per bloccare tutto il traffico",
    ["for_no_shaping"] = "per non tagliare la banda",
    ["initial_clone_protocols"] = "Le regole inziali dei protocolli saranno clonate",
    ["initial_empty_protocols"] = "Le regole inziali dei protocolli saranno vuote",
    ["manage_policies"] = "Gestisci Politiche",
    ["monthly_time_quota"] = "Quota di tempo mensile",
    ["monthly_traffic_quota"] = "Quota di traffico mensile",
    ["no_quota_applied"] = "Nessuna Quota",
    ["no_quota_data"] = "Nessuna Quota Impostata",
    ["no_quota_traffic"] = "Ancora nessun traffico relativo a quote",
    ["no_shapers_available"] = "Nessuno shaper disponibile",
    ["note_default_pool_config"] = "Le politiche e le quote sono solo applicate ai soli host pool definiti dagli utenti. Seleziona un pool differente o creane uno nuovo <a href='%{url}'>qui</a>.",
    ["note_drop_core"] = "Bloccare alcuni protocolli principali può avere effetti collaterali su altri protocolli. Per esempio se si blocca il traffico DNS,<br>i nomi simbolici degli hosts non saranno più risolti, ed in tal modo funzionerà solo la comunicazione con gli IP numerici.",
    ["note_quota_unlimited"] = "Imposta il traffico e la quota a 0 per traffico illimitao.",
    ["note_target_type"] = "Puoi modificare cosa viene mostrato nella colonna di target attraverso il campo Tipo di Target Politica nelle <a href=\"%{url}\">Preferenze di Bridging</a> in modalità esperto.",
    ["note_traffic_categories"] = "Puoi configurare le Categorie di Traffico dalla <a href=\"%{url}\">Pagina delle Applicazioni</a>.",
    ["notes"] = "NOTE",
    ["policy_from_pool"] = "politica dalla pool",
    ["protocol_families"] = "Famiglie di Protocolli",
    ["protocol_policy"] = "Politica sul Protocollo",
    ["protocols"] = "Protocolli",
    ["protocols_policies"] = "Politiche sui Protocolli",
    ["see_quotas_here"] = "Visita la <a href='%{url}'>Pagina delle Quote</a> dell'host pool per una visione completa delle quote per degli host pool attivi.",
    ["select_to_clone"] = "Seleziona un network esistente per clonare le regole del protocollo",
    ["set_max_rate_to"] = "Imposta la velocità massima a",
    ["shaper0_message"] = "Lo shaper 0 è il default per gli host locali che non hanno uno shaper definito",
    ["shaper_id"] = "Shaper Id",
    ["shapers_in_use_message"] = "Gli shapers possono essere eliminati solo se non sono applicati a nessuna rete",
    ["traffic_from"] = "Traffico da",
    ["traffic_to"] = "Traffico verso",
    ["weekly_time_quota"] = "Quota di tempo settimanale",
    ["weekly_traffic_quota"] = "Quota di traffico mensile",
  },
  ["show_alerts"] = {
    ["1_day"] = "1 giorno",
    ["1_hour"] = "1 ora",
    ["1_month"] = "1 mese",
    ["1_week"] = "1 settimana",
    ["1_year"] = "1 anno",
    ["30_min"] = "30 minuti",
    ["5_min"] = "5 minuti",
    ["6_months"] = "6 mesi",
    ["acknowledge_alert"] = "Riconosci avviso",
    ["acknowledged"] = "Riconosciuto",
    ["alarmable"] = "Allarmabile",
    ["alert_actions"] = "Azioni",
    ["alert_count"] = "Contatore",
    ["alert_counts"] = "Conteggio",
    ["alert_datetime"] = "Data/Ora",
    ["alert_description"] = "Descrizione",
    ["alert_duration"] = "Durata",
    ["alert_severity"] = "Severità",
    ["alert_severity_and_type"] = "Severità e Tipo",
    ["alert_type"] = "Tipo d'Allarme",
    ["alerts"] = "Allarmi",
    ["alerts_are_disabled_message"] = "Gli allarmi sono disabilitati. Controlla la relativa pagina delle impostazioni per abilitarli.",
    ["alerts_queries"] = "Query Allarmi",
    ["alerts_to_purge"] = "Allarmi da Eliminare",
    ["alerts_to_purge_x"] = "%{filter} avviso/i per l'eliminazione",
    ["cli_country"] = "Paese del Client",
    ["confirm_acknowledge_alert"] = "Vuoi davvero riconoscere l'avviso specificato?",
    ["confirm_acknowledge_alerts"] = "Gli avvisi memorizzati che corrispondono ai criteri specificati verranno riconosciuti. Continuare?",
    ["confirm_delete_alert"] = "Vuoi veramente eliminare l'allarme specificato",
    ["confirm_delete_alerts"] = "Gli avvisi memorizzati che corrispondono ai criteri specificati verranno eliminati. Continuare?",
    ["confirm_delete_filtered_alerts"] = "Gli avvisi memorizzati che corrispondono ai criteri di disattivazione specificati verranno eliminati.",
    ["confirm_external_link"] = "Stai per esserer reindirizzato a: ",
    ["confirm_filter_alert"] = "I controlli che soddisfano i criteri di esclusione specificati non verranno eseguiti e gli avvisi non verranno attivati.",
    ["confirm_label_alert"] = "Vuoi davvero etichettare l'avviso specificato?",
    ["confirm_label_alerts"] = "Gli avvisi memorizzati che corrispondono ai criteri specificati verranno etichettati. Continuare?",
    ["confirm_release_alert"] = "Rilasciare l'allarme specificato?",
    ["daily"] = "Per Giorno",
    ["delete_alert"] = "Elimina Allarme",
    ["delete_alerts_configuration"] = "Elimina la Configurazione Allarmi",
    ["delete_all"] = "Elimina Tutto",
    ["delete_config_btn"] = "Elimina la Configurazione comune degli %{conf}",
    ["delete_config_message"] = "Vuoi veramente eliminare la configurazione comune %{granularity} degli %{conf}s",
    ["detected_alerts"] = "Allarmi Riscontrati",
    ["disable_alerts"] = "Disabilita",
    ["disable_alerts_message"] = "Disabilitare gli allarmi \"%{type}\" su %{entity_value}?",
    ["disable_alerts_title"] = "Disabilita Allarme",
    ["disabled_alerts"] = "Allarmi Disabilitati",
    ["disabled_alerts_not_available"] = "Questa funzione è disponibile solo su ntopng Pro o superiore.",
    ["download_alerts"] = "Scarica Allarmi",
    ["dropped_alerts"] = "Allarmi Persi",
    ["enable_alerts"] = "Abilita",
    ["enable_alerts_message"] = "Abilitare gli allarmi \"%{type}\" su %{entity_value}?",
    ["enable_alerts_title"] = "Abilita Allarme",
    ["engaged"] = "In Corso",
    ["engaged_alerts"] = "Allarmi in Corso",
    ["exclude_alert_err"] = "Errore durante l'esclusione dell'avviso, controllare i parametri e riprovare",
    ["expand_action"] = "Espandi",
    ["explorer"] = "Esplora",
    ["filter_alert"] = "Escludi Controlli",
    ["filter_disable_check"] = "Qualsiasi host (disabilita il controllo)",
    ["filters"] = "Filtri",
    ["five_minutes"] = "Cinque minuti",
    ["flow_alerts"] = "Allarmi sui Flussi",
    ["flow_checks_drops_descr"] = "Emetti un allarme quando gli <a href='%{url}'>user script sui flussi</a> non sono eseguiti.",
    ["flow_checks_drops_title"] = "Allarme per Mancata Esecuzione di User Script sui Flussi",
    ["flush_data"] = "Elimina i Dati",
    ["host_delete_config_btn"] = "Elimina gli host %{host} Allarmi Configurati",
    ["host_delete_config_confirm"] = "Vuoi veramente eliminare la configurazione %{granularity} degli allarmi sull'host",
    ["hourly"] = "Per Ora",
    ["ids_ips_log"] = "Registro di rilevamento e prevenzione delle intrusioni",
    ["ids_ips_log_descr"] = "Registro degli eventi di rilevamento e prevenzione delle intrusioni, ad esempio quando un host è stato aggiunto (o rimosso) dal pool di host incarcerati. Gli eventi registrati vengono archiviati come avvisi negli avvisi dell'interfaccia di sistema.",
    ["iface_delete_config_btn"] = "Elimina la Configurazione dell'interfaccia %{iface}",
    ["iface_delete_config_confirm"] = "Vuoi veramente eliminare la configurazione %{granularity} degli allarmi sull'interfaccia",
    ["iface_engaged_dropped_alerts"] = "Allarmi in Corso vs Persi",
    ["in_the_selected_time_frame"] = "nel periodo di tempo selezionato",
    ["interface_drops_threshold"] = "Perdite di Pacchetti sull'Interfaccia",
    ["interface_drops_threshold_descr"] = "La massima percentuale di pacchetti persi sul totale dei pacchetti osservati dall'interfaccia",
    ["label_alert"] = "Avviso etichetta",
    ["last_day"] = "Ultimo Giorno",
    ["last_hour"] = "Ultima Ora",
    ["last_minute"] = "Ultimo Minuto",
    ["minute"] = "Minuto",
    ["network_delete_config_btn"] = "Elimina Allarmi Rete %{network}",
    ["network_delete_config_confirm"] = "Vuoi veramente eliminare la allarmi %{granularity} della configurazione per la data rete",
    ["no_recorded_alerts_message"] = "Nessun allarme rilevato per l'interfaccia %{ifname}",
    ["num_ignored_alerts"] = "Allarmi Ignorati",
    ["older_1_day_ago"] = " più vecchi di 1 giorno",
    ["older_1_hour_ago"] = "più vecchi di 1 ora fa",
    ["older_1_month_ago"] = "più vecchi di 1 mese fa",
    ["older_1_week_ago"] = "più vecchi di 1 settimana fa",
    ["older_1_year_ago"] = "più vecchi di 1 anno fa",
    ["older_30_minutes_ago"] = "più vecchi di 30 minuti fa",
    ["older_5_minutes_ago"] = "più vecchi di 5 minuti fa",
    ["older_6_months_ago"] = "più vecchi di 6 mesi fa",
    ["older_than"] = "older than",
    ["past"] = "Passati",
    ["past_alerts"] = "Allarmi Precedenti",
    ["past_flow"] = "di Flussi Passati",
    ["purge_all_alerts"] = "Elimina Tutti gli Allarmi",
    ["purge_num_alerts"] = "Elimina [%{num_alerts} allarmi]",
    ["purge_subj_alerts"] = "Elimina Allarmi %{subj}",
    ["purge_subj_alerts_confirm"] = "Vuoi veramente eliminare tutti gli Allarmi %{subj}?",
    ["release_alert"] = "Rilascia Allarme",
    ["release_alert_action"] = "Rilascia",
    ["remove_filters"] = "Rimuovi Filtri",
    ["reset_alert_database"] = "Svuota i Dati degli Allarmi",
    ["reset_alert_database_message"] = "Questa azione rimuoverà ogni allarme ed i relativi dati, inclusi<ul><li>Stato degli allarmi in esecuzione</li><li>Configurazione allarmi</li><li>Archiviazione persistente allarmi su SQLite</li></ul><br>Vuoi veramente continuare",
    ["short_ack"] = "Ack",
    ["srv_country"] = "Paese Server",
    ["startup"] = "Avvio",
    ["stored_alerts"] = "Allarmi Salvati",
    ["trigger_alert_descr"] = "Abilita l'emissione di allarmi su questo allarmabile",
    ["trigger_host_alert_descr"] = "Emetti allarmi generati da %{host}",
    ["trigger_iface_alert_descr"] = "Emetti allarmi generati da %{iface}",
    ["trigger_network_alert_descr"] = "Emetti allarmi generati da %{network}",
    ["presets"] = {
      ["30_min"] = "Ultimi 30 min",
      ["5_min"] = "Ultimi 5 min",
      ["all"] = "Da Sempre",
      ["day"] = "Da Ieri",
      ["hour"] = "Scorsa Ora",
      ["month"] = "Ultimo Mese",
      ["presets"] = "Presets",
      ["week"] = "Ultima Settimana",
      ["year"] = "Ultimo Anno",
    },
  },
  ["sites_page"] = {
    ["contacts"] = "Contatti",
    ["current_sites"] = "Siti Attuali",
    ["last_5_minutes_sites"] = "Siti negli Ultimi 5 Minuti",
    ["sites"] = "Siti",
    ["time_range"] = "Range Temporale:",
    ["top_sites_not_enabled_message"] = "To keep track of visited sites, enable the creation of top visited web sites for local hosts from the <a href=\"%{url}\"><i class=\"fas fa-flask\"></i> Preferences</a>.",
    ["top_sites_not_seen"] = "Current local host has not visited any website yet.",
    ["top_visited_sites"] = "Siti Più Visitati",
  },
  ["snmp"] = {
    ["AlertDescription"] = "Descrizione Allarme",
    ["ConsoleDisplayBufferText"] = "Testo Visualizzato",
    ["GeneralSerialNumber"] = "Numero Seriale",
    ["InputCurrentLevel"] = "Livello Corrente",
    ["InputDimUnit"] = "Unità Dim",
    ["InputMaxCapacity"] = "Capacità Massima",
    ["InputMediaDimFeedDirDeclared"] = "InputMediaDimFeedDirDeclared",
    ["InputMediaDimXFeedDirDeclared"] = "InputMediaDimXFeedDirDeclared",
    ["InputName"] = "InputName",
    ["MarkerColorantValue"] = "Colore Toner",
    ["MarkerLifeCount"] = "Tempo di Vita Toner",
    ["MarkerSuppliesDescription"] = "Descrizione Fornitura Toner",
    ["MarkerSuppliesLevel"] = "Livello Fornitura Toner",
    ["MarkerSuppliesMaxCapacity"] = "Massima Capacità Fornitura Toner",
    ["PrinterCurrentStatus"] = "Stato Corrente",
    ["PrinterOperationalStatus"] = "Stato Operazionale",
    ["absolute_stats"] = "Statistiche dal Boot",
    ["actual_stats"] = "Statistiche dall\\'ultimo Pool",
    ["add_device"] = "Aggiungi Nuovo Dispositivo",
    ["add_devices"] = "Aggiungi dispositivi",
    ["add_snmp_devices"] = "Aggiungi Dispositivi SNMP",
    ["alert_topology"] = "Nessun nodo da visualizzare!",
    ["alerted_interfaces"] = "Interfacce con Errori",
    ["all_interfaces"] = "Tutte le interfacce",
    ["all_snmp_interface_types"] = "Titti i Tipi",
    ["all_snmp_statuses"] = "Tutti gli Stati",
    ["and_one_more_interface"] = "e <a href='%{url}'>un'altra interfaccia</a>",
    ["authentication_passphrase"] = "PassPhrase",
    ["authentication_protocol"] = "Protocol",
    ["bound_interface_description"] = "Associare un'interfaccia di rete ad un'interfaccia SNMP è utile per comparare il traffico di rete monitorato da ntopng con quello riportato da SNMP",
    ["caching_in_progress"] = "La walk SNMP è in corso. Questo potrebbe richiedere fino a 5 minuti. La pagina si aggiornerà in automatico.",
    ["contact"] = "Contatto",
    ["cpuIdle"] = "CPU Idle",
    ["cpuSystem"] = "CPU System",
    ["cpuUser"] = "CPU User",
    ["delete_all_devices"] = "Elimina Tutti i Dispositivi",
    ["delete_unresponsive_devices"] = "Elimina Tutti i Dispositivi che non rispondono",
    ["device_example"] = "Esempio: %{example}",
    ["device_ip"] = "IP del Dispositivo",
    ["device_model"] = "Modello del Dispositivo",
    ["device_polling"] = "Polling del Dispositivo",
    ["device_polling_desc"] = "Esegue walk SNMP periodice per collezionare i dati del dispositivo, inclusi il traffico e lo stato delle interfacce di rete.",
    ["device_responsiveness"] = "Filtro Dispositivi",
    ["device_status"] = "Stato del Dispositivo",
    ["devices_added"] = "Dispositivo/i aggiunti",
    ["disable_port_monitor"] = "Disabilita Monitor Porte",
    ["discover_devices"] = "Ricerca Dispositivi",
    ["edit_device"] = "Modifica",
    ["edit_snmp_devices"] = "Modifica Dispositivo SNMP",
    ["enable_port_monitor"] = "Disabilita Monitor Porte",
    ["errors_filter"] = "Filtra Errori",
    ["ifName"] = "Nome Interfaccia",
    ["ifindex"] = "Indice Interfaccia",
    ["in_bytes"] = "Bytes In",
    ["in_discards"] = "Scartati In",
    ["in_errors"] = "Errori In",
    ["interface_device"] = "Interfaccia %{interface} [ %{device} ]",
    ["interfaces_with_discards"] = "Interfacce con Discard",
    ["interfaces_with_errors"] = "Interfacce con Errori",
    ["interfaces_with_errors_or_discards"] = "Interfacce con Errori/Discard",
    ["last_change"] = "Ultimo Cambiamento",
    ["lldpRemChassisId"] = "Device Remota",
    ["lldpRemPortDesc"] = "Descrizione Porta Remota",
    ["lldpRemPortId"] = "Identificatore Porta Remota",
    ["lldpRemSysDesc"] = "Descrizione Sistema Remoto",
    ["lldpRemSysName"] = "Nome Sistema Remoto",
    ["lldp_topology_changed"] = "Topologia LLDP Cambiata",
    ["lldp_topology_changed_description"] = "Rileva cambiamenti nella topologia di rete ed emette allarmi",
    ["lldp_topology_changed_title"] = "Monitor Topologia LLDP",
    ["low_port_speed"] = "Porta con Velocità Bassa",
    ["memAvailReal"] = "Memoria Disponibile",
    ["memTotalReal"] = "Memoria Totale",
    ["memTotalSwap"] = "Swap Totale",
    ["multiple_macs_non_trunk"] = "MAC multipli su una porta non di trunk",
    ["no_answer"] = "Nessuna risposta dall'host %{host}: assicurasi che SNMP sia attivo e che la comunità impostata sia corretta",
    ["no_host_discovered"] = "Nessun host trovato in %{host}/%{cidr}",
    ["no_polling_yet"] = "Non ancora",
    ["none"] = "Nessuno",
    ["note"] = "NOTE",
    ["object_id"] = "ObjectID",
    ["out_bytes"] = "Bytes Out",
    ["per_interface_settings"] = "Queste impostazioni si applicano solamente a \"%{iface}\". Controllare <a href=\"%{url}\">%{global}</a> per la configurazione globale.",
    ["ping_all_snmp_devices"] = "Esegui un Ping a tutti i dispositivi SNMP",
    ["ping_all_snmp_devices_confirm"] = "Vuoi veramente eseguire un ping a tutti i dispositivi SNMP?",
    ["ping_devices"] = "Ping ai Dispositivi",
    ["port_macs"] = "MAC della Porta",
    ["port_macs_short"] = "MACs",
    ["privacy_passphrase"] = "PassPhrase Privacy",
    ["privacy_protocol"] = "Protocollo Privacy",
    ["prune_unresponsive_snmp_devices"] = "Rimuovi dispositivi SNMP non raggiungibili",
    ["prune_unresponsive_snmp_devices_confirm"] = "Eliminare dispositivi SNMP non raggiungibili?",
    ["remove_all_snmp_devices"] = "Rimuovi tutti i Dispositivi SNMP",
    ["remove_all_snmp_devices_confirm"] = "Vuoi veramente eliminare tutti i dispositivi SNMP?",
    ["remove_snmp_device"] = "Rimuovi Dispositivo SNMP",
    ["remove_snmp_device_confirm"] = "Vuoi veramente eliminare il dispositivo SNMP %{device}?",
    ["responsiveness_all"] = "Tutti i dispositivi",
    ["responsiveness_responsive"] = "Dispositivi Raggiungibili",
    ["responsiveness_unresponsive"] = "Dispositivi non Raggiungibili",
    ["select_pool"] = "Seleziona un Pool",
    ["similarity"] = "Score Somiglianza",
    ["snmp_all_interfaces"] = "Tutte le Interfacce",
    ["snmp_also_seen_on"] = "Visto anche su",
    ["snmp_and_n_more_interfaces"] = "e <a href='%{url}'>%{count} altre interfacce</a>",
    ["snmp_authnopriv"] = "Auth. senza Privacy",
    ["snmp_authpriv"] = "Auth. con Privacy",
    ["snmp_avg_traffic"] = "Traffico Medio",
    ["snmp_community"] = "Comunità",
    ["snmp_device"] = "Dispositivo SNMP",
    ["snmp_device_does_not_respond"] = "Il dispositivo SNMP non ripsonde.",
    ["snmp_device_last_poll"] = "Tempo dall'Ultimo Poll",
    ["snmp_device_last_poll_duration"] = "Durata Ultimo Poll",
    ["snmp_device_polling_disabled"] = "Il polling periodico del dispositivo è disabilitato. Le informazioni visualizzate potrebbero essere parziali o vecchie. Il polling può essere abilitato dalle <a href=\"%{url}\"><i class=\"fas fa-flask\"></i> Preferenze</a>.",
    ["snmp_device_x"] = "%{device}",
    ["snmp_devices"] = "Dispositivi SNMP",
    ["snmp_error_unable_to_set_interface_admin_status"] = "Impossibile cambiare l'admin status. Controllare che il dispositivo SNMP abbia una write community valida settata.",
    ["snmp_host"] = "Host",
    ["snmp_interface"] = "Interfaccia SNMP",
    ["snmp_interface_type"] = "Tipo Interfaccia",
    ["snmp_interface_x"] = "Interfaccia: %{interface}",
    ["snmp_interfaces"] = "Interfacce",
    ["snmp_last_poll_errors"] = "Interfacce con Errori (Ultimo Poll)",
    ["snmp_level"] = "Livello Sicurezza",
    ["snmp_lldp_cdp_descr"] = "Le informazioni sulla topologia del dispositivo sono disponibili quando  <A HREF=\\\"https://en.wikipedia.org/wiki/Link_Layer_Discovery_Protocol\\\">LLDP</A> / <A HREF=\\\"https://en.wikipedia.org/wiki/Cisco_Discovery_Protocol\\\">CDP</A> MIB è presente nel dispositivo monitorato.",
    ["snmp_lldp_cdp_no_node"] = "Il nodo selezionato non è stato interrogato tramite SNMP",
    ["snmp_lldp_cdp_node_color"] = "Il colore del nodo rappresenta il modello del dispositivo.",
    ["snmp_lldp_cdp_zoom_descr"] = "Fare clic su un nodo per esplorare il dispositivo, utilizzare la rotellina del mouse per eseguire lo zoom, trascinare facendo clic su un'area vuota.",
    ["snmp_monitor_disable_confirm"] = "Vuoi veramente disabilitare il monitor delle porte del dispositivo SNMP %{device}?",
    ["snmp_monitor_enable_confirm"] = "Vuoi veramente abilitare il monitor delle porte del dispositivo SNMP %{device}? In tal modo verranno generati allarmi non appena una porta cambia di stato.",
    ["snmp_netsnmp"] = "CPU/Memoria",
    ["snmp_noauthnopriv"] = "Nessuno",
    ["snmp_note_bridge_mib"] = "Vengono eseguite delle walk SNMP periodiche sul Bridge MIB per popolare la tabella.",
    ["snmp_note_bridge_mib_status"] = "Lo stato visualizzato è lo dot1qTpFdbStatus Bridge MIB status associato al dispositivo MAC.",
    ["snmp_note_periodic_interfaces_polling"] = "Ad intervalli regolari di 5 minuti vengono eseguite delle walk SNMP per aggiornare le interfacce.",
    ["snmp_note_thpt_calc"] = "Il throughput prende in considerazione il traffico totale traffic (entrata + uscita) tra due walk SNMP consecutive più recenti.",
    ["snmp_note_thpt_span"] = "Le due walk SNMP consecutive più recenti sono state eseguite %{last} e %{prev}.",
    ["snmp_pool"] = "Pool",
    ["snmp_port_monitor"] = "Monitor delle porte SNMP",
    ["snmp_printer"] = "Stampante",
    ["snmp_read_community"] = "Read Community",
    ["snmp_similarity_note"] = "<b>Più è alto</b> il valore della colonna somiglianza, <b>più simili</b> sono le interfacce network.",
    ["snmp_similarity_time_note"] = "La somiglianza è basata sul traficco dell'interfaccia (RX + TX) delle scorse 24 ore.",
    ["snmp_timeseries_descr"] = "Le serie temporali per i dispositivi SNMP possono essere abilitate dalle <a href=\"%{url}\"><i class=\"fas fa-flask\"></i> Preferenze</a>. Sono necessari alcuni minuti per iniziare a vedere i primi dati",
    ["snmp_topology"] = "Topologia",
    ["snmp_username"] = "Username",
    ["snmp_version"] = "Versione SNMP",
    ["snmp_write_community"] = "Write Community",
    ["stats_since"] = "Statistiche Da",
    ["status_dormant"] = "Dormant",
    ["status_down"] = "Inattivo",
    ["status_lowerlayerdown"] = "Lower Layer Down",
    ["status_notpresent"] = "Non Presente",
    ["status_unknown"] = "Sconosciuto",
    ["status_up"] = "Attivo",
    ["status_up_in_use"] = "Up (in uso)",
    ["testing"] = "In Test",
    ["tp_fbd_status_invalid"] = "invalido",
    ["tp_fbd_status_learned"] = "appreso",
    ["tp_fbd_status_mgmt"] = "mgmt",
    ["tp_fbd_status_other"] = "altro",
    ["tp_fbd_status_self"] = "self",
    ["traffic_similarity"] = "Somiglianza del traffico delle interfacce di rete",
    ["trunk"] = "trunk",
    ["unable_to_delete_snmp_device"] = "Impossibile eliminare il dispositivo SNMP %{device}",
    ["unresponsive_device"] = "La walk SNMP è troppo lenta. I dati visualizzati potrebbero essere vecchi o parziali. Controllare la connettività di rete e la responsività del dispositivo. Un nuovo tentativo verrà eseguito entro 15 minuti.",
    ["view_device"] = "Visualizza Dispositivo",
    ["descriptions"] = {
      ["add_pool"] = "Nuovo Pool SNMP",
      ["authentication_passphrase"] = "SNMPv3 authentication passphrase",
      ["authentication_protocol"] = "SNMPv3 authentication protocol",
      ["host"] = "IP del dispositivo SNMP",
      ["level"] = "SNMPv3 security level",
      ["pool"] = "Pool SNMP da associare",
      ["privacy_passphrase"] = "SNMPv3 encryption passphrase",
      ["privacy_protocol"] = "SNMPv3 encryption protocol",
      ["read_community"] = "SNMP v1/v2c read-only community",
      ["username"] = "SNMPv3 authentication username",
      ["version"] = "Versione protocollo SNMP",
      ["write_community"] = "SNMP v1/v2c write community",
    },
    ["validation"] = {
      ["abort_add_snmp_devices"] = "Annullare?",
      ["abort_edit_snmp_devices"] = "Annullare?",
      ["invalid_host"] = "Immettere un indirizzp IPv4/IPv6 valido.",
      ["read_community_empty"] = "La Read Community non può essere vuota.",
    },
  },
  ["snmp_page"] = {
    ["guide_snmp_page_message"] = "Visita la pagina <a href='%{url}'> SNMP</a> per aggiungere questo host alla lista di dispositivi SNMP configurati.",
    ["not_configured_as_snmp_device_message"] = "L'host %{host_ip} non è stato configurato come un dispositivo SNMP.",
    ["showing_snmp_mib_info_default_community_message"] = "Visualizzazione delle informazioni MIB SNMP recuperate utilizzando la comunità predefinita '%{community}'",
    ["snmp_device_interface_not_found"] = "Dispositivo SNMP  \\\"%{host}\\\" interfaccia \\\"%{iface}\\\" non può essere trovata: forse è stata rimossa dalla lista di dispositivi SNMP configurati?",
    ["snmp_device_not_found"] = "Dispositivo SNMP \"%{host}\" non trovato: è stato rimosso dalla lista di dispositivi configurati?",
    ["trying_to_retrive_message"] = "In attesa di recuperare l'host SNMP MIB utilizzando la comunità predefinita '%{community}'",
    ["unable_to_retrive_snmp_default_community_message"] = "Impossibile recuperare MIB SNMP host utilizzando la comunità predefinita '%{community}'.",
  },
  ["snmp_ratio_errors"] = {
    ["flowdevice_timeseries_not_existing"] = "Non disponibile.",
    ["flowdevice_timeseries_query_failed"] = "Serie temporale non leggibile.",
    ["nan_ratio"] = "Ratio non è un numero.",
    ["negative_ratio"] = "Ratio negativo.",
    ["ok"] = "OK",
    ["snmp_timeseries_not_enabled"] = "Serie temporale disabilitata.",
    ["snmp_timeseries_not_existing"] = "Non disponibile.",
    ["snmp_timeseries_query_failed"] = "Serie temporale non leggibile.",
    ["snmp_timeseries_zero_total"] = "Nessun dato.",
    ["timeseries_start_time_mismatch"] = "Serie non allineate.",
  },
  ["sprobe_page"] = {
    ["active_memory"] = "Memoria attiva",
    ["latency"] = "Latenza",
    ["processes"] = "Processi",
    ["processes_traffic_tree"] = "Albero dei Processi di Traffico",
    ["show_more_info"] = "Posiziona sopra il puntatore del mouse per mostare l'informazione del processo o fai doppio click per mostrare altre informazioni.",
    ["system_interactions"] = "Interazioni del Sistema",
    ["top_processes"] = "Top Processi",
    ["top_users"] = "Top Utenti",
    ["tree"] = "Albero",
    ["users"] = "Utenti",
  },
  ["sub_interfaces"] = {
    ["add_sub_if"] = "Aggiungi Sotto-Interfaccia",
    ["add_sub_if_success"] = "Sotto-Interfaccia creata con successo. Riavviare ntopng affinché il cambiamento abbia effetto.",
    ["collection_filter_description"] = "Possono essere usati campi NetFlow come INPUT_SNMP, OUTPUT_SNMP, SRC_VLAN, DST_VLAN. Valori non-numerici possono essere forniti con apici singoli.",
    ["confirm_delete"] = ">Eliminare %{name}?",
    ["delete"] = "Elimina Sotto-Interfaccia",
    ["disaggregation"] = "Disaggregazione Custom del Traffico",
    ["empty_filter"] = "Filtro non specificato",
    ["filter_description"] = "Il formato esteso nBPF è supportato: oltre alla normale sintassi BPF, sono supportati anche i campi NetFlow, inclusi INPUT_SNMP, OUTPUT_SNMP, SRC_VLAN, DST_VLAN. Valori non numerici devono essere specificati all'interno di apici singoli.",
    ["filter_examples"] = "Esempi di Filtro",
    ["invalid_name_format"] = "Il nome dell'interfaccia deve contenere solo lettere, numeri, trattini.",
    ["invalid_name_length"] = "Nome interfaccia troppo breve (5 o più caratteri)",
    ["invalid_xnbpf"] = "Filtro nBPF non valido",
    ["note"] = "ntopng può usare filtri simil-BPF per disaggregare il traffico in ingresso, redirigendolo verso sotto-interfacce. Lo stesso traffico piò essere rediretto verso più interfacce quando i filtri si sovrappongono.",
    ["sub_if_name"] = "Nome Sotto-Interfaccia",
    ["traffic_filter_nbpf"] = "Filtro Traffico (Formato nBPF)",
    ["traffic_filter_xnbpf"] = "Filtro Traffico (Formato nBPF esteso)",
  },
  ["syslog"] = {
    ["add"] = "Aggiungi Produttore Log",
    ["bad_action_param"] = "\"action\" non valida",
    ["confirm_delete"] = "Eliminare questo produttore?",
    ["edit"] = "Modifica Produttore Log",
    ["empty_action"] = "Qualcosa è andato storto (azione vuota). Ricaricare la pagina e riprovare.",
    ["host_add_ok"] = "Host %{host} aggiunto con successo!",
    ["host_delete_ok"] = "Host %{host} eliminato con successo!",
    ["host_edit_ok"] = "Host %{host} modificato con successo!",
    ["host_exists"] = "Host \"%{host}\" già esistente",
    ["host_not_exists"] = "Host \"%{host}\" non trovato",
    ["ip_or_device"] = "IP o Nome Device",
    ["ip_or_device_note"] = "L'IP del Produttore di Log deve corrispondere all'IP o al Nome Device come appare nel messaggio syslog. Esempio:<br />Thu Apr 10 16:30:00 2020;<strong>10.0.0.1</strong>; <181>Thu Apr 10 16:30:00 <strong>sslvpn-device-01</strong> logserver: ...",
    ["producer_type"] = "Registro Tipo del Produttore",
    ["producers"] = "Produttori Log Syslog",
    ["syslog"] = "Syslog",
  },
  ["system_alerts_stats"] = {
    ["internal_alerts_queue"] = "Coda Interna",
    ["notes_enq_deq"] = "'Fill Level' è il rapporto di allarmi accodati rispetto alla grandezza totale della coda. 'Dropped' è il rapporto di allarmi persi, rispetto al numero totale di allarmi  generati",
    ["notes_internal_alerts_queue"] = "La coda interna è usata, in combinazione con SQLite, per gestire gli alert generati dal motore",
    ["queues_status"] = "Fill Level / Dropped",
  },
  ["system_interfaces_stats"] = {
    ["local_hosts"] = "Host Locali",
    ["remote_hosts"] = "Host Remoti",
  },
  ["system_stats"] = {
    ["cpu_states_monitor"] = "Monitor degli Stati della CPU",
    ["cpu_states_monitor_description"] = "Monitor degli stati della CPU (e.g., user, system, idle and iowait)",
    ["disk_monitor"] = "Monitor del Disco",
    ["disk_monitor_description"] = "Monitor dello spazio libero",
    ["dropped_points"] = "Punti Persi",
    ["export_failures"] = "Esportazioni Fallite",
    ["export_retries"] = "Esportazioni Riprovate",
    ["exported_points"] = "Punti Esportati",
    ["exported_vs_dropped_points"] = "Punti Totali vs Persi",
    ["exports"] = "Esportazioni Totali",
    ["exports_label"] = "Esportazioni",
    ["health"] = "Stato",
    ["high_series_cardinality"] = "Rallentamenti possono essere sperimentati a seguito di un'alta cardinalità",
    ["infludb_retried_vs_failed_exports"] = "Esportazioni riprovate/fallite",
    ["influxdb_health_green"] = "OK",
    ["influxdb_health_green_descr"] = "L'esportazione su InfluxDB sta funzionando correttamente.",
    ["influxdb_health_red"] = "Esportazione fallita.",
    ["influxdb_health_red_descr"] = "InfluxDB irraggiungibile o troppo lento, non riesce ad ingerire tutti i dati. Alcuni punti verranno persi.",
    ["influxdb_health_yellow"] = "Temporaneamente impossibile esportare a InfluxDB",
    ["influxdb_health_yellow_descr"] = "Impossibile esportare a InfluxDB. Nuovo tentativo in corso.",
    ["influxdb_note_docs"] = "Documentazione con le descrizioni dettagliate di tutti i campi è disponibile nella <a target='_blank' href='%{url}'>guida</a> di ntopng.",
    ["influxdb_storage"] = "Spazio Disco",
    ["last_ip"] = "Ultimo IP",
    ["series_cardinality"] = "Cardinalità",
    ["short_desc_influxdb_cardinality"] = "Indicazione di quanto è complessa la gestione dei punti da parte di InfluxDB.",
    ["short_desc_influxdb_dropped_points"] = "Numero totale di punti persi a seguito di scritture fallite su InfluxDB.",
    ["short_desc_influxdb_exported_points"] = "Numero totale di punti scritti con successo su InfluxDB.",
    ["short_desc_influxdb_exports"] = "Numero totale di scritture fatte con successo su InfluxDB",
    ["short_desc_influxdb_health"] = "Stato attuale dell'export a InfluxDB.",
    ["short_desc_influxdb_ram_memory"] = "Memoria totale che InfluxDB ha ottenuto dal sistema operativo.",
    ["short_desc_influxdb_storage_utilization"] = "Spazio disco totale usato da InfluxDB.",
    ["value"] = "Valore",
    ["write_througput"] = "Throughput di Scrittura",
    ["clickhouse"] = {
      ["clickhouse_alert_size"] = "Dimensioni delle tabelle dell'allarme",
      ["clickhouse_flow_size"] = "Dimensione della tabella dei Flussi",
      ["clickhouse_health_green"] = "OK",
      ["clickhouse_health_green_descr"] = "ClickHouse sta funzionando correttamente.",
      ["clickhouse_health_red"] = "Problemi di persistenza",
      ["clickhouse_health_red_descr"] = "Impossibile fare un query a ClickHouse. Aspettarsi che %{product} abbia un malfunzionamento.",
      ["clickhouse_monitor"] = "Monitor di ClickHouse",
      ["clickhouse_monitor_description"] = "Monitora la salute di ClickHouse",
      ["clickhouse_system_errors"] = "Ultimi Errori",
      ["clickhouse_system_errors_no_error"] = "Nessun errore rilevato.",
      ["command_hits"] = "%{cmd} Chiamate",
      ["commands"] = "Comandi",
      ["short_desc_clickhouse_alert_size"] = "Dimensione totale delle tabelle di avviso di ClickHouse.",
      ["short_desc_clickhouse_flows_size"] = "Dimensione totale delle tabelle dei flussi di ClickHouse.",
      ["short_desc_clickhouse_health"] = "Salute attuale di ClickHouse.",
      ["tables_size"] = "Dimensione Tabelle",
      ["tables_size_descr"] = "Dimensione totale delle tabelle ClickHouse, sia i flussi che le tabelle degli avvisi",
      ["top_clickhouse_commands"] = "Top Comandi Chiamate",
      ["tot_calls"] = "Chiamate Totali",
    },
    ["redis"] = {
      ["command_hits"] = "Chiamate %{cmd}",
      ["commands"] = "Comandi",
      ["redis_health_green"] = "OK",
      ["redis_health_green_descr"] = "Redis sta funzionando correttamente.",
      ["redis_health_red"] = "Problemi di Persistenza",
      ["redis_health_red_descr"] = "Redis non è in grado di rendere i dati persistenti. Ci saranno malfunzionamenti di %{product}.",
      ["redis_keys"] = "Chiavi del Database",
      ["redis_monitor"] = "Monitor di Redis",
      ["redis_monitor_description"] = "Monitor dello stato di Redis",
      ["short_desc_redis_health"] = "Stato Attuale di Redis.",
      ["short_desc_redis_keys"] = "Totale chiavi nel database di Redis.",
      ["short_desc_redis_ram_memory"] = "Total della memoria che Redis ha ottenuto dal sistema operativo.",
      ["top_redis_commands"] = "Top Chiamate",
      ["tot_calls"] = "Total Chiamate",
    },
  },
  ["telemetry_page"] = {
    ["crash_report"] = "Crash Report",
    ["crash_report_descr"] = "Un crash report è spedito quando %{product} termina in modo anomalo. Un crash report contiene informazioni quali la versione di %{product}, l'architettura di build, il sistema operativo, e le opzioni di startup. Niente riguardo al traffico monitorato viene spedito. Di seguito un esempio di crash report:",
    ["last_data_sent"] = "Ultimi Dati Trasmessi",
    ["send_telemetry_data"] = "Contribuisci trasmettendo dati di telemetria",
    ["telemetry_data"] = "Dati di Telemetria",
    ["telemetry_data_change_preference"] = "<a href='%{url}'>cambia</a>",
    ["telemetry_data_no_consent"] = "Preferenza non espressa",
  },
  ["top_host_sites"] = {
    ["0"] = "0:00 - 0:59",
    ["1"] = "1:00 - 1:59",
    ["10"] = "10:00 - 10:59",
    ["11"] = "11:00 - 11:59",
    ["12"] = "12:00 - 12:59",
    ["13"] = "13:00 - 13:59",
    ["14"] = "14:00 - 14:59",
    ["15"] = "15:00 - 15:59",
    ["16"] = "16:00 - 16:59",
    ["17"] = "17:00 - 17:59",
    ["18"] = "18:00 - 18:59",
    ["19"] = "19:00 - 19:59",
    ["2"] = "2:00 - 2:59",
    ["20"] = "20:00 - 20:59",
    ["21"] = "21:00 - 21:59",
    ["22"] = "22:00 - 22:59",
    ["23"] = "23:00 - 23:59",
    ["3"] = "3:00 - 3:59",
    ["4"] = "4:00 - 4:59",
    ["5"] = "5:00 - 5:59",
    ["6"] = "6:00 - 6:59",
    ["7"] = "7:00 - 7:59",
    ["8"] = "8:00 - 8:59",
    ["9"] = "9:00 - 9:59",
    ["current"] = "Attuale",
    ["day"] = "Ultimo Giorno",
    ["last_5_min"] = "Ultimi 5 minuti",
    ["note_protocol_used"] = "Protocolli applicazione che danno info: HTTP, TLS, QUIC, DNS.",
    ["note_time_range"] = "L'intervallo di tempo è ordinato dai dati correnti ai dati più vecchi conosciuti.",
    ["note_tot_entries"] = "Sono forniti al massimo 20 record per host.",
    ["note_yesterday_data"] = "I dati vengono conservati al massimo per 24 ore, il che significa che al massimo si possono vedere i dati di ieri. Per questo motivo qualsiasi ora nel menu successiva all'ora corrente è dai dati di ieri.",
  },
  ["top_hosts"] = {
    ["note"] = "<b>Nota</b><br>Questa pagina si aggiorna ogni pochi secondi per riportare le attività degli host principali.<p>Quando un host genera del traffico, viene visualizzata una barra verticale in modo da poter vedere il host più attivi.",
    ["top_hosts_local"] = "Top Host (Locali)",
  },
  ["topk_heuristic"] = {
    ["precision"] = {
      ["aggressive"] = "Aggressivo",
      ["disabled"] = "Disabilitato",
      ["less_accurate"] = "Meno Preciso",
      ["more_accurate"] = "Più Preciso",
    },
  },
  ["traffic_behaviour"] = {
    ["allowed"] = "Permesso",
    ["denied"] = "Non permesso",
    ["learning"] = "Apprendimento",
    ["undecided"] = "Non deciso",
  },
  ["traffic_page"] = {
    ["flow_distribution"] = "Distribuzione Flussi",
    ["flow_duration"] = "Durata Totale Flussi",
    ["flow_frequency"] = "Frequenza Creazione Nuovi Flussi",
    ["hosts_contacts_cardinality"] = "Numero di Contatti",
    ["l4_proto_overview"] = "Panoramica Protocolli L4",
    ["no_traffic_observed_message"] = "Nessun traffico è stato osservato per l'host in questione",
    ["num_contacted_hosts_as_client"] = "Host Contattati (come Client)",
    ["num_host_contacts_as_server"] = "Host Contattati (come Server)",
    ["rcvd"] = "Rcvt",
  },
  ["traffic_profiles"] = {
    ["advanced_filter_examples"] = "Esempi di filtri avanzati",
    ["all_profiles"] = "Tutti i Profili",
    ["confirm_delete_profile"] = "Vuoi veramente eliminare il profilo",
    ["delete_profile"] = "Elimina profilo",
    ["destination_network"] = "Rete di Destinazione",
    ["duplicate_profile"] = "Nome profilo duplicato",
    ["edit_traffic_profiles"] = "Modifica i Profili di Traffico",
    ["enter_profile_filter"] = "Inserisci il filtro del profilo",
    ["enter_profile_name"] = "Inserisci il nome del profilo",
    ["facebook_traffic"] = "Facebook",
    ["host_http_https"] = "Host HTTP/HTTPS",
    ["host_traffic"] = "Host",
    ["http_traffic"] = "HTTP",
    ["invalid_bpf"] = "Filtro BPF non valido",
    ["max_profiles_num"] = "Con questa versione di ntopng puoi creare fino a %{maxnum} profili. Aggiorna alla versione Enterprise per rimuovere questo limite",
    ["no_profiles"] = "Nessun profilo impostato",
    ["note"] = "Nota",
    ["note_0"] = "Nei nomi dei profili di traffico sono permessi caratteri alfanumerici, spazio e trattino basso",
    ["note_1"] = "I profili del traffico sono applicati ai flussi. Ogni flusso può avere fino ad un profilo. In caso di corrispondenza con profili multipli, viene selezionato solo il primo",
    ["profile_name"] = "Nome Profilo",
    ["see_ndpi_protos"] = "(utilizza %{option} per vedere tutti i protocolli nDPI supportati)",
    ["simple_filter_examples"] = "Esempi di filtri semplici",
    ["source_ethernet"] = "Indirizzo MAC Sorgente",
    ["traffic_between"] = "Traffico tra hosts",
    ["traffic_filter_bpf"] = "Filtro sul Traffico (Formato nBPF)",
    ["traffic_from_to"] = "Traffico da un host a un altro",
    ["traffic_profiles"] = "Profili di Traffico",
  },
  ["traffic_recording"] = {
    ["about_to_download_flow"] = "In procinto di estrarre traffico da <b>%{date_begin}</b> a <b>%{date_end}</b> corrispondente al flusso selezionato. Verrà scaricato un file PCAP.",
    ["about_to_extract"] = "Estrazione del traffico da <b>%{date_begin}</b> a <b>%{date_end}</b>",
    ["active_since"] = "Attivo Da",
    ["archive"] = "Archivia",
    ["completed"] = "Completato",
    ["continuous_recording"] = "Registrazione Continua del Traffico",
    ["continuous_recording_and_flows"] = "Registrazione Continua del Traffico e Visualizzazione Flussi",
    ["delete_all_jobs"] = "Cancella Tutte le Estrazioni",
    ["delete_all_jobs_confirm"] = "Vuoi veramente cancellare tutte le estrazioni? Eventuali estrazioni attive saranno fermate.",
    ["delete_job"] = "Cancella Estrazione",
    ["delete_job_confirm"] = "Vuoi veramente cancellare l'estrazione %{job_id} e i relativi file Pcap?",
    ["disabled"] = "Disabilitato",
    ["disk_space"] = "Spazio Massimo su Disco",
    ["disk_space_note"] = "Spazio massimo da usare per il traffico registrato su disco.",
    ["disk_space_note_in_use"] = "%{in_use} GB sono già in uso da questa istanza.",
    ["download"] = "Scarica",
    ["download_nth_pcap"] = "Scarica Pcap #{0}",
    ["dump_window"] = "Finestra di Registrazione",
    ["edit_filter"] = "Modifica Filtro",
    ["err_alloc"] = "errore allocazione memoria",
    ["err_filter"] = "filtro non supportato",
    ["err_open"] = "impossibile leggere i dati",
    ["err_stuck"] = "bloccato o interrotto",
    ["err_unknown"] = "errore inatteso",
    ["ext_interface"] = "Interfaccia Esterna",
    ["extract_action"] = "Estrai",
    ["extract_now"] = "Estrai ora",
    ["extracted_bytes"] = "Byte",
    ["extracted_packets"] = "Pacchetti",
    ["extraction_scheduled"] = "Estrazione schedulata, puoi controllare la pagina %{page} per scaricare il pcap appena pronto. ID per questa estrazione: ",
    ["failed"] = "Fallito",
    ["failure"] = "Fallimento",
    ["failure_note"] = "Preghiamo di controllare i log e provare a riavviare il servizio.",
    ["filter_bpf"] = "Filtro (formato nBPF)",
    ["filter_examples"] = "Esempi di filtri",
    ["global_settings_note"] = "Le impostazioni globali (ad es. la licenza) possono essere configurate dalle <a href=\"%{url}\">Preferenze di Registrazione del Traffico</a>.",
    ["invalid_bpf"] = "Filtro non valido",
    ["job_date_time"] = "Data/Ora Estrazione",
    ["job_id"] = "ID Estrazione",
    ["jobs"] = "Estrazioni",
    ["logs"] = "Log della Registrazione del Traffico",
    ["max_extracted_pcap_bytes_description"] = "Dimensione dei file pcap estratti dai dati registrati. Nel caso un'estrazione dovesse superare questa dimensione, il traffico estratto sarà suddiviso in più file.",
    ["max_extracted_pcap_bytes_title"] = "Dimensione Massima File Estratti",
    ["missing_data_msg"] = "La registrazione del traffico è stata fermata o riavviata. Alcuni dati potrebbero mancare.",
    ["missing_parameters"] = "Mancano dei parametri obbligatori",
    ["msg_err_missing_timeline"] = "Impossibile determinare la timeline di n2disk. Le estrazioni sono possibili solo utilizzando una versione aggiornata di n2disk con l'opzione --timeline-dir.",
    ["msg_err_timeline_path_not_existing"] = "Impossibile trovare il percorso della timeline nel sistema.",
    ["msg_err_unable_to_access_timeline"] = "Impossibile leggere i dati registrati. I permessi sono insufficienti. L'utente con cui gira ntopng deve appartenere allo stesso gruppo dell'utente n2disk.",
    ["msg_err_unknown"] = "Stato sconosciuto.",
    ["msg_external_providers_detected"] = "Rilevati provider di traffic recording esterni. Puoi lasciare che ntopng si occupi del traffic recording, oppure puoi <a href=\"%{url}\"><i class=\"fas fa-cog\"></i> configurare</a> un provider esterno.",
    ["msg_ok"] = "Pronto ad estrarre.",
    ["multiple_extracted_files"] = "L'estrazione ha generato più file da %{mb} MB, è possibile cambiare questo limite nelle Preferenze. Clicca sui link sotto per scaricare i file pcap.",
    ["no_data_in_interval"] = "Non è presente traffico registrato su disco (o è parziale) per l'intervallo temporale selezionato. Nota che il traffico in formato pcap viene registrato su disco al minuto (l'ultimo minuto non è disponibile) e il traffico più vecchio eccedente il limite di spazio su disco viene rimosso (puoi controllare la finestra di registrazione corrente nella <A HREF=\"%{page}\">pagina di stato</A>). Ti preghiamo di provare a selezionare un intervallo temporale diverso.",
    ["no_file"] = "Non è ancora presente del traffico registrato",
    ["no_recorded_data"] = "Non è presente del traffico registrato nell'intervallo specificato. Note che la registrazione del traffico ha bisogno di un po di tempo per registrare il traffico recente.",
    ["not_found"] = "File non trovato",
    ["not_granted"] = "Richiesta non permessa",
    ["note_dump"] = "Nelle reti ethernet, ogni pacchetto ha un <A HREF=\"https://en.wikipedia.org/wiki/Ethernet_frame\">overhead di 24 bytes</A> [preambolo, inizio del frame, CRC, IFG]. Questo overhead viene considerato nel calcolo del traffico sull'interfaccia, mentre non viene considerato nel traffico registrato su disco (in quanto non viene registrato). Per questo motivo il numero di byte in questa pagina potrebbe non corrispondere a quello riportato dal grafico dell'interfaccia.",
    ["packet_dumps"] = "Registrazioni Traffico",
    ["pcap_download"] = "Estrai pcap",
    ["pcap_download_message"] = "Avvia l'estrazione e controlla la pagina %{page} per scaricare il pcap appena pronto.",
    ["pcap_extract"] = "Estrai pcap",
    ["processing"] = "Processando",
    ["queue_as_job"] = "Accoda Attività di Estrazione",
    ["recording"] = "Registrando",
    ["restart"] = "Riavvia",
    ["restart_service"] = "Riavvia il servizio",
    ["settings"] = "Impostazioni",
    ["start_extraction"] = "Avvia Estrazione",
    ["stop_job"] = "Ferma Estrazione",
    ["stop_job_confirm"] = "Vuoi veramente fermare l'estrazione %{job_id}?",
    ["stopped"] = "Interrotto",
    ["storage"] = "Storage",
    ["storage_dir"] = "Percorso dello Storage",
    ["storage_directory_config"] = "Il percorso della Cartella dello Storage può essere cambiata specificando l'opzione <i>%{option}</i> nel file di configurazione di %{product}.",
    ["storage_utilization"] = "Utilizzo dello Storage",
    ["storage_utilization_pcap"] = "Utilizzo spazio disco per pcap",
    ["traffic_extraction_jobs"] = "Estrazioni Schedulate",
    ["traffic_extractions"] = "Estrazioni",
    ["traffic_on_disk"] = "Traffico su Disco",
    ["traffic_recording"] = "Registrazione del Traffico",
    ["traffic_recording_provider"] = "Provider Registrazione Traffico",
    ["traffic_recording_settings"] = "Impostazioni Registrazione del Traffico",
    ["traffic_recording_status"] = "Stato della Registrazione del Traffico",
    ["unsupported_fields"] = "Nota che il grafico selezionato fornisce una vista del traffico che potrebbe essere non completamente supportata dal tool di estrazione. Il traffico estratto potrà contenere più traffico di quello atteso.",
    ["waiting"] = "In attesa",
    ["waiting_nospace"] = "In Attesa (Spazio su Disco Non Sufficiente)",
  },
  ["traffic_report"] = {
    ["current_day"] = "Giorno Corrente",
    ["current_month"] = "Mese Corrente",
    ["current_week"] = "Settimana Corrente",
    ["daily"] = "Giornaliero",
    ["header_daily"] = "Report Giornaliero",
    ["header_monthly"] = "Report Mensile",
    ["header_weekly"] = "Report Settimanale",
    ["monthly"] = "Mensile",
    ["previous_day"] = "Giorno Precedente",
    ["previous_month"] = "Mese Precedente",
    ["previous_week"] = "Settimana Precedente",
    ["weekly"] = "Settimanale",
  },
  ["tree_map"] = {
    ["hosts_treemap"] = "Mappa ad Albero degli Host",
  },
  ["unknown_devices"] = {
    ["active_only"] = "Solo Attivi",
    ["all_devices"] = "Tutti i Dispositivi",
    ["assign_device_pool"] = "Assegna un Dispositivo ad un Pool",
    ["assign_pool"] = "Assegna ad un Pool",
    ["device_name"] = "Nome Dispositivo",
    ["devices_only_note"] = "Un dispositivo è considerato non assegnato se non ci sono membri di un host pool con quel MAC.",
    ["filter_devices"] = "Filtra Dispositivi",
    ["first_seen"] = "Visto per la Prima Volta",
    ["inactive_only"] = "Solo Inattivi",
    ["last_seen"] = "Visto per l'Ultima Volta",
    ["no_pools"] = "E' necessario creare un host pool per poter assegnarvi dei dispositivi sconosciuti.",
    ["select_pool"] = "Seleziona una host pool dal dropdown sottostante per assegnarvi il '%{mac}':",
    ["unassigned_devices"] = "Dispositivi Sconosciuti",
  },
  ["updates"] = {
    ["available"] = "Nuovo",
    ["check"] = "Controlla aggiornamenti",
    ["checking"] = "Controllo aggiornamenti in corso..",
    ["install_confirm"] = "Vuoi aggiornare e riavviare ntopng ora?",
    ["installing"] = "Installazione aggiornamento..",
    ["no_updates"] = "Nessun aggiornamento disponibile",
    ["update_failure_message"] = "Qualcosa è andato storto durante l'aggiornamento. Riprovare",
  },
  ["user_activity"] = {
    ["add_ntopng_user"] = "L'utente '%{user}' ha aggiunto un nuovo utente %{new_user}.",
    ["add_pool"] = "L'utente '%{user}' ha creato il nuovo pool <a href=\"%{pool_url}\">%{pool_name}</a>.",
    ["alerts_deleted"] = "L'utente '%{user}' ha rimosso gli avvisi '%{status}'.",
    ["bind_pool_member"] = "L'utente '%{user}' ha aggiunto il membro %{member} al pool <a href=\"%{pool_url}\">%{pool_name}</a>.",
    ["changed_preference"] = "L'utente '%{user}' ha cambiato le impostazioni per %{pref}.",
    ["delete_ntopng_user"] = "L'utente '%{user}' ha eliminato l'utente %{old_user}.",
    ["delete_pool"] = "L'utente '%{user}' ha eliminato il pool <a href=\"%{pool_url}\">%{pool_name}</a>.",
    ["deleted_all_interfaces_data"] = "L'utente '%{user}' ha cancellato i dati di tutte le interfacce.",
    ["deleted_host_data"] = "L'utente '%{user}' ha cancellato i dati per l'host %{host} sull'interfaccia %{ifname}.",
    ["deleted_inactive_interfaces_data"] = "L'utente '%{user}' ha cancellato i dati per tutte le interfacce inattive.",
    ["deleted_interface_data"] = "L'utente '%{user}' ha cancellato i dati per l'interfaccia %{ifname}.",
    ["deleted_network_data"] = "L'utente '%{user}' ha cancellato i dati per la rete %{network} dell'interfaccia %{ifname}.",
    ["disabled_preference"] = "L'utente '%{user}' ha disabilitato la preferenza %{pref}.",
    ["edit_pool"] = "L'utente '%{user}' ha modificato il pool <a href=\"%{pool_url}\">%{pool_name}</a>.",
    ["enabled_preference"] = "L'utente '%{user}' ha abilitato la preferenza %{pref}.",
    ["exported_data"] = "L'utente '%{user}' ha esportato i dati JSON per gli host '%{mode}'.",
    ["exported_data_host"] = "L'utente '%{user}' ha esportato i dati JSON per l'host %{host} '%{mode}'.",
    ["file_downloaded"] = "L'utente '%{user}' ha scaricato il file '%{file}'.",
    ["flows_downloaded"] = "L'utente '%{user}' ha scaricato i dati dei flussi da %{from} a %{to}.",
    ["host_json_downloaded"] = "L'utente '%{user}' ha scaricato i dati JSON per l'host %{host}.",
    ["live_capture"] = "L'utente '%{user}' ha catturato traffico live dall'interfaccia %{ifname}.",
    ["live_capture_host"] = "L'utente '%{user}' ha catturato traffico live dell'host %{host} dall'interfaccia %{ifname}.",
    ["live_capture_host_with_filter"] = "L'utente '%{user}' ha catturato traffico live dell'host %{host} con filtro '%{filter}' dall'interfaccia %{ifname}.",
    ["live_capture_with_filter"] = "L'utente '%{user}' ha catturato traffico live con filtro '%{filter}' dall'interfaccia %{ifname}.",
    ["live_extraction"] = "L'utente '%{user}' ha scaricato traffico dall'interfaccia %{ifname} da %{from} a %{to}.",
    ["live_extraction_with_filter"] = "L'utente '%{user}' ha scaricato traffico dall'interfaccia %{ifname} da %{from} a %{to} con filtro '%{filter}'.",
    ["login_not_authorized"] = "Autenticazione negata per l'utente '%{user}'.",
    ["login_successful"] = "L'utente '%{user}' si è autenticato.",
    ["password_changed"] = "Password per l'utente '%{pwd_user}' cambiata da '%{user}' con IP '%{ip}'.",
    ["recording_disabled"] = "L'utente '%{user}' ha disabilitato la registrazione del traffico sull'interfaccia %{ifname}.",
    ["recording_enabled"] = "L'utente '%{user}' ha abilitato la registrazione del traffico sull'interfaccia %{ifname}.",
    ["set_host_alias"] = "L'utente '%{user}' ha impostato l'alias %{alias} all'host %{host}.",
    ["set_pool_config"] = "L'utente '%{user}' ha importato la configurazione dei pool.",
    ["set_scripts_config"] = "L'utente '%{user}' ha importato la configurazione degli script utente.",
    ["snmp_device_added"] = "L'utente '%{user}' ha aggiunto il dispositivo SNMP %{ip}.",
    ["snmp_device_deleted"] = "L'utente '%{user}' ha cancellato il dispositivo SNMP %{ip}.",
    ["unknown_activity"] = "L'utente '%{user}' ha eseguito delle azioni non gestite dagli avvisi (%{scope}).",
    ["unknown_activity_function"] = "L'utente '%{user}' ha eseguito delle azioni non gestite dagli avvisi (%{name}).",
    ["user_added"] = "L'utente '%{add_user}' è stato aggiunto da '%{user}'.",
    ["user_deleted"] = "L'utente '%{del_user}' è stato rimosso da '%{user}'.",
  },
  ["user_info"] = {
    ["missing_user_name_message"] = "Nome utente mancante",
    ["processes"] = "Processi",
    ["processes_overview"] = "Processi",
    ["top_l4_protocols"] = "Top Protocolli L4",
    ["top_l7_protocols"] = "Top Applicazioni L7",
  },
  ["users"] = {
    ["add_new_user"] = "Aggiungi Nuovo Utente",
    ["add_user"] = "Aggiungi Utente",
    ["assigned_devices"] = "Dispositivi Associati",
    ["cat_protocols"] = "Protocolli %{cat}",
    ["categories"] = "Categories",
    ["client_policy"] = "Dispositivo Come Client",
    ["confirm_delete_user"] = "Vuoi veramente eliminare l'utente \"%{user}\"?",
    ["delete_user"] = "Elimina Utente",
    ["delete_user_error"] = "Impossibile eliminare l'utente. Controllare il log per informazioni aggiuntive.",
    ["devices"] = "Dispositivi",
    ["edit"] = "Modifica",
    ["edit_global_policies"] = "Modifica Le politiche di Default",
    ["fallback_policy"] = "Politica di Fallback",
    ["full_name"] = "Nome Completo",
    ["global"] = "Globale",
    ["global_policies"] = "Politiche di Default",
    ["no_users"] = "Non ci sono utenti",
    ["num_protocols"] = "Numero di Protocolli",
    ["policy"] = "Politica",
    ["protocols"] = "Protoclli",
    ["reset_to_defaults"] = "Ripristina Valori di Default",
    ["reset_to_defaults_confirm"] = "Vuoi veramente ripristinare le politiche per \"%{devtype}\" ai valori di default?",
    ["server_policy"] = "Dispositivo Come Server",
    ["settings"] = "Impostazioni",
    ["total_daily_time_quota"] = "Quota di Tempo Giornaliero",
    ["total_daily_traffic_quota"] = "Quota di Traffico Giornaliero",
    ["unchanged"] = "Immutato",
    ["user_exists"] = "L'utente \"%{user}\" esiste già",
    ["user_x"] = "Utente: %{user}",
    ["users"] = "Utenti",
    ["shapers"] = {
      ["default"] = "Default",
      ["drop"] = "Blocca",
      ["pass"] = "Passa",
      ["slow"] = "Passa Piano",
      ["slower"] = "Passa Molto Piano",
    },
  },
  ["validation"] = {
    ["FAILED_HTTP_REQUEST"] = "Richiesta Fallita. Riprovare.",
    ["empty_name"] = "Inserire un nome.",
    ["invalid_input"] = "Controllare l'input pattern.",
    ["missing_field"] = "Riempire l'input.",
  },
  ["vlan_details"] = {
    ["no_available_stats_for_vlan_message"] = "Non sono disponibili statistiche VLAN %{vlan_id}. Attendere qualche minuto per permettere a %{product} di raccogliere nuove statistiche.",
    ["vlan_alias"] = "AliasVlan",
    ["vlan_id_parameter_missing_or_invalid_message"] = "Il parametro Vlan_Id manca o non è valido",
  },
  ["vlan_stats"] = {
    ["vlan_id"] = "VLAN ID",
    ["vlans"] = "VLAN",
  },
  ["widgets_page"] = {
    ["actions"] = "Azioni",
    ["begin_time"] = "Ora di Inizio",
    ["datasource"] = "Fonte di dati",
    ["end_time"] = "Ora di Fine",
    ["interface"] = "Interfaccia",
    ["key"] = "Chiave",
    ["name"] = "Nome",
    ["params"] = "Parametri",
    ["type"] = "Tipo",
    ["widgets"] = "Widgets",
    ["modal"] = {
      ["add_widget"] = "Aggiungi Widget",
      ["delete_description"] = "Sei sicuro di voler rimuovere <b>%{widget}</b>? Anche i widgets nella pagina saranno disabilitati!",
      ["delete_widget"] = "Elimina Widget: %{widget}",
      ["edit_widget"] = "Modifica Widget: %{widget}",
      ["embed_widget"] = "Incorpora tag widget",
      ["embed_widget_desc_1"] = "Copia e incolla questo <b>HTML</b> tag nella tua pagina",
      ["embed_widget_desc_2"] = "Perfavore assicurati di aver incluso il widget del plugin ntop <code>ntop-widgets.js</code> nella tua pagina.",
    },
    ["validation"] = {
      ["abort_add"] = "Annullare l'aggiunta di un Widget?",
      ["abort_edit"] = "Annullare la modifica di un Widget?",
    },
  },
}

return lang
