local lang = {
  ["abort"] = "中止",
  ["action"] = "アクション",
  ["actions"] = "アクション",
  ["active_flows"] = "アクティブフロー",
  ["activities"] = "アクティビティ",
  ["activity"] = "アクティビティ",
  ["add"] = "追加",
  ["admin_status"] = "管理者",
  ["advanced"] = "高度な",
  ["aggregation"] = "集計",
  ["alias"] = "エイリアス",
  ["all"] = "すべての",
  ["allowed"] = "許可",
  ["and"] = "and",
  ["and_x_more"] = "and %{num} more",
  ["any"] = "any",
  ["app_name"] = "アプリケーション名",
  ["application"] = "アプリケーション",
  ["applications"] = "アプリケーション",
  ["apply"] = "適用",
  ["arp"] = "ARP",
  ["as"] = "AS",
  ["as_number"] = "AS番号",
  ["asn"] = "ASN",
  ["availability"] = "可用性",
  ["available"] = "利用可能",
  ["backup"] = "バックアップ",
  ["begin"] = "開始",
  ["bind"] = "Bind",
  ["breakdown"] = "分解",
  ["bridge"] = "ブリッジ",
  ["broadcast"] = "ブロードキャスト",
  ["broadcast_domain"] = "ブロードキャストドメイン",
  ["broadcast_traffic"] = "ブロードキャストトラフィック",
  ["browse_config"] = "インポートする設定ファイルを表示",
  ["busy"] = "ビジー",
  ["bytes"] = "バイト",
  ["cancel"] = "キャンセル",
  ["categories"] = "カテゴリ",
  ["category"] = "カテゴリ",
  ["change_number_of_rows"] = "１ページ当たりの行数を変更",
  ["chart"] = "チャート",
  ["click_to_expand"] = "拡張するためにクリック",
  ["client"] = "クライアント",
  ["client_to_server"] = "クライアントからサーバー",
  ["clone"] = "Clone",
  ["close"] = "クローズ",
  ["configure"] = "設定する",
  ["countries"] = "国",
  ["create"] = "作成する",
  ["dark"] = "ダーク",
  ["date_from"] = "開始 日付/時刻:",
  ["date_to"] = "終了 日付/時刻:",
  ["day"] = "日",
  ["default"] = "デフォルト",
  ["define"] = "定義",
  ["delete"] = "削除",
  ["description"] = "説明",
  ["destination"] = "宛先",
  ["developer"] = "開発者",
  ["device_protocols_description"] = "このページではデバイスタイプと方向ごとに許容可能なアプリケーションプロトコルを設定可能です。（アプリケーションプロトコル観点でクライアントあるいはサーバーとしてデバイスを設定できます). 優先配下のアラート設定でデバイスプロトコルアラートを有効化することにより, 異常なプロトコルを検出するとアラートが生成されます",
  ["devices"] = "装置",
  ["difference"] = "差分",
  ["disable"] = "無効にする",
  ["disabled"] = "無効化",
  ["discarded"] = "廃棄",
  ["dismiss"] = "却下する",
  ["dns"] = "DNS",
  ["domain"] = "ドメイン",
  ["download"] = "ダウンロード",
  ["download_x"] = "%{what} ダウンロード",
  ["downloaders"] = "ダウンロードボリューム",
  ["drilldown"] = "ドリルダウン",
  ["dscp"] = "DSCP",
  ["dscp_dst2src"] = "サーバー DSCP",
  ["dscp_src2dst"] = "クライアント DSCP",
  ["duplexstatus"] = "重複ステータス",
  ["duration"] = "経過時間",
  ["edit"] = "編集",
  ["edit_configuration"] = "設定を編集",
  ["egress"] = "出力",
  ["empty"] = "空",
  ["enable"] = "有効にする",
  ["enable_them"] = "それらを有効にする",
  ["enabled"] = "有効化",
  ["end"] = "終了",
  ["enterpriseOnly"] = "この機能はエンタープライズエディションでのみ利用可能です",
  ["error"] = "エラー",
  ["error_no_search_results"] = "検索結果はありませんでした。検索キーワードを見直してください",
  ["error_not_granted"] = "リクエストが許可されていません",
  ["error_rrd_cannot_complete_dump"] = "ローカル ホスト RRD dumpを完了できませんでした。 ディスクが遅いかローカル ホストが多すぎるかもしれません",
  ["error_rrd_low_resolution"] = "RRDで利用可能なものよりも低解像度なデータのフェッチが指示されました。これは無効なデータにつながります。<br>もしも同程度のデータ粒度が必要であれば <a href=\"%{prefs}\">プロトコル/ネットワーク時系列データ</a> 設定で設定してください",
  ["errors"] = "エラー",
  ["expired_csrf"] = "ページの有効期限が切れたため、要求されたアクションを実行できませんでした。ページを再読み込みして、アクションを再度実行してください。",
  ["export"] = "エクスポート",
  ["factory_reset"] = "工場出荷時にリセット",
  ["filter"] = "フィルタ",
  ["fingerprint_note"] = "アプリケーション名は、ntopngがZMQ経由でeBPFフィードを受信するように設定されている場合にのみ設定されます",
  ["flow"] = "フロー",
  ["flow_alerts_search_criteria"] = "フローアラートの検索条件",
  ["flow_dump_not_working"] = "詳細な情報はサービスログを確認してください. ファイルシステム権限、空きスペースに問題がある可能性があります.",
  ["flow_dump_not_working_title"] = "フローダンプが動作していません",
  ["flow_search_criteria"] = "フロー検索条件",
  ["flow_search_from_aggregated"] = "(集計)",
  ["flow_search_results"] = "フロー検索結果",
  ["flows"] = "フロー",
  ["flush_periodicity_map_data"] = "データをフラッシュ",
  ["flush_service_map_data"] = "データをフラッシュ",
  ["forbidden"] = "禁止",
  ["free"] = "フリー",
  ["frequency"] = "周波数",
  ["from"] = "から",
  ["full"] = "フル",
  ["gateway"] = "ゲートウェイ",
  ["geolocation_unavailable"] = "位置情報のサポートが有効になっていません。有効にするには、次のリンクの説明にしたがってください <a href='https://github.com/ntop/ntopng/blob/dev/doc/README.geolocation.md'>ドキュメンテーション</a>",
  ["geolocation_unavailable_title"] = "ジオロケーションサポートなしで起動しています",
  ["hassh_fingerprint"] = "HASSHフィンガープリント",
  ["health"] = "稼働状況",
  ["help"] = "ヘルプ",
  ["host"] = "ホスト %{host}",
  ["host_address"] = "ホストアドレス",
  ["host_explorer"] = "ホストエクスプローラ",
  ["hosts"] = "ホスト",
  ["hour"] = "時",
  ["howto_multiselect"] = "複数の待機アイテムを選択するには <kbd>Ctrl</kbd> (or <kbd>Cmd</kbd> on Macs).",
  ["http"] = "HTTP",
  ["http_s"] = "HTTP(S)",
  ["https"] = "HTTPS",
  ["icmp"] = "ICMP",
  ["icmpv4"] = "ICMPv4",
  ["icmpv6"] = "ICMPv6",
  ["iface_download"] = " %{iface}ダウンロード",
  ["iface_upload"] = " %{iface}アップロード",
  ["import"] = "インポート",
  ["import_export_config"] = "設定のインポート/エクスポート",
  ["incomingflows"] = "受信フロー カウント",
  ["info"] = "情報",
  ["ingress"] = "イングレス",
  ["interface"] = "インターフェイス",
  ["interface_ifname"] = "インターフェイス %{ifname}",
  ["interfaces"] = "インターフェイス",
  ["ip_address"] = "IP アドレス",
  ["ipv4"] = "IPv4",
  ["ipv6"] = "IPv6",
  ["ja3_fingerprint"] = "JA3フィンガープリント",
  ["key"] = "キー",
  ["language"] = "言語",
  ["last_seen"] = "最後に現れた",
  ["layer_2"] = "MACアドレス",
  ["legenda"] = "レジェンダ",
  ["light"] = "ライト",
  ["local_hosts"] = "ローカルホスト",
  ["local_traffic"] = "ローカルトラフィック",
  ["lua_c_api"] = "Lua/C API",
  ["mac_address"] = "MACアドレス",
  ["max_rate"] = "最大レート",
  ["memory"] = "メモリ",
  ["missing_x_parameter"] = "パラメーター \"%{param}\"が不足しています",
  ["model"] = "モデル",
  ["month"] = "月",
  ["mtu"] = "MTU",
  ["multicast"] = "マルチキャスト",
  ["name"] = "名前",
  ["netmask"] = "ネットマスク",
  ["networks"] = "ネットワーク",
  ["new"] = "新しい",
  ["next"] = "次",
  ["no"] = "No",
  ["noTraffic"] = "指定されたデータ/時間についてはトラフィックは観測されませんでした",
  ["no_data_available"] = "データなし",
  ["no_flows"] = "フローなし",
  ["no_ports"] = "ポートなし",
  ["no_results_found"] = "結果が見つかりません",
  ["no_talkers_for_the_host"] = "現在のホストにはtalkersは存在しません",
  ["not_admin"] = "管理者権限が必要です",
  ["note_flow_search_allowed_networks"] = "現在のユーザーは: %{nets}のみ閲覧可能です",
  ["note_flow_search_allowed_networks_applications"] = "許可されたネットワークに所属するホストはアプリケーショントーカーで非表示になります",
  ["note_flow_search_allowed_networks_counter"] = "検索結果のカウントは合計数ですが、許可されたネットワークのフローのみ表示されます",
  ["note_flow_search_allowed_networks_ipv6"] = "１つ以上の許可されたIPv6ネットワークが存在する場合、すべてのIPv6フローを表示させることになります",
  ["note_flow_search_allowed_networks_talkers"] = "どの許可されたネットワークにも所属していないホストは上位トーカーに表示されません",
  ["notes"] = "注意:",
  ["num_different_ports"] = "%{num}個のポート。トラフィックの %{threshold} %% +を実行していません。",
  ["num_uses"] = "利用回数",
  ["observations"] = "観察",
  ["off"] = "オフ",
  ["open_documentation"] = "マニュアルを開く",
  ["operating_systems"] = "オペレーティング システム",
  ["optional_field"] = "(このフィールドはオプションです)",
  ["os"] = "OS",
  ["other"] = "その他",
  ["others"] = "その他複数",
  ["outgoingflows"] = "送信フローカウント",
  ["overview"] = "概要",
  ["packets"] = "パケット",
  ["peers"] = "ピア",
  ["percentage"] = "パーセンテージ",
  ["periodicity_map"] = "周期マップ",
  ["pkts"] = "Pkts",
  ["plugin_browser"] = "プラグインビューアー:%{plugin_name}",
  ["plugin_contents"] = "プラグインファイル",
  ["plugins"] = "プラグイン",
  ["pollers"] = "ポーラー",
  ["port"] = "ポート",
  ["ports"] = "ポート",
  ["posted"] = "投稿される",
  ["previous"] = "前",
  ["protocol"] = "プロトコル",
  ["protocol_categories"] = "プロトコル カテゴリ",
  ["protocols"] = "プロトコル",
  ["purged_error_message"] = "有効期限切れにより%{product} はメモリからパージしたようです。<a href=\"%{url}\"><i class=\"fas fa-flask\"></i> 設定</a>のアイドルタイムアウトのパージ設定で変更が可能です",
  ["push_notification"] = "プッシュ通知",
  ["quotas"] = "クォータ",
  ["ram_available"] = "使用可能",
  ["ram_total"] = "合計",
  ["ram_used"] = "使用済み",
  ["reason"] = "理由",
  ["received"] = "受信",
  ["refresh"] = "再読み込み",
  ["remote_hosts"] = "リモートホスト",
  ["remote_networks"] = "リモート ネットワーク",
  ["remote_probe_clock_drift"] = "リモートプローブクロックのズレ",
  ["remove"] = "削除",
  ["request_failed_message"] = "リクエストは失敗しました！",
  ["reset"] = "リセット",
  ["restore"] = "リストア",
  ["retry"] = "リトライ",
  ["running"] = "実行中",
  ["save"] = "保存",
  ["save_configuration"] = "保存設定",
  ["save_settings"] = "保存設定",
  ["score"] = "スコア",
  ["score_as_client"] = "クライアントスコア",
  ["score_as_server"] = "サーバースコア",
  ["scripts"] = "スクリプト",
  ["search"] = "検索",
  ["search_host"] = "検索",
  ["seen_since"] = "監視開始からの経過時間",
  ["sent"] = "送信",
  ["server"] = "サーバー",
  ["server_to_client"] = "サーバーからクライアント",
  ["service_map"] = "サービスマップ",
  ["settings"] = "設定",
  ["showing_x_to_y_rows"] = " %{tot} 行の %{x} ～ %{y} を表示しています。",
  ["similarity"] = "類似性",
  ["speed"] = "速度",
  ["ssh"] = "SSH",
  ["start"] = "開始",
  ["status"] = "状態",
  ["stop"] = "停止",
  ["success"] = "成功",
  ["summary"] = "サマリ",
  ["system"] = "システム",
  ["system_alerts_status"] = "アラート",
  ["system_interface"] = "システムインターフェイス",
  ["system_interfaces_status"] = "インターフェイスステータス",
  ["system_status"] = "システムステータス",
  ["talkers"] = "トーカー",
  ["tcp_flags"] = "TCPフラグ",
  ["tcp_stats"] = "TCP統計",
  ["telemetry"] = "テレメトリ",
  ["threshold"] = "閾値",
  ["throughput"] = "スループット",
  ["time"] = "時間",
  ["time_in"] = "の",
  ["timeframe"] = "時間枠",
  ["tls"] = "TLS",
  ["tls_certificate"] = "TLS証明書",
  ["today"] = "今日",
  ["toggle_dark_theme"] = "ダークテーマに切り替える",
  ["too_many_flows"] = "フローが多すぎます",
  ["too_many_hosts"] = "ホストが多すぎます",
  ["tools"] = "ツール",
  ["total"] = "合計",
  ["total_incoming_alerted_flows"] = "アラートされたフロー合計 サーバーとして",
  ["total_incoming_unreachable_flows"] = "ポート未到達フローの合計 サーバーとして",
  ["total_outgoing_alerted_flows"] = "アラートされたフロー合計 クライアントとして",
  ["total_outgoing_unreachable_flows"] = "ポート未到達フローの合計 クライアントとして",
  ["total_retransmissions_rcvd"] = "再送信合計 受信",
  ["total_retransmissions_sent"] = "再送信合計 送信",
  ["traffic"] = "トラフィック",
  ["traffic_policy"] = "トラフィックポリシー",
  ["undo"] = "やりなおし",
  ["unknowers"] = "不明なトラフィックボリューム",
  ["unknown"] = "不明",
  ["unlimited"] = "&infin;",
  ["uploaders"] = "アップロードボリューム",
  ["used_by"] = "使われる",
  ["values"] = "値",
  ["version"] = "あなたのバージョンは %{vers}.",
  ["via"] = "経由",
  ["views"] = "ビュー",
  ["vlan"] = "VLAN",
  ["vlan_id"] = "VLAN",
  ["volume"] = "ボリューム",
  ["volume_size"] = "ボリュームサイズ",
  ["warning"] = "警告",
  ["week"] = "週",
  ["welcome"] = "ようこそ",
  ["welcome_to"] = "%{product}へようこそ",
  ["yes"] = "はい",
  ["about"] = {
    ["about"] = "情報",
    ["about_x"] = "%{product}について",
    ["active"] = "アクティブ",
    ["alert_defines"] = "アラート定義",
    ["alert_queries"] = "クエリ",
    ["alerts_dropped"] = "ドロップ",
    ["alerts_stored"] = "格納",
    ["built_on"] = "に基づいて",
    ["callback_directories"] = "ユーザースクリプトディレクトリ",
    ["checks"] = "ユーザースクリプト",
    ["configure_dhcp_range"] = "DHCPレンジを設定する",
    ["contribute_to_project"] = "貢献する",
    ["copyright"] = "著作権",
    ["cpu_load"] = "CPU負荷",
    ["cpu_states"] = "CPUステータス",
    ["create_license"] = "次を作成することを推奨します <code>ntopng.license</code> ファイル (あなたが受け取った説明に従って) redisのクリーナップでライセンスを失うのを防ぐ為に",
    ["create_license_l"] = "次のファイルを作成することにも注意してください <code>/etc/ntopng.license</code> ファイル このファイルは他のモジュールにも利用されます.",
    ["data_directory"] = "データ",
    ["datadir_warning"] = "データディレクトリのマイグレーションに関する注意",
    ["defs_directories"] = "定義ディレクトリ",
    ["dhcp_monitoring_title"] = "DHCPホストをモニターする",
    ["dhcp_range_missing_warning"] = "\" <a href='%{url}'> %{name}</a>設定は有効ではありません。 <a href='%{dhcp_url}'>DHCPレンジ</a>を正確に設定してください.",
    ["directories"] = "ディレクトリ",
    ["dropped_store_queries_alerts"] = "クエリ/保存/ドロップされたアラート",
    ["flow_checks_directory"] = "フロー",
    ["forced_community_notification"] = "コミュニティバージョンは --communityオプションを指定します. 有償ライセンスを有効にするにはこのオプションを削除してください.",
    ["forced_community_notification_short"] = "コミュニティバージョンは、--communityを使ってください",
    ["host_checks_directory"] = "ホスト",
    ["host_identifier_warning"] = "このインターフェイスはDHCPホストを監視しています. <a href='%{url}'> %{name}</a> 設定を \"MACアドレス\"に変更することを検討してください.",
    ["idle"] = "アイドル",
    ["interface_callbacks_directory"] = "インターフェィス",
    ["iowait"] = "iowait",
    ["last_log"] = "最新のログトレース",
    ["licence"] = "ライセンス",
    ["licence_expires_in"] = "ライセンスは %{time}に期限切れします",
    ["licence_generation"] = "上記URLをクリックしてプロフェッショナルバージョンのライセンスを生成するか、 <br><a href=\"%{purchase_url}\">e-shop</a>でライセンスを購入する. 利用目的が非営利、研究、教育の場合には<br> <a href=\"%{universities_url}\">をお読みください</a>.",
    ["licence_status"] = "ライセンス",
    ["maintenance"] = "\"メンテナンス: %{_until}まで [%{days_left} 日残っています]",
    ["maintenance_expired"] = "%{product} メンテナンス期間が有効期限切れしました",
    ["maxmind"] = "サポートします <a class='ntopng-external-link' href=\"%{maxmind_url}\">MaxMind <i class=\"fas fa-external-link-alt\"></i></a> と <a class='ntopng-external-link' href='https://db-ip.com'>DB-IP <i class=\"fas fa-external-link-alt\"></i></a> ジオロケーションデーターベース.",
    ["network_callbacks_directory"] = "ネットワーク",
    ["new_major_available"] = "新しい %{product} 安定版 (v.%{version}) 利用できます <a href=\"%{url}\">ダウンロード</a>: アップグレードしてください.",
    ["ntop_blog"] = "ntopブログ",
    ["platform"] = "プラットフォーム",
    ["ram_memory"] = "RAM",
    ["readme_and_manual"] = "ユーザーガイド",
    ["report_issue"] = "問題を報告",
    ["runtime_status"] = "実行状態",
    ["save_licence"] = "ライセンスを保存する",
    ["scripts_directory"] = "スクリプト",
    ["server_time"] = "サーバー時間",
    ["specify_licence"] = "ntopngライセンスをこちらで指定してください",
    ["startup_line"] = "起動コマンドライン",
    ["suggest_feature"] = "機能追加を提案",
    ["system"] = "システム",
    ["system_id"] = "システムID",
    ["telegram"] = "ヘルプとニュース",
    ["telemetry_data_opt_out_msg"] = " \"<a href=\"%{ntop_org}\">ntop.org</a>へ暗号化したデータを送信してプロジェクトに貢献, <a href=\"%{tel_url}\">匿名テレメトリデータ</a>を使用します。 : <a href=\"%{url}\"><i class=\"fas fa-flask\"></i>設定</a> ページに訪問して、設定を確認してください.",
    ["upgrade_to_professional"] = "プロフェッショナルバージョンにアップグレード",
    ["uptime"] = "起動時間",
    ["version"] = "バージョン",
    ["you_have_too_many_flows"] = "%{product} 設定におけるホスト/フローの設定が多すぎます。これはパケットロスとCPUの高負荷を生み出します. -xと-Xを増やして%{product} を再起動してください",
    ["you_have_too_many_hosts"] = "ホストが多すぎます %{product} パケットドロップ及びCPUの高いロードの原因となります. -x/-X オプションを定義して再起動してください %{product}",
    ["you_need_to_sync_remote_probe_time"] = "ntopng及びリモートプローブホストを時刻同期してください <a href=\"%{url}\">クロックのズレ</A> が多すぎます.収集したフローデータに影響します.",
  },
  ["active_monitoring_stats"] = {
    ["active_monitoring"] = "アクティブモニタ",
    ["active_monitoring_description"] = "ステータス及び指定ホストのレスポンス時間の監視",
    ["add_icmp"] = "ICMPモニタの追加",
    ["add_pool"] = "新しいアクティブモニタプール",
    ["add_record"] = "アクティブモニタレコードの追加",
    ["alert_status"] = "アラートステータス",
    ["alerted"] = "アラート済み",
    ["am_note_http"] = "HTTPとHTTPSでのWebページ読み込みの測定",
    ["am_note_icmp"] = "ICMP及びICMP(IPv6)を使ったicmp及びicmp6の測定",
    ["app_connect_descr"] = "\"アプリケーション接続\" 時間はTLSハンドシェイクの完了にかかる時間です.",
    ["availability"] = "可用性",
    ["bad_action_param"] = "不正な\"action\"パラメータ",
    ["config_reset_confirm"] = "本当に現在の設定を破棄してデフォルトにリセットしますか?",
    ["confirm_delete"] = "本当にこのレコードを削除しますか?",
    ["download_speed"] = "ダウンロード速度",
    ["edit_record"] = "アクティブモニタのレコードを編集します",
    ["empty_action"] = "何かが間違っています(空のアクション). ページを再読み込みしてリトライしてください",
    ["host_add_ok"] = "ホスト %{host} の追加に成功しました!",
    ["host_delete_ok"] = "ホスト %{host} の削除に成功しました!",
    ["host_edit_ok"] = "ホスト %{host} の編集に成功しました!",
    ["host_exists"] = "ホスト \"%{host}\" は既に存在しています",
    ["host_not_exists"] = "ホスト \"%{host}\" が見つかりません",
    ["icmp_continuous"] = "継続的な ICMP (IPv4)",
    ["icmp_continuous_v6"] = "継続的な ICMP (IPv6)",
    ["invalid_combination"] = "不正な測定もしくはホストの組み合わせです",
    ["invalid_host"] = "間違ったホスト定義です",
    ["jitter"] = "平均 RTT / Jitter",
    ["last_measurement"] = "最後の測定",
    ["measurement"] = "測定",
    ["measurement_time"] = "測定時間",
    ["msec"] = "ms",
    ["no_updates_yet"] = "アップデートが未だです",
    ["not_alerted"] = "アラートがありません",
    ["note3"] = "このプラグインは計測を実行する為に、設定されたホストに向かってトラフィックを生成します.",
    ["note_alert"] = "閾値を超過した場合、アラートが発報されます.",
    ["note_availability"] = "可用性の割合が、最後の日に設定した閾値の間であった計測値のパーセンテージを表示します.",
    ["note_periodicity_change"] = "周期的な変化が古い時系列データが削除される原因となります.",
    ["other_http_descr"] = "\"Name Lookup\" と \"Other\" の合計は、HTTPリクエストを完了する為の合計時間です.",
    ["other_https_descr"] = "\"App Connect\", \"Name Lookup\" 及び \"Other\" の合計は、HTTPSリクエストを完了する為の合計時間です.",
    ["pool"] = "プール",
    ["rtt_vs_jitter"] = "平均 RTT / Jitter",
    ["speedtest"] = "スピードテスト",
    ["upload_speed"] = "アップロードスピード",
    ["validation"] = {
      ["abort_add"] = "追加アクティブモニタリングレコードを中止しますか?",
      ["abort_edit"] = "編集アクティブモニタリングレコードを中止しますか?",
    },
  },
  ["activities_page"] = {
    ["host_activity"] = "ホストアクティビティ",
  },
  ["activity_descriptions"] = {
    ["5min"] = "5分間のタスクを実行します。RRDを使用すると、ホストの時系列データも書き込みます。",
    ["daily"] = "毎日のタスクを実行します。",
    ["dequeue_flows_for_hooks"] = "アイドル状態のハッシュテーブルを特定した重要なアクティビティをエントリし、パージします。そして、フローでユーザースクリプトを実行し、エクスポートします。",
    ["discover"] = "デバイスのネットワークをスキャンします。スキャンが要求/計画されていない場合、このアクティビティは何もしません。",
    ["hourly"] = "毎時のタスクを実行します。",
    ["housekeeping"] = "ディスクに保存する、もしくは通知するアラートをチェックします。n2disk抽出ジョブを実行し、必要に応じてカテゴリリストをリロードします。",
    ["minute"] = "分単位のタスクを実行し、分単位の時系列データを書き込みます。InfluxDBを使用する場合、ホストの時系列データを書き込みます。",
    ["second"] = "インターフェイスの時系列データ及びその他の高頻度データを書き込みます。",
    ["stats_update"] = "アクティブなトラフィック要素のスループットを計算します。",
    ["timeseries"] = "バッファリングされた時系列データポイントをチェックし、それらをInfluxDBにエクスポートします。",
  },
  ["alert_messages"] = {
    ["host_alert_countries_contacts"] = "％{entity} [％{value}＆gt;から連絡を受けた国が多すぎます ％{threshold}国]",
    ["a_new_device_has_connected"] = "新規デバイス <a href=\"%{url}\">%{device}</a> がネットワークに接続しました",
    ["anomalous_tcp_flags"] = "\"%{entity} は %{sent_or_rcvd} 多くのTCP RST flags vs SYN [Ratio: %{ratio}%%]",
    ["attack_mitigation_via_snmp_failure"] = "インターフェイスの設定に失敗. <a href=\"%{port_url}\">%{port}</a> SNMPデバイスの管理ステータス <a href=\"%{url}\">%{device}</a> へ %{admin_down}: %{granularity} <b>%{metric}</b> に近い %{entity} [%{value} %{op} %{threshold}]. SNMPデバイスが有効な書き込み権限があるか確認してください.",
    ["attack_mitigation_via_snmp_success"] = "インターフェイスの設定に失敗 <a href=\"%{port_url}\">%{port}</a> SNMPデバイスの管理者ステータス <a href=\"%{url}\">%{device}</a> が次に設定 %{admin_down}: %{granularity} <b>%{metric}</b> crossed by %{entity} [%{value} %{op} %{threshold}]",
    ["broadcast_domain_info"] = "ARPトラフィックが異なるブロードキャストドメインに属している様に見えます. ホストとネットワーク設定を確認してください.",
    ["broadcast_domain_too_large"] = " \"<a href=\"%{src_mac_url}\">%{src_mac}</a>からのARPトラフィック/<a href=\"%{spa_url}\">%{spa}</a> to <a href=\"%{dst_mac_url}\">%{dst_mac}</a>/<a href=\"%{tpa_url}\">%{tpa}</a> が検出されました。他のブロードキャストドメインに所属しているようにみえます。ホスト及びネットワーク設定を見直してください。.",
    ["device_has_connected"] = "デバイス <a href=\"%{url}\">%{device}</a> がネットワークに接続しました",
    ["device_has_disconnected"] = "デバイス <a href=\"%{url}\">%{device}</a> がネットワークから切断しました",
    ["flow_flood_attacker"] = " \"%{entity} はフローを溢れさせています [%{value} &gt; %{threshold} フローが送信されました]",
    ["flow_flood_victim"] = "\"%{entity}はフローフラッド攻撃を受けています [%{value} &gt; %{threshold} フローを受信しました]",
    ["host_entity"] = "ホスト %{entity_value}",
    ["host_log"] = "ホストログ",
    ["host_pool_has_connected"] = "ホストプール <a href=\"%{url}\">%{pool}</a> がネットワークに接続しました",
    ["host_pool_has_disconnected"] = "ホストプール <a href=\"%{url}\">%{pool}</a> がネットワークから切断しました",
    ["host_remote_to_remote"] = "リモートホスト <a href=\"%{url}\">%{ip}</a> %{mac} が別のリモートホストに接続しました。 リモート-to-リモート フローは<a href=\"%{flow_alert_url}\">フローアラートページ</a>で閲覧可能です",
    ["iface_alerts_dropped"] = "%{num_dropped}アラートがインターフェイス <a href=\"%{url}\">%{iface}</a>でドロップされました。",
    ["influxdb_dropped_points"] = "\"ポイントが下がりました.\" %{influxdb}\"を送信できません.",
    ["influxdb_migration_msg"] = "\"多くのローカルホストが存在します。 <a href='%{url}'>時系列データのパフォーマンス向上のために、RRDからInfluxDBへの移行を検討してください</a> .",
    ["influxdb_partial_write"] = "これはInfluxDB サーバー configの\"max-values-per-tag = 0\"という設定で設定可能です",
    ["influxdb_write_error"] = "時系列データを送信中にエラーが発生しました \"%{influxdb}\": %{err}",
    ["interface_entity"] = "インターフェイス %{entity_value}",
    ["ip_outside_dhcp_range_router_ip"] = " \"[ルータ: <a href=\"%{router_url}\"> %{router_ip}</a>]",
    ["ip_outsite_dhcp_range"] = "デバイス <a href=\"%{client_url}\"> %{client_mac}</a> がIPアドレスを取得しました <a href=\"%{client_ip_url}\">%{client_ip}</a> これは範囲外です <a href=\"%{dhcp_url}\">設定されたDHCPレンジ</a>. [送信者: <a href=\"%{sender_url}\">%{sender_mac}</a>]",
    ["lldp_arc_added"] = "新しいリンクが追加された. SNMPデバイス <a href=\"%{url1}\">%{node1}</a> と <a href=\"%{url2}\">%{node2}</a>",
    ["lldp_arc_removed"] = "リンクを削除します. SNMPデバイス <a href=\"%{url1}\">%{node1}</a> と <a href=\"%{url2}\">%{node2}</a>",
    ["mac_ip_association_change"] = "IP %{ip} とMACアドレスの関連付けが <a href=\"%{old_mac_url}\">%{old_mac}</a> から<a href=\"%{new_mac_url}\">%{new_mac}</a>に変更されました",
    ["measurement_too_high_msg"] = "計測が高すぎます. %{host} %{numeric_ip} [%{am_value} %{unit} > %{threshold} %{unit}].",
    ["measurement_too_low_msg"] = "継続が低すぎます. %{host} %{numeric_ip} [%{am_value} %{unit} < %{threshold} %{unit}].",
    ["misbehaving_flows_ratio"] = "\"%{entity} は%{sent_or_rcvd}合計フローと比較して不正なフローが多すぎます [Ratio: %{ratio}%%]",
    ["network_entity"] = "ネットワーク %{entity_value}",
    ["nfq_flushed"] = "インターフェイス <a href=\"%{url}\">%{name}</a> のパケットキューがフラッシュされました。 キュー %{pct}%% がいっぱいです %{tot} パケットと %{dropped} のドロップパケット",
    ["no_influxdb"] = " \" <A HREF=\"%{url}\">時系列データダンプのInfluxDBへの変換を有効にしてください</A>そしてページを再読み込みしてください",
    ["ntopng_anomalous_termination"] = "異常終了後に再起動しました。(<a href=\"%{url}\">バグ報告</a>)",
    ["ntopng_start"] = "開始",
    ["ntopng_stop"] = "停止",
    ["ntopng_update"] = "ソフトウェア更新後に開始",
    ["open_files_limit_too_small"] = "ntopng はMySQLがオープンできる最大のファイル数が少なすぎることを検出しました。これは [ファイルを開くときのリソース不足 './ntopng/flowsv6#P#p22.MYD' (Errcode: 24 - 開いているファイルが多すぎます)][23]のような理由によるフローエラーを引き起こします。 オープンファイル制限を増やすか、単純にこの警告を無視したければ優先のチェックをはずしてください",
    ["periodic_activity_not_executed"] = "定期的なアクティビティ \"%{script} \"がキューに存在しますが、まだ実行されていません。実行できるスレッドがありません。[%{pending_since}から保留中です]",
    ["ping_host_unreachable_v3"] = "ホスト %{host} %{numeric_ip}に到達できません",
    ["potentially_dangerous_protocol_description"] = "古いまたは脆弱なプロトコルが使用されているため、このフローはネットワークに有害な可能性があります。",
    ["proto_bytes_quotas"] = "%{proto} バイト",
    ["proto_time_quotas"] = "%{proto} 時間",
    ["remote_to_remote"] = "リモート %{entity} リモートホストにコンタクトしました",
    ["slow_periodic_activity"] = "%{script}\\の実行時間が長すぎる [more than %{max_duration}] を超えているか、実行が遅すぎる（キューでブロックされています）。",
    ["slow_purge"] = " \"<a href=\"%{url}\">ハッシュテーブルのアイドルエントリー</a> %{iface}のパージが遅すぎます.これはメモリ、データ の正確性、アラートの抜けにつながります. [&gt %{max_idle}%%]",
    ["speedtest_failed"] = "スピードテストの実行が失敗しました",
    ["subject_quota_exceeded"] = "ホストプール <a href=\"%{url}\">%{pool}</a> は%{subject} クォータ [%{value}&gt;%{quota}]を超過しました。",
    ["syn_flood_attacker"] = "\"%{entity} はSYNフラッダーです [%{value}&gt;%{threshold} SYNが送信]",
    ["syn_flood_victim"] = " \"%{entity}がSYNフラッド攻撃を受けています[%{value}&gt;%{threshold} SYNを受信]",
    ["syn_scan_attacker"] = "%{entity}はSYNスキャン攻撃者です[%{value}&gt;%{threshold} SYNが送信]",
    ["syn_scan_victim"] = "%{entity}はSYNスキャンを受けています[%{value}&gt;%{threshold} SYNを受信]",
    ["test_failed"] = "テスト失敗",
    ["threshold_crossed"] = "%{granularity} <b>%{metric} </ b>が%{entity} [%{value}%{op}%{threshold}]に到達しました",
    ["too_many_drops"] = "%{iface}にドロップされたパケットが多すぎます[&gt%{max_drops} %%",
    ["too_many_flows"] = "%{iface}のフローが多すぎます。 %{option}コマンドラインオプションを拡張してください",
    ["too_many_hosts"] = "%{iface}のホストが多すぎます。 %{option}コマンドラインオプションを拡張してください",
    ["x_alerts"] = "%{num}アラート",
  },
  ["alerts_dashboard"] = {
    ["host_alert_countries_contacts"] = "国の連絡先アラート",
    ["active_flows_anomaly"] = "アクティブフローの異常",
    ["alert"] = "アラート",
    ["alert_counts"] = "カウント",
    ["alert_duration"] = "期間",
    ["alert_severity"] = "重大度",
    ["alert_snmp_device_reset_description"] = "SNMPデバイス<a href=\"%{url}\"> %{device} </a>が再起動されました",
    ["alert_snmp_device_reset_title"] = "SNMPデバイスの再起動",
    ["alert_snmp_interface_mac_changed_description"] = "MAC <a href=\"%{mac_url}\">%{mac} </a> SNMPデバイス<a href=\"%{url}\">%{device} </a>インターフェイス<a href = \"%{port_url} \">%{port} </a>以前にSNMPデバイスで見られた<a href=\"%{prev_device_url}\">%{prev_device} </a>インターフェイス<a href =  \"%{prev_port_url} \">%{prev_port} </a> ",
    ["alert_snmp_interface_mac_changed_title"] = "SNMPインターフェィスMACの変更",
    ["alert_type"] = "タイプ",
    ["alerts"] = "アラート",
    ["alerts_dashboard"] = "アラートダッシュボード",
    ["alerts_ts"] = "プロセスアラート時系列データ",
    ["alerts_ts_description"] = "プロセスアラート時系列データの生成",
    ["all_hosts"] = "すべてのホスト",
    ["anomalous_tcp_flags"] = "異常なTCPフラグ",
    ["attack_mitigation_snmp_description"] = "設定した閾値をクライアントのスコアが超過した場合 ホストのSNMPアクセスポート管理の状態をダウンに設定します",
    ["attack_mitigation_snmp_title"] = "SNMPによる攻撃の緩和",
    ["blacklisted_country"] = "ブラックリストの国",
    ["blacklisted_country_descr"] = "ホストが指定した国から連絡を受けたとき、または連絡されたときにアラートをトリガーします",
    ["blacklisted_flow"] = "ブラックリストに登録されたフロー",
    ["blocked_flow"] = "ブロックされたフロー",
    ["broadcast_domain_too_large"] = "ブロードキャストドメイン",
    ["checks_calls_drops"] = "ドロップされたユーザースクリプトを呼び出し",
    ["client_and_server_countries_blacklisted"] = "クライアントの国 %{cli_country}およびサーバーの国 %{srv_country}はブラックリストに登録されています。",
    ["client_country_blacklisted"] = "クライアント国 %{country}はブラックリストに登録されています",
    ["connection_issues"] = "接続の問題",
    ["critical"] = "クリティカル",
    ["custom_period"] = "カスタム期間",
    ["debug"] = "デバック",
    ["device_connection"] = "デバイス接続",
    ["device_disconnection"] = "デバイスの切断",
    ["dns_anomaly"] = "DNS異常",
    ["emergency"] = "緊急事態",
    ["engaged_for_longest"] = "過去に発生したアラートが最も長く続いた",
    ["error"] = "エラー",
    ["experimental_checks"] = "実験的",
    ["experimental_checks_description"] = "実験的なアラートチェックを有効にする：TCPフラグ、異常なフロー率",
    ["external_alert"] = "外部アラート",
    ["five_mins"] = "前回の5分間",
    ["flow_alert_origins"] = "フローアラート元",
    ["flow_alert_targets"] = "フローアラートターゲット",
    ["flow_checks_calls_drops_description"] = " %{num_drops} <a href='%{url}'>フローユーザースクリプト</a>呼び出しは呼び出されませんでした。一部の不正なフローは気付かれない場合があります。",
    ["flow_logger"] = "フローロガー",
    ["flow_logger_descr"] = "コンソール/ファイルに新しいフローを記録します",
    ["flow_misbehaviour"] = "フローの誤動作",
    ["flow_risk"] = "フローリスク",
    ["flows_flood"] = "フローフラッド",
    ["ghost_network_detected"] = "ゴーストネットワークが検出されました",
    ["ghost_network_detected_description"] = "サブネット %{network}は <a href='%{url}'> %{entity}ネットワーク</a>に属していません",
    ["ghost_networks"] = "ゴーストネットワーク",
    ["ghost_networks_description"] = "ネットワークインターフェイスで検出された不明なネットワークのアラートをトリガーします。",
    ["host_log"] = "ホストログ",
    ["host_pool_connection"] = "ホストプール接続",
    ["host_pool_disconnection"] = "ホストプールの切断",
    ["host_score_description"] = "ホストスコアを計算し、指定されたしきい値を超えたときにアラートをトリガーします。",
    ["icmp_anomaly"] = "ICMP異常",
    ["iec104_error"] = "IEC104エラー",
    ["inactivity"] = "インアクティブ",
    ["influxdb_dropped_points"] = "InfluxDBのドロップポイント",
    ["influxdb_error"] = "InfluxDBエラー",
    ["influxdb_export_failure"] = "InfluxDBエクスポートの失敗",
    ["influxdb_monitor"] = "InfluxDBモニター",
    ["influxdb_monitor_description"] = "InfluxDBのヘルス状態を監視する",
    ["info"] = "情報",
    ["internal_issues"] = "内部問題",
    ["involving_msg"] = "関連するフローアラート",
    ["known_proto_on_non_std_port"] = "標準ポートを使っていないアプリケーション",
    ["known_proto_on_non_std_port_full"] = "アプリケーションが %{app} 標準ポート利用でないポートで検知されました %{port}",
    ["last_day"] = "昨日",
    ["last_hour"] = "1時間前",
    ["last_minute"] = "1分前",
    ["lateral_movement"] = "ラテラルムーブメント",
    ["list_download_failed"] = "リストのダウンロードに失敗しました",
    ["list_download_succeeded"] = "リストのダウンロードが成功しました",
    ["local_only"] = "ローカルホストのみ",
    ["local_origin_remote_target"] = "ローカルオリジン-リモートターゲット",
    ["login_failed"] = "ログインに失敗しました",
    ["mac_ip_association_change"] = "IP / MACの変更",
    ["malicious_signature_detected"] = "悪意のある署名",
    ["many_replies"] = " %{what} %{count}件の返信",
    ["many_requests"] = " %{what} %{count}件のリクエスト",
    ["memory_ts"] = "メモリ時系列データ",
    ["memory_ts_description"] = "メモリプロセス時系列データの生成",
    ["misbehaving_flows_ratio"] = "大量の不正フロー",
    ["misconfigured_app"] = "誤って設定されたアプリ",
    ["misconfigured_dhcp_range"] = "正しく構成されていないDHCP範囲",
    ["new_device"] = "新しいデバイス",
    ["nfq_flushed"] = "フラッシュされたパケットキュー",
    ["no_alert"] = "アラートなし",
    ["no_alerts"] = "アラートなし",
    ["none"] = "なし",
    ["not_engaged"] = "現在発生していない",
    ["notice"] = "お知らせ",
    ["one_day"] = "直近の日",
    ["one_hour"] = "直近の1時間",
    ["one_min"] = "直近の1分",
    ["one_reply"] = " %{what} %{count}件の返信",
    ["one_request"] = " %{what} %{count}件のリクエスト",
    ["out_of_x_total_packets"] = "%{tot} パケットの中から",
    ["periodic_activity_not_executed"] = "定期的なアクティビティが実行されていない",
    ["periodic_activity_not_executed_descr"] = "定期的なアクティビティが実行キューに入っている時にアラートをトリガーしますが、全てのワーカースレッドがビジーであるため実行されません。",
    ["ping_issues"] = "Ping",
    ["potentially_dangerous_protocol"] = "潜在的に危険なプロトコル",
    ["process"] = "処理",
    ["quota_exceeded"] = "クォータ超過",
    ["remote_only"] = "リモートホストのみ",
    ["remote_origin_local_target"] = "リモートオリジン-ローカルターゲット",
    ["remote_to_remote"] = "リモートからリモート",
    ["search_criteria"] = "ダッシュボードの設定",
    ["server_country_blacklisted"] = "サーバーを持つ国 %{country}はブラックリストに登録されています",
    ["slow_periodic_activity"] = "定期的な活動が遅い",
    ["slow_periodic_activity_descr"] = "定期的なアクティビティの実行に時間がかかりすぎる場合にアラートをトリガーします。",
    ["slow_purge"] = "アイドルパージが低速",
    ["snmp_device_is_unresponsive"] = "SNMPデバイス<a href=\"%{url}\"> %{device} </a>が応答しません",
    ["snmp_device_reset_script_description"] = "SNMPデバイスの再起動が検出されたときにアラートをトリガーする",
    ["snmp_interface_mac_changed_script_description"] = "MACがインターフェイスまたはデバイス間を移動したときにアラートをトリガーします",
    ["snmp_interface_mac_changed_script_title"] = "SNMPインターフェイスMACの変更",
    ["snmp_interfaces_rsi"] = "SNMPインターフェイスRSI",
    ["snmp_interfaces_rsi_descr"] = "インターフェイストラフィックの変化を検出し、アラートをトリガーします（実験的）",
    ["snmp_port_changed_duplex_status"] = "SNMPデバイス<a href=\"%{url}\">%{device} </a>のインターフェイス<a href=\"%{port_url}\">%{port} </a>がデュプレックスステータスを変更しました%{new_op}",
    ["snmp_port_changed_operational_status"] = "SNMPデバイス<a href=\"%{url}\">%{device} </a>のインターフェイス<a href=\"%{port_url}\">%{port} </a>がステータスを変更しました%{new_op}",
    ["snmp_port_duplexstatus_change"] = "インターフェイスデュプレックスステータスの変更",
    ["snmp_port_duplexstatus_change_description"] = "インターフェイスのデュプレックスステータスが変更されたときにアラートをトリガーする",
    ["snmp_port_errors"] = "インターフェイスエラー",
    ["snmp_port_errors_description"] = "インターフェイスの破棄/エラーカウンターが増加したときにアラートをトリガーする",
    ["snmp_port_errors_increased"] = "SNMPデバイス<a href=\"%{url}\">%{device} </のインターフェイス<a href =  \"%{port_url} \">%{port} </a>で破棄/エラーカウンターが増加",
    ["snmp_port_load_threshold_exceeded"] = "インターフェイスの負荷しきい値を超えました",
    ["snmp_port_load_threshold_exceeded_message"] = "SNMPデバイス<a href=\"%{url}\">%{device} </a>のインターフェイス<a href=\"%{port_url}\">%{port} </a>で負荷しきい値を超えました[しきい値：%{threshold} %%] [入力負荷：%{in_load} %%] [出力負荷：%{out_load} %%]",
    ["snmp_port_status_change"] = "インターフェイスの運用状態の変更",
    ["snmp_port_status_change_description"] = "インターフェイスの運用状態が変化したときにアラートをトリガーします",
    ["starting_on"] = "で起動する",
    ["submit"] = "ダッシュボードを更新",
    ["suspicious_activity"] = "不審な活動",
    ["suspicious_device_protocol"] = "疑わしいデバイスプロトコル",
    ["binary_application_transfer"] = "疑わしいファイル転送",
    ["binary_application_transfer_url"] = "%{type_icon} 疑わしいファイル転送を検知しました [%{url}]",
    ["tcp_syn_flood"] = "TCP SYNフラッド",
    ["tcp_syn_scan"] = "TCP SYNスキャン",
    ["threashold_cross"] = "しきい値を超える",
    ["too_low_replies_received"] = " %{entity}は %{requests}を送信しましたが、 %{replies}を受信しました[%{granularity}比率:%{ratio} %%] ",
    ["too_low_replies_sent"] = " %{entity}は %{requests}を受信しましたが、 %{replies}を送信しました[%{granularity}]",
    ["too_many_drops"] = "パケットドロップ",
    ["too_many_flows"] = "フロー過多アラート",
    ["too_many_flows_description"] = "アクティブなフローの数が多すぎるときにアラートをトリガーします",
    ["too_many_hosts"] = "ホスト過多アラート",
    ["too_many_hosts_description"] = "アクティブなホストの数が多すぎる場合にアラートをトリガーします",
    ["total_alerts"] = "アラートの合計",
    ["trailing_msg"] = "時間枠",
    ["trailing_msg_compact"] = "ウィンドウ",
    ["unresponsive_device"] = "応答しないデバイス",
    ["user_activity"] = "ユーザーアクティビディ",
    ["warning"] = "警告",
    ["web_mining"] = "Webマイニング",
    ["x_lost"] = "%{lost} ロスト",
    ["x_ooo"] = "%{ooo} 故障中",
    ["x_retx"] = "%{retx} 再送",
  },
  ["alerts_thresholds_config"] = {
    ["countries_contacts_title"] = "国の連絡先アラート",
    ["countries_contacts_description"] = "連絡したさまざまな国の数がしきい値を超えたときにアラートをトリガーします",
    ["active_local_hosts"] = "ローカルホストアラート",
    ["active_local_hosts_threshold_descr"] = "アクティブなローカルホストの数がしきい値を超えたときにアラートをトリガーする",
    ["activity_time"] = "活動時間差分アラート",
    ["alert_active_description"] = "アクティビティ時間の差分がしきい値を超えたときにアラートをトリガーします",
    ["alert_bytes_description"] = "レイヤー2バイトのデルタ数（送信+受信）がしきい値を超えたときにアラートをトリガーします",
    ["alert_bytes_title"] = "トラフィックアラート",
    ["alert_dns_description"] = "DNSトラフィックのデルタ（送信数+受信数）がしきい値を超えたときにアラートをトリガーします",
    ["alert_flows_description"] = "フローのデルタバイト数（クライアント+サーバーとして）がしきい値を超えたときにアラートをトリガーします",
    ["alert_flows_title"] = "フローアラート",
    ["alert_idle_description"] = "アイドル時間（前回のパケットが表示されてからの時間）がしきい値を超えたときにアラートをトリガーします",
    ["alert_network_egress_description"] = "デルタ出力バイト数がしきい値を超えたときにアラートをトリガーします",
    ["alert_network_ingress_description"] = "デルタ入力バイト数がしきい値を超えたときにアラートをトリガーします",
    ["alert_network_inner_description"] = "デルタ内部バイト数がしきい値を超えたときにアラートをトリガーします",
    ["alert_p2p_description"] = "P2Pトラフィックのレイヤー2バイトデルタ（送信数+受信数）がしきい値を超えたときにアラートをトリガーする",
    ["alert_packets_description"] = "デルタパケット（送信数+受信数）がしきい値を超えたときにアラートをトリガーします",
    ["alert_packets_title"] = "パケットアラート",
    ["alert_slow_purge_threshold"] = "ハッシュテーブル内のアイドルエントリアラート",
    ["alert_slow_purge_threshold_descr"] = "ハッシュテーブル内のエントリの総数に対するアイドルエントリの割合がしきい値を超えたときにアラートをトリガーする",
    ["alert_throughput_description"] = "平均スループット（送信+受信）がしきい値を超えたときにアラートをトリガーします",
    ["common_thresholds_local_sources"] = "%{source}共通設定",
    ["daily"] = "毎日",
    ["dns_traffic"] = "DNSトラフィックアラート",
    ["egress_traffic"] = "出口トラフィックアラート",
    ["every_5_minutes"] = "5分ごと",
    ["every_minute"] = "毎分",
    ["hourly"] = "毎時",
    ["idle_time"] = "アイドル時間アラート",
    ["ingress_traffic"] = "入口トラフィックアラート",
    ["inner_traffic"] = "内部トラフィックアラート",
    ["note_add_custom_scripts"] = "%{product} これらのカスタムスクリプトを探します <a href=\"%{url}\"> <i class=\"fas fa-folder\"></i> ディレクトリ</a>.",
    ["note_checks_on_active_hosts"] = "チェックはアクティブなホストに対してのみ実行されます。",
    ["note_consecutive_checks"] = "連続したチェックは、必ずしも連続した期間で実行されるとは限りません。たとえば、ホストがアイドル状態になった場合、再びアクティブになるまでそのしきい値はチェックされません。",
    ["note_control_threshold_checks_periods"] = "これらのタブにリストされているコールバックは定期的に実行されます。",
    ["note_create_custom_scripts"] = "コールバックを実行するためのカスタムスクリプトを作成するためのドキュメントは、次の<a class='ntopng-external-link' href=\"%{url}\">リンク <i class =  \"fas fa-external-link-alt \">で入手できます。 </ i></a>",
    ["note_thresholds_expressed_as_delta"] = "一部のコールバックはデルタを計算します。デルタは、2つの連続する同じ動作の間にある数値の変化です。",
    ["notes"] = "注意",
    ["p2p_traffic"] = "P2Pトラフィックアラート",
    ["threshold_type"] = "コールバック",
    ["thresholds_single_source"] = " %{source} %{alt_name}設定",
    ["throughput"] = "スループットアラート",
  },
  ["arp_page"] = {
    ["arp_type"] = "ARPタイプ",
  },
  ["as_details"] = {
    ["as"] = "自律システム",
    ["as_parameter_missing_message"] = "自律システムパラメータが不足しています(内部エラー?)",
    ["as_timeseries_enable_message"] = "<A HREF=\"%{url}\">%{icon_flask} 設定</A>のAS時系列データは有効化可能です. 最初のデータポイントを見るためには数分必要です",
    ["no_available_data_for_as"] = "AS %{asn}に関してデータがありません",
  },
  ["as_stats"] = {
    ["autonomous_systems"] = "自律システム",
  },
  ["blog_feed"] = {
    ["blog_updates"] = "ブログアップデート",
    ["blog_updates_info"] = "新しいブログが投稿された時に通知を受信します: <a href='https://www.ntop.org/blog' target='_blank'>https://ntop.org/blog</a>",
    ["news_from_blog"] = "<b>ntop Blog</b> から新しいお知らせ",
    ["nothing_to_show"] = "新しいお知らせはありません。また明日!",
  },
  ["capabilities"] = {
    ["active_monitoring"] = "アクティブモニタ",
    ["checks"] = "ユーザースクリプト",
    ["developer"] = "開発者",
    ["flowdevices"] = "フローデバイス",
    ["notifications"] = "お知らせ",
    ["pools"] = "プール群",
    ["preferences"] = "環境設定",
    ["snmp"] = "SNMP",
  },
  ["categories_page"] = {
    ["search_application"] = "検索プロトコル",
    ["target"] = "ターゲット",
  },
  ["category_lists"] = {
    ["category_lists"] = "カテゴリーリスト",
    ["connection_time_out"] = "サーバー接続は %{duration}秒後にタイムアウトします",
    ["disable_some_list"] = "これを修正するには、いくつかのリストを無効にしてください。",
    ["download_succeeded"] = "リスト '%{name}' ダウンロードが成功しました",
    ["edit_list"] = "カテゴリリストの編集",
    ["enabled"] = "有効",
    ["error_occurred"] = "リスト '%{name}'のダウンロード中にエラーが発生しました:%{err}",
    ["http_code"] = "[HTTPコード:%{err_code}]",
    ["last_update"] = "最新の更新",
    ["list_will_be_updated"] = "リスト '%{name}'は30秒以内に更新されます。",
    ["loading_stats"] = "最新のリロード（%{when}）：%{num_hosts}ドメイン、%{num_ips} IPアドレス、%{duration}の%{num_ja3} JA3シグネチャをロードしました。",
    ["ready_for_update"] = "更新準備完了",
    ["server_returned_error"] = "サーバーがエラーを返しました",
    ["too_many_hosts_loaded"] = "ドメイン名の限界に達しました（ %{limit}ドメイン）",
    ["too_many_ips_loaded"] = "IPアドレスの限界に達しました（ %{limit}アドレス）",
    ["too_many_ja3_loaded"] = "JA3署名の限界に達しました（ %{limit}署名）",
    ["update_frequency"] = "更新頻度",
    ["update_now"] = "今すぐアップデート",
  },
  ["checks"] = {
    ["categories"] = "カテゴリー",
    ["category"] = "カテゴリー",
    ["category_internals"] = "インターナル",
    ["category_internals_descr"] = "ntopngの内部機能（メモリ管理、ホストおよびフローのライフサイクルなど）",
    ["category_network"] = "ネットワーク",
    ["category_network_descr"] = "ネットワークの動作と異常（特定のしきい値を超えるトラフィック、TCPが期待どおりに機能しないなど）",
    ["category_other"] = "その他",
    ["category_other_descr"] = "未分類のスクリプトまたは他のどのカテゴリーにも含めることができないスクリプトのデフォルトカテゴリー",
    ["category_security"] = "セキュリティ",
    ["category_security_descr"] = "セキュリティの動作と異常（例：ブラックリストに登録されたホストとの間の連絡先、TCPおよびUDPスキャン）",
    ["category_system"] = "システム",
    ["category_system_descr"] = "ntopngが実行されているシステムの機能（ディスクスペースがいっぱい、負荷が高すぎるなど）",
    ["disable_all"] = "全て無効",
    ["disable_all_checks"] = "全ユーザースクリプトを無効にする",
    ["disable_all_message"] = "現在設定に対する全ユーザースクリプトを無効とする.",
    ["elephant_flows_descr"] = ">%{l2r_bytes}（L2R）,>%{r2l_bytes}（R2L）",
    ["exceptions"] = "例外：%{exceptions}",
    ["long_lived_flows_descr"] = ">%{duration}",
    ["note_apply_to_default"] = "<b>デフォルト</ b>構成は、他の構成に含まれていないターゲットに「適用」されます",
    ["note_configsets"] = "ユーザースクリプトは、構成ごとに有効化/無効化できます。複数の構成を作成でき、各構成を異なるターゲットに「適用」できます。",
    ["note_what_are_checks"] = "<a href=\"%{checks_url}\">ユーザースクリプト</a>%{checks_external}は定期的に、または特定のイベントが発生したときに実行されます。",
    ["stop_recording_after"] = "%{duration}後に記録を停止します",
    ["hint"] = {
      ["body"] = "予期していない設定 <a href='%{link_DHCP}' target='_about'>DHCP</a>, <a href='%{link_SMTP}' target='_about'>SMTP</a>, <a href='%{link_DNS}' target='_about'>DNS</a>, <a href='%{link_NTP}' target='_about'>NTP</a> サーバー.",
      ["title"] = "予期していないサーバー",
    },
  },
  ["conf_backup"] = {
    ["conf_backup"] = "設定をバックアップ",
    ["conf_restore"] = "設定をリストア",
  },
  ["config_scripts"] = {
    ["applied_to"] = "適用先",
    ["config_apply"] = "構成を適用",
    ["config_clone"] = "クローニングの構成",
    ["config_clone_message"] = "クローンの名前を入力します",
    ["config_delete"] = "構成の削除",
    ["config_delete_message"] = "この構成を本当に削除しますか？",
    ["config_entities"] = "エンティティの構成",
    ["config_export"] = "構成のエクスポート",
    ["config_import"] = "構成のインポート",
    ["config_name"] = "構成名",
    ["config_rename"] = "構成の名前変更",
    ["config_rename_message"] = "新しい構成の名前を入力します",
    ["config_reset"] = "設定をリセット",
    ["config_reset_confirm"] = "すべての構成を削除して、デフォルト構成を初期値にリセットしますか？",
    ["config_search"] = "検索構成：",
    ["config_x"] = " %{product}構成",
    ["empty_target_message"] = "1つ以上のターゲットを選択してください！",
    ["empty_value_message"] = "入力ボックスを空にすることはできません！",
    ["invalid_characters"] = "名前に無効な文字が含まれています！",
    ["invalid_file"] = "有効な構成ファイルではありません",
    ["max_input_length"] = "名前に使用できる文字数は最大で16文字です。",
    ["no_file"] = "設定ファイルが選択されていません",
    ["no_pools_applied"] = "この設定は特定のターゲットには適用されません！",
    ["select_interface"] = "インターフェイスの選択",
    ["select_network"] = "ネットワークを選択",
    ["type_targets"] = "ターゲットのタイプ",
    ["type_targets_example"] = "コンマで区切られたターゲットを入力します。例：192.168.1.20,192.168.0.0/16",
    ["actions"] = {
      ["apply_to"] = "に適用",
      ["clone"] = "クローン",
      ["delete"] = "削除",
      ["edit"] = "編集",
      ["rename"] = "リネーム",
    },
    ["granularities"] = {
      ["flow"] = "フロー",
      ["host"] = "ホスト",
      ["interface"] = "インターフェイス",
      ["network"] = "ネットワーク",
      ["snmp_device"] = "SNMPデバイス",
      ["syslog"] = "シスログ",
      ["system"] = "システム",
    },
  },
  ["configsets"] = {
    ["bad_target"] = "不正な構成ターゲット： %{target}",
    ["duplicate_target"] = "ターゲット \"%{target} \"は、設定 \"%{confname1} \"と \"%{confname2} の両方に割り当てられます",
    ["error_exists"] = "構成名 \"T%{name} \"は既に使用されています",
    ["unknown_check"] = "不明なユーザースクリプト%{check}",
    ["unknown_id"] = "ID%{confid}の設定が見つかりません",
  },
  ["contacts_page"] = {
    ["client_address"] = "クライアント アドレス",
    ["client_contacts_initiator"] = "クライアントコンタクト (イニシエーター)",
    ["contacts"] = "コンタクト",
    ["no_client_contacts_so_far"] = "これまでクライアントのコンタクトはありません",
    ["no_contacts_message"] = "このホストにはコンタクトがありません",
    ["no_server_contacts_so_far"] = "これまでにサーバーのコンタクトはありません",
    ["server_address"] = "サーバーアドレス",
    ["server_contacts_receiver"] = "サーバーコンタクト（受信）",
  },
  ["containers_stats"] = {
    ["all_containers"] = "すべてのコンテナ",
    ["all_pods"] = "すべてのポッド",
    ["avg_rtt"] = "平均RTT",
    ["avg_rtt_as_client"] = "クライアントとしての平均RTT",
    ["avg_rtt_as_server"] = "サーバーとしての平均RTT",
    ["avg_rtt_variance"] = "平均RTT分散",
    ["avg_rtt_variance_as_client"] = "クライアントとしての平均RTT分散",
    ["avg_rtt_variance_as_server"] = "サーバーとしての平均RTT分散",
    ["client_container"] = "クライアントコンテナ",
    ["client_pod"] = "クライアントポッド",
    ["client_rtt"] = "クライアントRTT",
    ["container"] = "コンテナ",
    ["containers"] = "コンテナ",
    ["containers_list"] = "コンテナリスト",
    ["containers_of_pod"] = "ポッドのコンテナ",
    ["pod"] = "ポッド",
    ["pods"] = "ポッド",
    ["pods_list"] = "ポッドリスト",
    ["server_container"] = "サーバーコンテナ",
    ["server_pod"] = "サーバーポッド",
    ["server_rtt"] = "サーバーRTT",
  },
  ["country_details"] = {
    ["country"] = "国",
    ["country_parameter_missing_message"] = "国のパラメータが不足しています (内部エラー?)",
    ["no_available_stats_for_country"] = "%{country}に関して統計データがありません",
  },
  ["custom_categories"] = {
    ["add"] = "追加",
    ["add_application"] = "アプリケーションを追加",
    ["add_custom_app"] = "カスタムアプリケーションを追加",
    ["add_to_categories"] = "カテゴリに追加",
    ["app_delete_error"] = "削除できません \"%{app}\" アプリケーション. 再起動してください %{product} ペンディングした変更がないか確認してください.",
    ["app_deleted"] = "アプリケーション \"%{app}\" 削除されました. 再起動してください %{product} 変更を適用する為に.",
    ["application_exists"] = "アプリケーション \"%{app} \"は既に存在します！",
    ["application_name"] = "アプリケーション名",
    ["application_not_exists"] = "アプリケーション \"%{app} \"は存在しません！",
    ["apps_and_categories"] = "アプリケーションとカテゴリ",
    ["custom_host_category"] = "カスタムホストカテゴリ",
    ["custom_hosts"] = "カスタムホスト",
    ["delete_app"] = "アプリケーションを削除",
    ["delete_app_confirm"] = "<b>警告</b>: アプリケーションの削除は, カスタムアプリケーションに定義した履歴データに影響を及ぼし操作を破壊する可能性があります. 本当に削除しますか %{app}?",
    ["domain_names_substrings"] = "ドメイン名は一致する部分文字列として解釈されます<br>例 \"%{s1}\" は \"%{s2}\" と \"%{s3}\"にマッチします",
    ["each_host_separate_line"] = "各ホストは異なる行に設定する必要があります",
    ["edit_custom_rules"] = "カスタムルールの編集",
    ["edit_hosts"] = "ホストを編集",
    ["example_port_range"] = "ポート範囲の例:\"%{example1} \"、\"%{example2} ",
    ["host_domain_or_cidr"] = "ホストはドメイン名かIPv4アドレス/CIDRネットワークアドレスである必要があります",
    ["host_domain_or_port"] = "ルールは、ドメイン名、IPv4アドレス、またはTCP/UDPポート範囲のいずれかです。",
    ["invalid_rule"] = "無効なルールは無視されました :%{rule}",
    ["new_app_added"] = "アプリケーション \"%{app} \"が設定されました。 %{product}を再起動して使用を開始してください。",
    ["non_empty_list_required"] = "少なくとも1つのルールは設定する必要があります.",
    ["option_needed"] = "<i>%{option} </ i>パラメーターが設定にないため、カスタムアプリケーションの設定は現在無効になっています。詳細については、<a href=\"%{url}\">ドキュメント</a>をご覧ください。",
    ["protos_reboot_necessary"] = "アプリケーションルールが作成されています。それらを適用するには、%{product}を再起動してください。",
    ["protos_unexpected_error"] = "アプリケーションルールの更新中に予期しないエラーが発生しました。詳細については、 %{product}ログをご覧ください。",
    ["select_url_category"] = "ドメインに付与するカテゴリを指定してください:",
    ["similar_host_found"] = " \"%{host}\"に似たホストがすでにカテゴリ %{category} に含まれています",
    ["the_following_is_a_list_of_hosts"] = "以下は%{category} のカスタムホストのリストです",
    ["the_following_is_a_list_of_hosts_app"] = "以下は、 %{application}アプリケーションのカスタムルールのリストです。",
    ["the_following_url_will_be_added"] = "以下のドメインが追加されます:",
  },
  ["dashboard"] = {
    ["actual_traffic"] = "実トラフィック",
    ["community_dashboard"] = "サンキ―ダイアグラム",
    ["gateways_users"] = "ゲートウェイとユーザー",
    ["lastday_app_traffic"] = "昨日の上位アプリケーション",
    ["lastday_traffic"] = "インターフェイス統計: 昨日",
    ["realtime_app_traffic"] = "リアルタイム上位アプリケーション",
    ["realtime_traffic"] = "インターフェイス統計: リアルタイムトラフィック",
    ["slow_dashboard_message"] = " %{product}応答の待ち時間が長すぎます。より高い <a href='%{url}'>リアルタイム統計リフレッシュレート</a>を設定することを検討してください。",
    ["top_local_talkers"] = "上位ローカルトーカー",
    ["top_remote_destinations"] = "上位リモート宛先",
    ["traffic_dashboard"] = "リアルタイム ダッシュボード",
  },
  ["datasources"] = {
    ["families"] = "ファミリ",
    ["key"] = "キー",
    ["metrics"] = "メトリック",
    ["new_source"] = "新しいソース",
    ["origin"] = "オリジン",
    ["retention"] = "保持",
    ["schema"] = "スキーマ",
    ["scope"] = "スコープ",
    ["select_family"] = "ファミリの選択",
    ["select_metrics"] = "メトリックの選択",
    ["select_schema"] = "スキーマの選択",
    ["select_schema_first"] = "最初にスキーマを選択してください",
    ["select_series"] = "シリーズを選択",
    ["series"] = "シリーズ",
    ["source_exists"] = "この様なソースは既に存在します!",
  },
  ["db_explorer"] = {
    ["abort"] = "中止",
    ["all"] = "すべて",
    ["app_between_talkers_pairs"] = "トーカーのペア間のアプリケーション",
    ["app_flows_between"] = "%{peer1} と %{peer2}の間の%{app} アプリケーションフロー",
    ["app_talkers"] = "%{app} トーカー",
    ["app_talkers_with"] = "%{app} とトーカー %{peer}",
    ["application_flows"] = "アプリケーションフロー",
    ["applications_between"] = " %{peer1} と %{peer2}の間のアプリケーション",
    ["average_throughput"] = "平均スループット",
    ["client_server_breakdown"] = "クライアント vs サーバー トラフィック分解",
    ["client_server_host"] = "クライアント/サーバー ホスト",
    ["dns_query"] = "DNSクエリ",
    ["download_flows"] = "ダウンロードフロー",
    ["download_flows_limit"] = "最大1000レコード.",
    ["empty_query_response"] = "空のクエリ応答 (データベースがダウンしたかクエリがキルされたかタイムアウト?)",
    ["extract_pcap"] = "pcap抽出",
    ["filter_bpf"] = "フィルター (BPF)",
    ["first_last_seen"] = "最初/最後",
    ["flow_peers"] = "フローピア",
    ["get_proto_talkers"] = "このプロトコルを利用するトーカーを取得",
    ["historical_data_explorer"] = "履歴データ検索",
    ["host_name"] = "ホスト名",
    ["host_talkers"] = "%{host} トーカー",
    ["hosts_talking_proto_with"] = "%{proto} を利用して {host}と通信しているホスト",
    ["http_host"] = "HTTPホスト",
    ["http_method"] = "HTTPメソッド",
    ["interface_name"] = "インスタンス名",
    ["last_hour_flows"] = "前回確認したフロー",
    ["ms_client"] = "%{client} ms (クライアント)",
    ["ms_server"] = "%{server} ms (サーバー)",
    ["network_latency_breakdown"] = "ネットワーク遅延分解",
    ["observation_period"] = "観測期間",
    ["ok_request_sent"] = "OK,リクエスト送信",
    ["packet_rate"] = "平均パケットレート",
    ["pcap_requests_and_status"] = "Pcapのリクエストとステータス",
    ["pcaps"] = "Pcap",
    ["protocol_flows_between"] = " %{peer1} と %{peer2}の%{proto} プロトコルフロー",
    ["protocol_id"] = "プロトコルID",
    ["query_in_progress"] = "データベースクエリ進行中, 結果受信待ち...",
    ["request_failed"] = "リクエスト失敗",
    ["response_code"] = "応答コード",
    ["search_flows"] = "検索フロー",
    ["selected_saved"] = "保存済みを選択",
    ["server_name"] = "サーバー名",
    ["summary"] = "サマリー",
    ["talkers_with_this_host"] = "このホストのトーカー",
    ["task_id"] = "タスクID",
    ["this_flow_has_been_reset"] = "このフローはリセットされました。 サーバーアプリケーションがダウンしている可能性があります",
    ["this_flow_is_active"] = "このフローはアクティブです",
    ["this_flow_is_completed"] = "このフローは完了しており、すぐに有効期限切れになります",
    ["top_flows_ipv4"] = "上位IPv4フロー [%{date_from} - %{date_to}]",
    ["top_flows_ipv6"] = "上位IPv6フロー [%{date_from} - %{date_to}]",
    ["top_proto_ipv4_flows"] = "上位 %{proto} IPv4フロー",
    ["top_proto_ipv6_flows"] = "上位 %{proto} IPv6フロー",
    ["total_flows"] = "フロー合計",
    ["total_packets"] = "パケット合計",
    ["total_traffic"] = "バイト合計",
    ["traffic_rate"] = "平均トラフィックレート",
    ["traffic_received"] = "バイト受信",
    ["traffic_sent"] = "バイト送信",
    ["traffic_sent_to_cli"] = "クライアントへバイト送信",
    ["traffic_sent_to_srv"] = "サーバーへバイト送信",
    ["traffic_volume"] = "トラフィックボリューム",
    ["unable_to_find_flow"] = "指定されたフローを発見できませんでした",
    ["unsave"] = "削除",
    ["vlan_id"] = "VLAN ID",
  },
  ["delete_data"] = {
    ["delete_active_interface_confirmation"] = "%{ifname}に関連したすべての永続データを削除しますがよろしいですか? %{product}再起動時にデータ削除は実行されます. 削除を実行するために %{product} を再起動してください",
    ["delete_active_interface_data_failed"] = "アクティブなインターフェイスデータの削除は不可能です",
    ["delete_active_interface_data_ok"] = "%{ifname}に関するデータは削除対象にマークされました。削除を実行するために %{product}を再起動してください",
    ["delete_confirmation"] = "%{host}に関連したすべての永続データを削除しますがよろしいですか?",
    ["delete_failed"] = "%{host}に関するデータの削除が不可能です",
    ["delete_inactive_interfaces_confirmation"] = "%{interfaces_list}はインアクティブです。これらのすべての永続データを削除しまいますがよろしいですか?",
    ["delete_inactive_interfaces_data_failed"] = "インアクティブなインターフェイスのデータの削除は不可能です",
    ["delete_inactive_interfaces_data_ok"] = "インアクティブなインターフェイスに関するデータを削除しました",
    ["delete_ok"] = "%{host} に関するデータを削除しました",
    ["msg_err_invalid_host"] = "指定されたホストのフォーマットが認識できませんでした",
    ["msg_err_no_fs_data"] = "指定されたホストに関する永続データは見つかりませんでした",
    ["msg_err_unable_to_delete_dir"] = "データディレクトリの削除は不可能です",
    ["msg_err_unable_to_delete_ts_data"] = "時系列データの削除は不可能です",
    ["msg_err_unknown"] = "不明なエラーが発生しました",
    ["node_nindex_flows"] = "警告：ndumpはフローダンプに対して有効になっています。フローの履歴情報は削除されません。",
    ["note_persistent_data"] = "すべての永続データが削除されました。データには蓄積された時系列データ, フロー, アラート, そして Redisキャッシュを含みます",
  },
  ["details"] = {
    ["additional_mac_address"] = "追加MACアドレス",
    ["address_resolution_protocol"] = "アドレス解決プロトコル",
    ["ago"] = "前",
    ["alerts"] = "アラート",
    ["arp_replies"] = "ARP応答",
    ["arp_requests"] = "ARP要求",
    ["as_client"] = "As クライアント",
    ["as_server"] = "As サーバー",
    ["details"] = "詳細",
    ["device_id"] = "デバイス ID",
    ["device_port"] = "デバイス ポート",
    ["device_type"] = "デバイス タイプ",
    ["first_last_seen"] = "最初/最後",
    ["first_observed_on"] = "最初に観測された",
    ["flow_snmp_localization"] = "フローSNMPローカリゼーション",
    ["flows_dropped_by_bridge"] = "トラフィックポリシーによりブロックされたフロー",
    ["flows_non_packet_iface"] = "直近のアクティブなフロー / 合計",
    ["flows_packet_iface"] = "アクティブなフロー / アクティブな総数 / 低グッドプット",
    ["fragments"] = "フラグメント",
    ["further_host_names_information"] = "その他のホスト名/情報",
    ["goodput"] = "グッドプット",
    ["host_pool"] = "ホストプール",
    ["host_purged"] = "このホストはもうアクティブではなく、メモリから削除されているはずです",
    ["host_snmp_localization"] = "SNMPローカリゼーション",
    ["input_device_port"] = "デバイス入力ポート",
    ["keep_alive"] = "キープアライブ",
    ["label_blacklisted_host"] = "ブラックリストに登録されたホスト",
    ["label_dhcp_server"] = "DHCP サーバー",
    ["label_dns_server"] = "DNS サーバー",
    ["label_local_host"] = "ローカルホスト",
    ["label_ntp_server"] = "NTP サーバー",
    ["label_private_ip"] = "プライベートIP",
    ["label_remote"] = "リモート",
    ["label_smtp_server"] = "SMTP サーバー",
    ["label_system_ip"] = "システムIP",
    ["lost"] = "ロスト",
    ["misbehaving_flows_reasons"] = "フローの誤動作の理由",
    ["note"] = "注意",
    ["note_hosts_located_snmp_device"] = "ホストは<A HREF=%{url}>Bridge MIB</A>を利用してSNMP デバイスとして位置が特定されました",
    ["out_of_order"] = "故障中",
    ["output_device_port"] = "デバイス出力ポート",
    ["peers"] = "ピア：アクティブ",
    ["rcvd"] = "受信",
    ["retransmissions"] = "再送",
    ["router_access_point_mac_address"] = "(ルーター/アクセスポイント) MACアドレス",
    ["sent_vs_received_traffic_breakdown"] = "送信 vs 受信トラフィック分解",
    ["show_hosts"] = "ホストを表示",
    ["source"] = "ソース",
    ["source_id"] = "ソースID",
    ["tcp_packets_sent_analysis"] = "TCPパケット送信分析",
    ["total_traffic"] = "合計トラフィック",
    ["traffic_sent_received"] = "トラフィック 送信 / 受信",
    ["vlan_id"] = "VLAN ID",
    ["whois_lookup"] = "ルックアップは誰か",
  },
  ["developer_section"] = {
    ["datasources_list"] = "データソース",
    ["widgets_list"] = "ウィジェット",
  },
  ["device_protocols"] = {
    ["alert"] = "トリガーアラート",
    ["device_protocol_policy"] = "デバイスプロトコルポリシー",
    ["device_protocols"] = "デバイスプロトコル",
    ["devtype_as_proto_client"] = "%{devtype} as <i>%{proto}</i> クライアント",
    ["devtype_as_proto_server"] = "%{devtype} as <i>%{proto}</i> サーバー",
    ["filter_device_protocols"] = "%{filter} デバイス プロトコル",
    ["ok"] = "OK",
  },
  ["device_types"] = {
    ["iot"] = "IoT",
    ["laptop"] = "ノートパソコン",
    ["multimedia"] = "マルチメディア",
    ["nas"] = "NAS",
    ["networking"] = "ルーター/スイッチ",
    ["phone"] = "電話",
    ["printer"] = "プリンタ",
    ["tablet"] = "タブレット",
    ["tv"] = "TV",
    ["unknown"] = "不明",
    ["video"] = "ビデオ",
    ["wifi"] = "ワイヤレスネットワーク",
    ["workstation"] = "コンピュータ",
  },
  ["dhcp"] = {
    ["delete_range"] = "DHCP範囲を削除",
    ["delete_range_confirm"] = "設定されたDHCP範囲%{range}を本当に削除しますか？",
    ["dhcp"] = "DHCP",
    ["dhcp_alert_note"] = "設定されたDHCP範囲外でIPアドレスが割り当てられると、アラートが生成されます",
    ["dhcp_configuration_note"] = "このページには、このインターフェイスに構成されたDHCP範囲がリストされます。これらは、使用中の実際のDHCPサーバー構成と一致する必要があります",
    ["invalid_ip_address"] = "無効なIPアドレス",
    ["no_dhcp_ranges"] = "DHCP範囲が構成されていません。",
  },
  ["discover"] = {
    ["device"] = "デバイス",
    ["discovered_devices"] = "%{manuf} %{sys} %{dev} デバイスが発見されました",
    ["err_unable_to_arp_discovery"] = "ARPネットワークディスカバリーを開始できませんでした",
    ["error_no_discovery_cached"] = "ディスカバリーデータはまだキャッシュされていません",
    ["error_unable_to_decode_json"] = "JSON形式のディスカバリーデータをデコードできません.",
    ["ghost_icon_descr"] = "%{ghost_icon} アイコンはゴーストホストをハイライトしています (例. それらはIPアドレスネットワークに所属していません)",
    ["network_discovery"] = "ネットワーク探索",
    ["network_discovery_datetime"] = "前回のネットワーク探索時刻",
    ["network_discovery_not_enabled"] = "ネットワーク探索を実行中です",
    ["too_many_devices_descr"] = "検出されたデバイスが多すぎます。 %{max_devices}台のデバイスで結果がトリミングされました",
  },
  ["dns_page"] = {
    ["dns_breakdown"] = "DNS分解",
    ["dns_query_sent_distribution"] = "DNSクエリ送信",
    ["dns_query_sent_vs_rcvd_distribution"] = "送信されたDNSクエリvsRcvdの分布",
    ["error_replies"] = "エラー応答",
    ["note"] = "注意",
    ["note_dns_ratio"] = "理想的にはDNSのリクエスト／応答の比率は1です (１つのリクエストに対して１つの応答). これよりもずっと低い値の場合には調査をする必要があります。なぜなら、応答が期待よりも少なく、応答しないDNSを利用しているか設定ミスの可能性があります。(例. 別のIPに移動している)",
    ["positive_replies"] = "ポジティブな回答",
    ["queries"] = "問い合わせ",
    ["ratio"] = "比",
    ["rcvd"] = "受信",
    ["replies"] = "リプライ",
    ["reply_breakdown"] = "リプライ分解",
    ["request_vs_reply"] = "リクエストとリプライの比率",
  },
  ["dscp_page"] = {
    ["statistics"] = "優先順位クラス",
    ["statistics_received"] = "受信",
    ["statistics_sent"] = "送信",
  },
  ["email_alert_endpoint"] = {
    ["carbon_copy"] = "CC",
    ["email_recipient"] = "Email受信者",
    ["email_sender"] = "Email送信者",
    ["smtp_password"] = "SMTPパスワード",
    ["smtp_server"] = "SMTPサーバー",
    ["smtp_username"] = "SMTPユーザー名",
    ["validation"] = {
      ["empty_SMTP_server"] = "IPv4/IPv6/ホストアドレスを入力してください.",
      ["empty_email"] = "emailアドレスを入力してください.",
      ["invalid_SMTP_server"] = "有効なIPv4/IPv6/ホストアドレスを入力してください.",
      ["invalid_email"] = "有効なemailアドレスを入力してください(例.name@domain.com).",
    },
  },
  ["endpoint_notifications"] = {
    ["actions"] = "アクション",
    ["add_new_endpoint"] = "通知先の追加",
    ["add_new_recipient"] = "受信者の追加",
    ["associated_to"] = "関連",
    ["associated_to_endpoints"] = "エンドポイントに紐づけました",
    ["associated_to_recipients"] = "受信者に紐づけました",
    ["create_endpoint_first"] = "最初に通知エンドポイントを作成してください!",
    ["edit_endpoint"] = "通知先の編集",
    ["edit_endpoints"] = "複数のエンドポイントの編集",
    ["edit_recipient"] = "受信者の編集",
    ["empty_endpoints"] = "エンドポイントが定義されていません.",
    ["empty_template"] = "受信者のパラメータが入力されていません.",
    ["endpoint"] = "エンドポイント",
    ["endpoint_config_already_existing"] = "同じ名前の設定が存在します!",
    ["endpoint_list"] = "通知先リスト",
    ["endpoint_recipient_already_existing"] = "同じ名前の受信者が存在します",
    ["enpoint_recipients_list"] = "受信者リスト",
    ["get_users_recipient_failed"] = "申し訳ございません, 受信ユーザーのロードに失敗しました. 再度試してください.",
    ["name"] = "通知先名",
    ["notifications"] = "通知",
    ["num_drops"] = "ドロップ",
    ["num_uses"] = "利用時間",
    ["recipient_category"] = "カテゴリフィルタ",
    ["recipient_list"] = "受信者リスト",
    ["recipient_name"] = "受信者名",
    ["recipient_select"] = "通知先の選択",
    ["recipient_severity"] = "最小重要度",
    ["recipient_type"] = "受信者リスト",
    ["recipient_users"] = "受信者に使っているプール %{recipient}",
    ["remove_endpoint"] = "通知先の削除",
    ["remove_endpoint_message"] = "本当にこの通知先を削除してもよろしいですか?",
    ["remove_recipient"] = "受信者の削除",
    ["remove_recipient_message"] = "本当にこの受信者を削除してもよろしいですか?",
    ["select_type"] = "エンドポイントタイプの選択",
    ["server_error"] = "Something went wrong. Try later.",
    ["test_recipient"] = "テスト受信",
    ["testing_recipient"] = "受信のテスト中",
    ["time_since_last_use"] = "最後の利用からの時間",
    ["timed_out"] = "受信テストはタイムアウトしました!",
    ["type"] = "通知先タイプ",
    ["used_by_recipients"] = "受信者によって利用された",
    ["working_recipient"] = "テストが成功しました!",
    ["zero_users_recipient"] = "この受信者に対するプールが存在しません.   %{link} ページからプールを追加してください.",
    ["factory_reset"] = {
      ["body"] = "再起動してもいいですか？ <b>%{reset_element}</b>?<br>工場出荷時のデフォルトにリセット <b>恒久的に設定された %{reset_element} </b> を失います.",
      ["title"] = "設定のリセット: %{reset_element}",
    },
    ["hints"] = {
      ["bind_pools"] = {
        ["body"] = "最後のステップ! アラートの配信を開始する為にプールに受信者をバインドしてください.",
        ["title"] = "受信者のバインド",
      },
      ["create_endpoint"] = {
        ["action"] = "作成",
        ["body"] = "<a target='_blank' href='%{link}'>エンドポイント</a> の作成. 外部にアラート配送の開始.",
        ["title"] = "エンドポイントの作成",
      },
      ["create_recipients"] = {
        ["body"] = "新しい <a target='_blank' href='%{link}'>受信者</a> の作成. アラート配信の開始.",
        ["title"] = "受信者の作成",
      },
    },
    ["recipients"] = {
      ["cc"] = "CC",
      ["email_recipient"] = "Email 受信者",
    },
    ["severities"] = {
      ["alert"] = "アラート",
      ["critical"] = "Critical",
      ["debug"] = "Debug",
      ["emergency"] = "Emergency",
      ["error"] = "Error",
      ["info"] = "Info",
      ["notice"] = "Notice",
      ["warning"] = "Warning",
    },
    ["validation"] = {
      ["abort_add_endpoint"] = "新しいエンドポイントの追加を中止します?",
      ["abort_add_recipient"] = "新しいエンドポイント受信者の追加を停止しますか?",
      ["abort_edit_endpoint"] = "エンドポイントの編集を停止しますか?",
      ["abort_edit_recipient"] = "エンドポイント受信者の編集を中止しますか?",
    },
  },
  ["entity_thresholds"] = {
    ["flow_attacker_description"] = "最大送信フロー/秒がオーバーしました。フラッダーと考えられます",
    ["flow_attacker_title"] = "フローフラッド攻撃者",
    ["flow_victim_description"] = "最大受信フロー/秒がオーバーしました。 フラッド攻撃を受けていると考えられます",
    ["flow_victim_title"] = "フローフラッド犠牲者",
    ["request_reply_ratio_description"] = "（異なるアプリケーションでの）応答数と要求の比率がしきい値を超えたときにアラートをトリガーする",
    ["request_reply_ratio_title"] = "返信/リクエスト率",
    ["syn_attacker_description"] = "最大送信TCP SYN パケット/秒がオーバーしました。SYNフラッダーと考えられます",
    ["syn_attacker_title"] = "SYNフラッド攻撃者",
    ["syn_scan_attacker_description"] = "送信されたSYNの数/分（応答なし）がしきい値を超えたときにアラートをトリガーする",
    ["syn_scan_attacker_title"] = "SYNスキャン攻撃者アラート",
    ["syn_scan_victim_description"] = "受信したSYNの数/分（応答なし）がしきい値を超えたときにアラートをトリガーする",
    ["syn_scan_victim_title"] = "SYNスキャン被害者アラート",
    ["syn_victim_description"] = "最大受信TCP SYN パケット/秒がオーバーしました。SYNフラッド攻撃を受けていると考えられます",
    ["syn_victim_title"] = "SYNフラッドの犠牲者",
  },
  ["error_page"] = {
    ["go_back"] = "戻る",
    ["greeting"] = "エラーが検知されました",
    ["presence"] = "ここにいるのは間違いです!",
  },
  ["event_exporters"] = {
    ["event_device_x"] = " %{device}イベントエクスポーター",
    ["event_exporter_ip"] = "イベントエクスポーターIP",
    ["event_exporters"] = "イベントエクスポーター",
  },
  ["export_data"] = {
    ["export_json_data"] = "エクスポートJSONデータ",
    ["note_active_hosts"] = "アクティブなホストのみリターンしました",
    ["note_maximum_number"] = "ダウンロード可能なホストの最大数は32,768です",
  },
  ["external_stats"] = {
    ["capture_drops"] = "ドロップされたパケット",
    ["capture_packets"] = "キャプチャされたパケット",
    ["signatures_failed"] = "署名に失敗しました",
    ["signatures_loaded"] = "ロードされた署名",
    ["title"] = "外部統計",
  },
  ["field_units"] = {
    ["bytes"] = "バイト",
    ["flow_sec"] = "フロー/秒",
    ["flows"] = "フロー",
    ["hosts"] = "ホスト",
    ["mbits"] = "Mbps",
    ["packets"] = "パケット",
    ["percentage"] = "%%",
    ["seconds"] = "秒",
    ["syn_min"] = "SYN/分",
    ["syn_sec"] = "SYN/秒",
  },
  ["flow_alerts_explorer"] = {
    ["by_target_port"] = "ターゲットポート別",
    ["flow_alert_origin"] = "オリジンアラート",
    ["flow_alert_target"] = "アラートターゲット",
    ["label"] = "アラートエクスプローラ",
    ["origins"] = "オリジン",
    ["search"] = "アラート検索",
    ["summary_cli2srv"] = "オリジンからターゲットへのトラフィックの総計",
    ["summary_n_origins"] = "オリジンの総計",
    ["summary_n_targets"] = "ターゲットの総計",
    ["summary_srv2cli"] = "オリジントラフィックへのターゲットの総計",
    ["summary_total"] = "フローアラートの総計",
    ["targets"] = "ターゲット",
    ["type_alerts_by_type"] = "タイプ別のフローアラート",
    ["type_explorer"] = "タイプエクスプローラ",
    ["visual_explorer"] = "ビジュアルエクスプローラ",
  },
  ["flow_checks"] = {
    ["application_detected"] = "検出されたアプリケーション",
    ["callback"] = "ユーザースクリプト",
    ["callback_config"] = "構成",
    ["callback_elapsed_time"] = "実行時間",
    ["callback_elapsed_time_avg"] = "フロー/秒",
    ["callback_enabled"] = "有効",
    ["callback_function"] = "フック",
    ["callback_function_duration"] = "期間",
    ["callback_function_duration_fmt_long"] = "%{num_calls} %{time}の呼び出し [%{speed} コール/秒]",
    ["callback_function_duration_fmt_short"] = "\"%{time}",
    ["callback_function_num_flows"] = "フロー",
    ["callback_function_throughput"] = "処理速度",
    ["callback_latest_run"] = "最新の実行",
    ["dns_esclusion_list"] = "除外リスト",
    ["dns_esclusion_list_example"] = "無効なDNSアラートから除外するサブドメインのリスト。カンマで区切ってください",
    ["idle"] = "アイドル状態",
    ["last_calls_per_sec"] = "最新のコール/秒",
    ["last_duration"] = "最新の期間",
    ["last_num_calls"] = "前回のコール数",
    ["no_callbacks_available"] = "利用可能なコールバックはありません。",
    ["no_callbacks_available_disabled_alerts"] = "利用可能なコールバックはありません。コールバックでは、 <a href=\"%{url}\"> <i class = \"fas fa-flask \"> </ i>設定</a>からアラートを有効にする必要があります。",
    ["no_callbacks_defined"] = "コールバックが見つかりません",
    ["note_add_custom_scripts"] = "%{product}は、これらの <a href=\"%{url}\"> <i class =  \"fas fa-folder \"> </ i>ディレクトリ</a>でカスタムスクリプトを探します。",
    ["note_create_custom_scripts"] = "コールバックを実行するためのカスタムスクリプトを作成するためのドキュメントは、次の <a class='ntopng-external-link' href=\"%{url}\">リンク <i class =  \"fas fa-external-link-alt \">で入手できます</ i></a>。",
    ["note_flow_application_detected"] = " \"検出されたアプリケーション \" コールバックは、アプリケーションの検出直後に実行されます。これは、パケットインターフェイスのフローの最初の12パケット内で、またはZMQインターフェイスのフローが受信された直後に発生します。",
    ["note_flow_idle"] = "\"アイドル\" コールバックは、フローがアクティビティを終了し、アイドル状態になったときに実行されます。",
    ["note_flow_lifecycle"] = "コールバックは、フローのライフサイクルの特定の瞬間に実行されます。",
    ["note_flow_periodic_update"] = " \"定期的な更新 \" コールバックは、フローが5分以上アクティブになった後、定期的に実行されます。",
    ["note_flow_staus_changed"] = " \"ステータスの変更 \"コールバックは、フローにステータスの変更があるたびに実行されます。例：再送信などの接続の問題が発生した場合",
    ["note_scripts_list"] = "利用可能な %{product}スクリプトの完全なリストは、 <a href=\"%{url}\">スクリプト</a>ページにあります。",
    ["notes"] = "注意",
    ["periodic_update"] = "定期的な更新",
    ["status_changed"] = "ステータスが変更されました",
  },
  ["flow_checks_config"] = {
    ["blacklisted"] = "ブラックリストに登録されたフロー",
    ["blacklisted_description"] = "ブラックリストに登録されたホストまたはドメインが検出されたときにアラートをトリガーします",
    ["data_exfiltration"] = "データ流出",
    ["data_exfiltration_description"] = "データ流出の可能性があるアクティビティが検出されたときにアラートをトリガーする",
    ["dev_proto_not_allowed"] = "許可されていないデバイスアプリケーション",
    ["dev_proto_not_allowed_description"] = "デバイスで異常なアプリケーションが検出されたときにアラートをトリガーします。ルールは <a href='%{url}'>こちら</a>で構成できます",
    ["dev_proto_not_allowed_nedge_description"] = " <i>未割り当て</ i>デバイスで許可されていないアプリケーションが検出されたときにアラートをトリガーします。ポリシーは <a href='%{url}'>ここ</a>で設定できます。",
    ["dns_data_exfiltration"] = "DNSデータの流出",
    ["dns_data_exfiltration_description"] = "DNSデータの流出アクティビティが検出されたときにアラートをトリガーする",
    ["elephant_flows"] = "エレファントフロー",
    ["elephant_flows_description"] = "フローが設定されたバイト数を超えて交換したときにアラートをトリガーします",
    ["ext_alert"] = "外部アラート",
    ["ext_alert_description"] = "外部エンドポイント（例：Suricata）からフローアラートを受信します",
    ["flow_risk"] = "フローリスク",
    ["flow_risk_description"] = "検知したnDPIフローリスクを基にアラートをトリガーする",
    ["flow_risk_list"] = "nDPIフローリスク:",
    ["invalid_dns_query"] = "無効なDNSクエリ",
    ["invalid_dns_query_description"] = "悪意のある可能性のあるDNSクエリが検出されたときにアラートをトリガーする",
    ["long_lived"] = "持続的な",
    ["long_lived_description"] = "フローが設定された期間を超えて継続したときにアラートをトリガーします",
    ["malicious_signature"] = "悪意のある署名",
    ["malicious_signature_description"] = "悪意のある可能性のある署名が検出されたときにアラートをトリガーする",
    ["nedge_blocked"] = "ブロックされたフロー",
    ["nedge_blocked_description"] = "設定されたポリシーが原因でフローがブロックされたときにアラートをトリガーします",
    ["no_input"] = "設定不要",
    ["not_purged"] = "パージされない",
    ["not_purged_description"] = "フローパージロジックのバグを検出します",
    ["old_tls_version"] = "古いTLSバージョン",
    ["old_tls_version_description"] = "古い（安全でない可能性のある）TLSバージョンが検出されたときにアラートをトリガーします",
    ["potentially_dangerous_protocol"] = "潜在的に危険なプロトコル",
    ["potentially_dangerous_protocol_description"] = "潜在的に危険なプロトコルが検出されたときにアラートをトリガーします",
    ["remote_to_remote"] = "リモートからリモートへのフロー",
    ["remote_to_remote_description"] = "フローにクライアントおよびサーバーとしてリモートホストがある場合にアラートをトリガーします",
    ["score"] = "フロースコア",
    ["suspicious_countries"] = "疑わしい国",
    ["suspicious_countries_description"] = "少なくとも1つのクライアントとサーバーが不審な国からのものである場合にアラートをトリガーします",
    ["tcp_issues_generic"] = "TCP再送信の問題",
    ["tcp_issues_generic_description"] = "TCP再送信、故障、パケット損失の問題を検出する",
    ["tcp_issues_packets"] = "TCP接続の問題",
    ["tcp_issues_packets_description"] = "TCPの低速データ交換、低グッドプット、接続拒否を検出",
    ["tcp_probing"] = "TCPプロービング",
    ["tcp_probing_description"] = "プローブの試みる可能性が検出されたときにアラートを発報する",
    ["tls_cert_expired"] = "TLS証明書の期限切れ",
    ["tls_cert_expired_description"] = "期限切れのTLS証明書が検出されたときにアラートを発報する",
    ["tls_cert_mismatch"] = "TLS証明書の問題",
    ["tls_cert_mismatch_description"] = "一致しないTLS証明書が検出されたときにアラートを発報する",
    ["tls_cert_selfsigned"] = "TLS証明書自己署名",
    ["tls_cert_selfsigned_description"] = "自己署名TLS証明書が検知された場合にアラートを発報する",
    ["tls_unsafe_ciphers"] = "安全でないTLS暗号化",
    ["tls_unsafe_ciphers_description"] = "安全でないTLS暗号化が検出されたときにアラートを発報する",
    ["udp_unidirectional"] = "単方向UDPフロー",
    ["udp_unidirectional_description"] = "単方向UDPフローが検出されたときにアラートを発報する",
    ["web_mining"] = "Webマイニング",
    ["web_mining_description"] = "暗号通貨マイニングを実行することがわかっているホストからのトラフィックが検出されたときに生成されるアラートを切り替えます",
  },
  ["flow_details"] = {
    ["acceptable_label"] = "受け入れ",
    ["actual_peak_throughput"] = "実際/ピークスループット",
    ["additional_alert_type"] = "追加フローステータス",
    ["additional_flow_elements"] = "追加のフロー要素",
    ["alert_type"] = "フロー状態",
    ["alerted_flows"] = "フローアラート",
    ["alerted_flows_severity"] = "アラートされたフローの重要度",
    ["application_latency"] = "アプリケーション遅延",
    ["as_prev_next"] = "AS 前回 / 次",
    ["as_src_dst"] = "AS 送信元 / 宛先",
    ["average_cpu_load"] = "平均CPU負荷",
    ["bittorrent_hash"] = "BitTorrentハッシュ",
    ["blacklisted_flow"] = "クライアント、サーバー、またはドメインはブラックリストに登録されています",
    ["blacklisted_flow_detailed"] = "ブラックリストに登録された %{who}",
    ["c_ip_addresses"] = "C IPアドレス",
    ["call_canceled"] = "コールがキャンセルされました",
    ["call_completed"] = "コール完了",
    ["call_error"] = "コールエラー",
    ["call_id"] = "コールID",
    ["call_initiator"] = "Callイニシエータ",
    ["call_quality_indicators"] = "コール品質インジケータ",
    ["call_started"] = "コール開始",
    ["call_state"] = "コール状態",
    ["called_party"] = "受信側",
    ["cancel_bye_failure_reason_cause"] = "キャンセル/バイ/失敗の理由原因",
    ["certificates_not_match"] = "証明書が一致しません",
    ["client_container_information"] = "クライアントコンテナの情報",
    ["client_process_information"] = "クライアントプロセス情報",
    ["client_requested"] = "クライアントのリクエスト",
    ["client_tls_supported_versions"] = "クライアントがサポートするTLSプロトコル",
    ["computed_as_tcp_window_size_rtt"] = "TCPウィンドウサイズ/ RTTとして計算",
    ["container_id"] = "コンテナID",
    ["could_not_add_host_to_category"] = "ホスト\"%{host}\"をカテゴリ\"%{category}\"に追加できませんでした",
    ["data_exfiltration"] = "データ流出",
    ["desirable_label"] = "望ましい",
    ["dns_data_exfiltration"] = "DNSデータの流出",
    ["dns_invalid_query"] = "無効なDNSクエリ",
    ["dns_query"] = "DNSクエリ",
    ["docker"] = "Docker",
    ["docker_name"] = "名前",
    ["drop_flow_traffic_btn"] = "ドロップフロートラフィック",
    ["dropped_packets"] = "ドロップされたパケット",
    ["dtmf_tones_sent"] = "呼び出し中にDTMFトーンが送信されました",
    ["dump_flow_traffic"] = "ダンプフロートラフィック",
    ["elephant_exceeded"] = " %{vol}を超えています",
    ["elephant_flow"] = "エレファントフロー",
    ["elephant_flow_descr"] = "エレファントフローは、設定された特定の量を超えるデータを転送するフローです。管理者は、設定からエレファントフローを構成できます。",
    ["elephant_flow_l2r"] = "エレファントフロー (ローカルからリモート)",
    ["elephant_flow_r2l"] = "エレファントフロー (リモートからローカル)",
    ["entropy"] = "エントロピー",
    ["error_and_higher"] = "エラーかつ高い",
    ["existing_rules_note"] = "既存のルールは、 <a href=\"%{url}\">%{name} </a>ページにあります。",
    ["external_alert"] = " %{scope}アラートを検出しました: %{msg} [%{maker}]",
    ["first_last_flow_sequence"] = "最初/最後フローシーケンス",
    ["flow_active_msg"] = "このフローはアクティブです",
    ["flow_alerted"] = "フローアラート",
    ["flow_anomalies"] = "フローリスクを検知しました",
    ["flow_blocked_by_bridge"] = "設定されたポリシーのためにフローはブロックされました",
    ["flow_cannot_be_found_message"] = "このフローは見つかりませんでした",
    ["flow_completed_msg"] = "このフローは完了しておりすぐに有効期限がきれます",
    ["flow_connecting_msg"] = "フローが接続されています。",
    ["flow_details"] = "フロー詳細",
    ["flow_emitted"] = "ネットワークインターフェイスがアラートされているときにフローが発生しました",
    ["flow_marker"] = "フローマーカー",
    ["flow_peer_roles_inaccurate_msg"] = "フローの開始がみられませんので: ピアのロール (クライアント/サーバー) は正確ではない可能性があります",
    ["flow_peers_client_server"] = "フローピア [ クライアント / サーバー ]",
    ["flow_quota"] = "フロークォータ",
    ["flow_reset_by_resetter_msg"] = "このフローは %{resetter}によってリセットされました",
    ["flow_reset_msg"] = "このフローはリセットされました",
    ["flow_score"] = "フロースコア",
    ["flow_shapers"] = "フローシェイパー",
    ["flow_traffic_is_dropped"] = "フロートラフィックがドロップされました",
    ["forward"] = "進む",
    ["gsm_cell_identifier"] = "GSMセル識別子",
    ["gsm_cell_lac"] = "GSMセルLAC（ロケーションエリアコード）",
    ["host_successfully_added_to_category"] = "ホスト\"%{host}\"はカテゴリ<a href=\"%{url}\">%{category}</a> に追加されました. 新規のフローのみルールに影響を受けます.",
    ["http_method"] = "HTTPメソッド",
    ["icmp_info"] = "ICMP情報",
    ["ids_alert"] = " %{scope}アラートを検出しました: %{msg} [%{maker}]",
    ["iec104_mask"] = "<A class='ntopng-external-link' href='https://en.wikipedia.org/wiki/IEC_60870-5'>IEC 60870-5-104 <i class='fas fa-external-link-alt'></i></A> <A class='ntopng-external-link' href=https://infosys.beckhoff.com/english.php?content=../content/1033/tcplclibiec870_5_104/html/tcplclibiec870_5_104_objref_overview.htm&id>TypeIDs <i class='fas fa-external-link-alt'></i></A>",
    ["imsi"] = "IMSI（国際モバイル加入者ID）",
    ["io_wait_time_percentage"] = "I / O待機時間",
    ["ip_packet_analysis"] = "IPパケット分析",
    ["jitter"] = "Jitter",
    ["k8s"] = "Kubernetes",
    ["k8s_name"] = "名前",
    ["k8s_ns"] = "名前空間",
    ["k8s_pod"] = "ポッド",
    ["known_proto_on_non_std_port"] = "標準ポートではない既知のアプリケーションを発見しました",
    ["longlived_exceeded"] = " %{amount}を超えています",
    ["longlived_flow"] = "持続期間の長いフロー",
    ["longlived_flow_descr"] = "持続期間の長いフローとは、構成された特定の時間より長く続くフローです。管理者は、設定からエレファントフローを構成できます。",
    ["looks_like_idle_flow_message"] = "これはキープアライブのためのだけの周期送信をしている <font color=red>アイドルフロー</font>とみなされます",
    ["lost_packets"] = "ロストパケット",
    ["low_goodput"] = "低グットプット",
    ["malicious_ja3_signature"] = "悪意のある可能性のある %{cli_or_srv}署名 [JA3：<a href=\"%{url}\">%{signature} </a>%{icon}]",
    ["max_estimated_tcp_throughput"] = "最大 (推定) TCP スループット",
    ["max_packet_interarrival_time"] = "最大パケット到着時間",
    ["mean_rtt"] = "平均 RTT",
    ["memory_actual_peak"] = "メモリ 実際/ピーク",
    ["mibehaving_flows"] = "不正なフロー",
    ["normal"] = "正常",
    ["not_recommended_label"] = "推奨されません",
    ["notice_and_lower"] = "通知かつ低い",
    ["now_purged"] = "このフローはアクティブではなくなったため、メモリから削除されているはずです。",
    ["nsapi"] = "NSAPI",
    ["ongoing_call"] = "コール進行中",
    ["packet_inter_arrival_time"] = "パケット到着時間 <br>[最小/平均/最大]",
    ["payload_type"] = "ペイロードタイプ",
    ["potentially_dangerous_protocol"] = "潜在的に危険なプロトコル",
    ["process_information_report_warning"] = "ntopngを <A HREF='%{url}'>nProbe</A> とそのプローブプラグインと一緒に利用しない限りはプロセス情報レポートは限定的です",
    ["process_pid_name"] = "プロセス PID/名",
    ["profile_name"] = "プロファイル名",
    ["pseudo_mos"] = "(疑似) MOS",
    ["r_factor"] = "Rファクター",
    ["reach_connection_label"] = "接続に到達",
    ["remote_to_remote"] = "リモートクライアントとリモートサーバー",
    ["response_code"] = "応答コード",
    ["reverse"] = "リバース",
    ["round_trip_time"] = "往復時間",
    ["rtp_codecs"] = "RTPコーデック",
    ["rtp_flow"] = "RTPフロー",
    ["rtp_protocol_information"] = "RTPプロトコル情報",
    ["rtp_stream_peers"] = "RTPストリームピア",
    ["rtp_transit_in_out"] = "RTPトランジットIN / OUT",
    ["rtt_breakdown"] = "往復時間の分解",
    ["rtt_distance"] = "クライアント/サーバー 推定距離",
    ["rtt_jitter"] = "RTTジッター",
    ["rule_type"] = "ルールタイプ",
    ["sac_service_area_code"] = "SAC（サービスエリアコード）",
    ["score_category_network"] = "ネットワーク",
    ["score_category_security"] = "セキュリティ",
    ["server_container_information"] = "サーバーコンテナ情報",
    ["server_name"] = "サーバー名",
    ["server_process_information"] = "サーバープロセス情報",
    ["sip_call_id"] = "SIPコールID",
    ["sip_protocol_information"] = "SIPプロトコル 情報",
    ["slow_application_header"] = "遅いアプリケーションヘッダー",
    ["slow_data_exchange"] = "遅いデータ交換",
    ["slow_tcp_connection"] = "遅いTCP接続",
    ["son_of_father_process"] = "<a href ='%{url}'>%{proc_father_pid}/%{proc_father_name}</a>の息子",
    ["ssh_signature"] = "SSH署名",
    ["suspicious_client_device_protocol"] = "クライアント プロトコル \"%{proto}\" は設定された<a href=\"%{url}\">%{devtype} プロトコルポリシーにより許可されません</a>",
    ["suspicious_device_protocol"] = "疑わしいデバイスプロトコル",
    ["suspicious_server_device_protocol"] = "サーバー プロトコル \"%{proto}\" は設定された<a href=\"%{url}\">%{devtype} プロトコルポリシーにより許可されません</a>",
    ["suspicious_tcp_probing"] = "疑わしいTCPプローブ",
    ["suspicious_tcp_syn_probing"] = "疑わしいTCP SYNプローブ",
    ["sync_source_id"] = "同期ソースID",
    ["tcp_connection_issues"] = "TCP接続の問題",
    ["tcp_connection_refused"] = "TCP接続が拒否されました",
    ["tcp_packet_analysis"] = "TCPパケット分析",
    ["tcp_severe_connection_issues"] = "重大なTCP接続の問題（再送信、OOO、紛失）",
    ["throughput_trend"] = "スループット傾向",
    ["tls_certificate"] = "TLS証明書",
    ["tls_certificate_expired"] = "TLS証明書の期限切れ",
    ["tls_certificate_invalid"] = "TLS証明書が無効です",
    ["tls_certificate_mismatch"] = "TLS証明書の不一致",
    ["tls_certificate_selfsigned"] = "TLS証明書自己署名",
    ["tls_certificate_validity"] = "TLS証明書の有効性",
    ["tls_old_protocol_version"] = "廃止されたTLSプロトコルバージョン",
    ["tls_server_names"] = "サーバー名",
    ["tls_unsafe_ciphers"] = "安全でないTLS暗号",
    ["tos"] = "<A class='ntopng-external-link' target='_blank' href='https://en.wikipedia.org/wiki/Differentiated_services'>DSCP <i class='fas fa-external-link-alt'></i></A> / <A class='ntopng-external-link' target='_blank' href='https://en.wikipedia.org/wiki/Explicit_Congestion_Notification'>ECN <i class='fas fa-external-link-alt'></i></A> [ クライアント / サーバー ]",
    ["udp_unidirectional"] = "単方向UDPトラフィック",
    ["unknown_status"] = "不明状態 (%{status})",
    ["url"] = "URL",
    ["user_name"] = "ユーザー名",
    ["vm_page_faults"] = "VMページフォールト",
    ["warning"] = "警告",
    ["web_mining_detected"] = "このウェブサイトは暗号通貨をクライアントデバイスでマイニングすることで知られています",
  },
  ["flow_devices"] = {
    ["active_sflow"] = "アクティブなsflowデバイス",
    ["all_device_ports"] = "すべての %{device}ポート",
    ["all_flow_devices"] = "すべてのフローデバイス",
    ["all_sflow_devices"] = "すべてのsFlowデバイス",
    ["duplex"] = "双方向",
    ["enable_flow_ratio"] = "フロー/SNMPレートを有効にする",
    ["flow_device_x"] = "フローデバイス %{device}",
    ["flow_ratio_snmp_instructions"] = "フロー/SNMPレートを有効にするには, このフローデバイスはSNMPをサポートする必要があり, <a href='%{href}'>added</a> からSNMPデバイス登録する必要があります.",
    ["flow_ratio_timeseries_instructions"] = "フロー/SNMPレートを有効にするには, <b>SNMPとフローデバイスの時系列</b> が有効である必要があります.",
    ["full_duplex"] = "全二重",
    ["half_duplex"] = "半二重",
    ["in_errors"] = "エラー",
    ["note_condition_1"] = "デバイス %{device} SNMPをサポートしており<a href=\"%{url}\">SNMP</a> デバイスページで設定する必要があります",
    ["note_condition_2"] = "SNMPとフローデバイス時系列データ <A HREF=\"%{url}\"><i class=\"fas fa-flask\"></i> 設定</A> (エキスパートビュー)から有効化される必要があります",
    ["note_snmp_device"] = "SNMP関連情報は、フローエクスポータと同じIPを持つSNMPデバイスが <a href=\"%{url}\">SNMPデバイス</a>で見つかった場合にのみ表示されます。",
    ["note_zmq_fields1"] = "フローデバイスはZMQ fields %%EXPORTER_IPV4_ADDRESS と %%NPROBE_IPV4_ADDRESSにより決定されます。 ２つのフィールドが有効な場合は前者が後者に優先します",
    ["note_zmq_fields2"] = "フロー デバイス インターフェイスはZMQ fields %%INPUT_SNMP と %%OUTPUT_SNMPにより決定されます",
    ["note_zmq_fields3"] = "sFlowデバイスはZMQ(トピック 'カウンター')を通じて受信されるsFlowフィールドデバイスIPにより決定されます",
    ["note_zmq_fields4"] = "sFlowデバイスはZMQ(トピック 'カウンター')を通じて受信されるifIndexフィールドにより決定されます",
    ["out_errors"] = "アウトエラー",
    ["flow_exporter_devices"] = "フローエクスポーターデバイス",
    ["exporter_ip"] = "フローエクスポーターIP",
    ["exporters"] = "エクスポーター",
    ["promisc"] = "無差別",
    ["ratio_not_ready"] = "フローレートの計算...",
    ["remember_counter_ratio"] = "カウンター/SNMPレートはこれらのステップに続いて有効となります:",
    ["remember_ratio"] = "フロー/SNMP レートは次のステップに続いて有効となります:",
    ["sflow_device_x"] = "sFlowデバイス %{device}",
    ["sflow_snmp_ratio"] = "カウンター/SNMP比率",
    ["sflow_snmp_ratio_descr"] = "カウンター / SNMP 比率は、sFlowカウンターサンプルを使用して取得した合計インターフェイストラフィックを、SNMP経由で読み取られた同じインターフェイスの合計トラフィックで割ることによって計算されます。",
    ["snmp_description"] = "SNMPの説明",
    ["snmp_devmodel"] = "SNMPデバイスモデル",
    ["snmp_devname"] = "SNMPデバイス名",
    ["snmp_ifindex"] = "SNMP インデックス",
    ["snmp_ifname"] = "SNMP インターフェイス",
    ["snmp_location"] = "SNMPロケーション",
    ["snmp_no_device_found"] = "一致するSNMPデバイスが見つかりません",
    ["snmp_polling"] = "フロー/SNMP レートは少なくとも30分間分のデータが必要です",
    ["snmp_ratio"] = "フロー/SNMP 比率",
    ["snmp_ratio_descr"] = "フロー/SNMP ratioは0..1の数値です。これは受信したフローが全体のトラフィック受信量を表現します。フローベースの分析は非IPでレイヤ2ヘッダはカウントされないので、 <br>典型的な比の値は0.8..0.9の範囲に収まる (例. 0.9はSNMPを通じて観測されたトラフィックの90%%がフローとして表現されているということです).<br>比率は以下の条件が整った場合にのみ一時間おきに計算されます",
    ["this_page_contains_couters"] = "このページには、sflow経由で収集されたインターフェイスカウンターが含まれています。 SNMPが設定されている場合、このデバイスの <A HREF=\"%{url}\">SNMPビュー</A>も表示できます。",
    ["timeseries_descr"] = "フローデバイス時系列データは <a href=\"%{url}\"><i class=\"fas fa-flask\"></i> 設定</a>.から有効にできます。最初のデータポイントを表示するのに必要な分はほとんどありません",
  },
  ["flow_fields_description"] = {
    ["appl_latency_ms"] = "アプリケーション遅延(msec), サーバーレスポンスタイム",
    ["application_id"] = "収集されたアプリケーションID（CiscoまたはIXIA）",
    ["application_name"] = "パロアルトAppID",
    ["biflow_direction"] = "1 =イニシエーター、2 =リバースイニシエーター",
    ["bittorrent_hash"] = "ビットトレントハッシュ",
    ["client_nw_latency_ms"] = "ネットワークRTT / 2クライアント<-> nprobe (msec)",
    ["community_id"] = "コミュニティID",
    ["cumulative_icmp_type"] = "ICMPタイプバケットの累積OR",
    ["dhcp_client_ip"] = "DHCPが割り当てたクライアントIPv4アドレス",
    ["dhcp_client_mac"] = "DHCPクライアントのMAC",
    ["dhcp_client_name"] = "DHCPクライアント名",
    ["dhcp_message_type"] = "DHCPメッセージタイプ",
    ["dhcp_remote_id"] = "DHCPエージェントリモートID",
    ["dhcp_subscriber_id"] = "DHCPサブスクライバーID",
    ["diameter_clr_cancel_type"] = "認証キャンセルタイプ",
    ["diameter_clr_flags"] = "認証CLRフラグ",
    ["diameter_exp_res_result_code"] = "認証応答実験結果コード",
    ["diameter_exp_res_vendor_id"] = "認証応答実験結果ベンダーID",
    ["diameter_hop_by_hop_id"] = "ホップ識別子による認証ホップ",
    ["diameter_req_msg_type"] = "認証リクエストメッセージタイプ",
    ["diameter_req_origin_host"] = "認証オリジンホストリクエスト",
    ["diameter_req_user_name"] = "認証リクエストユーザー名",
    ["diameter_rsp_msg_type"] = "認証リクエストメッセージタイプ",
    ["diameter_rsp_origin_host"] = "認証オリジンホストレスポンス",
    ["diameter_rsp_result_code"] = "認証応答結果コード",
    ["direction"] = "フロー 方向 [0=src->dst, 1=dst->src]",
    ["dns_num_answers"] = "返された回答のDNS＃",
    ["dns_query"] = "DNSクエリ",
    ["dns_query_id"] = "DNSクエリトランザクションID",
    ["dns_query_type"] = "DNSクエリタイプ (例 1=A, 2=NS..)",
    ["dns_response"] = "DNSレスポンス",
    ["dns_ret_code"] = "DNSリターンコード(例. 0=no エラー)",
    ["dns_ttl_answer"] = "最初のAレコードのTTL (ある場合)",
    ["dns_tx_id"] = "DNSトランザクションID",
    ["dot1q_dst_vlan"] = "宛先VLAN (QinQの外部VLAN)",
    ["dot1q_src_vlan"] = "送信元VLAN (QinQの外部VLAN)",
    ["downstream_session_id"] = "ダウンストリームセッション識別子（L2TPなど）または不明の場合は0",
    ["downstream_tunnel_id"] = "ダウンストリームトンネル識別子 (例 GTP TEID、VXLAN VNIなど) orまたは不明の場合は0",
    ["dst_as"] = "宛先BGP AS",
    ["dst_as_map"] = "DST_ASの組織名",
    ["dst_as_path_1"] = "宛先ASパス位置1",
    ["dst_as_path_10"] = "宛先ASパス位置10",
    ["dst_as_path_2"] = "宛先ASパス位置2",
    ["dst_as_path_3"] = "宛先ASパス位置3",
    ["dst_as_path_4"] = "宛先ASパス位置4",
    ["dst_as_path_5"] = "宛先ASパス位置5",
    ["dst_as_path_6"] = "宛先ASパス位置6",
    ["dst_as_path_7"] = "宛先ASパス位置7",
    ["dst_as_path_8"] = "宛先ASパス位置8",
    ["dst_as_path_9"] = "宛先ASパス位置9",
    ["dst_father_proc_name"] = "宛先父プロセス名",
    ["dst_father_proc_pid"] = "宛先父プロセスPID",
    ["dst_fragments"] = "フラグメント化されたパケット dst->src",
    ["dst_ip_city"] = "dst IPが配置されている都市",
    ["dst_ip_country"] = "dst IPが配置されている国",
    ["dst_ip_lat"] = "dst IPが配置されている緯度",
    ["dst_ip_long"] = "dst IPが配置されている経度",
    ["dst_proc_actual_memory"] = "Dstプロセスの実際のメモリ(バイト)",
    ["dst_proc_average_cpu_load"] = "Dstプロセスの平均不可(%% * 100)",
    ["dst_proc_name"] = "Dstプロセス名",
    ["dst_proc_num_page_faults"] = "Dstプロセスnumページフォルト",
    ["dst_proc_pctg_iowait"] = "Srcプロセスiowait時間 %% (%% * 100)",
    ["dst_proc_peak_memory"] = "Dstプロセスpeakメモリ (バイト)",
    ["dst_proc_pid"] = "DstプロセスPID",
    ["dst_proc_uid"] = "DstプロセスUID",
    ["dst_proc_user_name"] = "Dstプロセスユーザー名",
    ["dst_to_src_avg_throughput"] = "Dstからsrcへの平均スループット (bps)",
    ["dst_to_src_max_throughput"] = "Dstからsrcへの最大スループット (bps)",
    ["dst_to_src_min_throughput"] = "Dstからsrcへの最小スループット (bps)",
    ["dst_tos"] = "TOS/DSCP (dst->src)",
    ["dst_vlan"] = "宛先VLAN(QinQの内部VLAN)",
    ["duration_in"] = "クライアントからサーバーへのストリーム期間 (msec)",
    ["duration_out"] = "クライアントからサーバーへのストリーム期間 (msec)",
    ["engine_id"] = "フロースイッチングエンジンのID",
    ["engine_type"] = "フロースイッチングエンジン",
    ["file_gaps"] = "ファイルギャップ",
    ["file_id"] = "ファイルID",
    ["file_name"] = "ファイル名",
    ["file_size"] = "ファイルサイズ",
    ["file_state"] = "ファイルの状態",
    ["file_stored"] = "保存されたファイル",
    ["firewall_event"] = "ファイアウォールイベント0 =無視, 1=フロー作成, 2=フロー削除, 3=フロー拒否, 4=フローアラート, 5=フロー更新",
    ["first_switched"] = "最初のフローパケットのシステム稼働時間 (msec)",
    ["flow_active_timeout"] = "フローキャッシュエントリのアクティビティタイムアウト",
    ["flow_duration_microseconds"] = "フロー期間 (usec)",
    ["flow_duration_milliseconds"] = "フロー期間 (msec)",
    ["flow_end_microseconds"] = "最初のフローパケットのuSec (エポック) ",
    ["flow_end_milliseconds"] = "前回のフローパケットのMsec (エポック)",
    ["flow_end_reason"] = "フロー終了理由",
    ["flow_end_sec"] = "前回のフローパケットの秒 (エポック) ",
    ["flow_id"] = "シリアルフロー識別子",
    ["flow_inactive_timeout"] = "フローキャッシュエントリの無活動タイムアウト",
    ["flow_proto_port"] = "L7ポートフロープロトコルを特定するL7ポートあるいは未知のものであれば0",
    ["flow_server_name"] = "フローサーバー名 (ある場合)",
    ["flow_start_microseconds"] = "最初のフローパケットのuSec (エポック)",
    ["flow_start_milliseconds"] = "最初のフローパケットのMsec (エポック)",
    ["flow_start_sec"] = "最初のフローパケットの秒 (エポック)",
    ["flow_user_name"] = "トンネルのフローユーザー名 (ある場合)",
    ["frame_length"] = "元のL2フレーム長",
    ["ftp_command"] = "FTPクライアントコマンド",
    ["ftp_command_ret_code"] = "FTPクライアントコマンドリターンコード",
    ["ftp_login"] = "FTPクライアントログイン",
    ["ftp_password"] = "FTPクライアントパスワード",
    ["gtpv0_apn_name"] = "GTPv0 APN名",
    ["gtpv0_end_user_ip"] = "GTPv0 エンドユーザーIPアドレス",
    ["gtpv0_end_user_msisdn"] = "GTPv0 エンドユーザーMSISDN",
    ["gtpv0_rai_cell_lac"] = "GTPv0セルロケーションエリアコード",
    ["gtpv0_rai_cell_rac"] = "GTPv0セルルーティングエリアコード",
    ["gtpv0_rai_mcc"] = "GTPv0モバイル国コード",
    ["gtpv0_rai_mnc"] = "GTPv0モバイルネットワークコード",
    ["gtpv0_req_msg_type"] = "GTPv0リクエストメッセージタイプ",
    ["gtpv0_response_cause"] = "GTPv0操作の原因",
    ["gtpv0_rsp_msg_type"] = "GTPv0リクエストメッセージタイプ",
    ["gtpv0_tid"] = "GTPv0トンネル識別子",
    ["gtpv1_apn_name"] = "GTPv1APN名",
    ["gtpv1_c2s_teid_ctrl"] = "GTPv1 クライアント->サーバー トンネルIDコントロール",
    ["gtpv1_c2s_teid_data"] = "GTPv1 クライアント->サーバー トンネルIDデータ",
    ["gtpv1_end_user_imei"] = "GTPv1 エンドユーザーIMEI",
    ["gtpv1_end_user_imsi"] = "GTPv1 エンドユーザーIMSI",
    ["gtpv1_end_user_ip"] = "GTPv1 エンドユーザーIPアドレス",
    ["gtpv1_end_user_msisdn"] = "GTPv1エンドユーザーMSISDN",
    ["gtpv1_rai_lac"] = "GTPv1 RAIロケーションエリアコード",
    ["gtpv1_rai_mcc"] = "GTPv1 RAIモバイル国コード",
    ["gtpv1_rai_mnc"] = "GTPv1 RAIモバイルネットワークコード",
    ["gtpv1_rai_rac"] = "GTPv1 RAIルーティングエリアコード",
    ["gtpv1_rat_type"] = "GTPv1 RATタイプ",
    ["gtpv1_req_msg_type"] = "GTPv1リクエストメッセージタイプ",
    ["gtpv1_response_cause"] = "GTPv1 操作の原因",
    ["gtpv1_rsp_msg_type"] = "GTPv1応答メッセージタイプ",
    ["gtpv1_s2c_teid_ctrl"] = "GTPv1 サーバー->クライアントトンネルIDコントロール",
    ["gtpv1_s2c_teid_data"] = "GTPv1 サーバー->クライアントトンネルIDデータ",
    ["gtpv1_uli_cell_ci"] = "GTPv1 ULIセルCI",
    ["gtpv1_uli_cell_lac"] = "GTPv1 ULIセルロケーションエリアコード",
    ["gtpv1_uli_mcc"] = "GTPv1 ULIモバイル国コード",
    ["gtpv1_uli_mnc"] = "GTPv1 ULIモバイルネットワークコード",
    ["gtpv1_uli_sac"] = "GTPv1 ULI SAC",
    ["gtpv2_apn_name"] = "GTPv2 APN名",
    ["gtpv2_c2s_s1u_gtpu_ip"] = "GTPv2 クライアント->サーバー S1U GTPU IP",
    ["gtpv2_c2s_s1u_gtpu_teid"] = "GTPv2 クライアント->サーバー S1U GTPU TEID",
    ["gtpv2_c2s_s5_s8_gtpc_ip"] = "GTPv2 クライアント->サーバー S5/S8 GTPC IP",
    ["gtpv2_c2s_s5_s8_gtpu_ip"] = "GTPv2 クライアント->サーバー S5/S8 PGW GTPU IP",
    ["gtpv2_c2s_s5_s8_gtpu_teid"] = "GTPv2 クライアント->サーバー S5/S8 PGW GTPU TEID",
    ["gtpv2_c2s_s5_s8_sgw_gtpu_ip"] = "GTPv2 クライアント->サーバー S5/S8 SGW GTPU IP",
    ["gtpv2_c2s_s5_s8_sgw_gtpu_teid"] = "GTPv2 クライアント->サーバー S5/S8 SGW GTPU TEID",
    ["gtpv2_end_user_imei"] = "GTPv2エンドユーザーIMEI",
    ["gtpv2_end_user_imsi"] = "GTPv2エンドユーザーIMSI",
    ["gtpv2_end_user_msisdn"] = "GTPv2エンドユーザーMSISDN",
    ["gtpv2_pdn_ip"] = "GTPV2 PDNIPアドレス",
    ["gtpv2_rat_type"] = "GTPv2 RATタイプ",
    ["gtpv2_req_msg_type"] = "GTPv2リクエストメッセージタイプ",
    ["gtpv2_response_cause"] = "GTPv2操作の原因",
    ["gtpv2_rsp_msg_type"] = "GTPv2応答メッセージタイプ",
    ["gtpv2_s2c_s1u_gtpu_ip"] = "GTPv2 サーバー->クライアント S1U GTPU IP",
    ["gtpv2_s2c_s1u_gtpu_teid"] = "GTPv2 サーバー->クライアント S1U GTPU TEID",
    ["gtpv2_s2c_s5_s8_gtpc_ip"] = "GTPv2 サーバー->クライアント S5/S8 GTPC IP",
    ["gtpv2_s2c_s5_s8_gtpu_ip"] = "GTPv2 サーバー->クライアント S5/S8 PGW GTPU IP",
    ["gtpv2_s2c_s5_s8_gtpu_teid"] = "GTPv2 サーバー->クライアント S5/S8 PGW GTPU TEID",
    ["gtpv2_s2c_s5_s8_sgw_gtpu_ip"] = "GTPv2 サーバー->クライアント S5/S8 SGW GTPU IP",
    ["gtpv2_s2c_s5_s8_sgw_gtpu_teid"] = "GTPv2 サーバー->クライアント S5/S8 SGW GTPU TEID",
    ["gtpv2_s5_s8_gtpc_teid"] = "GTPv2 S5/S8 SGW GTPC TEIDs",
    ["gtpv2_uli_cell_id"] = "GTPv2セル識別子",
    ["gtpv2_uli_cell_tac"] = "GTPv2トラッキングエリアコード",
    ["gtpv2_uli_mcc"] = "GTPv2モバイル国コード",
    ["gtpv2_uli_mnc"] = "GTPv2モバイルネットワークコード",
    ["http_host"] = "HTTPホスト名 (IXIAホスト名)",
    ["http_length"] = "HTTPコンテンツの大きさ",
    ["http_method"] = "HTTPメソッド",
    ["http_mime"] = "HTTP Mimeタイプ",
    ["http_protocol"] = "HTTPプロトコル",
    ["http_referer"] = "HTTPリファラー",
    ["http_ret_code"] = "HTTP戻りコード (例 200, 304...)",
    ["http_site"] = "HTTPホスト名なしのサーバー",
    ["http_ua"] = "HTTPユーザエージェント",
    ["http_url"] = "HTTPURL (IXIA URI)",
    ["http_via"] = "HTTP経由",
    ["http_x_forwarded_for"] = "HTTP X-Forwarded-For",
    ["icmp_ipv4_code"] = "ICMPコード",
    ["icmp_ipv4_type"] = "ICMPタイプ",
    ["icmp_type"] = "ICMPタイプ * 256 + ICMPコード",
    ["imap_login"] = "メール送信者",
    ["in_bytes"] = "受信フローバイト (src->dst)",
    ["in_dst_mac"] = "宛先MACアドレス",
    ["in_pkts"] = "受信フロー パケット (src->dst)",
    ["in_src_mac"] = "ソースMACアドレス",
    ["in_src_osi_sap"] = "OSIソースSAP (OSIトラフィックのみ)",
    ["ingress_vrfid"] = "イングレスVRFID",
    ["input_snmp"] = "入力インターフェイス SNMP idx",
    ["ip_protocol_version"] = "[4=IPv4][6=IPv6]",
    ["ipfix_sampling_algorithm"] = "サンプリングアルゴリズム",
    ["ipv4_dst_addr"] = "IPv4宛先アドレス",
    ["ipv4_dst_mask"] = "IPv4宛先サブネットマスク(/<bits>)",
    ["ipv4_next_hop"] = "IPv4ネクストホップアドレス",
    ["ipv4_src_addr"] = "IPv4ソースアドレス",
    ["ipv4_src_mask"] = "IPv4ソースサブネットマスク(/<bits>)",
    ["ipv6_dst_addr"] = "IPv6宛先アドレス",
    ["ipv6_dst_mask"] = "IPv6宛先マスク",
    ["ipv6_next_hop"] = "IPv6ネクストホップアドレス",
    ["ipv6_src_addr"] = "IPv6ソースアドレスs",
    ["ipv6_src_mask"] = "IPv6ソースマスク",
    ["l4_dst_port"] = "レイヤ4宛先ポート",
    ["l4_dst_port_map"] = "レイヤ4宛先ポートシンボリック名",
    ["l4_src_port"] = "レイヤ4ソースポート",
    ["l4_src_port_map"] = "レイヤ4ソースポートシンボリック名",
    ["l4_srv_port"] = "レイヤ4サーバーポート",
    ["l4_srv_port_map"] = "レイヤ4サーバーポートシンボリック名",
    ["l7_proto"] = "レイヤ7プロトコル(ニューメリック)",
    ["l7_proto_name"] = "レイヤ7プロトコル名",
    ["last_switched"] = "前回のフローパケットのシステム稼働時間(msec)",
    ["longest_flow_pkt"] = "フローの最長パケット (バイト) ",
    ["max_ttl"] = "最大フローTTL",
    ["min_ttl"] = "最小フローTTL",
    ["mpls_label_1"] = "位置1のMPLSラベル",
    ["mpls_label_10"] = "位置10のMPLSラベル",
    ["mpls_label_2"] = "位置2のMPLSラベル",
    ["mpls_label_3"] = "位置3のMPLSラベル",
    ["mpls_label_4"] = "位置4のMPLSラベル",
    ["mpls_label_5"] = "位置5のMPLSラベル",
    ["mpls_label_6"] = "位置6のMPLSラベル",
    ["mpls_label_7"] = "位置7のMPLSラベル",
    ["mpls_label_8"] = "位置8のMPLSラベル",
    ["mpls_label_9"] = "位置9のMPLSラベル",
    ["mysql_appl_latency_usec"] = "MySQLリクエスト->レスポンス遅延 (usec)",
    ["mysql_db"] = "使用中のMySQLデータベース",
    ["mysql_query"] = "MySQLクエリ",
    ["mysql_response"] = "MySQLサーバーレスポンス",
    ["mysql_server_version"] = "MySQLサーバーバージョン",
    ["mysql_username"] = "MySQLユーザー名",
    ["netbios_query_name"] = "NETBIOSクエリ名",
    ["netbios_query_os"] = "NETBIOSクエリOS",
    ["netbios_query_type"] = "NETBIOSクエリタイプ",
    ["netbios_response"] = "NETBIOSクエリレスポンス",
    ["nprobe_ipv4_address"] = "ホストのIPv4アドレスはnProbeが利用しています",
    ["num_pkts_1024_to_1514_bytes"] = "# IPサイズ > 1024 および <= 1514のパケット",
    ["num_pkts_128_to_256_bytes"] = "# IPサイズ > 128 および <= 256のパケット",
    ["num_pkts_256_to_512_bytes"] = "# IPサイズ > 256 および < 512のパケット",
    ["num_pkts_512_to_1024_bytes"] = "# IPサイズ > 512 および < 1024のパケット",
    ["num_pkts_over_1514_bytes"] = "# IPサイズがを超えるパケット > 1514",
    ["num_pkts_ttl_128_160"] = "# TTL > 128および <= 160のパケット",
    ["num_pkts_ttl_160_192"] = "# TTL > 160および <= 192のパケット",
    ["num_pkts_ttl_192_224"] = "# TTL > 192および <= 224のパケット",
    ["num_pkts_ttl_224_255"] = "# TTL > 224および <= 255のパケット",
    ["num_pkts_ttl_2_5"] = "# TTL > 1およびTTL <= 5のパケット",
    ["num_pkts_ttl_32_64"] = "# TTL > 32および<= 64のパケット",
    ["num_pkts_ttl_5_32"] = "# TTL > 5 および TTL <= 32のパケット",
    ["num_pkts_ttl_64_96"] = "# TTL > 64および <= 96パケット",
    ["num_pkts_ttl_96_128"] = "# TTL > 96および<= 128のパケット",
    ["num_pkts_ttl_eq_1"] = "# TTL = 1のパケット",
    ["num_pkts_up_to_128_bytes"] = "# IPサイズ<= 128以下のパケット",
    ["observation_point_id"] = "観測点ID",
    ["observation_point_type"] = "観測点タイプ",
    ["ooorder_in_pkts"] = "異常なTCPフローパケットの数(dst->src)",
    ["ooorder_out_pkts"] = "異常なTCPフローパケットの数(src->dst)",
    ["oracle_query"] = "Oracleクエリ",
    ["oracle_query_duration"] = "Oracleクエリ期間(msec)",
    ["oracle_rsp_code"] = "Oracleレスポンスコード",
    ["oracle_rsp_string"] = "Oracle応答文字列",
    ["oracle_username"] = "Oracleユーザー名",
    ["out_bytes"] = "発信フローバイト (dst->src) [%%DST_TO_SRC_BYTESにエイリアス]",
    ["out_dst_mac"] = "観測点の後にミドルボックス機能によって潜在的に変更された宛先MACアドレス",
    ["out_dst_osi_sap"] = "OSI宛先SAP(OSI トラフィック のみ)",
    ["out_pkts"] = "発信フローパケット(dst->src) [%%DST_TO_SRC_PKTSにエイリアス]",
    ["out_src_mac"] = "観測点の後にミドルボックス関数によって潜在的に変更されたソースMAC アドレス",
    ["output_snmp"] = "出力インターフェイスSNMP idx",
    ["packet_section_offset"] = "パケットセクションオフセット",
    ["packets_observed"] = "確認されたパケットの合計数",
    ["packets_selected"] = "サンプリング用に選択されたパケットの合計数",
    ["payload_hash"] = "初期フローペイロードハッシュ",
    ["plugin_name"] = "このフローが使用するプラグイン名 (ある場合)",
    ["pop_user"] = "POP3ユーザーログイン",
    ["post_napt_dst_transport_port"] = "Post NAPT宛先トランスポートのポート",
    ["post_napt_src_transport_port"] = "Post NAPT Source Transport ポート",
    ["post_nat_dst_ipv4_addr"] = "Post NAT宛先IPv4アドレス",
    ["post_nat_src_ipv4_addr"] = "Post NAT送信元IPv4アドレス",
    ["exporter_ipv4_address"] = "フローエクスポーターIPv4アドレス",
    ["exporter_ipv6_address"] = "フローエクスポーターIPv6アドレス",
    ["protocol"] = "IPプロトコルバイト",
    ["protocol_map"] = "IP プロトコル名",
    ["radius_acct_in_octets"] = "RADIUSアカウンティング入力オクテット",
    ["radius_acct_in_pkts"] = "RADIUSアカウンティング入力パケット",
    ["radius_acct_out_octets"] = "RADIUSアカウンティング出力オクテット",
    ["radius_acct_out_pkts"] = "RADIUSアカウンティング出力パケット",
    ["radius_acct_session_id"] = "RADIUSアカウンティングセッション名",
    ["radius_acct_status_type"] = "RADIUSアカウンティングステータスタイプ",
    ["radius_called_station_id"] = "RADIUSはステーションIDをコールしました",
    ["radius_calling_station_id"] = "RADIUSはステーションIDをコール中です",
    ["radius_framed_ip_addr"] = "RADIUSフレーム付きIP",
    ["radius_nas_identifier"] = "RADIUS NAS識別子",
    ["radius_nas_ip_addr"] = "RADIUS NASIPアドレス",
    ["radius_req_msg_type"] = "RADIUSリクエストメッセージタイプ",
    ["radius_rsp_msg_type"] = "RADIUSレスポンスメッセージタイプ",
    ["radius_user_imei"] = "RADIUSユーザーMSISDN (拡張)",
    ["radius_user_imsi"] = "RADIUSユーザーIMSI (拡張)",
    ["radius_user_name"] = "RADIUSユーザー名(アクセスのみ)",
    ["retransmitted_in_bytes"] = "再送されたTCPフローバイト数 (src->dst)",
    ["retransmitted_in_pkts"] = "再送されたTCPフローパケットの数 (src->dst)",
    ["retransmitted_out_bytes"] = "再送されたTCPフローバイト数(dst->src)",
    ["retransmitted_out_pkts"] = "再送されたTCPフローパケットの数 (dst->src)",
    ["rtp_dtmf_tones"] = "コール中に送信されたDTMFトーン（存在する場合）",
    ["rtp_first_seq"] = "最初のフローRTPシーケンス番号",
    ["rtp_first_ts"] = "最初のフローRTPタイムスタンプ",
    ["rtp_in_jitter"] = "RTPジッター (ms * 1000)",
    ["rtp_in_max_delta"] = "連続するパケット間の最大デルタ(ms*100)(src->dst)",
    ["rtp_in_mos"] = "連続RTP疑似MOS (値 * 100) (src->dst)",
    ["rtp_in_payload_type"] = "RTPペイロードタイプ",
    ["rtp_in_pkt_drop"] = "ジッタバッファによって破棄されたパケット(src->dst)",
    ["rtp_in_pkt_lost"] = "ストリームで失われたパケット (src->dst)",
    ["rtp_in_r_factor"] = "RTP疑似R_FACTOR (値 * 100) (src->dst)",
    ["rtp_in_transit"] = "RTPトランジット (値 * 100) (src->dst)",
    ["rtp_last_seq"] = "前回のフローRTPシーケンス番号",
    ["rtp_last_ts"] = "前回のフローRTPタイムスタンプ",
    ["rtp_mos"] = "RTP擬似MOS (値 * 100) (平均両方向)",
    ["rtp_out_jitter"] = "RTPジッター (ms * 1000)",
    ["rtp_out_max_delta"] = "連続するパケット間の最大デルタ(ms*100)(dst->src)",
    ["rtp_out_mos"] = "RTP擬似MOS (値 * 100) (dst->src)",
    ["rtp_out_payload_type"] = "RTPペイロードタイプ",
    ["rtp_out_pkt_drop"] = "ジッタバッファによって破棄されたパケット(dst->src)",
    ["rtp_out_pkt_lost"] = "ストリームで失われたパケット(dst->src)",
    ["rtp_out_r_factor"] = "RTP疑似R_FACTOR (値 * 100) (dst->src)",
    ["rtp_out_transit"] = "RTPトランジット(値 * 100) (dst->src)",
    ["rtp_r_factor"] = "RTP疑似R_FACTOR (値 * 100) (平均両方向)",
    ["rtp_rtt"] = "RTP往復時間 (ms)",
    ["rtp_sip_call_id"] = "このRTPストリームに対応するSIPコールID",
    ["rtp_ssrc"] = "RTP同期ソースID",
    ["s1ap_cause_enb_to_mme"] = "ENBからMMEへのS1AP原因",
    ["s1ap_detailed_cause_enb_to_mme"] = "ENBからMMEへのS1APの詳細な原因",
    ["s1ap_enb_ue_s1ap_id"] = "S1AP ENB識別子",
    ["s1ap_mme_ue_s1ap_id"] = "S1AP MME識別子",
    ["s1ap_msg_emm_type_enb_to_mme"] = "ENBからMMEへのS1AP EMMメッセージタイプ",
    ["s1ap_msg_emm_type_mme_to_enb"] = "MMEからENBへのS1AP EMMメッセージタイプ",
    ["s1ap_msg_esm_type_enb_to_mme"] = "ENBからMMEへのS1AP ESMメッセージタイプ",
    ["s1ap_msg_esm_type_mme_to_enb"] = "MMEからENBへのS1AP ESMメッセージタイプ",
    ["sampled_packet_id"] = "サンプリングされたパケットID",
    ["sampled_packet_size"] = "サンプリングされたパケットサイズ",
    ["sampling_algorithm"] = "サンプリングタイプ（決定的/ランダム）",
    ["sampling_interval"] = "サンプリングレート",
    ["sampling_population"] = "母集団のサンプリング",
    ["sampling_size"] = "サンプルするパケットの数",
    ["selector_id"] = "セレクターID",
    ["selector_name"] = "サンプラー名",
    ["server_nw_latency_ms"] = "ネットワークRTT/2 nprobe <-> サーバー (msec)",
    ["shortest_flow_pkt"] = "フローの最短パケット(バイト)",
    ["sip_bye_ok_time"] = "BYE OKのSIP時間(エポック)",
    ["sip_bye_time"] = "BYEのSIP時間(エポック)",
    ["sip_c_ip"] = "SIP C IPアドレス",
    ["sip_call_id"] = "SIP コールID",
    ["sip_call_state"] = "SIPコール状態",
    ["sip_called_party"] = "SIP受信側",
    ["sip_calling_party"] = "SIPコールイニシエーター",
    ["sip_cancel_ok_time"] = "CANCEL OKのSIP時間(エポック)",
    ["sip_cancel_time"] = "CANCELのSIP時間(エポック)",
    ["sip_invite_failure_time"] = "INVITE FAILUREのSIP時間(エポック)",
    ["sip_invite_ok_time"] = "INVITE OKのSIP時間(エポック)",
    ["sip_invite_time"] = "INVITEのSIP時間(エポック)",
    ["sip_reason_cause"] = "SIPキャンセル/バイ/失敗の理由の原因",
    ["sip_response_code"] = "SIP障害応答コード",
    ["sip_ringing_time"] = "RINGINGのSIP時間(エポック)",
    ["sip_rtp_codecs"] = "SIP RTPコーデック",
    ["sip_rtp_ipv4_dst_addr"] = "SIP RTPストリームの宛先IP",
    ["sip_rtp_ipv4_src_addr"] = "SIP RTPストリームソースIP",
    ["sip_rtp_l4_dst_port"] = "SIP RTPストリームの宛先",
    ["sip_rtp_l4_src_port"] = "SIP RTPストリームソース",
    ["sip_trying_time"] = "試行のSIP時間(エポック)",
    ["smtp_mail_from"] = "メール送信者",
    ["smtp_rcpt_to"] = "メール受信者",
    ["src_as"] = "送信元BGP AS",
    ["src_as_map"] = "SRC_ASの組織名",
    ["src_as_path_1"] = "Src ASパス位置1",
    ["src_as_path_10"] = "Src ASパス位置10",
    ["src_as_path_2"] = "Src ASパス位置2",
    ["src_as_path_3"] = "Src ASパス位置3",
    ["src_as_path_4"] = "Src ASパス位置4",
    ["src_as_path_5"] = "Src ASパス位置5",
    ["src_as_path_6"] = "Src ASパス位置6",
    ["src_as_path_7"] = "Src ASパス位置7",
    ["src_as_path_8"] = "Src ASパス位置8",
    ["src_as_path_9"] = "Src ASパス位置9",
    ["src_father_proc_name"] = "Src fatherプロセス名",
    ["src_father_proc_pid"] = "Src fatherプロセスPID",
    ["src_fragments"] = "フラグメント化されたバケット src->dst",
    ["src_ip_city"] = "src IPがある都市",
    ["src_ip_country"] = "src IPがある国",
    ["src_ip_lat"] = "src IPが配置されている緯度",
    ["src_ip_long"] = "src IPが配置されている経度",
    ["src_proc_actual_memory"] = "Srcプロセスの実際のメモリ (バイト)",
    ["src_proc_average_cpu_load"] = "Srcプロセスの平均負荷 (%% * 100)",
    ["src_proc_name"] = "Srcプロセス名",
    ["src_proc_num_page_faults"] = "Srcプロセスnumページフォルト",
    ["src_proc_pctg_iowait"] = "Srcプロセスiowait時間 %% (%% * 100)",
    ["src_proc_peak_memory"] = "Srcプロセスピークメモリ(バイト)",
    ["src_proc_pid"] = "SrcプロセスPID",
    ["src_proc_uid"] = "SrcプロセスUID",
    ["src_proc_user_name"] = "Srcプロセスユーザー名",
    ["src_to_dst_avg_throughput"] = "Srcからdstの平均スループット (bps)",
    ["src_to_dst_max_throughput"] = "Srcからdstの最大スループット (bps)",
    ["src_to_dst_min_throughput"] = "Srcからdstの最小スループット (bps)",
    ["src_tos"] = "TOS/DSCP (src->dst)",
    ["src_vlan"] = "送信元VLAN (QinQの内部VLAN)",
    ["ssdp_host"] = "SSDPホスト",
    ["ssdp_method"] = "SSDPメソッド",
    ["ssdp_server"] = "SSDPサーバー",
    ["ssdp_type"] = "SSDPタイプ",
    ["ssdp_usn"] = "SSDP USN",
    ["suricata_app_proto"] = "Suricataアプリケーションプロトコル",
    ["suricata_flow_id"] = "SuricataフローID",
    ["tcp_flags"] = "すべてのフローTCPフラグの累積",
    ["tcp_win_max_in"] = "最大TCPウィンドウ (src->dst)",
    ["tcp_win_max_out"] = "最大TCPウィンドウ (dst->src)",
    ["tcp_win_min_in"] = "最小TCPウィンドウ(src->dst)",
    ["tcp_win_min_out"] = "最小TCPウィンドウ (dst->src)",
    ["tcp_win_mss_in"] = "TCP最大セグメントサイズ(src->dst)",
    ["tcp_win_mss_out"] = "TCP最大セグメントサイズ (dst->src)",
    ["tcp_win_scale_in"] = "TCPウィンドウスケール (src->dst)",
    ["tcp_win_scale_out"] = "TCPウィンドウスケール (dst->src)",
    ["tls_cert_after"] = "以後のTLS証明書",
    ["tls_cert_dn"] = "TLS証明書　識別名",
    ["tls_cert_not_before"] = "以前のTLS証明書",
    ["tls_cert_sha1"] = "TLS証明書のフィンガープリント",
    ["tls_cert_sn"] = "TLS証明書のシリアルナンバー",
    ["tls_cert_subject"] = "TLS証明書サブジェクト",
    ["tls_server_name"] = "TLS サーバー 名",
    ["tls_version"] = "TLSバージョン",
    ["total_bytes_exp"] = "エクスポートされた合計バイト",
    ["total_flows_exp"] = "エクスポートされたフローの総数",
    ["total_pkts_exp"] = "エクスポートされたフローの合計",
    ["untunneled_ipv4_dst_addr"] = "トンネル化されていないIPv4宛先アドレス",
    ["untunneled_ipv4_src_addr"] = "トンネル化されていないIPv4送信元アドレス",
    ["untunneled_ipv6_dst_addr"] = "トンネル化されていないIPv6宛先アドレス",
    ["untunneled_ipv6_src_addr"] = "トンネル化されていないIPv6送信元アドレス",
    ["untunneled_l4_dst_port"] = "トンネル化されていないIPv4宛先ポート",
    ["untunneled_l4_src_port"] = "トンネル化されていないIPv4送信元ポート",
    ["untunneled_protocol"] = "トンネル化されていないIPプロトコルバイト",
    ["upstream_session_id"] = "アップストリームセッション識別子 (例 L2TP) または不明の場合は0",
    ["upstream_tunnel_id"] = "アップストリームトンネル識別子 (例 GTP TEID, VXLAN VNI)または不明の場合は0",
    ["user_name"] = "ユーザ名",
  },
  ["flow_risk"] = {
    ["ndpi_binary_application_transfer"] = "疑わしいファイル転送",
    ["ndpi_dns_suspicious_traffic"] = "疑わしい DNS トラフィック",
    ["ndpi_http_numeric_ip_host"] = "HTTP 数字 IP ホスト",
    ["ndpi_http_suspicious_header"] = "HTTP 疑わしい ヘッダ",
    ["ndpi_http_suspicious_url"] = "HTTP 疑わしい URL",
    ["ndpi_http_suspicious_user_agent"] = "HTTP 疑わしい User-Agent",
    ["ndpi_known_protocol_on_non_standard_port"] = "標準ポートでない知られたアプリケーション",
    ["ndpi_malformed_packet"] = "壊れたパケット",
    ["ndpi_no_risk"] = "リスクでない",
    ["ndpi_smb_insecure_version"] = "SMB セキュアでない バージョン",
    ["ndpi_ssh_obsolete_client_version_or_cipher"] = "廃れた SSH クライアントバージョン もしくは 暗号",
    ["ndpi_ssh_obsolete_server_version_or_cipher"] = "廃れた SSH サーバー バージョン もしくは 暗号",
    ["ndpi_suspicious_dga_domain"] = "疑わしい DGA ドメイン名",
    ["ndpi_tls_certificate_expired"] = "TLS 証明書期限切れ",
    ["ndpi_tls_certificate_mismatch"] = "TLS 証明書不一致",
    ["ndpi_tls_missing_sni"] = "TLS SNI が存在しない",
    ["ndpi_tls_not_carrying_https"] = "TLS (と思われる) HTTPS通信ではない",
    ["ndpi_tls_obsolete_version"] = "廃れた TLS バージョン",
    ["ndpi_tls_selfsigned_certificate"] = "TLS 証明書 自己著名",
    ["ndpi_tls_suspicious_esni_usage"] = "疑わしい TLS ESNI 利用",
    ["ndpi_tls_weak_cipher"] = "脆弱な TLS 暗号",
    ["ndpi_unsafe_protocol"] = "安全でない プロトコル",
    ["ndpi_url_possible_rce_injection"] = "可能性のあるリモートコード実行(RCE)",
    ["ndpi_url_possible_sql_injection"] = "SQLインジェクションの可能性",
    ["ndpi_url_possible_xss"] = "XSSの可能性",
  },
  ["flows_page"] = {
    ["active_flows"] = "アクティブな %{filter} フロー",
    ["actual_throughput"] = "実際のスループット",
    ["alert_type_severity"] = "重大度",
    ["alerted"] = "アラート",
    ["all_alerted"] = "すべてのアラート",
    ["all_ases"] = "すべての AS",
    ["all_categories"] = "すべてのカテゴリ",
    ["all_devices"] = "すべてのフローエクスポーター",
    ["all_flows"] = "すべてのフロー",
    ["all_hosts"] = "すべてのホスト",
    ["all_inIfIdx"] = "すべての入力インターフェイス",
    ["all_ip_versions"] = "すべてのバージョン",
    ["all_l4_protocols"] = "すべてのプロトコル",
    ["all_misbehaving"] = "すべての不正な動作",
    ["all_outIfIdx"] = "すべての出力インターフェイス",
    ["all_proto"] = "すべてのプロトコル",
    ["all_vlan_ids"] = "すべてのVLAN",
    ["applications"] = "アプリケーション",
    ["blocked"] = "ブロック",
    ["client_as"] = "クライアントAS",
    ["device_ip"] = "フローエクスポーター",
    ["direction"] = "方向",
    ["flows"] = "%{filter} フロー",
    ["flows_as_client"] = "クライアントのフロー",
    ["flows_as_server"] = "サーバーのフロー",
    ["host"] = " [ ホスト %{host} ]",
    ["hosts"] = "ホスト",
    ["icmp_only"] = "ICMPのみ",
    ["idle_flows_not_listed"] = "アイドルフローはリストされていません。",
    ["inIfIdx"] = "入力インターフェイス",
    ["ip_version"] = "IPバージョン",
    ["ipv4_only"] = "IPv4のみ",
    ["ipv6_only"] = "IPv6のみ",
    ["l4_protocol"] = "プロトコル",
    ["local_cli_remote_srv"] = "ローカルクライアント - リモートサーバー",
    ["local_only"] = "ローカルのみ",
    ["local_srv_remote_cli"] = "ローカルサーバー - リモートクライアント",
    ["misbehaving"] = "不正な動作",
    ["multicast"] = "マルチキャスト/ブロードキャスト",
    ["network"] = " [ ネットワーク %{network} ]",
    ["non_multicast"] = "非マルチキャスト/非ブロードキャスト",
    ["normal"] = "ノーマル",
    ["one_way_multicast"] = "単方向マルチキャスト/ブロードキャスト",
    ["one_way_non_multicast"] = "単方向の非マルチキャスト/非ブロードキャスト",
    ["outIfIdx"] = "出力インターフェイス",
    ["port"] = " [ ポート %{port} ]",
    ["flow_exporters"] = "フローエクスポーター",
    ["recently_active_flows"] = "最近のアクティブな %{filter} フロー",
    ["remote_only"] = "リモートのみ",
    ["server_as"] = "ASサーバー",
    ["sflow_devices"] = "sFlowデバイス",
    ["source_id"] = "ソースID",
    ["tcp_only"] = "TCPのみ",
    ["tcp_state"] = "TCPの状態",
    ["tcp_state_closed"] = "クローズ",
    ["tcp_state_connecting"] = "接続中",
    ["tcp_state_established"] = "確立",
    ["tcp_state_reset"] = "リセット",
    ["tcp_state_unknown"] = "不明",
    ["total_bytes"] = "合計バイト",
    ["total_bytes_rcvd"] = "受信した総バイト数",
    ["total_bytes_sent"] = "送信された総バイト数",
    ["total_bytes_unknown"] = "不明なアプリケーションの総バイト数",
    ["udp_only"] = "UDPのみ",
    ["vlan"] = "VLAN",
  },
  ["geo_map"] = {
    ["browser_reported_home_map"] = "ブラウザがホームマップの場所を報告しました",
    ["configure_geo_ip"] = "ジオIPの設定",
    ["geo_ip"] = "ジオIP",
    ["geo_map"] = "ジオマップ",
    ["geolocation_error"] = "ジオロケーションエラー",
    ["hosts_geomap"] = "ホストジオマップ",
    ["latitude"] = "緯度",
    ["longitude"] = "経度",
    ["note"] = "注意",
    ["note_active_flows"] = "ピア間のパブリックIPアドレスを通じたアクティブなフロー",
    ["note_compiled_ntopng_with_geolocation"] = "ソフトウェアを位置情報オプションでコンパイル開始しました",
    ["note_google_maps_browser_api_key"] = "Google Maps Browser APIキーが必須です。キー生成の詳細は<a href='%{url_google}'>で見つかります</a>.<br>一旦、生成されれば<a href='%{url_prefs}'><i class=\"fas fa-flask\"></i> 設定</a>で提出できます",
    ["note_html_browser_geolocation"] = "HTML <A HREF=%{url}>ブラウザのジオロケーション</A> がホストをマップ上に配置する際に利用されます",
    ["note_requirements_visualize_maps"] = "マップを表示するには以下が必要です",
    ["note_working_internet_connection"] = "有効なインターネット接続",
    ["unavailable_geolocation"] = "ジオロケーションがブラウザでサポートされていないか無効化されています",
    ["using_default_location"] = "デフォルトの位置を利用します",
    ["warning_accuracy"] = "ジオロケーションの正確さは、データーベースに依存します.",
  },
  ["graphs"] = {
    ["95_perc"] = "95thパーセントタイル",
    ["active_devices"] = "アクティブなデバイス",
    ["active_flows"] = "アクティブなフロー",
    ["active_host_contacts"] = "アクティブなホストコンタクト",
    ["active_hosts"] = "アクティブなホスト",
    ["active_http_servers"] = "アクティブな HTTP サーバー",
    ["active_monitoring"] = "アクティブモニタ",
    ["actual_memory"] = "実際のメモリ",
    ["app_connect"] = "App 接続",
    ["arp_replies"] = "ARP応答",
    ["arp_requests"] = "ARP要求",
    ["arp_rqst_sent_rcvd_rpls"] = "ARP送信リクエスト/ Rcvdリプライ",
    ["average"] = "平均",
    ["average_traffic"] = "平均トラフィック/sec",
    ["avg"] = "平均",
    ["binary_operators_note"] = "EMA, SMAとRSIインジケーターに関しての詳細は<a href=\"%{url}\">オンラインドキュメンテーション</a>をチェックしてください",
    ["blocked_flows"] = "ブロックされたフロー",
    ["bytes_sent_non_unicast"] = "非ユニキャスト送信バイト",
    ["bytes_sent_unicast"] = "ユニキャスト送信バイト数",
    ["custom"] = "カスタム",
    ["discarded_probing_bytes"] = "破棄されたプローブトラフィック",
    ["discarded_probing_packets"] = "破棄されたプローブパケット",
    ["dns_qry_rcvd_rsp_sent"] = "DNS Rcvdクエリ/送信されたリプライ",
    ["dns_qry_sent_rsp_rcvd"] = "DNS送信クエリ/ Rcvdリプライ",
    ["download_chart_data"] = "現在のチャートデータをダウンロードする",
    ["dropped_flows"] = "ドロップされたフロー",
    ["dscp_classes"] = "DSCPクラス",
    ["dumped_flows"] = "エクスポートされたフロー",
    ["echo_reply_packets"] = "応答パケットを表示する",
    ["echo_request_packets"] = "リクエストパケットを表示する",
    ["export_flows_as_txt"] = "フローをテキストファイルでエクスポート",
    ["flows_status_overview"] = "%{protocol}フローステータスの概要",
    ["flows_vs_local_hosts"] = "アクティブなフロー vs ローカル ホスト",
    ["flows_vs_traffic"] = "アクティブなフロー vs トラフィック",
    ["get_permanent_link"] = "パーマネントリンクを取得",
    ["graph_info"] = "グラフ情報",
    ["host_unreachable_flows"] = "ホストへ到達出来なかったフローの合計",
    ["http_stats"] = "HTTP 統計",
    ["influxdb_not_responding"] = "InfluxDBが応答しないため、クエリは中止されました。クエリのタイムアウトは、<a href=\"%{url}\">%{flask_icon}設定</a>から設定できます。",
    ["last_ms"] = "前回のミリ秒",
    ["max"] = "最大",
    ["max_ms"] = "最大のミリ秒",
    ["max_rtt"] = "最大RTT時間",
    ["memory_vs_hosts_flows"] = "プロセスメモリvsホスト/フロー",
    ["min_rtt"] = "最小RTT時間",
    ["minute_interface_top_stats"] = "分あたりのインターフェイス 上位トラフィック統計",
    ["name_lookup"] = "名前解決",
    ["new_flows"] = "新しいフロー",
    ["note_ases_rcvd"] = "ASで受信されたトラフィック",
    ["note_ases_sent"] = "ASより送信されたトラフィック",
    ["note_ases_traffic"] = "ASより送受信されたトラフィック",
    ["note_chart_zoom"] = "チャート上でクリックしてドラッグするとズームインし、ダブルクリックでズームアウトします",
    ["note_protocols_shown"] = "選択された時間枠で表示されないプロトコルはドロップダウンにリストされません.",
    ["num_calls"] = "コール数",
    ["num_ms_rtt"] = "往復時間 ms",
    ["num_nfq_drops"] = "ドロップされたパケット",
    ["num_nfq_pct"] = "キューフィル %%",
    ["packet_drops"] = "パケットドロップ",
    ["packets_vs_drops"] = "パケット vs ドロップ",
    ["peak_memory"] = "ピークメモリ",
    ["percentile"] = "95 <a href=\"%{url}\">パーセントタイル</a>",
    ["permanent_link"] = "パーマネントリンク",
    ["pkt_drops"] = "パケットドロップ",
    ["process_alerts"] = "プロセスアラート",
    ["process_memory"] = "プロセスメモリ",
    ["query_too_much_time"] = "クエリの処理完了に時間がかかっています",
    ["query_too_slow"] = "クエリの処理にあまりに時間がかかっています",
    ["query_was_aborted"] = "クエリは中止されました",
    ["raw_flows"] = "生フロー",
    ["raw_proto_flows"] = "原 %{proto} フロー",
    ["red_line_unreachable"] = "ホストが到達できていない時間を赤い線が表しています.",
    ["reset_zoom"] = "ズームをリセット",
    ["right_axis_note"] = "(右軸)",
    ["tcp_finack_packets"] = "TCP FIN + ACKパケット",
    ["tcp_packets"] = "TCPパケット",
    ["tcp_packets_keep_alive"] = "TCPパケットのキープアライブ",
    ["tcp_packets_lost"] = "失われたTCPパケット",
    ["tcp_packets_ooo"] = "TCPパケットの異常",
    ["tcp_packets_retr"] = "TCP再送信パケット",
    ["tcp_retr_ooo_lost"] = "TCPの順序が乱れて再送信された",
    ["tcp_rst_packets"] = "TCP RSTパケット",
    ["tcp_rx_stats"] = "TCP RX統計",
    ["tcp_syn_packets"] = "TCP SYNパケット",
    ["tcp_syn_vs_tcp_synack"] = "TCP SYNvsSYN+ACKパケット",
    ["tcp_synack_packets"] = "TCP SYN+ACKパケット",
    ["tcp_tx_stats"] = "TCP TX統計",
    ["the_select_timeframe_too_small_top_talkers"] = "選択された時間枠は上位トーカーを決定するには短かすぎます。少なくとも1分間分は選択してください",
    ["top_checks"] = "期間別の上位%{subdir}スクリプト",
    ["top_clients"] = "上位 %{protocol} クライアント",
    ["top_interfaces"] = "上位インターフェイス",
    ["top_l7_contacts"] = "上位 %{protocol} L7 Contacts",
    ["top_ndpi_categories"] = "上位カテゴリ",
    ["top_ndpi_protocols"] = "上位プロトコル",
    ["top_profiles"] = "上位プロファイル",
    ["top_receivers"] = "上位受信者",
    ["top_senders"] = "上位送信者",
    ["top_servers"] = "上位 %{protocol} サーバー",
    ["tot_ms"] = "合計ミリ秒",
    ["total_alerted_flows"] = "アラートされたフローの合計",
    ["total_unreachable_flows"] = "合計ポート到達不能フロー",
    ["traffic_rxtx"] = "RX/TXトラフィック",
    ["traffic_vs_errors"] = "トラフィックvsエラー",
    ["trend"] = "傾向",
    ["udp_packets"] = "UDPパケット",
    ["udp_sent_unicast_vs_non_unicast"] = "UDP送信ユニキャストvsUDP送信非ユニキャスト",
    ["unreachable_host"] = "到達できないホスト",
    ["zmq_flow_coll_drops"] = "フローコレクションドロップ",
    ["zmq_flow_coll_udp_drops"] = "コレクションソケットドロップ",
    ["zmq_msg_rcvd_vs_drops"] = "収集されたZMQメッセージvsドロップされたZMQメッセージ",
    ["zmq_received_flows"] = "ZMQ受信フロー",
    ["metrics_prefixes"] = {
      ["broadcast_bytes"] = "ブロードキャスト バイト",
      ["drops"] = "ドロップ",
      ["num_devices"] = "デバイス数",
      ["num_flows"] = "フロー数",
      ["num_hosts"] = "ホスト数",
      ["num_http_hosts"] = "HTTP ホスト数",
      ["num_ms_rtt"] = "RTT ms",
      ["packets"] = "パケット",
    },
    ["metrics_suffixes"] = {
      ["by_as"] = "ASごとの %{metric}",
      ["egress"] = "Egress",
      ["ingress"] = "Ingress",
      ["inner"] = "インナー",
      ["rcvd"] = "受信",
      ["sent"] = "送信",
    },
  },
  ["hash_table"] = {
    ["AggregatedFlowHash"] = "集約されたフロー",
    ["AutonomousSystemHash"] = "自律システム",
    ["CountriesHash"] = "国",
    ["FlowHash"] = "フロー",
    ["HostHash"] = "ホスト",
    ["MacHash"] = "MAC",
    ["VlanHash"] = "VLAN",
  },
  ["host_config"] = {
    ["active_monitor_enable"] = "アクティブモニタは現在無効です. 有効化できます <a href=\"%{url}\">こちら</a>.",
    ["drop_all_host_traffic"] = "すべてのホストトラフィックをドロップ",
    ["hide_from_top"] = "上位トーカーから隠す",
    ["hide_host_from_top_descr"] = "ホスト %{host} を上位トーカーの計算に組み入れません",
    ["host_alias"] = "ホストエイリアス",
    ["host_pool"] = "ホストプール",
    ["host_traffic_policy"] = "ホストトラフィックポリシー",
    ["modify_host_pool_policy_btn"] = "ホストプールポリシーの変更",
    ["trigger_alerts_for_host"] = "ホスト%{host}のアラートを発報する",
    ["trigger_host_alerts"] = "ホストアラートの発報する",
    ["unblock_host_traffic"] = "ホストトラフィックのブロック解除",
  },
  ["host_details"] = {
    ["active_alerted_flows"] = "アクティブなアラートのフロー",
    ["blocked_traffic"] = "ブロックされたトラフィック",
    ["client_score"] = "クライアント",
    ["disabled_alert_type_help"] = "フローステータスに基づいてフローアラート生成をオフにする",
    ["host"] = "ホスト",
    ["host_cannot_be_found_message"] = "ホスト %{host} は見つかりませんでした",
    ["host_parameter_missing_message"] = "ホストパラメータが不足しています (内部エラー ?)",
    ["host_restore_in_progress"] = "ホストをリストア中 %{host}...",
    ["host_traffic_blocked_quota"] = "一部のホストトラフィックは、クォータを超えたためにブロックされました",
    ["host_traffic_blocked_quota_and_shaper"] = "一部のホストトラフィックは、クォータの超過とブロッキングシェーパーによってブロックされています",
    ["host_traffic_blocked_shaper"] = "いくつかのトラフィックがブロッキングシェーパーによりブロックされました",
    ["inactive_host_x"] = "%{host} [アクティブでない]",
    ["list_of_general_purpose"] = "汎用デバイス: %{list}",
    ["list_of_special_purpose"] = "特別な目的のデバイス: %{list}",
    ["multiple_selection"] = "Ctrlキーを押して複数の要素を選択できます",
    ["reset_host_stats"] = "ホスト統計のリセット",
    ["reset_host_stats_confirm"] = "本当に \"%{host} \"ランタイム統計をリセットしますか？",
    ["reset_host_stats_note"] = "<b>注</ b>:このホストの履歴チャートは、リセット後にギャップが起こる場合があります。",
    ["reset_stats_in_progress"] = "ホスト統計は5秒以内にリセットされます",
    ["restore_from_cache_message_v1"] = "<a href=%{js_code}> こちら</a> をクリックすることによりキャッシュから復旧します",
    ["server_score"] = "サーバー",
    ["snmp"] = "SNMP",
    ["status_ignore"] = "無効なフローステータスアラート",
    ["trace_debug_host_info"] = "ホスト:%{hostinfo}, VLAN: %{vlan}",
    ["trace_debug_host_ip"] = "ホスト:%{hostip}, VLAN: %{vlan}",
    ["trace_debug_restored_host_info"] = "復元されたホスト情報",
    ["unknown_device_type"] = "不明なデバイスタイプ",
  },
  ["host_explorer_page"] = {
    ["alerted_flows"] = "警告フロー",
    ["all_flows"] = "全てのフロー",
    ["dns_queries"] = "DNSクエリ vs リプライ",
    ["syn_distribution"] = "SYN 配布",
    ["syn_vs_rst"] = "SYN vs RST",
    ["syn_vs_synack"] = "SYN vs SYNACK",
    ["tcp_bytes_sent_vs_rcvd"] = "TCP Bytes 送信/受信",
    ["tcp_pkts_sent_vs_rcvd"] = "TCP Packets 送信/受信",
    ["unreach_flows"] = "未到達のフロー",
  },
  ["host_pools"] = {
    ["active_pool_members"] = "アクティブなプールメンバー",
    ["add_host_pool_member"] = "ホストプールメンバーを追加",
    ["alias_or_manufacturer"] = "エイリアス / 製造者",
    ["and_associated_members"] = "そのRRDデータと関連付けられたメンバー",
    ["cannot_delete_cp"] = "関連付けられたキャプティブポータルユーザがいる場合にはプールは削除できません",
    ["change_member_pool"] = "メンバープールを変更",
    ["change_pool"] = "プール切替",
    ["children_safe"] = "チャイルドセーフ",
    ["cidr"] = "CIDR",
    ["click_here_host_pool"] = "ホストプールページに行くのにここをクリックしてください",
    ["config_export"] = "設定のエクスポート",
    ["config_import"] = "設定のインポート",
    ["config_import_message"] = "注意: 同じ名前のホストプールは置き換えられます。",
    ["confirm_delete_pool"] = "ホストプールを本当に削除しますか",
    ["confirm_empty_pool"] = "ホストプールから全てのメンバーを本当に削除しますか",
    ["confirm_remove_member"] = "メンバーを本当に削除しますか",
    ["confirm_reset_pool_quotas"] = "プール\"%{pool}\"のカウンタをリセットしますか？",
    ["confirm_reset_quotas"] = "すべてのクォータカウンタをリセットしますか?",
    ["create_new_quotas_here"] = "新しいクォータを <a href=\"%{url}\">こちら</a>で作成できます",
    ["create_pool_hint"] = "新しいプールをプール管理タブから作成できます.",
    ["create_pools"] = "プール管理",
    ["delete_pool"] = "プール削除",
    ["duplicate_member"] = "デュプリケートメンバーアドレス",
    ["duplicate_pool"] = "デュプリケートプール名",
    ["edit_host_pool_member"] = "メンバーの編集",
    ["edit_host_pools"] = "ホストプール編集",
    ["empty_pool"] = "空のプール",
    ["enforce_quotas_per_pool_member"] = "メンバーごとのクォータ",
    ["enforce_shapers_per_pool_member"] = "メンバーごとのシェーパー",
    ["from_pool"] = "ホストプールから",
    ["host_members"] = "ホストプールメンバー",
    ["host_pools"] = "ホストプール",
    ["invalid_member"] = "無効なメンバーアドレスフォーマット",
    ["ip_address"] = "IPアドレス",
    ["ip_filter"] = "IPv4/IPv6",
    ["ipv4"] = "IPv4",
    ["ipv6"] = "IPv6",
    ["mac_address"] = "MACアドレス",
    ["mac_filter"] = "MACアドレス",
    ["manage_pools"] = "プールメンバーシップを管理する",
    ["manufacturer_filter"] = "製造者: %{manufacturer}",
    ["max_members_message"] = "このバージョンのntopngでは1ホストプールあたり%{maxnum}メンバーのみをアサインできます。この制約を除外するにはエンタープライズバージョンにアップグレードしてください",
    ["max_pools_message"] = "このバージョンのntopngでは%{maxnum}のホストプールのみ作成できます。この制約を除外するにはエンタープライズバージョンにアップグレードしてください",
    ["member_address"] = "メンバーアドレス",
    ["member_exists"] = "メンバー \"%{member_name}\" は追加されませんでした. \"%{member_pool}\"にすでにアサインされています。",
    ["member_filter"] = "メンバー: %{member}",
    ["member_type"] = "メンバータイプ",
    ["members_limit_reached"] = "メンバーの制限に達しました",
    ["network"] = "ネットワーク",
    ["network_normalized"] = "ネットワーク \"%{network}\" は0でないホスト識別子を持っています。 \"%{network_normalized}\"を利用しています",
    ["new_host_pool"] = "新しいホストプール",
    ["new_pool"] = "新しいプール",
    ["no_hosts_defined"] = "ホストプールが定義されていません.",
    ["no_pools_defined"] = "ホストプールが定義されていません",
    ["per_member_quotas"] = "メンバーごとのクォータオプションが設定されている場合、各ホストは異なるクォータカウントを持ちます。設定されていない場合、すべてのホストトラフィックはクォータリミットとしてカウントされます",
    ["per_member_shapers"] = "メンバーごとのシェーパーオプションが設定されている場合、各ホストは独自のシェーパーを使用します。設定解除すると、同じプールに属するすべてのホストが同じシェーパーを共有します",
    ["pool"] = "プール名",
    ["pool_name"] = "プール名",
    ["pool_name_pattern_error"] = "小文字と数字とアンダスコアのみ入力してください",
    ["remove_member"] = "メンバー削除",
    ["remove_member_pool"] = "メンバーを本当に削除しますか: '%{member}' このプールから '%{pool}'?",
    ["reset_quotas"] = "クォータをリセット",
    ["search_member"] = "メンバー検索",
    ["select_host_pool"] = "現在の <b>ホストプール</b>",
    ["select_member_type"] = "メンバータイプを選択",
    ["select_new_pool"] = "以下のドロップダウンからメンバー \"%{member}\" をアサインするプールを選択してください:",
    ["specify_member_address"] = "IPv4/IPv6 アドレスかネットワーク、あるいはMACアドレスを指定してください",
    ["specify_pool_name"] = "プール名を指定",
    ["type_valid"] = "タイプは有効 %input",
    ["view"] = "ビュー",
    ["validation"] = {
      ["abort_add_member"] = "新しホストプールメンバーの追加を中止しますか?",
      ["abort_add_pool"] = "新しいプールの追加を中止しますか?",
      ["abort_edit_pool"] = "プールの編集を中止しますか?",
    },
  },
  ["hosts_comparison"] = {
    ["hosts_parameter_missing_message"] = "ホストパラメータが不足しています(内部エラー ?)",
  },
  ["hosts_stats"] = {
    ["all"] = "すべての",
    ["all_hosts"] = "すべてのホスト",
    ["as_geolocation"] = "ASジオロケーション",
    ["as_info"] = "AS情報",
    ["as_path"] = "ASパス",
    ["as_prefixes"] = "ASプレフィックス",
    ["asn_title"] = "AS %{asn}",
    ["bgp_updates"] = "BGP更新",
    ["blacklisted"] = "ブラックリスト",
    ["blacklisted_hosts_only"] = "ブラックリストに登録されたホストのみ",
    ["blocking_traffic_policy_popup_msg"] = "ホストトラフィックはトラフィックポリシーのブロックに影響されています",
    ["broadcast_domain_hosts_only"] = "ブロードキャストドメインホストのみ",
    ["country_title"] = "国 %{country}",
    ["filter_hosts"] = "フィルターホスト",
    ["filtered"] = "フィルター",
    ["filtered_hosts_only"] = "ブロックされたトラフィックを持つホスト",
    ["host_pool"] = "ホストプール %{pool_name}",
    ["hosts"] = "ホスト",
    ["hosts_list"] = "ホストリスト",
    ["hosts_page_title"] = "%{all} %{traffic_type} %{local_remote} %{protocol} %{network} %{ip_version} %{os} ホスト %{country_asn_or_mac} %{vlan}",
    ["idle_hosts_not_listed"] = "アイドルホストがリストされていません。",
    ["in_network"] = " %{network}で",
    ["ipver_title"] = "IPv%{version_num}",
    ["label_blacklisted_host"] = "ブラックリストに登録されたホスト",
    ["label_broadcast_domain_host"] = "ブロードキャストドメイン",
    ["label_local_host"] = "ローカルホスト",
    ["label_remote_host"] = "リモートホスト",
    ["local"] = "ローカル",
    ["local_hosts_only"] = "ローカルホストのみ",
    ["location"] = "ロケーション",
    ["mac_title"] = "MAC%{mac}で",
    ["more_info_about_as_popup_msg"] = "ASに関する詳細情報",
    ["pool_title"] = "プール %{poolname}用",
    ["remote"] = "リモート",
    ["remote_hosts_only"] = "リモートホストのみ",
    ["source_id"] = "ソースID",
    ["traffic_type_all"] = "すべてのホスト",
    ["traffic_type_one_way"] = "単方向トラフィック",
    ["traffic_type_two_ways"] = "双方向トラフィック",
    ["user"] = "ユーザー %{pool_name}",
    ["user_title"] = "ユーザー %{poolname}",
    ["vlan_title"] = "VLAN %{vlan}",
  },
  ["http_page"] = {
    ["distribution"] = "分布",
    ["http_queries"] = "送信されたHTTPクエリ",
    ["http_responses"] = "受信したHTTP応答",
    ["method"] = "メソッド",
    ["other_method"] = "その他のメソッド",
    ["requests"] = "リクエスト",
    ["requests_served"] = "提供されたリクエスト",
    ["response_code"] = "応答コード",
    ["response_code_1xx"] = "1xx (情報)",
    ["response_code_2xx"] = "2xx (成功)",
    ["response_code_3xx"] = "3xx (リダイレクション)",
    ["response_code_4xx"] = "4xx (クライアントエラー)",
    ["response_code_5xx"] = "5xx (サーバーエラー)",
    ["responses"] = "応答",
    ["traffic_received"] = "トラフィック受信",
    ["traffic_sent"] = "トラフィック送信",
    ["virtual_hosts"] = "仮想ホスト",
  },
  ["http_servers_stats"] = {
    ["actual_requests"] = "実際のリクエスト",
    ["bytes_received"] = "バイト受信",
    ["bytes_sent"] = "バイト送信",
    ["http_server_ip"] = "HTTPサーバーのIP",
    ["http_servers"] = "HTTPサーバー",
    ["http_virtual_host"] = "HTTP仮想ホスト",
    ["local_http_servers"] = "ローカルHTTPサーバー",
    ["total_requests"] = "合計リクエスト",
  },
  ["http_status_code"] = {
    ["bad_request"] = "設定したURLが不正/危険な文字を含んできます",
    ["enterprise_only"] = "エンタープライズエディションライセンスが必要です",
    ["forbidden"] = "禁止されたページ",
    ["internal_error"] = "内部サーバーエラー",
    ["not_found"] = "リクエストページが見つかりません",
    ["pro_only"] = "プロフェッショナルエディションライセンスが必要です",
    ["shut_start"] = "リクエストを処理できません,起動中かシャットダウン中の可能性があります",
  },
  ["icmp_page"] = {
    ["icmp_code"] = "コード",
    ["icmp_message"] = "ICMPメッセージ",
    ["icmp_port_unreachable_extra"] = "%{unreach_protocol}ポート%{unreach_port}は%{unreach_host}で到達不能です",
    ["icmp_type"] = "タイプ",
    ["last_rcvd_peer"] = "最後に受信したピア",
    ["last_sent_peer"] = "最後に送信したピア",
    ["packets_received"] = "パケット受信",
    ["packets_sent"] = "パケット送信",
  },
  ["if_stats_config"] = {
    ["companion_interface"] = "コンパニオンインターフェイス",
    ["custom_name"] = "カスタム名",
    ["custom_name_popup_msg"] = "インターフェイスのエイリアスを指定します",
    ["discard_probing_traffic"] = "プローブトラフィックを破棄",
    ["dump_flows_to_database"] = "データベースへのフローのダンプ",
    ["flows_only_interface"] = "フロー専用インターフェイス（再起動が必要）",
    ["hide_from_top_networks"] = "上位ネットワークから隠す",
    ["hide_from_top_networks_descr"] = "例 %{example}",
    ["interface_network_discovery"] = "定期的なインターフェイスネットワーク検出",
    ["interface_speed"] = "インターフェイス速度 (Mbit/s)",
    ["interface_speed_measure_unit"] = "(Mbit/s)",
    ["interface_speed_popup_msg"] = "最大インターフェイス速度を指定",
    ["interface_top_talkers_creation"] = "インターフェイス上位トーカーの作成",
    ["is_mirrored_traffic"] = "Mirrored トラフィック",
    ["referesh_rate_measure_unit"] = "(sec)",
    ["scaling_factor"] = "Ingress パケットサンプリングレート",
    ["scaling_factor_popup_msg"] = "これは、キャプチャインターフェイスのサンプリングレートと一致する必要があります",
    ["show_dyn_iface_traffic"] = "分解されたトラフィックの複製",
    ["show_dyn_iface_traffic_note"] = "このオプションを有効にすると、動的またはカスタムのサブインターフェイスに向けられたトラフィックは、依然としてマスターインターフェイスに表示されます",
    ["trigger_interface_alerts"] = "トリガーインターフェイスアラート",
  },
  ["if_stats_networks"] = {
    ["note_ghost_bcast_domains"] = "%{ghost_icon}は、ゴーストブロードキャストドメインを強調表示します（つまり、インターフェイスIPアドレスネットワークのいずれにも属していない）。",
    ["note_iface_addresses"] = "IPアドレスは、インターフェイス構成から読み取られます。",
    ["note_iface_bcast_domains"] = "ブロードキャストドメインは、ARPトラフィックを検査することにより推測されます。",
  },
  ["if_stats_overview"] = {
    ["active"] = "アクティブ",
    ["alert_events"] = "アラートイベント",
    ["all_counters"] = "すべてのカウンター",
    ["blocked_flows"] = "ブロックされたフロー",
    ["bridged_traffic"] = "ブロックされたトラフィック",
    ["buffer_full"] = "バッファフル",
    ["collected_flows"] = "収集されたフロー",
    ["collected_logs"] = "収集されたログ",
    ["conntrack_flow_entries"] = "Conntrackフローエントリ",
    ["direct_mode"] = "ダイレクトモード",
    ["discarded_probing_traffic"] = "破棄されたプローブトラフィック",
    ["dispatched_logs"] = "配布されたイベント",
    ["dropped_alerts_info"] = "アラートキューがいっぱいのためディスクに書き込むことができなかった現在のアラート数。",
    ["dropped_flows"] = "ドロップされたフロー",
    ["dropped_packets"] = "ドロップされたパケット",
    ["drops_only"] = "ドロップのみ",
    ["dumped_files"] = "ダンプされたファイル",
    ["dumped_packets"] = "ダンプされたパケット",
    ["egress_packets"] = "Egressパケット",
    ["export_disabled"] = "ランタイム設定内でフローエクスポートが無効です",
    ["exported_flows"] = "エクスポートされたフロー",
    ["family"] = "ファミリー",
    ["flows_export_statistics"] = "フローエクスポート統計",
    ["handled_logs"] = "検知されたイベント",
    ["has_traffic_directions"] = "RX/TX 方向",
    ["host_correlations"] = "ホスト収集",
    ["id"] = "ID",
    ["iface_switch_active_msg"] = "選択したインターフェイス<b>%{interface}</b> [ifid: %{ifid}] がアクティブになりました",
    ["iface_switch_empty_session_msg"] = "空のセッション",
    ["iface_switch_error_msg"] = "インターフェイスの切り替え中のエラー",
    ["in_path_interface"] = "インパスインターフェイス (Bump in the Wire)",
    ["ingress_packets"] = "Ingressパケット",
    ["ingress_traffic"] = "Ingressトラフィック",
    ["interface_ip"] = "インターフェイスIP",
    ["interface_name"] = "インターフェイス名",
    ["interface_rx_updates"] = "インターフェイスRXアップデート",
    ["malformed_logs"] = "壊れたログ",
    ["nf"] = "ネットフィルター",
    ["nf_enobufs"] = "バッファスペースなし",
    ["nf_handle_packet_failed"] = "キューに入れられたパケット処理の失敗",
    ["nf_queue_dropped"] = "ドロップされたパケット (キューがいっぱいです)",
    ["nf_queue_pct"] = "キューフィルレベル",
    ["nf_queue_total"] = "キューに入れられたパケット",
    ["nf_queue_user_dropped"] = "ドロップされたパケット (ユーザースペースに送信できませんでした)",
    ["note"] = "注意",
    ["note_drop_ifstats_dynamic"] = "ドロップはメインインターフェイスで参照可能です",
    ["note_drops_sflow"] = "ドロップはネットフローがそれを報告しないためsFlowのみ表示されます。<br><A HREF=https://www.ietf.org/rfc/rfc3176.txt>sFlow</A> ドロップsFlowエージェントがサンプル対象のパケットがリソース不足によってドロップされた数をカウントします。 ドロップカウンターはエージェントが最後にリセットされてからの総数を報告します。 高いドロップレートは管理エージェントがハードウェアにより生成されたサンプルを処理できないことを示します。sFlowサンプリングレートはドロップレートを減らします。 ドロップを検出できないエージェントは常に0と報告します",
    ["note_packets"] = "イーサネットネットワークでは, 各パケットは <A HREF=\"https://en.wikipedia.org/wiki/Ethernet_frame\">24バイトのオーバーヘッド</A> [preamble (7 バイト), フレームの開始 (1 byte), CRC (4 バイト), および <A HREF=\"http://en.wikipedia.org/wiki/Interframe_gap\">IFG</A> (12 バイト)]. このようなオーバヘッドはインターフェイストラフィックにカウントされます。しかしIPアドレス間の通信には追加されません。 これはそのようなデータがインターフェイスの負荷増加になるためです。しかしこれはホストにより交換されるトラフィックにはカウントされません。よってホストとインターフェイスの間にほとんど不整合はありません",
    ["note_probe_zmq_drops_export_queue_full"] = "リモートのプローブはこれらのフロー処理に成功しましたが、 エクスポート キューが満杯であるためエンキューできませんでした",
    ["note_probe_zmq_drops_flow_collection_drops"] = "ドロップされたフローパケット（NetFlow v9）またはドロップされたフロー（NetFlow v5 / IPFIX）",
    ["note_probe_zmq_drops_flow_collection_udp_socket_drops"] = "nProbe収集ソケットでドロップされたフローパケット",
    ["note_probe_zmq_timeout_lifetime"] = "nProbeで通知されたフローライフタイム間隔と収集されたフローで計算された実際のライフタイム.",
    ["note_remote_probe_time"] = "ntopngとnProbe間の時間差分: 0に近いのが理想です. この差分が大きい場合はnProbe/ntopngの時間を同期させてください",
    ["paused"] = "一時停止",
    ["probe_ip"] = "プローブIP",
    ["probe_throughput"] = "プローブ スループット",
    ["probe_timeout_idle"] = "アイドルタイムアウト",
    ["probe_timeout_lifetime"] = "ライフタイムタイムアウト",
    ["probe_zmq_drops_export_queue_full"] = "キューが完全にドロップされたフロー",
    ["probe_zmq_drops_flow_collection_drops"] = "フローコレクションドロップ",
    ["probe_zmq_drops_flow_collection_udp_socket_drops"] = "コレクションソケットドロップ",
    ["probe_zmq_num_endpoints"] = "アクティブプローブZMQエンドポイント",
    ["probe_zmq_num_flow_exports"] = "エクスポートされたフロー",
    ["public_probe_ip"] = "パブリックプローブIP",
    ["received_traffic"] = "受信トラフィック",
    ["remote_flow_lifetime"] = "フローライフタイム",
    ["remote_probe"] = "リモートプローブ",
    ["remote_probe_collector_mode"] = "コレクターモード",
    ["remote_probe_time"] = "nProbe 時刻のズレ",
    ["reset_counters"] = "カウンターをリセット",
    ["rrd_timeseries"] = "RRD時系列データ",
    ["send_error"] = "エラーを送信",
    ["sflow_counter_updates"] = "sFlowカウンターの更新",
    ["shaped_filtered_packets"] = "整形/フィルタリングされたパケット",
    ["snmp"] = "SNMP",
    ["state"] = "状態",
    ["syslog_statistics"] = "Syslog収集統計",
    ["traffic_breakdown"] = "トラフィック分解",
    ["traffic_statistics"] = "トラフィックの統計",
    ["unhandled_flows"] = "処理できないフロー",
    ["unhandled_logs"] = "サポートしていないイベント",
    ["zmq_avg_msg_flows"] = "メッセージあたりの平均フロー",
    ["zmq_encryption_public_key"] = "ZMQ暗号化公開鍵",
    ["zmq_encryption_public_key_note"] = "フローエクスポーターとしてnProbeを使用している場合は、-zmq-encryption-key ' %{key}'をnProbe構成に追加してください。",
    ["zmq_message_drops"] = "ZMQメッセージドロップ",
    ["zmq_message_rcvd"] = "収集されたZMQメッセージ",
    ["zmq_rx_statistics"] = "ZMQ RXの統計",
  },
  ["index_page"] = {
    ["applications"] = "プロトコル",
    ["asns"] = "ASNs",
    ["dashboard"] = "ダッシュボード",
    ["hosts"] = "ホスト",
    ["live_update"] = "リアルタイム 更新",
    ["minute"] = "分",
    ["never"] = "決して",
    ["no_packet_warning"] = "インターフェイス %{ifname}上でまだパケットを検出していません。<p>このページが再読み込みするまで%{countdown}秒お待ちください",
    ["refresh"] = "再読み込み",
    ["refresh_frequency"] = "再読み込み頻度",
    ["refresh_graph_popup_msg"] = "再読み込みグラフ",
    ["seconds"] = "秒",
    ["senders"] = "送信者",
    ["top_flow_talkers"] = "上位フロートーカー",
  },
  ["internals"] = {
    ["alert_drops"] = "アラートドロップ",
    ["alert_drops_descr"] = "インターフェイスのアラートがディスクに書き込めなかったり、ドロップされた場合にアラートをトリガーします。",
    ["all_check_targets"] = "すべてのユーザースクリプトターゲット",
    ["all_hash_tables"] = "すべてのハッシュテーブル",
    ["all_periodic_activities"] = "すべての定期的な活動",
    ["any_periodic_activity_issue"] = "問題のある活動",
    ["chart_script_duration"] = "%{script}期間",
    ["chart_script_rrds"] = "%{script} TS統計",
    ["check"] = "ユーザースクリプト",
    ["check_target"] = "ターゲット",
    ["checks"] = "ユーザースクリプト",
    ["degraded_performance"] = "パフォーマンス低下",
    ["duration"] = "期間",
    ["flow_calls_stats"] = "flow.lua呼び出し統計",
    ["flow_lua_duration"] = "flow.luaの期間",
    ["hash_entries"] = "ハッシュエントリ",
    ["hash_table"] = "ハッシュ表",
    ["hash_table_utilization"] = "ハッシュテーブルの使用率",
    ["hash_tables"] = "ハッシュテーブル",
    ["high_idle_entries"] = "多数のアイドルエントリ",
    ["iface_hash_tables"] = " %{iface}ハッシュテーブル",
    ["iface_periodic_scripts"] = " %{iface}定期的なアクティビティ",
    ["in_progress_since"] = "進行開始から",
    ["last_duration_ms"] = "前回の間隔",
    ["last_num_calls"] = "前回のコール数",
    ["last_start_time"] = "前回の開始",
    ["last_start_time_ago"] = "%{time}前",
    ["lua"] = "Lua",
    ["max_duration_ms"] = "最大期間",
    ["max_duration_secs"] = "最大期間",
    ["missed_idle"] = "アイドル状態から復帰",
    ["missed_periodic_update"] = "定期更新の失敗",
    ["missed_proto_detected"] = "プロト検出の失敗",
    ["num_calls"] = " %{script}コール数",
    ["num_calls_vs_duration"] = "Luaのコールvs間隔",
    ["num_drops"] = "ドロップされたポイント",
    ["num_failed_enqueues"] = "キューイングの失敗",
    ["num_missed_calls"] = "呼び出しを失敗した回数",
    ["num_writes"] = "書き込まれたポイント",
    ["pending_periodic_update"] = "定期更新の保留",
    ["pending_proto_detected"] = "プロト検出の保留",
    ["periodic_activities"] = "定期的な活動",
    ["periodic_activities_descr"] = "説明を読むには、リストされたアクティビティのいずれかにマウスを合わせてください",
    ["periodic_activities_last_start_time_descr"] = "前回の実施は、定期的なアクティビティがいつ最後に開始されたかを示します。",
    ["periodic_activities_max_duration_secs_descr"] = "最大期間は、スクリプトの実行を許可する期間を示します。",
    ["periodic_activities_not_shown"] = "少なくとも1回スケジュールされた定期的なアクティビティのみが表示されます。",
    ["periodic_activities_periodicity_descr"] = "周期は、定期的なアクティビティの実行がスケジュールされる頻度を示します。",
    ["periodic_activities_rrd_descr"] = "TSはTimeseries(時系列データ)の略です。",
    ["periodic_activities_tot_not_executed_descr"] = "Total Not Executedは、実行のスケジュールがされていない定期アクティビティの数をカウントします。理由としては、既に実行状態(遅い実行)もしくは、既にスケジュールされている(利用出来るスレッドがない)といった場合があります。",
    ["periodic_activities_tot_running_slow_descr"] = "トータルランニングスローは、定期的なアクティビティの完了に時間がかかりすぎた回数をカウントします。",
    ["periodic_activity"] = "定期的な活動",
    ["periodic_activity_issues"] = "問題",
    ["periodicity"] = "周期性",
    ["queue"] = "キュー",
    ["queued"] = "キューされた",
    ["queues"] = "複数のキュー",
    ["rrd_drops"] = "TSドロップ",
    ["script_deadline_exceeded"] = "スクリプトの完了に時間がかかりすぎています",
    ["script_deadline_exceeded_descr"] = "スクリプトの完了に時間がかかりすぎています",
    ["script_duration"] = " %{script}期間",
    ["script_not_executed"] = "未実行",
    ["script_not_executed_descr"] = "スクリプトはスケジュールされていますが、まだ実行されていません",
    ["script_num_calls"] = " %{script}コール数",
    ["script_stats"] = " %{script}vs合計",
    ["sleeping"] = "スリープ",
    ["state_active"] = "アクティブ状態のエントリ",
    ["state_idle"] = "アイドル状態のエントリ",
    ["state_idle_vs_active"] = "アイドルエントリvsアクティブエントリ",
    ["status_description"] = "定期的なアクティビティは、次のステータスのいずれかになります:",
    ["status_queued_descr"] = "アクティビティはスケジュールされていますが、まだ実行されていません",
    ["status_running_descr"] = "アクティビティは現在実行中です",
    ["status_sleeping_descr"] = "アクティビティは完了し、次のスケジュールを待っています",
    ["successful_calls"] = "成功したコール",
    ["system_iface_periodic_scripts"] = "システム定期アクティビティ",
    ["time_exceeded"] = "時間超過",
    ["time_usage"] = "使用した時間の割合",
    ["timeseries_queue_length"] = "TS キュー長さ",
    ["timeseries_queued_points"] = "キューされたポイント",
    ["timeseries_writes"] = "TS書き込み",
    ["tot_not_executed"] = "未実行",
    ["tot_running_slow"] = "ランニングスロー",
    ["total_duration"] = " %{subdir}スクリプト継続時間の総計",
    ["total_flow_duration"] = "所要時間の総計",
    ["total_num_calls"] = " %{subdir}スクリプトコール数の総計",
    ["total_stats"] = " %{subdir}スクリプト統計の総計",
    ["work_completion"] = "完了",
  },
  ["live_capture"] = {
    ["active_live_captures"] = "リアルタイム キャプチャー",
    ["note"] = "ホスト %{hostinfo}に対してリアルタイムキャプチャーします。このストリームは1分か100Kパケット受信すると自動的に停止します",
  },
  ["live_traffic"] = {
    ["error_generic"] = "ジェネリックエラー",
    ["error_not_found"] = "見つかりません",
    ["error_not_granted"] = "リクエストが許可されていません",
    ["modal_vlan_tagged_with_bpf_confirmation"] = "このホストはVLANタグ付きです。BPFフィルターを提出しました。 適切にパケットをマッチさせるためにはVLAN 51を含む必要があります。 VLANはフィルター内で <br> <pre>VLAN %{vlan} と <i>...残りのフィルター...</i></pre>といったように指定できます",
    ["modal_vlan_tagged_with_bpf_continue"] = "コンティニュー",
    ["modal_vlan_tagged_with_bpf_title"] = "ホストpcapダウンロード",
  },
  ["local_flow_matrix"] = {
    ["error_no_local_hosts"] = "ローカルホストは見つかりませんでした",
    ["local_flow_matrix"] = "ローカルフローマトリックス",
    ["local_hosts_active_flows_matrix"] = "ローカルホストアクティブフローマトリックス",
  },
  ["local_hosts_stats"] = {
    ["criteria"] = "条件",
    ["looking_glass"] = "ルッキンググラス",
  },
  ["locales"] = {
    ["cz"] = "チェコ語",
    ["de"] = "ドイツ語",
    ["en"] = "英語",
    ["it"] = "イタリア語",
    ["jp"] = "日本語",
    ["pt"] = "ポルトガル語",
  },
  ["login"] = {
    ["add_captive_portal_user"] = "キャプティブポータルユーザーを追加",
    ["add_web_user"] = "Webユーザーを追加",
    ["auth_success"] = "認証成功",
    ["blacklisted_ip_notice"] = "認証失敗が多すぎます。しばらく経ってからリトライしてください",
    ["change_password"] = "パスワード変更",
    ["confirm_password"] = "パスワード確認",
    ["device_label"] = "デバイスラベル",
    ["donation"] = "もし %{product}が有用だと感じたら、寄付を通じて私たちをサポートしてください <A href=\"%{donation_url}\">寄付</A>. あなたのサポートはこのプロジェクトの開発を促進します。ご協力感謝します",
    ["enter_credentials"] = "このネットワークにアクセスするためのあなたの資格情報を入力してください",
    ["informative_captive_join"] = "インターネットにアクセス",
    ["informative_captive_portal_tos"] = "私は <a target='_blank' href='%{url}'>サービス利用条件</a>を読み合意します",
    ["internet_redirecting"] = "インターネットにリダイレクトしています...",
    ["license"] = "%{product}は<a href=\"%{license_url}\">%{license}</a>のライセンスでリリースされています.",
    ["logging_out"] = "ログアウト...",
    ["login"] = "ログイン",
    ["logout"] = "ログアウト",
    ["logout_message"] = "ログアウトしますか？",
    ["logout_user_x"] = "ログアウト %{user}",
    ["must_change_password"] = "デフォルトの管理者パスワードは変更する必要があります。以下に新しいパスワードを入力してください",
    ["password"] = "パスワード",
    ["password_mismatch"] = "パスワード不一致",
    ["password_not_valid"] = "別のパスワードを指定してください",
    ["password_pattern_error"] = "少なくとも5文字を入力してください",
    ["password_ph"] = "パスワード(デフォルト admin)",
    ["passwords_dont_match"] = "パスワードが間違っています",
    ["unable_to_login"] = "ログイン出来ませんか?",
    ["unsupported_browser"] = "ブラウザがサポート外です。<br>最新のFireFoxかChromeを使用してください",
    ["username"] = "ユーザー名",
    ["username_ph"] = "ユーザー名 (デフォルト admin)",
    ["web_users"] = "Webユーザー",
    ["welcome_to"] = "%{product}へようこそ",
  },
  ["mac_details"] = {
    ["and_n_more_hosts"] = "<a href='%{host_url}'> %{host_label} </a>および<a href='%{url}'> %{num}個のホスト</a>",
    ["and_one_more_host"] = "<a href='%{host_url}'> %{host_label} </a>および<a href='%{url}'>もう1台のホスト</a>",
    ["json_error_inactive"] = "JSONエラー:インアクティブなホストがパージされたか%{product}が終了しました",
    ["mac"] = "MAC",
    ["mac_cannot_be_found_message"] = "MAC %{mac} は見つかりませんでした",
    ["mac_host"] = "<a href='%{host_url}'> %{host_label}</a>",
    ["mac_parameter_missing_message"] = "MACパラメータが不足しています (内部エラー?)",
    ["reset_mac_stats"] = "MAC統計をリセットする",
    ["reset_mac_stats_note"] = "<b>注</ b>:このMACアドレスの履歴チャートは、リセット後にギャップが起こる場合があります。",
    ["reset_stats_in_progress"] = "MAC統計は5秒以内にリセットされます",
  },
  ["mac_stats"] = {
    ["all_devices"] = "すべての",
    ["all_layer_2_devices"] = "すべてのMACアドレス",
    ["all_manufacturers"] = "すべての製造者",
    ["arp_total"] = "ARP",
    ["dev_layer_2_devices"] = "%{device_type} MACアドレス",
    ["dhcp_only"] = "DHCPのみ",
    ["filter_macs"] = "MACをフィルター",
    ["idle_devices_not_listed"] = "表示されていないアイドルデバイス.",
    ["layer_2_dev_devices"] = "%{device_type} MACアドレス",
    ["layer_2_devices_with_manufacturer"] = "%{manufacturer} %{title}",
    ["layer_2_source_devices"] = "%{device_type} ソースMACアドレス",
    ["manufacturer"] = "製造者",
    ["source_macs"] = "ソースのみ",
  },
  ["manage_configurations"] = {
    ["active_monitoring"] = "アクティブモニタ",
    ["after_reset_request"] = "<b>再起動</b> %{product} 設定を工場出荷時時にリセットします.",
    ["checks"] = "ユーザースクリプト",
    ["config_import"] = "インポート <span class='item'>{{ type }}</span> 設定",
    ["everything"] = "全ての %{product} 設定 (ユーザー、設定、以下の設定全て)",
    ["manage_configuration"] = "設定管理",
    ["manage_configurations"] = "複数設定の管理",
    ["notifications"] = "エンドポイントと受信者",
    ["pool_endpoint_recipients"] = "プール(SNMP,アクティブモニタ,ユーザースクリプト,エンドポイントと受信者含む)",
    ["snmp"] = "SNMP",
    ["factory_reset"] = {
      ["body"] = "工場出荷時に設定をリセットする. 全ての変更が失われます.",
      ["title"] = "リセット <span class='selected-item'>%{reset_element}</span> 設定",
    },
    ["import_modal"] = {
      ["all_title"] = "インポート %{product} 設定",
      ["title"] = "インポート <span class='selected-item'>%{import_element}</span> 設定",
    },
    ["messages"] = {
      ["import_success"] = "選択した設定は、正常にインポートされました!",
      ["reset_all_success"] = "工場出荷時へのリセットが成功しました. <b>再起動 %{product}</b> 工場出荷時の設定にリセットされます.",
      ["reset_success"] = "選択された設定は、正常にリセットされました!",
    },
  },
  ["manage_data"] = {
    ["all_hosts"] = "すべての",
    ["delete"] = "ホストデータ削除",
    ["delete_active_interface"] = "インターフェイスデータ削除",
    ["delete_inactive_interfaces"] = "非アクティブなインターフェイスの削除",
    ["delete_system_interface_data"] = "システムインターフェイスデータの削除",
    ["delete_tab"] = "削除",
    ["export"] = "ホストデータエクスポート",
    ["export_tab"] = "エクスポート",
    ["hosts"] = "ホスト",
    ["inactive_interfaces_data"] = "アクティブでないインターフェイス",
    ["inactive_interfaces_description"] = "データを管理f <b>アクティブでないインターフェイス</b>, これらのインターフェイスのキャッシュ情報を削除することができます.<br><br>アクティブでないインターフェイスが設定されるとボタンが有効となります.",
    ["ip_or_mac_address"] = "IPかMAC アドレス",
    ["local_hosts"] = "ローカル",
    ["mac_or_ip_required"] = "有効なIPかMACアドレスを入力してください",
    ["manage_data"] = "保存データを管理",
    ["remote_hosts"] = "リモート",
    ["single"] = "シングル",
    ["system_interface_data"] = "システムインターフェイス",
    ["system_interface_description"] = "キャッシュデータの管理 <b>System Interface</b>, 空き領域を確保するためにデータを削除することができます.<br><br>削除要求がない時に下のボタンが有効となります.",
    ["system_interface_note"] = "システムインターフェイスデータには、\"システム \"メニューエントリの下にあるすべてのデータ（CPU、InfluxDB、SNMPデータなど）が含まれます。",
  },
  ["manage_users"] = {
    ["add_new_user"] = "新規ユーザー追加",
    ["administrator"] = "管理者",
    ["allow_pcap_download"] = "PCAPダウンロードを許可",
    ["allow_pcap_download_descr"] = "ライブトラフィックとPCAPのダウンロードを許可する",
    ["allowed_interface"] = "許可されたインターフェイス",
    ["allowed_networks"] = "許可されたネットワーク",
    ["allowed_networks_descr"] = "このユーザーが表示可能なカンマ区切りのネットワークのリストです。例:",
    ["allowed_passwd_charset"] = "許可された文字はISO 8859-1 (latin1)の大文字・小文字、数字と記号です",
    ["any_interface"] = "すべてのインターフェイス",
    ["authentication_lifetime"] = "認証の有効期間",
    ["change_user_password"] = "パスワード変更",
    ["change_user_preferences"] = "ユーザー設定の変更",
    ["confirm_delete_user"] = "%{user}を本当に削除しますがよろしいですか？",
    ["enable_captive_portal"] = "<a href=\"%{url}\">キャプティブポータルを有効化します。</a> キャプティブポータルユーザーを定義することを許可します",
    ["expires_after"] = "期限切れ",
    ["group"] = "グループ",
    ["host_pool_id"] = "ホストプールID",
    ["manage"] = "管理",
    ["manage_user_x"] = "ユーザー管理 %{user}",
    ["manage_users"] = "ユーザー管理",
    ["new_password"] = "新パスワード",
    ["new_password_confirm"] = "新パスワードを確認します",
    ["no_host_pools"] = "ホストプールが定義されていません。 新規に作成してください <a href='%{url}'>作成</a>",
    ["non_privileged_user"] = "非管理者ユーザー",
    ["old_password"] = "古いパスワード",
    ["the_auth_can_be_perpetual"] = "認証は永久か有効期限付きとなります",
    ["the_host_pool_associated"] = "ホストプールは認証成功後にユーザーに関連付けられます",
    ["unlimited"] = "無制限",
    ["user_role"] = "ユーザーロール",
    ["web_captive_users"] = "Web/キャプティブポータルユーザー",
  },
  ["metrics"] = {
    ["days"] = "日",
    ["hours"] = "時",
    ["mins"] = "分",
    ["minutes"] = "記録",
    ["secs"] = "秒",
  },
  ["ndpi_categories"] = {
    ["Advertisement"] = "広告",
    ["Allowed_Site"] = "許可されたサイト",
    ["Antimalware"] = "マルウェア対策",
    ["Banned_Site"] = "禁止サイト",
    ["Chat"] = "チャット",
    ["Cloud"] = "クラウド",
    ["Collaborative"] = "協調",
    ["DataTransfer"] = "データ転送",
    ["Database"] = "データベース",
    ["Download-FileTransfer-FileSharing"] = "ダウンロード-ファイル転送-ファイル共有",
    ["Email"] = "Eメール",
    ["FileSharing"] = "ファイル共有",
    ["Game"] = "ゲーム",
    ["Malware"] = "マルウェア対策",
    ["Media"] = "メディア",
    ["Mining"] = "マイニング",
    ["Music"] = "音楽",
    ["Network"] = "ネットワーク",
    ["Productivity"] = "生産性",
    ["RPC"] = "RPC",
    ["RemoteAccess"] = "リモートアクセス",
    ["Shopping"] = "ショッピング",
    ["Site_Unavailable"] = "利用できないサイト",
    ["SocialNetwork"] = "ソーシャルネットワーク",
    ["SoftwareUpdate"] = "ソフトウェアアップデート",
    ["Streaming"] = "ストリーミング",
    ["System"] = "システム",
    ["Unspecified"] = "不特定",
    ["VPN"] = "VPN",
    ["Video"] = "ビデオ",
    ["VoIP"] = "VoIP",
    ["Web"] = "Web",
  },
  ["ndpi_page"] = {
    ["custom_applications"] = "カスタムアプリケーション",
    ["live_flows_count"] = "リアルタイムフローカウント",
    ["note"] = "注意",
    ["note_historical_per_protocol_traffic"] = "%{what} のトラフィックデータ履歴は以下で有効化可能です <a href='%{url}'>%{flask_icon} 設定</a>.",
    ["note_live_flows_chart"] = "このチャートはTCP接続のみ表示します",
    ["note_possible_probing_alert"] = " %{icon} のようなアイコンはプロービングの可能性を示します (あるいはアプリケーションサーバーのダウン) ホストが特定のアプリケーションプロトコルに対して受信したが応答していないことを示します。<A HREF='%{url}'>履歴レポート</A> を利用することによりこの問題を解析できます",
    ["note_protocol_usage_time"] = "プロトコル利用時間は不連続な間隔で計算されます",
    ["num_contacted_services_as_client"] = "コンタクトされたクライアントサービス",
    ["num_contacted_services_as_client_descr"] = "(TLS, HTTP と DNS 名)",
    ["overview"] = "%{what} 概要",
    ["rcvd"] = "受信",
    ["received_only"] = "受信のみ",
    ["sent"] = "送信",
    ["sent_only"] = "送信のみ",
    ["total_since_startup"] = "合計 (起動からの累計)",
    ["unable_to_find_host"] = "%{host_ip}を見つけることができませんでした (データが失効?)",
  },
  ["nedge"] = {
    ["a_member_address_is"] = "メンバー アドレスはあるデバイスをあるユーザに割り当てるルールです。 以下の優先順位が適用されます:",
    ["a_zero_quota_is_unlimited"] = "クォータ0は無制限のトラフィックを意味します。",
    ["active_user"] = "アクティブユーザ",
    ["add_gateway"] = "ゲートウェイ追加",
    ["add_member"] = "メンバー追加",
    ["add_port_forwarding_rule"] = "ポート転送ルールを追加",
    ["add_routing_policy"] = "ルーティングポリシー追加",
    ["add_some_gateways_first"] = "ゲートウェイが定義されていません。ゲートウェイを設定してください<a href=\"%{url}\">設定</a>.",
    ["add_static_lease"] = "静的リースを追加",
    ["add_static_route"] = "静的ルートを追加",
    ["add_system_user"] = "システムユーザを追加",
    ["add_user_member"] = "ユーザーメンバーを追加",
    ["applied_policy"] = "ポリシーを適用",
    ["apply_and_reboot"] = "適用して再起動",
    ["apply_and_restart_self"] = "適用して再起動",
    ["apply_configuration"] = "設定を適用",
    ["apply_configuration_and_reboot"] = "新規の設定を適用します。ネットワークインターフェイスは指定の設定に従い再設定され、システムは再起動されます。誤った設定はシステムがIP接続を失うことにつながります。設定に問題がないことに確信がある場合にのみ先に進んでください",
    ["apply_configuration_and_restart_self"] = "新規の設定を適用します。これは%{product}の再起動が必要です。このGUIは数秒間無反応になります",
    ["apply_the_initial_device_configuration"] = "初期デバイス設定を適用します",
    ["authentication_gui_and_captive_portal"] = "認証オプションは%{product} GUIと<a href=\"%{url}\">キャプティブポータル</a> へのアクセスに利用されます",
    ["bandwidth_allocation"] = "帯域幅の割り当て",
    ["bind_to_any"] = "任意のインターフェース",
    ["bind_to_lan"] = "LANインターフェイス",
    ["cannot_rename_gateway"] = "ゲートウェイをリネームできません.",
    ["cannot_rename_policy"] = "ポリシーをリネームできません.",
    ["cannot_rename_static_route"] = "静的ルートをリネームできません.",
    ["captive_portal_other_methods"] = "キャプティブポータルは <a href=\"%{url}\">認証設定</a> で指定可能な認証オプション(例 Radius)を利用します",
    ["categories_policies"] = "カテゴリポリシー",
    ["configure_routing_policies"] = "ルーティングポリシーの構成",
    ["configured_members"] = "構成されたメンバー",
    ["confirm_delete_gateway"] = "ゲートウェイ\"%{gw}\"を本当に削除しますか?",
    ["confirm_delete_policy"] = "ポリシー \"%{policy}\"を本当に削除しますか?",
    ["confirm_delete_port_forwarding_rule"] = "指定したルールを本当に削除しますか？",
    ["confirm_delete_static_lease"] = "\"%{hostname}\" 静的リースを本当に削除しますか?",
    ["confirm_delete_static_route"] = "静的ルート \"%{sr}\"を本当に削除しますか",
    ["confirm_remove_member_from_user"] = "ユーザ %{user}からメンバー %{member}を削除しますか?",
    ["could_not_add_port_forwarding"] = "ポート転送ルールを追加できませんでした",
    ["custom"] = "カスタム",
    ["custom_datetime"] = "現在の時刻",
    ["custom_datetime_descr"] = "システムの日時をマニュアル設定します",
    ["custom_policy"] = "カスタムポリシー",
    ["data_reset"] = "データリセット",
    ["data_reset_msg"] = "すべてのデバイスデータを本当に削除しますか? データには保存された時系列データ, フロー, アラート, Redisキャッシュとホストカウンターを含みます。システムセットアップのデバイス設定はリセットにより影響を受けません。<br>この操作は%{product}の再起動につながり、GUIは数秒無反応になります。",
    ["date_time"] = "日付と時刻",
    ["default_categories"] = "デフォルトカテゴリポリシー",
    ["default_categories_with_a_defined_policy"] = "デフォルトカスタムカテゴリポリシー",
    ["default_categories_with_a_defined_quota"] = "デフォルトクォータカテゴリポリシー",
    ["default_categories_with_policy_x"] = "デフォルト %{x} カテゴリポリシー",
    ["default_gateway"] = "デフォルトゲートウェイ",
    ["default_policy"] = "デフォルトポリシー",
    ["default_protocols"] = "デフォルトプロトコルポリシー",
    ["default_protocols_with_a_defined_policy"] = "デフォルトカスタムプロトコル ポリシー",
    ["default_protocols_with_a_defined_quota"] = "デフォルトクォータプロトコル ポリシー",
    ["default_protocols_with_policy_x"] = "デフォルト %{x} プロトコルポリシー",
    ["default_shaper"] = "パス",
    ["default_shaper_descr"] = "インターフェイス速度のデフォルトの帯域幅の割合。",
    ["delete_gateway"] = "ゲートウェイ削除",
    ["delete_policy"] = "ポリシー削除",
    ["delete_port_forwarding_rule"] = "ポート転送ルールを削除",
    ["delete_static_lease"] = "静的リース削除",
    ["delete_static_route"] = "静的ルート削除",
    ["device_protocol_policy_has_higher_priority"] = "デバイスポリシーはプロトコルポリシーに優先します",
    ["device_protocols_blocked_warning"] = "%{device_protocols_policies} が無効化されているためトラフィックはブロック <b>されません</b> ",
    ["device_protocols_policies"] = "デバイスポリシー",
    ["dhcp_disabled_warning"] = "DHCPサーバーがネットワーク上で利用可能でない場合には、クライアントは自動的には接続しません",
    ["dhcp_first_ip"] = "最初のIPアドレス",
    ["dhcp_first_ip_descr"] = "DHCPアドレスプールの最初のIPアドレス",
    ["dhcp_last_ip"] = "前回のIP アドレス",
    ["dhcp_last_ip_descr"] = "DHCPアドレスプールの前回のIPアドレス",
    ["dhcp_leases"] = "DHCPリース",
    ["dhcp_server"] = "DHCPサーバー",
    ["dhcp_server_description"] = "DHCPサーバーを有効にします。",
    ["dhcp_server_not_enabled"] = "DHCPサーバーが有効化されていません。<a href=\"%{url}\">有効化</a>で有効化できます.",
    ["dns_configuration"] = "DNS 設定",
    ["dns_server_preset"] = "DNSサーバープリセット",
    ["dns_server_preset_descr"] = "DNSサーバーはリストから選択するか、指定することができます",
    ["download_description"] = "インターフェイスの最大ダウンロード速度",
    ["download_speed"] = "ダウンロード 速度",
    ["edit_gateway"] = "ゲートウェイ編集",
    ["edit_quotas"] = "クォータ編集",
    ["edit_routing_policy"] = "ルーティングポリシー編集",
    ["edit_static_route"] = "静的ルート編集",
    ["edit_user"] = "ユーザーを編集",
    ["edit_users"] = "ユーザーを編集",
    ["enable_device_protocols_policies"] = "デバイスプロトコルポリシー",
    ["enable_interface"] = "インターフェイス有効化",
    ["enable_interface_descr"] = "ネットワークインターフェイスの有効化あるいは無効化",
    ["enable_nat"] = "NAT有効化",
    ["enable_nat_descr"] = "有効にすると、クライアントのIPアドレスはWANインターフェースのIPアドレスで偽装されます。",
    ["enforce_global_dns"] = "グローバルDNSを強制",
    ["enforce_global_dns_description"] = "DNSフォージングを有効にして、指定されたグローバルDNSをデバイスに強制的に使用させます。",
    ["enternal_port_overlap"] = "外部ポートは、 %{ip}に割り当てられたポート%{port}と重複しています",
    ["external_port"] = "外部ポート（WAN）",
    ["external_port_or_range"] = "外部ポートまたはポート範囲",
    ["factory_reset"] = "工場出荷初期状態にリセット",
    ["factory_reset_msg"] = "デバイスを工場初期出荷状態に本当に戻しますか? 全ての設定、データが失われます",
    ["filter_policies"] = "フィルターポリシー",
    ["gateway"] = "ゲートウェイ",
    ["gateway_exists"] = "ゲートウェイ %{gw} は既に存在します",
    ["gateway_interface_descr"] = "ゲートウェイIPアドレスを決定するためのネットワークインターフェイスを指定してください",
    ["gateway_name"] = "ゲートウェイ名",
    ["gateway_type"] = "モード",
    ["gateway_type_address_descr"] = "ゲートウェイのIPアドレスを指定してください",
    ["gateway_x"] = "ゲートウェイ %{gw}",
    ["gateways"] = "ゲートウェイ",
    ["gateways_status"] = "ゲートウェイの状態",
    ["global_dns"] = "グローバルDNS",
    ["global_shapers_rates"] = "グローバル帯域幅制御",
    ["high_priority"] = "優先",
    ["higher_priority"] = "更に優先",
    ["hostname"] = "ホスト名",
    ["hostname_lease_exists"] = "ホスト名\"%{hostname}\"に対する静的リースが既に存在しています",
    ["interfaces_configuration"] = "インターフェイス設定",
    ["internal_ip"] = "内部IP（LAN）",
    ["internal_port"] = "内部ポート（LAN）",
    ["internal_port_number"] = "内部ポート番号",
    ["internal_port_overlap"] = "内部ポートは、既に %{external_port}にマップされている外部ポート%{internal_port}と重複しています",
    ["invalid_dhcp_range"] = "DHCP範囲が無効。DHCP IPアドレスがLANのネットワーク範囲内であるかを確認してください。あなたのデバイスはインターネット接続できません",
    ["invalid_port_range"] = "無効なポート範囲",
    ["ip_address_takes_preced"] = "IPアドレスはネットワークアドレスよりも優先されます",
    ["ip_lease_exists"] = "IP \"%{ip}\" に対する静的リースがすでに存在しています",
    ["lan"] = "LAN",
    ["lan_ip_addr"] = "LANIP アドレス",
    ["lan_ip_addr_descr"] = "LAN側IPアドレスを指定してください",
    ["lan_recovery_ip"] = "LANリカバリIP",
    ["lan_recovery_ip_descr"] = "LANインターフェイス上の%{product}にリーチするためのLANインターフェイス上のデバイスの追加IPアドレス",
    ["lan_recovery_warning"] = "これがほかの定義されているネットワークと競合しないことを確認してください",
    ["link_not_ready"] = "リンク準備ができていません",
    ["linkdown"] = "リンクダウン",
    ["low_priority"] = "優先度が低い",
    ["lower_priority"] = "更に優先度が低い",
    ["mac_address_takes_preced"] = "MACアドレスはIPアドレスに優先します",
    ["mac_lease_exists"] = "MAC \"%{mac}\"に対する静的リースがすでに存在してます",
    ["max_bps_per_host"] = "最大 %{bps} per ホスト",
    ["max_perc"] = "最大 %{perc}%%",
    ["max_rrt"] = "最大 RTT",
    ["max_rrt_description"] = "モニターアドレスへの最大往復時間。測定されたRTTがこの値を超える場合、ゲートウェイはダウンしていると見なされます。",
    ["maximum_enforced"] = "最大値強制",
    ["maximum_enforced_descr"] = "最大値強制を設定すると、使用可能な帯域幅に関係なく、最大帯域幅の割り当てが強制されます。",
    ["member_address"] = "メンバーアドレス",
    ["member_exists"] = "メンバー %{member} は追加されませんでした。なぜならば%{existing_user}にすでにアサインされているからです",
    ["members"] = "メンバー",
    ["members_for_user_x"] = "ユーザーのメンバー %{user}",
    ["min_perc"] = "最小 %{perc}%%",
    ["minimum_guaranteed"] = "最低保証",
    ["minimum_guaranteed_descr"] = "最小保証に設定すると、最小帯域幅の割り当てのみが保証されます。",
    ["mode"] = "モード",
    ["nedge"] = "nEdge",
    ["network_conf_dhcp"] = "DHCP",
    ["network_conf_iface_descr"] = "インターフェイス IPアドレスモードを指定してください.",
    ["network_conf_iface_gw"] = "インターフェイスのデフォルトゲートウェイのIPアドレスIP(例. 192.168.1.1)。",
    ["network_conf_iface_ip"] = "インターフェイスのIPアドレス (例. 192.168.1.10)。",
    ["network_conf_iface_nmask"] = "インターフェースのネットワークマスク(例：255.255.255.0)。",
    ["network_conf_iface_title"] = "%{ifrole} インターフェイス %{ifname}",
    ["network_conf_static"] = "静的",
    ["network_conf_vlan_trunk"] = "VLANトランク",
    ["no_gateways_defined"] = "ゲートウェイが定義されていません",
    ["no_interfaces_available"] = "有効なインターフェイスがありません",
    ["no_routing_policies_defined"] = "ルーティング ポリシーが定義されていません",
    ["no_rules"] = "ルールなし",
    ["no_static_leases"] = "静的リースがありません",
    ["no_static_routes_defined"] = "静的ルートが定義されていません",
    ["ntp_sync"] = "NTP同期",
    ["ntp_sync_descr"] = "NTPサーバーとの同期を切り替える",
    ["per_ip_rates"] = "ホストごとの帯域幅制御",
    ["ping_address"] = "モニターアドレス",
    ["ping_address_description"] = "ゲートウェイ状態を確認するためのモニターのアドレス",
    ["please_fill_out_this_field"] = "このフィールドを埋めてください",
    ["policy_due_to_source"] = "%{source}による%{policy} ",
    ["policy_name"] = "ポリシー名",
    ["port_forwarding"] = "ポート転送",
    ["power_off"] = "電源を切る",
    ["power_off_confirm"] = "デバイスの電源を切ってもよいですか?",
    ["priority_x"] = "優先度 %{x}",
    ["product_status"] = "%{product} ステータス",
    ["protocol_policy_has_higher_priority"] = "プロトコルポリシーはカテゴリポリシーに優先します",
    ["protocols_policies"] = "プロトコルポリシー",
    ["reboot"] = "再起動",
    ["reboot_corfirm"] = "デバイスを再起動しますか?",
    ["remote_assistance"] = "リモートアシスタンス",
    ["remove_all_members"] = "すべてのメンバーを削除",
    ["remove_all_members_confirm"] = "ユーザー%{user}のメンバーを全員削除しますか?",
    ["reset_and_reboot"] = "リセットと再起動",
    ["reset_and_restart_self"] = "リセットと再起動",
    ["restrict_weg_gui_access_description"] = "この管理用WebGUIをLANからのみアクセス可能にします (より安全) あるいは他のインターフェイスからのアクセスを可能にします (安全でない).",
    ["restrict_weg_gui_access_title"] = "管理WebGUIアクセス",
    ["router"] = "ルーター",
    ["router_mode_requires_enterprise"] = "ルーターモードはエンタープライズエディションでのみ利用可能です",
    ["routing"] = "ルーティング",
    ["routing_policies"] = "ルーティングポリシー",
    ["routing_policy"] = "ルーティングポリシー",
    ["routing_policy_exists"] = "ルーティングポリシー \"%{policy}\"は既に存在します",
    ["safe_search_description"] = "不適切なコンテンツをブロックするためにチャイルドセーフDNSを利用します",
    ["search_categories"] = "カテゴリ検索",
    ["search_protocols"] = "プロトコル検索",
    ["security"] = "セキュリティ",
    ["set_the_device_mode"] = "デバイス操作モードを設定します",
    ["setup_apply"] = "適用",
    ["setup_config_edited_descr"] = "新しい設定を有効にするために変更を適用します。あるいは現状の設定を利用するために変更を破棄します。",
    ["setup_config_edited_title"] = "システム設定が編集されました!",
    ["setup_discard"] = "破棄",
    ["setup_mode"] = "オペレーティングモード",
    ["setup_network"] = "ネットワーク",
    ["shapers"] = "帯域幅制御",
    ["single_port_router"] = "シングルポートルーター",
    ["slow_pass_shaper_descr"] = "インターフェイス速度における遅いパス帯域幅の割合。",
    ["slower_pass_shaper_descr"] = "インターフェイス速度における更に遅いパス帯域幅の割合。",
    ["specify_a_protocol"] = "アプリケーションプロトコルを指定",
    ["static_dhcp_leases"] = "DHCPサーバー静的リース",
    ["static_route_address_descr"] = "静的ルートの宛先を指定",
    ["static_route_exists"] = "静的ルート %{sr}は既に存在します",
    ["static_route_is_local_descr"] = "これがローカルネットワークへの静的ルートであるかどうかを指定します。ローカルネットワークへの静的ルートのすべてのホストは、ローカルホストとして扱われます。",
    ["static_route_is_local_title"] = "ローカルネットワークの静的ルート",
    ["static_route_name"] = "静的ルート名",
    ["static_route_netmask_descr"] = "静的ルートの宛先に適用するネットマスクを指定します。",
    ["static_route_via_descr"] = "静的ルートのネクストホップとして使用するIPアドレスを指定します。",
    ["static_route_x"] = "静的ルート %{sr}",
    ["static_routes"] = "静的ルート",
    ["static_routes_in_policies"] = "静的ルートは、定義されている <a href=\"%{url}\">ルーティング ポリシー</a>のいずれかに適用されます（優先されます）。",
    ["status_activate"] = "アクティブ",
    ["status_connected"] = "接続済み",
    ["status_connecting"] = "接続中",
    ["status_deactivate"] = "無効化",
    ["status_disabled"] = "無効化",
    ["status_disconnected"] = "切断",
    ["status_down"] = "ダウン",
    ["status_failed"] = "失敗",
    ["status_inactive"] = "インアクティブ",
    ["status_unreachable"] = "到達不可",
    ["status_up"] = "アップ",
    ["system_setup"] = "システム設定",
    ["system_users"] = "システムユーザ",
    ["test_policies"] = "テストポリシー",
    ["test_policies_page_description"] = "このページを使用して、現在の構成に基づいて、どのデバイスが%{user} デバイスに適用されるかを確認できます。",
    ["the_fallback_policy"] = "<a href=\"%{url}\">フォールバックポリシー</a>は、特定のアプリケーションとその対応するカテゴリのユーザーポリシーとグローバルポリシーの両方が\"Default \"に設定されている場合に適用されます。",
    ["the_not_assigned_user_contains"] = "\"Not Assigned\"ユーザーを使用して、どのユーザーにも割り当てられていないデバイスのポリシーを定義できます。",
    ["the_primary_dns_server"] = "プライマリーDNSサーバー",
    ["the_secondary_dns_server"] = "セカンダリDNSサーバー",
    ["timezone"] = "タイムゾーン",
    ["timezone_descr"] = "システムのタイムゾーンを設定します",
    ["upload_description"] = "インターフェイスの最大アップロード速度",
    ["upload_speed"] = "アップロード速度",
    ["user"] = "ユーザー",
    ["user_categories"] = "ユーザー%{user}のカテゴリポリシー",
    ["user_categories_with_a_defined_policy"] = " ユーザー%{user}のカスタムカテゴリポリシー",
    ["user_categories_with_a_defined_quota"] = " ユーザー%{user}のクォータカテゴリポリシー",
    ["user_categories_with_policy_x"] = "ユーザー%{user}の%{x} カテゴリポリシー ",
    ["user_category_policies"] = "ユーザーカテゴリポリシー",
    ["user_device_protocols"] = "%{user} デバイスのプロトコル ポリシー ",
    ["user_protocols"] = "ユーザ %{user}のプロトコルポリシー ",
    ["user_protocols_with_a_defined_policy"] = "ユーザ %{user}のカスタムプロトコルポリシー",
    ["user_protocols_with_a_defined_quota"] = "ユーザ %{user}のQuota プロトコルポリシー",
    ["user_protocols_with_policy_x"] = "ユーザ %{user}の%{x} プロトコルポリシー ",
    ["users_list"] = "リスト",
    ["view_quotas"] = "クォータを表示",
    ["wan"] = "WAN",
    ["wan_interface"] = "WANインターフェイス",
    ["with_quota"] = "クォータあり",
    ["x_shaper_per_host_descr"] = "%{shaper} ホストあたりの最大帯域量確保。 0は無制限を意味する.",
    ["you_can_assign_routing_policy"] = "\"ルーティングポリシーを<a href=\"%{url}\">ユーザー</a>に割り当てることができます。\"",
    ["you_can_assign_shapers_to_users_protocols"] = "<a href=\"%{url}\">ユーザー</a>の特定のアプリケーションに帯域幅ポリシーを割り当てるか、<a href=\"%{url2}\">フォールバックポリシー</a>としてグローバルに設定できます。",
    ["you_can_set_static_dhcp_lease_here"] = "静的DHCPリースは<a href=\"%{url}\">こちら</a>で設定できます。",
    ["you_can_use_gateways_in_policies"] = "<a href=\"%{url}\">ルーティング ポリシー</a>でゲートウェイの優先順位を指定できます。",
    ["policy_source_labels"] = {
      ["policy_source_category"] = "カテゴリポリシー",
      ["policy_source_default"] = "アンノウン",
      ["policy_source_device_protocol"] = "デバイスプロトコルポリシー",
      ["policy_source_pool"] = "デフォルトポリシー",
      ["policy_source_protocol"] = "プロトコルポリシー",
    },
  },
  ["network_alert_config"] = {
    ["trigger_alerts_for_network"] = "ネットワーク%{network}のアラートをトリガー",
    ["trigger_network_alerts"] = "ネットワークアラートのトリガー",
  },
  ["network_details"] = {
    ["network"] = "ネットワーク",
    ["network_alias"] = "ネットワークエイリアス",
    ["network_parameter_missing_message"] = "ネットワークパラメータが不足しています(internalエラー?)",
    ["no_available_stats_for_network"] = "ネットワーク %{network}の統計が利用不可能です",
  },
  ["network_stats"] = {
    ["network_list"] = "ネットワークリスト",
    ["network_name"] = "ネットワーク名",
    ["networks"] = "ネットワーク",
    ["networks_traffic_with_ipver"] = "IPv%{ipver} トラフィックの %{networks}",
    ["note_broader_network"] = "より広いネットワークは小さなネットワークのホストは含みません",
    ["note_overlapping_networks"] = "注意: オーバラップした ネットワークを定義している場合:",
    ["note_see_both_network_entries"] = "上記テーブルで両方のネットワークエントリを見ることができます",
  },
  ["nindex"] = {
    ["max_results"] = "最大の結果",
    ["run_query"] = "クエリを実行",
    ["select_note"] = "すべてのフィールドまたはフィールドのコンマ区切りリスト（<field1>、<field2>、...）に*を使用します",
    ["time_range"] = "時間の範囲",
    ["where_note"] = "&lt; field&gt;を使用&lt; operator&gt; &lt; value&gt;条件、こちらで&lt; operator&gt; &lt;,&gt;,=,！=のようになります。 AND / OR演算子も条件を組み合わせることができます。文字列フィールドには引用符を使用してください（例:INFO = 'www.ntop.org'）",
  },
  ["os_stats"] = {
    ["hosts_by_operating_system"] = "オペレーティングシステムによるホスト",
  },
  ["packets_page"] = {
    ["arp_distribution"] = "ARP分布",
    ["ip_version_distribution"] = "IPバージョン分布",
    ["lost"] = "ロスト",
    ["out_of_order"] = "故障中",
    ["received_distribution"] = "受信分布",
    ["retransmissions"] = "再送信",
    ["sent_distribution"] = "送信分布",
    ["sent_vs_rcvd_distribution"] = "送信済みvsRcvdの分布",
    ["size_distribution"] = "サイズ分布",
    ["tcp_flags_distribution"] = "TCPフラグの分布",
    ["tcp_flags_vs_arp_distribution"] = "TCPフラグ/ ARP分布",
    ["tcp_packets_analysis"] = "TCPパケット分析",
    ["version_vs_flags_distribution"] = "IPバージョン/ TCPフラグの分布",
  },
  ["peers_page"] = {
    ["host"] = "ホスト",
    ["no_active_flows_message"] = "指定のホストではアクティブなフローが観測されません",
    ["top_peer_protocol"] = "上位ピアプロトコル",
    ["top_peers_for_host"] = "上位 %{hostkey} ピア",
    ["traffic_volume"] = "トラフィックボリューム",
  },
  ["periodic_activities_descr"] = {
    ["5min.lua"] = "アクティブなホストを繰り返し、その時系列データを書き込みます。そして、5分単位でユーザースクリプトを実行します.",
    ["daily.lua"] = "古いデータのクリーンアップを実行します。毎日の粒度でユーザースクリプトを実行します.",
    ["dequeue_flows_for_hooks.lua"] = "フロー、ホスト、その他のハッシュエントリのメモリ内ライフサイクルを処理します。フローユーザースクリプトを実行します。フローをデータベースにダンプします.",
    ["discover.lua"] = "要求された定期的なネットワーク検出を実行します.",
    ["hourly.lua"] = "毎時にユーザースクリプトを実行します.",
    ["housekeeping.lua"] = "n2diskのジョブを確認し、リストをリロードします.",
    ["minute.lua"] = "アプリケーションなどの拡張インターフェイス時系列データを保存します.ユーザースクリプト、ポッド、コンテナの時系列データを記述します.毎分ユーザースクリプトを実行します.",
    ["notifications.lua"] = "アラートの通知を処理します. そして設定した受信者に送信します.",
    ["periodic_checks.lua"] = "周期的なユーザースクリプトを実行します.",
    ["second.lua"] = "総トラフィックなどの基本的なインターフェイス時系列データを書き込みます.",
    ["stats_update.lua"] = "システムステータスをアップデートします (例: CPUロード).",
    ["timeseries.lua"] = "InfluxDBに時系列データを書き込みます. InfluxDBを使用しない場合は反映されません.",
  },
  ["please_wait_page"] = {
    ["command"] = "コマンド",
    ["database"] = "データベース",
    ["host"] = "ホスト",
    ["id"] = "ID",
    ["info"] = "情報",
    ["operations_on_database_msg"] = "データベースに対して現在実行中の操作は以下です: ",
    ["state"] = "状態",
    ["time"] = "時刻",
    ["user"] = "ユーザー",
    ["waiting_for_db_msg"] = "データベーススキーマの移行を実行中です。 データベース <b>%{dbname}</b> が操作可能になるのを待っています。 データベースの準備が整うとすぐにリダイレクトされます。既存のデータベースのサイズによっては、この操作に時間がかかる場合があります。移行は一度だけ実行され、メジャーバージョンのアップグレード中にのみ実行されます。疑問がある場合は、 <a href='mailto:info@ntop.org'>info@ntop.org</a>の開発者に連絡してください。",
  },
  ["plugins_overview"] = {
    ["action_view"] = "コードエディタでユーザースクリプトソースを開く",
    ["alert_key"] = "アラートキー",
    ["edition_only"] = "%{edition} のみ",
    ["filters"] = "フィルター",
    ["hooks"] = "フック",
    ["loaded_plugins"] = "ロードされたプラグイン",
    ["plugin"] = "プラグイン",
    ["reload_plugins"] = "プラグインをリロードする",
    ["script"] = "スクリプト",
    ["source_location"] = "ソースの場所",
    ["status_key"] = "ステータスキー",
    ["type"] = "タイプ",
  },
  ["policy_presets"] = {
    ["business"] = "ビジネス",
    ["business_description"] = "会社従業員のプリセット",
    ["child"] = "子供",
    ["child_description"] = "子供のプリセット。SNSをブロックしゲーム時間を制限します",
    ["default"] = "デフォルト",
    ["guests"] = "ゲスト",
    ["guests_description"] = "ゲストのプリセット。基本的なナビゲーションのみ許可します",
    ["no_obfuscation"] = "秘匿化なし",
    ["no_obfuscation_description"] = "トラフィック秘匿化を妨げるプリセット",
    ["walled_garden"] = "Walled Garden",
    ["walled_garden_description"] = "VPNとソーシャルネットワークをブロック",
  },
  ["pool_details"] = {
    ["host_pool"] = "ホストプール",
    ["host_pool_timeseries_enable_message"] = "ホストプールの時系列データは<A HREF=\"%{url}\">%{icon_flask} 設定</A>で有効化できます。最初のデータポイントを見るのに数分かかります",
    ["no_available_data_for_host_pool_message"] = "ホストプール '%{pool_name}'にデータがありません",
    ["pool_parameter_missing_message"] = "パラメータが不足しています (内部エラー ?)",
  },
  ["pool_stats"] = {
    ["host_pool_list"] = "ホストプールリスト",
  },
  ["pools"] = {
    ["config_import"] = "設定のインポート",
    ["config_import_message"] = "<b>注意</b>: 同じ名前のプールは上書きされます.",
    ["configuration"] = "ユーザースクリプト設定",
    ["edit_host_pool"] = "ホストプールの編集",
    ["edit_pools"] = "プールの編集",
    ["edit_recipients"] = "受信者の編集",
    ["family"] = "ファミリー",
    ["id"] = "ID",
    ["insert_name"] = "例. MyPool",
    ["members"] = "メンバー",
    ["name"] = "名前",
    ["name_requirements"] = "名前は、1文字以上で空文字は受け付けられません.",
    ["no_recipients"] = "上記のプールに対する受信者が設定されていません.",
    ["not_assigned"] = "アサインされていません",
    ["pool"] = "プール",
    ["pools"] = "プール群",
    ["recipients"] = "受信者",
    ["remove_pool"] = "本当に'%{pool}'プールを削除してもよろしいですか?",
    ["select_config"] = "ユーザースクリプト設定の選択",
    ["some_recipients"] = "${recipients} 上記プールに設定されている受信者.",
    ["unbounded_members"] = "他に所属していないユーザー",
    ["pool_name"] = {
      ["active_monitoring"] = "アクティブモニタ",
      ["flow"] = "フロー",
      ["host"] = "ホスト",
      ["host_pool"] = "ホストプール",
      ["interface"] = "インターフェイス",
      ["local_network"] = "ローカルネットワーク",
      ["mac"] = "デバイス",
      ["snmp_device"] = "SNMPデバイス",
      ["system"] = "システム",
    },
    ["pool_names"] = {
      ["active_monitoring"] = "アクティブモニタ",
      ["all"] = "全て",
      ["flow"] = "フロー",
      ["host"] = "ホスト",
      ["host_pool"] = "ホストプール",
      ["interface"] = "インターフェイス",
      ["local_network"] = "ローカルネットワーク",
      ["mac"] = "デバイス",
      ["snmp_device"] = "SNMPデバイス",
      ["system"] = "システム",
    },
    ["pool_names_suffix"] = {
      ["active_monitoring"] = "アクティブモニタプール",
      ["interface"] = "インターフェイスプール",
      ["local_network"] = "ローカルネットワークプール",
      ["snmp"] = "SNMPプール",
    },
  },
  ["port_details"] = {
    ["active_flows_for_host_and_port"] = "%{host}:%{port}のアクティブなフロー",
    ["active_flows_on_port"] = "ポート %{port}のアクティブなフロー",
    ["active_flows_on_port_symbolic"] = "ポート %{port} [%{symbolic_port}]のアクティブなフロー",
    ["l4_port"] = "L4ポート",
  },
  ["ports_page"] = {
    ["client_ports"] = "クライアントポート",
    ["num_contacted_ports"] = "ユニークポートコンタクトの合計",
    ["num_contacted_ports_as_client"] = "リモートコンタクトされたポート (クライアントとして)",
    ["num_host_contacted_ports_as_server"] = "ローカルでコンタクトされたポート (サーバーとして)",
    ["server_ports"] = "サーバーポート",
  },
  ["prefs"] = {
    ["active_local_host_cache_interval_description"] = "アクティブなホストのキャッシュダンプの間隔 デフォルト: 1時間.",
    ["active_local_host_cache_interval_title"] = "アクティブなローカルホストのキャッシュ間隔",
    ["admin_group_description"] = "管理者として認証するためのユーザーのグループ名",
    ["admin_group_title"] = "LDAP管理グループ",
    ["alert_entity"] = "アラートエンティティ",
    ["alerts_notifications"] = "エンドポイントのアラート",
    ["alerts_retention"] = "アラートの保持",
    ["alerts_settings"] = "アラートの設定",
    ["all"] = "すべて",
    ["auth_methods_order"] = "認証オプションの優先順位はリストの順番と同一です。例. LDAPとローカル認証がともに有効化されている場合にはLDAPが最初にチェックされ、認証に失敗した場合にはローカル認証がチェックされます",
    ["authentication"] = "認証",
    ["authentication_duration"] = "認証期間",
    ["authentication_duration_descr"] = "認証済みユーザーのセッションが終了し、再認証が必要になるまでに行った変更は、新しく作成されたセッションにのみ影響します。",
    ["authentication_duration_title"] = "認証されたセッションの期間",
    ["authentication_midnight_expiration_descr"] = "真夜中にセッションを切り替えて、認証されたすべてのユーザーに真夜中以降の再認証を強制します。",
    ["authentication_midnight_expiration_title"] = "真夜中に認証を行ったセッションの終了",
    ["behaviour"] = "トラフィックの振る舞い",
    ["bind_dn_description"] = "LDAPサーバーのバインド識別名. 例:",
    ["bind_dn_title"] = "LDAPバインドDN",
    ["bind_pwd_description"] = "LDAPサーバーでの認証に使用されるバインドパスワード.",
    ["bind_pwd_title"] = "LDAPバインド認証パスワード",
    ["both"] = "両方",
    ["cache_settings"] = "キャッシュ設定",
    ["captive_portal_url_description"] = "あなたのクライアントがキャプティブポータルの認証に成功した場合にはこのURLにリダイレクトされます",
    ["captive_portal_url_title"] = "リダイレクトURL",
    ["client_x509_auth"] = "HTTPSクライアント認証",
    ["client_x509_auth_descr"] = "X.509証明書に基づいてクライアントの認証を切り替えます。クライアントX.509共通名（CN）がntopngユーザーのいずれかと等しい場合、クライアントはntopngユーザーとして認証されます。この設定を変更するには、ntopngを再起動する必要があります。",
    ["client_x509_auth_title"] = "クライアントX.509証明書認証",
    ["could_not_contact_influxdb"] = "InfluxDBデータベースにコンタクトできませんでした: %{msg}",
    ["data_retention"] = "データの保持",
    ["data_retention_descr"] = "ダンプされたフロー、過去の時系列データ、上位トーカーを保持する日数。デフォルト:30日。",
    ["default_snmp_community_description"] = "SNMPデバイスページで設定がなされていない場合、選択されたローカルホストのSNMP MIBをサーチするときにはデフォルトSNMPコミュニティが利用されます",
    ["default_snmp_community_title"] = "デフォルトSNMPコミュニティ",
    ["default_snmp_proto_version_description"] = "デフォルトSNMPプロトコルバージョン.",
    ["default_snmp_proto_version_title"] = "デフォルトSNMPバージョン",
    ["default_snmp_timeout_description"] = "SNMP エージェントレスポンスタイムアウト (秒).",
    ["default_snmp_timeout_title"] = "SNMP エージェントレスポンスタイムアウト",
    ["device_identifier"] = "デバイス識別子",
    ["device_identifier_descr"] = "通常、デバイスはMACアドレスにより識別されますが、デバイスがルータの背後にある場合には代わりにIPアドレスによる識別が望ましいです",
    ["disable_alerts_generation_description"] = "アラートの全体的な生成を切り替えます。",
    ["disable_alerts_generation_title"] = "アラートを有効にする",
    ["dns"] = "DNS",
    ["dns_service"] = "サービス",
    ["dump_settings"] = "フローデータベースダンプ",
    ["dynamic_interfaces_creation_description"] = "ntopngは受信トラフィックを分解するためにクライテリアを利用可能です。 分解のためのクライテリアが選択された場合、ntopngはクライテリアの値を受信トラフィックの分解のために利用される動的に作成されたインターフェイスに利用します。例えば、'VLANID'クライテリアが選択されると観測されたVLANIDごとにインターフェイスが生成され、VLANIDとして入力された値により一つの動的インターフェイスにルートされます",
    ["dynamic_interfaces_creation_note_0"] = "'物理インターフェイスだけでなくZMQ経由で受信したパケットについてもVLAN ID'による分解がサポートされています。 他の分解条件はZMQ フローには適用可能ですが物理インターフェイスに対しては無効です",
    ["dynamic_interfaces_creation_note_1"] = "クライテリアの変更は既存のインターフェイスには影響しません。 <b>変更を有効にするにはntopngの再起動が必要です</b>.",
    ["dynamic_interfaces_creation_note_2"] = "非sflowデバイスに'Ingressインターフェイス'を利用する場合、 %%INPUT_SNMP がnprobeテンプレートに必要です",
    ["dynamic_interfaces_creation_note_3"] = "'インターフェイス'を分解するクライテリアは２つの動的インターフェイスを追加します. %%INPUT_SNMPは最初の動的インターフェイスを選択するのに利用され、 %%OUTPUT_SNMPの値は2つ目のインターフェイスを選択するのに利用されます",
    ["dynamic_interfaces_creation_note_4"] = "BPFのようなフィルターによるカスタムトラフィック分解は、カスタムトラフィック分解タブで動的トラフィック分解を無効にする（「なし」に設定する）ことで使用できます。",
    ["dynamic_interfaces_creation_title"] = "分解クライテリア",
    ["dynamic_network_interfaces"] = "Dynamic ネットワーク インターフェイス",
    ["elasticsearch_notification"] = "Elasticsearch 通知",
    ["elasticsearch_send_error"] = "Elasticsearch に接続できません. [レスポンスコード: %{code}][Response: %{resp}].",
    ["elasticsearch_sent_successfully"] = "Elasticsearch に接続しています.",
    ["elasticsearch_unable_to_send_alerts"] = "アラートを Elasticsearch に送信できません",
    ["elasticsearch_unsupported_version"] = "サポートしていない Elasticsearch バージョン %{version}. 少なくともバージョン 7 が必要です.",
    ["elephant_flow_local_to_remote_bytes_description"] = "データはエレファントフローとみなされる前にアップロード可能です",
    ["elephant_flow_local_to_remote_bytes_title"] = "エレファントフローしきい値(ローカルからリモートへ)",
    ["elephant_flow_remote_to_local_bytes_description"] = "データはエレファントフローとみなされる前にダウンロード可能です",
    ["elephant_flow_remote_to_local_bytes_title"] = "エレファントフローしきい値(リモートからローカルへ)",
    ["email_notification"] = "Email通知",
    ["email_notification_password_description"] = "認証用のSMTPサーバーのパスワード（オプション）。",
    ["email_notification_password_title"] = "パスワード",
    ["email_notification_recipient_description"] = "受信者Emailアドレス.",
    ["email_notification_recipient_title"] = "受信者",
    ["email_notification_sender_description"] = "送信者Emailアドレス.",
    ["email_notification_sender_title"] = "送信者",
    ["email_notification_server_description"] = "Email配送のためのSMTP サーバー",
    ["email_notification_server_title"] = "SMTPサーバー",
    ["email_notification_username_description"] = "認証用のSMTPサーバーのユーザー名（オプション）。",
    ["email_notification_username_title"] = "ユーザー名",
    ["email_send_error"] = "Email送信エラー。 詳細は%{product}のログをご確認ください",
    ["email_sent_successfully"] = "Email送信成功",
    ["errors"] = "エラー",
    ["errors_and_warnings"] = "エラーと警告",
    ["ewma_alpha_percent_description"] = "指数加重移動平均（EWMA）アルファ係数はパーセンテージとして表示されます。 ASの往復時間予想として利用されます",
    ["ewma_alpha_percent_title"] = "EWMAアルファ係数 (パーセント)",
    ["expert_view"] = "エキスパートビュー",
    ["featured_dns_description"] = "安全なDNSサーバーのリスト。グローバルDNSサーバーかチャイルドセーフプロバイダーとして利用可能です",
    ["featured_dns_title"] = "注目のDNSサーバー",
    ["flow_interface"] = "インターフェイス",
    ["flow_max_idle_description"] = "フローがアイドルとして判定されるまでの非活動時間。 デフォルト: 1分",
    ["flow_max_idle_title"] = "フローアイドルタイムアウト",
    ["flows_dump"] = "フローダンプ",
    ["flows_dump_limited_days"] = "このntopngバージョンでは、フローダンプは %{days}日に制限されています。この制限を削除するには、エンタープライズバージョンにアップグレードしてください。",
    ["global_dns_description"] = "クライアントのDNSリクエストを非チャイルドセーフホストプールにリダイレクトするDNS サーバー。もし空の場合、リダイレクションは無効化されデフォルトDNSが利用されます",
    ["global_dns_title"] = "グローバルプライマリーDNS",
    ["gui"] = "ユーザーインターフェース",
    ["host_mask"] = "マスクホストIPアドレス",
    ["http_auth"] = "HTTP認証",
    ["http_auth_server_description"] = "ユーザー認証のためのHTTPサーバーのURL",
    ["http_auth_server_title"] = "HTTPサーバー",
    ["idle_timeout_settings"] = "アイドルタイムアウト設定",
    ["ignored_interfaces_description"] = "'Ingressインターフェイス'が分解基準として選択された場合、 %%INPUT_SNMPを持つ動的インターフェイスは追加しないでください。'インターフェイス' 分解が選択された場合、%%INPUT_SNMP (%%OUTPUT_SNMP)をもつ動的インターフェイスは追加しないでください。1つ以上のカンマ区切りのSNMPインターフェイス idsを指定してください。変更は%{product}の再起動を伴います。",
    ["ignored_interfaces_title"] = "無視されたインターフェイス",
    ["incompatible_influxdb_version_found"] = "互換性のないInfluxDBバージョン:必要な \" %{required} \"が見つかりました \"%{found} \" <a href=' %{url}'>公式サイト</a>から更新されたバージョンをインストールしてください。 ",
    ["influxdb_auth_description"] = "InfluxDB認証を有効化",
    ["influxdb_auth_title"] = "InfluxDB認証",
    ["influxdb_create_error"] = "\"%{db}\"のデータベース作成不可能: %{msg}",
    ["influxdb_dbname_description"] = "時系列データ保存のためのデータベース. 既存データは移行されません",
    ["influxdb_dbname_title"] = "InfluxDBデータベース",
    ["influxdb_password_description"] = "InfluxDB データベース認証のためのパスワード",
    ["influxdb_query_timeout_description"] = "InfluxDBクエリの最大期間（秒）。この期間を超えるクエリは中止されます。",
    ["influxdb_query_timeout_title"] = "クエリタイムアウト",
    ["influxdb_retention_error"] = "データベース \"%{db}\"に保有ポリシーを設定できませんでした: %{msg}",
    ["influxdb_storage_note"] = "<b>注</ b>：データは %{interval}の間隔で削除されます。これは、InfluxDB <a href='%{url}'>シャード期間</a>に対応します。 ",
    ["influxdb_url_description"] = "運転中のInfluxDBインスタンスを指し示すURL",
    ["influxdb_url_title"] = "InfluxDBURL",
    ["influxdb_username_description"] = "InfluxDBデータベース認証のためのユーザー名",
    ["ingress_flow_interface"] = "Ingress インターフェイス",
    ["ingress_vrf_id"] = "Ingress VRF ID",
    ["interfaces_timeseries"] = "インターフェイス時系列データ",
    ["internal_log"] = "内部ログ",
    ["is_influxdb_running"] = "InfluxDBは実行されていますか？",
    ["l2_devices_timeseries"] = "デバイス時系列データ",
    ["ldap_authentication"] = "LDAP認証",
    ["ldap_server_address_description"] = "LDAP サーバーのIPアドレスとポート(例. ldaps://localhost:636). デフォルト: \"ldap://localhost:389\".",
    ["ldap_server_address_title"] = "LDAPサーバー アドレス",
    ["license"] = "ライセンス",
    ["local_auth"] = "ローカル認証",
    ["local_host_cache_duration_description"] = "cached ローカル ホストがcacheから削除されるまでの時間. デフォルト: 1 時.",
    ["local_host_cache_duration_title"] = "ローカルホストキャッシュ期間",
    ["local_host_mask"] = "マスクローカルホスト",
    ["local_host_max_idle_description"] = "ローカル ホストがアイドルと判断されるまでの時間。 アイドルローカルホストはそのカウンタ値がキャッシュにダンプされ、再度アクティブになったときに利用されます。カウンターはレイヤ7アプリケーションごとのパケットとバイトトータルを含みます。デフォルト: 5分.",
    ["local_host_max_idle_title"] = "ローカルホストアイドルタイムアウト",
    ["local_hosts_cache_settings"] = "ローカルホストキャッシュ設定",
    ["local_hosts_timeseries"] = "ローカルホスト時系列データ",
    ["logging"] = "ロギング",
    ["longlived_flow_duration_description"] = "あるフローが持続期間が長いフローと判断されるまでの時間(分)",
    ["longlived_flow_duration_title"] = "持続期間が長いフロー期間",
    ["max_num_alerts_per_entity_description"] = "アラーム可能なエンティティごとのアラートの最大数。アラーム可能なエンティティは、ホスト、ネットワーク、インターフェイスです。アラートは定期的にチェックされ、実際のアラート数が特定のエンティティの最大数を超えた場合、構成された最大数の80%%に等しいアラート数を保持するために最も古いアラートが削除されます。デフォルト：1024。",
    ["max_num_alerts_per_entity_title"] = "エンティティごとのアラートの最大数",
    ["max_num_bytes_per_tiny_flow_description"] = "フローが小さなフローと判断される最大バイト数。デフォルト: 64.",
    ["max_num_bytes_per_tiny_flow_title"] = "小さなフローごとの最大バイト数",
    ["max_num_flow_alerts_description"] = "フローアラートの最大数。アラートは定期的にチェックされ、フローアラートの実際の数が設定された最大数を超えた場合、設定された最大数の80%%に等しいアラート数を保持するために最も古いアラートが削除されます。デフォルト：16384",
    ["max_num_flow_alerts_title"] = "フローアラートの最大数",
    ["max_num_packets_per_tiny_flow_description"] = "フローが小さなフローと判断されるための最大パケット数。デフォルト: 3.",
    ["max_num_packets_per_tiny_flow_title"] = "小さなフローの最大パケット数",
    ["max_ui_strlen_description"] = "指定された文字数よりも長い文字列を短縮する",
    ["max_ui_strlen_title"] = "最大表示文字列",
    ["mgmt_acl_description"] = "カンマ区切りのIPv4ネットワークリスト。各ネットワークは明示的に許可か不許可として指定する必要があります。 \"+\"(許可)あるいは \"-\"(不許可)。 重複するネットワークは ロンゲストマッチが適用されます。例: \"+127.0.0.0/8\" はインターフェイスがローカル ホストからのみアクセス許可。, \"+0.0.0.0/0,-192.168.0.0/16\", は192.168.0.0/16以外からのすべてのアクセスを許可。 %{product}を再起動することにより反映されます",
    ["mgmt_acl_title"] = "アクセス制御リスト",
    ["misc"] = "その他",
    ["multiple_ldap_account_type_description"] = "アカウントの種類を選択してください",
    ["multiple_ldap_account_type_title"] = "LDAPアカウントタイプ",
    ["multiple_timeseries_database_description"] = "時系列データを蓄積／取得するためのドライバー.",
    ["multiple_timeseries_database_title"] = "時系列データドライバー",
    ["mysql"] = "MySQL",
    ["n2disk_license_description"] = "連続的なトラフィックレコーディングにはn2diskライセンスが必要です。<a href=\"%{purchase_url}\">e-shop</a>で購入ください。 非営利、研究、教育機関でご利用の場合には <a href=\"%{universities_url}\">こちら</a>をご一読ください",
    ["n2disk_license_description_enterprise_l"] = "ntopng Enterprise L 既に n2disk 1Gをご利用いただけます, この場合追加のライセンスは必要ありません.",
    ["n2disk_license_systemid"] = "システムID: %{systemid}",
    ["n2disk_license_title"] = "n2diskライセンスキー",
    ["n2disk_license_version"] = "n2diskバージョン: %{version}",
    ["n2n_supernode_description"] = "<a class='ntopng-external-link' href=\"%{url}\">n2n <i class=\"fas fa-external-link-alt\"></i></a> スーパーノードのアドレス。 完全なプライベートリモートアクセスを実現するにはあなた自身の専用のスーパーノードを設定してください<br>注意: この変更はリモートアシスタンスの再起動後にのみ反映されます.",
    ["n2n_supernode_title"] = "スーパーノード",
    ["network_discovery"] = "ネットワーク探索",
    ["network_discovery_interval_description"] = "ネットワーク探索間隔 デフォルト: 15分.",
    ["network_discovery_interval_title"] = "アクティブなネットワーク検出間隔",
    ["network_interfaces"] = "ネットワークインターフェイス",
    ["no_host_mask"] = "ホストをマスクしないでください",
    ["non_local_host_max_idle_description"] = "リモート ホストがアイドルと判断されるまでの時間。 デフォルト: 1分.",
    ["non_local_host_max_idle_title"] = "リモートホストアイドルタイムアウト",
    ["none"] = "なし",
    ["note_timeseries_resolution_disabled"] = "注意: \"%{pref}\" がデフォルト値に設定されていないため無効化されています",
    ["note_update_frequency_disabled"] = "<b>注</ b>：これは \" %{pref} \"要件のため無効になっています。",
    ["other_timeseries"] = "その他の時系列データ",
    ["per_category"] = "カテゴリごと",
    ["per_protocol"] = "プロトコルごと",
    ["policy_target_type"] = "ポリシーターゲットタイプ",
    ["policy_target_type_description"] = "どの種類のポリシーターゲットを適用するか指定してください",
    ["posix"] = "Posix",
    ["preferences"] = "環境設定",
    ["primary_dns"] = "プライマリーDNS",
    ["probe_ip_address"] = "プローブIP",
    ["probe_ip_and_ingress_iface_idx"] = "プローブIP + Ingress インターフェイス",
    ["protocols"] = "プロトコル",
    ["radius_admin_group_description"] = "RADIUSにより返却されたフィルター-ID属性値ペアがこのフィールドの一つと合致した場合にユーザーはadminとして認証されます。 合致がない場合、ユーザーは通常ユーザーとして認証されます",
    ["radius_admin_group_title"] = "RADIUS管理者フィルターID",
    ["radius_auth"] = "RADIUS認証",
    ["radius_secret_descroption"] = "RADIUSサーバーとの共通パスワード",
    ["radius_secret_title"] = "RADIUSシークレット",
    ["radius_server_description"] = "RADIUSサーバーのホスト名とポート (例. %{example}).",
    ["radius_server_title"] = "RADIUSサーバーアドレス",
    ["recording"] = "トラフィック記録",
    ["remote_host_mask"] = "マスクリモートホスト",
    ["report"] = "レポート",
    ["restart_needed"] = "変更を適用するには、 %{product}を再起動してください",
    ["runtime_prefs"] = "ランタイム設定",
    ["safe_search_dns_description"] = "<a href='%{url}'>Child Safe</a>を実行するDNSサーバー.<br>チャイルドセーフホストプール設定ページから個別のホスト単位で設定できます。デフォルト: 208.67.222.123.",
    ["safe_search_dns_title"] = "チャイルドセーフDNS",
    ["samaccount"] = "sAMアカウント",
    ["search_path_description"] = "検索に利用されるルートパス .",
    ["search_path_title"] = "LDAP検索パス",
    ["search_preferences"] = "検索設定",
    ["secondary_dns"] = "セカンダリDNS",
    ["secondary_dns_description"] = "グローバルプライマリーDNSサーバーが利用不可能時に利用するフォールバックDNS サーバー",
    ["secondary_dns_title"] = "グローバルセカンダリDNS",
    ["security_alerts"] = "セキュリティ警告",
    ["send_test_elasticsearch"] = "テスト接続",
    ["send_test_mail"] = "テストメールを送信",
    ["send_test_slack"] = "テストメッセージを送信",
    ["send_test_webhook"] = "テストメッセージを送信",
    ["sender_username_description"] = "スラック通知の送信者名",
    ["sender_username_title"] = "通知送信者のユーザー名",
    ["simple_view"] = "シンプルビュー",
    ["slack_channel"] = "Slackチャンネル",
    ["slack_channel_names"] = "チャンネル名",
    ["slack_channel_names_descr"] = "各アラートが送信されるSlackチャンネルの設定。 正しくアラート表示するためにチャンネルは実在する必要があります",
    ["slack_integration"] = "Slack統合",
    ["slack_notification_severity_preference_description"] = "エラー (エラーのみ), エラー と警告 (エラーと警告、情報は無し), すべての (すべてのアラート)",
    ["slack_notification_severity_preference_title"] = "通知の重大度",
    ["slack_send_error"] = "Slackメッセージ送信エラー",
    ["slack_sent_successfully"] = "Slackメッセージが #%{channel} チャンネルに送信されました",
    ["slack_webhook_description"] = "このslack URLにあなたの通知を送ります",
    ["slack_webhook_title"] = "Webhook通知",
    ["snmp"] = "SNMP",
    ["snmp_alerts"] = "SNMPアラート",
    ["snmp_port_load_threshold_description"] = "ポートの負荷が指定された制限（平均5分間）を超えたときにアラートをトリガーするしきい値（ポート速度の割合として）を設定します。",
    ["snmp_port_load_threshold_title"] = "インターフェイスへの負荷しきい値アラート",
    ["snmp_timeseries_config_link"] = "SNMP時系列データは、<a href=\"%{url}\"> SNMPセクション</a>から有効にできます。",
    ["stats_reset"] = "統計をリセット",
    ["status_alerts"] = "ステータスアラート",
    ["successfully_connected_influxdb"] = "InfluxDB(%{version})上で\"%{db}\" を初期化しました",
    ["syslog_alert_format_description"] = "syslogアラートの出力フォーマットを選択して下さい",
    ["syslog_alert_format_json"] = "JSON",
    ["syslog_alert_format_plaintext"] = "プレーンテキスト",
    ["syslog_alert_format_title"] = "syslogアラートの形式",
    ["syslog_notification"] = "syslog通知",
    ["syslog_sent_successfully"] = "メッセージがログされました",
    ["system_probes_timeseries"] = "時系列データのプローブ",
    ["telemetry"] = "テレメトリ",
    ["telemetry_contribute"] = "貢献する",
    ["telemetry_do_not_contribute"] = "貢献しない",
    ["telemetry_email_description"] = "テレメトリデータの異常が検出された場合にntopチームから連絡を取りたい場合は、電子メールアドレスを入力します。このアドレスを指定しないと、ntopはユーザーを識別できません。",
    ["telemetry_email_title"] = "連絡先",
    ["timeseries"] = "時系列データ",
    ["timeseries_database"] = "時系列データベース",
    ["timeseries_resolution_resolution_description_2"] = "ローカルホストとインターフェイスL7アプリケーション間の分析時系列データにおけるデータポイント間隔<br>注意: 大きなネットワークで詳細な分析を利用するとストレージやメモリに影響を与える可能性があります",
    ["timeseries_resolution_resolution_title"] = "L7アプリケーション分析",
    ["toggle_access_log_description"] = "データダンプディレクトリにHTTPアクセスログの生成をオン・オフする。この設定は次回の %{product} 開始時に反映されます",
    ["toggle_access_log_title"] = "HTTPアクセスログを有効にする",
    ["toggle_active_local_host_cache_enabled_description"] = "アクティブなローカルホストのデータキャッシュをオン・オフする。周期的にアクティブなローカルホストをキャッシュするとホストカウンターの耐障害性(例., 電源断)が高まります。 これはほとんどアイドルにならないローカルホストに対して有効です",
    ["toggle_active_local_host_cache_enabled_title"] = "アクティブなローカルホストキャッシュ",
    ["toggle_alert_syslog_description"] = "syslogシステムへのアラートロギングをオン・オフする.",
    ["toggle_alert_syslog_title"] = "Syslog通知の切り替え",
    ["toggle_alerts_notifications_description"] = "サードパーティのエンドポイントへのアラートレポートに切り替えます。",
    ["toggle_alerts_notifications_title"] = "サードパーティのエンドポイントへのアラート",
    ["toggle_asn_rrds_description"] = "ASに関するアプリケーション 時系列データとバイトの生成をオン・オフする",
    ["toggle_asn_rrds_title"] = "自律システム",
    ["toggle_autologout_description"] = "ユーザーのwebインターフェイスからの自動ログオフをオン・オフする",
    ["toggle_autologout_title"] = "自動ログアウト",
    ["toggle_autoupdates_description"] = "夜間の自動ntopng更新を切り替えます。更新が正常に完了すると、ntopngが自動的に再起動されることに注意してください。",
    ["toggle_autoupdates_title"] = "自動更新",
    ["toggle_behaviour_analysis_description"] = "ホスト及びインターフェイス振る舞いトラフィック分析を有効にします. 設定を有効にするには再起動が必要です.",
    ["toggle_behaviour_analysis_title"] = "トラフィック振る舞い分析を有効にする",
    ["toggle_captive_portal_description"] = "ネットワークユーザーを認証する際のwebキャプティブポータルをオン・オフする",
    ["toggle_captive_portal_title"] = "キャプティブポータル",
    ["toggle_country_rrds_description"] = "国のバイト時系列データの作成を切り替えます.",
    ["toggle_country_rrds_title"] = "国",
    ["toggle_dst_with_post_nat_dst_description"] = "IPv4宛先アドレス (%%IPV4_DST_ADDR)とポートを(%%L4_DST_PORT)ポストNAT値(%%POST_NAT_DST_IPV4_ADDR and %%POST_NAPT_DST_TRANSPORT_PORT)に置換する",
    ["toggle_dst_with_post_nat_dst_title"] = "ポストNAT宛先IPv4 アドレスとポートを利用する",
    ["toggle_elasticsearch_notification_description"] = "アラート通知を Elasticsearch に切り替えます.",
    ["toggle_elasticsearch_notification_title"] = "Elasticsearch 通知の切り替え",
    ["toggle_elephant_flows_alerts_description"] = "エレファントフローが検出されたときに生成されるアラートを切り替えます。これは、望まない動作（データの流出など）を検出する際に役立ちます。",
    ["toggle_elephant_flows_alerts_title"] = "エレファントフローアラート",
    ["toggle_email_notification_description"] = "Emailでのアラート通知をオン・オフする",
    ["toggle_email_notification_title"] = "メール通知の切り替え",
    ["toggle_enable_runtime_flows_dump_description"] = "設定データベースにフローのダンプを切り替え.",
    ["toggle_enable_runtime_flows_dump_title"] = "フローダンプ",
    ["toggle_flow_rrds_description"] = "ZMQ経由で受信したリモートデバイスのバイト時系列データの生成をオン・オフします(例. sFlow/NetFlow/SNMP)<br>sFlowでないデバイスについては %%INPUT_SNMP と %%OUTPUT_SNMP nprobeテンプレート上で表示されます",
    ["toggle_flow_rrds_title"] = "フローデバイス",
    ["toggle_host_mask_description"] = "プライバシーの理由からホストIPアドレスのマスクが必要かもしれません。例えばあなたがISPであればどのローカル アドレスがリモートホストにアクセスしているか知ることは想定されていません",
    ["toggle_host_mask_title"] = "MaskホストIPアドレス",
    ["toggle_host_pools_log_description"] = "ホストプールに関連したログの生成をオン・オフします。ホストとホストプールの関係性に関連するログを含みます",
    ["toggle_host_pools_log_title"] = "有効化ホストプールイベントログ",
    ["toggle_host_tskey_description"] = "ローカルブロードキャストドメイン内のホストのデータおよび時系列データダンプに使用するキー。 DHCPネットワークでは、通常「MACアドレス」",
    ["toggle_host_tskey_title"] = "ローカルブロードキャストドメインホスト識別子",
    ["toggle_http_auth"] = "HTTP認証切り替え",
    ["toggle_http_auth_descr"] = "HTTPオーセンティケーターによるHTTP 認証をオン・オフします",
    ["toggle_ids_alert_description"] = "侵入検知システムによって生成されたアラートを切り替えます。 syslogを介したSuricata統合を参照してください",
    ["toggle_ids_alert_title"] = "IDSアラート",
    ["toggle_informative_captive_portal_description"] = "ユーザーを認証しません。ユーザーにインターネットアクセスさせる前に有益なページのみ表示します",
    ["toggle_informative_captive_portal_title"] = "有益な",
    ["toggle_internals_rrds_description"] = "<a href=\"%{url}\">内部</a> 時系列データの作成を切り替えます。 たとえば、ハッシュテーブルエントリ、定期的なアクティビティの期間と書き込まれた時系列ポイントの数、プラグインの期間と呼び出しの数。",
    ["toggle_internals_rrds_title"] = "内部",
    ["toggle_ldap_anonymous_bind_description"] = "匿名バインディング有効化",
    ["toggle_ldap_anonymous_bind_title"] = "LDAP匿名バインディング",
    ["toggle_ldap_auth"] = "LDAP認証のオン・オフ",
    ["toggle_ldap_auth_descr"] = "LDAPを用いたGUI認証をオン・オフします",
    ["toggle_ldap_referrals_description"] = "ユーザーがLDAPサーバーにより返却された参照に自動的に従うかを指定します",
    ["toggle_ldap_referrals_title"] = "参照に従う",
    ["toggle_local_auth"] = "ローカル認証のオン・オフ",
    ["toggle_local_auth_descr"] = "%{product} 定義のユーザーによるGUI認証をオン・オフします.",
    ["toggle_local_host_cache_enabled_description"] = "アイドルなローカルホストのキャッシュエントリの生成をオン・オフします。キャッシュされたローカルホストカウンターは追加のホストトラフィック検出時に自動的にリストアされます",
    ["toggle_local_host_cache_enabled_title"] = "アイドルローカルホストキャッシュ",
    ["toggle_local_hosts_one_way_ts_description"] = "一方向トラフィックの時系列作成を切り替えます. 一方向トラフィックはプローブを実行するリモートホストでしばしば生成されます、これを無効にしておくことを推奨します.",
    ["toggle_local_hosts_one_way_ts_title"] = "一方向トラフィック時系列",
    ["toggle_local_hosts_ts_creation_description"] = "完全なホスト時系列データ作成を有効にする（フル）、バイトとスコアに制限する（ライト）、またはオフにします。",
    ["toggle_local_hosts_ts_creation_title"] = "ホスト時系列データ",
    ["toggle_log_to_file_description"] = "syslogのかわりにntopng.logの生成を行う機能をオン・オフします。この設定は%{product}再起動時に反映されます",
    ["toggle_log_to_file_title"] = "トレースログのオン・オフ",
    ["toggle_logging_level_description"] = "ランタイムロギングレベルの選択",
    ["toggle_logging_level_title"] = "ログレベル",
    ["toggle_longlived_flows_alerts_description"] = "長期間のフローが検出されたときに生成されるアラートを切り替えます。これは、望まない動作（データの流出など）を検出するのに役立ちます。",
    ["toggle_longlived_flows_alerts_title"] = "長期フローアラート",
    ["toggle_midnight_stats_reset_description"] = "真夜中に自動統計リセットを切り替えます。これには、ホストおよびMACアドレスの統計が含まれます。",
    ["toggle_midnight_stats_reset_title"] = "真夜中の統計をリセット",
    ["toggle_mysql_check_open_files_limit_description"] = "MySQL open_files_limitの周期チェックによるMySQL アラートの生成を有効化します。<br>open_files_limitはデータベース故障などによりMySQLのopen file数が多いときの検出に役立ちます",
    ["toggle_mysql_check_open_files_limit_title"] = "MySQL アラート有効化",
    ["toggle_ndpi_flows_rrds_description"] = "レイヤー7アプリケーションごとの合計フロー数の時系列データの作成を切り替えます。",
    ["toggle_ndpi_flows_rrds_title"] = "レイヤー7合計フロー",
    ["toggle_ndpi_timeseries_creation_description"] = "Layer-7 アプリケーション 時系列データの生成をオン・オフします。プロトコル毎の時系列データはより多くのディスク利用や追加のI/Oを要求します。原則的にこの設定は不要です",
    ["toggle_ndpi_timeseries_creation_title"] = "Layer-7アプリケーション",
    ["toggle_network_discovery_description"] = "ARPスキャンやMDNS,SSDPを利用したネットワークデバイスの周期探索をオン・オフします。<p><b>注意:</b> 探索 <u>のみ</u> はネットワークにデータ送受信可能な物理インターフェイスに限られます(例. ZMQではない)。(例. ミラーポートのような受信専用のインターフェイスには働きません) ",
    ["toggle_network_discovery_title"] = "アクティブなネットワーク検出",
    ["toggle_pools_rrds_description"] = "指定したホストプールに対するアプリケーションプロトコル時系列データの生成をオン・オフします",
    ["toggle_pools_rrds_title"] = "ホストプール",
    ["toggle_radius_auth"] = "RADIUS認証オン・オフ",
    ["toggle_radius_auth_descr"] = "RADIUSサーバーを利用してのGUI認証をオン・オフします",
    ["toggle_send_telemetry_data_description"] = "暗号化された<a href=\"%{url}\">匿名のテレメトリデータ</a>を<a href=\"%{ntop_org}\"> ntop.org </a>に送信して、プロジェクトに貢献します。データには、%{product}のステータスのみが含まれます（稼働時間、ステータス、クラッシュレポートなど）。監視対象のトラフィックに関する情報は一切送信されません。」",
    ["toggle_send_telemetry_data_title"] = "テレメトリデータ",
    ["toggle_shaping_directions_description"] = "このオプションを有効化することによりingressとegressに対して異なるシェーピングポリシーを適用できます",
    ["toggle_shaping_directions_title"] = "分割成形方向",
    ["toggle_slack_notification_description"] = "slackを利用したアラート通知をオン・オフします。 設定方法は<a href='%{url}'>こちら</a>を参照して下さい",
    ["toggle_slack_notification_title"] = "<a href=\"%{url}\">Slack</a>通知を切り替えます",
    ["toggle_snmp_alerts_port_duplexstatus_change_description"] = "インターフェイスのデュプレックスステータスが変更されたときに生成されるアラートを切り替えます",
    ["toggle_snmp_alerts_port_duplexstatus_change_title"] = "インターフェイスデュプレックスステータス変更アラート",
    ["toggle_snmp_alerts_port_errors_description"] = "インターフェイスの削除またはエラーカウンターが増加したときに生成されるアラートを切り替えます",
    ["toggle_snmp_alerts_port_errors_title"] = "インターフェイスの削除/エラーアラート",
    ["toggle_snmp_alerts_port_status_change_description"] = "インターフェイスの動作状態が変化したときに生成されるアラートを切り替えます",
    ["toggle_snmp_alerts_port_status_change_title"] = "インターフェイスの動作状態変更アラート",
    ["toggle_snmp_debug_description"] = "SNMPデバイスのデバッグを切り替えます。これを有効にすると、追加情報がntopngログに出力されます。",
    ["toggle_snmp_debug_title"] = "SNMPデバッグ",
    ["toggle_snmp_port_admin_status_description"] = "インターフェイス管理ステータスを切り替えます. 切り替えは迅速に行われますが実際のステータスは次のポーリングでリフレッシュされます.",
    ["toggle_snmp_port_admin_status_title"] = "インターフェイス管理ステータス",
    ["toggle_snmp_rrds_description"] = "SNMPデバイスの各ポートに対してバイト時系列データの生成をオン・オフします。 各デバイスポートに対してingress/egress バイトとともにRRDが生成されます",
    ["toggle_snmp_rrds_title"] = "SNMPデバイス時系列データ",
    ["toggle_src_and_dst_using_ports_description"] = "ポート番号を使用して、受信したフローの送信元と宛先を決定します。小さい方のポート番号が宛先ポートと見なされます。より大きいポート番号が送信元ポートであると想定されます。理論的には、一般に、ソースは一時ポートを使用して既知の宛先ポートに接続し、一時ポートは一般に既知の宛先ポートよりも大きいということです。このオプションを有効にすると、宛先ポートが送信元ポートよりも大きいことが判明した場合、送信元と宛先が交換されます。",
    ["toggle_src_and_dst_using_ports_title"] = "ポート番号を使用して送信元と宛先を決定する",
    ["toggle_src_with_post_nat_src_description"] = "IPv4ソースアドレス (%%IPV4_SRC_ADDR) とポート (%%L4_SRC_PORT) がPost-Nat値(%%POST_NAT_SRC_IPV4_ADDR and %%POST_NAPT_SRC_TRANSPORT_PORT)に置き換えられます",
    ["toggle_src_with_post_nat_src_title"] = "Post-NatソースIPv4 アドレスとポートを使用する",
    ["toggle_system_probes_timeseries_description"] = "RTTやRedisモニターなどのシステムプローブの時系列データの作成を切り替えます。",
    ["toggle_system_probes_timeseries_title"] = "プローブ",
    ["toggle_theme_description"] = "テーマを切り替えます。",
    ["toggle_theme_title"] = "テーマ",
    ["toggle_thpt_content_description"] = "トラフィック レポートで表示すべきスループット単位を選択します",
    ["toggle_thpt_content_title"] = "スループット単位",
    ["toggle_tiny_flows_dump_description"] = "数パケットしかない小さいフローのエクスポートのオン・オフをします。データベース上でのデータ紐づけを抑止するためデータ挿入や検索を早くします。ダンプ情報の可視性を保ちながらデータ紐づけについて制限します",
    ["toggle_tiny_flows_dump_title"] = "小さなフローをエクスポート",
    ["toggle_top_sites_description"] = "ローカルホストからの上位訪問Webサイトの生成をオン・オフします。 上位サイトは <a href=\"%{url}\">ヒューリスティック</a>を用いて生成でき、ローカルホストあたり最大で20サイトを保持します。ヒューリスティックは完全にメモリ上で動作しディスク操作は要求しません。上位サイトは5分ごとにリフレッシュされます。上位訪問Webサイトの履歴はローカルホストごとに周期的にJSONデータをホスト 詳細ページ<i class=\"fas fa-download fa-sm\"></i>のダウンロードリンクからポーリングすることで生成可能です",
    ["toggle_top_sites_title"] = "上位HTTPサイト",
    ["toggle_traffic_rrd_creation_description"] = "バイトとパケットの時系列データの生成をオン・オフします",
    ["toggle_traffic_rrd_creation_title"] = "トラフィック",
    ["toggle_users_rrds_description"] = "定義したユーザーに対してバイトおよびアプリケーションプロトコル時系列データの生成のオン・オフします",
    ["toggle_users_rrds_title"] = "ホストプール",
    ["toggle_vlan_rrds_description"] = "VLANに関するバイトおよびアプリケーション時系列データの生成をオン・オフします",
    ["toggle_vlan_rrds_title"] = "VLAN",
    ["toggle_webhook_notification_description"] = "webhook（HTTP）経由のアラート通知を切り替えます。",
    ["toggle_webhook_notification_title"] = "Webhook通知の切り替え",
    ["topk_heuristic_precision_description"] = "トラフィックレポートを作成する際のヒストリカル上位ホスト, 国等を集計するときに履歴を使用します。 長期間のレポートを作成するときに役立ちます",
    ["topk_heuristic_precision_title"] = "Top-Kヒューリスティック",
    ["traffic_bridging"] = "トラフィックブリッジ",
    ["traffic_shaping"] = "トラフィックシェーピング",
    ["ts_resolution_note2"] = "低い解像度は既存データ内のギャップを作成します. 次が推奨されます <a target=_blank href=\"%{url}\">データーベースの削除</a> %{external_icon}.",
    ["updates"] = "更新情報",
    ["user_authentication"] = "ユーザー認証",
    ["user_group_description"] = "一般ユーザーとして認証する際のグループ名",
    ["user_group_title"] = "LDAPユーザーグループ",
    ["vlan"] = "VLANID",
    ["web_user_interface"] = "Webユーザーインターフェイス",
    ["webhook_notification"] = "Webhook通知",
    ["webhook_notification_severity_preference_description"] = "エラー（エラーのみ）、エラーと警告（エラーと警告、情報なし）、すべて（すべての種類のアラートが通知）",
    ["webhook_notification_severity_preference_title"] = "通知の重大度",
    ["webhook_password_description"] = "HTTP認証の場合に使用されるパスワード（オプション）。",
    ["webhook_send_error"] = "webhookへのメッセージ送信エラー。",
    ["webhook_sent_successfully"] = "メッセージがwebhookに送信されました",
    ["webhook_sharedsecret_description"] = "（ユーザー定義の）シークレットを設定して、受信者が検証します（オプション）。",
    ["webhook_sharedsecret_title"] = "秘密の共有",
    ["webhook_url_description"] = "JSON形式でこのURLに通知を送信します。",
    ["webhook_url_title"] = "通知URL",
    ["webhook_username_description"] = "HTTP認証の場合に使用されるユーザー名（オプション）。",
    ["zmq_interfaces"] = "ZMQインターフェイス",
  },
  ["processes_stats"] = {
    ["active_processes_title"] = "アクティブなプロセス: リアルタイムビュー",
    ["active_since"] = "アクティブになってから",
    ["flows_count"] = "フローカウント",
    ["legend"] = "レジェンド",
    ["lines"] = "ライン",
    ["missing_pid_name_message"] = "PID名がありません",
    ["no_traffic_detected"] = "このプロセスについてはトラフィックは検出されないか、プロセスが有効期限切れか、プロセスが終了しています",
    ["processes_timeline_title"] = "プロセスタイムライン",
    ["stack"] = "スタック",
    ["timeline"] = "タイムライン",
    ["top_hosts"] = "上位ホスト",
    ["top_hosts_traffic"] = "上位ホストトラフィック",
    ["traffic_rcvd"] = "トラフィック受信",
    ["traffic_sent"] = "トラフィック送信",
    ["type"] = "タイプ",
  },
  ["queue"] = {
    ["activeFlowsToDump"] = "アクティブフローダンプ",
    ["hookFlowEnd"] = "flowEnd フローユーザースクリプトフック",
    ["hookPeriodicUpdate"] = "periodicUpdate フローユーザースクリプトフック",
    ["hookProtocolDetected"] = "protocolDetected フローユーザースクリプトフック",
    ["idleFlowsToDump"] = "アイドルフローダンプ",
  },
  ["queue_description"] = {
    ["activeFlowsToDump"] = "キューにある待機アクティブフローがダンプされます",
    ["hookFlowEnd"] = "flowEndユーザースクリプトフックが実行される待機アイドルフロー",
    ["hookPeriodicUpdate"] = "periodicUpdateユーザースクリプっとフックされる待機アクティブフロー",
    ["hookProtocolDetected"] = "protocolDetectedユーザースクリプトフックが実行される新しい待機フロー",
    ["idleFlowsToDump"] = "待機アイドルフローキューがダンプされました",
  },
  ["remote_assistance"] = {
    ["accept_and_enable"] = "受諾と有効化",
    ["admin_access"] = "一時的な管理者アクセス",
    ["admin_access_descr"] = "リモートアシスタンスGUIアクセスのための一時的な管理者パスワードを作成します",
    ["admin_access_key_descr"] = "一時的な管理者パスワードは <b>%{pwd}</b>",
    ["ask_admin"] = "これを実行する前にネットワーク管理者にお問い合わせください",
    ["check_out_the_preferences"] = "<a href=\"%{url}\"> <i class =  \"fas fa-flask \"> </ i>リモートアシスタンスの設定</a>から、より多くのリモートアシスタンスパラメータを調整できます。",
    ["connection_script"] = "接続スクリプト",
    ["connection_script_descr"] = "接続スクリプトをダウンロードしサポートチームに提供しこのntopngインスタンスにアクセスさせてください。スクリプトはn2nパッケージがインストールされたシステム上で動作します",
    ["enable_remote_assistance"] = "リモートアシスタンスを有効にする",
    ["gui_access_restricted_info"] = "GUIアクセスは、構成された<a href='%{url_acl}'> ACL </a>またはcliオプション（%{cli_options}）によって制限されます。リモート管理アクセスがブロックされる場合があります。",
    ["product_remote_assistance"] = "%{product} リモートアシスタンス",
    ["remember_disable"] = "必要ないときはリモートアシスタンスを無効化してください",
    ["remote_assistance"] = "リモートアシスタンス",
    ["remote_ip_msg"] = "この %{product} インスタンスはリモートからIPアドレス %{ip}でアクセス可能です",
    ["tos_notice"] = " \"%{button}\" をクリックすることによって、リモートからのアクセスを提供し、ネットワークセキュリティポリシーがそれを許可することになります。 ntopは、それによって生じたいかなる損害、セキュリティ違反、脆弱性についても責任を負いません",
    ["will_create_virtual_network"] = "有効化されると、リモートアシスタンスはあなたのデバイスにアクセスするための暗号化された仮想ネットワークを作成します",
  },
  ["report"] = {
    ["application_breakdown"] = "アプリケーション分解",
    ["applications"] = "アプリケーション",
    ["begin_date_time"] = "開始日/時刻",
    ["date"] = "%{month}-%{day}-%{year}",
    ["end_date_time"] = "終了日/時刻",
    ["filter_report"] = "フィルターレポート",
    ["generate"] = "生成",
    ["invalid_begin"] = "無効な開始",
    ["invalid_begin_date"] = "無効な開始日",
    ["invalid_begin_end"] = "無効な開始/終了",
    ["invalid_to"] = "無効な宛先",
    ["local_networks"] = "ローカルネットワーク",
    ["local_remote"] = "ローカル/リモート",
    ["network_interface"] = "ネットワークインターフェイス",
    ["period"] = "間隔",
    ["please_check_format"] = "フォーマットを確認してください",
    ["please_choose_valid"] = "有効な開始/終了日と時刻を指定してください",
    ["please_choose_valid_date_and_time"] = "有効な日付と時刻を指定してください",
    ["receivers"] = "受信者",
    ["remote_local"] = "リモート/ローカル",
    ["report_for_subject"] = "レポート - %{num_min} - for %{subject} starting %{dt}",
    ["reports_professional_only"] = "レポートはプロフェッショナルバージョンでのみ利用可能です",
    ["senders"] = "送信者",
    ["starting"] = "起動",
    ["submit_filter"] = "送信フィルター",
    ["symbolic_host_name_at_visualization"] = "シンボルホスト名は、グラフ変換中に解決されます。",
    ["toggle_all"] = "すべてのオン・オフ",
    ["top_asn"] = "上位ASN",
    ["top_countries"] = "上位国",
    ["top_local_hosts"] = "上位ローカル ホスト",
    ["top_local_os"] = "上位ローカル OS",
    ["top_networks"] = "上位ネットワーク",
    ["top_non_local_os"] = "上位非ローカルOS",
    ["top_remote_hosts"] = "上位リモート ホスト",
    ["top_talkers"] = "上位トーカー",
    ["total_traffic"] = "総トラフィック",
    ["traffic_report"] = "トラフィックレポート",
    ["use_server_timezone"] = "サーバータイムゾーンを利用",
  },
  ["rest_consts"] = {
    ["ADD_POOL_FAILED"] = "サブミットされたパラメータでプールを追加できません",
    ["BAD_CONTENT"] = "悪いコンテンツ",
    ["BAD_FORMAT"] = "悪いフォーマット",
    ["BIND_POOL_MEMBER_ALREADY_BOUND"] = "他のプールに所属しているメンバーをバインドできません",
    ["BIND_POOL_MEMBER_FAILED"] = "サブミットされたメンバーをプールにバインドできません",
    ["CONFIGURATION_FILE_MISMATCH"] = "アップロードされたファイルは有効な設定ファイルではありません!",
    ["DELETE_POOL_FAILED"] = "サブミットされたパラメータでプールを削除することができません",
    ["EDIT_POOL_FAILED"] = "サブミットされたパラメータでプールを編集することができません",
    ["INTERNAL_ERROR"] = "内部エラー",
    ["INVALID_ARGUMENTS"] = "不正な引数",
    ["INVALID_HOST"] = "不正なホスト",
    ["INVALID_INTERFACE"] = "不正なインターフェイス",
    ["NAME_RESOLUTION_FAILED"] = "ホスト名をIPアドレスに解決できません",
    ["NOT_FOUND"] = "見つかりません",
    ["NOT_GRANTED"] = "権限がありません",
    ["NO_SNMP_DEVICE_DISCOVERED"] = "SNMPデバイスが見つかりません",
    ["OK"] = "成功",
    ["PARTIAL_IMPORT"] = "設定が正しくインポートされましたが、いくつかの要素は既に存在し上書きはされませんでした.",
    ["POOL_ADDED_SUCCESSFULLY"] = "プールの追加が成功しました",
    ["POOL_DELETED_SUCCESSFULLY"] = "プールの削除が成功しました",
    ["POOL_EDITED_SUCCESSFULLY"] = "プールの編集が成功しました",
    ["POOL_MEMBER_BOUND_SUCCESSFULLY"] = "プールメンバーの割り当てが成功しました",
    ["POOL_NOT_FOUND"] = "プ―ルが見つかりません",
    ["SNMP_DEVICE_ADDED_SUCCESSFULLY"] = "SNMPデバイスの追加が成功しました",
    ["SNMP_DEVICE_ALREADY_ADDED"] = "SNMPデバイスは既に追加されています",
    ["SNMP_DEVICE_DELETED_SUCCESSFULLY"] = "SNMPデバイスの削除に成功しました",
    ["SNMP_DEVICE_EDITED_SUCCESSFULLY"] = "SNMPデバイスの編集に成功しました",
    ["SNMP_DEVICE_INTERFACE_STATUS_CHANGE_FAILED"] = "SNMPデバイスインターフェイスの状態変更に失敗しました",
    ["SNMP_DEVICE_UNREACHABLE"] = "SNMPデバイスから応答がありません",
  },
  ["restart"] = {
    ["confirm"] = "%{product}を今すぐ再起動しますか？",
    ["restart"] = "再起動",
    ["restart_now"] = "今すぐ再起動",
    ["restart_product"] = "再起動 %{product}",
    ["restart_required"] = "再起動が必要です",
    ["restarting"] = "%{product}を再起動しています...（これには数秒かかります）",
  },
  ["scripts_list"] = {
    ["all"] = "すべて",
    ["are_you_sure"] = "編集内容はまだ保存されていません！このダイアログを本当に閉じますか？",
    ["attention"] = "注意",
    ["back_scripts_page"] = "スクリプトに戻る",
    ["blacklisted_country"] = "国コード値をコンマで区切って書いてください。 (例: IT,FR,DE,UK)",
    ["config"] = "構成",
    ["filter_dropdown"] = "フィルターカテゴリー",
    ["items_list_comma"] = " ,\",を除く特殊文字は使用できません",
    ["missing_configset"] = "要求された構成セットが見つかりませんでした！",
    ["previous_page"] = "前のページに戻るにはこちらをクリックしてください！",
    ["reset_default"] = "デフォルトにリセット",
    ["script_search"] = "スクリプトの検索:",
    ["scripts_x"] = "スクリプト %{subdir}-%{config}",
    ["select_operator"] = "演算子を選択してください！",
    ["template_not_found"] = "テンプレートが見つかりませんでした！",
    ["actions"] = {
      ["edit"] = "編集",
      ["edit_script"] = "スクリプトを編集",
      ["view"] = "ビュー",
      ["view_src_script"] = "ソーススクリプトを表示",
    },
    ["templates"] = {
      ["blacklisted_country_list"] = "ブラックリストに登録された国リスト",
      ["elephant_flows_l2r"] = "エレファントフローのしきい値（ローカルからリモート）",
      ["elephant_flows_r2l"] = "エレファントフローのしきい値（リモートからローカル）",
      ["excluded_applications"] = "除外されたアプリケーションとカテゴリ",
      ["flow_duration_threshold"] = "フロー継続時間のしきい値",
      ["template_not_implemented"] = "テンプレートはまだ実装されていません！",
    },
  },
  ["severity_groups"] = {
    ["group_error_or_higher"] = "エラーもしくはそれ以上",
    ["group_none"] = "無し",
    ["group_notice_or_lower"] = "通知もしくはそれ以下",
    ["group_warning"] = "警告",
  },
  ["sflows_stats"] = {
    ["active_flows"] = "アクティブなフロー",
    ["client_peer"] = "クライアントピア",
    ["client_process"] = "クライアントプロセス",
    ["server_peer"] = "サーバーピア",
    ["server_process"] = "サーバープロセス",
    ["total_bytes"] = "合計バイト",
  },
  ["shaping"] = {
    ["applied_to"] = "に適用",
    ["bandwidth_manager"] = "帯域幅マネージャー",
    ["confirm_delete_all_policies"] = "本当に\"%{pool_name}\" プールポリシーを削除してよろしいですか",
    ["confirm_delete_policy"] = "本当に削除してよいですか",
    ["confirm_delete_shaper"] = "シェイパーを削除してもよいですか ",
    ["daily_time"] = "毎日の時間(現在の / クォータ)",
    ["daily_time_quota"] = "毎日の時間クォータ",
    ["daily_traffic"] = "毎日のトラフィック (現在の / クォータ)",
    ["daily_traffic_quota"] = "毎日のトラフィッククォータ",
    ["delete_all_policies"] = "すべてのポリシーを削除",
    ["delete_policy"] = "ポリシーを削除",
    ["delete_shaper"] = "シェイパーを削除",
    ["for_dropping_all_traffic"] = "すべてのトラフィックをドロップ",
    ["for_no_shaping"] = "シェーピングなし",
    ["initial_clone_protocols"] = "初期のプロトコルルールは複製されます",
    ["initial_empty_protocols"] = "初期のプロトコルルールは空です",
    ["manage_policies"] = "ポリシーを管理する",
    ["no_quota_applied"] = "クォータなし",
    ["no_quota_data"] = "クォータが設定されていません",
    ["no_quota_traffic"] = "これまでにクォータ関連のトラフィックはありません",
    ["no_shapers_available"] = "利用可能なシェーパーはありません",
    ["note_default_pool_config"] = "ポリシーとクォータはユーザー定義のホスト プールにのみ適用可能です。異なるプールを選択するか新規のプールを作成してください<a href='%{url}'>こちら</a>.",
    ["note_drop_core"] = "一部のコアプロトコルをドロップすると、他のプロトコルに副作用が生じる可能性があります。たとえば、DNSをブロックすると、<br>シンボリックホスト名は解決されなくなり、数値IPとの通信のみが機能します。",
    ["note_quota_unlimited"] = "トラフィックと時間クォータを0に設定するとトラフィックに制限はかかりません",
    ["note_target_type"] = "ターゲット列に何を表示するかは<a href=\"%{url}\">トラフィック Bridging 設定</a> のエキスパートビューのポリシーターゲットタイプフィールドで変更できます",
    ["note_traffic_categories"] = "カテゴリは<a href=\"%{url}\">プロトコルページ</a>から設定できます",
    ["notes"] = "注意",
    ["policy_from_pool"] = "プールからのポリシー",
    ["protocol_families"] = "プロトコルファミリー",
    ["protocol_policy"] = "トラフィックポリシー",
    ["protocols"] = "プロトコル",
    ["protocols_policies"] = "プロトコルポリシー",
    ["see_quotas_here"] = "アクティブホストプールクォータでホスト プール <a href='%{url}'>クォータページ</a>の全概要を表示することができます",
    ["select_to_clone"] = "既存のネットワークから選択し、プロトコルルールを複製します",
    ["set_max_rate_to"] = "最大レートを設定",
    ["shaper0_message"] = "シェイパー0 デフォルトのローカルホストで利用されるシェイパーでありシェイパーは定義されていません",
    ["shaper_id"] = "シェイパーID",
    ["shapers_in_use_message"] = "いかなるネットワークにも適用されていない場合にのみシェイパーは削除可能です",
    ["traffic_from"] = "からのトラフィック",
    ["traffic_to"] = "へのトラフィック",
  },
  ["show_alerts"] = {
    ["1_day"] = "1日",
    ["1_hour"] = "1時間",
    ["1_month"] = "1ヵ月",
    ["1_week"] = "1週間",
    ["1_year"] = "1年",
    ["30_min"] = "30分",
    ["5_min"] = "5分",
    ["6_months"] = "6ヵ月",
    ["alarmable"] = "驚くべき",
    ["alert_actions"] = "アクション",
    ["alert_count"] = "カウント",
    ["alert_counts"] = "カウント",
    ["alert_datetime"] = "日付/時刻",
    ["alert_description"] = "表記",
    ["alert_duration"] = "期間",
    ["alert_severity"] = "重要度",
    ["alert_type"] = "アラートタイプ",
    ["alerts"] = "アラート",
    ["alerts_are_disabled_message"] = "アラートは無効です。設定ページをチェックして有効にしてください。",
    ["alerts_queries"] = "アラートクエリ",
    ["alerts_to_purge"] = "パージするアラート",
    ["confirm_delete_alert"] = "指定したアラートを本当に削除しますか",
    ["confirm_release_alert"] = "指定したアラートを本当にリリースしますか？",
    ["daily"] = "毎日",
    ["delete_alert"] = "アラート削除",
    ["delete_alerts_configuration"] = "設定を削除",
    ["delete_all"] = "すべて削除",
    ["delete_config_btn"] = "%{conf} コモン設定削除",
    ["delete_config_message"] = "%{conf}のコモン %{granularity} アラート設定を本当に削除しますか",
    ["detected_alerts"] = "検出されたアラート",
    ["disable_alerts"] = "無効にする",
    ["disable_alerts_message"] = "%{entity_value}で \" %{type} \"アラートを無効にしますか？",
    ["disable_alerts_title"] = "アラートを無効にする",
    ["disabled_alerts"] = "無効なアラート",
    ["dropped_alerts"] = "ドロップされたアラート",
    ["enable_alerts"] = "有効にする",
    ["enable_alerts_message"] = "パージ %{entity_value}で \" %{type} \"アラートを本当に有効にしますか？",
    ["enable_alerts_title"] = "アラートを有効にする",
    ["engaged"] = "現在",
    ["engaged_alerts"] = "現在のアラート",
    ["explorer"] = "エクスプローラ",
    ["five_minutes"] = "5分",
    ["flow_alerts"] = "フローアラート",
    ["flow_checks_drops_descr"] = "一部の<a href=' %{url}'> フローユーザースクリプト</a>が実行されない場合にアラートをトリガーします",
    ["flow_checks_drops_title"] = "フローユーザースクリプトドロップアラート",
    ["flush_data"] = "データをフラッシュ",
    ["host_delete_config_btn"] = "ホスト %{host} の設定されたアラートを本当に削除しますか",
    ["host_delete_config_confirm"] = "ホストに対する%{granularity} アラート設定を本当に削除しますか",
    ["hourly"] = "毎時",
    ["iface_delete_config_btn"] = "インターフェイス %{iface} 設定削除",
    ["iface_delete_config_confirm"] = "インターフェイスに対する%{granularity} アラート設定を本当に削除しますか",
    ["iface_engaged_dropped_alerts"] = "ビジーなアラート vs ドロップされたアラート",
    ["in_the_selected_time_frame"] = "選択した期間内",
    ["interface_drops_threshold"] = "インターフェイスパケットドロップ",
    ["interface_drops_threshold_descr"] = "全体のインターフェイスで観測されたパケットのうち最大のパケットドロップ率",
    ["last_day"] = "前回の日付",
    ["last_hour"] = "前回の時",
    ["last_minute"] = "前回の分",
    ["minute"] = "分",
    ["network_delete_config_btn"] = "ネットワーク %{network}設定されたアラートを削除する",
    ["network_delete_config_confirm"] = "ネットワークの %{granularity} アラート設定を本当に削除しますか",
    ["no_recorded_alerts_message"] = "インターフェイス %{ifname}のアラートは記録されていません",
    ["num_ignored_alerts"] = "無視されたアラート",
    ["older_1_day_ago"] = " 1日以上前",
    ["older_1_hour_ago"] = "1時間以上前",
    ["older_1_month_ago"] = "1ヵ月以上前",
    ["older_1_week_ago"] = "1週間以上前",
    ["older_1_year_ago"] = "1年以上前",
    ["older_30_minutes_ago"] = "30分以上前",
    ["older_5_minutes_ago"] = "5分以上前",
    ["older_6_months_ago"] = "6月以上前",
    ["older_than"] = "以上前",
    ["past"] = "過去",
    ["past_alerts"] = "過去のアラート",
    ["past_flow"] = "過去フロー",
    ["purge_all_alerts"] = "全アラートをパージする",
    ["purge_num_alerts"] = "[%{num_alerts}アラートを消去]",
    ["purge_subj_alerts"] = "%{subj}アラートを消去",
    ["purge_subj_alerts_confirm"] = "すべての%{subj}アラートを完全に削除しますか？",
    ["release_alert"] = "リリースアラート",
    ["release_alert_action"] = "リリース",
    ["reset_alert_database"] = "アラートデータのフラッシュ",
    ["reset_alert_database_message"] = "このアクションはすべてのアラート関連のデータを削除します。このデータには<ul><li>Runtime アラート状態</li><li>アラート設定</li><li>永続的なアラートSQLiteストレージ</li></ul>を含みます<br>本当によろしいでしょうか",
    ["startup"] = "起動",
    ["stored_alerts"] = "保存されたアラート",
    ["trigger_alert_descr"] = "このアラームのトリガーとなるアラートを有効にする",
    ["trigger_host_alert_descr"] = " %{host}によって生成されたアラートのトリガー",
    ["trigger_iface_alert_descr"] = " %{iface}によって生成されたアラートのトリガー",
    ["trigger_network_alert_descr"] = " %{network}によって生成されたアラートのトリガー",
  },
  ["sites_page"] = {
    ["contacts"] = "コンタクト",
    ["current_sites"] = "現在のサイト",
    ["last_5_minutes_sites"] = "最後5分のサイト",
    ["sites"] = "サイト",
    ["top_sites_not_enabled_message"] = "訪問したサイトのトラックを続けるために、 ローカルホストに対し上位訪問したWebサイトを生成してください<a href=\"%{url}\"><i class=\"fas fa-flask\"></i> 設定</a>.",
    ["top_sites_not_seen"] = "現在のローカルホストはいかなるWebサイトも訪問していません",
    ["top_visited_sites"] = "上位訪問したサイト",
  },
  ["slack_alert_endpoint"] = {
    ["notification_webhook"] = "通知 Webhook",
    ["sender_username"] = "送信者 ユーザー名",
  },
  ["snmp"] = {
    ["AlertDescription"] = "アラートの説明",
    ["ConsoleDisplayBufferText"] = "テキストを表示",
    ["GeneralSerialNumber"] = "シリアル番号",
    ["InputCurrentLevel"] = "現在のレベル",
    ["InputDimUnit"] = "Dim単位",
    ["InputMaxCapacity"] = "最大キャパシティ",
    ["InputMediaDimFeedDirDeclared"] = "入力メディアDimの宣言済フィード",
    ["InputMediaDimXFeedDirDeclared"] = "入力メディアDimXの宣言済フィード",
    ["InputName"] = "インプット名",
    ["MarkerColorantValue"] = "マーカーで着色",
    ["MarkerLifeCount"] = "マーカーライフカウント",
    ["MarkerSuppliesDescription"] = "マーカーサプライの説明",
    ["MarkerSuppliesLevel"] = "マーカーサプライレベル",
    ["MarkerSuppliesMaxCapacity"] = "マーカーサプライ最大キャパシティ",
    ["PrinterCurrentStatus"] = "現在の状態",
    ["PrinterOperationalStatus"] = "運用状態",
    ["absolute_stats"] = "ブート以降の統計",
    ["actual_stats"] = "前回の調査以降の統計",
    ["add_device"] = "新しいデバイスを追加",
    ["add_devices"] = "複数デバイスの追加",
    ["add_snmp_devices"] = "SNMPデバイスを追加",
    ["alert_topology"] = "表示するノードが存在しません!",
    ["alerted_interfaces"] = "エラーのあるインターフェイス",
    ["all_interfaces"] = "すべてのインターフェイス",
    ["all_snmp_interface_types"] = "すべてのタイプ",
    ["all_snmp_statuses"] = "すべての状態",
    ["and_one_more_interface"] = "および<a href='%{url}'>さらに1つのインターフェイス</a>",
    ["authentication_passphrase"] = "認証パスフレーズ",
    ["authentication_protocol"] = "認証プロトコル",
    ["bound_interface_description"] = "ネットワークインターフェイスをSNMPインターフェイスにバインドします。これはSNMPで報告されるトラフィックとntopngで観測されるトラフィックを比較するときに役立ちます",
    ["caching_in_progress"] = "SNMPデバイスの検索が進行中です。これは最大で5分ほどかかります。このページは周期的にリフレッシュします",
    ["contact"] = "コンタクト",
    ["cpuIdle"] = "CPU アイドル",
    ["cpuSystem"] = "CPU システム",
    ["cpuUser"] = "CPU ユーザー",
    ["delete_all_devices"] = "すべてのデバイス削除",
    ["delete_unresponsive_devices"] = "無応答の削除",
    ["device_example"] = "例:%{example}",
    ["device_ip"] = "デバイスIP",
    ["device_model"] = "デバイスモデル",
    ["device_polling"] = "デバイスポーリング",
    ["device_polling_desc"] = "定期的なSNMPウォークを実行して、デバイスインターフェイスのトラフィックやステータスなどのデバイスデータを収集します。",
    ["device_responsiveness"] = "デバイスフィルタ",
    ["device_status"] = "デバイスの状態",
    ["devices_added"] = "追加されたデバイス",
    ["disable_port_monitor"] = "ポートモニター無効化",
    ["discover_devices"] = "デバイスの発見",
    ["edit_device"] = "編集",
    ["edit_snmp_devices"] = "SNMP デバイスの編集",
    ["enable_port_monitor"] = "ポートモニター有効化",
    ["errors_filter"] = "エラーフィルター",
    ["ifName"] = "インターフェイス名",
    ["ifindex"] = "インデックス",
    ["in_bytes"] = "インバイト",
    ["in_discards"] = "削除中",
    ["in_errors"] = "エラーで",
    ["interface_device"] = "インターフェイス%{interface} [%{device} ]",
    ["interfaces_with_discards"] = "破棄インターフェイス",
    ["interfaces_with_errors"] = "エラーのあるインターフェイス",
    ["interfaces_with_errors_or_discards"] = "エラー/破棄インターフェイス",
    ["last_change"] = "前回の変更",
    ["lldpRemChassisId"] = "リモートデバイス",
    ["lldpRemPortDesc"] = "リモートポートの説明",
    ["lldpRemPortId"] = "リモートポート識別子",
    ["lldpRemSysDesc"] = "リモートシステムの説明",
    ["lldpRemSysName"] = "リモートシステム名",
    ["lldp_topology_changed"] = "LLDP トポロジー変更",
    ["lldp_topology_changed_description"] = "SNMP ネットワークトポロジーで変更を検知しました. アラートを発報します",
    ["lldp_topology_changed_title"] = "LLDP トポロジーモニター",
    ["low_port_speed"] = "低ポート速度",
    ["memAvailReal"] = "利用可能なメモリ",
    ["memTotalReal"] = "メモリ合計",
    ["memTotalSwap"] = "スワップ合計",
    ["multiple_macs_non_trunk"] = "非トランクポート上の複数のMAC",
    ["no_answer"] = "ホスト%{host}から応答がありません: SNMPがアップしており正しいコミュニティが設定されていることを確認してください",
    ["no_host_discovered"] = "%{host}/%{cidr}にホストは見つかりませんでした",
    ["no_polling_yet"] = "ポーリングが一度もされていません",
    ["none"] = "無しです",
    ["note"] = "注意",
    ["object_id"] = "オブジェクトID",
    ["out_bytes"] = "アウトバイト",
    ["per_interface_settings"] = "これらの設定は、インターフェイス \"%{iface} \"にのみ影響します。グローバル設定については、<a href=\"%{url}\">%{global} </a>をご覧ください。",
    ["port_macs"] = "MACアドレス",
    ["port_macs_short"] = "MAC",
    ["privacy_passphrase"] = "プライバシーパスフレーズ",
    ["privacy_protocol"] = "プライバシープロトコル",
    ["prune_unresponsive_snmp_devices"] = "応答のないSNMPデバイスを削除",
    ["prune_unresponsive_snmp_devices_confirm"] = "本当に応答のないSNMPデバイスを全て削除しますか?",
    ["remove_all_snmp_devices"] = "すべてのSNMPデバイスを削除",
    ["remove_all_snmp_devices_confirm"] = "すべてのSNMPデバイスを本当に削除しますか?",
    ["remove_snmp_device"] = "SNMPデバイスを削除する",
    ["remove_snmp_device_confirm"] = "SNMPデバイス%{device}を本当に削除しますか?",
    ["responsiveness_all"] = "全てのデバイス",
    ["responsiveness_responsive"] = "応答のあるデバイス",
    ["responsiveness_unresponsive"] = "応答がないデバイス",
    ["select_pool"] = "プールを選択",
    ["snmp_all_interfaces"] = "すべてのインターフェイス",
    ["snmp_also_seen_on"] = "再度見る",
    ["snmp_and_n_more_interfaces"] = "および<a href='%{url}'>%{count}個のインターフェイス</a>",
    ["snmp_authnopriv"] = "プライバシー無しの認証",
    ["snmp_authpriv"] = "プライバシー有りの認証",
    ["snmp_community"] = "SNMPコミュニティ",
    ["snmp_device"] = "SNMPデバイス",
    ["snmp_device_does_not_respond"] = "SNMPデバイスは応答しません",
    ["snmp_device_last_poll"] = "前回の調査からの時間",
    ["snmp_device_last_poll_duration"] = "最後のポーリングからの間隔",
    ["snmp_device_polling_disabled"] = "周期デバイスポーリングは無効化されています。表示されている情報はないか古いかのどちらかです。 ポーリングは次から有効化できます <a href=\"%{url}\"><i class=\"fas fa-flask\"></i> 設定</a>.",
    ["snmp_device_x"] = "%{device}",
    ["snmp_devices"] = "SNMPデバイス",
    ["snmp_error_unable_to_set_interface_admin_status"] = "管理者ステータスに切り替えられません. SNMPデバイスが有効な書き込みコミュニティ権限が設定されているか確認してください.",
    ["snmp_host"] = "ホスト",
    ["snmp_interface"] = "SNMPインターフェイス",
    ["snmp_interface_type"] = "インターフェイスタイプ",
    ["snmp_interface_x"] = "インターフェイス:%{interface}",
    ["snmp_interfaces"] = "インターフェイス",
    ["snmp_last_poll_errors"] = "インターフェイスエラー(最後のポーリング)",
    ["snmp_level"] = "セキュリティレベル",
    ["snmp_lldp_descr"] = "デバイストポロジ情報は、<A HREF=\"https://en.wikipedia.org/wiki/Link_Layer_Discovery_Protocol\"> LLDP </A>プロトコルが監視対象デバイスで有効になっており、SNMP経由で読み取り可能になっている場合に利用できます。",
    ["snmp_lldp_node_color"] = "ノードの色はデバイスモデルを表します。",
    ["snmp_lldp_zoom_descr"] = "ノードをクリックしてデバイスを探索し、マウスホイールを使用してズームし、空の領域をクリックしてドラッグします。",
    ["snmp_monitor_disable_confirm"] = "SNMP デバイス%{device}のポート状態確認を本当に停止しますか?",
    ["snmp_monitor_enable_confirm"] = "SNMP デバイス%{device}のポート状態確認を本当に停止しますか? ポートが状態変更した場合にはアラートは常に生成されます",
    ["snmp_netsnmp"] = "CPU/メモリー",
    ["snmp_noauthnopriv"] = "無し",
    ["snmp_note_bridge_mib"] = "Bridge-MIBに関する周期的なSNMP探索はテーブルを埋めるために実行されます",
    ["snmp_note_bridge_mib_status"] = "表示されるステータスは、MACアドレスに関連付けられたdot1qTpFdbStatus Bridge MIBステータスです。",
    ["snmp_note_periodic_interfaces_polling"] = "周期的なSNMP探索は通常、インターフェイス更新のために5分後ことに実行されます",
    ["snmp_note_thpt_calc"] = "2つの連続したSNMP探索の連続合計のトラフィック(in + out)に対するスループットアカウント",
    ["snmp_note_thpt_span"] = "2つの連続したSNMP探索は%{last} と%{prev}に実行されました",
    ["snmp_pool"] = "プール",
    ["snmp_port_monitor"] = "SNMPポートモニター",
    ["snmp_printer"] = "プリンター",
    ["snmp_read_community"] = "読み込みコミュニティ",
    ["snmp_timeseries_descr"] = "SNMPデバイス 時系列データは<a href=\"%{url}\"><i class=\"fas fa-flask\"></i> 設定</a>から有効化できます. 最初のデータポイントをみるためには数分間かかります",
    ["snmp_topology"] = "トポロジー",
    ["snmp_username"] = "ユーザー名",
    ["snmp_version"] = "SNMPバージョン",
    ["snmp_write_community"] = "書き込みコミュニティ",
    ["stats_since"] = "以降の統計",
    ["status_dormant"] = "休眠状態",
    ["status_down"] = "ダウン",
    ["status_lowerlayerdown"] = "低レイヤー ダウン",
    ["status_notpresent"] = "存在しません",
    ["status_unknown"] = "不明な",
    ["status_up"] = "アップ",
    ["status_up_in_use"] = "アップ (利用中)",
    ["testing"] = "テスト中",
    ["tp_fbd_status_invalid"] = "無効な",
    ["tp_fbd_status_learned"] = "学習した",
    ["tp_fbd_status_mgmt"] = "管理",
    ["tp_fbd_status_other"] = "その他",
    ["tp_fbd_status_self"] = "セルフ",
    ["trunk"] = "トランク",
    ["unable_to_delete_snmp_device"] = "SNMPデバイス%{device}を削除できません",
    ["unresponsive_device"] = "SNMP探索は遅すぎます。表示されたデータは有効期限切れか不足しています。 ネットワークコネクティビティとデバイスレスポンシブネスをチェックして下さい。15分後に新規のスキャンは開始されます",
    ["view_device"] = "デバイスビュー",
    ["descriptions"] = {
      ["add_pool"] = "新しいSNMPプール",
      ["authentication_passphrase"] = "SNMPv3認証パスフレーズ",
      ["authentication_protocol"] = "SNMPv3認証プロトコル",
      ["host"] = "SNMPエージェントIPアドレス",
      ["level"] = "SNMPv3セキュリティレベル",
      ["pool"] = "バインドするSNMPプール",
      ["privacy_passphrase"] = "SNMPv3暗号パスフレーズ",
      ["privacy_protocol"] = "SNMPv3暗号プロトコル",
      ["read_community"] = "SNMP v1/v2c 読み込み権限 community",
      ["username"] = "SNMP v3認証ユーザー名",
      ["version"] = "SNMP プロトコルバージョン",
      ["write_community"] = "SNMP v1/v2c 書き込み権限 community",
    },
    ["validation"] = {
      ["abort_add_snmp_devices"] = "新しいSNMPデバイスの追加を中止しますか?",
      ["abort_edit_snmp_devices"] = "SNMPデバイスの編集を中止しますか?",
      ["invalid_host"] = "IPv4 もしくは IPv6 アドレスを入力してください.",
      ["read_community_empty"] = "読み込み権限 Community は空で設定できません.",
    },
  },
  ["snmp_page"] = {
    ["guide_snmp_page_message"] = "ページ <a href='%{url}'> SNMP</a> にアクセスして、このホストを構成済みSNMPデバイスのリストに追加します。",
    ["not_configured_as_snmp_device_message"] = "ホスト%{host_ip}はSNMPデバイスとして設定されていません",
    ["showing_snmp_mib_info_default_community_message"] = "デフォルトのコミュニティ '%{community}' を利用してを使用して取得したSNMP MIB情報を表示しています",
    ["snmp_device_not_found"] = "SNMP デバイス \"%{host}\" 見つかりません: 設定されたSNMPデバイスのリストから削除されていませんか?",
    ["trying_to_retrive_message"] = "デフォルトのコミュニティ '%{community}'を使用してホストSNMP MIBを取得しようとしています",
    ["unable_to_retrive_snmp_default_community_message"] = "デフォルトコミュニティ '%{community}'を利用してSNMP MIBを取得できませんでした",
  },
  ["snmp_ratio_errors"] = {
    ["flowdevice_timeseries_not_existing"] = "利用出来ません.",
    ["flowdevice_timeseries_query_failed"] = "時系列に到達できません",
    ["nan_ratio"] = "レートは番号ではありません.",
    ["negative_ratio"] = "マイナスのレートです.",
    ["ok"] = "OK",
    ["snmp_timeseries_not_enabled"] = "時系列が無効.",
    ["snmp_timeseries_not_existing"] = "利用できません.",
    ["snmp_timeseries_query_failed"] = "時系列が読めません.",
    ["snmp_timeseries_zero_total"] = "データがありません.",
    ["timeseries_start_time_mismatch"] = "時系列の位置があっていません.",
  },
  ["sprobe_page"] = {
    ["active_memory"] = "アクティブなメモリ",
    ["latency"] = "遅延",
    ["processes"] = "プロセス",
    ["processes_traffic_tree"] = "プロセストラフィックツリー",
    ["show_more_info"] = "クリックでプロセス情報を表示するか、さらなる情報を表示するためにダブルクリックしてください",
    ["system_interactions"] = "システムの相互作用",
    ["top_processes"] = "上位プロセス",
    ["top_users"] = "上位ユーザー",
    ["tree"] = "ツリー",
    ["users"] = "ユーザー",
  },
  ["sub_interfaces"] = {
    ["add_sub_if"] = "サブインターフェイスを追加",
    ["add_sub_if_success"] = "サブインターフェイスが正常に作成されました。変更を有効にするには、ntopngの再起動が必要です。",
    ["collection_filter_description"] = "NetFlowフィールドを使うことができます, これは次を含み明日 INPUT_SNMP, OUTPUT_SNMP, SRC_VLAN, DST_VLAN. 数字でない値はシングルクォートとなります.",
    ["confirm_delete"] = "%{name}を削除してもよろしいですか？",
    ["delete"] = "サブインターフェイスを削除",
    ["disaggregation"] = "カスタムトラフィック分解",
    ["filter_description"] = "拡張nBPF形式がサポートされます。標準BPF構文に加えて、INPUT_SNMP、OUTPUT_SNMP、SRC_VLAN、DST_VLANなどのNetFlowフィールドも使用できます。",
    ["filter_examples"] = "フィルターの例",
    ["invalid_xnbpf"] = "無効なnBPFフィルター",
    ["note"] = "ntopngは、BPFのようなフィルターを使用して、受信トラフィックを分解し、論理サブインターフェイスに迂回させることができます。フィルターが重複している場合、同じトラフィックを複数のインターフェイスに迂回させることができます。",
    ["sub_if_name"] = "サブインターフェイス名",
    ["traffic_filter_nbpf"] = "トラフィックフィルター（nBPF形式）",
    ["traffic_filter_xnbpf"] = "トラフィックフィルター（拡張nBPF形式）",
  },
  ["syslog"] = {
    ["add"] = "ログプロデューサーの追加",
    ["bad_action_param"] = "間違った \"action\" パラメータ",
    ["confirm_delete"] = "このプロデューサーを本当に削除してもよろしいですか?",
    ["edit"] = "ログプロデューサーの編集",
    ["empty_action"] = "何か間違っています(空のアクション). ページをリロードし、再度試してください.",
    ["host_add_ok"] = "ホスト %{host} の追加に成功しました!",
    ["host_delete_ok"] = "ホスト %{host} の削除に成功しました!",
    ["host_edit_ok"] = "ホスト %{host} の編集に成功しました!",
    ["host_exists"] = "ホスト \"%{host}\" 既に存在します",
    ["host_not_exists"] = "ホスト \"%{host}\" が見つかりません",
    ["ip_or_device"] = "IPもしくはデバイス名",
    ["ip_or_device_note"] = "ログプロデューサーIPは、syslogメッセージに表示されるデバイス名もしくはIPアドレスに一致する必要があります. 例:<br />Thu Apr 10 16:30:00 2020;<strong>10.0.0.1</strong>; <181>Thu Apr 10 16:30:00 <strong>sslvpn-device-01</strong> logserver: ...",
    ["producer_host"] = "ログプロデューサーのIPアドレス",
    ["producer_type"] = "ログプロデューサータイプ",
    ["producers"] = "Syslogログプロデューサー",
  },
  ["syslog_alert_endpoint"] = {
    ["content"] = "コンテンツ",
    ["syslog_alert_format"] = "フォーマット",
    ["text"] = "テキスト",
  },
  ["system_alerts_stats"] = {
    ["internal_alerts_queue"] = "内部キュー",
    ["notes_enq_deq"] = "'Fill Level' はキューにあるアラートの分数です.キューサイズを参照しています. 'Dropped' はキューが一杯の為アラートがドロップされた分数です. アラートの合計を参照します",
    ["notes_internal_alerts_queue"] = "内部キューはコアに生成されたアラートを処理する為にSQLiteとのコンビネーションで利用されます.そしてUI内部に表示します",
    ["queues_status"] = "Fill Level / Dropped",
  },
  ["system_interfaces_stats"] = {
    ["local_hosts"] = "ローカルホスト",
    ["remote_hosts"] = "リモートホスト",
  },
  ["system_stats"] = {
    ["cpu_states_monitor"] = "CPU状態モニター",
    ["cpu_states_monitor_description"] = "CPUの状態（例: ユーザー、システム、アイドル、iowait）を監視します",
    ["disk_monitor"] = "ディスクモニター",
    ["disk_monitor_description"] = "空きディスク容量を監視します",
    ["dropped_points"] = "ドロップされたポイント",
    ["export_failures"] = "失敗したエクスポート",
    ["export_retries"] = "再試行されたエクスポート",
    ["exported_points"] = "エクスポートされたポイント",
    ["exported_vs_dropped_points"] = "エクスポートされたポイントvsドロップされたポイント",
    ["exports"] = "エクスポートの合計",
    ["exports_label"] = "エクスポート",
    ["health"] = "稼働状況",
    ["high_series_cardinality"] = "カーディナリティが高いと、データベースの速度が低下する可能性があります",
    ["infludb_retried_vs_failed_exports"] = "再試行/失敗したエクスポート",
    ["influxdb_health_green"] = "OK",
    ["influxdb_health_green_descr"] = "InfluxDBへのエクスポートは正常に機能しています。",
    ["influxdb_health_red"] = "エクスポートに失敗しました",
    ["influxdb_health_red_descr"] = "InfluxDBが到達不能または遅すぎて、それに追いつくことができません。ポイントを落とします。",
    ["influxdb_health_yellow"] = "一時的にエクスポートできません",
    ["influxdb_health_yellow_descr"] = "現在、InfluxDBにエクスポートできません。再試行中です。",
    ["influxdb_note_docs"] = "表示されるすべてのフィールドの詳細な説明を含むドキュメントは、ntopng <a target='_blank' href='%{url}'>ユーザーズガイド</a>で入手できます。",
    ["influxdb_storage"] = "ストレージ（%{dbname}）",
    ["last_ip"] = "前回のIP",
    ["series_cardinality"] = "シリーズのカーディナリティ",
    ["short_desc_influxdb_cardinality"] = "InfluxDBが書かれたポイントを処理することの難しさを示しています。",
    ["short_desc_influxdb_dropped_points"] = "InfluxDBの書き込みエラーによりドロップされたポイントの総数。",
    ["short_desc_influxdb_exported_points"] = "InfluxDBに正常に書き込まれたポイントの総数。",
    ["short_desc_influxdb_exports"] = "InfluxDBへの成功した書き込みの総数。",
    ["short_desc_influxdb_health"] = "現在のInfluxDBエクスポートの稼働状況",
    ["short_desc_influxdb_ram_memory"] = "InfluxDBがオペレーティングシステムから取得した合計メモリ。",
    ["short_desc_influxdb_storage_utilization"] = "InfluxDBが使用する合計ディスク容量。",
    ["value"] = "値",
    ["write_througput"] = "書き込みスループット",
    ["redis"] = {
      ["command_hits"] = "%{cmd}コール",
      ["commands"] = "コマンド",
      ["redis_health_green"] = "OK",
      ["redis_health_green_descr"] = "Redisは正常に動作しています。",
      ["redis_health_red"] = "永続性の問題",
      ["redis_health_red_descr"] = "Redisはデータを保持できません。%{product}が誤動作する可能性があります。",
      ["redis_keys"] = "データベースキー",
      ["redis_monitor"] = "Redisモニター",
      ["redis_monitor_description"] = "Redisの稼働状況を監視する",
      ["short_desc_redis_health"] = "現在のRedisの稼働状況。",
      ["short_desc_redis_keys"] = "Redisデータベースの合計キー。",
      ["short_desc_redis_ram_memory"] = "Redisがオペレーティングシステムから取得した合計メモリ。",
      ["top_redis_commands"] = "上位コマンドの呼び出し",
      ["tot_calls"] = "合計コール数",
    },
  },
  ["telemetry_page"] = {
    ["crash_report"] = "クラッシュレポート",
    ["crash_report_descr"] = "%{product}の異常終了時にクラッシュレポートが送信されます。クラッシュレポートには、バージョン、ビルドアーキテクチャ、オペレーティングシステム、起動オプションなどの%{product}情報が含まれます。監視対象トラフィックに関する何も送信されません。クラッシュレポートの例を次に示します。",
    ["last_data_sent"] = "最後に送信されたデータ",
    ["send_telemetry_data"] = "テレメトリデータを送信して貢献する",
    ["telemetry_data"] = "テレメトリデータ",
    ["telemetry_data_change_preference"] = "<a href='%{url}'>変更</a>",
    ["telemetry_data_no_consent"] = "優先度が示されていない",
  },
  ["top_hosts"] = {
    ["note"] = "<b>注意</b><br>このページは、トップホストアクティビティをレポートするために数秒でリフレッシュされます.<p>ホストがトラフィックを生成すると、垂直バーが表示されます.もっともアクティブなホストを発見することができます.",
    ["top_hosts_local"] = "上位ホスト (ローカル)",
  },
  ["topk_heuristic"] = {
    ["precision"] = {
      ["aggressive"] = "アグレッシブ",
      ["disabled"] = "無効化",
      ["less_accurate"] = "精度が低い",
      ["more_accurate"] = "より正確な",
    },
  },
  ["traffic_page"] = {
    ["flow_distribution"] = "フロー分布",
    ["flow_duration"] = "総フロー時間",
    ["flow_frequency"] = "新しいフロー作成頻度",
    ["hosts_contacts_cardinality"] = "ユニークホストコンタクトの合計",
    ["l4_proto_overview"] = "L4プロトコル概要",
    ["no_traffic_observed_message"] = "指定のホストにはトラフィックは観測されていません",
    ["num_contacted_hosts_as_client"] = "コンタクトされたホスト (クライアントとして)",
    ["num_host_contacts_as_server"] = "ホスト コンタクト (サーバーとして)",
    ["rcvd"] = "受信",
  },
  ["traffic_profiles"] = {
    ["advanced_filter_examples"] = "高度なフィルター例",
    ["all_profiles"] = "すべてのプロファイル",
    ["confirm_delete_profile"] = "プロファイルを本当に削除しますか",
    ["delete_profile"] = "プロファイル削除",
    ["destination_network"] = "宛先ネットワーク",
    ["duplicate_profile"] = "プロファイル名を複製",
    ["edit_traffic_profiles"] = "トラフィックプロファイル",
    ["enter_profile_filter"] = "プロファイルフィルターを入力",
    ["enter_profile_name"] = "プロファイル名を入力",
    ["facebook_traffic"] = "Facebook",
    ["host_http_https"] = "ホスト HTTP/HTTPS",
    ["host_traffic"] = "ホスト",
    ["http_traffic"] = "HTTP",
    ["invalid_bpf"] = "無効なBPFフィルター",
    ["max_profiles_num"] = "ntopngのこのバージョンについては%{maxnum} プロファイルを作成可能です。 この制限を取り払うためにはエンタープライズバージョンをご利用ください",
    ["no_profiles"] = "プロファイルが設定されていません",
    ["note"] = "注意",
    ["note_0"] = "トラフィックプロファイル名は英数文字、スペースとアンダスコアのみ許可します",
    ["note_1"] = "トラフィック プロファイルはフローに適用されます。各フローは最大で１つのフローを持ちます。このため複数のプロファイルがマッチした場合には最初の一つが選択されます",
    ["profile_name"] = "プロファイル名",
    ["see_ndpi_protos"] = "(%{option}を使うとnDPIでサポートされているすべてのプロトコルが表示されます)",
    ["simple_filter_examples"] = "Simpleフィルター例",
    ["source_ethernet"] = "ソースMACアドレス",
    ["traffic_between"] = "ホスト間のトラフィック",
    ["traffic_filter_bpf"] = "トラフィックフィルター(nBPF Format)",
    ["traffic_from_to"] = "ホストから別のホストへのトラフィック",
    ["traffic_profiles"] = "トラフィックプロファイル",
  },
  ["traffic_recording"] = {
    ["about_to_extract"] = "トラフィックを <b>%{date_begin}</b> から <b>%{date_end}</b>までで抽出中です",
    ["active_since"] = "アクティブになってから",
    ["archive"] = "アーカイブ",
    ["completed"] = "完了",
    ["continuous_recording"] = "継続的なトラフィック記録",
    ["continuous_recording_and_flows"] = "継続的なトラフィックの記録とフローの可視性",
    ["delete_all_jobs"] = "すべてのジョブを削除",
    ["delete_all_jobs_confirm"] = "すべての抽出ジョブを本当に削除しますか？アクティブなジョブは削除されます",
    ["delete_job"] = "ジョブの削除",
    ["delete_job_confirm"] = "ジョブ%{job_id} と関連するPcapファイルの削除を本当に実行しますか?",
    ["disabled"] = "無効化",
    ["disk_space"] = "最大ディスク容量",
    ["disk_space_note"] = "ディスクに記録されたトラフィックを記録するための最大スペース",
    ["disk_space_note_in_use"] = "%{in_use} GBが既にインスタンスにより利用されています.",
    ["download"] = "ダウンロード",
    ["download_nth_pcap"] = "ダウンロードPcap #{0}",
    ["dump_window"] = "ダンプウィンドウ",
    ["edit_filter"] = "フィルター編集",
    ["err_alloc"] = "メモリ確保エラー",
    ["err_filter"] = "サポートされていないフィルター",
    ["err_open"] = "ダンプを開けません",
    ["err_stuck"] = "立ち往生または中断",
    ["err_unknown"] = "予期しないエラー",
    ["ext_interface"] = "外部インターフェイス",
    ["extract_action"] = "抽出",
    ["extract_now"] = "今すぐ抽出",
    ["extracted_bytes"] = "抽出されたバイト",
    ["extracted_packets"] = "抽出されたパケット",
    ["extraction_scheduled"] = "新しい抽出ジョブがスケジュールされました。ファイルの準備ができ次第、%{page}ページをチェックしてpcapをダウンロードしてください。この抽出のジョブIDは ",
    ["failed"] = "失敗",
    ["failure"] = "失敗",
    ["failure_note"] = "ログトレースをチェックしてサービスを再開してください",
    ["filter_bpf"] = "フィルター (nBPF Format)",
    ["filter_examples"] = "フィルター例",
    ["global_settings_note"] = "グローバル設定（ライセンスキーなど）は、 <a href=\"%{url}\">トラフィック記録の設定</a>から設定できます。.",
    ["invalid_bpf"] = "無効なフィルター",
    ["job_date_time"] = "ジョブ作成日時",
    ["job_id"] = "ジョブID",
    ["jobs"] = "ジョブ",
    ["logs"] = "トラフィック記録ログ",
    ["max_extracted_pcap_bytes_description"] = "記録されたデータから抽出されるpcapファイルの最大サイズ。 抽出がこのサイズを超える場合、出力は複数のファイルに分かれます",
    ["max_extracted_pcap_bytes_title"] = "最大抽出ファイルサイズ",
    ["missing_data_msg"] = "トラフィックの記録が停止したか再起動しました。いくつかのデータが欠落した可能性があります",
    ["missing_parameters"] = "必須パラメータが不足しています",
    ["msg_err_missing_timeline"] = "現在実行中のn2diskのタイムラインを特定できません。抽出は、n2diskが最新で、オプション--timeline-dirで実行されている場合にのみ実行できます。",
    ["msg_err_timeline_path_not_existing"] = "システムにタイムラインパスが見つかりません。",
    ["msg_err_unable_to_access_timeline"] = "記録されたデータを読み取れません。権限が不十分です。 ntopngを実行しているユーザーが、n2diskを実行しているユーザーと同じグループに属していることを確認してください。",
    ["msg_err_unknown"] = "不明なステータス。",
    ["msg_external_providers_detected"] = "外部トラフィック記録プロバイダーが検出されました。 ntopngは、このインターフェイスを外部プロバイダーに自動的にバインドできます。このページを使用してntopngに記録を処理させるか、<a href=\"%{url}\"> <i class =  \"fas fa-cog \"> </ i>外部トラフィック記録プロバイダー</a>に設定することができます。 ",
    ["msg_ok"] = "抽出する準備ができました",
    ["multiple_extracted_files"] = "トラフィック抽出により、%{mb} MBの複数のpcapファイルが生成されました。この制限は設定から変更できます。 以下のリンクをクリックして、pcapファイルをダウンロードします。",
    ["no_data_in_interval"] = "選択した時間間隔の間、ディスク上に（または部分的な）pcapデータはありません。pcapデータは毎分ダンプされ（最新の分は利用できない）、古いデータは削除されます。(<A HREF=\"%{page}\">ステータスページ</A>)で現在のダンプウィンドウを確認して下さい）別の時間間隔を選択してみてください。",
    ["no_file"] = "まだファイルがダンプされていません",
    ["no_recorded_data"] = "指定された時間枠にデータが記録されていません。トラフィックレコーダーは、最近のデータを書き込むのに時間がかかります。",
    ["not_found"] = "ファイルが見つかりません",
    ["not_granted"] = "要求は否認されました",
    ["note_dump"] = "イーサネットネットワークでは各パケットは<A HREF=\"https://en.wikipedia.org/wiki/Ethernet_frame\">24バイトのオーバーヘッド</A> [プリアンブル, フレームの開始, CRC, IFG]を持ちます。 このようなオーバヘッドはインターフェイストラフィックには考慮されています。しかしディスクにダンプされるトラフィックには追加されません。このため、インターフェイスチャートにレポートされるものと、このページで表示されるものはバイト数が異なる場合がありえます",
    ["packet_dumps"] = "パケットダンプ",
    ["pcap_download"] = "pcapを抽出する",
    ["pcap_download_message"] = "ファイルの準備ができ次第、抽出を開始し、%{page}ページをチェックしてpcapをダウンロードします",
    ["processing"] = "処理中",
    ["queue_as_job"] = "ジョブとしてキュー",
    ["recording"] = "記録中",
    ["restart"] = "再起動",
    ["restart_service"] = "サービスを再起動",
    ["settings"] = "設定",
    ["start_extraction"] = "抽出開始",
    ["stop_job"] = "ジョブ停止",
    ["stop_job_confirm"] = "ジョブ%{job_id}を本当に停止ししますか?",
    ["stopped"] = "停止",
    ["storage"] = "ストレージ",
    ["storage_dir"] = "ストレージディレクトリ",
    ["storage_directory_config"] = "ストレージディレクトリのパスは、<i>%{option}</i> オプションを%{product} 構成ファイルに指定することで変更できます。",
    ["storage_utilization"] = "ストレージ使用率",
    ["storage_utilization_pcap"] = "PCAPストレージ使用率",
    ["traffic_extraction_jobs"] = "トラフィック抽出ジョブ",
    ["traffic_extractions"] = "抽出",
    ["traffic_on_disk"] = "ディスク上のトラフィック",
    ["traffic_recording"] = "トラフィック記録",
    ["traffic_recording_provider"] = "トラフィック記録プロバイダー",
    ["traffic_recording_settings"] = "トラフィック記録設定",
    ["traffic_recording_status"] = "トラフィック記録状態",
    ["unsupported_fields"] = "選択されたチャートは抽出ツールで100%%フルサポートされていないトラフィックも表示します。このため期待よりも多くのトラフィックが表示される可能性があります",
    ["waiting"] = "待機中",
    ["waiting_nospace"] = "待機中 (ディスクに空きがありません)",
  },
  ["traffic_report"] = {
    ["current_day"] = "現在の日",
    ["current_month"] = "現在の月",
    ["current_week"] = "現在の週",
    ["daily"] = "毎日",
    ["header_daily"] = "日次レポート",
    ["header_monthly"] = "月次レポート",
    ["header_weekly"] = "週次レポート",
    ["monthly"] = "月次",
    ["previous_day"] = "前日",
    ["previous_month"] = "前月",
    ["previous_week"] = "前週",
    ["weekly"] = "週次",
  },
  ["tree_map"] = {
    ["hosts_treemap"] = "ホストツリーマップ",
  },
  ["unknown_devices"] = {
    ["active_only"] = "アクティブのみ",
    ["all_devices"] = "すべてのデバイス",
    ["assign_device_pool"] = "デバイスをプールに割り付け",
    ["assign_pool"] = "プールに割り付け",
    ["device_name"] = "デバイス名",
    ["devices_only_note"] = "MACアドレスでプールメンバーが定義されていない場合、デバイスは未割り当てと見なされます。IPベースのメンバーアドレスはここでは適用されません。",
    ["filter_devices"] = "フィルターデバイス",
    ["first_seen"] = "最初に確認した",
    ["inactive_only"] = "インアクティブのみ",
    ["last_seen"] = "最後に確認した",
    ["no_pools"] = "不明なデバイスをアサインするためにホストプールの生成が必要です",
    ["select_pool"] = "'%{mac}'をアサインするためにホストプールを選択してください:",
    ["unassigned_devices"] = "未割り付けデバイス",
  },
  ["updates"] = {
    ["available"] = "新着",
    ["check"] = "アップデートを確認",
    ["checking"] = "アップデートの確認..",
    ["install"] = "アップデートのインストール",
    ["install_confirm"] = "今ntopngをアップデートし再起動しますか?",
    ["installing"] = "アップデートをインストール中..",
    ["maintenance_expired"] = "このプロダクトに対するメンテナンスの有効期限が切れています, アップグレードの後にライセンスを更新する必要があります.",
    ["no_updates"] = "利用できるアップデートがありません",
    ["update_failure_message"] = "更新中に問題が発生しました。再試行するか、手動で試してください",
  },
  ["user_activity"] = {
    ["add_ntopng_user"] = "ユーザー '%{user}' 新しいユーザーを追加しました %{new_user}.",
    ["add_pool"] = "ユーザー '%{user}' 新しいプールを作成しました <a href=\"%{pool_url}\">%{pool_name}</a>.",
    ["alerts_deleted"] = "ユーザー '%{user}'は '%{status}'アラートを削除しました。",
    ["bind_pool_member"] = "ユーザー '%{user}' メンバーを追加しました %{member} プール名 <a href=\"%{pool_url}\">%{pool_name}</a>.",
    ["changed_preference"] = "ユーザー '%{user}' は、%{pref}の設定を変更しました",
    ["delete_ntopng_user"] = "ユーザー '%{user}' ユーザーを削除しました %{old_user}.",
    ["delete_pool"] = "ユーザー '%{user}' プールを削除しました <a href=\"%{pool_url}\">%{pool_name}</a>.",
    ["deleted_all_interfaces_data"] = "ユーザー '%{user}' は、すべてのインターフェイスについてデータを削除しました",
    ["deleted_host_data"] = "ユーザー '%{user}'は、ホスト%{host} のインターフェイス%{ifname}についてデータを削除しました",
    ["deleted_inactive_interfaces_data"] = "ユーザー '%{user}'は、すべてのインアクティブなインターフェイスについてデータを削除しました",
    ["deleted_interface_data"] = "ユーザー'%{user}'は、インターフェイス%{ifname}についてデータを削除しました",
    ["deleted_network_data"] = "ユーザー '%{user}'はインターフェイス%{ifname}のネットワーク%{network}のデータを削除しました",
    ["disabled_preference"] = "ユーザー'%{user}'は、%{pref}の設定を無効化しました",
    ["edit_pool"] = "ユーザー '%{user}' プールを編集しました <a href=\"%{pool_url}\">%{pool_name}</a>.",
    ["enabled_preference"] = "ユーザー'%{user}'は、%{pref}の設定を有効化しました",
    ["exported_data"] = "ユーザー'%{user}'は、'%{mode}' ホストのJSONデータをエクスポートしました",
    ["exported_data_host"] = "ユーザー'%{user}'は、ホスト%{host}の'%{mode}'データをJSONとしてエクスポートしました ",
    ["file_downloaded"] = "ユーザー '%{user}'は、'%{file}'をダウンロードしました",
    ["flows_downloaded"] = "ユーザー'%{user}'は、%{from} から%{to}へのフローデータをダウンロードしました",
    ["host_json_downloaded"] = "ユーザー '%{user}'は、%{host}に関するデータをダウンロードしました",
    ["live_capture"] = "ユーザー '%{user}'は、インターフェイス%{ifname}のリアルタイム キャプチャーを実施しました",
    ["live_capture_host"] = "ユーザー'%{user}'はホスト%{host} のインターフェイス%{ifname}についてリアルタイム キャプチャを実施ししました",
    ["live_capture_host_with_filter"] = "ユーザー '%{user}'はホスト%{host}のインターフェイス%{ifname}でフィルタ'%{filter}'にマッチするデータをリアルタイム キャプチャーを実施しました",
    ["live_capture_with_filter"] = "ユーザー'%{user}'はインターフェイス%{ifname}についてフィルタ'%{filter}'にマッチするデータをリアルタイム キャプチャーしました",
    ["live_extraction"] = "ユーザー'%{user}'は インターフェイス%{ifname}について%{from}から%{to}へのデータをダウンロードしました",
    ["live_extraction_with_filter"] = "ユーザー'%{user}'はインターフェイス%{ifname}について%{from} から%{to}でフィルタ'%{filter}'にマッチするトラフィックをダウンロードしました",
    ["login_not_authorized"] = "ユーザー'%{user}'のログインが拒否されました",
    ["login_successful"] = "ユーザー'%{user}'がログインしました.",
    ["password_changed"] = "'%{pwd_user}'に関するパスワードが'%{user}'により変更されました",
    ["recording_disabled"] = "ユーザー'%{user}'がインターフェイス%{ifname}についてトラフィック記録を無効化しました",
    ["recording_enabled"] = "ユーザー'%{user}'がインターフェイス%{ifname}についてトラフィック記録を有効化しました",
    ["remote_assistance_disabled"] = "ユーザー '%{user}'がリモートアシスタンスを無効化しました",
    ["remote_assistance_enabled"] = "ユーザー'%{user}'がリモートアシスタンスを有効化しました",
    ["set_host_alias"] = "ユーザー '%{user}' エイリアスを設定 ホスト %{host} から %{alias}.",
    ["set_pool_config"] = "ユーザー '%{user}' プール設定をインポートしました.",
    ["set_scripts_config"] = "ユーザー '%{user}' ユーザースクリプト設定をインポートした.",
    ["snmp_device_added"] = "ユーザー'%{user}'がSNMPデバイス%{ip}を追加しました",
    ["snmp_device_deleted"] = "ユーザー'%{user}'がSNMPデバイス%{ip}を削除しました",
    ["unknown_activity"] = "ユーザー'%{user}'が未知の動作(%{scope})を実行しました。レポートしてください",
    ["unknown_activity_function"] = "ユーザー'%{user}'が未知の動作(%{name})を実行しました。レポートしてください",
    ["user_added"] = "ユーザー'%{add_user}'が'%{user}'により追加されました.",
    ["user_deleted"] = "ユーザー'%{del_user}'が'%{user}'により削除されました",
  },
  ["user_info"] = {
    ["missing_user_name_message"] = "ユーザー名が不足しています",
    ["processes"] = "プロセス",
    ["processes_overview"] = "プロセス概要",
    ["top_l4_protocols"] = "上位L4プロトコル",
    ["top_l7_protocols"] = "上位L7プロトコル",
  },
  ["users"] = {
    ["add_new_user"] = "新規ユーザーを追加",
    ["add_user"] = "ユーザーを追加",
    ["assigned_devices"] = "割り当て済みデバイス",
    ["cat_protocols"] = "%{cat} プロトコル",
    ["categories"] = "カテゴリ",
    ["client_policy"] = "クライアントとして動作するデバイス",
    ["confirm_delete_user"] = "本当にユーザー\"%{user}\"を削除しますか?",
    ["delete_user"] = "ユーザー削除",
    ["delete_user_error"] = "ユーザーを削除できませんでした。詳細については、プログラムログをご覧ください。",
    ["devices"] = "デバイス",
    ["edit"] = "編集",
    ["edit_global_policies"] = "デフォルトポリシー編集",
    ["fallback_policy"] = "フォールバックポリシー",
    ["full_name"] = "フルネーム",
    ["global"] = "グローバル",
    ["global_policies"] = "デフォルト ポリシー",
    ["no_users"] = "ユーザーがありません",
    ["num_protocols"] = "プロトコル数",
    ["policy"] = "ポリシー",
    ["protocols"] = "プロトコル",
    ["reset_to_defaults"] = "デフォルトにリセット",
    ["reset_to_defaults_confirm"] = "デバイスタイプ \"%{devtype}\" のポリシーを本当にデフォルトに戻しますか?",
    ["server_policy"] = "サーバーとして動作するデバイス",
    ["settings"] = "設定",
    ["total_daily_time_quota"] = "1日の合計クォータ",
    ["total_daily_traffic_quota"] = "1日の合計トラフィッククォータ",
    ["unchanged"] = "変更なし",
    ["user_exists"] = "ユーザー \"%{user}\"は既に存在します",
    ["user_x"] = "ユーザー:%{user}",
    ["users"] = "ユーザー",
    ["shapers"] = {
      ["default"] = "デフォルト",
      ["drop"] = "ドロップ",
      ["pass"] = "パス",
      ["slow"] = "遅いパス",
      ["slower"] = "更に遅いパス",
    },
  },
  ["validation"] = {
    ["FAILED_HTTP_REQUEST"] = "リクエストは失敗しました.後ほど再度試してください.",
    ["empty_name"] = "名前を入力してください.",
    ["invalid_input"] = "入力パターンを確認してください.",
    ["missing_field"] = "入力を埋めてください.",
  },
  ["vlan_details"] = {
    ["no_available_stats_for_vlan_message"] = "VLAN%{vlan_id}に関して有効な統計がありません。%{product}が統計を収集するまで数分お待ちください",
    ["vlan_id_parameter_missing_or_invalid_message"] = "VLANIDパラメータが不足しているか無効です",
  },
  ["vlan_stats"] = {
    ["vlan_id"] = "VLAN ID",
    ["vlans"] = "VLAN",
  },
  ["webhook_alert_endpoint"] = {
    ["password"] = "パスワード",
    ["shared_secret"] = "共通された秘密",
    ["url"] = "URL",
    ["username"] = "ユーザー名",
  },
}

return lang
